Package: clusterExperiment
Title: Compare Clusterings for Single-Cell Sequencing
Version: 2.32.0
Description: Provides functionality for running and comparing many
    different clusterings of single-cell sequencing data or other large mRNA Expression data sets.
Authors@R: c(person("Elizabeth", "Purdom", email = "epurdom@stat.berkeley.edu",
            role = c("aut", "cre", "cph")),
	     person("Davide","Risso", role = "aut",email = "risso.davide@gmail.com"))
BugReports: https://github.com/epurdom/clusterExperiment/issues
License: Artistic-2.0
Encoding: UTF-8
Depends: R (>= 3.6.0), SingleCellExperiment, SummarizedExperiment (>=
        1.15.4), BiocGenerics
Imports: methods, NMF, RColorBrewer, ape (>= 5.0), cluster, stats,
        limma, locfdr, matrixStats, graphics, parallel, BiocSingular,
        kernlab, stringr, S4Vectors, grDevices, DelayedArray (>=
        0.7.48), HDF5Array (>= 1.7.10), Matrix, Rcpp, edgeR, scales,
        zinbwave, phylobase, pracma, mbkmeans
Suggests: BiocStyle, knitr, testthat, MAST, Rtsne, scran, igraph,
        rmarkdown
VignetteBuilder: knitr
LazyData: false
LazyLoad: false
RoxygenNote: 7.3.3
biocViews: Clustering, RNASeq, Sequencing, Software, SingleCell
Collate: 'AllChecks.R' 'AllClassesCE.R' 'AllClassesCF.R'
        'AllGenerics.R' 'AllHelper.R' 'AllHelperClusterFunction.R'
        'AllHelperDendro.R' 'AllHelperFilter.R' 'JiashinJiCode.R'
        'RcppExports.R' 'addClusterings.R' 'assignUnassigned.R'
        'subsampleClustering.R' 'internalDendroFunctions.R'
        'internalClusterFunctions.R' 'internalFunctions.R'
        'builtInClusterFunctions.R' 'clusterContrasts.R'
        'clusterMany.R' 'clusterSingle.R' 'dataCreation.R'
        'deprecateFunctions.R' 'getClusterIndex.R' 'getFeatures.R'
        'getParams.R' 'getReducedData.R' 'mainClustering.R'
        'makeBlankData.R' 'makeConsensus.R' 'makeDendrogram.R'
        'makeFilterStats.R' 'makeReducedDims.R' 'mergeClusters.R'
        'plotBarplot.R' 'plotClusters.R' 'plotClustersTable.R'
        'plotClustersWorkflow.R' 'plotContrastHeatmap.R'
        'plotDendrogram.R' 'plotFeatureBoxplot.R'
        'plotFeatureScatter.R' 'plotHeatmap.R' 'plotReduceDim.R'
        'plottingHelpers.R' 'rsec.R' 'seqCluster.R' 'subsampleLoop.R'
        'subset.R' 'transformFunction.R' 'updateObject.R'
        'workflowClusters.R'
LinkingTo: Rcpp
git_url: https://git.bioconductor.org/packages/clusterExperiment
git_branch: RELEASE_3_23
git_last_commit: dae5064
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 21:44:47 UTC; biocbuild
Author: Elizabeth Purdom [aut, cre, cph],
  Davide Risso [aut]
Maintainer: Elizabeth Purdom <epurdom@stat.berkeley.edu>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-29 04:37:28 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ClusterExperiment)
export(ClusterFunction)
export(bigPalette)
export(internalFunctionCheck)
export(listBuiltInFilterStats)
export(listBuiltInFunctions)
export(listBuiltInReducedDims)
export(listBuiltInType01)
export(listBuiltInTypeK)
export(makeBlankData)
export(makeRsecFluidigmObject)
export(massivePalette)
export(numericalAsCharacter)
export(seqCluster)
export(seqPal1)
export(seqPal2)
export(seqPal3)
export(seqPal4)
export(seqPal5)
export(setBreaks)
export(showHeatmapPalettes)
export(showPalette)
exportClasses(ClusterExperiment)
exportClasses(ClusterFunction)
exportMethods("[")
exportMethods("clusterLabels<-")
exportMethods("clusterLegend<-")
exportMethods("clusterTypes<-")
exportMethods("coClustering<-")
exportMethods("nodeLabels<-")
exportMethods("orderSamples<-")
exportMethods("primaryClusterIndex<-")
exportMethods("transformation<-")
exportMethods(ClusterExperiment)
exportMethods(ClusterFunction)
exportMethods(RSEC)
exportMethods(addClusterings)
exportMethods(addToColData)
exportMethods(algorithmType)
exportMethods(assignUnassigned)
exportMethods(checkDendrogram)
exportMethods(clusterContrasts)
exportMethods(clusterDendrogram)
exportMethods(clusterLabels)
exportMethods(clusterLegend)
exportMethods(clusterMany)
exportMethods(clusterMatrix)
exportMethods(clusterMatrixColors)
exportMethods(clusterMatrixNamed)
exportMethods(clusterSingle)
exportMethods(clusterTypes)
exportMethods(clusteringInfo)
exportMethods(coClustering)
exportMethods(colDataClusters)
exportMethods(combineMany)
exportMethods(convertClusterLegend)
exportMethods(convertToDendrogram)
exportMethods(defaultNDims)
exportMethods(dendroClusterIndex)
exportMethods(eraseMergeInfo)
exportMethods(filterData)
exportMethods(filterNames)
exportMethods(getBestFeatures)
exportMethods(getBuiltInFunction)
exportMethods(getClusterIndex)
exportMethods(getClusterManyParams)
exportMethods(getMergeCorrespond)
exportMethods(getPostProcessingArgs)
exportMethods(getReducedData)
exportMethods(getSingleClusterIndex)
exportMethods(inputType)
exportMethods(mainClustering)
exportMethods(makeConsensus)
exportMethods(makeDendrogram)
exportMethods(makeFilterStats)
exportMethods(makeReducedDims)
exportMethods(mergeClusterIndex)
exportMethods(mergeClusters)
exportMethods(mergeCutoff)
exportMethods(mergeMethod)
exportMethods(nClusterings)
exportMethods(nClusters)
exportMethods(nFeatures)
exportMethods(nInternalNodes)
exportMethods(nNodes)
exportMethods(nSamples)
exportMethods(nTips)
exportMethods(nodeIds)
exportMethods(nodeLabels)
exportMethods(nodeMergeInfo)
exportMethods(orderSamples)
exportMethods(plotBarplot)
exportMethods(plotClusterLegend)
exportMethods(plotClusters)
exportMethods(plotClustersTable)
exportMethods(plotClustersWorkflow)
exportMethods(plotCoClustering)
exportMethods(plotContrastHeatmap)
exportMethods(plotDendrogram)
exportMethods(plotFeatureBoxplot)
exportMethods(plotFeatureScatter)
exportMethods(plotHeatmap)
exportMethods(plotReducedDims)
exportMethods(primaryCluster)
exportMethods(primaryClusterIndex)
exportMethods(primaryClusterLabel)
exportMethods(primaryClusterNamed)
exportMethods(primaryClusterType)
exportMethods(recolorClusters)
exportMethods(removeClusterings)
exportMethods(removeClusters)
exportMethods(removeUnassigned)
exportMethods(renameClusters)
exportMethods(requiredArgs)
exportMethods(sampleDendrogram)
exportMethods(setToCurrent)
exportMethods(setToFinal)
exportMethods(show)
exportMethods(subsampleClustering)
exportMethods(subsetByCluster)
exportMethods(tableClusters)
exportMethods(transformData)
exportMethods(transformation)
exportMethods(updateObject)
exportMethods(workflowClusterDetails)
exportMethods(workflowClusterTable)
exportMethods(workflowClusters)
import(Matrix)
import(SingleCellExperiment)
import(SummarizedExperiment)
import(limma)
import(methods)
import(phylobase, except = plot)
importClassesFrom(DelayedArray,DelayedArray)
importClassesFrom(DelayedArray,DelayedArray1)
importClassesFrom(DelayedArray,DelayedMatrix)
importClassesFrom(DelayedArray,RleArray)
importClassesFrom(DelayedArray,RleMatrix)
importClassesFrom(HDF5Array,HDF5Matrix)
importClassesFrom(phylobase,phylo4)
importClassesFrom(phylobase,phylo4d)
importFrom(BiocGenerics,updateObject)
importFrom(BiocSingular,runPCA)
importFrom(NMF,aheatmap)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,metadata)
importFrom(ape,as.hclust.phylo)
importFrom(ape,as.phylo)
importFrom(ape,as.phylo.hclust)
importFrom(ape,is.ultrametric)
importFrom(ape,node.depth.edgelength)
importFrom(ape,node_depth)
importFrom(ape,node_depth_edgelength)
importFrom(ape,node_height)
importFrom(ape,node_height_clado)
importFrom(ape,nodelabels)
importFrom(ape,phydataplot)
importFrom(ape,plot.phylo)
importFrom(ape,unrooted.xy)
importFrom(cluster,daisy)
importFrom(cluster,pam)
importFrom(cluster,silhouette)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,colors)
importFrom(grDevices,gray)
importFrom(grDevices,rgb)
importFrom(graphics,barplot)
importFrom(graphics,boxplot)
importFrom(graphics,plot)
importFrom(kernlab,specc)
importFrom(limma,makeContrasts)
importFrom(locfdr,locfdr)
importFrom(matrixStats,rowIQRs)
importFrom(matrixStats,rowMads)
importFrom(matrixStats,rowMeans2)
importFrom(matrixStats,rowMedians)
importFrom(matrixStats,rowVars)
importFrom(parallel,mclapply)
importFrom(scales,alpha)
importFrom(stats,IQR)
importFrom(stats,as.dendrogram)
importFrom(stats,as.hclust)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,quantile)
importFrom(stringr,str_order)
importFrom(stringr,str_pad)
importFrom(stringr,str_sort)
importFrom(zinbwave,glmWeightedF)
importMethodsFrom(phylobase,"nodeLabels<-")
importMethodsFrom(phylobase,nNodes)
importMethodsFrom(phylobase,nTips)
importMethodsFrom(phylobase,nodeLabels)
useDynLib(clusterExperiment, .registration = TRUE)

# R package: clusterExperiment

Functions for running and comparing many different clusterings of single-cell sequencing data.

## News and Updates

* Version 2.3.0 is on Bioconductor (development version) with many new changes. Checkout out a [brief description](https://github.com/epurdom/clusterExperiment/blob/master/update2.4.md) of the major changes. For complete details, see the [NEWS](https://github.com/epurdom/clusterExperiment/blob/master/NEWS) file.

* Summary of changes from previous versions: 
	- Version 2.0.0  [brief description](https://github.com/epurdom/clusterExperiment/blob/master/update2.0.md) 

## Publications

* The paper acompanying this package can be found at:

	Risso D, Purvis L, Fletcher R, Das D, Ngai J, Dudoit S, Purdom E (2018) "clusterExperiment and RSEC: A Bioconductor package and framework for clustering of single-cell and other large gene expression datasets" PLoS Comput Biol. 2018 Sep 4;14(9):e1006378 [link](http://dx.plos.org/10.1371/journal.pcbi.1006378)
	
	There is a github repository ([epurdom/RSECPaper](https://github.com/epurdom/RSECPaper) for this paper that gives the code for reproducing the analysis in that manuscript.

* A [F1000 workflow](https://f1000research.com/articles/6-1158/v1) demonstrating the use of RSEC for clustering as part of trajectory estimation with the package `slingshot` and normalization with `zinbwave`

	Perraudeau F, Risso D, Street K, Purdom E, and Dudoit S (2017) "Bioconductor workflow for single-cell RNA sequencing: Normalization, dimensionality reduction, clustering, and lineage inference" F1000Research 6:1158. 

* A compiled version of the vignette (i.e. tutorial) of the github version of `clusterExperiment` (corresponding to the `master` branch) can be found (here)[http://bioconductor.org/packages/devel/bioc/vignettes/clusterExperiment/inst/doc/clusterExperimentTutorial.html] 

* The compiled version of the vignette corresponding to the version of `clusterExperiment` available with the latest release of Bioconductor can be found (here)[http://bioconductor.org/packages/release/bioc/vignettes/clusterExperiment/inst/doc/clusterExperimentTutorial.html] 

## Installation instructions

### Installation From Bioconductor

We recommend installation of the package via bioconductor.

```r
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("clusterExperiment")
```

To install the most recent version of the package available on the development branch of bioconductor, follow the above instructions, after downloading the development version of bioconductor (see  [here](https://www.bioconductor.org/developers/how-to/useDevel/) for instructions).

If you are having problems installing/updating the package `gsl` from source because your gsl installation is not found, [this answer](https://stackoverflow.com/a/26616008) (and the comment following it) on stackoverflow may be of help.

### Installation of Github Version:

We generally try to keep the bioconductor *devel* version up-to-date with the *master* branch of this git repository. But since this can require installing R and/or bioconductor development version, it can be convenient to be able to get just the most recent version from github. 

You can install the github version via

```r
library(devtools)
install_github("epurdom/clusterExperiment")
```

### Development branch:

The `develop` branch is our development branch where we are actively updating features, and may contain bugs or be in the process of being updated. You should not use the `develop` branch unless it passes TravisCI checks (see below) and you want to be using a *very* beta version.

The development branch can be installed via the `install_github` command above, but indicating the `develop` branch:

```r
library(devtools)
install_github("epurdom/clusterExperiment", ref="develop")
```

## Status Checks

Below are status checks for the package. Note that occassionally errors do not appear here immediately. Clicking on the link will give you the most up-to-date status.

| Resource:     |  Status   |
| ------------- | ------------ |
| Bioc Release  | [![BiocDevel Status](http://bioconductor.org/shields/build/release/bioc/clusterExperiment.svg)](http://bioconductor.org/checkResults/release/bioc-LATEST/clusterExperiment/)|
| Bioc Development  | [![BiocDevel Status](http://bioconductor.org/shields/build/devel/bioc/clusterExperiment.svg)](http://bioconductor.org/checkResults/devel/bioc-LATEST/clusterExperiment/)|
| Travis CI master   | [![Build Status](https://travis-ci.org/epurdom/clusterExperiment.svg?branch=master)](https://travis-ci.org/epurdom/clusterExperiment) |
| Travis CI develop   | [![Build Status](https://travis-ci.org/epurdom/clusterExperiment.svg?branch=develop)](https://travis-ci.org/epurdom/clusterExperiment) |
| Appveyor master | [![AppVeyor Build Status](https://ci.appveyor.com/api/projects/status/github/epurdom/clusterExperiment?branch=master&svg=true)](https://ci.appveyor.com/project/epurdom/clusterExperiment) |
| Appveyor develop | [![AppVeyor Build Status](https://ci.appveyor.com/api/projects/status/github/epurdom/clusterExperiment?branch=develop&svg=true)](https://ci.appveyor.com/project/epurdom/clusterExperiment) |
<!-- | Test coverage |  [![Coverage Status](https://coveralls.io/repos/github/epurdom/clusterExperiment/badge.svg?branch=develop)](https://coveralls.io/github/epurdom/clusterExperiment?branch=develop) | -->

## Issues and bug reports

Please use https://github.com/epurdom/clusterExperiment/issues to submit issues, bug reports, and comments.

#' @include AllChecks.R
#' @importClassesFrom HDF5Array HDF5Matrix
#' @importClassesFrom DelayedArray DelayedArray DelayedArray1 DelayedMatrix RleArray RleMatrix
#' @importClassesFrom phylobase phylo4 phylo4d
#' @rawNamespace import(phylobase, except = plot)
#' @import Matrix
setClassUnion("matrixOrMissing",members=c("matrix", "missing"))
setClassUnion("phylo4OrNULL",members=c("phylo4d", "NULL"))
setClassUnion("matrixOrNULL",members=c("matrix", "NULL"))
setClassUnion("listOrNULL",members=c("list", "NULL"))
setClassUnion("functionOrNULL",members=c("function", "NULL"))
setClassUnion("data.frameOrNULL",members=c("data.frame", "NULL"))
setClassUnion("matrixOrHDF5",members=c("matrix", "DelayedArray","HDF5Matrix")) #Sometimes it appears necessary to have HDF5Matrix listed separately, not sure why, but otherwise not finding it. 
setClassUnion("matrixOrHDF5OrNULL",members=c("matrix","DelayedArray","HDF5Matrix","NULL"))
setClassUnion("sparseOrHDF5OrNULL",members=c("numeric","sparseMatrix","DelayedArray","HDF5Matrix","NULL"))

#############################################################
#############################################################
############### ClusterExperiment Class #####################
#############################################################
#############################################################
#' @title Class ClusterExperiment
#'
#' @description \code{ClusterExperiment} is a class that extends
#' \code{SingleCellExperiment} and is used to store the data
#' and clustering information.
#'
#' @docType class
#' @aliases ClusterExperiment ClusterExperiment-class ClusterExperiment
#'
#' @description In addition to the slots of the \code{SingleCellExperiment}
#' class, the \code{ClusterExperiment} object has the additional slots described
#' in the Slots section.
#'
#' @description There are several methods implemented for this class. The most
#' important methods (e.g., \code{\link{clusterMany}}, \code{\link{makeConsensus}},
#' ...) have their own help page. Simple helper methods are described in the
#' Methods section. For a comprehensive list of methods specific to this class
#' see the Reference Manual.
#'
#' @slot transformation function. Function to transform the data by when methods
#' that assume normal-like data (e.g. log)
#' @slot clusterMatrix matrix. A matrix giving the integer-valued cluster ids
#' for each sample. The rows of the matrix correspond to clusterings and columns
#' to samples. The integer values are assigned in the order that the clusters
#' were found, if found by setting sequential=TRUE in clusterSingle. "-1" indicates
#' the sample was not clustered.
#' @slot primaryIndex numeric. An index that specifies the primary set of
#' labels.
#' @slot clusterInfo list. A list with info about the clustering.
#' If created from \code{\link{clusterSingle}}, clusterInfo will include the
#' parameter used for the call, and the call itself. If \code{sequential = TRUE}
#' it will also include the following components.
#' \itemize{
#' \item{\code{clusterInfo}}{if sequential=TRUE and clusters were successfully
#' found, a matrix of information regarding the algorithm behavior for each
#' cluster (the starting and stopping K for each cluster, and the number of
#' iterations for each cluster).}
#' \item{\code{whyStop}}{if sequential=TRUE and clusters were successfully
#' found, a character string explaining what triggered the algorithm to stop.}
#' }
#' @slot merge_index index of the current merged cluster
#' @slot merge_cutoff value for the cutoff used to determine whether to merge
#'   clusters
#' @slot merge_dendrocluster_index index of the cluster merged with the current
#'   merge
#' @slot merge_nodeMerge data.frame of information about nodes merged in the
#'   current merge. See \code{\link{mergeClusters}}
#' @slot merge_nodeProp data.frame of information of proportion estimated
#'   non-null at each node of dendrogram. See \code{\link{mergeClusters}}
#' @slot merge_method character indicating method used for merging. See 
#'   \code{\link{mergeClusters}}
#' @slot merge_demethod character indicating the DE method used for merging. See 
#'   \code{\link{mergeClusters}}
#' @slot clusterTypes character vector with the origin of each column of
#' clusterMatrix.
#' @slot dendro_samples \code{\link[phylobase]{phylo4d}} object. A dendrogram
#'   containing the cluster relationship (leaves are samples; see
#'   \code{\link{clusterDendrogram}} for details).
#' @slot dendro_clusters \code{\link[phylobase]{phylo4d}} object. A dendrogram
#'   containing the cluster relationship (leaves are clusters; see see
#'   \code{\link{sampleDendrogram}} for details).
#' @slot dendro_index numeric. An integer giving the cluster that was used to
#'   make the dendrograms. NA_real_ value if no dendrograms are saved.
#' @slot coClustering One of \itemize{ 
#' \item{\code{NULL}, i.e. empty} 
#' \item{a
#'   numeric vector, signifying the indices of the clusterings in the
#'   clusterMatrix that were used for \code{makeConsensus}. This allows for the
#'   recreation of the distance matrix (using hamming distance) if needed for
#'   function \code{plotClusters} but doesn't require storage of full NxN
#'   matrix.} 
#' \item{a \code{\link[Matrix]{sparseMatrix}} object -- a sparse 
#'   representation of the NxN matrix with the cluster co-occurrence 
#'   information; this can either be based on subsampling or on co-clustering 
#'   across parameter sets (see \code{clusterMany}). The matrix is a square
#'   matrix with number of rows/columns equal to the number of samples.}
#' }
#' @slot clusterLegend a list, one per cluster in \code{clusterMatrix}. Each
#' element of the list is a matrix with nrows equal to the number of different
#' clusters in the clustering, and consisting of at least two columns with the
#' following column names: "clusterId" and "color".
#' @slot orderSamples a numeric vector (of integers) defining the order of
#' samples to be used for plotting of samples. Usually set internally by other
#' functions.
#' @seealso \code{\link[Matrix]{sparseMatrix}} \code{\link[phylobase]{phylo4d}} 
#' @name ClusterExperiment-class
#' @aliases ClusterExperiment
#' @rdname ClusterExperiment-class
#' @import SingleCellExperiment
#' @import SummarizedExperiment
#' @import methods
#' @export
#'
setClass(
  Class = "ClusterExperiment",
  contains = "SingleCellExperiment",
  slots = list(
    transformation="function",
    clusterMatrix = "matrix",
    primaryIndex = "numeric",
    clusterInfo = "list",
    clusterTypes = "character",
    clusterLegend="list",
    orderSamples="numeric",
    dendro_samples = "phylo4OrNULL",
    dendro_clusters = "phylo4OrNULL",
    dendro_index = "numeric",
    coClustering = "sparseOrHDF5OrNULL",
	merge_index="numeric",
	merge_dendrocluster_index="numeric",
	merge_method="character",
	merge_demethod="character",
	merge_cutoff="numeric",
	merge_nodeProp="data.frameOrNULL",
	merge_nodeMerge="data.frameOrNULL"

    ),
    prototype = prototype(    
        dendro_samples = NULL,
        dendro_clusters = NULL,
        dendro_index=NA_real_, 
        merge_index=NA_real_,
    	merge_dendrocluster_index=NA_real_,
        coClustering = NULL,
    	merge_method=NA_character_,
    	merge_demethod=NA_character_,
    	merge_cutoff=NA_real_,
        merge_nodeProp=NULL,
    	merge_nodeMerge=NULL
        
        )
)

setValidity("ClusterExperiment", function(object) {
    ####
    #test that clusterInfo not have names
    ####
    if(!is.null(names(object@clusterInfo))) return("clusterInfo should not have names")

	ch<-.checkClusterMatrix(object)
	if(!is.logical(ch))  return(ch)
	ch<-.checkPrimaryIndex(object)
	if(!is.logical(ch))  return(ch)
	ch<-.checkClusterTypes(object)
	if(!is.logical(ch))  return(ch)
	ch<-.checkClusterLegend(object)
	if(!is.logical(ch))  return(ch)
	ch<-.checkOrderSamples(object)
	if(!is.logical(ch))  return(ch)
	ch<-.checkClusterLabels(object)
	if(!is.logical(ch))  return(ch)
	ch<-.checkMerge(object)
	if(!is.logical(ch))  return(ch)
	ch<-.checkDendrogram(object)
	if(!is.logical(ch))  return(ch)
	ch<-.checkCoClustering(object)
	if(!is.logical(ch))  return(ch)
	return(TRUE)
})

#'@description The constructor \code{ClusterExperiment} creates an object of the
#'  class \code{ClusterExperiment}. However, the typical way of creating these
#'  objects is the result of a call to \code{\link{clusterMany}} or
#'  \code{\link{clusterSingle}}.
#'
#'@description Note that when subsetting the data, the co-clustering and
#'  dendrogram information are lost.
#'
#'@param object a matrix or \code{SummarizedExperiment} or
#'  \code{SingleCellExperiment} containing the data that was clustered.
#'@param clusters can be either a numeric or character vector, a factor, or a
#'  numeric matrix, containing the cluster labels.
#'@param transformation function. A function to transform the data before
#'  performing steps that assume normal-like data (i.e. constant variance), such
#'  as the log.
#'@param ... The arguments \code{transformation}, \code{clusterTypes} and
#'  \code{clusterInfo} to be passed to the constructor for signature
#'  \code{SingleCellExperiment,matrix}.
#'
#'@return A \code{ClusterExperiment} object.
#'
#'@examples
#'
#'sce <- matrix(data=rnorm(200), ncol=10)
#'labels <- gl(5, 2)
#'
#'cc <- ClusterExperiment(sce, as.numeric(labels), transformation =
#'function(x){x})
#'
#' @rdname ClusterExperiment-class
#' @export
setGeneric(
  name = "ClusterExperiment",
  def = function(object,  clusters,...) {
    standardGeneric("ClusterExperiment")
  }
)
#' @rdname ClusterExperiment-class
#' @export
setMethod(
  f = "ClusterExperiment",
  signature = signature("matrixOrHDF5","ANY"),
  definition = function(object, clusters, ...){
    ClusterExperiment(SummarizedExperiment(object), clusters, ...)
  })
#' @rdname ClusterExperiment-class
setMethod(
  f = "ClusterExperiment",
  signature = signature("SummarizedExperiment", "ANY"),
  definition = function(object, clusters, ...){
  ClusterExperiment(as(object, "SingleCellExperiment"),clusters,...)
})
#' @rdname ClusterExperiment-class
setMethod(
  f = "ClusterExperiment",
  signature = signature("SingleCellExperiment", "numeric"),
  definition = function(object, clusters, ...){
  ClusterExperiment(object,matrix(clusters, ncol=1),...)
})
#' @rdname ClusterExperiment-class
setMethod(
  f = "ClusterExperiment",
  signature = signature("SingleCellExperiment","character"),
  definition = function(object, clusters,...){
    ClusterExperiment(object,matrix(clusters,ncol=1),...)
    })
#' @rdname ClusterExperiment-class
setMethod(
  f = "ClusterExperiment",
  signature = signature("SingleCellExperiment","factor"),
  definition = function(object, clusters,...){
    clusters <- as.character(clusters)
    ClusterExperiment(object,clusters,...)
  })
#'@rdname ClusterExperiment-class
#'@param clusterTypes a string describing the nature of the clustering. The
#'  values `clusterSingle`, `clusterMany`, `mergeClusters`, `makeConsensus` are
#'  reserved for the clustering coming from the package workflow and should not
#'  be used when creating a new object with the constructor.
#'@param clusterInfo a list with information on the clustering (see Slots).
#'@param primaryIndex integer. Sets the `primaryIndex` slot (see Slots).
#'@param orderSamples a vector of integers. Sets the `orderSamples` slot (see
#'  Slots).
#'@param dendro_samples phylo4 object. Sets the `dendro_samples` slot (see Slots).
#'@param dendro_clusters phylo4 object. Sets the `dendro_clusters` slot (see
#'  Slots).
#'@param dendro_index numeric. Sets the \code{dendro_index} slot (see Slots).
#'@param coClustering matrix. Sets the \code{coClustering} slot (see Slots).
#'@param checkTransformAndAssay logical. Whether to check the content of the
#'  assay and given transformation function for whether they are valid.
#'@param merge_index integer. Sets the \code{merge_index} slot (see Slots)
#'@param merge_cutoff numeric. Sets the \code{merge_cutoff} slot (see Slots)
#'@param merge_dendrocluster_index integer. Sets the
#'  \code{merge_dendrocluster_index} slot (see Slots)
#'@param merge_demethod character, Sets the
#'  \code{merge_demethod} slot (see Slots)
#'@param merge_nodeMerge data.frame. Sets the \code{merge_nodeMerge} slot (see
#'  Slots)
#'@param merge_nodeProp data.frame. Sets the \code{merge_nodeProp} slot (see
#'  Slots)
#'@param merge_method character, Sets the \code{merge_method} slot (see Slots)
#'@param clusterLegend list, Sets the \code{clusterLegend} slot (see details).

#' @details The \code{clusterLegend} argument to \code{ClusterExperiment} 
#'  must be a valid clusterLegend format and match the values in \code{clusters}, 
#' in that the "clusterIds" column must matches the value in the clustering matrix
#'  \code{clusters}. If \code{names(clusterLegend)==NULL}, it is assumed that the 
#'  entries of \code{clusterLegend} are in the same order as the columns of 
#'  \code{clusters}. Generally, this is not a good way for users to set the
#' clusterLegend slot.
#' @details The \code{ClusterExperiment} constructor function gives
#'   clusterLabels based on the column names of the input
#'   matrix/SingleCellExperiment. If missing, will assign labels
#'   "cluster1","cluster2", etc.
#' @details Note that the validity check when creating a new
#'   \code{ClusterExperiment} object with \code{new} is less extensive than when
#'   using \code{ClusterExperiment} function with
#'   \code{checkTransformAndAssay=TRUE} (the default). Users are advised to use
#'   \code{ClusterExperiment} to create new \code{ClusterExperiment} objects.
setMethod(
  f = "ClusterExperiment",
  signature = signature("SingleCellExperiment","matrix"),
  definition = function(object, clusters,
                        transformation=function(x){x},
                        primaryIndex=1,
                        clusterTypes="User",
                        clusterInfo=NULL,
                        orderSamples=seq_len(ncol(object)),
                        dendro_samples=NULL,
                        dendro_index=NA_real_,
                        dendro_clusters=NULL,
                        coClustering=NULL,
                        merge_index=NA_real_,
                        merge_cutoff=NA_real_,
                        merge_dendrocluster_index=NA_real_,
                        merge_nodeProp=NULL,
                        merge_nodeMerge=NULL,
                        merge_method=NA_character_,
												merge_demethod=NA_character_,
                        clusterLegend=NULL,
                        checkTransformAndAssay=TRUE
  ){
    if(NCOL(object) != nrow(clusters)) {
      stop("`clusters` must be a matrix of rows equal to the number of
           samples.")
    }
    if(length(clusterTypes)==1) {
      clusterTypes <- rep(clusterTypes, length=NCOL(clusters))
    }
    if(is.null(clusterInfo)) {
      clusterInfo<-rep(list(NULL),length=NCOL(clusters))
    }
    if(length(clusterTypes)!=NCOL(clusters)) {
      stop("clusterTypes must be of length equal to number of clusters in
           `clusters`")
    }
    #fix up names of clusters and match
    
    if(is.null(colnames(clusters))){
      colnames(clusters)<-paste("cluster",seq_len(NCOL(clusters)),sep="")
    }
    if(any(duplicated(colnames(clusters)))){#probably not possible
      colnames(clusters)<-make.names(colnames(clusters),unique=TRUE)
    }
    if(length(clusterTypes) == 1) {
      clusterTypes <- rep(clusterTypes, length=NCOL(clusters))
    }
    if(is.null(clusterInfo)) {
      clusterInfo <- rep(list(NULL), length=NCOL(clusters))
    }
    #make clusters consecutive integer valued:
    if(nrow(clusters)>0){
	    tmp<-.makeColors(clusters, colors=massivePalette,matchClusterLegend=clusterLegend,matchTo="givenIds")
	    if(is.null(clusterLegend)){
	    	clusterLegend<-tmp$colorList
	    } 
	    else{
	      #need to check matches the clusters, which .makeColors doesn't do.
	      clusterLegend<-unname(clusterLegend)
	      ch<-.checkClustersWithClusterLegend(clusters,clusterLegend)
	      if(!is.logical(ch)) stop(ch)
	 			clusterLegend<-tmp$colorList      
	    }
	    clustersNum<-tmp$numClusters
	    colnames(clustersNum)<-colnames(clusters)
		
    }
	else{
		clustersNum<-clusters
		clusterLegend<-lapply(seq_len(ncol(clusters)),function(ii){
			out<-matrix(nrow=0,ncol=3)
			colnames(out)<-c("clusterIds","color","name")
			return(out)
		})
	}
    #can just give object in constructor, and then don't loose any information!
    out <- new("ClusterExperiment",
               object,
               transformation=transformation,
               clusterMatrix = clustersNum,
               primaryIndex = primaryIndex,
               clusterTypes = unname(clusterTypes),
               clusterInfo=unname(clusterInfo),
               clusterLegend=unname(clusterLegend),
               orderSamples=seq_len(ncol(object)),
               dendro_samples=dendro_samples,
               dendro_clusters=dendro_clusters,
               dendro_index=dendro_index,
               merge_index=merge_index,
               merge_cutoff=merge_cutoff,
               merge_dendrocluster_index=merge_dendrocluster_index,
               merge_nodeProp=merge_nodeProp,
               merge_nodeMerge=merge_nodeMerge,
               merge_method=merge_method,
			   merge_demethod=merge_demethod,
               coClustering=coClustering
               
    )
    if(checkTransformAndAssay){
      chass<-.checkAssays(out)
      if(is.logical(chass) && !chass) stop(chass)
      chtrans<-.checkTransform(out)
      if(is.logical(chtrans) && !chtrans) stop(chtrans)
    }
    return(out)
  })

context("Dimensionality Reduction")


test_that("makeReducedDims works as promised",{
  # check for all objects that get expected
  nDim<-3
  true3<-abs(reducedDim(sceSimDataDimRed,"PCA")[,1:nDim])

  #note: cc gives rownames to everything, so need to unname it
  expect_silent(dr3<-makeReducedDims(ceSimData,reducedDims="PCA",maxDims=nDim))
  expect_equal(unname(true3),unname(abs(reducedDim(dr3,"PCA"))))

  expect_silent(dr3<-makeReducedDims(simData,reducedDims="PCA",maxDims=nDim))
  expect_equal(true3,abs(reducedDim(dr3,"PCA")))

  expect_silent(dr3<-makeReducedDims(seSimData,reducedDims="PCA",maxDims=nDim))
  expect_equal(true3,abs(reducedDim(dr3,"PCA")))

  expect_silent(dr3<-makeReducedDims(sceSimData,reducedDims="PCA",maxDims=nDim))
  expect_equal(true3,abs(reducedDim(dr3,"PCA")))

  #check don't lose them if call on existing object
  expect_silent(dr<-makeReducedDims(sceSimDataDimRed,reducedDims="PCA",maxDims=nDim))
  expect_equal(reducedDims(sceSimDataDimRed),reducedDims(dr))

  #check with maxDims<1 (picks 4 of dimensions apparently -- never checked was correct)
  expect_silent(dr2<-makeReducedDims(cc,reducedDims="PCA",maxDims=0.5))
  expect_equal(abs(reducedDim(dr2,"PCA")[,1:4]), abs(reducedDim(dr2,"PCA")))

})

test_that("Filter functions work as expected",{

	#when there are no filters
	expect_silent(filterNames(sceSimData))
	expect_equal(clusterExperiment:::filterStats(sceSimData),rowData(sceSimData)[,"b",drop=FALSE])
	expect_error(clusterExperiment:::filterStats(sceSimData,type="Filter1"),"None of the values of 'type' argument are valid filter names")

	##Redo when no rowData.
	testSe<-SummarizedExperiment(mat)
	expect_equal(length(filterNames(testSe)),0)
	expect_equal(ncol(clusterExperiment:::filterStats(testSe)),0)
	expect_error(clusterExperiment:::filterStats(testSe,type="Filter1"),"There are no filter statistics saved for this object")

	##Redo when no valid filters.
	testSe<-SummarizedExperiment(mat,rowData=gData[,c("a","c")])
	expect_equal(length(filterNames(testSe)),0)
	expect_equal(ncol(clusterExperiment:::filterStats(testSe)),0)
	expect_error(clusterExperiment:::filterStats(testSe,type="Filter1"),"None of the values of 'type' argument are valid filter names")

	set.seed(352)
	filter<-rnorm(nrow(sce))
	filter2<-matrix(rnorm(2*nrow(sce)),ncol=2)

	#tests on adding unnamed filters
	#value type of replace:
	sceTest<-sce
	expect_silent(clusterExperiment:::filterStats(sceTest,type="Filter1")<-filter)
	expect_equal(colnames(clusterExperiment:::filterStats(sceTest)),c("b","Filter1"))

	sceTest<-sce
	expect_silent(clusterExperiment:::filterStats(sceTest,type=c("Filter1","Filter2"))<-filter2)
	expect_equal(colnames(clusterExperiment:::filterStats(sceTest)),c("b","Filter1","Filter2"))

	#matrix type of replace:
	sceTest<-sce
	expect_error(clusterExperiment:::filterStats(sceTest)<-filter,"must give matrix of values with column names")
	sceTest<-sce
	expect_error(clusterExperiment:::filterStats(sceTest)<-filter2,"must give matrix of values with column names")

	####Now work with existing filters
	colnames(filter2)<-c("Filter1","Filter2")
	sceTest<-sce
	expect_silent(clusterExperiment:::filterStats(sceTest)<-filter2)
	expect_silent(filterNames(sceTest))
	expect_equivalent(clusterExperiment:::filterStats(sceTest),DataFrame(b=rowData(sce)[,"b"],filter2))
	expect_equivalent(clusterExperiment:::filterStats(sceTest,type="Filter1"),DataFrame(filter2)[,"Filter1",drop=FALSE])
	expect_error(clusterExperiment:::filterStats(sceTest,type="Myfilter"),"None of the values of 'type' argument are valid filter names")
	expect_error(clusterExperiment:::filterStats(sceTest,type=1),"unable to find an inherited method")

	##Check Replace with existing filters in place and check actually change them, etc.
	sceTest<-SummarizedExperiment(assay(sce))
	expect_silent(clusterExperiment:::filterStats(sceTest)<-filter2)
	filter3<-filter2+1
	expect_silent(clusterExperiment:::filterStats(sceTest) <- filter3)
	expect_equivalent(clusterExperiment:::filterStats(sceTest),DataFrame(filter3))

	sceTest<-SummarizedExperiment(assay(sce))
	expect_silent(clusterExperiment:::filterStats(sceTest)<-filter2)
	filter4<-filter2+5
	colnames(filter4)[2]<-"Myfilter"
	expect_silent(clusterExperiment:::filterStats(sceTest) <- filter4)
	expect_equivalent(clusterExperiment:::filterStats(sceTest),DataFrame(filter4[,"Filter1",drop=FALSE],filter2[,"Filter2",drop=FALSE],filter4[,"Myfilter",drop=FALSE]))

	#Need more checks about replacement, etc.

})

test_that("reduce and filter work with hdf5",{
	expect_silent(filterNames(hdfSCE))

	expect_silent(f1<-filterData(hdfSCE,filterStats="Filter1",cutoff=1))
	expect_silent(f1<-filterData(hdfSCE,filterStats="Filter1",cutoff=1))

	expect_silent(fs<-makeFilterStats(hdfSCE,filterStats="var"))
	expect_silent(fs<-makeFilterStats(hdfSCE,filterStats=c("mean","var")))
	expect_silent(out<-filterData(fs,filterStats=c("mean"),cutoff=1))
	expect_silent(fs<-makeFilterStats(assay(hdfSCE),filterStats="var"))


	expect_silent(defaultNDims(hdfObj,"PCA"))

	#add pca to it
	nDim<-3
	expect_silent(dr3<-makeReducedDims(hdfObj,reducedDims="PCA",maxDims=nDim))
	expect_equal(defaultNDims(dr3,"PCA"),3)

	#test directly on hdf5
	expect_silent(dr3<-makeReducedDims(assay(hdfObj),reducedDims="PCA",maxDims=nDim))


	#test transformation -- need make CE object
    expect_silent(clustNothing1 <- clusterSingle(hdfObj,
		  reduceMethod = "none",
  		  mainClusterArgs=list( clusterArgs=list(k=3),clusterFunction=listBuiltInFunctions()[[1]]),
  	       subsample=FALSE, sequential=FALSE, isCount=FALSE))

	expect_silent(transformation(clustNothing1)<-function(x){exp(x)})
	expect_equal(exp(assay(clustNothing1)),unname(transformData(clustNothing1)))



})


test_that("filterData works as expected",{

	###Cutoff filter
	set.seed(352)
	filter2<-matrix(rnorm(2*nrow(sceSimData)),ncol=2)
	colnames(filter2)<-c("Filter1","Filter2")
	expect_silent(clusterExperiment:::filterStats(sceSimData)<-filter2)


	tf<-filter2[,"Filter1"]>1
	expect_silent(f1<-filterData(sceSimData,filterStats="Filter1",cutoff=1))
	expect_equal(NROW(f1),sum(tf))
	expect_equal(assay(f1),assay(sceSimData)[tf,])

	tf<-abs(filter2[,"Filter1"])>1
	expect_silent(f1<-filterData(sceSimData,filterStats="Filter1",cutoff=1,absolute=TRUE))
	expect_equal(NROW(f1),sum(tf))
	expect_equal(assay(f1),assay(sceSimData)[tf,])

	tf<-abs(filter2[,"Filter1"])<1
	expect_silent(f1<-filterData(sceSimData,filterStats="Filter1",cutoff=1,keepLarge=FALSE,absolute=TRUE))
	expect_equal(NROW(f1),sum(tf))
	expect_equal(assay(f1),assay(sceSimData)[tf,])
	tf<-filter2[,"Filter1"]<1
	expect_silent(f1<-filterData(sceSimData,filterStats="Filter1",cutoff=1,keepLarge=FALSE,absolute=FALSE))
	expect_equal(NROW(f1),sum(tf))
	expect_equal(assay(f1),assay(sceSimData)[tf,])

	#percentile number filter
	tf<-order(filter2[,"Filter1"],decreasing=TRUE)[1:10]
	expect_silent(f1<-filterData(sceSimData,filterStats="Filter1",percentile=10))
	expect_equal(NROW(f1),length(tf))
	expect_equal(assay(f1),assay(sceSimData)[tf,])

	tf<-order(abs(filter2[,"Filter1"]),decreasing=TRUE)[1:10]
	expect_silent(f1<-filterData(sceSimData,filterStats="Filter1",percentile=10,absolute=TRUE))
	expect_equal(NROW(f1),length(tf))
	expect_equal(assay(f1),assay(sceSimData)[tf,])

	tf<-order(abs(filter2[,"Filter1"]),decreasing=FALSE)[1:10]
	expect_silent(f1<-filterData(sceSimData,filterStats="Filter1",percentile=10,keepLarge=FALSE,absolute=TRUE))
	expect_equal(NROW(f1),length(tf))
	expect_equal(assay(f1),assay(sceSimData)[tf,])
	tf<-order((filter2[,"Filter1"]),decreasing=FALSE)[1:10]
	expect_silent(f1<-filterData(sceSimData,filterStats="Filter1",percentile=10,keepLarge=FALSE,absolute=FALSE))
	expect_equal(NROW(f1),length(tf))
	expect_equal(assay(f1),assay(sceSimData)[tf,])

	###Need to add test for percentile in (0,1)


})

test_that("getReducedData works as promised",{
  expect_silent(clustNothingDimRed <- clusterSingle(sceSimDataDimRed,
                                               reduceMethod = "none",
                                               mainClusterArgs=list( clusterArgs=list(k=3),clusterFunction=listBuiltInFunctions()[[1]]),
                                               subsample=FALSE, sequential=FALSE, isCount=FALSE))  	
  expect_silent(clustNothing <- clusterSingle(sceSimData,
                                               reduceMethod = "none",
                                               mainClusterArgs=list( clusterArgs=list(k=3),clusterFunction=listBuiltInFunctions()[[1]]),
                                               subsample=FALSE, sequential=FALSE, isCount=FALSE))  	
  

  #dimReduce
  expect_silent(out1<-getReducedData(clustNothing,reduceMethod="PCA"))
  expect_true("PCA" %in% reducedDimNames(out1))
  expect_equal(colData(out1),colData(clustNothing))
  expect_equal(rownames(out1),rownames(clustNothing))
  expect_equal(colnames(out1),colnames(clustNothing))
  expect_equal(metadata(out1),metadata(clustNothing))
  
  
  expect_warning(getReducedData(clustNothingDimRed,reduceMethod="PCA"),"will not add reduced dataset to object because already exists method with that name")
  ## FIXME:
  #expect_silent(
  out1<-getReducedData(clustNothing,
      reduceMethod="PCA",reducedDimName="MyPCA")
  #)
  expect_true("MyPCA" %in% reducedDimNames(out1))
  expect_false("PCA" %in% reducedDimNames(out1))
  
  expect_warning(getReducedData(clustNothingDimRed,reduceMethod="PCA",nDim=200,reducedDimName="MyPCA",returnValue="list"),"requesting an existing dimensionality reduction but with greater number of dimensions than available")
  expect_silent(out3<-getReducedData(clustNothingDimRed,reduceMethod="PCA",nDim=50,reducedDimName="MyPCA",returnValue="list"))
  expect_equal(nrow(out3$dataMatrix),50)
  
  #filters
  expect_silent(out2<-getReducedData(clustNothing,reduceMethod="mad",nDim=50))
  expect_true("mad_clusterSingle" %in% filterNames(out2))
  expect_true("filteredBy_mad_clusterSingle" %in% reducedDimNames(out2))
  expect_equal(ncol(reducedDim(out2,"filteredBy_mad_clusterSingle")),50)
  
  expect_silent(out5<-getReducedData(clustNothing,reduceMethod="mad",filterIgnoresUnassigned=FALSE))
  expect_true("mad" %in% filterNames(out5))
  expect_true("filteredBy_mad" %in% reducedDimNames(out5))
  
})
  

test_that("makeFilterStats works as promised",{
	expect_silent(fs<-makeFilterStats(mat,filterStats="var"))
	expect_silent(fs<-makeFilterStats(mat,filterStats=c("mean","var")))

	#does all:
	expect_silent(fs<-makeFilterStats(mat))
	for(ii in listBuiltInFilterStats()){
		if(ii=="abscv"){
			x<-abs(sqrt(apply(mat,1,var))/apply(mat,1,mean))
			x<-unname(x)
		}
		else x<-(unname(apply(mat,1,ii)))
		x<-DataFrame(x)
		colnames(x)<-ii
		expect_equivalent(clusterExperiment:::filterStats(fs,type=ii),x)
	}


	expect_silent(fs<-makeFilterStats(cc,filterStats="var"))
	expect_silent(fs<-makeFilterStats(cc,filterStats=c("mean","var")))
	expect_silent(fs<-makeFilterStats(cc))
	expect_silent(fs<-makeFilterStats(cc,whichClusterIgnore="Cluster1"))

	expect_silent(fs<-makeFilterStats(se))
	expect_silent(fs<-makeFilterStats(se,filterStats="var"))
	expect_silent(fs<-makeFilterStats(se,filterStats=c("mean","var")))
	expect_equal(sort(filterNames(fs)),sort(c("b","mean","var")))
	expect_silent(fs<-makeFilterStats(fs,filterStats=c("var")))
	expect_equal(sort(filterNames(fs)),sort(c("b","mean","var")))
	expect_silent(fs<-makeFilterStats(fs,filterStats=c("abscv")))
	expect_equal(sort(filterNames(fs)),sort(c("b","mean","var","abscv")))

	expect_silent(fs<-makeFilterStats(sceSimData,filterStats="var"))
	expect_silent(fs<-makeFilterStats(sceSimData,filterStats=c("mean","var")))
	expect_silent(fs<-makeFilterStats(sceSimData))

	###Check getting out correct ones.
	contData<-simData[,1:20]
    expect_silent(temp<-makeFilterStats(contData))
	v<-apply(contData,1,var)
	expect_equal(clusterExperiment:::filterStats(temp,type="var"),DataFrame("var"=v))
	#should keep top 10
	expect_silent(x<-filterData(temp,percentile=10,filterStats="var",keepLarge=TRUE,absolute=FALSE))
	whTop<-order(v,decreasing=TRUE)[1:10]
	expect_equal(DataFrame("var"=v[whTop]),clusterExperiment:::filterStats(x,type="var"))
	expect_equal(contData[whTop,],unname(assay(x)))

	###Check getting out correct ones with transformation (isCount=TRUE)
	countData<-simCount[,1:20]
    expect_silent(temp<-makeFilterStats(countData,isCount=TRUE))
	v<-apply(log2(countData+1),1,var)
	expect_equal(clusterExperiment:::filterStats(temp,type="var"),DataFrame("var"=v))
	#should keep top 10
	expect_silent(x<-filterData(temp,percentile=10,filterStats="var",keepLarge=TRUE,absolute=FALSE))
	whTop<-order(v,decreasing=TRUE)[1:10]
	expect_equal(DataFrame("var"=v[whTop]),clusterExperiment:::filterStats(x,type="var"))
	expect_equal(log2(countData[whTop,]+1),unname(transformData(x,isCount=TRUE)))


})

test_that("defaultNDims works",{
    nDim<-3
    expect_silent(dr3<-makeReducedDims(ceSimData,reducedDims="PCA",maxDims=nDim))
    expect_equal(defaultNDims(dr3,"PCA"),3)

	expect_equal(defaultNDims(SingleCellExperiment(mat),"PCA"),min(dim(mat)))
	expect_equal(defaultNDims(SingleCellExperiment(ceSimData),"PCA"),50)

})

context("Constructor")
		  
test_that("`ClusterExperiment` constructor works with matrix and SummarizedExperiments and SingleCellExperiment", {
            expect_error(ClusterExperiment(mat), 
                "is missing, with no default")
			#Also creates warnings, which show up in summary, sort of annoying.
			expect_error(suppressWarnings(ClusterExperiment(mat,
                as.numeric(numLabels), transformation=log)), 
                info="Error checking transFun")
            expect_error(ClusterExperiment(mat, numLabels[1:2]),
                         "must be a matrix of rows equal")
            expect_error(ClusterExperiment(as.data.frame(mat), numLabels),
                         "unable to find an inherited method for function")
            #test character input
            expect_silent(ccChar<-ClusterExperiment(mat, chLabels))
            expect_is(primaryCluster(ccChar),"numeric")
            expect_is(primaryClusterNamed(ccChar),"character")
            expect_equal(sort(unique(primaryClusterNamed(ccChar))),
                sort(unique(chLabels)))

            #test factor input
            expect_silent(ClusterExperiment(mat, numLabels))

            expect_is(cc, "ClusterExperiment")
            expect_is(cc, "SummarizedExperiment")

            expect_equal(nSamples(cc),ncol(mat))
            expect_equal(nFeatures(cc),nrow(mat))
            expect_equal(nClusterings(cc),2)
            expect_equal(NCOL(clusterMatrix(ccSE)), NCOL(labMat))
            
            expect_equal(colData(ccSE),colData(se)) 
            expect_equal(rownames(ccSE),rownames(se)) 
            expect_equal(colnames(ccSE),colnames(se)) 
            expect_equal(metadata(ccSE),metadata(se)) 
            expect_equal(rowData(ccSE),rowData(se)) 
            
            expect_output(show(ccSE))
			
			expect_silent(ccSCE<-ClusterExperiment(sce,as.numeric(numLabels)))
			###Need to add things specific to sce, but first need to build a sce object with new things.
			
})


test_that("`getClusterIndex` works", {
	expect_silent( getClusterIndex(ccSE,whichClusters="all"))
	expect_silent(getClusterIndex(ccSE,whichClusters="primary"))
	expect_silent(getClusterIndex(ccSE,whichClusters="dendro"))
	expect_silent(getClusterIndex(ccSE,whichClusters="workflow"))

	expect_error(getClusterIndex(ccSE,whichClusters="dendro",noMatch="throwError"),"There is no dendrogram")
	expect_error(getClusterIndex(ccSE,whichClusters="workflow",noMatch="throwError"),"There are no workflow clusterings")
	

	expect_error(getClusterIndex(ccSE,whichClusters="dendro",noMatch="throwError"),"There is no dendrogram")
	expect_error(getClusterIndex(ccSE,whichClusters="workflow",noMatch="throwError"),"There are no workflow clusterings")
	
	expect_error( getSingleClusterIndex(ccSE,whichClusters="primary"),"The argument of this function is 'whichCluster'")
	expect_error( getSingleClusterIndex(ccSE,whichCluster="all"),"Invalid value for 'whichCluster'.")
	expect_silent(getSingleClusterIndex(ccSE,whichCluster="primary"))
	expect_error(getSingleClusterIndex(ccSE,whichCluster="dendro"),"There is no dendrogram")
	expect_error(getSingleClusterIndex(ccSE,whichCluster="workflow"),"There are no workflow clusterings")
	
	
    expect_message(ccM<-makeConsensus(ceSim,proportion=.2,clusterLabel="mySpecialLabel"),"no clusters specified to combine")
    expect_message(ccM<-makeConsensus(ccM,proportion=.2),"no clusters specified to combine")
    expect_silent(indCM<-getClusterIndex(ccM,wh=c("clusterMany")))
    expect_equal(length(indCM),sum(clusterTypes(ccM)=="clusterMany"))
    expect_equal(getClusterIndex(ccM,wh=c("mySpecialLabel","makeConsensus")),c(2,1))
    expect_equal(getClusterIndex(ccM,wh=c("mySpecialLabel","clusterMany")),c(2,indCM))
	expect_equal(getClusterIndex(ccM,wh=c("makeConsensus")),1)
	expect_equal(getClusterIndex(ccM,wh=c("makeConsensus","garbage"),noMatch="silentlyRemove"),1)
})

test_that("`ClusterExperiment` constructor works with hdf5",{
    #test creation
    expect_silent(ClusterExperiment(assay(hdfSCE), sample(1:3,size=ncol(hdfSCE),replace=TRUE)))
	
	
})

test_that("whichClusters works with clusterMatrix",{
	 expect_silent(x<-dim(clusterMatrix(ceSim)))
	 expect_equal(dim(clusterMatrix(ceSim,whichClusters="all")),x)
	 expect_equal(ncol(clusterMatrix(ceSim,whichClusters="workflow")),12)
	 expect_equal(ncol(clusterMatrix(ceSim,whichClusters=1:3)),3)
	 expect_equal(ncol(clusterMatrix(ceSim,whichClusters="dendro")),0)
})

test_that("subsetting works as promised",{
	#test zero-length subsetting
	expect_output(show(cc[,logical(length=0L)]))
	
  ###Note, this test only works because grabbing samples with clustering Index 1. Otherwise will renumber.
	newName<-letters[1:nClusters(cc)["Cluster1"]]
	names(newName)<-as.character(1:nClusters(cc)["Cluster1"])
	expect_silent(ccNamed<-renameClusters(cc,whichCluster="Cluster1",value=newName,matchTo="clusterIds"))
	expect_equal(tableClusters(cc,whichCluster="Cluster1",useNames =FALSE),tableClusters(ccNamed,whichCluster="Cluster1",useNames =FALSE))
	
	cc<-ccNamed
	expect_silent(test1<-clusterMatrix(cc[1:2,2]))
	expect_silent(test2<-clusterMatrix(cc)[2,,drop=FALSE])
  expect_equal(test1,test2) 
  
	#test if have duplicated names
	# changed ccNamed so not unique names in Cluster1 
	ccNamed<-renameClusters(ccNamed,whichCluster="Cluster1",c("1"="b"),matchTo="clusterIds")
	newName<-LETTERS[1:nClusters(cc)["Cluster2"]]
	names(newName)<-as.character(1:nClusters(cc)["Cluster2"])
	ccNamed<-renameClusters(ccNamed,whichCluster="Cluster2",newName,matchTo="clusterIds")	
	expect_warning(sub<-ccNamed[,1:5],"Some clusterings do not have unique names")
	
	#####
	#test pulls colors and names correctly. 
	#####
	# get clusterLegend info without subsetting
	expect_silent(ids<-clusterMatrix(cc)[c(13,10,4),"Cluster1"]) 
	expect_silent(cl<-clusterLegend(cc)[["Cluster1"]])
	oldNames<-cl[cl[,"clusterIds"] %in% as.character(ids),"name"]

	# get it via subsetting
	expect_silent(cl2<-clusterLegend(cc[,c(13,10,4)])[["Cluster1"]]) #do it via subsetting
	#check that all of new in old and vice versa(i.e. didn't give them new names)
	expect_equal(sort(cl2[,"name"]),oldNames)
	#check right color with name
	m<-match(cl2[,"name"],cl[,"name"])
	expect_equal(cl2[,c("color","name")],cl[m,c("color","name")])
	
	###Subset entire clusters and check get them all, etc.
	wh<-which(clusterMatrixNamed(cc) %in% oldNames)
	#check same tabulations
	expect_equal(tableClusters(cc[,wh],whichCluster="Cluster1"),tableClusters(cc,whichCluster="Cluster1")[oldNames])
	
  ###But this tests names stay the same regardless, even when renumber.
  expect_equal(clusterMatrixNamed(cc[1:2,1:5]),clusterMatrixNamed(cc)[1:5,,drop=FALSE]) 
  
  expect_equal(clusterMatrix(cc[1:2,-c(1, 2)]),clusterMatrix(cc)[-c(1, 2),,drop=FALSE]) 
  
  #test subsetting of genes
  expect_equal(clusterMatrix(cc[1:2,c(1, 2)]),clusterMatrix(cc)[c(1, 2),,drop=FALSE]) 
  expect_equal(dim(cc[1:2,c(1, 2)]),c(2,2))
  
  #test subsetting of samples
  expect_equal(clusterMatrix(cc[,c(1, 2)]),clusterMatrix(cc)[c(1, 2), ,drop=FALSE])
  logVec<-rep(FALSE,length=nSamples(cc))
  logVec[1:2]<-TRUE
  expect_equal(clusterMatrix(cc[,logVec]),clusterMatrix(cc)[logVec, ,drop=FALSE]) 
  expect_equal(clusterMatrix(cc[,c("Sample 1" , "Sample 2")]),clusterMatrix(cc)[c(1, 2),,drop=FALSE]) 

  
  
  ##########
  #checks SE info (which isn't biggest place needs unit tests since uses callNextMethod() so should work)
  #therefore currently just really checks no errors. 
  expect_silent(x1<-ccSE[,5:8])
  ###Check retain SE info
  expect_equal(colData(x1),colData(se[,5:8]) )
  expect_equal(rownames(x1),rownames(se[,5:8])) 
  expect_equal(colnames(x1),colnames(se[,5:8])) 
  expect_equal(metadata(x1),metadata(se[,5:8])) 
  expect_equal(rowData(x1),rowData(se[,5:8])) 
  
  expect_silent(x2<-ccSE[1:2,])
  ###Check retain SE info
  expect_equal(colData(x2),colData(se[1:2,]) )
  expect_equal(rownames(x2),rownames(se[1:2,])) 
  expect_equal(colnames(x2),colnames(se[1:2,])) 
  expect_equal(metadata(x2),metadata(se[1:2,])) 
  expect_equal(rowData(x2),rowData(se[1:2,])) 
  
  expect_silent(x3<-ccSE[,3])
  ###Check retain SE info
  expect_equal(colData(x3),colData(se[,3]) )
  expect_equal(rownames(x3),rownames(se[,3])) 
  expect_equal(colnames(x3),colnames(se[,3])) 
  expect_equal(metadata(x3),metadata(se[,3])) 
  expect_equal(rowData(x3),rowData(se[,3])) 
  
  expect_silent(x4<-ccSE[3,])
  ###Check retain SE info
  expect_equal(colData(x4),colData(se[3,]) )
  expect_equal(rownames(x4),rownames(se[3,])) 
  expect_equal(colnames(x4),colnames(se[3,])) 
  expect_equal(metadata(x4),metadata(se[3,])) 
  expect_equal(rowData(x4),rowData(se[3,])) 
  
  
  expect_equal(clusterExperiment:::filterStats(sceSimData[1:10,]),head(clusterExperiment:::filterStats(sceSimData),10))
})


test_that("adding clusters work as promised",{
  ##########
  #addClusterings
	vec<-rep(c(-1, 1, 2), each=5)
  expect_silent(c1 <- addClusterings(ccSE, vec,clusterTypes="newUser"))
	expect_silent(c2 <- addClusterings(ccSE, as.character(vec),clusterTypes="newUser"))
	expect_silent(c3 <- addClusterings(ccSE, factor(vec),clusterTypes="newUser"))
	expect_equal(c1,c2)
	expect_equal(c1,c3)
  ###Check retain SE info
  expect_equal(colData(c1),colData(se)) 
  expect_equal(rownames(c1),rownames(se)) 
  expect_equal(colnames(c1),colnames(se)) 
  expect_equal(metadata(c1),metadata(se)) 
  expect_equal(rowData(c1),rowData(se)) 
  #Other checks
  expect_equal(NCOL(clusterMatrix(c1)), nClusterings(ccSE)+1)
  expect_equal(unname(clusterTypes(c1)), unname(c(clusterTypes(ccSE),"newUser")))
  expect_equal(length(clusteringInfo(c1)), nClusterings(ccSE)+1)
  expect_equal(primaryCluster(c1), primaryCluster(ccSE))
  primaryClusterIndex(c1) <- 3
  expect_false(all(primaryCluster(c1)==primaryCluster(ccSE)))
  
  #make sure converts characters and assigns names correctly.
  set.seed(151789)
  charValue<-sample(rep(c("-1","A","B","C","D"),times=c(2,3,2,1,7)))
  newCC<-addClusterings(ccSE,charValue,clusterLabel="characterCluster")
  #have to do this because have all kinds of attributes different
  expect_equal(t(as.matrix(tableClusters(newCC,"characterCluster",useNames=TRUE))),t(as.matrix(table(charValue))))
  
  ####check adding a ClusterExperiment to existing CE
  expect_error(addClusterings(ccSE,smSimCE),"Cannot merge clusters from different data") #assays don't match
  expect_silent(c3<-addClusterings(ccSE,ccSE))
  expect_equal(NCOL(clusterMatrix(c3)), nClusterings(ccSE)*2)
  expect_equal(length(clusterTypes(c3)), nClusterings(ccSE)*2)
  expect_equal(length(clusteringInfo(c3)), nClusterings(ccSE)*2)
  expect_equal(primaryCluster(c3), primaryCluster(ccSE))
  ###Check retain SE info
  expect_equal(colData(c3),colData(se)) 
  expect_equal(rownames(c3),rownames(se)) 
  expect_equal(colnames(c3),colnames(se)) 
  expect_equal(metadata(c3),metadata(se)) 
  expect_equal(rowData(c3),rowData(se)) 
  #Other checks after adding CE object  
  expect_error(clusterLabels(c3)[1:2]<-c("User","User"),"cannot have duplicated clusterLabels")
  clusterLabels(c3)[1:2]<-c("User1","User2")
  clusterLabels(c3)[1]<-"User4"
  expect_error(clusterLabels(c3)[1]<-"User2","cannot have duplicated clusterLabels")
  expect_equal(length(clusterLabels(c3)),nClusterings(ccSE)*2)
  
  ###check adding matrix of clusters
  expect_silent(c4<-addClusterings(ccSE,clusterMatrix(ccSE),clusterTypes="New"))
  expect_silent(newLeng<-2*nClusterings(ccSE))
  expect_equal(NCOL(clusterMatrix(c4)), newLeng)
  expect_equal(length(clusterTypes(c4)), newLeng)
  expect_equal(length(clusteringInfo(c4)), newLeng)
  expect_equal(primaryCluster(c4), primaryCluster(ccSE))
  ###Check retain SE info
  expect_equal(colData(c4),colData(se)) 
  expect_equal(rownames(c4),rownames(se)) 
  expect_equal(colnames(c4),colnames(se)) 
  expect_equal(metadata(c4),metadata(se)) 
  expect_equal(rowData(c4),rowData(se)) 
  
  ## Check coCluster slot with indices
  expect_silent(c5<-addClusterings(ccSE,clusterMatrix(ccSE),clusterTypes="New"))
  expect_silent(coClustering(c5)<-c(4,1))
  expect_silent(c6<-addClusterings(c5,ccSE))
  expect_equal(coClustering(c5),coClustering(c6))
  expect_null(coClustering(addClusterings(ccSE,c5)))
  
})  

test_that("removeClusterings work as promised",{
    ##########
    #check removeClusterings
    #single cluster
    expect_silent(c4<-addClusterings(ccSE,clusterMatrix(ccSE),clusterTypes="New"))
    expect_silent(c5<-removeClusterings(c4,1))
    expect_equal(NCOL(clusterMatrix(c5)), nClusterings(c4)-1)
    expect_equal(length(clusterTypes(c5)), nClusterings(c4)-1)
    expect_equal(length(clusteringInfo(c5)), nClusterings(c4)-1)
    expect_equal(primaryCluster(c4), primaryCluster(removeClusterings(c4,2)))
    ###Check retain SE info 
    expect_equal(colData(c5),colData(se)) 
    expect_equal(rownames(c5),rownames(se)) 
    expect_equal(colnames(c5),colnames(se)) 
    expect_equal(metadata(c5),metadata(se)) 
    expect_equal(rowData(c5),rowData(se)) 

    #vector clusters
    expect_silent(c6<-removeClusterings(c4,c(1,3)))
    expect_equal(NCOL(clusterMatrix(c6)), nClusterings(c4)-2)
    expect_equal(length(clusterTypes(c6)), nClusterings(c4)-2)
    expect_equal(length(clusteringInfo(c6)), nClusterings(c4)-2)
    ###Check retain SE info 
    expect_equal(colData(c6),colData(se)) 
    expect_equal(rownames(c6),rownames(se)) 
    expect_equal(colnames(c6),colnames(se)) 
    expect_equal(metadata(c6),metadata(se)) 
    expect_equal(rowData(c6),rowData(se)) 

    expect_error(removeClusterings(c4,c(1,nClusterings(c4)+1)),
        "Invalid value for 'whichCluster'")

    expect_silent(c7<-removeClusterings(c4,"User")) #two have "user" label
    expect_equal(NCOL(clusterMatrix(c7)), nClusterings(c4)-2)
    expect_equal(length(clusterTypes(c7)), nClusterings(c4)-2)
    expect_equal(length(clusteringInfo(c7)), nClusterings(c4)-2)

    ## Check coCluster slot with indices
    expect_silent(c4<-addClusterings(ccSE,clusterMatrix(ccSE),clusterTypes="New"))
    expect_error(coClustering(c4)<-c(4,7,1),"CoClustering slot is a vector, but doesn't match indices of clusterMatrix of the object") #can't give more than 4
    expect_silent(coClustering(c4)<-c(4,1))
    ## remove one of the co-clusterings
    expect_warning(c8<-removeClusterings(c4,c(4,3)), "removing clusterings that were used in makeConsensus")
    expect_null(coClustering(c8))
    ## remove separate clustering
    ## remove one of the co-clusterings
    expect_silent(c9<-removeClusterings(c4,c(2)))
    expect_equal(coClustering(c9),c(3,1))
  
})

test_that("removeClusters work as promised",{
  ##########
  ###Check removing a cluster assignment
  ########## 
  c1<-ccSE
	ind<-primaryClusterIndex(c1)
	expect_silent(c1<-renameClusters(c1,whichCluster="primary",letters[1:nClusters(c1)[ind]]))
  cL<-clusterLegend(c1)
	clmat<-cL[[ind]]
  rmCl<-c(2,3)
  rmNms<-clmat[clmat[,"clusterIds"] %in% as.character(rmCl),"name"]
	#note removeClusters creates new clustering with those clusters removed
  expect_silent(x<-removeClusters(c1,whichCluster="primary",clustersToRemove=rmCl,makePrimary=TRUE))
  expect_equal(sum(primaryCluster(x)==-1), sum(primaryCluster(c1) %in% c(-1,2,3)))
  newnms<-clusterLegend(x)[[primaryClusterIndex(x)]][,"name"]
	#what expect clusterLegend to be after removal
  oldnms<-clmat[!clmat[,"name"] %in% rmNms,"name"]
  expect_equal(oldnms,newnms)
  
   expect_silent(x<-removeClusters(c1,whichCluster="primary",clustersToRemove=rmNms,makePrimary=TRUE))
   expect_equal(sum(primaryCluster(x)==-1), sum(primaryCluster(c1) %in% c(-1,2,3)))
   newnms<-clusterLegend(x)[[primaryClusterIndex(x)]][,"name"]
   oldnms<-clmat[!clmat[,"name"] %in% rmNms,"name"]
   expect_equal(oldnms,newnms)
 
  
})

test_that("removeUnassigned work as promised",{
  
  ##########
  #check removing unclustered samples
  ##########
  #no -1 in primary cluster
  matTemp<-abs(labMat)
  expect_silent(ccTemp<-ClusterExperiment(mat,matTemp,transformation=function(x){x}))
  expect_equal(ccTemp, removeUnassigned(ccTemp)) 
  
### The remainder code sometimes causes problems if tested interactively more than once
### Some problem in the environment???
### Also triggers problems in subsetting (next)
	whUn<-which(primaryCluster(ccSE) <0)
  expect_silent(ccR<-removeUnassigned(ccSE))
  expect_equal(NCOL(ccR), NCOL(ccSE)-length(whUn))

  ###Check retain SE info
  expect_equal(colData(ccR),colData(se[,-whUn]) )
  expect_equal(rownames(ccR),rownames(se))
  expect_equal(colnames(ccR),colnames(se[,-whUn]))
  expect_equal(metadata(ccR),metadata(se))
  expect_equal(rowData(ccR),rowData(se))
  
})

test_that("rename/color works as promised",{
	#need better tests here that actually do what wanted. Currently just test no error.
	newName<-letters[1:nClusters(cc)["Cluster1"]]
	names(newName)<-as.character(1:nClusters(cc)["Cluster1"])
	expect_silent(ccNamed<-renameClusters(cc,whichCluster="Cluster1",value=newName))

	newColors<-palette()[1:nClusters(cc)["Cluster1"]]
	names(newColors)<-as.character(1:nClusters(cc)["Cluster1"])
	expect_silent(ccColored<-recolorClusters(cc,whichCluster="Cluster1",value=newColors))
})

test_that("subsetting by cluster works as promised",{
	newName<-letters[1:nClusters(cc)["Cluster1"]]
	names(newName)<-as.character(1:nClusters(cc)["Cluster1"])
	expect_silent(ccNamed<-renameClusters(cc,whichCluster="Cluster1",value=newName))
	expect_silent(x<-subsetByCluster(ccNamed,clusterValue=c("a","b")))
	expect_silent(y<-subsetByCluster(ccNamed,clusterValue=c("1","2"),matchTo="clusterIds"))
	expect_equal(x,y)
})

test_that("check clusterLegend manipulations work as promised", {
    local_edition(3)
    x<-clusterLegend(cc)
    expect_silent(clusterLegend(cc)<-x)
    expect_silent(c4<-addClusterings(ccSE,clusterMatrix(ccSE),clusterTypes="New",clusterLabels=c("new1","new2"),clusterLegend=clusterLegend(ccSE)))
    expect_silent(clusterLegend(c4)[1:2]<-x[1:2])
    expect_silent(clusterLegend(c4)[[1]]<-x[[1]])
#add wrong dimensions:
    expect_error(clusterLegend(c4)[3]<-list(x[[1]][1:2,]),"each element of `clusterLegend` must be matrix with")
    expect_error(clusterLegend(c4)[[3]]<-x[[1]][1:2,],"must be matrix with")
	
    ##########
	#check adding clusterLegend directly to constructor
    ##########
	newcc<-ClusterExperiment(as(cc,"SingleCellExperiment"),clusters=clusterMatrix(cc),clusterLegend=clusterLegend(cc))
	expect_equal(cc,newcc,ignore_function_env=TRUE)

	newCL<-clusterLegend(cc)
	newCL[[1]][,"name"]<-letters[1:nrow(newCL[[1]])]
	cc1<-cc
	clusterLegend(cc1)<-newCL
	newClusters<-convertClusterLegend(cc1,output="matrixNames")
	newCL[[1]][,"clusterIds"]<-newCL[[1]][,"name"]
	newCL[[1]][,"name"]<-LETTERS[1:nrow(newCL[[1]])]
  newcc<-ClusterExperiment(as(cc,"SingleCellExperiment"),clusters=newClusters,clusterLegend=newCL) 
  expect_equal(clusterLegend(newcc)[[1]][,c("name","color")], newCL[[1]][,c("name","color")])
	     
})

test_that("accessing transformed data works as promised",{
  expect_is(transformation(ccSE),"function")
  expect_equal(transformData(cc), transformation(cc)(assay(cc)))
})

test_that("assigning unassigned samples works as promised",{
  #also indirectly tests getReducedData!
	expect_silent(assignUnassigned(cc))
	expect_silent(ccUn<-assignUnassigned(cc,whichCluster="Cluster2"))
	expect_true("Cluster2_AllAssigned" %in% clusterLabels(ccUn))
	expect_silent(assignUnassigned(ceSim,reduceMethod="b"))
	expect_silent(assignUnassigned(ceSim,reduceMethod="mad"))
	expect_silent(ceUn<-assignUnassigned(ceSim,reduceMethod="PCA"))
	expect_equal(reducedDimNames(ceUn),"PCA")
	expect_silent(ceUn2<-assignUnassigned(ceSim,reduceMethod="PCA",makePrimary=FALSE))
	expect_true(all.equal(primaryCluster(ceSim),primaryCluster(ceUn2)))


	#check basic error catching
	cc2<-addClusterings(cc,rep(-1,ncol(cc)),clusterLabel="allUn")
	expect_error(assignUnassigned(cc2,whichCluster="allUn"),"All cells are unassigned, cannot assign them")
	cc2<-addClusterings(cc2,rep(2,ncol(cc2)),clusterLabel="allAss")
	expect_error(assignUnassigned(cc2,whichCluster="allAss"),"No cells are unassigned in the designated cluster")

	#should check whichAssay....
	
})

test_that("workflow functions work",{
  ##########
  #check workflow stuff
  expect_silent(ppC<-addClusterings(cc,cbind(rep(c(-1, 1,2), each=5),rep(c(2, 1,3), each=5)),clusterTypes=c("clusterMany","mergeClusters")))
  expect_equal(dim(workflowClusters(ppC)),c(nSamples(cc),2))
  
  expect_silent(ppC<-addClusterings(cc,cbind(rep(c(-1, 1,2), each=5)),clusterTypes=c("clusterMany")))
  expect_equal(dim(workflowClusters(ppC)),c(nSamples(cc),1))
  
  expect_silent(ppC<-addClusterings(cc,cbind(rep(c(-1, 1,2), each=5),rep(c(2, 3,1), each=5)),clusterTypes=c("clusterMany","mergeClusters.1")))
  expect_equal(dim(workflowClusters(ppC)),c(nSamples(cc),1))
  expect_equal(dim(workflowClusters(ppC,iteration=NA)),c(nSamples(cc),2))
  expect_null(workflowClusters(cc,iteration=NA))
          
  expect_message(ceNew<-makeConsensus(ceSim,proportion=0.7))
  expect_message(ceNew<-makeConsensus(ceNew,proportion=0.3,clusterLabel="makeConsensus,v2"))
  expect_equal(clusterLabels(ceNew)[1:2],c("makeConsensus,v2","makeConsensus.1"))
  expect_equal(clusterTypes(ceNew)[1:2],c("makeConsensus","makeConsensus.1"))
  expect_silent(ceNew2<-setToCurrent(ceNew,whichCluster="makeConsensus.1"))
  expect_equal(clusterLabels(ceNew2)[1:2],c("makeConsensus,v2","makeConsensus"))
  expect_equal(clusterTypes(ceNew2)[1:2],c("makeConsensus.2","makeConsensus"))
  expect_silent(ceNew3<-setToCurrent(ceNew2,whichCluster="makeConsensus.2"))
  expect_equal(clusterLabels(ceNew3)[1:2],c("makeConsensus,v2","makeConsensus.3"))
  expect_equal(clusterTypes(ceNew3)[1:2],c("makeConsensus","makeConsensus.3"))

  expect_silent(ceNew4<-setToFinal(ceNew,whichCluster="makeConsensus,v2",clusterLabel="Final Version"))
  expect_equal(primaryClusterIndex(ceNew4),1)
  expect_equal(clusterLabels(ceNew4)[primaryClusterIndex(ceNew4)],"Final Version")
  expect_equal(clusterTypes(ceNew4)[primaryClusterIndex(ceNew4)],"final")
  
  expect_silent(ceNew5<-setToFinal(ceNew,whichCluster="makeConsensus.1",clusterLabel="Final Version"))
  expect_equal(primaryClusterIndex(ceNew5),2)
  expect_equal(clusterLabels(ceNew5)[primaryClusterIndex(ceNew5)],"Final Version")
  expect_equal(clusterTypes(ceNew5)[primaryClusterIndex(ceNew5)],"final")
})
test_that("tableClusters works",{
	expect_silent(ccNamed<-renameClusters(cc,whichCluster="primary",value=letters[1:5]))
	expect_silent(tableClusters(cc,tableMethod="union"))
	expect_silent(tableClusters(ccNamed,whichClusters=c(1,2),tableMethod="union",useNames=FALSE))
	expect_silent(tableClusters(ccNamed,whichClusters=c(1,2),tableMethod="union",useNames=TRUE))
	expect_silent(tableClusters(ccNamed,whichClusters=c(1,2),tableMethod="intersect",useNames=FALSE))
	expect_silent(tableClusters(ccNamed,whichClusters=c(1,2),tableMethod="intersect",useNames=TRUE))
})
test_that("adding to colData works",{
	expect_silent(ccNamed<-renameClusters(ccSE,whichCluster="primary",value=letters[1:5]))
	expect_silent(df1<-colDataClusters(ccNamed,"Cluster1",useNames=FALSE,makeFactor=FALSE))
	expect_true(is.numeric(df1$Cluster1))
	expect_silent(df1<-colDataClusters(ccNamed,"Cluster1",useNames=FALSE,makeFactor=TRUE))
	expect_true(is.factor(df1$Cluster1))
	expect_silent(df1<-colDataClusters(ccNamed,"Cluster1",useNames=TRUE,makeFactor=FALSE))
	expect_true(is.character(df1$Cluster1))
	expect_silent(df1<-colDataClusters(ccNamed,"Cluster1",useNames=TRUE,makeFactor=TRUE))
	expect_true(is.factor(df1$Cluster1))

	expect_silent(x1<-addToColData(ccNamed,"Cluster1"))
	expect_equal(colData(x1),colDataClusters(ccNamed,"Cluster1"))
})

.availMergeMethods<-c("adjP", "locfdr", "JC","PC","Storey")
.validMergeMethods<-c("adjP", "locfdr", "MB","JC","PC","Storey") #back compatibility with old "MB"

#' @title Merge clusters based on dendrogram
#'
#' @description Takes an input of hierarchical clusterings of clusters and
#'   returns estimates of number of proportion of non-null and merges those
#'   below a certain cutoff.
#' @name mergeClusters
#' @aliases mergeClusters mergeClusters,matrixOrHDF5-method
#'
#' @param x data to perform the test on. It can be a matrix or a 
#'   \code{\link{ClusterExperiment}}.
#' @param cl A numeric vector with cluster assignments to compare to. ``-1'' 
#'   indicates the sample was not assigned to a cluster.
#' @param dendro dendrogram providing hierarchical clustering of clusters in cl.
#'   If x is a matrix, then the default is \code{dendro=NULL} and the function 
#'   will calculate the dendrogram with the given (x, cl) pair using 
#'   \code{\link{makeDendrogram}}. If x is a \code{\link{ClusterExperiment}} 
#'   object, the dendrogram in the slot \code{dendro_clusters} will be used. In 
#'   this case, this means that \code{\link{makeDendrogram}} needs to be called 
#'   before \code{mergeClusters}.
#' @param mergeMethod method for calculating proportion of non-null that will be
#'   used to merge clusters (if 'none', no merging will be done). See details 
#'   for description of methods.
#' @param cutoff minimimum value required for NOT merging a cluster, i.e. two 
#'   clusters with the proportion of DE below cutoff will be merged. Must be a 
#'   value between 0, 1, where lower values will make it harder to merge 
#'   clusters.
#' @param plotInfo what type of information about the merging will be shown on 
#'   the dendrogram. If 'all', then all the estimates of proportion non-null 
#'   will be plotted at each node of the dendrogram; if 'mergeMethod', then only
#'   the value used in the \code{mergeClusters} command is plotted at each node.
#'   If 'none', then no proportions will be added to the dendrogram, though the 
#'   dendrogram will be drawn. 'plotInfo' can also be one of the valid input to 
#'   \code{mergeMethod} (even if that method is not the method chosen in 
#'   \code{mergeMethod} argument). \code{plotInfo} can also show the information
#'   corresponding  to "adjP" with a fold-change cutoff, by giving a value to 
#'   this argument in  the form of "adjP_2.0", for example.
#' @param plot logical as to whether to plot the dendrogram with the merge 
#'   results
#' @param nodePropTable Only for matrix version. Matrix of results from previous
#'   run of \code{mergeClusters} as returned by matrix version of 
#'   \code{mergeClusters}. Useful if just want to change the cutoff. Not 
#'   generally intended for user but used internally by package.
#' @param calculateAll logical. Whether to calculate the estimates for all 
#'   methods. This reduces computation costs for any future calls to 
#'   \code{mergeClusters} since the results can be passed to future calls of 
#'   \code{mergeClusters} (and for \code{ClusterExperiment} objects this is done
#'   automatically).
#' @param showWarnings logical. Whether to show warnings given by the methods. 
#'   The 'locfdr' method in particular frequently spits out warnings (which may 
#'   indicate that its estimates are not reliable). Setting 
#'   \code{showWarnings=FALSE} will suppress all warnings from all methods (not 
#'   just "locfdr"). By default this is set to \code{showWarnings=FALSE} by 
#'   default to avoid large number of warnings being produced by "locfdr", but 
#'   users may want to be more careful to check the warnings for themselves.
#' @param logFCcutoff Relevant only if the \code{mergeMethod} selected is 
#'   "adjP", in which case the calculation of the proportion of individual tests
#'   significant will also require that the estimated log-fold change of the 
#'   features to be at least this large in absolute value. Value will be rounded
#'   to nearest tenth of an integer via \code{round(logFCcutoff,digits=1)}. For 
#'   any other method, this parameter is ignored. Note that the logFC is based
#'   on \code{log2} (the results of \code{\link{getBestFeatures}})
#' @param forceCalculate This forces the function to erase previously saved
#'   merge results and recalculate the merging.
#' @param DEMethod character vector describing how the differential expression 
#'   analysis should be performed that will be used in the estimation of the
#'  percentage DE per node. See \link{getBestFeatures} for current options. 
#'   See details.
#' @param ... for signature \code{matrix}, arguments passed to the 
#'   \code{\link{plot.phylo}} function of \code{ape} that plots the dendrogram. 
#'   For signature \code{ClusterExperiment} arguments passed to the method for 
#'   signature \code{matrix} and then if do not match those arguments, will be 
#'   passed onto \code{\link{plot.phylo}}.
#' @inheritParams clusterMany
#' @inheritParams getBestFeatures
#'
#' @details \strong{Estimation of proportion non-null} "Storey" refers to the
#'   method of Storey (2002). "PC" refers to the method of Pounds and Cheng
#'   (2004). "JC" refers to the method of Ji and Cai (2007), and implementation
#'   of "JC" method is copied from code available on Jiashin Ji's website,
#'   December 16, 2015
#'   (http://www.stat.cmu.edu/~jiashun/Research/software/NullandProp/). "locfdr"
#'   refers to the method of Efron (2004) and is implemented in the package
#'   \code{\link{locfdr}}.  "adjP"
#'   refers to the proportion of genes that are found significant based on a FDR
#'   adjusted p-values (method "BH") and a cutoff of 0.05. Previous versions offered the method "MB", a method of Meinshausen and Buhlmann
#'   (2005), but the package  \code{howmany} is no longer supported for its implementation.
#' @details \strong{Control of Plotting} If \code{mergeMethod} is not equal to
#'   'none' then the plotting will indicate where the clusters will be merged by
#'   making dotted lines of edges that are merged together (assuming
#'   \code{plotInfo} is not 'none'). \code{plotInfo} controls simultaneously
#'   what information will be plotted on the nodes as well as whether the dotted
#'   lines will be shown for the merged cluster. Notice that the choice of
#'   \code{plotInfo} (as long as it is not 'none') has no effect on how the
#'   dotted edges are drawn -- they are always drawn based on the
#'   \code{mergeMethod}. If you choose \code{plotInfo} to not be equal to the
#'   \code{mergeMethod}, then you will have a confusing picture where the dotted
#'   edges will be based on the clustering created by \code{mergeMethod} while
#'   the information on the nodes is based on a different method. Note that you
#'   can override \code{plotInfo} by setting \code{show.node.label=FALSE}
#'   (passed to plot.phylo), so that no information is plotted on the nodes, but
#'   the dotted edges are still drawn. If you just want plot of the dendrogram,
#'   with no merging performed nor demonstrated on the plot, see
#'   \code{\link{plotDendrogram}}.
#' @details \strong{Saving and Reusing of results} By default, the function
#'   saves the results in the \code{ClusterExperiment} object and will not
#'   recalculate them if not needed. Note that by default
#'   \code{calculateAll=TRUE}, which means that regardless of the value of
#'   \code{mergeMethod}, all the methods will be calculated so that those
#'   results will be stored and if you change the mergeMethod, no additional
#'   calculations are needed. Since the computationally intensive step is the
#'   running the DE method on the genes, this is a big savings (all of the
#'   methods then calculate the proportion from those results). However, note
#'   that if \code{calculateAll=TRUE} and ANY of the methods returned NA for any
#'   value, the calculation will be redone. Thus if, for example, the
#'   \code{locfdr} function does not run successfully and returns NA, the
#'   function will always recalculate each time, even if you don't specifically
#'   want the results of \code{locfdr}. In this case, it makes sense to turn
#'   \code{calculateAll=FALSE}.
#' @details If the dendrogram was made with option
#'   \code{unassignedSamples="cluster"} (i.e. unassigned were clustered in with
#'   other samples), then you cannot choose the option
#'   \code{leafType='samples'}. This is because the current code cannot reliably
#'   link up the internal nodes of the sample dendrogram to the internal nodes
#'   of the cluster dendrogram when the unassigned samples are intermixed.
#' @return If `x` is a matrix, it returns (invisibly) a list with elements
#'\itemize{ 
#'\item{\code{clustering}}{ a vector of length equal to ncol(x) giving
#'the integer-valued cluster ids for each sample. "-1" indicates the sample was
#'not clustered.} 
#'\item{\code{oldClToNew}}{ A table of the old cluster labels to
#'the new cluster labels.} 
#'\item{\code{nodeProp}}{ A table of the proportions
#'that are DE on each node.This table is saved in the \code{merge_nodeProp} slot
#'of a \code{ClusterExperiment} object and can be accessed along with the
#'nodeMerge info with the \code{nodeMergeInfo} function. 
#'}
#'\item{\code{nodeMerge}}{ 
#'A table of indicating for each node whether merged or
#'not and the cluster id in the new clustering that corresponds to the node.
#'Note that a node can be merged and not correspond to a node in the new
#'clustering, if its ancestor node is also merged. But there must be some node
#'that corresponds to a new cluster id if merging has been done. This table is
#'saved in the \code{merge_nodeMerge} slot of a \code{ClusterExperiment} object
#'and can be accessed along with the nodeProp info with the \code{nodeMergeInfo}
#'function. 
#'} 
#'\item{\code{updatedClusterDendro}}{ The dendrogram on which the
#'merging was based (based on the original clustering).
#'} 
#'\item{\code{cutoff}}{
#'The cutoff value for merging.
#'} 
#'}
#' @return If `x` is a \code{\link{ClusterExperiment}}, it returns a new
#'   \code{ClusterExperiment} object with an additional clustering based on the
#'   merging. This becomes the new primary clustering. Note that even if
#'   \code{mergeMethod="none"}, the returned object will erase any old merge
#'   information, update the work flow numbering, and return the newly calculated 
#'   merge information.  
#' @references Ji and Cai (2007), "Estimating the Null and the Proportion
#' of Nonnull Effects in Large-Scale Multiple Comparisons", JASA 102: 495-906.
#' @references Efron (2004) "Large-scale simultaneous hypothesis testing:
#' the choice of a null hypothesis," JASA, 99: 96-104.
#' @references Meinshausen and Buhlmann (2005) "Lower bounds for the
#' number of false null hypotheses for multiple testing of associations",
#' Biometrika 92(4): 893-907.
#' @references Storey (2002) "A direct approach to false discovery rates", J. R.
#'   Statist. Soc. B 64 (3)": 479-498.
#' @references Pounds and Cheng (2004). "Improving false discovery rate
#'   estimation." Bioinformatics 20(11): 1737-1745.

#' @seealso makeDendrogram, plotDendrogram, getBestFeatures
#' @examples
#' data(simData)
#'
#' #create a clustering, for 8 clusters (truth was 3)
#' cl<-clusterSingle(simData, subsample=FALSE,
#' sequential=FALSE, mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=8)))
#'
#' #give more interesting names to clusters:
#' newNames<- paste("Cluster",clusterLegend(cl)[[1]][,"name"],sep="")
#' clusterLegend(cl)[[1]][,"name"]<-newNames
#' #make dendrogram
#' cl <- makeDendrogram(cl)
#'
#' #plot showing the before and after clustering
#' #(Note argument 'use.edge.length' can improve
#' #readability)
#' merged <- mergeClusters(cl, plotInfo="all",
#' mergeMethod="adjP", use.edge.length=FALSE, DEMethod="limma")
#'
#' #Simpler plot with just dendrogram and single method
#' merged <- mergeClusters(cl, plotInfo="mergeMethod",
#' mergeMethod="adjP", use.edge.length=FALSE, DEMethod="limma",
#' leafType="clusters",plotType="name")
#'
#' #compare merged to original
#' tableClusters(merged,whichClusters=c("mergeClusters","clusterSingle"))
#'
#' @export
#' @importFrom locfdr locfdr
#' @rdname mergeClusters
setMethod(
    f = "mergeClusters",
    signature = signature(x = "matrixOrHDF5"),
    definition = function(x, cl, dendro=NULL,
                          mergeMethod=c("none", "Storey","PC","adjP", "locfdr", "JC"), #"MB" removed,
                          plotInfo="none",
                          nodePropTable=NULL, calculateAll=TRUE, showWarnings=FALSE,
                          cutoff=0.05, plot=TRUE,DEMethod, logFCcutoff=0, weights=NULL,...){
        
        if(any(c("whichCluster","whichClusters") %in% names(list(...)))) stop("The argument 'whichCluster' is not accepted for this function. The clustering used for merging will always be taken as that clustering that created the currently stored dendrogram (given by 'dendroClusterIndex')")
        dendroSamples<-NULL #currently option is not implemented for matrix version...
        if(!is.numeric(logFCcutoff) || logFCcutoff<0) stop("Invalid value for the parameter 'logFCcutoff'")
        logFCcutoff<-round(logFCcutoff,digits=1)
        if(!is.numeric(cutoff) || cutoff<0 || cutoff > 1) stop("Invalid value for the parameter 'cutoff'.")
        if(is.factor(cl)){
            warning("cl is a factor. Converting to numeric, which may not result in valid conversion")
            cl <- .convertToNum(cl)
        }
        if(!is.null(dendro)){
            #check valid
            ncluster <- length(table(cl[cl>0]))
            dendro <- .convertToPhyClasses(dendro,"phylo4d")
            if(phylobase::nTips(dendro) != ncluster) {
                stop("Not a valid input dendrogram (not equal to the number of non -1 clusters in cl).")
            }
        }
        mergeMethod <- match.arg(mergeMethod)
        ############
        #determine what methods asked to be calculated
        ############
        rmFCString<-function(x){
            wh<-grep("adjP",x)
            if(length(wh)>0){
                x[wh]<-"adjP"
            }
            return(x)
        }
        addFCString<-function(fc){
            paste("adjP", format(fc,nsmall=1),sep="_")
        }
        adjPFCMethod<-if(logFCcutoff>0) addFCString(logFCcutoff) else NULL
        if(calculateAll) whMethodCalculate=.availMergeMethods
        else{
            whMethodCalculate<-if(!mergeMethod=="none") mergeMethod else vector("character")
            if(plotInfo=="all") whMethodCalculate<-.availMergeMethods
            if(plotInfo%in% .availMergeMethods) whMethodCalculate<-unique(c(whMethodCalculate,plotInfo))
        }
        if(logFCcutoff>0 && "adjP" %in% whMethodCalculate){
            whMethodCalculate<-c(whMethodCalculate,adjPFCMethod)
            if(mergeMethod=="adjP") mergeMethod<-addFCString(logFCcutoff)
            
        }
        
        ### check valid existing table
        if(!is.null(nodePropTable)){
            if(!all(c("NodeId","Contrast") %in% colnames(nodePropTable))) stop("nodePropTable must have columns with names 'Node' and 'Contrast'")
            if(!all(.availMergeMethods %in% colnames(nodePropTable))) stop("All of the methods' names must be included in colnames of nodePropTable (with NA if not calculated):", paste(.availMergeMethods,collapse=",",sep=""))
            otherVals<-colnames(nodePropTable)[!colnames(nodePropTable)%in%c("NodeId","Contrast")]
            otherVals<-c(otherVals,adjPFCMethod)
        }
        else otherVals<-adjPFCMethod
        
        possibleValues<-unique(c("none", "all","mergeMethod",.availMergeMethods,otherVals))
        plotInfo<-match.arg(plotInfo,possibleValues)
        
        if(mergeMethod=="none" & plotInfo=="none" & !calculateAll) stop("mergeMethod and plotInfo both equal 'none'; nothing to be done.")
        if(plotInfo=="mergeMethod" & mergeMethod=="none") {
            stop("can only plot 'mergeMethod' results if one method is selected")
        }
        
        ############
        #determine whether need to calculate, or if already in nodePropTable
        ############
        needCalculate<-is.null(nodePropTable) || any(!whMethodCalculate %in% names(nodePropTable)) || any(is.na(nodePropTable[,whMethodCalculate]))
        
        ############
        ### calculate the estimated proportions
        ############
        if(needCalculate){
            #get per-gene test-statistics for the contrasts corresponding to each node (and return all)
            sigTable <- getBestFeatures(x, cl,
                                        contrastType=c("Dendro"), dendro=dendro,
                                        contrastAdj=c("All"),
                                        number=nrow(x), p.value=1, DEMethod=DEMethod, weights=weights)
            #divide table into each node and calculate proportion.
            sigByNode <- by(sigTable, sigTable$ContrastName, function(x) {
                storey<-if("Storey" %in% whMethodCalculate)  .myTryFunc(pvalues=x$P.Value, FUN=.m1_Storey,showWarnings=showWarnings) else NA
                pc <-if("PC" %in% whMethodCalculate)  .myTryFunc(pvalues=x$P.Value, FUN=.m1_PC,showWarnings=showWarnings) else NA
                # mb <-if("MB" %in% whMethodCalculate)  .myTryFunc(pvalues=x$P.Value, FUN=.m1_MB,showWarnings=showWarnings) else NA
                locfdr <-if("locfdr" %in% whMethodCalculate)  .myTryFunc(tstats=x$t, FUN=.m1_locfdr,showWarnings=showWarnings) else NA
                
                jc <-if("JC" %in% whMethodCalculate)  .myTryFunc(tstats=x$t, FUN=.m1_JC,showWarnings=showWarnings) else NA
                
                adjP<-if("adjP" %in% whMethodCalculate)
                    .myTryFunc(FUN=.m1_adjP,adjPvalues=x$adj,logFC=x$logFC, logFCcutoff=0,showWarnings=showWarnings) else NA
                out<-c("Storey"=storey,"PC"=pc,"adjP"=adjP, "locfdr"=locfdr, "JC"=jc) #"MB"=mb,
                if(!is.null(adjPFCMethod) && adjPFCMethod %in% whMethodCalculate){
                    adjPFC<-.myTryFunc(FUN=.m1_adjP,adjPvalues=x$adj,logFC=x$logFC, logFCcutoff=logFCcutoff,showWarnings=showWarnings)
                    out<-c(out,adjPFC)
                    names(out)[length(names(out))]<-adjPFCMethod
                    return(out)
                }
                else return(out)
            })
        }
        else{
            .mynote(paste("Using existing results of per-node significance -- no new tests on individual features will be made."))
            sigByNode<-by(nodePropTable,nodePropTable$NodeId,function(x){x})
        }
        newcl <- cl
        ############
        ### #go up tree and merge clusters
        ############
        if(mergeMethod != "none"){
			valsPerNode <- sapply(sigByNode, function(x) {signif(x[[mergeMethod]], 4)})
            nodesBelowCutoff <- names(valsPerNode)[which(valsPerNode<cutoff)] #names of nodes below cutoff
            #--------------
            #find nodes where *all* descendants are below cutoff and merge only those
            #given by nodesIndexToMerge (index of node in the tree)
            #--------------
			## First find out which nodes have all descendants who should be merged:
            nodeIndexBelowCutoff <- .matchToDendroData(inputValue=nodesBelowCutoff, dendro=dendro, matchColumn="NodeId",returnColumn="NodeIndex")
            tipNodes<-phylobase::getNode(dendro, type=c("tip"))
            whToMerge <- sapply(nodeIndexBelowCutoff,function(node){
                desc <- phylobase::descendants(dendro, node, type = c("all"))
                return(all(desc %in% nodeIndexBelowCutoff | desc %in% tipNodes))
            })
            if(length(whToMerge)>0 && length(which(whToMerge)) > 0){
                ## whToMerge indicates which nodes have ALL nodes below them that need merging
                ## (Still need to figure out if they are descendants of others that will merge)
                nodesIndexToMerge <- nodeIndexBelowCutoff[whToMerge]
                
                #per node, now which nodes are at the top
                #do this by per node finding anscestors and keeping only those 
                #WITHOUT any ancestors that need merging
                whAnc <- sapply(nodesIndexToMerge, function(node){
                    anc <- phylobase::ancestors(dendro, node, type="all")
                    return(!any(anc %in% nodesIndexToMerge))
                })
                nodesIndexAtTop <- nodesIndexToMerge[whAnc]
                
                #now have "unique" nodes
                #So per node find all tips that need merging and give them number
                #temp is unimportant; the lapply assigns the value globally into newcl
                temp <- lapply(nodesIndexAtTop, function(node){
                    tips <- phylobase::descendants(dendro, node, type="tips")
                    #check cluster names match...
                    tips<-.getClusterIds(tipIndex=tips,clusterDendro=dendro,returnValue="ClusterIdDendro")
                    if(any(!tips %in% as.character(cl))) {
                        stop("coding error-- tips don't match values of cl")
                    }
                    newcl[as.character(cl) %in% tips] <<- as.numeric(tips[[1]])
                })
                #make consecutive integers
                uc<-sort(unique(newcl[newcl>0]))
                newcl <- as.numeric(factor(newcl, levels=uc,labels=as.character(seq_along(uc))))
                #deal with -1/-2
                newcl[is.na(newcl)] <- cl[is.na(newcl)]
            }
        }
        ############
        ### save/update merge information
        ############
        nodePropTableGiven<-nodePropTable
        whNotProp <- which(c("NodeId", "Contrast") %in% names(nodePropTableGiven)) #interger 0 if is NULL
        if(!needCalculate){
            #------------
            ## Just split the table apart
            #------------
            nodePropTable <- nodePropTableGiven[, -whNotProp]
            annotTable <- nodePropTableGiven[, whNotProp]
			annotTable$NodeIndex <- .matchToDendroData(annotTable$NodeId, dendro, matchColumn="NodeId", returnColumn="NodeIndex")
        }
        else{
            #------------
            #create new table with calculated results
            #------------
            nodePropTable <- do.call("rbind", sigByNode)
            annotTable <- data.frame("NodeId"=names(sigByNode),
                                     "Contrast" = as.character(sigTable$Contrast[ match(names(sigByNode), sigTable$ContrastName) ]),
                                     stringsAsFactors =FALSE)
            annotTable$NodeIndex <- .matchToDendroData(annotTable$NodeId, dendro, matchColumn="NodeId", returnColumn="NodeIndex")
            if (!is.null(nodePropTableGiven)) {
                #------------
                #add results from nodePropTableGiven (original table) to nodePropTableGiven
                #------------
                #check same nodes and contrasts with original table
                if (!identical(unname(sort(nodePropTableGiven$NodeId)),unname(sort(annotTable$NodeId))))
                    stop("different nodes in given `nodePropTable` than those just calculated")
                if (!identical(unname(sort(nodePropTableGiven$Contrast)), sort(unname(annotTable$Contrast))))
                    stop("different contrast values in given `nodePropTable` than those just calculated")
                #----------
                # Find methods in nodePropTableGiven not calculated this time
                #----------
                whTransfer<-colnames(nodePropTableGiven)[-whNotProp]
                #1. do those that shared between two tables, but all NA in current, and were calculated previously
                whTransferExist<-whTransfer[whTransfer %in% colnames(nodePropTable)]
                whTransferExist<-whTransferExist[sapply(whTransferExist,function(x){ all(!is.na(nodePropTableGiven[,x])) })]
                whTransferExist<-whTransferExist[which(!whTransferExist %in% whMethodCalculate)]
                if(length(whTransferExist)>0){
                    m<-match(annotTable$NodeId,nodePropTableGiven$NodeId)
                    
                    nodePropTable[,whTransferExist]<-data.matrix(nodePropTableGiven[m,whTransferExist,drop=FALSE])
                    
                }
                #2. Add those in original table (that were calculated), but don't yet exist in current nodePropTable
                whTransfer<-whTransfer[!whTransfer %in% colnames(nodePropTable)]
                if(length(whTransfer)>0){
                    nodePropTable<-cbind(nodePropTable,
                                         data.matrix(nodePropTableGiven[,whTransfer,drop=FALSE])
                    )
                }
                if(any(duplicated(colnames(nodePropTable)))) stop("Internal Coding error -- have multiple copies of a method values.")
                #3. Go ahead and sort those that are the logFC so nice looking table (and make sure at the end of the table)
                whSort<-which(!colnames(nodePropTable) %in% .availMergeMethods)
                if(length(whSort)>0){
                    ord<-order(colnames(nodePropTable)[whSort])
                    nodePropTable<-cbind(nodePropTable[,.availMergeMethods], nodePropTable[,whSort[ord],drop=FALSE])
                }
            }
            
        }
        ############
        #add merge information:
        #also determine whether node corresponds to a cluster in merge clusters
        ############
        if (mergeMethod != "none" &&
            length(whToMerge) > 0 && length(which(whToMerge)) > 0) {
			
			logicalMerge <- annotTable$NodeIndex %in% nodesIndexToMerge
            
            #gives the names of original cluster ids
            corrspCluster <- sapply(annotTable$NodeIndex, function(node) {
                tips <- phylobase::descendants(dendro, node, type = c("tips")) #names of tips
                tips<-.getClusterIds(tipIndex=tips,clusterDendro=dendro,returnValue="ClusterIdDendro")
                if (any(!tips %in% as.character(cl))) {
                    stop("coding error-- tips don't match values of cl")
                }
                m <- match(tips, as.character(cl)) #gives index of first match of tips to cl
                if (length(unique(newcl[m])) == 1) return(unique(newcl[m]))
                else NA
            })
            #Need to decide if any of these are nested inside each other!
            #Should be the largest is the parent that was merged
            if(length(na.omit(corrspCluster))!=length(unique(na.omit(corrspCluster)))){
                uniqueCorr<-unique(na.omit(corrspCluster))
                correctNode<-sapply(uniqueCorr,function(x){
                    nodes<-annotTable$NodeIndex[which(corrspCluster==x)]
                    if(length(nodes)>1){
                        ntips<-sapply(nodes,function(node){length(phylobase::descendants(dendro, node, type = c("tips")))})
                        maxnode<-nodes[which.max(ntips)]
                        #check true assumption, no weird cases
                        maxdesc<-phylobase::descendants(dendro, maxnode, type = c("all"))
                        if(!all(nodes[-which.max(ntips)] %in% maxdesc)) stop("coding error -- largest samples wasn't parent node")
                        return(maxnode)
                    }
                    else return(nodes)
                })
                corrspCluster<-rep(NA,length(corrspCluster))
                corrspCluster[match(correctNode,annotTable$NodeIndex)]<-uniqueCorr
            }
        } else{
            logicalMerge <- rep(FALSE, length = nrow(annotTable))
            corrspCluster <- rep(NA, length = nrow(annotTable))
        }
        
        #########
        # Create information to return
        #########
		
		if("NodeIndex" %in% names(annotTable)) annotTable<-annotTable[,-grep("NodeIndex",names(annotTable))]
        nodeMergeTable<-data.frame(annotTable,
                                   "isMerged"=logicalMerge,
                                   "mergeClusterId"=corrspCluster,
                                   stringsAsFactors=FALSE)
        nodePropTable<-data.frame(annotTable,
                                  nodePropTable,
                                  stringsAsFactors=FALSE)
        
        if(mergeMethod=="none"){
            newcl<-NULL #was just the original and nothing changed, so don't return something that makes it look like theres a new clustering
            oldClToNew<-NULL
        }
        else{
            oldClToNew=table(Original=cl, New=newcl)
            #check node identification from above
            nmerge<-apply(oldClToNew,2,function(x){sum(x>0)})
            clustersThatMerge<-colnames(oldClToNew)[which(nmerge>1)]
            tableMergedClusters<- unname(sort(as.character(na.omit(nodeMergeTable$mergeClusterId))))
            actualMergedClusters<-sort(unname(clustersThatMerge))
            if(!identical(tableMergedClusters,actualMergedClusters)) stop("coding error -- wrong identification of merged clusters")
				
			#---------
	        #Add new merge cluster ids to dendro
			#---------
			data.cl<-phylobase::tdata(dendro)
	        m<-.matchToDendroData(inputValue=nodeMergeTable$NodeId, dendro=dendro, matchColumn="NodeId", returnColumn="NodeIndex")
	        clusterIdMerge<-rep(NA,length=nrow(data.cl))
	        clusterIdMerge[m]<-paste("ClusterId",nodeMergeTable$mergeClusterId,sep="")
	        clusterIdMerge[m][is.na(nodeMergeTable$mergeClusterId)]<-NA
	        data.cl$ClusterIdMerge<-clusterIdMerge
			
			##create diagonal of oldClToNew of those not merged
			dendroNotMerged<-which(apply(oldClToNew[,nmerge==1,drop=FALSE],1,function(x){sum(x>0)==1}))
			mergeNotMerged<-apply(oldClToNew[dendroNotMerged, ,drop=FALSE],1,function(x){which(x>0)})
			diagMerge<-oldClToNew[dendroNotMerged,mergeNotMerged,drop=FALSE]
			#rm -1 if exist
			whMiss<-which(rownames(diagMerge)%in%c("-1","-2"))
			if(length(whMiss)>0) diagMerge<-diagMerge[-whMiss,-whMiss,drop=FALSE]
			if(nrow(diagMerge)>0){
				#Otherwise, all non-missing clusters were merged...
	   		 	nodeIdNotMerged <- .matchToDendroData( inputValue=paste("ClusterId",rownames(diagMerge),sep=""), dendro=dendro, matchColumn="ClusterIdDendro", returnColumn="NodeIndex")
				data.cl$ClusterIdMerge[nodeIdNotMerged]<-paste("ClusterId",colnames(diagMerge),sep="")	
			}
	        phylobase::tdata(dendro)<-data.cl
		
        }       
        out<-list(clustering=newcl, oldClToNew=oldClToNew, cutoff=cutoff, logFCcutoff=logFCcutoff,
                  nodeProp=nodePropTable, nodeMerge=nodeMergeTable,updatedClusterDendro=dendro,mergeMethod=mergeMethod)
        
        ###############
        # PLOTTING
        ###############			  
        if(plot){
            clMat<-cbind(Original=cl, mergeCluster=newcl)
            if(!is.null(dendroSamples)){
                if(is.null(names(cl))){
                    warning("dendroSamples argument will be ignored because cl does not have names to allow for linkage to the dendroSamples values")
                    dendroSamples<-NULL
                }
                else{
                    rownames(clMat)<-names(cl)
                }
            }
            combTable<-.nodeMergeInfo(nodeProp=out$nodeProp,nodeMerge=out$nodeMerge)
            
            if(!is.null(dendroSamples)) .plotDendro(dendroSamples,leafType="samples",mergeOutput=combTable,mergePlotType=plotInfo,mergeMethod=mergeMethod,clObj=cl,plotType="name",...)
            else .plotDendro(dendro,leafType="clusters",mergeOutput=combTable,mergePlotType=plotInfo,mergeMethod=mergeMethod,clObj=clMat,plotType="name",...)
            
        }
        invisible(out)
    })

#' @rdname mergeClusters
#' @export
#' @param clusterLabel a string used to describe the type of clustering. By
#'   default it is equal to "mergeClusters", to indicate that this clustering is
#'   the result of a call to mergeClusters (only if x is a ClusterExperiment object)
#' @param plotType if plotting, then whether leaves of dendrogram should be
#'   labeled by rectangular blocks of color ("colorblock")  or with the names of
#'   the leaves ("name") (only if x is a ClusterExperiment object).
#' @param leafType if plotting, whether the leaves should be the clusters or the
#'   samples. Choosing 'samples' allows for visualization of how many samples
#'   are in the merged clusters (only if x is a ClusterExperiment object), which
#'   is the main difference between choosing "clusters" and "samples",
#'   particularly if \code{plotType="colorblock"}
#' @param whichAssay numeric or character specifying which assay to use. See
#'   \code{\link[SummarizedExperiment]{assay}} for details.
#' @details When the input is a \code{ClusterExperiment} object, the function
#'     attempts to update the merge information in that object. This is done by
#'     checking that the existing dendrogram stored in the object  (and run on
#'     the clustering stored in the slot \code{dendro_index}) is the same
#'     clustering that is stored in the slot \code{merge_dendrocluster_index}.
#'     For this reason, new calls to \code{\link{makeDendrogram}} will erase the merge
#'     information saved in the object.
#' @details If \code{mergeClusters} is run with \code{mergeMethod="none"}, the
#'   function may still calculate the proportions per node if \code{plotInfo} is
#'   not equal to "none" or \code{calculateAll=TRUE}. If the input object was a
#'   \code{ClusterExperiment} object, the resulting information will be still
#'   saved, though no new clustering was created; if there was not an existing
#'   merge method, the slot \code{merge_dendrocluster_index} will be updated.

setMethod(
    f = "mergeClusters",
    signature = signature(x = "ClusterExperiment"),
    definition = function(x, eraseOld=FALSE,
                          mergeMethod="none",
                          plotInfo="all",
                          clusterLabel="mergeClusters",
                          leafType=c("samples","clusters"),
                          plotType=c("colorblock","name","ids"),
                          plot=TRUE,
                          whichAssay=1,
                          forceCalculate=FALSE,
                          weights=if("weights" %in% assayNames(x)) "weights" else NULL,
                          DEMethod,
                          ...) {
        if(forceCalculate) x<-.eraseMerge(x)
        else{
            if(!is.na(x@merge_demethod)){
                if(!missing(DEMethod) && x@merge_demethod != DEMethod) stop("Setting argument 'DEMethod' is not allowed if there is already saved merge information that will be used. To rerun merge with different 'DEMethod' set 'forceCalculate=TRUE'.")
                else DEMethod<-x@merge_demethod
            }
            
        }
        plotType<-match.arg(plotType)
        leafType<-match.arg(leafType)
        if(is.null(x@dendro_clusters)) {
            stop("`makeDendrogram` needs to be called before `mergeClusters`")
        }
        else{
            cl<-clusterMatrix(x)[,dendroClusterIndex(x)]
            .mynote(paste("Merging will be done on '",clusterLabels(x)[dendroClusterIndex(x)],"', with clustering index",dendroClusterIndex(x)))
        }
        if(!.hasOutBranch(x)){
            if(any(cl<0) & leafType=="samples"){
                warning("You cannot set 'leafType' to 'samples' in plotting mergeClusters unless the dendrogram was made with unassigned/missing (-1,-2) set to an outgroup (see makeDendrogram)")
                leafType<-"clusters"
            }
        }
        ###Divide ... into mergeCluster arguments and plotting arguments
        ###Remove mergeClusters arguments in ... so can pass arguments to
        passedArgs<-list(...)
        mergeArgs<-.methodFormals("mergeClusters","matrix",
            envir=as.environment(passedArgs))         
        if(any(!names(passedArgs) %in% names(mergeArgs))){
            plotArgs<-passedArgs[which(!names(passedArgs) %in% names(mergeArgs))]
        }
        else plotArgs<-NULL
        
        
        
        ###Note, plot=FALSE, and then manually call .plotDendro afterwards to allow for passage of colors, etc.
        ##########
        #check to make sure all match, same, etc.
        ##########
        if(!is.na(x@merge_index)){
            if(x@merge_dendrocluster_index==x@dendro_index) propTable<-x@merge_nodeProp
            else propTable<-NULL
        }
        else propTable<-NULL
        if(!is.null(weights) && (is.character(weights) || (is.vector(weights) && is.numeric(weights)))  && length(weights)==1){
            weights<-assay(x, weights) 
        }
        outlist <- mergeClusters(x=if(DEMethod=="limma") transformData(x, whichAssay=whichAssay) else assay(x, whichAssay),
                                 cl=cl, nodePropTable=propTable,
                                 dendro=x@dendro_clusters, plotInfo=plotInfo,plot=FALSE,
                                 mergeMethod=mergeMethod, DEMethod=DEMethod, weights=weights,...)
        nodeMerge<-outlist$nodeMerge
        
        #-----
        ##Check if pipeline already ran previously and if so increase and erase old merge
        ##Note even if didn't ask to actually create merge information, calculating the merge information will have the same effect.
        #-----
        x<-.updateCurrentWorkflow(x, eraseOld, newTypeToAdd="mergeClusters", newLabelToAdd=clusterLabel)
        x<-.eraseMerge(x)
        
        if(mergeMethod!="none"){#
            #######################
            ##add a new cluster, but only if there was a merging
            #######################
            
            didMerge<-any(apply(outlist$oldClToNew,2,function(x){sum(x>0)>1}))
            if(!didMerge) .mynote("merging with these parameters did not result in any clusters being merged.")
            newObj <- ClusterExperiment(x, outlist$clustering,
                                        transformation=transformation(x),
                                        clusterTypes="mergeClusters",
                                        checkTransformAndAssay=FALSE)
            #check fixed the internal coding issue, i.e. same exact numbers as outlist$clustering:
            ckTab<-table(primaryCluster(newObj),outlist$clustering)
            if(nrow(ckTab)!=ncol(ckTab)) stop("coding error -- not same number of clusters after making ClusterExperiment object")
            if(!all(rownames(ckTab)==colnames(ckTab))) stop("coding error -- not same cluster ids after making ClusterExperiment object")
            if(any(diag(ckTab)==0)) stop("coding error -- not same cluster ids after making ClusterExperiment object")
            if(any(ckTab[upper.tri(ckTab)]!=0)) stop("coding error -- not same cluster ids after making ClusterExperiment object")
            
            #-----
            #add "m" to name of cluster
            #-----
            newObj<-.addPrefixToClusterNames(newObj,prefix="m",whCluster=1)
            clusterLabels(newObj) <- clusterLabel
            
            
            if(!is.null(x)) retval<-.addNewResult(newObj=newObj,oldObj=x)
            else retval<-.addBackSEInfo(newObj=newObj,oldObj=x)
            #-----
            #Add merge info to merge slots
            #Do manually here, because need joint object to dendro_index stuff
            # (otherwise get validity errors)
            #Note will save merge info, even if no change from merging.
            #-----
            retval@merge_nodeProp<-outlist$nodeProp
            retval@merge_index<-1
            retval@merge_method<-outlist$mergeMethod
            retval@merge_dendrocluster_index<-retval@dendro_index #update here because otherwise won't be right number.
            retval@merge_cutoff<-outlist$cutoff
            retval@merge_nodeMerge<-nodeMerge
			retval@dendro_clusters<-outlist$updatedClusterDendro
	        ch<-.checkDendrogram(retval)
	        if(!is.logical(ch)) stop(ch)
            #------------
            ##Align the colors between mergeClusters and makeConsensus
            #------------
            tryAlign<-try(plotClusters(retval,resetColors = TRUE, whichClusters=c("mergeClusters","makeConsensus"),plot=FALSE),silent=TRUE)
            if(!inherits(tryAlign,"try-error"))
                retval<-tryAlign
            else .mynote(paste("Unable to align mergeClusters with makeConsensus clusters because of the following error:",tryAlign))
            
        }
        else{
            ##############
            #When no merging done, still need to save merge info so don't have to redo it.
            #But do not update the clustering, etc from above.
            ##############
            retval<-x
            retval@merge_nodeProp <-outlist$nodeProp
            retval@merge_dendrocluster_index<-retval@dendro_index
        }
        retval@merge_demethod<-DEMethod
        ch<-.checkMerge(retval)
        if(!is.logical(ch)) stop(ch)
        if(plot){
			convertedDends<-.setNodeLabels(retval,labelType="name",useMergeClusters=FALSE,overrideExistingNode=FALSE,singletonCluster=c("sample"))
			dend<- switch(leafType,"samples"=convertedDends$dendro_samples,"clusters"=convertedDends$dendro_clusters)
            if(leafType=="samples" & mergeMethod!="none" & plotType=="colorblock"){
                whClusters<-c(retval@dendro_index,primaryClusterIndex(retval))
                leg<-clusterLegend(retval)[whClusters]
                cl<-clusterMatrix(retval,whichClusters=whClusters)
                rownames(cl)<-if(!is.null(colnames(retval))) colnames(retval) else .makeSampleNames(seq_len(ncol(retval)))
                
            }
            else{
                leg<-clusterLegend(retval)[[retval@dendro_index]]
                cl<-switch(leafType,"samples"=clusterMatrix(retval)[,retval@dendro_index],"clusters"=NULL)
                if(leafType=="samples"){
                    names(cl)<-if(!is.null(colnames(retval))) colnames(retval) else .makeSampleNames(seq_len(length(cl)))
                }
                
            }
            
            if(plotType=="id") leg[,"name"]<-leg[,"clusterIds"]
            label<-switch(plotType,"name"="name","colorblock"="colorblock","ids"="name")
            outbranch<-FALSE
            if(leafType=="samples" & any(cl<0)) outbranch<-.hasOutBranch(retval)
            #if(leafType=="samples" & any(cl<0)) outbranch<-TRUE
            
            # outbranch<-any(clusterMatrix(retval)[,retval@dendro_index]<0)
            # cl<-clusterMatrix(retval,whichCluster=retval@dendro_index)
            # rownames(cl)<-colnames(retval)
            # dend<-ifelse(leafType=="samples", retval@dendro_samples,retval@dendro_clusters)
            if(!"legend" %in% names(plotArgs)) plotArgs$legend<-"none"
            
			 do.call(".plotDendro", c(list(dendro=dend, leafType=leafType, mergeOutput=.nodeMergeInfo(outlist$nodeProp,outlist$nodeMerge), mergePlotType=plotInfo, mergeMethod=mergeMethod, clObj=cl, clusterLegendMat=leg, plotType=label,  removeOutbranch=outbranch), plotArgs))
        }
        
        invisible(retval)
    }
)


#' @rdname mergeClusters
#' @return \code{nodeMergeInfo} returns information collected about the nodes
#'   during merging as a data.frame with the following entries:
#' \itemize{ \item{\code{Node}}{ Name of the node}
#' \item{\code{Contrast}}{The
#' contrast compared at each node, in terms of the cluster ids}
#' \item{\code{isMerged}}{ Logical as to whether samples from that node which were
#' merged into one cluster during merging}
#' \item{\code{mergeClusterId}}{ If a
#' node corresponds to a new, merged cluster, gives the cluster id it
#' corresponds to. Otherwise NA}
#' \item{\code{...}}{The remaining columns give
#' the estimated proportion of genes differentially expressed for each method. A
#' column of NAs means that the method in question hasn't been calculated yet.}
#' }
#' @aliases nodeMergeInfo
#' @export
setMethod(
  f = "nodeMergeInfo",
  signature = "ClusterExperiment",
  definition = function(x) {
    if(!is.na(x@merge_dendrocluster_index)){
      if(!is.na(x@merge_index)){
        nodeMerge<-x@merge_nodeMerge
      }
      else{ #if run calculate all, can have the prop but no merge index
        nodeMerge<-cbind(
          x@merge_nodeProp[,c("NodeId","Contrast")],
          matrix(NA,nrow=nrow(x@merge_nodeProp),ncol=2)
        )
        colnames(nodeMerge)<-c("NodeId","Contrast","isMerged","mergeClusterId")
      }
      return(.nodeMergeInfo(x@merge_nodeProp,nodeMerge))
    }

    else return(NULL)
  }
)

.nodeMergeInfo<-function(nodeProp,nodeMerge){
  if(!nrow(nodeProp)==nrow(nodeMerge)) stop("Coding error -- nodeProp and nodeMerge had different number of rows")
  out<-merge(x=nodeMerge,y=nodeProp,by=c("NodeId","Contrast"),all=TRUE)
  if(!nrow(out)==nrow(nodeProp)) stop("Coding error -- merging of nodeProp and nodeMerge indicates not same values of 'NodeId' and 'Contrast'")
  row.names(out)<-NULL
  return(out)

}
#' @rdname mergeClusters
#' @return \code{mergeCutoff} returns the cutoff used for the current merging.
#' @aliases mergeCutoff
#' @export
setMethod(
  f = "mergeCutoff",
  signature = "ClusterExperiment",
  definition = function(x) {
    x@merge_cutoff
  }
)
#' @rdname mergeClusters
#' @return \code{mergeMethod} returns the method used for the current merge.
#' @aliases mergeMethod
#' @export
setMethod(
  f = "mergeMethod",
  signature = "ClusterExperiment",
  definition = function(x) {
    x@merge_method
  }
)
#' @rdname mergeClusters
#' @return \code{mergeClusterIndex} returns the index of the clustering used for the current merge.
#' @aliases mergeClusterIndex
#' @export
setMethod(
  f = "mergeClusterIndex",
  signature = "ClusterExperiment",
  definition = function(x) {
    x@merge_index
  }
)

#' @rdname mergeClusters
#' @return \code{eraseMergeInfo} returns object with all previously saved merge info removed.
#' @aliases eraseMergeInfo
#' @export
setMethod(
  f = "eraseMergeInfo",
  signature = "ClusterExperiment",
  definition = function(x) {
    x<-.eraseMerge(x)
	return(x)
  }
)
.eraseMerge<-function(x){
  x@merge_index<-NA_real_
  x@merge_dendrocluster_index<-NA_real_
  x@merge_method<-NA_character_
  x@merge_cutoff<-NA_real_
  x@merge_nodeProp<-NULL
  x@merge_nodeMerge<-NULL
  if(!is.null(x@dendro_clusters)){
	  cldend<-x@dendro_clusters
	  phylobase::tdata(cldend)$ClusterIdMerge<-NA
	  x@dendro_clusters<-cldend  	
  }
  ch<-.checkMerge(x)      
  if(!is.logical(ch)) stop(ch)
  ch<-.checkDendrogram(x)      
  if(!is.logical(ch)) stop(ch)
  else return(x)
}

.myTryFunc<-function(FUN,showWarnings,...){

	if(!showWarnings) suppressWarnings(x<-try(FUN(...),silent=TRUE))
	else x<-try(FUN(...),silent=TRUE)
  if(!inherits(x, "try-error")){
		#make sure not negative or >1
		x<-(pmax(x,0))
		x<-(pmin(x,1))
		return(x)
	}
  else return(NA)
}

#functions for estimating m1/m, the proportion of **non-null**
.m1_Storey<-function(pvalues,lambda=0.5){
	m<-length(pvalues)
	num<-length(which(pvalues>lambda))
	pi0<-num/(1-lambda)/m
	return(1-pi0)

}
.m1_PC<-function(pvalues){
	pi0<-2*mean(pvalues)
	return(1-pi0)

}

# .m1_MB<-function(pvalues){
#   nCorrect<-max(howmany::lowerbound(howmany::howmany(pvalues))) #the most you can call correctly
#   return(nCorrect/length(pvalues))
# }
.m1_adjP<-function(adjPvalues,logFC,logFCcutoff){
	if(length(adjPvalues)!=length(logFC)) stop("coding error -- adjPvalues and logFC must be of same length")
   sum(adjPvalues<=0.05 & abs(logFC)>=logFCcutoff)/length(adjPvalues)
}
.m1_locfdr<-function(tstats){
  locfdrResults<-locfdr::locfdr(tstats,plot=0)#ignore issue of df of t-statistic -- topTable doesn't give it out, and with large samples won't matter.
  p0<-locfdrResults$fp0["mlest","p0"] #estimate proportion null; ignore estimate of variability for now
  return(1-p0)
}
.m1_JC<-function(tstats){
  #copied code from Jianshin's website
  musigma<-try(.EstNull.func(tstats),silent=TRUE)
  .epsest.func(tstats,musigma$mu,musigma$s) #gives proportion of non-null
}


#' @param by indicates whether output from \code{getMergeCorrespond} should be
#'   a vector/list with elements corresponding to merge cluster ids or elements
#'   corresponding to the original clustering ids. See return value for details.
#' @rdname mergeClusters
#' @return \code{getMergeCorrespond} returns the correspondence between the
#'   merged cluster and its originating cluster. If \code{by="original"} returns
#'   a named vector, where the names of the vector are the cluster ids of the
#'   originating cluster and the values of the vector are the cluster ids of the
#'   merged cluster. If \code{by="merge"} the results returned are organized by
#'   the merged clusters. This will generally be a list, with the names of the
#'   list equal to the clusterIds of the merge clusters and the entries the
#'   clusterIds of the originating clusters. However, if there was no merging
#'   done (so that the clusters are identical) the output will be a vector like
#'   with \code{by="original"}.
#' @aliases getMergeCorrespond
#' @export
setMethod(
  f = "getMergeCorrespond",
  signature = "ClusterExperiment",
  definition = function(x,by=c("merge","original")){
	by<-match.arg(by)
	if(is.na(x@merge_index)) stop("there is no merge clustering in this object")
    mCl<-clusterMatrix(x)[,x@merge_index]
    dCl<-clusterMatrix(x)[,x@merge_dendrocluster_index]
	mUnique<-as.character(unique(mCl[mCl>0]))
	dUnique<-as.character(unique(dCl[dCl>0]))
	nDCl<-length(dUnique)
	nMCl<-length(mUnique)
	notSubsetMessage<-"Current merge cluster is no longer a merge of the cluster in this -- indicated clustering that creates merge is not a partition of the merge cluster"
	if(nDCl<nMCl) stop(notSubsetMessage)
	if(nMCl==1){ #all d clusters are in the one cluster
		if(by=="merge"){
			out<-list(dUnique)
			names(out)<-mUnique
		}
		else{
			out<-lapply(dUnique,function(z){return(mUnique)})
		}
		return(out)

	}
    tab<-table(mCl,dCl) #match them up #might not work if only 1 cluster in one of these...
	if(is.null(dim(tab))) stop("coding error -- should get matrix of tabulations")
	margin<-switch(by,"merge"=1,"original"=2)
	dimvalue<-switch(by,"merge"=2,"original"=1)
	dimnms<-dimnames(tab)[[dimvalue]]
	nn<-apply(tab,margin,function(z){
        mm<-dimnms[z>0]
        if(by=="original" & length(mm)>1) stop(notSubsetMessage)
        else if (length(mm)==0) stop("coding error -- should have a corresponding merge cluster")
        else return(mm)
	})
	names(nn)<-dimnames(tab)[[margin]]
	return(nn)
})

#' @title Resampling-based Sequential Ensemble Clustering
#'
#' @description Implementation of the RSEC algorithm (Resampling-based
#'   Sequential Ensemble Clustering) for single cell sequencing data. This is a
#'   wrapper function around the existing ClusterExperiment workflow that
#'   results in the output of RSEC.
#' @param k0s the k0 parameter for sequential clustering (see
#'   \code{\link{seqCluster}})
#' @param consensusProportion passed to \code{proportion} in
#'   \code{\link{makeConsensus}}
#' @param consensusMinSize passed to \code{minSize} in
#'   \code{\link{makeConsensus}}
#' @param consensusArgs list of additional arguments to be passed to
#'    \code{\link{makeConsensus}}
#' @param dendroReduce passed to \code{reduceMethod} in
#'   \code{\link{makeDendrogram}}
#' @param dendroNDims passed to \code{nDims} in \code{\link{makeDendrogram}}
#' @param mergeMethod passed to \code{mergeMethod} in
#'   \code{\link{mergeClusters}}
#' @param mergeCutoff passed to \code{cutoff} in \code{\link{mergeClusters}}
#' @param mergeLogFCcutoff passed to \code{logFCcutoff} in
#'   \code{\link{mergeClusters}}
#' @param mergeDEMethod passed to \code{DEMethod} argument in
#'   \code{\link{mergeClusters}}. By default, unless otherwise chosen by the
#'   user, if \code{isCount=TRUE}, then \code{mergeDEMethod="limma-voom"}, 
#' otherwise \code{mergeDEMethod="limma"}. These choices are for speed
#'  considerations and the user may want to try \code{mergeDEMethod="edgeR"} 
#' on smaller datasets of counts.
#' @param rerunClusterMany logical. If the object is a ClusterExperiment object,
#'   determines whether to rerun the clusterMany step. Useful if want to try
#'   different parameters for combining clusters after the clusterMany step,
#'   without the computational costs of the clusterMany step.
#' @param stopOnErrors logical. If \code{FALSE}, if RSEC hits an error
#'   \emph{after} the \code{clusterMany} step, it will return the results up to
#'   that point, rather than generating a stop error. The text of error will be
#'   printed as a NOTE. This allows the user to get the results to that point,
#'   so as to not have to rerun the computationally heavy earlier steps. The
#'   \code{TRUE} option is only provided for debugging purposes.
#' @return A \code{\link{ClusterExperiment}} object is returned containing all
#'   of the clusterings from the steps of RSEC
#' @details Note that the argument \code{isCount} is mainly used when the input is a matrix or SingleCellExperiment Class and passed to \code{clusterMany} to set the transformation function of the data. However, if RSEC is being re-called on an existing \code{ClusterExperiment} object, it does not reset the transformation; in this case the only impact it will have is in setting the default value for \code{DEMethod} for \code{mergeClusters} step, but ONLY if \code{mergeClusters} hasn't already been calculated. To set arguments that allow you to recalculate the non-null probabilities of the hierarchy see \code{\link{mergeClusters}}.
#' @inheritParams clusterMany
#' @name RSEC
#' @aliases RSEC RSEC-methods RSEC,ClusterExperiment-method RSEC,matrix-method RSEC,SingleCellExperiment-method RSEC,SummarizedExperiment-method
#' @inheritParams mergeClusters
#' @export
#' @rdname RSEC
setMethod(
    f = "RSEC",
    signature = signature(x = "SummarizedExperiment"),
    definition = function(x, ...){
        RSEC(as(x,"SingleCellExperiment"),...)
        
    })

#' @export
#' @rdname RSEC
setMethod(
    f = "RSEC",
    signature = signature(x = "data.frame"),
    definition = function(x,...){RSEC(data.matrix(x),...)}
)

#' @export
#' @rdname RSEC
setMethod(
    f = "RSEC",
    signature = signature(x = "ClusterExperiment"),
    definition = function(x, eraseOld=FALSE, rerunClusterMany=FALSE,...){
        passedArgs<-list(...)
        if(!"mergeDEMethod" %in% names(passedArgs) & is.na(x@merge_demethod) & "isCount" %in% names(passedArgs)){
            if(passedArgs$isCount) passedArgs$mergeDEMethod<-"limma-voom"
            else passedArgs$mergeDEMethod<-"limma"
            # wh<-which(names(passedArgs)=="isCount")
            # 		passedArgs<-passedArgs[-wh]
        }
        if(rerunClusterMany | !"clusterMany" %in% clusterTypes(x)){
            if(any(c("transFun","isCount") %in% names(passedArgs)))
                warning("The internally saved transformation function of a ClusterExperiment object must be used when given as input and setting 'transFun' or 'isCount' for a 'ClusterExperiment' has no effect other than to set a default for 'mergeDEMethod' (if not set by user).")
            newObj <- do.call("RSEC",c(list(x=as(x,"SingleCellExperiment"),  transFun=transformation(x)),passedArgs))
            ##Check if pipeline already ran previously and if so increase
            x<-.updateCurrentWorkflow(x,eraseOld,newTypeToAdd=.workflowValues[-1],newLabelToAdd=NULL)
            if(!is.null(x)) retval<-.addNewResult(newObj=newObj,oldObj=x) #make decisions about what to keep.
            else{
                retval<-.addBackSEInfo(newObj=newObj,oldObj=x)
                
            }
            filterStats(retval)<-filterStats(newObj)
            reducedDims(retval)<-reducedDims(newObj)
        }
        else{
            if(!is.na(x@merge_demethod)) passedArgs$mergeDEMethod<-x@merge_demethod
            retval<-do.call(".postClusterMany",c(list(x=x),passedArgs))
        }
        
        return(retval)
    })

#' @export
#' @rdname RSEC
setMethod(
    f = "RSEC",
    signature = signature(x = "matrixOrHDF5"),
    definition = function(x, ...){
        return(RSEC(SingleCellExperiment(x),...))
        
    })

#' @rdname RSEC
#' @param whichAssay numeric or character specifying which assay to use. See
#'   \code{\link[SummarizedExperiment]{assay}} for details.
#' @export
setMethod(
    f = "RSEC",
    signature = signature(x = "SingleCellExperiment"),
    definition = function(x,
                          isCount=FALSE,
                          transFun=NULL,
                          reduceMethod="PCA",
                          nFilterDims=defaultNDims(x,reduceMethod,type="filterStats"),
                          nReducedDims=defaultNDims(x,reduceMethod,type="reducedDims"),
                          k0s=4:15,
                          subsample=TRUE,
                          sequential=TRUE,
                          clusterFunction="hierarchical01", #listBuiltInType01(),
                          alphas=c(0.1,0.2,0.3),betas=0.9, minSizes=1,
                          makeMissingDiss=if(ncol(x)<1000) TRUE else FALSE,
                          consensusProportion=0.7,
                          consensusMinSize,
                          dendroReduce,
                          dendroNDims,
                          mergeMethod="adjP",
                          mergeCutoff,
                          mergeLogFCcutoff,
                          mergeDEMethod=if(isCount) "limma-voom" else "limma",
                          verbose=FALSE,
                          parameterWarnings=FALSE,
                          mainClusterArgs=NULL,
                          subsampleArgs=NULL,
                          seqArgs=NULL, 
                          consensusArgs=NULL,
                          whichAssay = 1,
                          ncores=1, random.seed=NULL,
                          stopOnErrors=FALSE, run=TRUE
    )
    {
        if(reduceMethod=="none"){
            nReducedDims<-NA
            nFilterDims<-NA
        }
        if(is.null(seqArgs))
            seqArgs<-list(verbose=FALSE)
        else seqArgs[["verbose"]]<-FALSE #turn off sequential messages
        ce<-clusterMany(x,ks=k0s,clusterFunction=clusterFunction,
                        alphas=alphas,betas=betas,minSizes=minSizes,
                        sequential=sequential,removeSil=FALSE,
                        subsample=subsample,
                        silCutoff=0,distFunction=NA,
                        isCount=isCount,transFun=transFun,
                        reduceMethod=reduceMethod, 
                        nFilterDims=eval(nFilterDims),
                        nReducedDims=eval(nReducedDims),
                        mainClusterArgs=mainClusterArgs,
                        subsampleArgs=subsampleArgs,
                        seqArgs=seqArgs,makeMissingDiss=makeMissingDiss,
                        ncores=ncores,random.seed=random.seed,run=run,
                        whichAssay=whichAssay, verbose=verbose,
                        parameterWarnings=parameterWarnings)
        
        if(run){
            #first add ones that have default value
            passedArgs<-list(x=ce,consensusProportion=consensusProportion,
                             mergeMethod=mergeMethod,whichAssay=whichAssay,
                             stopOnErrors=stopOnErrors,
                             makeMissingDiss=eval(makeMissingDiss),
                             consensusArgs=consensusArgs)
            #add those who will use default value from the function -- is there easier way
            if(!missing(consensusProportion))
                passedArgs<-c(passedArgs,consensusProportion=consensusProportion)
            if(!missing(consensusMinSize))
                passedArgs<-c(passedArgs,consensusMinSize=consensusMinSize)
            if(!missing(dendroReduce))
                passedArgs<-c(passedArgs,dendroReduce=dendroReduce)
            if(!missing(dendroNDims))
                passedArgs<-c(passedArgs,dendroNDims=dendroNDims)
            if(!missing(mergeCutoff))
                passedArgs<-c(passedArgs,mergeCutoff=mergeCutoff)
            if(!missing(mergeLogFCcutoff))
                passedArgs<-c(passedArgs,mergeLogFCcutoff=mergeLogFCcutoff)
            mergeDEMethod<-eval(mergeDEMethod)
            if(!missing(mergeDEMethod))
                passedArgs<-c(passedArgs,mergeDEMethod=eval(mergeDEMethod))
            ce<-do.call(".postClusterMany",passedArgs)
        }
        return(ce)
    })


## argument envir is only used for evaluating conditional statements -- can give additional arguments that are values of the default arguments
.methodFormals <- function(f, signature = character(), envir) {
    #to find defaults of RSEC
    #from this conversation:
    #http://r.789695.n4.nabble.com/Q-Get-formal-arguments-of-my-implemented-S4-method-td4702420.html
    fdef <- getGeneric(f)
    method <- selectMethod(fdef, signature)
    genFormals <- base::formals(fdef)
    b <- body(method)
    if(is(b, "{") && is(b[[2]], "<-") && identical(b[[2]][[2]], as.name(".local"))) {
        local <- eval(b[[2]][[3]])
        if(is.function(local)){
            forms<-formals(b[[2]][[3]])
            whConditional<-which(sapply(forms,function(x){class(x)=="if"}))
            if(length(whConditional)>0){
                # note, converts it to a list (rather than pairlist) class, but doesn't matter for me.
                formEnvir<-forms
                # find those defined already by user so can use in evaluating the conditionals
                whDefined<-sapply(names(forms),exists,envir=envir)
                formEnvir[whDefined]<-sapply(names(forms)[whDefined],get,envir=envir)
                forms[whConditional]<-sapply(forms[whConditional],function(x){
                    eval(x,envir=formEnvir)
                })
            }
            return(forms)
        }
        warning("Expected a .local assignment to be a function. Corrupted method?")
    }
    genFormals
}
.postClusterMany<-function(x,stopOnErrors=FALSE,...){
    ## Note: can't have any arguments in RSEC that depend on evaluating x
    ## Because it will fail here; can only have conditions that are on other arguments that have default values. x doesn't have a default value.
    passedArgs<-list(...)
    defaultArgs<-.methodFormals("RSEC",signature="SingleCellExperiment", envir=as.environment(c(list(x=x,stopOnErrors=stopOnErrors),passedArgs)))
    #remove those without anything defined
    defaultArgs<-defaultArgs[which(sapply(defaultArgs,
        function(x){!isTRUE(x=="")}))]
    whNotShared<-which(!names(defaultArgs)%in%names(passedArgs) )
    if(length(whNotShared)>0) 
        passedArgs<-c(passedArgs,defaultArgs[whNotShared])
    #------------
    ## makeConsensus
    #------------
    args1<-passedArgs$consensusArgs
    if("consensusProportion" %in% names(passedArgs))
        args1[["proportion"]]=passedArgs$consensusProportion
    if("consensusMinSize" %in% names(passedArgs))
        args1[["minSize"]]<-passedArgs$consensusMinSize
    whClusters<-if("whichClusters" %in% names(passedArgs)) 
        passedArgs$whichClusters  	
        else "clusterMany"
    combineTry<-try(do.call("makeConsensus",
                            c(list(x=x,whichClusters=whClusters),args1)), 
                    silent=TRUE)
    if(!inherits(combineTry,"try-error")){
        x<-combineTry
        #------------
        ##makeDendrogram
        #------------
        args1<-list()
        if("dendroReduce" %in% names(passedArgs)){
            args1<-c(args1,"reduceMethod"=passedArgs$dendroReduce, 
                "whichAssay"=passedArgs$whichAssay)
            if(passedArgs$dendroReduce=="none") passedArgs$dendroNDims<-NA
        }
        if("dendroNDims" %in% names(passedArgs)) args1<-c(args1,"nDims"=passedArgs$dendroNDims)
        dendroTry<- try(do.call( "makeDendrogram", c(list(x=x,filterIgnoresUnassigned=TRUE), args1)), silent=TRUE)
        
        #------------
        #mergeClusters
        #------------
        if(!inherits(dendroTry,"try-error")){
            x<-dendroTry
            
            if("mergeMethod" %in% names(passedArgs) && passedArgs$mergeMethod!="none"){
                args1<-list()
                args1 <- c(args1, "whichAssay"=passedArgs$whichAssay)
                args1<-c(args1,"mergeMethod"=passedArgs$mergeMethod)
                if("mergeCutoff" %in% names(passedArgs)) args1<-c(args1,"cutoff"=passedArgs$mergeCutoff)
                if("mergeLogFCCutoff" %in% names(passedArgs)){
                    args1<-c(args1,"logFCcutoff="=passedArgs$mergeLogFCCutoff)
                }
                if("mergeDEMethod" %in% names(passedArgs)){
                    args1<-c(args1,"DEMethod"=passedArgs$mergeDEMethod)
                    mergeTry <- try(do.call( mergeClusters,c(list(x=x,plot=FALSE,plotInfo="none"), args1 )), silent=TRUE)
                }
                else{
                    mergeTry<-"mergeDEMethod argument is missing with no default"
                    class(mergeTry)<-"try-error"
                }
                if(!inherits(mergeTry,"try-error")){
                    x<-mergeTry
                }
                else{
                    if(!stopOnErrors).mynote(paste("mergeClusters encountered following error and therefore clusters were not merged:\n", mergeTry))
                    else stop(mergeTry)
                }
            }
            else .mynote("clusters will not be merged because argument 'mergeMethod' was not given (or was equal to 'none')")
        }
        else{
            if(!stopOnErrors).mynote(paste("makeDendrogram encountered following error and therefore clusters were not merged:\n", dendroTry))
            else stop(dendroTry)
            
        }
    }
    else{
        if(!stopOnErrors).mynote(paste("makeConsensus encountered following error and therefore clusters from clusterMany were not combined:\n", combineTry))
        else stop(combineTry)
        
    }
    return(x)
}

####### Note: These are notexported, so the documentation is commented out.

setGeneric("filterStats", function(object,type,...) { standardGeneric("filterStats")})
setGeneric("filterStats<-", function(object, ..., value) standardGeneric("filterStats<-"))

##Note: have set so that all logicals on the filtering and dimReduce are coded here, so that if change things about filterNames, etc. only effects this code.
setGeneric("isFilterStats", function(object, ...) standardGeneric("isFilterStats"))
setGeneric("anyValidFilterStats", function(object, ...) standardGeneric("anyValidFilterStats"))
setGeneric("isReducedDims", function(object, ...) standardGeneric("isReducedDims"))
setGeneric("anyValidReducedDims", function(object, ...) standardGeneric("anyValidReducedDims"))
setGeneric("isBuiltInFilterStats", function(type, ...) standardGeneric("isBuiltInFilterStats"))
setGeneric("isBuiltInReducedDims", function(type, ...) standardGeneric("isBuiltInReducedDims"))
setGeneric("ncolReducedDims",function(object,...) standardGeneric("ncolReducedDims"))
setGeneric("isPossibleReducedDims",function(object,...) standardGeneric("isPossibleReducedDims"))
setGeneric("isPossibleFilterStats",function(object,...) standardGeneric("isPossibleFilterStats"))

setMethod( "isPossibleFilterStats","SingleCellExperiment",function(object,type){
	type %in% c(listBuiltInFilterStats(),filterNames(object))
})
setMethod( "isPossibleReducedDims","SingleCellExperiment",function(object,type){
	type %in% c(listBuiltInReducedDims(),reducedDimNames(object))
})
setMethod( "ncolReducedDims","SingleCellExperiment",function(object){
	sapply(reducedDims(object),ncol)
})
setMethod( "ncolReducedDims","SingleCellExperiment",function(object){
	sapply(reducedDims(object),ncol)
})
setMethod( "isBuiltInReducedDims","character",function(type){
	type %in% listBuiltInReducedDims()
})
setMethod( "isBuiltInFilterStats","character",function(type){
	type %in% listBuiltInFilterStats()
})		

setMethod( "isReducedDims","SingleCellExperiment",function(object,type){
	type %in% reducedDimNames(object)
})
setMethod( "isFilterStats","SummarizedExperiment",function(object,type){
	type %in% filterNames(object)
})		
setMethod( "anyValidFilterStats","SummarizedExperiment",function(object){
	length(filterNames(object))>0

})		
setMethod( "anyValidReducedDims","SingleCellExperiment",function(object){
	length(reducedDimNames(object))>0

})		
	

# #' @rdname reduceFunctions
# #' @aliases filterNames<-
# #' @export
# setReplaceMethod("filterNames", "SummarizedExperiment", function(object,value) {
# 	fs<-filterStats(object)
# 	if(length(value)!=NCOL(fs)) stop("value must be a vector of length equal to NCOL(filterStats(object)):",NCOL(filterStats(object)))
# 	colnames(fs) <- value
# 	filterStats(object)<-fs
# 	validObject(object)
# 	return(object)
# 	}
# )


# #' @param object A SummarizedExperiment object
# #' @param type a type of filter to retrieve. Should match the filter name.
# #' @aliases filterStats,SummarizedExperiment,character-method filterStats
# #' @rdname reduceFunctions
setMethod( 
  f="filterStats",
  c("SummarizedExperiment","character"),
  function(object,type){
    if(ncol(rowData(object))==0) stop("There are no filter statistics saved for this object")
    fn<-filterNames(object)
    if(!all(type %in% fn)){
      type<-type[type%in%fn]
      if(length(type)==0) stop("None of the values of 'type' argument are valid filter names ")
      else warning(paste("Not all values of '",type, "' are names of a valid filtering statistic held in rowData",sep=""))
    }
    return(rowData(object)[,type,drop=FALSE] )
  })
# #' @rdname reduceFunctions
setMethod( "filterStats",c("SummarizedExperiment","missing"),
	function(object,type){
		return(rowData(object)[,filterNames(object),drop=FALSE])
	})

# #' @rdname reduceFunctions
# #' @details Note that the replacement functions never actually completely
# #'   replace the slot \code{filterStats} unless the replacement value is NULL
# #'   They update existing filters of the
# #'   same name and add filters with new names to the existing filters.
# #' @aliases filterStats<-
# #' @importFrom S4Vectors DataFrame
setReplaceMethod("filterStats", "SummarizedExperiment", function(object, type, ...,value) {
  isMatrixLike<-is.matrix(value) || is(value,"DataFrame")
  if(missing(type)){
    # if(is.null(value)){
    # 	object@filterStats<-NULL
    # 	return(object)
    # }
    if(!isMatrixLike || is.null(colnames(value))){
      stop("If not indicating a type in the replacement, must give matrix of values with column names")
    }
    type<-colnames(value)
  }
  if(length(type)>1){
    if(!isMatrixLike || ncol(value)!=length(type))
      stop("If replacing several filtering statistics at the same time, new value must be a matrix")
    if(!is.null(colnames(value)) && !all(type==colnames(value)))
      stop("If replacement value has names must match type")
    colnames(value)<-type
  }
  else if(length(type)==1){
    if(!isMatrixLike) value<-matrix(value,ncol=1)
    colnames(value)<-type
  }
  fs <- rowData(object) #all rowData
  if(NCOL(fs)>0){
    whTypeExist<-which(type %in% colnames(fs))
    if(length(whTypeExist)>0){
      fs[,type[whTypeExist]] <- S4Vectors::DataFrame(value[,whTypeExist,drop=FALSE])
    }
    whTypeNew<-which(!type %in% colnames(fs))
    if(length(whTypeNew)>0){
      fs<-S4Vectors::DataFrame(fs,value[,whTypeNew,drop=FALSE])
    }
    rowData(object) <- fs
  }
  else rowData(object)<-value
  validObject(object)
  return(object)
})

#' @title Return matrix from ClusterExperiment with reduced dimensions
#' @description Returns a matrix of data from a \code{ClusterExperiment} object
#'   based on the choices of dimensionality reduction given by the user.
#' @rdname reduceFunctions
#' @aliases getReducedData
#' @param object  For \code{makeReducedDims},\code{makeFilterStats},
#'   \code{defaultNDims} either matrix-like, \code{SingleCellExperiment}, or
#'   \code{ClusterExperiment} object. For \code{getReducedData} only a
#'   \code{ClusterExperiment} object allowed.
#' @param nDims The number of dimensions to keep from \code{reduceMethod}. If
#'   missing calls \code{\link{defaultNDims}}.
#' @param whichAssay numeric or character specifying which assay to use. See
#'   \code{\link[SummarizedExperiment]{assay}} for details.
#' @param filterIgnoresUnassigned logical. Whether filtering statistics should 
#'   ignore the unassigned samples within the clustering. Only relevant if 
#'   'reduceMethod' matches one of built-in filtering statistics in 
#'   \code{\link{listBuiltInFilterStats}()}, in which case the clustering 
#'   identified in \code{whichCluster} is passed to \code{makeFilterStats} and
#'   the unassigned samples are excluded in calculating the statistic. See 
#'   \code{\link{makeFilterStats}}  for more details. 
#' @param returnValue The format of output. Users will generally want to keep
#'   the default (see details)
#' @param reducedDimName The name given to the reducedDims slot storing result 
#'   (if \code{returnValue="object"}). If missing, the function will create a
#'   default name: if \code{reduceMethod} is a dimensionality reduction, then
#'   \code{reduceMethod} will be given as the name; if a filtering statistic,
#'   "filteredBy_" followed by \code{reduceMethod}.
#' @inheritParams getClusterIndex
#' @details \code{getReducedData} determines the matrix of values that can be used for
#'   computation based on the user's choice of dimensionality methods. The
#'   methods can be either of the filtering kind or the more general
#'   dimensionality reduction. The function will first look at any stored
#'   \code{ReducedDims} or filtering statistics already present in the data, and
#'   if missing, will assume that \code{reduceMethod} is one of the built-in
#'   method provided by the package and calculate the necessary. Note that if
#'   \code{reduceMethod} is a filtering statistic, in addition to filtering the
#'   features, the function will also perform the stored transformation of the
#'   data.
#' @details Note that this is used internally by functions, but is mainly only
#'   of interest for the user if they want to have the filtered, transformed
#'   data available as a matrix for continual use.
#' @details If \code{returnValue="object"}, then the output is a single, updated
#'   \code{ClusterExperiment} object with the reduced data matrix stored as an 
#'   element of the list in \code{reducedDims} slot (with name given by
#'   \code{reducedDimName} if given). If "list", then a list with one element
#'   that is the object and the other that is the reduced data matrix. Either
#'   way, the object returned in the list will be updated to contain with the
#'   filtering statistics or the dimensionality reduction. The only difference
#'   is that if "list", the reduced dimension matrix is NOT saved in the object
#'   (and so only really makes a difference if the \code{reduceMethod} argument
#'   is a filtering method). The option "list" is mainly for internal use, where
#'   we do not want to continually save subseted datasets.
#' @details If \code{nDims} is missing, it will be given a default value 
#'   depending on the value of \code{reduceMethod}. See 
#'   \code{\link{defaultNDims}} for details.
#' @details If \code{filterIgnoresUnassigned} is missing, then it is set to TRUE
#'   \emph{unless}: \code{reduceMethod} matches a stored filtering statistic in
#'   \code{rowData} AND does not match a built-in filtering method provided by
#'   the package.
#' @return If \code{returnValue="object"}, a \code{ClusterExperiment} object.
#' @return If \code{returnValue="list"} a list with elements:
#' \itemize{ \item{\code{objectUpdate}}{object, potentially updated if had to
#' calculate dimensionality reduction or filtering statistic}
#' \item{\code{dataMatrix}}{the reduced dimensional matrix with the samples in
#' columns, features in rows} }
#' @seealso \code{\link{makeFilterStats}},\code{\link{makeReducedDims}},
#'   \code{\link{filterData}}, \code{\link[SingleCellExperiment]{reducedDim}}
#' @export
setMethod(
  f = "getReducedData",
  signature = "ClusterExperiment",
  definition = function(object,reduceMethod,filterIgnoresUnassigned, 
                        nDims=defaultNDims(object,reduceMethod),whichCluster="primary", 
                        whichAssay=1, returnValue=c("object","list"),reducedDimName){
    if(isReducedDims(object,reduceMethod) & isFilterStats(object,reduceMethod)) stop(paste(reduceMethod,"is the name of both a stored filtering statistic and a stored dimensionality reduction -- cannot create reduced data"))
    whCl<-getSingleClusterIndex(object,whichCluster)
    returnValue<-match.arg(returnValue)
    reduceMethodName<-reduceMethod
    if(missing(filterIgnoresUnassigned)){
      if(isFilterStats(object,reduceMethod) & !isBuiltInFilterStats(reduceMethod)) filterIgnoresUnassigned<-FALSE
      else filterIgnoresUnassigned<-TRUE
    }
    if(filterIgnoresUnassigned & isBuiltInFilterStats(reduceMethod)){
      reduceMethodName<-.makeClusterFilterStats(reduceMethod,clusterLabels(object)[whCl])
    }
    if(length(nDims) > 1) {
      stop("getReducedData only handles one choice of dimensions.")
    }
    if(!is.na(nDims) & reduceMethod=="none") {
      warning("specifying nDims has no effect if reduceMethod==`none`")
    }
    
    #deal with name of new assay
    if(returnValue=="object" ){
      addToObject<-TRUE
      if(isReducedDims(object,reduceMethod)){
        addToObject<-FALSE 
        warning("will not add reduced dataset to object because already exists method with that name")
      }  
      else{
        if(missing(reducedDimName)){
          if(isFilterStats(object,reduceMethod) || isBuiltInFilterStats(reduceMethod)){
            reducedDimName<-paste0("filteredBy_",reduceMethodName)
          }
          else reducedDimName<-reduceMethod
        }
      }
    }
    else addToObject<-FALSE

    #check nDims less than nDims of saved object...otherwise will have to recalculate it, if possible, and give it new name
    if(isReducedDims(object,reduceMethod) && nDims > ncolReducedDims(object)[[reduceMethod]]){
        warning("requesting an existing dimensionality reduction but with greater number of dimensions than available. Will ignore nDims and use max dimensions")
        nDims<-ncolReducedDims(object)[reduceMethod]
    }
    ###Calculate filters/reduceMethod if needed...
    if(!isReducedDims(object,reduceMethod) & isBuiltInReducedDims(reduceMethod)){
      object<-makeReducedDims(object,reducedDims=reduceMethod,maxDims=nDims,whichAssay=whichAssay)
    }
    else if(!isFilterStats(object,reduceMethodName) & isBuiltInFilterStats(reduceMethod)){
      object<-makeFilterStats(object,filterStat=reduceMethod, whichAssay=whichAssay,
                              whichClusterIgnoreUnassigned=if(filterIgnoresUnassigned) whCl else NULL)
    }
    
    if(reduceMethod=="none")
      dat<-transformData(object, whichAssay=whichAssay)
    else if(isReducedDims(object,reduceMethod))
      dat<-t(reducedDim(object,type=reduceMethod)[,seq_len(nDims)])
    else if(isFilterStats(object,reduceMethodName))
      dat<-transformData(filterData(object,filterStats=reduceMethodName,percentile=nDims), whichAssay=whichAssay)
    else stop("'object' does not contain the given 'reduceMethod' value nor does 'reduceMethod' value match any built-in filters or dimensionality reduction options.")
    
    if(addToObject){
      if(isReducedDims(object,reduceMethod) & reduceMethod!=reducedDimName){
        #means was already calcualted and saved with name 'reduceMethod' but not right name
        wh<-which(reducedDimNames(object)==reduceMethod)
        if(length(wh)>1) stop("internal coding error -- multiple matches of reduceMethod")
        names(reducedDims(object))[wh]<-reducedDimName
      }
      else reducedDim(object,reducedDimName) <- t(dat)
    }
    if(returnValue=="list"){
      return(list(objectUpdate=object,dataMatrix=dat))
    }
    if(returnValue=="object"){
      return(object)
    }
  }
)
#redo filtering statistic on subset determined by cluster unassigned
.makeClusterFilterStats<-function(filterStats,clusterName){
  make.names(paste(filterStats,clusterName,sep="_"))
}
#reduce reduced dims to get larger number of Dims
.makeClusterReducedRedo<-function(reducedDims,nDims){
  make.names(paste(reducedDims,nDims,sep="_"))
}
#' @rdname reduceFunctions
#' @param reduceMethod character. A method (or methods) for reducing the size of
#'   the data, either by filtering the rows (genes) or by a dimensionality
#'   reduction method. Must either be 1) must match the name of a built-in
#'   method, in which case if it is not already existing in the object will be
#'   passed to \code{\link{makeFilterStats}} or \code{link{makeReducedDims}}, or
#'   2) must match a stored filtering statistic or dimensionality reduction in
#'   the object
#' @param typeToShow character (optional). If given, should be one of
#'   "filterStats" or "reducedDims" to indicate of the values in the
#'   reduceMethod vector, only show those corresponding to "filterStats" or
#'   "reducedDims" options.
#' @return \code{defaultNDims} returns a numeric vector giving the default
#'   dimensions the methods in \code{clusterExperiment} will use for reducing
#'   the size of the data. If \code{typeToShow} is missing, the resulting vector
#'   will be equal to the length of \code{reduceMethod}. Otherwise, it will be a
#'   vector with all the unique valid default values for the \code{typeToShow}
#'   (note that different dimensionality reduction methods can have different
#'   maximal dimensions, so the result may not be of length one in this case).
#' @details For a \code{reduceMethod} that corresponds to a filtering statistics
#'   the current default is 1000 (or the length of the number of features, if
#'   less). For a dimensionality reduction saved in the reducedDims slot the
#'   default is 50 or the maximum number of dimensions if less than 50.
#' @details \code{reduceMethod} will first be checked to see if it corresponds
#'   with an existing saved filtering statistic or a dimensionality reduction to
#'   determine which of these two types it is. If it does not match either, then
#'   it will be checked against the built in functions provided by the package.
#'   @examples 
#'   se<-SingleCellExperiment(matrix(rnorm(5000*100),nrow=5000,ncol=100))
#'   defaultNDims(se,"PCA")
#'   defaultNDims(se,"mad")
#' @aliases defaultNDims defaultNDims,SingleCellExperiment-method
#' @export
setMethod( 
  f="defaultNDims",
  "SingleCellExperiment",
  function(object,reduceMethod,typeToShow){
    nDims<-rep(NA,length(reduceMethod))
		isFilter<-isBuiltInFilterStats(reduceMethod) | isFilterStats(object,reduceMethod)
		isRed<-isReducedDims(object,reduceMethod ) | isBuiltInReducedDims(reduceMethod)
    isAnyFilter<-any(isFilter)
    isAnyRed<-any(isRed)
    if(isAnyFilter)
      nDims[isBuiltInFilterStats(reduceMethod) | isFilterStats(object,reduceMethod)]<-min(1000,NROW(object))
    if(isAnyRed){
			if(any(isReducedDims(object,reduceMethod )))
      nDims[isReducedDims(object,reduceMethod)] <- ncolReducedDims(object)[reduceMethod[isReducedDims(object,reduceMethod )]]
    	if(any(!isReducedDims(object,reduceMethod )& isBuiltInReducedDims(reduceMethod))) nDims[!isReducedDims(object,reduceMethod )& isBuiltInReducedDims(reduceMethod)]<-min(c(50,dim(object)))
		}
		if(!missing(typeToShow)){ #means pick a single one for each type
      if(typeToShow=="filterStats"){
        if(any(isFilter)) nDims<-min(unique(nDims[isFilter])) else nDims<-NA
      }
      if(typeToShow=="reducedDims"){
        if(any(isRed)) nDims<-min(unique(nDims[isRed])) else nDims<-NA
      }
      if(length(nDims)==0) nDims<-NA
    }
    return(nDims)
    
  })

#' @rdname reduceFunctions
#' @export
setMethod( 
  f="defaultNDims","matrixOrHDF5",function(object,...){
    return(defaultNDims(SingleCellExperiment(object),...))
  })

context("Dendrogram")
# library(devtools)
# load_all()


test_that("`makeDendrogram` works with matrix",{
    #test matrix version
    expect_silent(makeDendrogram(mat, primaryCluster(cc)))
    expect_silent(makeDendrogram(mat, primaryCluster(cc), unassigned="cluster"))
    expect_silent(makeDendrogram(mat, primaryCluster(cc), unassigned="remove"))

 
    #test matrix version
    expect_equal(phylobase::nTips(makeDendrogram(mat, primaryCluster(cc), unassigned="remove")$samples),
                 length(primaryCluster(cc))-2)
})
			 
test_that("`makeDendrogram` works with ClusterExperiment objects", {
    #test CE version
    expect_silent(makeDendrogram(cc))
    expect_silent(makeDendrogram(cc, unassigned="cluster"))
    expect_error(makeDendrogram(cc, unassigned="remove"),"should be one of") #not valid option

    #test proper error if only single cluster:
    fakeCluster<-rep(1,nSamples(cc))
    expect_silent(cc1<-addClusterings(cc,fakeCluster))
    expect_silent(primaryClusterIndex(cc1)<-3)
    expect_error(makeDendrogram(cc1),"Only 1 cluster given")
    fakeCluster[c(1:2)]<- -1
    expect_silent(cc1<-addClusterings(cc,fakeCluster))
    expect_silent(primaryClusterIndex(cc1)<-3)
    expect_error(makeDendrogram(cc1),"Only 1 cluster given")
	
	#check whichClusters for clusterMatrix
 	expect_silent(dend<-makeDendrogram(ceSim))
	expect_equal(ncol(clusterMatrix(dend,whichClusters="dendro")),1)
	
	#--------
    #Check filters created when ignoriing cluster for variance
	#--------
    expect_silent(cl1 <- clusterSingle(smSimData, subsample=FALSE, sequential=FALSE, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=6)),isCount=FALSE))
    leg<-clusterLegend(cl1)[[primaryClusterIndex(cl1)]]
    leg[,"name"]<-letters[1:6]
    expect_silent(clusterLegend(cl1)[[primaryClusterIndex(cl1)]]<-leg)
    expect_silent(clustWithDendro <- makeDendrogram(cl1))
	expect_equal(filterNames(clustWithDendro),"mad_clusterSingle")
    #redo
	expect_silent(clustWithDendro <- makeDendrogram(clustWithDendro))
	expect_equal(filterNames(clustWithDendro),"mad_clusterSingle")
    
	
	#--------
    #Check remove clusters when have dendrogram
	#--------
	expect_message(clustMerged <- mergeClusters(clustWithDendro, mergeMethod="adj", plotInfo="none",DEMethod="limma"),"Merging will be done on ' clusterSingle ', with clustering index 1")
    expect_silent(removeClusterings(clustMerged,whichClusters="mergeClusters")) #remove merged, keep one with dendrogram
    expect_silent(removeClusterings(clustMerged,whichClusters=2)) #remove one with dendrogram
  
    #--------
	#test subsetting works if have dendrogram attached:
    #--------
	expect_silent(x<-makeDendrogram(ccSE,reduceMethod="PCA",nDims=3))
    expect_silent(x[1:3,1:2])
  
})

test_that("`makeDendrogram` works with hdf5",{
    expect_silent(clustNothing <- clusterMany(hdfSCE,
		 ks=c(3,4),clusterFunction="pam",
		 subsample=FALSE, sequential=FALSE,
		 isCount=FALSE,verbose=FALSE))
    expect_silent(clustNothing<-makeConsensus(clustNothing, proportion=1,whichClusters = "clusterMany"))
	expect_silent(makeDendrogram(clustNothing))
	
})

test_that("`makeDendrogram` preserves the colData and rowData of SE", {

  expect_silent(whCl<-primaryClusterIndex(ccSE))
  expect_silent(dend <- makeDendrogram(ccSE,reduceMethod="mad", whichCluster=whCl))

  expect_equal(colData(dend),colData(se))
  expect_equal(rownames(dend),rownames(se))
  expect_equal(colnames(dend),colnames(se))
  expect_equal(metadata(dend),metadata(se))
  x<-rowData(dend)
  expect_silent(newFilter<-clusterExperiment:::.makeClusterFilterStats("mad",clusterLabels(ccSE)[whCl]))
  expect_equal(x[,-grep(newFilter,colnames(x))],rowData(se))

})

test_that("`makeDendrogram` with reduceMethod options", {
    expect_silent(x<-makeDendrogram(ccSE,reduceMethod="PCA",nDims=3))
	expect_error(makeDendrogram(ccSE,reduceMethod=c("PCA","var"),nDims=3))
    expect_silent(x2<-makeDendrogram(ccSE,reduceMethod=c("PCA"),nDims=3,filterIgnoresUnassigned=TRUE))
    expect_equal(x,x2)
    expect_silent(makeDendrogram(ccSE,reduceMethod=c("var"),nDims=3,filterIgnoresUnassigned=FALSE))
    expect_silent(makeDendrogram(ccSE,reduceMethod=c("var"),nDims=3,filterIgnoresUnassigned=TRUE))
    expect_silent(makeDendrogram(ccSE,reduceMethod=c("abscv"),nDims=3,filterIgnoresUnassigned=FALSE))
    expect_silent(makeDendrogram(ccSE,reduceMethod=c("abscv"),nDims=3,filterIgnoresUnassigned=TRUE))
    expect_silent(makeDendrogram(ccSE,reduceMethod=c("mad"),nDims=3,filterIgnoresUnassigned=FALSE))
    expect_silent(makeDendrogram(ccSE,reduceMethod=c("mad"),nDims=3,filterIgnoresUnassigned=TRUE))
    
})
test_that("`makeDendrogram` works with whichCluster and updates dendrogram correctly", {
    expect_silent(x1<-makeDendrogram(ccSE,whichCluster="Cluster2"))
    expect_silent(x2<-makeDendrogram(ccSE,whichCluster=2))
    expect_equal(x1,x2)
    
    bigCE<-ceSim
    expect_silent(bigCE<-makeDendrogram(bigCE,whichCluster="cluster1"))
    x1<-bigCE
    #--- check clusterMany updates dendrogram correctly
    expect_silent(bigCE<-clusterMany(bigCE,
        k=2:8,clusterFunction="hierarchicalK",
        verbose=FALSE,
        makeMissingDiss=TRUE))
    expect_equal(clusterLabels(bigCE)[bigCE@dendro_index],
        clusterLabels(x1)[x1@dendro_index])
    expect_equal(bigCE@dendro_clusters,x1@dendro_clusters) 
    #takes a long time!
    #expect_equal(bigCE@dendro_samples,x1@dendro_samples) 
    expect_error(makeDendrogram(bigCE,whichCluster="workflow"),
        "Invalid value for 'whichCluster'")
 
    #--- check makeConsensus updates dendrogram correctly
    expect_message(bigCE<-makeConsensus(bigCE,proportion=0.3),
        "no clusters specified to combine, using results from clusterMany")
    expect_equal(clusterLabels(bigCE)[bigCE@dendro_index],
        clusterLabels(x1)[x1@dendro_index])
    expect_equal(bigCE@dendro_clusters,x1@dendro_clusters) 
    expect_equal(bigCE@dendro_samples,x1@dendro_samples) 
    expect_silent(makeDendrogram(bigCE,whichCluster="makeConsensus") )
    
    
    #--- check mergeClusters updates dendrogram correctly
    expect_message(bigCE<-mergeClusters(bigCE,
        mergeMethod="adjP",cutoff=0.2, DEMethod="limma"),
        "Merging will be done on ")
    expect_equal(clusterLabels(bigCE)[bigCE@dendro_index],
        clusterLabels(x1)[x1@dendro_index])
    #Note, x1 doesn't give any merged clusters in dendrogram because before mergeClusters step...  so going to remove that element from bigCE
	tdf<-phylobase::tdata(bigCE@dendro_clusters)
	tdf$ClusterIdMerge<-NA
	phylobase::tdata(bigCE@dendro_clusters)<-tdf
	expect_equal(bigCE@dendro_clusters,x1@dendro_clusters)  
    expect_equal(bigCE@dendro_samples,x1@dendro_samples) 
    
    expect_error(getBestFeatures(bigCE,contrastType="Dendro"),
        "only single cluster in clustering -- cannot run getBestFeatures")
	expect_silent(primaryClusterIndex(bigCE)<-3)
	expect_error( getBestFeatures(bigCE,contrastType="Dendro"),"does not match either the cluster on which the dendrogram was made or the merge cluster from this dendrogram")
})

test_that("plotDendrogram works with colData", {
  leg<-clusterLegend(ccSE)[[primaryClusterIndex(ccSE)]]
  leg[,"name"]<-letters[1:nrow(leg)]
  clusterLegend(ccSE)[[primaryClusterIndex(ccSE)]]<-leg
	expect_silent(dend <- makeDendrogram(ccSE))
	expect_silent(plotDendrogram(dend,colData="A"))
	expect_warning(plotDendrogram(dend,colData=c("A","B","C")),"implies using columns of colData that are continuous")
	
	#note that legA doesn't give colors for everything -- only some. 
	legA<-leg[4:7,]
	legA[,"color"]<-tail(massivePalette,4)
	expect_silent(plotDendrogram(dend,colData="A",clusterLegend=list("A"=legA)))

	expect_silent(plotDendrogram(dend,colData=c("A","C"),clusterLegend=list("A"=legA)))
	
})

test_that("plotDendrogram works with nodeColors and labels", {
  leg<-clusterLegend(ccSE)[[primaryClusterIndex(ccSE)]]
  leg[,"name"]<-letters[1:nrow(leg)]
  clusterLegend(ccSE)[[primaryClusterIndex(ccSE)]]<-leg
  dend <- makeDendrogram(ccSE)
  expect_silent(plotDendrogram(dend))
  expect_silent(nl<-paste("A", 1:nInternalNodes(dend)))
  expect_error(nodeLabels(dend) <- nl )
  expect_silent(names(nl)<-nodeLabels(dend))
  expect_silent(nodeLabels(dend) <- nl )
  expect_silent(plotDendrogram(dend,show.node.label=TRUE))
  expect_silent(plotDendrogram(dend,show.node.label=TRUE,leafType="samples",plotType="name"))
  expect_silent(plotDendrogram(dend,show.node.label=TRUE, leafType="samples",plotType="name",removeOutbranch=FALSE))
  expect_silent(plotDendrogram(dend, show.node.label=TRUE,leafType="samples", plotType="colorblock"))
  expect_silent(plotDendrogram(dend, show.node.label=TRUE, leafType="clusters",plotType="colorblock"))
  expect_silent(plotDendrogram(dend,show.node.label=TRUE,leafType="clusters",plotType="name"))
	
  nc<-palette()[1:nInternalNodes(dend)]
  expect_silent(names(nc)<-nodeIds(dend,type="internal"))
  expect_silent(plotDendrogram(dend,nodeColors=nc,leafType="clusters",plotType="name"))
  
})

test_that("plotDendrogram works with outgroup", {
    leg<-clusterLegend(ccSE)[[primaryClusterIndex(ccSE)]]
    leg[,"name"]<-letters[1:nrow(leg)]
    clusterLegend(ccSE)[[primaryClusterIndex(ccSE)]]<-leg
  dend <- makeDendrogram(ccSE)
  expect_silent(plotDendrogram(dend))
  
  ## make all -2
  cl<-clusterMatrix(ccSE)[,1]
  cl[1]<- -2
  dend2<-addClusterings(ccSE,cl,clusterLabel="newCluster")
  primaryClusterIndex(dend2)<-3
  dend2 <- makeDendrogram(dend2)
  expect_silent(plotDendrogram(dend2,leafType="clusters",plotType="colorblock"))
  expect_silent(plotDendrogram(dend2,leafType="samples",plotType="colorblock"))
  expect_silent(plotDendrogram(dend2, leafType="samples", plotType="colorblock",removeOutbranch=FALSE))

  ## make only single sample -2
  cl<-clusterMatrix(ccSE)[,1]
  cl[1]<-1
  expect_silent(dend3<-addClusterings(ccSE,cl,clusterLabel="newCluster"))
  expect_silent(primaryClusterIndex(dend3)<-3)
  expect_silent(dend3 <- makeDendrogram(dend3))
  expect_silent(plotDendrogram(dend3,leafType="clusters",plotType="colorblock"))
  expect_silent(plotDendrogram(dend3,leafType="samples",plotType="colorblock"))
  expect_silent(plotDendrogram(dend3, leafType="samples", plotType="colorblock", removeOutbranch=FALSE))

  # This test breaks something. Needs to be figured out. 
  # ## make all -1 but two samples
  # ## can't be only 1 sample because then only 1 cluster so can't make a dendrogram...
  # cl<-rep(-1,length=nSamples(ccSE))
  # cl[1]<-3
  # cl[2]<-1
  # dend4<-addClusterings(ccSE,cl,clusterLabel="missingCluster")
  # primaryClusterIndex(dend4)<-3
  # dend4 <- makeDendrogram(dend4)
  # plotDendrogram(dend4,leafType="clusters",plotType="colorblock")
  # plotDendrogram(dend4,leafType="samples",plotType="colorblock")
  # plotDendrogram(dend4,leafType="samples",plotType="colorblock",removeOutbranch=FALSE)

  ## make all -1 but one sample -- should get error bc only 1 cluster, can't make dendrogram; 
  ## in case this changes, this test will catch that need to fix plotDendrogram, which makes assumption that not possible.
  cl<-rep(-1,length=nSamples(ccSE))
  cl[1]<-3
  expect_silent(dend5<-addClusterings(ccSE,cl,clusterLabel="missingCluster"))
  expect_silent(primaryClusterIndex(dend5)<-3)
  expect_error(makeDendrogram(dend5,reduceMethod="none"),"Only 1 cluster given. Can not make a dendrogram.")
  expect_error(plotDendrogram(dend5,leafType="clusters",plotType="colorblock"),"No dendrogram is found for this ClusterExperiment Object. Run makeDendrogram first.")

})

test_that("plotDendrogram works with whichClusters", {
  leg<-clusterLegend(ccSE)[[primaryClusterIndex(ccSE)]]
    leg[,"name"]<-letters[1:nrow(leg)]
    clusterLegend(ccSE)[[primaryClusterIndex(ccSE)]]<-leg
  expect_silent(dend <- makeDendrogram(ccSE))
  expect_message(dend<-mergeClusters(dend,DEMethod="limma"))
  expect_silent(plotDendrogram(dend, whichClusters="all", leafType="samples", plotType="colorblock"))  
})


test_that("plotDendrogram works with cluster missing", {
  expect_silent(leg<-clusterLegend(ccSE)[[primaryClusterIndex(ccSE)]])
  leg[,"name"]<-letters[1:nrow(leg)]
  expect_silent(clusterLegend(ccSE)[[primaryClusterIndex(ccSE)]]<-leg)
  expect_silent(dend <- makeDendrogram(ccSE,unassignedSamples = c("cluster")))
  expect_silent(plotDendrogram(dend))
  expect_silent(plotDendrogram(dend,show.node.label=TRUE))
  expect_silent(plotDendrogram(dend,leafType="samples",plotType="name"))
  expect_silent(plotDendrogram(dend,leafType="samples",plotType="colorblock"))
  expect_silent(plotDendrogram(dend,leafType="clusters",plotType="colorblock"))
  expect_silent(plotDendrogram(dend,leafType="clusters",plotType="name"))
  
  ## make all the <0 ones -2 value
  dend2<-dend
  dmat<-clusterMatrix(dend2)
  dmat[1,1]<- -2
  dend2@clusterMatrix<-dmat
  leg<-dend2@clusterLegend[[1]]
  leg<-leg[-which(leg[,"clusterIds"]== -1),]
  dend2@clusterLegend[[1]]<-leg
  expect_silent(dend2 <- makeDendrogram(dend2,unassignedSamples = c("cluster")))
  expect_silent(plotDendrogram(dend2,leafType="clusters",plotType="colorblock"))
  expect_silent(plotDendrogram(dend2,leafType="samples",plotType="colorblock"))
  
})

#' Barplot of 1 or 2 clusterings
#' 
#' Make a barplot of sample's assignments to clusters for single clustering, or 
#' cross comparison for two clusterings.
#' 
#' @aliases plotBarplot
#' @docType methods
#' @param object A matrix of with each column corresponding to a clustering and
#'   each row a sample or a \code{\link{ClusterExperiment}} object.
#' @param colPalette a vector of colors used for the different clusters. See
#'   details.
#' @param xNames names for the clusters on x-axis (i.e. clustering given 1st). 
#'   By default uses names of the 1st column of clusters matrix. See details.
#' @param legNames names for the clusters dividing up the 1st clusters (will 
#'   appear in legend). By default uses names of the 2nd cluster of clusters
#'   matrix. If only one clustering, \code{xNames} and \code{legNames} refer to
#'   the same clustering. See details.
#' @param legend whether to plot the legend
#' @param xlab label for x-axis. By default or if equal NULL the column name of 
#'   the 1st cluster of clusters matrix
#' @param legend.title label for legend. By default or if equal NULL the column 
#'   name of the 2st cluster of clusters matrix
#' @param labels if object is a ClusterExperiment object, then labels defines 
#'   whether the clusters will be identified by their names values in 
#'   clusterLegend (labels="names", the default) or by their clusterIds value in
#'   clusterLegend (labels="ids").
#' @param ... for \code{plotBarplot} arguments passed either to the method of
#'   \code{plotBarplot} for matrices or ultimately to \code{\link{barplot}}.
#' @details The first column of the cluster matrix will be on the x-axis and the
#'   second column (if present) will separate the groups of the first column.
#' @details All arguments of the matrix version can be passed to the 
#'   \code{ClusterExperiment} version. As noted above, however, some arguments 
#'   have different interpretations.
#' @details If \code{whichClusters = "workflow"}, then the most recent two 
#'   clusters of the workflow will be chosen where recent is based on the 
#'   following order (most recent first): final, mergeClusters, makeConsensus, 
#'   clusterMany.
#' @details \code{xNames}, \code{legNames} and \code{colPalette} should all be
#'   named vectors, with the names referring to the clusters they should match
#'   to (for \code{ClusterExperiment} objects, it is determined by the argument
#'   \code{labels} as to whether the names should match the cluster names or the
#'   clusterIds). \code{colPalette} and \code{legNames} must be same length of
#'   the number of clusters found in the second clustering, or if  only a single
#'   clustering, the same length as the number of clusters in that clustering.
#' @return A plot is produced, nothing is returned
#' @author Elizabeth Purdom
#' @inheritParams plotClusters
#' @inheritParams getClusterIndex
#' @examples
#' \dontrun{
#' #clustering using pam: try using different dimensions of pca and different k
#' data(simData)
#'
#' cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reduceMethod="PCA",
#' clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
#' removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)
#'
#' plotBarplot(cl)
#' plotBarplot(cl,whichClusters=1:2)
#' }
#' @rdname plotBarplot
#' @export
setMethod(
  f = "plotBarplot",
  signature = signature(object = "ClusterExperiment"),
  definition = function(object, whichClusters="primary",labels=c("names","ids"),...)
  { 
	whichClusters<-getClusterIndex(object,whichClusters=whichClusters,noMatch="throwError")
	whichClusters<-head(whichClusters,2) #limit it to 2
	labels<-match.arg(labels)
    legend<-clusterLegend(object)[[tail(whichClusters,1)]]
    args<-list(...)
    if(!"colPalette" %in% names(args)){
      useBuiltInColors<-TRUE
      colPalette<-legend[,"color"]
    }
    else{
      colPalette<-args$colPalette
      useBuiltInColors<-FALSE
      args<-args[-grep("colPalette",names(args))]
    }
    numClusterMat<-clusterMatrix(object,whichClusters=whichClusters)
    if(labels=="names"){
	  clusterMat<-convertClusterLegend(object,output="matrixNames")[,whichClusters]
      if(useBuiltInColors) names(colPalette)<-legend[,"name"]
      #make sure "-1" stays "-1" 
      clusterMat[numClusterMat== -1]<- "-1"
      clusterMat[numClusterMat== -2]<- "-2"
      if(any(legend[,"clusterIds"]== "-1") & useBuiltInColors){
        names(colPalette)[which(legend[,"clusterIds"]== "-1")]<-"-1"
      }
      if(any(legend[,"clusterIds"]== "-2") & useBuiltInColors){
        names(colPalette)[which(legend[,"clusterIds"]== "-2")]<-"-2"
      }
    }
    else{
      clusterMat<-numClusterMat
      if(useBuiltInColors) names(colPalette)<-legend[,"clusterIds"]
    }
    if(!"unassignedColor" %in% names(args) & any(legend[,"clusterIds"]== "-1")){
      args$unassignedColor<-legend[legend[,"clusterIds"]== "-1","color"]
    }
    if(!"missingColor" %in% names(args) & any(legend[,"clusterIds"]== "-2")){
      args$missingColor<-legend[legend[,"clusterIds"]== "-2","color"]
    }
    
    do.call("plotBarplot",c(list(object=clusterMat,colPalette=colPalette),args))
    
  })


#' @rdname plotBarplot
#' @export
setMethod(
  f = "plotBarplot",
  signature = signature(object = "vector"),
  definition = function(object, ...){
    plotBarplot(matrix(object,ncol=1),...)
  })

#' @rdname plotBarplot
#' @importFrom graphics barplot
#' @export
setMethod(
  f = "plotBarplot",
  signature = signature(object = "matrix"),
  definition = function(object,  xNames=NULL, legNames=NULL, legend=ifelse(ncol(object)==2,TRUE,FALSE), xlab=NULL, legend.title=NULL, unassignedColor="white", missingColor="grey", colPalette=NULL,...){
    if(ncol(object)>2) stop("if 'object' a matrix, must contain at most 2 clusters (i.e. 2 columns)")
    clLeg<-object[,1]
    if(is.null(xlab)) xlab<-colnames(object)[1]
    if(ncol(object)==2){
      clX<-object[,2]
      x<-t(table(clLeg,clX))
      nX<-nrow(x)
      xnames<-rownames(x)
    }
    else{
      x<-table(clLeg)
      nX<-length(x)
      xnames<-names(x)
    }
    #-----------
    ###Check colors given:
    #-----------
    if(is.null(colPalette)){
      colPalette<-bigPalette[seq_len(nX)]
      names(colPalette)<-xnames
    }
    if(is.null(names(colPalette)) & length(colPalette)>1) stop("must give names to colPalette") 
    if(length(colPalette)==1){
      if(ncol(object)==1){
        colPalette<-rep(colPalette,length=nX)
        names(colPalette)<-xnames
      }
      else stop("cannot give a single color to colPalette if comparing 2 clusters.")
    } 
    if(!all(xnames %in% names(colPalette))) stop("invalid names for colPalette")
    colPalette<-colPalette[rownames(x)]
    
    if(ncol(object)==2){
      pair<-TRUE
      
      #references is on the columns, alt on rows
      if(is.null(legend.title)) legend.title<-colnames(object)[2]	
      ###cols of x go along the x-axis ("Ref")
      ###rows of x divide up the cols of x ("Alt")
      ###Note: colPalette is of length of rows of x ("Alt")
      #change name and color of missing/unassigned
      whAltNotAssigned<-which(row.names(x)=="-1")
      whAltMissing<-which(row.names(x)=="-2")
      whRefNotAssigned<-which(colnames(x)=="-1")
      whRefMissing<-which(colnames(x)=="-2")
      if(length(whAltNotAssigned)>0){
        row.names(x)[whAltNotAssigned]<-"Not Assigned"
        colPalette[whAltNotAssigned]<-unassignedColor
      }
      if(length(whAltMissing)>0){
        row.names(x)[whAltMissing]<-"Not Included in Clustering"
        colPalette[whAltMissing]<-missingColor
      }
      if(length(whRefNotAssigned)>0){
        colnames(x)[whRefNotAssigned]<-"Not Assigned"
      }
      if(length(whRefMissing)>0){
        colnames(x)[whRefMissing]<-"Not Included in Clustering"
      }
      #change order so those are last
      if(any(length(whAltNotAssigned)>0 | length(whAltMissing)>0)){
        nm<-row.names(x)
        wh<-c(whAltNotAssigned,whAltMissing)
        x<-rbind(x[-wh,,drop=FALSE],x[wh,,drop=FALSE])
        rownames(x)<-c(nm[-wh],nm[wh]) #annoying, but otherwise still loose the names
        colPalette<-c(colPalette[-wh],colPalette[wh])
      }
      if(any(length(whRefNotAssigned)>0 | length(whRefMissing)>0)){
        nm<-colnames(x)
        wh<-c(whRefNotAssigned,whRefMissing)
        x<-cbind(x[,-wh,drop=FALSE],x[,wh,drop=FALSE])
        colnames(x)<-c(nm[-wh],nm[wh]) #annoying, but otherwise still loose the names
      }
      if(is.null(legNames)){
        legNames<-colnames(x)
        names(legNames)<-colnames(x)
        labs<-legNames
      }
      else{
        
        if(is.null(names(legNames))) stop("must give names to legNames that match values of reference cluster")
        if(length(legNames)!=ncol(x)) stop("Invalid reference cluster names -- not same length as number of reference clusters")
        if(!identical(sort(names(legNames)),sort(colnames(x)))) stop("Invalid names for reference cluster names -- not match names of reference clusters")
        #put in same order
        legNames<-legNames[colnames(x)]
        labs<-paste(legNames," (",colnames(x),")",sep="")
      }
    }
    else{
      if(is.null(legNames)){
        if("-1" %in% names(x)){
          names(x)[names(x)=="-1"]<-"Not Assigned"
        }
        if("-2" %in% names(x)){
          names(x)[names(x)=="-2"]<-"Not Included in Clustering"
        }
        legNames<-names(x)
        names(legNames)<-names(x)
        labs<-legNames
      }
      else{
        if(is.null(names(legNames))) stop("must give names to legNames that match values of reference cluster")
        if(length(legNames)!=ncol(x)) stop("Invalid reference cluster names -- not same length as number of reference clusters")
        if(!identical(sort(names(legNames)),sort(names(x)))) stop("Invalid names for reference cluster names -- not match names of reference clusters")
        #put in same order
        legNames<-legNames[names(x)]
        labs<-paste(legNames," (",names(x),")",sep="")
      }
      
    }
    par(mar=c(9.1,4.1,4.1,1.1),las=2)
    bp<-graphics::barplot(x,col=colPalette,legend=legend,args.legend=list(title=legend.title), names.arg=rep("",length(labs)),xlab="",...)
    xsize<-diff(par("usr")[3:4])
    text(bp, par("usr")[3]+.0*xsize, labels=labs, srt=45, adj=c(1,2), xpd=TRUE)
    title(xlab=xlab,line=7)
    
  })

#' @rdname plotClustersWorkflow
#' @aliases plotClustersWorkflow
#' @title A plot of clusterings specific for clusterMany and workflow
#'   visualization
#' @description A realization of \code{\link{plotClusters}} call specific to
#'   separating out the results of \code{clusterMany} and other clustering
#'   results.
#' @param object A \code{ClusterExperiment} object on which
#'   \code{\link{clusterMany}} has been run
#' @param whichClusterMany indicate which clusterings to plot in the bulk of the
#'   plot, see argument \code{whichClusters} of \code{\link{getClusterIndex}}
#'   for description of format allowed.
#' @param whichClusters which clusterings to "highlight", i.e draw separately
#'   from the bulk of the plot, see argument \code{whichClusters} of
#'   \code{\link{getClusterIndex}} for description of format allowed.
#' @param nBlankLines the number of blank (i.e. white) rows to add between the
#'   clusterMany clusterings and the highlighted clusterings.
#' @param nSizeResult the number of rows each highlighted clustering should take up.
#'   Increasing the number increases the thickness of the rectangles
#'   representing the highlighted clusterings.
#' @param clusterManyLabels either logical, indicating whether to plot the
#'   labels for the clusterings from clusterMany identified in the
#'   \code{whichClusterMany}, or a character vector of labels to use.
#' @param clusterLabels either logical, indicating whether to plot the labels
#'   for the clusterings identified to be highlighted in the
#'   \code{whichClusters} argument, or a character vector of labels to use.
#' @param sortBy how to align the clusters. If "highlighted" then the
#'   highlighted clusters indicated in the argument \code{whichClusters} are
#'   first in the alignment done by \code{plotClusters}. If "clusterMany", then
#'   the clusterMany results are first in the alignment. (Note this does not
#'   determine where they will be plotted, but how they are ordered in the
#'   aligning step done by \code{plotClusters})
#' @param highlightOnTop logical. Whether the highlighted clusters should be
#'   plotted on the top of clusterMany results or underneath.
#' @param existingColors one of "ignore","all","highlightOnly". Whether the plot
#'   should use the stored colors in the \code{ClusterExperiment} object given.
#'   "highlightOnly" means only the highlighted clusters will use the stored
#'   colors, not the clusterMany clusterings.
#' @param ... arguments passed to the matrix version of
#'     \code{\link{plotClusters}}
#' @details This plot is solely intended to make it easier to use the 
#'     \code{\link{plotClusters}} visualization when there are a large number of
#'     clusterings from a call to \code{\link{clusterMany}}. This plot separates
#'     out the \code{\link{clusterMany}} results from a designated clustering of
#'     interest, as indicated by the \code{whichClusters} argument
#'     (by default clusterings from a call to \code{\link{makeConsensus}} or 
#'     \code{\link{mergeClusters}}). In addition the highlighted clusters are
#'     made bigger so that they can be easily seen.
#' @seealso \code{\link{plotClusters}}, \code{\link{clusterMany}}
#' @return A plot is produced, nothing is returned.
#' @examples
#' #clustering using pam: try using different dimensions of pca and different k
#' \dontrun{
#' data(simData)
#'
#' cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reduceMethod="PCA",
#' clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
#' removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)
#' cl <- makeConsensus(cl, proportion=0.7)
#' plotClustersWorkflow(cl)
#' }
#' @export
setMethod(
  f = "plotClustersWorkflow",
  signature = signature(object = "ClusterExperiment"),
  definition = function(object, whichClusters=c("mergeClusters","makeConsensus"), whichClusterMany=NULL, nBlankLines=ceiling(nClusterings(object)*.05), existingColors=c("ignore","all","highlightOnly"),
                        nSizeResult=ceiling(nClusterings(object)*.02), clusterLabels=TRUE, clusterManyLabels=TRUE, sortBy=c("highlighted","clusterMany"), highlightOnTop=TRUE,...)
  {
    sortBy<-match.arg(sortBy)
    existingColors<-match.arg(existingColors)
    allClusterMany<-which(clusterTypes(object)=="clusterMany")
    if("colData" %in% names(list(...))) stop("this function does not (yet) allow the designation of 'colData' argument. You must use plotClusters for this option.")
    if(is.null(whichClusterMany)){
      whichClusterMany<-allClusterMany
    }
    #convert to indices
	whichClusters<-getClusterIndex(object,whichClusters=whichClusters,noMatch="silentlyRemove")
    if(length(whichClusters)==0) stop("invalid identification of clusters for whichClusters argument")
	whichClustersMany<-getClusterIndex(object,whichClusters=whichClusterMany,noMatch="silentlyRemove")
	if(length(whichClusterMany)==0) stop("invalid identification of clusters for whichClusters argument")
    
    
    #result labels (yaxis):
    if(is.logical(clusterLabels)){
      if(clusterLabels) clusterLabels<-clusterLabels(object)[whichClusters]
      else clusterLabels<-rep("",length(whichClusters))  
    }
    else{
      if(length(clusterLabels)!=length(whichClusters) & !is.null(clusterLabels)){
        stop("number of cluster labels given in clusterLabels must be equal to the number of clusterings in 'whichClusters'")
        
      }
    }
    #clusterMany labels (yaxis):
    if(is.logical(clusterManyLabels)){
      if(clusterManyLabels) clusterManyLabels<-clusterLabels(object)[whichClusterMany]
      else clusterManyLabels<-rep("",length(whichClusterMany))
    }
    else{
      if(length(clusterManyLabels)!=length(whichClusterMany) & !is.null(clusterMany)){
        stop("number of cluster labels given in clusterManyLabels must be equal to the number of clusterings in 'whichClusterMany'")
        
      }
    }
    ###Get the sorted index using the matrix version of plotClusters
    ### out is the result of plotClusters
    if(sortBy=="highlighted"){
      orderOfClusters<-c(whichClusters,whichClusterMany)
      highIndex<-c(seq_along(whichClusters))
      cmIndex<-seq_along(orderOfClusters)[-highIndex]
    }
    else{
      orderOfClusters<-c(whichClusterMany,whichClusters)
      cmIndex<-c(seq_along(whichClusterMany))
      highIndex<-seq_along(orderOfClusters)[-cmIndex]
      
    }
    tempClusters<-clusterMatrix(object)[,orderOfClusters,drop=FALSE]
    out<-plotClusters(tempClusters,plot=FALSE)	 	
      
    ### Create color matrix
    ### resM is the highlighted clusters (columns the clusters)
    ### cmM is the clusterMany clusters (columns the clusters)
    
    if(existingColors!="ignore") 
      existingColorMat<-convertClusterLegend(object, whichClusters=orderOfClusters, output="matrixColors")
    
    if(existingColors %in% c("all","highlightOnly")){
      resM<-existingColorMat[,highIndex,drop=FALSE]
    }
    else{
      resM<-out$colors[,highIndex,drop=FALSE]
      
    }
    if(existingColors =="all"){
      cmM<-existingColorMat[,cmIndex,drop=FALSE]
    }
    else{
      cmM<-out$colors[,cmIndex,drop=FALSE] 
    }
    
    # make replication of results
    repResults<-lapply(seq_len(ncol(resM)),function(ii){
      x<-resM[,ii]
      mat<-matrix(x,nrow=length(x),ncol=nSizeResult,byrow=FALSE)
      colnames(mat)<-rep("",ncol(mat))
      colnames(mat)[ceiling(ncol(mat)/2)]<-clusterLabels[ii]
      return(mat)
    })
    repResults<-do.call("cbind",repResults)
    ##Add blanks
    if(highlightOnTop){
      bd<-makeBlankData(data=t(cbind(resM,cmM)),
						groupsOfFeatures=list("Results"=seq_along(whichClusters),"ClusterMany"=(length(whichClusters)+1):(length(whichClusters)+length(whichClusterMany))),
						nBlankFeatures=nBlankLines
						)
      whNotRes<-(length(whichClusters)+1):nrow(bd$dataWBlanks) #includes blanks
      whCM<-whNotRes[-c(seq_len(nBlankLines))] #no blanks
    } 	
    else{
      bd<-makeBlankData(data=t(cbind(cmM,resM)), groupsOfFeatures= list("ClusterMany"=seq_along(whichClusterMany), "Results"=(length(whichClusterMany)+1):(length(whichClusterMany)+length(whichClusters))),nBlankFeatures=nBlankLines)
      whNotRes<-  seq_len(length(whichClusterMany)+nBlankLines) #includes blanks
      whCM<-  seq_along(whichClusterMany) #no blanks
    }  
    test<-t(bd$dataWBlanks)
    test[is.na(test)]<-"white"
    
    ###ClusterLabels
    
    colnames(test)<-rep("",ncol(test))
    colnames(test)[whCM]<-clusterManyLabels
    
    if(highlightOnTop) test<-cbind(repResults,test[,whNotRes])
    else test<-cbind(test[,whNotRes],repResults)
    
    plotClusters(test[out$orderSamples,],input="colors",...)
    
  })

#' Convert clusterLegend into useful formats
#'
#' Function for converting the information stored in the clusterLegend slot into
#' other useful formats.
#'
#' @param object a \code{ClusterExperiment} object.
#' @param output character value, indicating desired type of conversion.
#' @inheritParams getClusterIndex
#' @details convertClusterLegend pulls out information stored in the
#'   \code{clusterLegend} slot of the object and returns it in useful format.
#'
#' @return If \code{output="plotAndLegend"}, \code{"convertClusterLegend"} will
#'   return a list that provides the necessary information to color samples
#'   according to cluster and create a legend for it:
#'  \itemize{
#'  \item{"colorVector"}{ A vector the same length as the number of samples,
#'  assigning a color to each cluster of the primaryCluster of the object.}
#'  \item{"legendNames"}{ A vector the length of the number of clusters of
#'  primaryCluster of the object giving the name of the cluster.}
#'  \item{"legendColors"}{ A vector the length of the number of clusters of
#'  primaryCluster of the object giving the color of the cluster.}
#' }
#' @return If \code{output="aheatmap"} a conversion of the clusterLegend to be
#'   in the format requested by \code{\link[NMF]{aheatmap}}. The column 'name'
#'   is used for the names and the column 'color' for the color of the clusters.
#' @return If \code{output="matrixNames"} or \code{"matrixColors"} a matrix the
#'   same dimension of \code{clusterMatrix(object)}, but with the cluster color
#'   or cluster name instead of the clusterIds, respectively.
#'
#' @importFrom RColorBrewer brewer.pal brewer.pal.info
#' @export
#' @rdname plottingFunctions
#' @aliases convertClusterLegend convertClusterLegend,ClusterExperiment-method
setMethod(
  f = "convertClusterLegend",
  signature = c("ClusterExperiment"),
  definition = function(object,output=c("plotAndLegend","aheatmapFormat","matrixNames","matrixColors"),whichClusters=ifelse(output=="plotAndLegend","primary","all")){
    output<-match.arg(output)
    whichClusters<-getClusterIndex(object,whichClusters=whichClusters,noMatch="throwError")
    if(output=="aheatmapFormat"){
      outval<-.convertToAheatmap(clusterLegend(object)[whichClusters])
    }
    if(output%in% c("matrixNames","matrixColors")){
      outval<-do.call("cbind",lapply(seq_along(whichClusters),function(ii){
        cl<-clusterMatrix(object)[,whichClusters,drop=FALSE][,ii]
        colMat<-clusterLegend(object)[whichClusters][[ii]]
        m<-match(cl,colMat[,"clusterIds"])
        colReturn<-if(output=="matrixNames") "name" else "color"
        return(colMat[m,colReturn])
      }))
      colnames(outval)<-clusterLabels(object)[whichClusters]
      
    }
    if(output=="plotAndLegend"){
      if(length(whichClusters)>1) stop("given whichClusters indicates more than 1 clustering which is not allowed for option 'plotAndLegend'")
      cl<-clusterMatrix(object)[,whichClusters]
      colMat<-clusterLegend(object)[[whichClusters]]
      clColor<-colMat[match(cl,colMat[,"clusterIds"]),"color"]
      legend<-colMat[,"name"]
      color<-colMat[,"color"]
      outval<-list(colorVector=clColor,legendNames=legend,legendColors=color)
      
    }
    return(outval)
    
  }
)

.convertToClusterLegend<-function(acol){
	if(!is.list(acol)){ #single vector
		acol<-list(acol)
	}
	out<-lapply(acol,function(x){
		if(!is.null(names(x))) nms<-names(x)
		else nms<-as.character(seq_along(x))
		cols<-unname(x)
		return(cbind("color"=cols,"name"=nms))
	})
	names(out)<-names(acol)
	return(out)
}

.convertToAheatmap<-function(clusterLegend, names=FALSE){
  outval<-lapply(clusterLegend,function(x){
    if(!is.null(dim(x))){
      z<-x[,"color"]
      if(names) {
        names(z)<-x[,"name"]
      } else {
        names(z)<-x[,"clusterIds"]
      }
      z<-z[order(names(z))]
      return(z)
    }
    else return(x)
  })
  return(outval)
  
}

#' @title Various functions useful for plotting
#' @description Most of these functions are called internally by plotting
#'   functions, but are exported in case the user finds them useful.
#' @name plottingFunctions
#' @aliases bigPalette showPalette
#' @details \code{bigPalette} is a long palette of colors (length 58) used by 
#'   \code{\link{plotClusters}} and accompanying functions. \code{showPalette}
#'   creates plot that gives index of each color in a vector of colors.
#'   \code{massivePalette} is a combination of \code{bigPalette} and the
#'   non-grey colors of \code{\link{colors}()} (length 487). 
#'   \code{massivePalette} is mainly useful for when doing
#'   \code{\link{plotClusters}} of a very large number of clusterings, each with
#'   many clusters, so that the code doesn't run out of colors. However, many of
#'   the colors will be very similar to each other.
#' @param which numeric. Which colors to plot. Must be a numeric vector with
#'   values between 1 and length of \code{colPalette}. If missing, all colors
#'   plotted.
#' @param cex numeric value giving the cex for the text of the plot.
#' @param colPalette a vector of character colors. By default, the palette 
#'   \code{bigPalette} is used
#' @details \code{showPalette} will plot the \code{colPalette} colors with their
#'   labels and index.
#'
#' @export
#'
#' @examples
#' showPalette()
#' showPalette(massivePalette,cex=0.6)
showPalette<-function(colPalette=bigPalette,which=NULL,cex=1){
  oldPar<-par(no.readonly = TRUE)
  wh<-which
  if(is.null(wh)){
    wh<-seq_along(colPalette)
  }
  else{ colPalette<-colPalette[wh]}
  n<-ceiling(sqrt(length(colPalette)))
  nblank<-n^2-length(colPalette)
  xwide<-n
  yup<-n
  x1<-rep(c(seq_len(xwide))-.5,yup)
  x2<-rep(c(seq_len(xwide))+.5,yup)
  xtext<-rep(c(seq_len(xwide)),yup)
  ycolor1<-rep(seq(1,yup*2,by=2)-.5,each=xwide)
  ycolor2<-rep(seq(1,yup*2,by=2)+.5,each=xwide)
  ytext<-rep(seq(2,yup*2,by=2)+.5,each=xwide)

  par(mar=c(0,0,0,0),omi=c(0,0,0,0))
  plot.new()
  plot.window(xlim=c(.5,xwide+.5),ylim=c(.5,(yup*2)+.5))
  rect(x1,ycolor1,x2,ycolor2,col=c(colPalette,rep("white",nblank)),border=FALSE)
  if(length(colPalette)>100){
	  half<-ceiling(length(colPalette)/2)
	 adj.text<-cbind(rep(.5,half*2),rep(c(0,1),times=half))
	 adj.text<-adj.text[seq_along(colPalette),]
  }
  else adj.text<-matrix(c(0.5,0),nrow=length(colPalette),ncol=2,byrow=TRUE)
  for(i in seq_along(colPalette)){
      text(xtext[i],ytext[i]-1,colPalette[i],cex=cex,adj=adj.text[i,])
      if(length(colPalette)<=100) text(xtext[i],ytext[i]-2,wh[i],cex=cex,adj=c(0.5,1))
  }
	par(oldPar)
}

#' @rdname plottingFunctions
#' @export
bigPalette<-c(
	'#E31A1C',
	'#1F78B4',
	'#33A02C',
	'#FF7F00',
	'#6A3D9A',
	'#B15928',
	'#A6CEE3',
	'#bd18ea',
	'cyan',
	'#B2DF8A',
	'#FB9A99',
	"deeppink4",
	'#00B3FFFF',
	'#CAB2D6',
	'#FFFF99',
	'#05188a',
	'#CCFF00FF',
	'cornflowerblue',
	'#f4cc03',
	'black',
	'blueviolet',
	'#4d0776',
	'maroon3',
	'blue',
#	'grey',
	'#E5D8BD',
	'cadetblue4',
	'#e5a25a',
	"lightblue1",
	'#F781BF',
	'#FC8D62',
	'#8DA0CB',
	'#E78AC3',
	'green3',
	'#E7298A',
	'burlywood3',
	'#A6D854',
	"firebrick",
	'#FFFFCC',
	"mediumpurple",
	'#1B9E77',
	'#FFD92F',
	'deepskyblue4',
	"yellow3",
	'#00FFB2FF',
	'#FDBF6F',
	'#FDCDAC',
	"gold3",
	'#F4CAE4',
	'#E6F5C9',
	'#FF00E6FF',
	'#7570B3',
	"goldenrod",
	'#85848f',
	"lightpink3",
	"olivedrab",
#	"plum",
#	"lightskyblue3",
#	"mediumturquoise",
	'cadetblue3'
)

#' @importFrom grDevices colors
.rcolors<-function(){
	#so sure that setting seed doesn't mess up when installing package
	set.seed(23589)
	return(sample(colors()[-c(152:361)]))
}

#' @rdname plottingFunctions
#' @export
massivePalette<-unique(c(bigPalette,.rcolors()))




#' @param breaks either vector of breaks, or number of breaks (integer) or a
#'   number between 0 and 1 indicating a quantile, between which evenly spaced
#'   breaks should be calculated. If missing or NA, will determine evenly spaced
#'   breaks in the range of the data.
#' @param makeSymmetric whether to make the range of the breaks symmetric around zero (only used if not all of the data is non-positive and not all of the data is non-negative)
#' @param returnBreaks logical as to whether to return the vector of breaks. See details.
#' @details if returnBreaks if FALSE, instead of returning the vector of breaks, the function will just return the second smallest and second largest value of the breaks. This is useful for alternatively just setting values of the data matrix larger than these values to this value if breaks was a percentile. This argument is only used if \code{breaks<1}, indicating truncating the breaks for large values of data.
#' @rdname plottingFunctions
#' @details \code{setBreaks} gives a set of breaks (of length 52) equally spaced
#'   between the boundaries of the data. If breaks is between 0 and 1, then the
#'   evenly spaced breaks are between these quantiles of the data.
#' @export
#' @examples
#' setBreaks(data=simData,breaks=.9)
setBreaks<-function(data,breaks=NA,makeSymmetric=FALSE,returnBreaks=TRUE){
  if(all(is.na(data))) stop("data consists only of NA values")
  if(length(unique(na.omit(as.vector(data))))==1){
    warning("data consists of only a single non NA value")
    val<-unique(na.omit(as.vector(data)))
    return(seq(val-1,val+1,length=52))
  }
  isPositive<-all(na.omit(as.vector(data))>=0)
  isNegative<-all(na.omit(as.vector(data))<=0)
  #
  #get arround bug in aheatmap
  #if colors are given, then get back 51 colors, unless give RColorBrewer, in which case get 101! Assume user has to give palette. So breaks has to be +1 of that length
  #TO DO: might not need any more with updated aheatmap.
  ncols<-51
  minData<-min(data,na.rm=TRUE)
  maxData<-max(data,na.rm=TRUE)
  maxAbsData<-max(abs(data),na.rm=TRUE)
  if(!is.vector(breaks)) stop("breaks argument must be a vector")
  if(missing(breaks) || is.na(breaks)){
    #go from minimum to max
    if(makeSymmetric & !isPositive & !isNegative){
      breaks<-seq(-maxAbsData,maxAbsData,length=ncols+1)
      seconds<-c(-maxAbsData,maxAbsData)
    }
    else{
      breaks<-seq(minData,maxData,length=ncols+1)
      seconds<-c(minData,maxData)
    }
    
  }
  else if(length(breaks)>0 && !is.na(breaks)){
    if(length(breaks)==1){
      if(breaks<1){
        if(breaks<0.5) breaks<-1-breaks
        #
        uppQ<-if(isPositive) quantile(data[which(data>0)],breaks,na.rm=TRUE) else quantile(data,breaks,na.rm=TRUE)
        lowQ<-if(isPositive) min(data,na.rm=TRUE) else quantile(data,1-breaks,na.rm=TRUE)
        
        if(makeSymmetric & !isPositive & !isNegative){
          #make breaks symmetric around 0
          absq<-max(abs(c(lowQ,uppQ)),na.rm=TRUE)
          absm<-max(abs(c(min(data,na.rm=TRUE),max(data,na.rm=TRUE))))
          #is largest quantile also max of abs(data)?
          quantAllMax <- if( isTRUE( all.equal(round(absq,5), round(absm,5)))) TRUE else FALSE
          if(!quantAllMax){
            breaks <- c(-absm, seq(-absq,absq,length=ncols-1), absm)
            seconds<-c(-absq,absq)
          }
          else{
            #equally spaced
            breaks <- seq(-absm,absm,length=ncols+1)
            seconds<-c(-absm,absm)
          }
        }
        else{
          #determine if those quantiles are min/max of data
          quantMin <- if( isTRUE( all.equal(round(lowQ,5), round(minData,5)))) TRUE else FALSE
          quantMax<-if( isTRUE( all.equal(round(uppQ,5),round(maxData,5)))) TRUE else FALSE
          
          if(!quantMin & !quantMax){
            breaks <- c(minData, seq(lowQ,uppQ,length=ncols-1), maxData)
            seconds<-c(lowQ,uppQ)
          }
          if(!quantMin & quantMax){
            breaks <- c(minData, seq(lowQ,maxData,length=ncols))
            seconds<-c(lowQ,maxData)
          }
          if(quantMin & !quantMax){
            breaks <- c(seq(minData,uppQ,length=ncols), maxData)
            seconds<-c(minData,uppQ)
          }
          if(quantMin & quantMax){
            breaks<-seq(minData,maxData,length=ncols+1)
            seconds<-c(minData,maxData)
          }
        }
      }
      else{ #breaks is number of breaks
        if(length(breaks)!=52) warning("Because of bug in aheatmap, breaks should be of length 52 -- otherwise the entire spectrum of colors will not be used")
        if(makeSymmetric& !isPositive & !isNegative){
          breaks<-seq(-maxAbsData,maxAbsData,length=breaks)
          seconds<-c(-maxAbsData,maxAbsData)
        }
        else{
          breaks<-seq(minData,maxData,length=breaks)
          seconds<-c(minData,maxData)
        }
      }
    }
  }
  if(!length(unique(breaks))==length(breaks)){
    breaks<-sort(unique(breaks))
    warning("setBreaks did not create unique breaks, and therefore the resulting breaks will not be of exactly the length requested. If you have replicable code that can recreate this warning, we would appreciate submitting it to the issues tracker on github (existing issue: #186)")
    seconds<-c(min(breaks),max(breaks))
  }
  if(returnBreaks) return(breaks)
  else return(seconds)
  
}



#' @rdname plottingFunctions
#'
#' @details \code{seqPal1}-\code{seqPal4} are palettes for the heatmap.
#'   \code{showHeatmapPalettes} will show you these palettes.
#'
#' @export
#'
#' @examples
#'
#' #show the palette colors
#' showHeatmapPalettes()
#'
#' #compare the palettes on heatmap
#' cl <- clusterSingle(simData, subsample=FALSE,
#' sequential=FALSE, 
#' mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=8)))
#'
#' \dontrun{
#' par(mfrow=c(2,3))
#' plotHeatmap(cl, colorScale=seqPal1, main="seqPal1")
#' plotHeatmap(cl, colorScale=seqPal2, main="seqPal2")
#' plotHeatmap(cl, colorScale=seqPal3, main="seqPal3")
#' plotHeatmap(cl, colorScale=seqPal4, main="seqPal4")
#' plotHeatmap(cl, colorScale=seqPal5, main="seqPal5")
#' par(mfrow=c(1,1))
#' }
#'
showHeatmapPalettes<-function(){
	palettesAll<-list(seqPal1=seqPal1,seqPal2=seqPal2,seqPal3=seqPal3,seqPal4=seqPal4,seqPal5=seqPal5)
	maxLength<-max(sapply(palettesAll,length))
	palettesAllAdj<-lapply(palettesAll,function(x){
		if(length(x)<maxLength) x<-c(x,rep("white",maxLength-length(x)))
			return(x)})
	ll<-list()
	sapply(seq_along(palettesAllAdj),function(ii){ll[[2*ii-1]]<<-palettesAllAdj[[ii]]})
	sapply(seq_len(length(palettesAllAdj)-1),function(ii){ll[[2*ii]]<<-rep("white",length=maxLength)})
	names(ll)[seq(1,length(ll),by=2)]<-names(palettesAll)
	names(ll)[seq(2,length(ll),by=2)]<-rep("",length(palettesAll)-1)
	mat<-do.call("cbind",ll)
	plotClusters(mat,input="colors")
}

#' @rdname plottingFunctions
#' @export
seqPal5<- colorRampPalette(c("black","navyblue","mediumblue","dodgerblue3","aquamarine4","green4","yellowgreen","yellow"))(16)
#' @rdname plottingFunctions
#' @export
seqPal2<-colorRampPalette(c("yellow","orange","black","blue","dodgerblue"))(16)[-c(2,14)]
seqPal2<-rev(seqPal2)
#seqPal2<- colorRampPalette(c("yellow","orange","black","blue","dodgerblue"))(16)
#seqPal2<-(c("yellow","gold2",seqPal2))
#' @rdname plottingFunctions
#' @export
seqPal3<-rev(RColorBrewer::brewer.pal(11, "RdBu"))
#' @rdname plottingFunctions
#' @export
#make it symmetric around white
seqPal4<-colorRampPalette(c("black","blue","white","red","orange","yellow"))(16)[c(1:9,12,13,14,16)]
#' @rdname plottingFunctions
#' @export
seqPal1<-rev(RColorBrewer::brewer.pal(11, "Spectral"))


#' @export
#' @param title title for the clusterLegend plot
#' @param clusterNames vector of names for the clusters; vector should have names
#'  that correspond to the clusterIds in the ClusterExperiment object. If this
#'  argument is missing, will use the names in the "name" column of the clusterLegend
#'  slot of the object.
#' @param add logical. Whether legend should be added to the existing plot.
#' @param location character passed to \code{x} argument of legend indicating 
#'  where to place legend.
#' @param ... arguments passed to legend
#' @inheritParams getClusterIndex
#' @rdname plottingFunctions
#' @aliases plotClusterLegend
setMethod(
  f = "plotClusterLegend",
  signature = c("ClusterExperiment"),
  definition = function(object,whichCluster="primary",clusterNames,title,add=FALSE,location=if(add)"topright" else "center",...){
    whichCluster<-getSingleClusterIndex(object,whichCluster,list(...))
    legMat<-clusterLegend(object)[[whichCluster]]
    if(!missing(clusterNames)){
      if(is.null(names(clusterNames))) stop("clusterNames must be named vector")
      m<-match(legMat[,"clusterIds"],names(clusterNames))
      if(any(is.na(m))){
        whClusters<-which(legMat[,"clusterIds"]>0)
        if(any(is.na(m[whClusters]))) stop("not all clusters are found in names of clusterNames argument")
        else{ #give default names to -1 and -2
          whMiss1<-which(legMat[,"clusterIds"]== -1 & is.na(m))
          if(length(whMiss1)>0) clusterNames<-c(clusterNames,"-1"="Not assigned")
          whMiss2<-which(legMat[,"clusterIds"]== -2 & is.na(m))
          if(length(whMiss2)>0) clusterNames<-c(clusterNames,"-2"="Not clustered")
        }
      }
      m<-match(legMat[,"clusterIds"],names(clusterNames))
      clusterNames<-clusterNames[m]
    }
    else{
      clusterNames<-legMat[,"name"]
    }
    if(missing(title)){
      title<-paste("Clusters of",clusterLabels(object)[whichCluster])
    }
    if(any(as.numeric(legMat[,"clusterIds"])<0)){
      #put -1/-2 last
      isNeg<-as.numeric(legMat[,"clusterIds"])<0
      ord<-c(which(!isNeg),which(isNeg))
    }
    else ord<-seq_len(nrow(legMat))
    if(!add){
			graphics::plot(0,0,type="n",xaxt="n",yaxt="n",xlab="",ylab="",bty="n")
			graphics::legend(x=location,legend=clusterNames[ord],fill=legMat[ord,"color"],title=title,...)
		}
		else{
			graphics::legend(x=location,legend=clusterNames[ord],fill=legMat[ord,"color"],title=title,...)
		}
    
    
  })


 ###Old definition of bigPalette (.thisPal):
 # bigPalette = c(
 # 	"#A6CEE3",#1 light blue
 # 	"#1F78B4",#2 dark blue
 # 	"#B2DF8A",#3 light green
 # 	"#33A02C",#4 dark green
 # 	"#FB9A99",#5 pink
 # 	"#E31A1C",#6 red
 # 	"#FDBF6F",#7 light orange
 # 	"#FF7F00",#8 orange
 # 	"#CAB2D6",#9 light purple
 # 	"#6A3D9A",#10 purple
 # 	"#FFFF99",#11 light yellow
 # 	"#B15928",#12 brown
 # 	"#bd18ea", #13 magenta
 # 	"#2ef4ca", #14 aqua
 # 	"#f4cced", #15 pink,
 # 	"#05188a", #16 navy,
 # 	"#f4cc03", #17 lightorange
 # 	"#e5a25a", #18 light brown
 # 	"#06f106", #19 bright green
 # 	"#85848f", #20 med gray
 # 	"#000000", #21 black
 # 	"#076f25", #22 dark green
 # 	"#93cd7f",#23 lime green
 # 	"#4d0776", #24 dark purple
 # 	"maroon3",
 # 	"blue",
 # 	"grey"
 # 		)
 # bigPalette<-bigPalette[c(2,4,6,8,10,12,14,13,1,3,5,7,9,11,16,19,20,15,17,18,21:27)]
 # .brewers<-RColorBrewer::brewer.pal.info[RColorBrewer::brewer.pal.info[,"category"]=="qual",]
 # .brewers<-.brewers[c(1:3,6:8,4:5),]
 # bigPalette<-c(bigPalette,palette()[-1],unlist(sapply(1:nrow(.brewers),function(ii){RColorBrewer::brewer.pal(.brewers[ii,"maxcolors"], rownames(.brewers)[ii])})))
 # bigPalette<-unique(bigPalette)
 # bigPalette<-bigPalette[-c(32,34,36,37,40,45:47,49:53,56,62:71,73,75,76,84,90,92 )] #remove because too similar to others
 # bigPalette<-bigPalette[-34] #very similar to 2
 # bigPalette<-bigPalette[-31] #very similar to 7

context("Non-heatmap related plot functions")



test_that("`plotClusters` works with matrix, ClusterExperiment objects", {

    #test matrix version
    x<-plotClusters(object=clusterMatrix(ceSim))
    expect_equal(dim(clusterMatrix(ceSim)),dim(x$colors))
    expect_equal(dim(clusterMatrix(ceSim)),dim(x$aligned))
    expect_equal(length(x$clusterLegend),ncol(clusterMatrix(ceSim)))

    #test CE version
    x<-plotClusters(ceSim)
    expect_is(x,"ClusterExperiment")
    expect_equal( x,ceSim)

    xx<-plotClusters(ceSim,whichClusters="clusterMany")
    xx2<-plotClusters(object=ceSim,whichClusters="workflow") #only clusterMany values so should be the same
    expect_equal(xx2,xx)

    #check reset -- should add combinations of resetColors and resetNames to make sure works independently.
    par(mfrow=c(1,2)) #so can visually check if desired.
    xx3<-plotClusters(ceSim,resetOrderSamples=TRUE,resetColors=TRUE,resetNames=TRUE)
    plotClusters(xx3,existingColors="all")
    expect_false(isTRUE(all.equal(xx2,xx3))) #not a great test. Doesn't really say whether does it right, just whether it does something!

    nm<-as.numeric(unlist(lapply(clusterLegend(xx3),function(x){x[,"name"]})))
    col<-(unlist(lapply(clusterLegend(xx3),function(x){x[,"color"]})))
    wh<-which(col %in% c("white","grey"))
    expect_equal(match(col[-wh],bigPalette),nm[-wh])
    nmOld<-as.numeric(unlist(lapply(clusterLegend(ceSim),function(x){x[,"name"]})))
    expect_false(isTRUE(all.equal(nm,nmOld)))
    idOld<-as.numeric(unlist(lapply(clusterLegend(ceSim),function(x){x[,"clusterIds"]})))
    idNew<-as.numeric(unlist(lapply(clusterLegend(xx3),function(x){x[,"clusterIds"]})))
    expect_equal(idOld,idNew)

    #check existing colors
    x2<-plotClusters(ceSim,existingColors="all")

    #test -1
    plotClusters(ceSim)

    #CE object with mixture of workflow and other types
    x1<-plotClusters(object=ceSim,whichClusters="workflow",resetColors=TRUE)
    x2<-plotClusters(object=removeClusterings(ceSim,"User"),resetColors=TRUE)
    whP<-getClusterIndex(ceSim,"workflow")
    expect_equal(clusterLegend(x2),clusterLegend(x1)[whP])

    #test specifying indices
    wh<-c(3,4,NCOL(clusterMatrix(ceSim)))
    x3<-plotClusters(ceSim,whichClusters=wh,axisLine=-2,resetColors=TRUE)
    x4<-plotClusters(ceSim,whichClusters=wh[c(3,2,1)],axisLine=-2,resetColors=TRUE)
    expect_false(isTRUE(all.equal(x3,x4)))

    par(mfrow=c(1,1)) #otherwise will affect other tests.
})



test_that("`plotClusters` rerun above tests with colData included", {

    #test matrix version
    expect_silent(x<-plotClusters(object=clusterMatrix(ceSim),
        colData=as.data.frame(colData(ceSim))))
    expect_equal(ncol(clusterMatrix(ceSim))+ncol(colData(ceSim)),
        ncol(x$colors))
    expect_equal(ncol(clusterMatrix(ceSim))+ncol(colData(ceSim)),
        ncol(x$aligned))
    expect_equal(length(x$clusterLegend),
        ncol(clusterMatrix(ceSim))+ncol(colData(ceSim)))

    #---    
    #test CE version
    #---    
    
    #no colData in test
    expect_silent( test<- clusterMany(simCount,
        reduceMethod="PCA", verbose=FALSE,
        nReducedDims=c(5,10,50), isCount=TRUE, makeMissingDiss=TRUE,
        clusterFunction="pam",ks=2:4,findBestK=c(TRUE,FALSE)
    ) )
    expect_error(plotClusters(test,
        colData=as.data.frame(colData(ceSim))),
        "no colData for object data")
    expect_error(plotClusters(ceSim,
        colData=as.data.frame(colData(ceSim))),
        "invalid values for pulling sample data from colData of object")
    expect_silent(plotClusters(ceSim,colData="all"))
    par(mfrow=c(1,2))
    expect_silent(x2<-plotClusters(ceSim,colData="all",resetColors=TRUE))
    expect_silent(x1<-plotClusters(ceSim,resetColors=TRUE))


    #check NAs
    naSim<-ceSim
    colData(naSim)[sample(size=10,x=1:nrow(naSim)),c("A","B")]<-NA
    expect_silent(plotClusters(naSim,colData=c("A","B")))

    #test the new TRUE option for colData
    expect_silent(plotClusters(naSim,colData=TRUE))

  #this is not working because first one puts -1/-2 last and second puts them first, and so then assigns different colors to the groups
#  expect_equal(x1,x2)
#   par(mfrow=c(1,2))
#   x2<-plotClusters(ceSim,colData="all",resetColors=FALSE)
#   x1<-plotClusters(ceSim,resetColors=FALSE)
    par(mfrow=c(1,1))

})


test_that("plotClustersWorkflow", {
	expect_silent(cc<-clusterMany(mat, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(3,4),reduceMethod=c("none","PCA","var"),clusterFunction="pam",
	                       subsample=FALSE, sequential=FALSE,run=TRUE,verbose=FALSE,
	                       isCount=FALSE))
	expect_silent(cc<-makeConsensus(cc,proportion=.7,whichClusters = "clusterMany"))
	expect_silent(plotClustersWorkflow(cc))
	expect_silent(plotClustersWorkflow(cc,clusterManyLabels=FALSE))
	expect_silent(plotClustersWorkflow(cc,sortBy="clusterMany"))
	expect_silent(plotClustersWorkflow(cc,sortBy="clusterMany",highlightOnTop=FALSE))
	expect_silent(plotClustersWorkflow(cc,highlightOnTop=FALSE))
	expect_silent(plotClustersWorkflow(cc,clusterManyLabels=FALSE,clusterLabels="test"))
	expect_error(plotClustersWorkflow(cc,clusterManyLabels=c("1","2"),clusterLabels="test"),"number of cluster labels given in clusterManyLabels")
	expect_error(plotClustersWorkflow(cc,clusterManyLabels=TRUE,clusterLabels=c("A","test")),"number of cluster labels given in clusterLabels")

})


test_that("plotting helpers", {
  expect_silent(convertClusterLegend(smSimCE,output="aheatmap"))
  expect_silent(convertClusterLegend(smSimCE,output="plotAndLegend"))
  expect_silent(convertClusterLegend(smSimCE,output="matrixNames"))
  expect_silent(convertClusterLegend(smSimCE,output="matrixColors"))
  expect_silent(convertClusterLegend(smSimCE,output="matrixNames",whichClusters=c("cluster1")))
  expect_silent(convertClusterLegend(smSimCE,output="matrixNames",whichClusters=1:3))
  expect_silent(convertClusterLegend(smSimCE,output="plotAndLegend",whichClusters=c("cluster1")))
  expect_error(convertClusterLegend(smSimCE,output="plotAndLegend",whichClusters=1:3),"given whichClusters indicates more than 1 clustering which is not allowed for option")

  expect_silent(plotClusterLegend(smSimCE))
  expect_silent(plotClusterLegend(smSimCE,whichCluster="cluster1"))
  expect_silent(showPalette())
  expect_silent(showPalette(massivePalette))
})



test_that("`plotBarplot` works with matrix, ClusterExperiment objects", {

    #test numeric matrix version
    expect_silent(plotBarplot(object=clusterMatrix(ceSim)[,1:2]))
    #test vector version
    expect_silent(plotBarplot(object=clusterMatrix(ceSim)[,1]))
    #check error
    expect_error(plotBarplot(object=clusterMatrix(ceSim)),"if 'object' a matrix, must contain at most 2 clusters")

    #test CE version with no defaults
    expect_silent(plotBarplot(ceSim))
    #test CE version whichClusters arguments
    expect_silent(plotBarplot(ceSim,whichClusters="workflow"))
    expect_silent(plotBarplot(ceSim,whichClusters="primaryCluster"))
    expect_silent(plotBarplot(ceSim))


    test<-ceSim
    expect_silent(clusterLegend(test)[[1]][,"name"]<-LETTERS[1:nrow(clusterLegend(ceSim)[[1]])])
    #test character matrix version
    expect_silent(plotBarplot(object=convertClusterLegend(test,output="matrixNames")[,1:2]))
    #test character vector version
    expect_silent(plotBarplot(object=convertClusterLegend(test,output="matrixNames")[,1]))
    #test labels argument
    expect_silent(plotBarplot(test,whichClusters=1:2,labels="id"))
    expect_silent(plotBarplot(test,whichClusters=1:2,labels="name"))
    #plotBarplot(ceSim,whichClusters="primaryCluster")

})




test_that("plotReducedDims works",{
	expect_silent(cl <- clusterMany(simData, 
        nReducedDims=c(5, 10, 50), reduceMethod="PCA",
	    clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
	    removeSil=c(TRUE,FALSE), 
        verbose=FALSE,makeMissingDiss=TRUE))
	expect_silent(plotReducedDims(cl,legend="bottomright"))
	expect_silent(plotReducedDims(cl,legend=TRUE))
	expect_silent(plotReducedDims(cl,legend=FALSE))
    whCl<-grep("k=4,nReducedDims=10,findBestK=FALSE,removeSil=TRUE",
        clusterLabels(cl))
    expect_true(length(whCl)==1)
    whCl<-clusterLabels(cl)[whCl] #so is character value, to test.
	expect_silent(clusterLegend(cl)[[whCl]][,"name"]<-LETTERS[1:5])
	expect_silent(plotReducedDims(cl, whichCluster = whCl, legend=TRUE))

	#test on object that doesn't have saved:
	expect_warning(clD<-plotReducedDims(ceSimData,reducedDim="PCA"),
        "will be run on the FIRST assay")
	expect_equal(NCOL(reducedDim(clD,type="PCA")),2) #default.

	#higher dims.
	expect_silent(plotReducedDims(cl,whichCluster=whCl,
        legend=TRUE,whichDims=1:4))
	expect_error(plotReducedDims(cl,whichCluster=whCl,
        legend=TRUE,whichDims=158:200),
        "Invalid value for whichDims: larger than row or column")
	#force it to recalculate:
	expect_warning(plotReducedDims(cl,whichCluster=whCl,legend=TRUE,
        whichDims=51:58),"will be run on the FIRST assay")



})

test_that("plotFeatureBoxplot works",{
	expect_silent(cl <- clusterMany(simData, 
        nReducedDims=c(5, 10, 50), reducedDim="PCA",
		clusterFunction="pam", ks=2:4, 
        findBestK=c(TRUE,FALSE), verbose=FALSE,
		removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE))
	expect_silent(clusterLegend(cl)[[1]][,"name"] <- 
        letters[1:nClusters(cl,ignoreUnassigned =FALSE)[1]])
	expect_silent(plotFeatureBoxplot(object=cl,feature=1))
	expect_silent(plotFeatureBoxplot(cc,feature=rownames(cc)[2]))
	expect_silent(plotFeatureBoxplot(cc,
        plotUnassigned=TRUE,feature=rownames(cc)[2]))
	#check if only 1 non-negative cluster in clustering
	expect_silent(out<-plotFeatureBoxplot(cl[,1:10],
        whichCluster=2,feature=2,plotUnassigned=FALSE))
	expect_equal(ncol(out$stats),1)
	#check if only 1 cluster in clustering
	expect_silent(out<-plotFeatureBoxplot(cl[,1:20],
        whichCluster="primary",feature=2))
	expect_equal(ncol(out$stats),1)
})

test_that("plotClustersTable works",{
  
	# These are slow!
	skip_on_os("windows")
	skip_on_os("mac")
	#test where should be diagonal
	expect_silent(plotClustersTable(cc,whichClusters=c(1,2)))
	expect_silent(plotClustersTable(cc,whichClusters=c(1,2),ignoreUnassigned=TRUE,margin=2))
	expect_silent(plotClustersTable(cc,whichClusters=c(1,2),ignoreUnassigned=TRUE,margin=0))
	expect_silent(plotClustersTable(cc,whichClusters=c(1,2),ignoreUnassigned=TRUE,margin=NA))
	expect_silent(plotClustersTable(tableClusters(cc,whichClusters=c(1,2))))

	
	#the following gives a wicked output which ignoreUnassigned=TRUE: zero overlap because some in one cluster are all -1 in makeConsensus so NaN value in proportion and only single cluster in makeConsensus
	expect_silent(cc<-clusterMany(mat, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(3,4),reduceMethod=c("none","PCA","var"),clusterFunction="pam",
	                       subsample=FALSE, sequential=FALSE,run=TRUE,verbose=FALSE,
	                       isCount=FALSE))
	expect_message(cc<-makeConsensus(cc,proportion=0.7),"no clusters specified to combine")
	expect_error(plotClustersTable(cc,whichClusters=c(1,2),ignoreUnassigned=TRUE) ,"Cannot create heatmap when there is only 1 column or row in the table")
	expect_silent(plotClustersTable(cc,whichClusters=c(1,2),ignoreUnassigned=TRUE,plotType="bubble")) #gives a 
	
	
	#test more complicated
	#so different numbers of clusters in the two clusters
	expect_silent(ceSim<-renameClusters(ceSim,whichCluster=1,val=letters[1:nClusters(ceSim)[1]]))
	expect_silent(ceSim<-subsetByCluster(ceSim,whichCluster=1,c("a","b","d")))
	expect_silent(plotClustersTable(ceSim,whichClusters=c(1,2),xlab="Cluster1",margin=2,legend=TRUE))
	expect_silent(plotClustersTable(ceSim,whichClusters=c(1,2),xlab=NULL,ylab=NA,margin=0,legend=TRUE))
	expect_silent(plotClustersTable(ceSim,whichClusters=c(1,2),xlab=NA,ylab=NULL,margin=1,legend=TRUE))
	expect_silent(plotClustersTable(ceSim,whichClusters=c(1,2),margin=NA,ylab="Cluster2",legend=TRUE ))
	expect_silent(plotClustersTable(ceSim,whichClusters=c(1,2),margin=NULL,legend=TRUE))
	
	expect_silent(plotClustersTable(ceSim,whichClusters=c(1,2),xlab="Cluster1",margin=2,plotType="bubble"))
	expect_silent(plotClustersTable(ceSim,whichClusters=c(1,2),xlab=NULL,ylab=NA,margin=0,plotType="bubble"))
	expect_silent(plotClustersTable(ceSim,whichClusters=c(1,2),xlab=NA,ylab=NULL,margin=1,plotType="bubble"))
	expect_silent(plotClustersTable(ceSim,whichClusters=c(1,2),margin=NA,ylab="Cluster2",plotType="bubble" ))
	expect_silent(plotClustersTable(ceSim,whichClusters=c(1,2),margin=NULL,plotType="bubble"))
	
	
})

test_that("plotFeatureScatter works",{
	expect_silent(plotFeatureScatter(object=cc,features=c(1,2),whichCluster=1,pch=19))
	expect_silent(plotFeatureScatter(object=cc,features=c(1,2,3),whichCluster=1,pch=19))
	expect_error(plotFeatureScatter(object=cc,features=c("Gene1","Gene4"),whichCluster=1),"not all of features match one")

	expect_silent(plotFeatureScatter(object=cc,features=c("Gene 1","Gene 4"),whichCluster=1,pch=19))
	expect_silent(plotFeatureScatter(object=cc,features=c("Gene 1","Gene 4","Gene 10"),whichCluster=1,pch=19))

	expect_silent(plotFeatureScatter(object=cc,features=c("Gene 1","Gene 4","Gene 10"),whichCluster=1,pch=19,plotUnassigned=FALSE))
	expect_silent(plotFeatureScatter(object=cc,features=c("Gene 1","Gene 4"),unassignedColor="black",whichCluster=1,pch=19,legend="topright"))

	cc2<-cc
	rownames(cc2)<-NULL
	plotFeatureScatter(object=cc2,features=c(1,2),whichCluster=1,pch=19)
	plotFeatureScatter(object=cc2,features=c(1,2,3),whichCluster=1,pch=19)

})

test_that("plotting works with hdf5 assays objects",{
	##plotClusters
    expect_silent(cl1 <- clusterSingle(hdfSCE, reduceMethod="PCA",
            subsample=FALSE, sequential=FALSE,
			mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=6)),
			isCount=FALSE))
	expect_silent(plotClusters(cl1))
	
	##plotBarplot
	expect_silent(plotBarplot(cl1))
	
	##plotReducedDims
	expect_silent(plotReducedDims(cl1,legend="bottomright"))

	##plotFeatureBoxplot
	expect_silent(plotFeatureBoxplot(object=cl1,feature=1))

	

})

##put separate so that not too long without output...
context("Heatmap functions")


plotAll<-FALSE #set to true to actually SEE the plots; otherwise for TravisCI, where no visual, runs quicker with FALSE
###Note some are still run with plot=TRUE to check that works with aheatmap. Only a fraction not plotted.
test_that("`setBreaks`", {
	setBreaks(smSimData)
	setBreaks(smSimData,breaks=0.99)
	x<-setBreaks(smSimData,breaks=0.99,makeSymmetric=TRUE)
	expect_equal(max(x),-min(x))
	expect_equal(x,setBreaks(smSimData,breaks=0.01,makeSymmetric=TRUE))
	expect_warning(y<-setBreaks(smSimData,breaks=10))
	expect_equal(length(y),10)
})
test_that("`plotHeatmap` works with matrix objects", {
	# these require that they actually be plotted...
    expect_silent(x1<-plotHeatmap(data=smSimData))
    a1<-NMF::aheatmap(smSimData)
    expect_equal(x1$aheatmapOut,a1)
    expect_silent(x2<-plotHeatmap(data=smSimCount,
        clusterSamplesData=smSimData,
        clusterFeaturesData=smSimData))
    #for some reason, labels on dendrogram move from character to numeric so can't test entire object...
    expect_equal(x1$aheatmapOut$rowInd,x2$aheatmapOut$rowInd) 
    expect_equal(x1$aheatmapOut$colInd,x2$aheatmapOut$colInd) 
    
    #check internal alignment of colData (alignColData=TRUE) is working:
    colData<-clusterMatrix(smSimCE)
    expect_silent(alList<-plotClusters(colData))
    alCol<-clusterExperiment:::.convertToAheatmap(alList$clusterLegend, names=FALSE)
   #these should be same plots:
    expect_silent(x1<-plotHeatmap(data=smSimData[,alList$orderSamples],
        colData=colData[alList$orderSamples,1:10],
        clusterLegend=alCol,clusterSamples=FALSE,
        clusterFeatures=FALSE,plot=plotAll))
    expect_silent(x2<-plotHeatmap(data=smSimData[,alList$orderSamples],
        colData=colData[alList$orderSamples,1:10],
        alignColData=TRUE,clusterFeatures=FALSE,
        clusterSamples=FALSE,plot=plotAll))
#   Should get this working so proper test, but more a problem because in different order, otherwise the same. Don't want to deal with this right now.
#    expect_equal(lapply(x1$clusterLegend,function(x){x[,c("clusterIds","color")]}),lapply(x2$clusterLegend,function(x){x[,c("clusterIds","color")]}))

    expect_error( plotHeatmap(data=smSimData,Rowv=TRUE, plot=plotAll),
        "arguments to aheatmap cannot be set by the user")
    expect_error( plotHeatmap(data=smSimData,Colv=TRUE, plot=plotAll),
        "arguments to aheatmap cannot be set by the user")
    expect_error( plotHeatmap(data=smSimData,
        colorScale=seqPal5,color=TRUE, plot=plotAll),
        "arguments to aheatmap cannot be set by the user")

    expect_error( plotHeatmap(data=smSimData,
        annCol=rnorm(n=ncol(smSimData)), plot=plotAll),
        "arguments to aheatmap cannot be set by the user")
    expect_error( plotHeatmap(data=smSimData,
        annColors=list(a=c("blue","green")), plot=plotAll),
        "arguments to aheatmap cannot be set by the user")
    
    ##Should add tests that pass aheatmap arguments correctly.
})


test_that("`plotHeatmap` works with hdf5 objects", {
	# These are slow!
	skip_on_os("windows")
	skip_on_os("mac")
	
    expect_silent(plotHeatmap(hdfObj))

})
test_that("`plotHeatmap` works with ClusterExperiment and SummarizedExperiment objects", {
  
  expect_silent(plotHeatmap(cc))
  expect_silent(plotHeatmap(cc,whichClusters="none", plot=plotAll))
  expect_warning(plotHeatmap(cc,whichClusters="workflow", plot=plotAll) ,
      "whichClusters value does not match any clusters") #there are no workflow for this one
  
  expect_warning(plotHeatmap(smSimCE,whichClusters="workflow", plot=plotAll,
      overRideClusterLimit=TRUE),
      "More than 10 annotations/clusterings can result in incomprehensible errors in aheamap")
  expect_warning(plotHeatmap(smSimCE,whichClusters=15:20, plot=plotAll),
      "given whichClusters value does not match any clusters")
  expect_error( plotHeatmap(smSimCE,whichClusters="all", plot=plotAll, 
      alignColData=TRUE, overRideClusterLimit=FALSE), 
      "More than 10 annotations/clusterings")
  expect_warning(plotHeatmap(smSimCE, plot=plotAll,
      whichClusters="all",
      alignColData=FALSE,overRideClusterLimit=TRUE))
  
  
  #test colData
  expect_error(plotHeatmap(cc,colData="A", plot=plotAll), "no colData for object data")
  
  expect_silent(plotHeatmap(smSimCE,colData="all", plot=plotAll))
  expect_silent(plotHeatmap(smSimCE,colData="A",plot=plotAll))
  expect_silent(plotHeatmap(smSimCE,colData=2:3,plot=plotAll))
  
  #check that it pulls the names, not the clusterIds.
  expect_silent(clusterLegend(cc)[[1]][,"name"]<-letters[1:nrow(clusterLegend(cc)[[1]])])
  expect_silent(plotHeatmap(cc, plot=plotAll))
  
  #check user setting clusterLegend
  x<-palette()[1:7]
  names(x)<-clusterLegend(cc)$Cluster1[,"name"]
  expect_silent(plotHeatmap(cc,
      clusterLegend=list("Cluster1"=x),plot=plotAll))
  
  expect_silent(plotHeatmap(cc, plot=plotAll,
      clusterLegend=list("Cluster1"=palette()[1:7])))
  expect_silent(plotHeatmap(smSimCE,
      colData="A",clusterLegend=list("A"=palette()[1:4]),
      plot=plotAll))
  
  ########################
  ########################
  # the following checks work outside of the test but  inside test_that, they hit errors
  # possibly issue with testthat? Not evaluating for now.
  ########################
  ########################
  #
  # plotHeatmap(smSimCE, colData="all", whichClusters="none")
  #
  # #this test doesn't work -- for some reason, expect_warning environment hits error that don't see at the consule.
  # plotHeatmap(smSimCE,whichClusters="all",alignColData=TRUE,overRideClusterLimit=TRUE)
  # expect_warning( plotHeatmap(smSimCE, whichClusters="all", alignColData=TRUE, overRideClusterLimit=TRUE)
  # , "More than 10 annotations/clusterings")
  #
  # # create some names to see if keeps names with alignColData=TRUE
  # # only can check manually, not with testthat.
  # # BUG!: doesn't work. looses their -1/-2 designation... haven't fixed yet.
  # clLeg<-clusterLegend(smSimCE)
  # clLeg[[1]][,"name"]<-LETTERS[1:nrow(clLeg[[1]])]
  # clusterLegend(smSimCE)<-clLeg
  # plotHeatmap(smSimCE, whichClusters="all", alignColData=TRUE,overRideClusterLimit=TRUE)
  #
})

test_that("`plotHeatmap` visualization choices/feature choices all work", {

    expect_silent(plotHeatmap(smSimCE,
        visualizeData=smSimCount,plot=plotAll))
    expect_silent(plotHeatmap(smSimCE,
        visualizeData="transformed",plot=plotAll))
    expect_silent(plotHeatmap(smSimCE,
        visualizeData="original",plot=plotAll))
    expect_silent(plotHeatmap(smSimCE,
        visualizeData="centeredAndScaled",plot=plotAll))
    # even if visualizeData="orginal, still clsuter on transformed. Should make unit test out of below that get same:
    expect_silent(plotHeatmap(smSimCE, 
      visualizeData="transformed", clusterSamplesData="hclust", 
    plot=plotAll))
    orderSamples(smSimCE)<-sample(1:nSamples(smSimCE))
    expect_silent(plotHeatmap(smSimCE, 
      visualizeData="transformed",
      clusterSamplesData="orderSamplesValue", plot=plotAll))
    expect_silent(plotHeatmap(smSimCE, 
      visualizeData="transformed", 
      clusterSamplesData="primaryCluster", plot=plotAll))
    expect_silent(plotHeatmap(smSimCE,
      visualizeData="transformed", 
      clusterSamplesData=c(3,4,5), plot=plotAll))
    expect_silent(plotHeatmap(smSimCE,
      visualizeData="transform", 
      clusterFeaturesData="all", plot=plotAll))
  
    ###The following subset the genes
    expect_silent(plotHeatmap(smSimCE,
      visualizeData="transform", 
      clusterFeaturesData="var", nFeatures=3, plot=plotAll))
    expect_silent(plotHeatmap(smSimCE,
      visualizeData="transform", 
      clusterFeaturesData=3:5, nFeatures=3, plot=plotAll))
    ## expect error because no row names in smSimCE
    expect_error(plotHeatmap(smSimCE, 
      visualizeData="transform", 
      clusterFeaturesData=paste("Gene",3:5), nFeatures=3), 
      "Cannot give feature names in clusterFeaturesData unless")
    ##set rownames and should work
    smSimCE2<-smSimCE
    row.names(smSimCE2)<-paste("Gene",1:NROW(smSimCE))
    expect_silent(plotHeatmap(smSimCE2, plot=plotAll, 
      visualizeData="transform", 
      clusterFeaturesData=paste("Gene",3:5), nFeatures=3))

    expect_silent(plotHeatmap(smSimCE, 
      visualizeData="transform", 
      clusterFeaturesData="PCA", nFeatures=10, 
      clusterSamplesData="hclust", plot=plotAll))
    expect_silent(plotHeatmap(smSimCE, 
     visualizeData="transform", 
     clusterSamplesData="dendrogramValue", plot=plotAll))
    #test works with outside dataset
    expect_silent(plotHeatmap(smSimCE,
     visualizeData=assay(smSimCE)[1:10,],plot=plotAll))
    expect_error(plotHeatmap(smSimCE, 
     visualizeData=assay(smSimCE)[,1:5]))
})

test_that("`makeBlankData` works", {
    ##call directly, features
    gps<-list(c(3,6,7),c(2,1))
    expect_silent(xx<-makeBlankData(assay(smSimCE),
      groupsOfFeatures=gps))
    expect_equal(nrow(xx$dataWBlanks),length(xx$rowNamesWBlanks))
    whBlankNames<-which(xx$rowNamesWBlanks=="")
    expect_equal(xx$rowNamesWBlanks[-whBlankNames],
      as.character(unlist(gps)) )
    whBlankRows<-as.numeric(which(apply(xx$dataWBlanks,1,
      function(x){all(is.na(x))})))
    expect_equal(whBlankRows,whBlankNames)
    expect_equal(whBlankRows,4)

    ##call directly, samples
    gps<-list(c(3,6,7),c(2,1))
    expect_silent(xy<-makeBlankData(assay(smSimCE),groupsOfSamples=gps))
    expect_equal(ncol(xy$dataWBlanks),length(xy$colNamesWBlanks))
    whBlankNames<-which(xy$colNamesWBlanks=="")
    expect_equal(xy$colNamesWBlanks[-whBlankNames],
      as.character(unlist(gps)) )
    whBlankCols<-as.numeric(which(apply(xy$dataWBlanks,2,
      function(x){all(is.na(x))})))
    expect_equal(whBlankCols,whBlankNames)
    expect_equal(whBlankCols,4)


    ##call directly, features and samples
    gpsR<-list(c(3,6,7),c(2,1))
    gpsC<-list(c(8,1,3),c(9,10))
    expect_silent(yy<-makeBlankData(assay(smSimCE),
      groupsOfFeatures=gpsR,groupsOfSamples=gpsC))
    expect_equal(ncol(yy$dataWBlanks),length(yy$colNamesWBlanks))
    expect_equal(nrow(yy$dataWBlanks),length(yy$rowNamesWBlanks))
    whBlankNames<-which(yy$colNamesWBlanks=="")
    expect_equal(yy$colNamesWBlanks[-whBlankNames],
      as.character(unlist(gpsC)) )
    whBlankCols<-as.numeric(which(apply(yy$dataWBlanks,2,
      function(x){all(is.na(x))})))
    expect_equal(whBlankCols,whBlankNames)
    expect_equal(whBlankCols,4)
  
    whBlankNames<-which(yy$rowNamesWBlanks=="")
    expect_equal(yy$rowNamesWBlanks[-whBlankNames],
      as.character(unlist(gpsR)) )
    whBlankRows<-as.numeric(which(apply(yy$dataWBlanks,1,
        function(x){all(is.na(x))})))
    expect_equal(whBlankRows,whBlankNames)
    expect_equal(whBlankRows,4)


    ##call within plotHeatmap (serves as test of NA capabilities)
    expect_silent(plotHeatmap(smSimCE,clusterFeaturesData=gps))
    expect_warning(plotHeatmap(smSimCE,clusterFeaturesData=gps,breaks=40))
    expect_silent(plotHeatmap(smSimCE,clusterFeaturesData=gps,breaks=.99))
})

test_that("`plotCoClustering` works", {
    expect_error(plotCoClustering(smSimCE),"coClustering slot is empty")
    #following gives all -1, but creates coClustering
    expect_silent(smMin1<-
        makeConsensus(smSimCE,whichClusters=10:13,proportion=.99))
    #  smMin1<-makeConsensus(smSimCE,whichClusters=1:8,proportion=.95) #use to give all -1, but creates coClustering but something changed -- couldn't figure it out!!!
    expect_silent(plotCoClustering(smMin1,clusterSamplesData="hclust"))
    ## Have changed so now changes it internally to primary cluster then hclust
    expect_warning( plotCoClustering(smMin1, 
        clusterSamplesData="dendrogramValue", plot=plotAll), 
        "cannot make dendrogram from 'data'")
    expect_silent(sm<-makeConsensus(smSimCE,whichClusters=1:4,proportion=.5))
    expect_silent(plotCoClustering(sm,clusterSamplesData="dendrogramValue"))

    # ## Test on object that has a merge done on it
    # ## FIXME:
    # ## This has surprising error and warning due to phylo conversion
    # ##    -- need to go back to it
    # ## (also error on release version!)
    # expect_silent(clustNothing <- clusterMany(mat,
    #     ks=c(3,4),clusterFunction="pam",
    #     subsample=FALSE, sequential=FALSE,
    #     isCount=FALSE,verbose=FALSE))
    # expect_silent(clustNothing<-makeConsensus(clustNothing,
    #     proportion=1,minSize=1,whichClusters = "clusterMany"))
    # expect_silent(clustNothing <- makeDendrogram(clustNothing))
    # expect_message(clustNothing<- mergeClusters(clustNothing,
    #     DEMethod="limma",
    #     mergeMethod="adjP",plotInfo="none"),
    #     "Note: Merging will be done on")
    # expect_silent(plotCoClustering(clustNothing))
})

#' @name transformData
#' @title Transform the original data in a ClusterExperiment object
#' @aliases transformData,matrixOrHDF5-method
#' @description Provides the transformed data
#' @param object a matrix, SummarizedExperiment, SingleCellExperiment or
#'   ClusterExperiment object.
#' @param transFun a transformation function to be applied to the data. If the
#'   transformation applied to the data creates an error or NA values, then the
#'   function will throw an error. If object is of class
#'   \code{ClusterExperiment}, the stored transformation will be used and giving
#'   this parameter will result in an error.
#' @param isCount if \code{transFun=NULL}, then \code{isCount=TRUE} will
#'   determine the transformation as defined by \code{function(x){log2(x+1)}},
#'   and \code{isCount=FALSE} will give a transformation function
#'   \code{function(x){x}}. Ignored if \code{transFun=NULL}. If object is of
#'   class \code{ClusterExperiment}, the stored transformation will be used and
#'   giving this parameter will result in an error.
#' @param ... Values passed on the the 'matrix' method.
#' @return A DataFrame defined by \code{assay(x)} suitably transformed
#' @details The data matrix defined by \code{assay(x)} is transformed based on
#'   the transformation function either defined in x (in the case of a
#' \code{ClusterExperiment} object) or by user given values for other classes.
#'
#'
#' @examples
#' mat <- matrix(data=rnorm(200), ncol=10)
#' mat[1,1] <- -1 #force a negative value
#' labels <- gl(5, 2)
#' cc <- ClusterExperiment(mat, as.numeric(labels), transformation =
#' function(x){x^2}) #define transformation as x^2
#' z<-transformData(cc)
#' @export
setMethod(
  f = "transformData",
  signature = "matrixOrHDF5",
  definition = function(object,transFun=NULL,isCount=FALSE) {
	  transFun<-.makeTransFun(transFun=transFun,isCount=isCount)
	  x <- try(transFun(object), silent=TRUE)
	  if(inherits(x, "try-error"))
	    stop("User-supplied `transFun` produces the following error on the input data matrix:\n",x)
	  if(anyNA(x))
	    stop("User-supplied `transFun` produces NA values")
	  return(x)
  }
)
#' @export
#' @param whichAssay numeric or character specifying which assay to use. See
#'   \code{\link[SummarizedExperiment]{assay}} for details.
#' @rdname transformData
setMethod(
  f = "transformData",
  signature = "ClusterExperiment",
  definition = function(object, whichAssay=1, ...) {
  	if(any(c("transFun","isCount") %in% names(list(...))))
  		stop("The internally saved transformation function of a ClusterExperiment object must be used when given as input and setting 'transFun' or 'isCount' for a 'ClusterExperiment' is not allowed.")
	  return(transformData(assay(object, whichAssay),transFun=transformation(object)))
  }
)
#' @export
#' @rdname transformData
setMethod(
  f = "transformData",
  signature = "SingleCellExperiment",
  definition = function(object, whichAssay=1, ...) {
	  return(transformData(assay(object, whichAssay),...))
  }
)
#' @export
#' @rdname transformData
setMethod(
  f = "transformData",
  signature = "SummarizedExperiment",
  definition = function(object,...) {
	  return(transformData(as(object,"SingleCellExperiment"),...))
  }
)

#small function to uniformally return transformation function from combination of transFun and isCount
.makeTransFun<-function(transFun=NULL,isCount=FALSE){
  if(is.null(transFun)){
    transFun <- if(isCount) function(x){log2(x+1)} else function(x){x}
  }
  return(transFun)
}

## ----GlobalOptions, results="hide", include=FALSE, cache=FALSE----------------
knitr::opts_chunk$set(fig.align="center", cache=FALSE, cache.path = "clusterExperimentTutorial_cache/",  fig.path="clusterExperimentTutorial_figure/",error=FALSE, #make it stop on error
fig.width=6,fig.height=6,autodep=TRUE,out.width="600px",out.height="600px",
message=FALSE)
#knitr::opts_knit$set(stop_on_error = 2L) #really make it stop
#knitr::dep_auto()

## ----cache=FALSE--------------------------------------------------------------
set.seed(14456) ## for reproducibility, just in case
library(clusterExperiment)
data(fluidigmData) ## list of the two datasets (tophat_counts and rsem_tpm)
data(fluidigmColData)
se<-SummarizedExperiment(fluidigmData,colData=fluidigmColData)

## -----------------------------------------------------------------------------
assay(se)[1:5,1:10]
colData(se)[,1:5]
NCOL(se) #number of samples

## ----filter-------------------------------------------------------------------
wh_zero <- which(rowSums(assay(se))==0)
pass_filter <- apply(assay(se), 1, function(x) length(x[x >= 10]) >= 10)
se <- se[pass_filter,]
dim(se)

## ----normalization------------------------------------------------------------
#provides matrix of normalized counts
fq <- round(limma::normalizeQuantiles(assay(se)))

## ----addToAssays--------------------------------------------------------------
assays(se) <- c(SimpleList(normalized_counts=fq),assays(se))
assays(se)

## ----colnames-----------------------------------------------------------------
wh<-which(colnames(colData(se)) %in% c("Cluster1","Cluster2"))
colnames(colData(se))[wh]<-c("Published1","Published2")

## ----RSECLoad-----------------------------------------------------------------
data("rsecFluidigm")
# package version the object was created under
metadata(rsecFluidigm)$packageVersion

## ----include=FALSE------------------------------------------------------------
# #check still valid object, in case updated class definitions
# if(!validObject(rsecFluidigm)) stop("rsecFluidigm is not a valid ClusterExperiment object")
# clusterExperiment:::checkRsecFluidigmObject(rsecFluidigm)

## ----RSEC, eval=FALSE---------------------------------------------------------
# ## Example of RSEC call (not run by vignette)
# ## Will overwrite the rsecFluidigm brought in above.
# library(clusterExperiment)
# system.time(rsecFluidigm<-RSEC(se, isCount = TRUE,
#     reduceMethod="PCA", nReducedDims=10,
#     k0s = 4:15,
#     alphas=c(0.1, 0.2, 0.3),
#     ncores=1, random.seed=176201))

## ----RSECprint----------------------------------------------------------------
rsecFluidigm

## ----primaryCluster-----------------------------------------------------------
head(primaryCluster(rsecFluidigm),20)
tableClusters(rsecFluidigm)

## ----clusterMatrix------------------------------------------------------------
head(clusterMatrix(rsecFluidigm)[,1:4])

## ----clusterLabels------------------------------------------------------------
head(clusterLabels(rsecFluidigm))

## ----clusterParams------------------------------------------------------------
head(getClusterManyParams(rsecFluidigm))

## ----plotClusterRSEC----------------------------------------------------------
defaultMar<-par("mar")
plotCMar<-c(1.1,8.1,4.1,1.1)
par(mar=plotCMar)
plotClusters(rsecFluidigm,main="Clusters from RSEC", whichClusters="workflow", colData=c("Biological_Condition","Published2"), axisLine=-1)

## ----plotClustersWorkflow-----------------------------------------------------
par(mar=plotCMar)
plotClustersWorkflow(rsecFluidigm)

## ----plotBarplotRSEC----------------------------------------------------------
plotBarplot(rsecFluidigm,main=paste("Distribution of samples of",primaryClusterLabel(rsecFluidigm)))

## ----plotBarplotRSEC.2--------------------------------------------------------
plotBarplot(rsecFluidigm,whichClusters=c("makeConsensus" ))

## ----plotBarplot2-------------------------------------------------------------
plotBarplot(rsecFluidigm,whichClusters=c("mergeClusters" ,"makeConsensus"))

## ----plotClustersTable--------------------------------------------------------
plotClustersTable(rsecFluidigm,whichClusters=c("mergeClusters" ,"makeConsensus"), margin=1)

## ----plotClustersTableBubble--------------------------------------------------
plotClustersTable(rsecFluidigm,whichClusters=c("mergeClusters" ,"makeConsensus"), margin=1,plotType="bubble")

## ----plotCoClustering_rsec----------------------------------------------------
plotCoClustering(rsecFluidigm,whichClusters=c("mergeClusters","makeConsensus"))

## ----plotDendroRSEC-----------------------------------------------------------
plotDMar<-c(8.1,1.1,5.1,8.1)
par(mar=plotDMar)
plotDendrogram(rsecFluidigm,whichClusters=c("makeConsensus","mergeClusters"))

## ----plotReducedDims----------------------------------------------------------
plotReducedDims(rsecFluidigm)

## ----plotReducedDimsMany------------------------------------------------------
plotReducedDims(rsecFluidigm,whichDims=c(1:4))

## ----recallRSEC---------------------------------------------------------------
rsecFluidigm<-RSEC(rsecFluidigm,isCount=TRUE,consensusProportion=0.6,mergeMethod="JC",mergeCutoff=0.05,stopOnErrors =TRUE)

## ----plotClusterRSECRecall----------------------------------------------------
defaultMar<-par("mar")
plotCMar<-c(1.1,8.1,4.1,1.1)
par(mar=plotCMar)
plotClusters(rsecFluidigm,main="Clusters from RSEC", whichClusters=c("mergeClusters.1","makeConsensus.1","mergeClusters","makeConsensus"), colData=c("Biological_Condition","Published2"), axisLine=-1)

## ----plotClusterRSECReset-----------------------------------------------------
data(rsecFluidigm)

## ----getBestFeatures----------------------------------------------------------
pairsAllRSEC<-getBestFeatures(rsecFluidigm,contrastType="Pairs",p.value=0.05,
                          number=nrow(rsecFluidigm),DEMethod="edgeR")
head(pairsAllRSEC)

## ----getBestFeatures_size-----------------------------------------------------
# We have duplicates of some genes:
any(duplicated(pairsAllRSEC$Feature))

## ----getBestFeatures_heatmap--------------------------------------------------
plotHeatmap(rsecFluidigm, whichClusters=c("makeConsensus","mergeClusters"),clusterSamplesData="dendrogramValue",
            clusterFeaturesData=unique(pairsAllRSEC[,"IndexInOriginal"]),
            main="Heatmap of features w/ significant pairwise differences",
            breaks=.99)

## ----plotContrastHeatmap------------------------------------------------------
plotContrastHeatmap(rsecFluidigm, signif=pairsAllRSEC,nBlankLines=40,whichCluster="primary")

## ----clusterMany--------------------------------------------------------------
ce<-clusterMany(se, clusterFunction="pam",ks=5:10, minSizes=5,
      isCount=TRUE,reduceMethod=c("PCA","var"),nFilterDims=c(100,500,1000),
      nReducedDims=c(5,15,50),run=TRUE)

## ----plotClusterEx1-----------------------------------------------------------
defaultMar<-par("mar")
par(mar=plotCMar)
plotClusters(ce,main="Clusters from clusterMany", whichClusters="workflow", colData=c("Biological_Condition","Published2"), axisLine=-1)

## ----plotCluster_params-------------------------------------------------------
cmParams<-getClusterManyParams(ce)
head(cmParams)

## ----plotCluster_newOrder-----------------------------------------------------
ord<-order(cmParams[,"reduceMethod"],cmParams[,"nReducedDims"])
ind<-cmParams[ord,"clusteringIndex"]
par(mar=plotCMar)
plotClusters(ce,main="Clusters from clusterMany", whichClusters=ind, colData=c("Biological_Condition","Published2"), axisLine=-1)

## ----plotCluster_newLabels----------------------------------------------------
clOrig<-clusterLabels(ce)
clOrig<-gsub("Features","",clOrig)
par(mar=plotCMar)
plotClusters(ce,main="Clusters from clusterMany", whichClusters=ind, clusterLabels=clOrig[ind], colData=c("Biological_Condition","Published2"), axisLine=-1)

## -----------------------------------------------------------------------------
ce<-makeConsensus(ce,proportion=1)

## ----lookAtCombineMany--------------------------------------------------------
head(clusterMatrix(ce)[,1:3])
par(mar=plotCMar)
plotClustersWorkflow(ce)


## ----makeConsensus_changeLabel------------------------------------------------
wh<-which(clusterLabels(ce)=="makeConsensus")
if(length(wh)!=1) stop() else clusterLabels(ce)[wh]<-"makeConsensus,1"

## ----makeConsensus_newCombineMany---------------------------------------------
ce<-makeConsensus(ce,proportion=0.7,clusterLabel="makeConsensus,0.7")
par(mar=plotCMar)
plotClustersWorkflow(ce)


## ----makeConsensus_changeMinSize----------------------------------------------
ce<-makeConsensus(ce,proportion=0.7,minSize=3,clusterLabel="makeConsensus,final")
par(mar=plotCMar)
plotClustersWorkflow(ce,whichClusters=c("makeConsensus,final","makeConsensus,0.7","makeConsensus,1"),main="Min. Size=3")

## ----plotCoClustering_quickstart----------------------------------------------
plotCoClustering(ce)

## ----makeDendrogram-----------------------------------------------------------
ce<-makeDendrogram(ce,reduceMethod="var",nDims=500)
plotDendrogram(ce)

## ----makeDendrogram_show------------------------------------------------------
ce

## ----mergeClustersPlot--------------------------------------------------------
mergeClusters(ce,mergeMethod="adjP",DEMethod="edgeR",plotInfo="mergeMethod")

## ----mergeClusters------------------------------------------------------------
ce<-mergeClusters(ce,mergeMethod="adjP",DEMethod="edgeR",cutoff=0.05)

## ----mergeClusters_coClustering-----------------------------------------------
par(mar=plotCMar)
plotClustersWorkflow(ce,whichClusters="workflow") 
plotCoClustering(ce,whichClusters=c("mergeClusters","makeConsensus"),
                 colData=c("Biological_Condition","Published2"),annLegend=FALSE)

## ----plotHeatmap--------------------------------------------------------------
plotHeatmap(ce,clusterSamplesData="dendrogramValue",breaks=.99,
            colData=c("Biological_Condition", "Published1", "Published2"))

## ----quasiRsecCode, eval=FALSE------------------------------------------------
# rsecOut<-RSEC(se, isCount=TRUE, reduceMethod="PCA", nReducedDims=c(50,10), k0s=4:15,
#         alphas=c(0.1,0.2,0.3),betas=c(0.8,0.9), minSizes=c(1,5), clusterFunction="hierarchical01",
#         consensusProportion=0.7, consensusMinSize=5,
#         dendroReduce="mad",dendroNDims=500,
#         mergeMethod="adjP",mergeCutoff=0.05,
# 		)

## ----stepCode,eval=FALSE------------------------------------------------------
# ce<-clusterMany(se,ks=4:15,alphas=c(0.1,0.2,0.3),betas=c(0.8,0.9),minSizes=c(1,5),
# 	clusterFunction="hierarchical01", sequential=TRUE,subsample=TRUE,
#          reduceMethod="PCA",nFilterDims=c(50,10),isCount=TRUE)
# ce<-makeConsensus(ce, proportion=0.7, minSize=5)
# ce<-makeDendrogram(ce,reduceMethod="mad",nDims=500)
# ce<-mergeClusters(ce,mergeMethod="adjP",DEMethod="edgeR",cutoff=0.05,plot=FALSE)

## ----show---------------------------------------------------------------------
ce

## ----CEHelperCommands1--------------------------------------------------------
head(clusterMatrix(ce))[,1:5]
primaryCluster(ce)

## ----CEHelperCommands2--------------------------------------------------------
head(clusterLabels(ce),10)

## ----CEHelperCommands3--------------------------------------------------------
head(clusterTypes(ce),10)

## ----SECommandsOnCE-----------------------------------------------------------
colData(ce)[,1:5]

## ----CEClusterLengend---------------------------------------------------------
length(clusterLegend(ce))
clusterLegend(ce)[1:2]

## ----CEClusterLengendAssign---------------------------------------------------
newName<-gsub("m","M",clusterLegend(ce)[[1]][,"name"])
renameClusters(ce,whichCluster=1,value=newName)
print(ce)

## ----basicSubsetting----------------------------------------------------------
smallCe<-ce[1:5,1:10]
smallCe


## ----matrixAfterSubset--------------------------------------------------------
clusterMatrix(smallCe)[,1:4]
clusterMatrix(ce)[1:10,1:4]

## ----lookAtSubset-------------------------------------------------------------
clusterLegend(smallCe)[["mergeClusters"]]
clusterLegend(ce)[["mergeClusters"]]

## ----afterSubsetting----------------------------------------------------------
nodeMergeInfo(ce)
nodeMergeInfo(smallCe)

## ----subsampleByCluster-------------------------------------------------------
subCe<-subsetByCluster(ce,whichCluster="mergeClusters",clusterValue=c("m1","m2"))
subCe

## ----assignNeg----------------------------------------------------------------
ce<-assignUnassigned(ce,whichCluster="mergeClusters",reduceMethod="PCA",nDim=50,makePrimary=FALSE, filterIgnoresUnassigned=TRUE,clusterLabel="mergeClusters_AssignToCluster")
tableClusters(ce,whichCluster="mergeClusters_AssignToCluster")

## ----assignNeg_plotClusters---------------------------------------------------
plotClusters(ce,whichCluster=c("mergeClusters_AssignToCluster","mergeClusters"))

## ----buildInDimReduce---------------------------------------------------------
listBuiltInReducedDims()
listBuiltInFilterStats()

## ----plotClusterEx1_redo------------------------------------------------------
par(mar=plotCMar)
plotClusters(ce,main="Clusters from clusterMany", whichClusters="workflow", 
             axisLine=-1)

## ----plotClusterEx1_newOrder--------------------------------------------------
par(mar=plotCMar)
plotClusters(ce,whichClusters="clusterMany",
               main="Only Clusters from clusterMany",axisLine=-1)

## ----plotClusterEx1_addData---------------------------------------------------
par(mar=plotCMar)
plotClusters(ce,whichClusters="workflow", colData=c("Biological_Condition","Published2"), 
               main="Workflow clusters plus other data",axisLine=-1)

## ----plotClusterEx1_origColors------------------------------------------------
clusterLegend(ce)[c("mergeClusters","makeConsensus,final")]

## ----plotClusterLegend_before,out.width="300px",out.height="300px"------------
plotClusterLegend(ce,whichCluster="makeConsensus,final")

## ----plotClusterEx1_setColors-------------------------------------------------
ce_temp<-plotClusters(ce,whichClusters="workflow", colData=c("Biological_Condition","Published2"), 
               main="Cluster Alignment of Workflow Clusters",clusterLabels=FALSE, axisLine=-1, resetNames=TRUE,resetColors=TRUE,resetOrderSamples=TRUE)

## ----plotClusterEx1_newColors-------------------------------------------------
clusterLegend(ce_temp)[c("mergeClusters","makeConsensus,final")]

## ----plotClusterEx1_clusterLegend---------------------------------------------
ce_temp<-ce
clusterLegend(ce_temp)[["mergeClusters"]]

## ----plotClusterEx1_assignColors----------------------------------------------
newColors<-c("white","blue","green","cyan","purple")
newNames<-c("Not assigned","Cluster1","Cluster2","Cluster3","Cluster4")
#note we make sure they are assigned to the right cluster Ids by giving the 
#clusterIds as names to the new vectors
names(newColors)<-names(newNames)<-clusterLegend(ce_temp)[["mergeClusters"]][,"name"]
renameClusters(ce_temp,whichCluster="mergeClusters",value=newNames)
recolorClusters(ce_temp,whichCluster="mergeClusters",value=newColors)

## ----plotClusterLegend,out.width="300px",out.height="300px"-------------------
plotClusterLegend(ce_temp,whichCluster="mergeClusters")

## ----plotClusterEx1_firstOnlyNoSave-------------------------------------------
plotClusters(ce_temp,whichClusters="workflow", colData=c("Biological_Condition","Published2"), clusterLabels=FALSE,
               main="Clusters from clusterMany, different order",axisLine=-1,existingColors="firstOnly")

## ----plotClusterEx1_firstOnly-------------------------------------------------
ce_temp<-plotClusters(ce_temp,whichClusters="workflow", colData=c("Biological_Condition","Published2"), resetColors=TRUE,
               main="Clusters from clusterMany, different order",axisLine=-1, clusterLabels=FALSE, existingColors="firstOnly",plot=FALSE)

## ----plotClusterEx1_forceColors,fig.width=18,fig.height=9---------------------
par(mfrow=c(1,2))
plotClusters(ce_temp, colData=c("Biological_Condition","Published2"),
             whichClusters="workflow", existingColors="all", clusterLabels=FALSE,
             main="All Workflow Clusters, use existing colors",axisLine=-1)

plotClusters(ce_temp, colData=c("Biological_Condition","Published2"), 
             existingColors="all", whichClusters="clusterMany", clusterLabels=FALSE,
             main="clusterMany Clusters, use existing colors",
             axisLine=-1)

## ----plotClusterEx1_compareForceColors,fig.width=18,fig.height=18-------------
par(mfrow=c(2,2))
plotClusters(ce_temp, colData=c("Biological_Condition","Published2"), 
             existingColors="all", whichClusters="clusterMany", clusterLabels=FALSE,
             main="clusterMany Clusters, use existing colors",
             axisLine=-1)
plotClusters(ce_temp, colData=c("Biological_Condition","Published2"), 
               existingColors="firstOnly", whichClusters="clusterMany", clusterLabels=FALSE,
               main="clusterMany Clusters, use existing of first row only",
               axisLine=-1)
plotClusters(ce_temp, colData=c("Biological_Condition","Published2"), 
            existingColors="ignore", whichClusters="clusterMany", clusterLabels=FALSE,
            main="clusterMany Clusters, default\n(ignoring assigned colors)",
            axisLine=-1)

## ----showPalette--------------------------------------------------------------
showPalette()

## ----showPaletteOptions-------------------------------------------------------
showPalette(which=1:10)
showPalette(palette())

## ----showPaletteMassive-------------------------------------------------------
showPalette(massivePalette,cex=0.5)

## ----removeBlack--------------------------------------------------------------
plotClusters(ce,whichClusters="workflow", colData=c("Biological_Condition","Published2"), unassignedColor="black",colPalette=bigPalette[-grep("black",bigPalette)],
               main="Setting unassigned color",axisLine=-1)


## ----plotWorkflow-------------------------------------------------------------
plotClustersWorkflow(ce, axisLine= -1)

## ----plotWorkflow_resort------------------------------------------------------
par(mar=plotCMar)
plotClustersWorkflow(ce,whichClusters=c("makeConsensus,final","makeConsensus,0.7","makeConsensus,1"),main="Different choices of proportion in makeConsensus",sortBy="clusterMany",highlightOnTop=FALSE, axisLine= -1)

## ----plotHeatmap_Ex1----------------------------------------------------------
par(mfrow=c(1,1))
par(mar=defaultMar)
plotHeatmap(ce,main="Heatmap with clusterMany")

## ----plotHeatmap_Ex1.1--------------------------------------------------------
whClusterPlot<-1:2
plotHeatmap(ce,whichClusters=whClusterPlot, annLegend=FALSE)

## ----plotHeatmap_primaryCluster-----------------------------------------------
plotHeatmap(ce,clusterSamplesData="primaryCluster",
            whichClusters="primaryCluster",
            main="Heatmap with clusterMany",annLegend=FALSE)

## ----showCE_dendrogram--------------------------------------------------------
show(ce)

## ----plotHeatmap_dendro-------------------------------------------------------
plotHeatmap(ce,clusterSamplesData="dendrogramValue",
            whichClusters=c("mergeClusters","makeConsensus"),
            main="Heatmap with clusterMany",
            colData=c("Biological_Condition","Published2"),annLegend=FALSE)

## ----plotHeatmap_break99------------------------------------------------------
plotHeatmap(ce,clusterSamplesData="primaryCluster",
            whichClusters="primaryCluster", breaks=0.99,
            main="Heatmap with clusterMany, breaks=0.99",annLegend=FALSE)

## ----plotHeatmap_break95------------------------------------------------------
plotHeatmap(ce,clusterSamplesData="primaryCluster",
            whichClusters="primaryCluster", breaks=0.95,
            main="Heatmap with clusterMany, breaks=0.95",annLegend=FALSE)

## ----plotDendro_redoBasic-----------------------------------------------------
plotDendrogram(ce)

## ----plotDendro_Types---------------------------------------------------------
par(mfrow=c(1,2))
plotDendrogram(ce,leafType="clusters",plotType="colorblock")
plotDendrogram(ce,leafType="clusters",plotType="name")

## ----plotDendro_MultiClusters-------------------------------------------------
par(mar=plotDMar)
plotDendrogram(ce,whichClusters=c("makeConsensus","mergeClusters"))

## ----plotDendro_Merge---------------------------------------------------------
par(mar=plotDMar)
plotDendrogram(ce,whichClusters=c("makeConsensus","mergeClusters"),merge="mergeMethod")

## ----plotDendro_colData-------------------------------------------------------
plotDendrogram(ce,whichClusters=c("makeConsensus","mergeClusters"),colData=c("Biological_Condition","Published1","Published2"),merge="mergeMethod")

## ----plotDendro_NodeNames-----------------------------------------------------
par(mar=plotDMar)
plotDendrogram(ce,whichClusters=c("makeConsensus","mergeClusters"),show.node.label=TRUE)

## ----plotDendro_NodeColors----------------------------------------------------
nodeNames<-paste("NodeId",1:6,sep="")
nodeColors<-massivePalette[1:length(nodeNames)]
names(nodeColors)<-nodeNames
par(mar=plotDMar)
plotDendrogram(ce,whichClusters=c("makeConsensus","mergeClusters"),merge="mergeMethod",nodeColors=nodeColors)

## ----builtInFunctions---------------------------------------------------------
listBuiltInFunctions()

## ----getInputType-------------------------------------------------------------
inputType(c("kmeans","pam","hierarchicalK"))

## ----getAlgorithmType---------------------------------------------------------
algorithmType(c("kmeans","hierarchicalK","hierarchical01"))

## ----builtInKFunctions--------------------------------------------------------
listBuiltInTypeK()

## ----builtIn01Functions-------------------------------------------------------
listBuiltInType01()

## ----requiredArgs-------------------------------------------------------------
requiredArgs(c("hierarchical01","hierarchicalK"))

## ----mainClusterArgsSyntax,eval=FALSE-----------------------------------------
# clusterMany(x,clusterFunction="hierarchicalK", ... , mainClusterArgs=list(clusterArgs=list(method="single") ))

## ----subsampleArgsSyntax,eval=FALSE-------------------------------------------
# clusterMany(x,..., subsampleArgs=list(resamp.num=100,samp.p=0.5,clusterFunction="hiearchicalK", clusterArgs=list(method="single") ))

## ----seqArgsSyntax,eval=FALSE-------------------------------------------------
# clusterMany(x,..., seqArgs=list( remain.n=10))

## ----tableArguments, echo=FALSE, message=FALSE, warnings=FALSE, results='asis'----
# simple table creation here
tabl <- "  
Argument| Dependencies |  Passed to | Argument passed to
---------------|-----------------|:-------------:|------:|
ks             | sequential=TRUE |  seqCluster   |  k0 
-    | sequential=FALSE, findBestK=FALSE, clusterFunction of type 'K' | mainClustering | k
-              | sequential=FALSE, findBestK=FALSE, subsample=TRUE | subsampleClustering | k
-               | sequential=FALSE, findBestK=TRUE, clusterFunction of type 'K' | mainClustering | kRange
reduceMethod      | none            | transform     | reduceMethod
nFilterDims       | reduceMethod in 'mad','cv','var' | transform | nFilterDims
nReducedDims       | reduceMethod='PCA' | transform     | nReducedDims
clusterFunction| none            | mainClustering      | clusterFunction    
minSizes       | none            | mainClustering      | minSize
distFunction   | subsample=FALSE | mainClustering      | distFunction
alphas         | clusterFunction of type '01'| mainClustering | alpha
findBestK      | clusterFunction of type 'K' | mainClustering | findBestK
removeSil      | clusterFunction of type 'K' | mainClustering | removeSil
silCutoff      | clusterFunction of type 'K' | mainClustering | silCutoff
betas          | sequential=TRUE | seqCluster    | beta
"
cat(tabl) # output the table in a format good for HTML/PDF/docx conversion

## ----defineDist---------------------------------------------------------------
corDist<-function(x){(1-cor(t(x),method="pearson"))/2}
spearDist<-function(x){(1-cor(t(x),method="spearman"))/2}

## ----visualizeSubsamplingD----------------------------------------------------
ceSub<-clusterSingle(ce,reduceMethod="mad",nDims=1000,subsample=TRUE,subsampleArgs=list(clusterFunction="pam",clusterArgs=list(k=8)),clusterLabel="subsamplingCluster",mainClusterArgs=list(clusterFunction="hierarchical01",clusterArgs=list(alpha=0.1),minSize=5), saveSubsamplingMatrix=TRUE)
plotCoClustering(ceSub)

## ----visualizeSpearmanDist----------------------------------------------------
dSp<-spearDist(t(transformData(ce,reduceMethod="mad",nFilterDims=1000)))
plotHeatmap(dSp,isSymmetric=TRUE)

## ----clusterManyDiffDist_calculateMad,fig.width=15,fig.height=6---------------
ceDist<-makeFilterStats(ce,filterStats="mad")
ceDist

## ----clusterManyDiffDist,fig.width=15,fig.height=6----------------------------
ceDist<-clusterMany(ceDist, k=7:9, alpha=c(0.35,0.4,0.45), 
    clusterFunction=c("tight","hierarchical01","pam","hierarchicalK"),
    findBestK=FALSE,removeSil=c(FALSE), distFunction=c("corDist","spearDist"),
    makeMissingDiss=TRUE,
    reduceMethod=c("mad"),nFilterDims=1000,run=TRUE)
clusterLabels(ceDist)<-gsub("clusterFunction","alg",clusterLabels(ceDist))
clusterLabels(ceDist)<-gsub("Dist","",clusterLabels(ceDist))
clusterLabels(ceDist)<-gsub("distFunction","dist",clusterLabels(ceDist))
clusterLabels(ceDist)<-gsub("hierarchical","hier",clusterLabels(ceDist))
par(mar=c(1.1,15.1,1.1,1.1))
plotClusters(ceDist,axisLine=-2,colData=c("Biological_Condition"))

## ----custom-------------------------------------------------------------------
library(scran)
library(igraph)
SNN_wrap <- function(inputMatrix, k, steps = 4, ...) {
  snn <- buildSNNGraph(inputMatrix, k = k, d = NA, transposed = FALSE) ##scran package
  res <- cluster_walktrap(snn, steps = steps) #igraph package
  return(res$membership)
}

## ----custom_validate----------------------------------------------------------
internalFunctionCheck(SNN_wrap, inputType = "X", algorithmType = "K",
                      outputType="vector")

## ----custom_s4----------------------------------------------------------------
SNN <- ClusterFunction(SNN_wrap, inputType = "X", algorithmType = "K",
                     outputType="vector")

## ----custom_clusterSingle-----------------------------------------------------
ceCustom <- clusterSingle(ce, reduceMethod="PCA", 
    nDims=50, subsample = TRUE, 
    sequential = FALSE,
    mainClusterArgs = list(clusterFunction = "hierarchical01",
                         clusterArgs = list(alpha = 0.3),
                         minSize = 1),
    subsampleArgs = list(resamp.num=100,
                       samp.p = 0.7,
                       clusterFunction = SNN,
                       clusterArgs = list(k = 10),
                       ncores = 1,
                       classifyMethod='InSample')
)
ceCustom

## ----custom_getBuiltIn--------------------------------------------------------
clFuns<-getBuiltInFunction(c("pam","kmeans"))

## ----custom_addSNN------------------------------------------------------------
clFuns<-c(clFuns, "SNN"=SNN)

## ----run_custom---------------------------------------------------------------
ceCustom <- clusterMany(ce, dimReduce="PCA",nPCADims=50,
                        clusterFunction=clFuns,
                        ks=4:15, findBestK=FALSE)


## ----clusterManyCheckParam----------------------------------------------------
checkParam<-clusterMany(se, clusterFunction="pam", ks=2:10,
                        removeSil=c(TRUE,FALSE), isCount=TRUE,
                        reduceMethod=c("PCA","var"), makeMissingDiss=TRUE,
                        nFilterDims=c(100,500,1000),nReducedDims=c(5,15,50),run=FALSE)
dim(checkParam$paramMatrix) #number of rows is the number of clusterings

## ----clusterManyCheckParam2,eval=FALSE----------------------------------------
# # ce<-clusterMany(se,  paramMatrix=checkParam$paramMatrix, mainClusterArgs=checkParam$mainClusterArgs, seqArgs=checkParam$seqArgs,subsampleArgs=checkParam$subsampleArgs)
# ce<-clusterMany(ce, clusterFunction="pam",ks=2:10,findBestK=TRUE,removeSil=c(TRUE), isCount=TRUE,reduceMethod=c("PCA","var"),nFilterDims=c(100,500,1000),nReducedDims=c(5,15,50),run=TRUE)

## ----makeConsensus_detailed---------------------------------------------------
plotCoClustering(ce)

## ----sub_getParams------------------------------------------------------------
params <- getClusterManyParams(ce)
head(params)

## ----sub_makeConsensus--------------------------------------------------------
clusterIndices15 <- params$clusteringIndex[which(params$nReducedDims == 15)]
clusterIndices50 <- params$clusteringIndex[which(params$nReducedDims == 50)]
#note, the indices will change as we add clusterings!
clusterNames15 <- clusterLabels(ce)[clusterIndices15]
clusterNames50 <- clusterLabels(ce)[clusterIndices50]
shortNames15<-gsub("reduceMethod=PCA,nReducedDims=15,nFilterDims=NA,","",clusterNames15)
shortNames50<-gsub("reduceMethod=PCA,nReducedDims=50,nFilterDims=NA,","",clusterNames50)

ce <- makeConsensus(ce, whichClusters = clusterNames15, proportion = 0.7,
                       clusterLabel = "consensusPCA15")
ce <- makeConsensus(ce, whichClusters = clusterNames50, proportion = 0.7,
                       clusterLabel = "consensusPCA50")

## ----sub_visualize,fig.width=12,fig.height=6----------------------------------
par(mar=plotCMar,mfrow=c(1,2))
plotClustersWorkflow(ce, whichClusters ="consensusPCA15",clusterLabel="Consensus",whichClusterMany=match(clusterNames15,clusterLabels(ceSub)),clusterManyLabels=c(shortNames15),axisLine=-1,nBlankLines=1,main="15 PCs")
plotClustersWorkflow(ce, whichClusters = c("consensusPCA50"),clusterLabel="Consensus",clusterManyLabels=shortNames50, whichClusterMany=match(clusterNames50,clusterLabels(ceSub)),nBlankLines=1,main="50 PCs")

## ----makeConsensus_chooseClusters---------------------------------------------
wh<-getClusterManyParams(ce)$clusteringIndex[getClusterManyParams(ce)$reduceMethod=="var"]
ce<-makeConsensus(ce,whichCluster=wh,proportion=0.7,minSize=3,
                clusterLabel="makeConsensus,nVAR")
plotCoClustering(ce)

## ----makeConsensus_showDifferent----------------------------------------------
wh<-grep("makeConsensus",clusterTypes(ce))
par(mar=plotCMar)
plotClusters(ce,whichClusters=rev(wh),axisLine=-1)

## ----makeDendrogram_reducedFeatures-------------------------------------------
ce<-makeDendrogram(ce,reduceMethod="var",nDims=500)
plotDendrogram(ce)

## ----showCe-------------------------------------------------------------------
show(ce)

## ----remakeMakeDendrogram-----------------------------------------------------
ce<-makeDendrogram(ce,reduceMethod="var",nDims=500,
                   whichCluster="makeConsensus,final")

## ----plotRemadeDendrogram-----------------------------------------------------
plotDendrogram(ce,leafType="sample",plotType="colorblock")

## ----plotRemadeDendrogram_compare---------------------------------------------
par(mar=plotDMar)
whCM<-grep("makeConsensus",clusterTypes(ce))
plotDendrogram(ce,whichClusters=whCM,leafType="sample",plotType="colorblock")

## ----showCurrentPrimaryCluster------------------------------------------------
primaryClusterLabel(ce)
show(ce)

## ----clusterTypeOfCombineMany-------------------------------------------------
whFinal<-which(clusterLabels(ce)=="makeConsensus,final")
head(clusterMatrix(ce,whichCluster=whFinal))
clusterTypes(ce)[whFinal]

## ----getBackCombineMany-------------------------------------------------------
ce<-setToCurrent(ce,whichCluster="makeConsensus,final")
show(ce)

## ----printClusterDendrogram---------------------------------------------------
clusterDendrogram(ce)
head(sampleDendrogram(ce))

## ----showPhylobase------------------------------------------------------------
library(phylobase)
nodeLabels(clusterDendrogram(ce))
descendants(clusterDendrogram(ce),node="NodeId3")

## ----changeNodeLabels---------------------------------------------------------
newNodeLabels<-LETTERS[1:nNodes(ce)]
names(newNodeLabels)<-nodeLabels(ce)
nodeLabels(ce)<-newNodeLabels

## ----checkWhatDendro----------------------------------------------------------
ce

## ----runMergeDetail-----------------------------------------------------------
ce<-mergeClusters(ce,mergeMethod="adjP",plotInfo=c("adjP"),calculateAll=FALSE)

## ----getMergeInfo-------------------------------------------------------------
mergeMethod(ce)
mergeCutoff(ce)
nodeMergeInfo(ce)

## ----mergeClusters_plot,fig.width=12------------------------------------------
ce<-mergeClusters(ce,mergeMethod="none",plotInfo="all")

## ----showMergeAll-------------------------------------------------------------
nodeMergeInfo(ce)

## ----mergeClusters_ex---------------------------------------------------------
ce<-mergeClusters(ce,cutoff=0.05,mergeMethod="adjP",clusterLabel="mergeClusters,v2",plot=FALSE)
ce

## ----mergeClusters_reexamineNode----------------------------------------------
nodeMergeInfo(ce)

## ----mergeClusters_redo-------------------------------------------------------
ce<-mergeClusters(ce,cutoff=0.15,mergeMethod="Storey",
                  clusterLabel="mergeClusters,v3",plot=FALSE)
ce

## ----mergeClusters_compareMerges----------------------------------------------
par(mar=c(1.1,1.1,6.1,2.1))
plotDendrogram(ce,whichClusters=c("mergeClusters,v3","mergeClusters,v2"),mergeInfo="mergeMethod")

## ----mergeClusters_logFC------------------------------------------------------
ce<-mergeClusters(ce,cutoff=0.05,mergeMethod="adjP", logFCcutoff=2,
                  clusterLabel="mergeClusters,FC1",plot=FALSE)
ce

## ----mergeClusters_compareMergesAgain-----------------------------------------
par(mar=c(1.1,1.1,6.1,2.1))
plotDendrogram(ce,whichClusters=c("mergeClusters,FC1","mergeClusters,v3","mergeClusters,v2"),mergeInfo="mergeMethod")

## ----workflowTable------------------------------------------------------------
workflowClusterTable(ce)

## ----workflowDetails----------------------------------------------------------
head(workflowClusterDetails(ce),8)

## ----markFinal----------------------------------------------------------------
ce<-setToFinal(ce,whichCluster="mergeClusters,v2",
               clusterLabel="Final Clustering")
par(mar=plotCMar)
plotClusters(ce,whichClusters="workflow")

## ----rsecTable, echo=FALSE, message=FALSE, warnings=FALSE, results='asis'-----
# simple table creation here
tabl <- "  
| | Arguments in original function internally fixed | Arguments in RSEC for the user |   |  |
|:-----------------|:-----------------|:-------------|:------|:------|
     | | *Name of Argument in original function (if different)* | *Notes*|
*clusterMany*| sequential=TRUE | k0s              | ks | RSEC only sets 'k0', no other k 
- | distFunction=NA | clusterFunction | |
- | removeSil=FALSE | reduceMethod | | 
- | subsample=TRUE | nFilterDims | | 
- |  silCutoff=0 | nReducedDims | | 
- |  | alphas | |
- |  | betas | |
- |  | minSizes | |
-  | | mainClusterArgs | |
-  | | subsampleArgs | |
- |  | seqArgs | |
- |  | run | |
- |  | ncores | |
- |  | random.seed | |
- |  | isCount | |
- |  | transFun | |
-  |  | isCount | |  
*makeConsensus*  | propUnassigned = *(default)* | consensusProportion | proportion 
-  | consensusMinSize    | minSize | |
*makeDendrogram* | filterIgnoresUnassigned=TRUE | dendroReduce      | reduceMethod | 
-  | unassignedSamples= *(default)* | dendroNDims       | nDims |  
*mergeClusters*   | plot=FALSE | mergeMethod     | | 
-  | | mergeCutoff | cutoff | 
-  | | mergeDEMethod | DEMethod |
-  | | mergeLogFCcutoff | logFCcutoff |
"

cat(tabl) # output the table in a format good for HTML/PDF/docx conversion


## ----getBestFeatures_onlyTopPairs---------------------------------------------
pairsAllTop<-getBestFeatures(rsecFluidigm,contrastType="Pairs",DEMethod="edgeR",p.value=0.05)
dim(pairsAllTop)
head(pairsAllTop)

## ----getBestFeatures_oneAgainstAll--------------------------------------------
best1vsAll<-getBestFeatures(rsecFluidigm,contrastType="OneAgainstAll",DEMethod="edgeR",p.value=0.05,number=NROW(rsecFluidigm))
head(best1vsAll)

## ----getBestFeatures_oneHeatmap-----------------------------------------------
plotContrastHeatmap(rsecFluidigm,signifTable=best1vsAll,nBlankLines=10, whichCluster="primary")

## ----getBestFeatures_dendro---------------------------------------------------
bestDendro<-getBestFeatures(rsecFluidigm,contrastType="Dendro",DEMethod="edgeR",p.value=0.05,number=NROW(rsecFluidigm))
head(bestDendro)

## ----dendroContrastLevels-----------------------------------------------------
levels((bestDendro)$Contrast)

## ----getBestFeatures_dendroHeatmap--------------------------------------------
plotContrastHeatmap(rsecFluidigm,signifTable=bestDendro,nBlankLines=10)

## ----dendroWithNodeNames------------------------------------------------------
plotDendrogram(rsecFluidigm,show.node.label=TRUE,whichClusters=c("makeConsensus","mergeClusters"),leaf="samples",plotType="colorblock")

## ----exampleCode,eval=FALSE---------------------------------------------------
# assay(rsecFluidigm,"weights")<- myweights

## ----slotnames----------------------------------------------------------------
assayNames(se)

## ----clusterUnnormalized, eval=FALSE------------------------------------------
# rsecTPM <- RSEC(se, whichAssay = "rsem_tpm", transFun = log1p, mergeDEMethod="limma",
#               reduceMethod="PCA", nReducedDims=10, consensusMinSize=3,
#               ncores=1, random.seed=176201)

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

#' Create a matrix of clustering across values of parameters
#'
#' Given a range of parameters, this function will return a matrix with the
#' clustering of the samples across the range, which can be passed to
#' \code{plotClusters} for visualization.
#'
#' @name clusterMany
#' @aliases clusterMany,matrixOrHDF5-method
#' @param x the data matrix on which to run the clustering. Can be object of the
#'   following classes: matrix (with genes in rows),
#'   \code{\link[SummarizedExperiment]{SummarizedExperiment}},
#'   \code{\link[SingleCellExperiment]{SingleCellExperiment}}
#'   or \code{ClusterExperiment}.
#' @param ks the range of k values (see details for the meaning of \code{k} for
#'   different choices of other parameters).
#' @param alphas values of alpha to be tried. Only used for clusterFunctions of
#'   type '01'. Determines tightness required in creating clusters from the
#'   dissimilarity matrix. Takes on values in [0,1]. See documentation of
#'   \code{\link{ClusterFunction}}.
#' @param betas values of \code{beta} to be tried in sequential steps. Only used
#'   for \code{sequential=TRUE}. Determines the similarity between two clusters
#'   required in order to deem the cluster stable. Takes on values in [0,1]. See
#'   documentation of \code{\link{seqCluster}}.
#' @param clusterFunction function used for the clustering. This must be either 
#'   1) a character vector of built-in clustering techniques, or 2) a
#'   \emph{named} list of \code{\link{ClusterFunction}} objects. Current
#'   functions can be found by typing \code{listBuiltInFunctions()} into the
#'   command-line.
#' @param minSizes the minimimum size required for a cluster (in the
#'   \code{mainClustering} step). Clusters smaller than this are not kept and samples
#'   are left unassigned.
#' @param parameterWarnings logical, as to whether warnings and comments 
#'   from checking the validity of the parameter combinations should be printed. 
#' @param distFunction a vector of character strings that are the names of
#'   distance functions found in the global environment. See the help pages of
#'   \code{\link{clusterSingle}} for details about the required format of
#'   distance functions. Currently, this distance function must be applicable
#'   for all clusterFunction types tried. Therefore, it is not possible in
#'   \code{clusterMany} to intermix type "K" and type "01" algorithms if you
#'   also give distances to evaluate via \code{distFunction} unless all
#'   distances give 0-1 values for the distance (and hence are possible for both
#'   type "01" and "K" algorithms).
#' @param nFilterDims vector of the number of the most variable features to keep
#'   (when "var", "abscv", or "mad" is identified in \code{reduceMethod}).
#' @param nReducedDims vector of the number of dimensions to use (when
#'	\code{reduceMethod} gives a dimensionality reduction method).
#' @param eraseOld logical. Only relevant if input \code{x} is of class
#'   \code{ClusterExperiment}. If TRUE, will erase existing workflow results
#'   (clusterMany as well as mergeClusters and makeConsensus). If FALSE, existing
#'   workflow results will have "\code{_i}" added to the clusterTypes value,
#'   where \code{i} is one more than the largest such existing workflow
#'   clusterTypes.
#' @param findBestK logical, whether should find best K based on average
#'   silhouette width (only used when clusterFunction of type "K").
#' @param silCutoff Requirement on minimum silhouette width to be included in
#'   cluster (only for combinations where removeSil=TRUE).
#' @param removeSil logical as to whether remove when silhouette < silCutoff
#'   (only used if clusterFunction of type "K")
#' @inheritParams clusterSingle
#' @inheritParams mainClustering
#' @param ncores the number of threads
#' @param random.seed a value to set seed before each run of clusterSingle (so
#'   that all of the runs are run on the same subsample of the data). Note, if
#'   'random.seed' is set, argument 'ncores' should NOT be passed via
#'   subsampleArgs; instead set the argument 'ncores' of clusterMany directly
#'   (which is preferred for improving speed anyway). 
#' @param run logical. If FALSE, doesn't run clustering, but just returns matrix
#'   of parameters that will be run, for the purpose of inspection by user (with
#'   rownames equal to the names of the resulting column names of clMat object
#'   that would be returned if \code{run=TRUE}). Even if \code{run=FALSE},
#'   however, the function will create the dimensionality reductions of the data
#'   indicated by the user input.
#' @param ... For signature \code{matrix}, arguments to be passed on to mclapply
#'   (if ncores>1). For all the other signatures, arguments to be passed to the
#'   method for signature \code{matrix}.
#' @param verbose logical. If TRUE it will print informative messages.
#' @details Some combinations of these parameters are not feasible. See the
#'   documentation of \code{\link{clusterSingle}} for important information on
#'   how these parameter choices interact.
#' @details While the function allows for multiple values of clusterFunction,
#'   the code does not reuse the same subsampling matrix and try different
#'   clusterFunctions on it. This is because if sequential=TRUE, different
#'   subsample clusterFunctions will create different sets of data to subsample
#'   so it is not possible; if sequential=FALSE, we have not implemented
#'   functionality for this reuse. Setting the \code{random.seed} value,
#'   however, should mean that the subsampled matrix is the same for each, but
#'   there is no gain in computational complexity (i.e. each subsampled
#'   co-occurence matrix is recalculated for each set of parameters).
#'
#' @details The argument \code{ks} is interpreted differently for different
#'   choices of the other parameters. When/if sequential=TRUE, \code{ks} defines
#'   the argument \code{k0} of \code{\link{seqCluster}}. Otherwise, \code{ks}
#'   values are the \code{k} values for \strong{both} the mainClustering and
#'   subsampling step (i.e. assigned to the \code{subsampleArgs} and
#'   \code{mainClusterArgs} that are passed to \code{\link{mainClustering}} and
#'   \code{\link{subsampleClustering}} unless \code{k} is set appropriately in
#'   \code{subsampleArgs}. The passing of these arguments via
#'   \code{subsampleArgs} will only have an effect if `subsample=TRUE`.
#'   Similarly, the passing of \code{mainClusterArgs[["k"]]} will only have an
#'   effect when the clusterFunction argument includes a clustering algorithm of
#'   type "K". When/if "findBestK=TRUE", \code{ks} also defines the
#'   \code{kRange} argument of \code{\link{mainClustering}} unless \code{kRange}
#'   is specified by the user via the \code{mainClusterArgs}; note this means
#'   that the default option of setting \code{kRange} that depends on the input
#'   \code{k} (see \code{\link{mainClustering}}) is not available in
#'   \code{clusterMany}, only in \code{\link{clusterSingle}}.
#' @details If the input is a \code{ClusterExperiment} object, current
#'   implementation is that existing \code{orderSamples},\code{coClustering} or
#'   the many dendrogram slots will be retained.
#' @details If \code{run=FALSE}, the function will still calculate reduced 
#'   dimensions or filter statistics if not already calculated and saved in the object.
#'   Moreover the results of these calculations will not be save. Therefore, if these 
#' steps are lengthy for large datasets it is
#'   recommended to do them before calling the function.
#' @return If \code{run=TRUE} will
#'   return a \code{ClusterExperiment} object, where the results are stored as
#'   clusterings with clusterTypes \code{clusterMany}. Depending on
#'   \code{eraseOld} argument above, this will either delete existing such
#'   objects, or change the clusterTypes of existing objects. See argument
#'   \code{eraseOld} above. Arbitrarily the first clustering is set as the
#'   primaryClusteringIndex.
#'
#' @return If \code{run=FALSE} a list with elements:
#' \itemize{
#'   \item{\code{paramMatrix}}{ a matrix giving the parameters of each
#'   clustering, where each column is a possible parameter set by the user and
#'   passed to \code{\link{clusterSingle}} and each row of paramMatrix
#'   corresponds to a clustering in \code{clMat}} 
#'   \item{\code{mainClusterArgs}}{
#'   a list of (possibly modified) arguments to mainClusterArgs}
#'   \item{\code{seqArgs=seqArgs}}{a list of (possibly modified) arguments to
#'   seqArgs} 
#'   \item{\code{subsampleArgs}}{a list of (possibly modified)
#'   arguments to subsampleArgs} }
#'
#' @examples
#' \dontrun{
#' data(simData)
#'
#' #Example: clustering using pam with different dimensions of pca and different
#' #k and whether remove negative silhouette values
#' #check how many and what runs user choices will imply:
#' checkParams <- clusterMany(simData,reduceMethod="PCA", makeMissingDiss=TRUE,
#'    nReducedDims=c(5,10,50), clusterFunction="pam", isCount=FALSE,
#'    ks=2:4,findBestK=c(TRUE,FALSE),removeSil=c(TRUE,FALSE),run=FALSE)
#' print(head(checkParams$paramMatrix))
#'
#' #Now actually run it
#' cl <- clusterMany(simData,reduceMethod="PCA", nReducedDims=c(5,10,50),  isCount=FALSE,
#'    clusterFunction="pam",ks=2:4,findBestK=c(TRUE,FALSE),makeMissingDiss=TRUE, 
#'    removeSil=c(TRUE,FALSE))
#' print(cl)
#' head(colnames(clusterMatrix(cl)))
#'
#' #make names shorter for plotting
#' clNames <- clusterLabels(cl)
#' clNames <- gsub("TRUE", "T", clNames)
#' clNames <- gsub("FALSE", "F", clNames)
#' clNames <- gsub("k=NA,", "", clNames)
#'
#' par(mar=c(2, 10, 1, 1))
#' plotClusters(cl, axisLine=-2,clusterLabels=clNames)
#'
#'
#'	#following code takes around 1+ minutes to run because of the subsampling
#'	#that is redone each time:
#'	system.time(clusterTrack <- clusterMany(simData, ks=2:15,
#'	    alphas=c(0.1,0.2,0.3), findBestK=c(TRUE,FALSE), sequential=c(FALSE),
#'	    subsample=c(FALSE), removeSil=c(TRUE), clusterFunction="pam", 
#'	    makeMissingDiss=TRUE,
#'      mainClusterArgs=list(minSize=5, kRange=2:15), ncores=1, random.seed=48120))
#' }
#'
#' @rdname clusterMany
#' @importFrom parallel mclapply
#' @export
# not currently functional:
# @param paramMatrix matrix or data.frame. If given, the algorithm will bypass
#   creating the matrix of possible parameters, and will use the given matrix.
#   There are basically no checks as to whether this matrix is in the right
#   format, and is only intended to be used to feed the results of setting
#   \code{run=FALSE} back into the algorithm (see example).
##Was in @examples:
# #get rid of some of the choices manually
# #note that the supplement arguments could have been changed too, so
# #we give those to clusterMany as well.
# checkParamsMat <- checkParams$paramMatrix[-c(1,2),]
#
# clSmaller <- clusterMany(simData, nReducedDims=c(5,10,50),  reduceMethod="PCA",
# paramMatrix=checkParamsMat, subsampleArgs=checkParams$subsampleArgs,
# seqArgs=checkParams$seqArgs, mainClusterArgs=checkParams$mainClusterArgs)
#' @export
setMethod(
    f = "clusterMany",
    signature = signature(x = "matrixOrHDF5"),
    definition = function(x,
                          reduceMethod="none",nReducedDims=NA, transFun=NULL,isCount=FALSE, ...
    ){
        ####Basically, matrix version calls makeReducedDims and makeFilterStats and then sends it to the SingleCellExperiment version.
        if(missing(reduceMethod)) reduceMethod<-"none"
        # if(anyNA(nReducedDims)){
        # 		if(!"none" in reduceMethod) reduceMethod<-c(reduceMethod,"none")
        # 		nReducedDims<-na.omit(nReducedDims)
        # 	}
        if(any(dim(x)==0)) stop("x must have non zero dimensions")
        reduceMethod<-unique(reduceMethod)
        doNone<-any(reduceMethod=="none")
        #check can given reduceMethod values match built in options.
        dimNam<-reduceMethod[isBuiltInReducedDims(reduceMethod)]
        filtNam<-reduceMethod[isBuiltInFilterStats(reduceMethod)]
        nValid<-length(c(dimNam,filtNam))
        if(doNone) nValid<-nValid+1
        if(!doNone & length(dimNam)==0 & length(filtNam)==0)
            stop("reduceMethod values given are not in built-in dimensionality reduction or built-in filters (and there is no such stored objects if a SingleCellExperiment object). Option 'none' also not given, so nothing to do.")
        else if(length(reduceMethod)!=nValid)
            warning("Some reduceMethod values given are not in built in dimensionality reduction or built in filters (and there is no such stored objects if a SingleCellExperiment object). Ignoring options.")
        if(length(dimNam)>0 | length(filtNam)>0){
            if(length(dimNam)>0){
                nReducedDims<-na.omit(nReducedDims)
                if(length(nReducedDims)==0)
                    stop("Must give nReducedDims values if choose a reduceMethod option not equal to 'none' and not in stored reducedDims slot.")
                maxDims<-max(nReducedDims)
                x<-makeReducedDims(x,reducedDims=dimNam,
                                   maxDims=maxDims,transFun=transFun,isCount=isCount)
            }
            if(length(filtNam)>0){
                #Need to think how can pass options to filterData...
                x<-makeFilterStats(x,filterStat=filtNam, transFun=transFun,isCount=isCount)
            }
        }
        else{
            x<-SingleCellExperiment(x)
        }
        return(clusterMany(x,reduceMethod=reduceMethod,nReducedDims=nReducedDims,transFun=transFun,isCount=isCount,...))
    }
)

#' @rdname clusterMany
#' @param whichAssay numeric or character specifying which assay to use. See
#'   \code{\link[SummarizedExperiment]{assay}} for details.
#' @details The given \code{reduceMethod} values must either be \emph{all}
#'   precalculated filtering/dimensionality reduction stored in the appropriate
#'   location, or must \emph{all} be character values giving a built-in
#'   filtering/dimensionality reduction methods to be calculated. If some of the
#'   filtering/dimensionality methods are already calculated and stored, but not
#'   all, then they will \emph{all} be recalculated (and if they are not all
#'   built-in methods, this will give an error). So to save computational time
#'   with pre-calculated dimensionality reduction, the user must make sure they
#'   are \emph{all} precalculated. Also, user-defined values (i.e. not built-in
#'   functions) cannot be mixed with built-in functions unless they have already
#'   been precalculated (see \code{\link{makeFilterStats}} or
#'   \code{\link{makeReducedDims}}).
#' @export
setMethod(
    f = "clusterMany",
    signature = signature(x = "SingleCellExperiment"),
    definition = function(x, ks=NA, clusterFunction,
                          reduceMethod="none",
                          nFilterDims= defaultNDims(x, 
                              reduceMethod, type="filterStats"),
                          nReducedDims= defaultNDims(x, 
                              reduceMethod, type="reducedDims"),
                          alphas=0.1, findBestK=FALSE,
                          sequential=FALSE, removeSil=FALSE, subsample=FALSE,
                          silCutoff=0, distFunction=NA,
                          betas=0.9, minSizes=1,
                          transFun=NULL,isCount=FALSE,
                          verbose=TRUE,
                          parameterWarnings=FALSE,
                          mainClusterArgs=NULL,
                          subsampleArgs=NULL,
                          seqArgs=NULL,
                          whichAssay=1,
                          makeMissingDiss=if(ncol(x)<1000) TRUE else FALSE,
                          ncores=1, random.seed=NULL, run=TRUE,
                          ...
    )
    {
        
        #need so can pass all the args, not just the ...
        inputArgs<-as.list(environment()) 
        transFun<-.makeTransFun(transFun=transFun,isCount=isCount)
        paramMatrix<-NULL
        if(!is.null(random.seed)){
            if(!is.null(subsampleArgs) && "ncores" %in% names(subsampleArgs)){
                if(subsampleArgs[["ncores"]]>1) stop("setting random.seed will not be reproducible if ncores given to subsampleArgs")
            }
        }
        
        #######################
        ### Deal with reduceMethods
        #######################
        #issue: have to send reduceMethod, but don't know which are which type
        isExisting<-isReducedDims(x,reduceMethod) | isFilterStats(x,reduceMethod)
        isBuiltIn<- isBuiltInReducedDims(reduceMethod) | isBuiltInFilterStats(reduceMethod)
        isNone<-reduceMethod=="none"
        if(any(isNone)){
            isExisting[isNone]<-TRUE
            isBuiltIn[isNone]<-TRUE
        }
        isBuiltInNotExisting<-isBuiltIn & !isExisting
        
        # anyFilter<-anyValidFilterStats(x)
        # anyDim<-anyValidReducedDims(x)
        # anyFilterSaved<-anyFilter && any(isFilterStats(x,reduceMethod))
        # anyDimSaved<-anyDim && any(isReducedDims(x,reduceMethod))
        # anyDimBuiltIn<-any(isBuiltInReducedDims(reduceMethod))
        # anyFilterBuiltIn<-any(isBuiltInFilterStats(reduceMethod))
        if(!all(isNone | isBuiltIn | isExisting))
            stop("Some values of 'reduceMethod' do not match any stored or built-in filtering statistics or dimensionality reduction")
        if(!all(isNone | isBuiltIn) & !all(isNone | isExisting))
            stop("All values of 'reduceMethod' need to either match an existing (i.e. stored) filtering/dimensionality reduction or they need to all match a built-in function to be calculated")
        
        

        if(all(isBuiltIn) & any(isBuiltInNotExisting) ){
            reduceMethod<-unique(reduceMethod)
            doNone<-any(reduceMethod=="none")
            
            .mynote(paste0("Not all of the methods requested in 'reduceMethod' have been calculated. Will calculate all the methods requested (any pre-existing values -- filtering statistics or dimensionality reductions -- with these names will be recalculated and overwritten): ",paste(reduceMethod,collapse=","),"."))
               
            #check can given reduceMethod values match built in options.
            dimNam<-reduceMethod[isBuiltInReducedDims(reduceMethod)]
            filtNam<-reduceMethod[isBuiltInFilterStats(reduceMethod)]
            nValid<-length(c(dimNam,filtNam))
            
            if(doNone) nValid<-nValid+1
            
            if(!doNone & length(dimNam)==0 & length(filtNam)==0)
                stop("reduceMethod values given are not in built-in dimensionality reduction or built-in filters (and there is no such stored objects if a SingleCellExperiment object). Option 'none' also not given, so nothing to do.")
            else if(length(reduceMethod)!=nValid)
                warning("Some reduceMethod values given are not in built in dimensionality reduction or built in filters (and there is no such stored objects if a SingleCellExperiment object). Ignoring options.")
            
            if(length(dimNam)>0 | length(filtNam)>0){
                if(length(dimNam)>0){
                    nReducedDims<-na.omit(nReducedDims)
                    if(length(nReducedDims)==0)
                        stop("Must give nReducedDims values if choose a reduceMethod option not equal to 'none' and not in stored reducedDims slot.")
                    maxDims<-max(nReducedDims)
                    x<-makeReducedDims(x,reducedDims=dimNam, 
                        whichAssay = whichAssay,
                        maxDims=maxDims,transFun=transFun,
                        isCount=isCount)
                }
                if(length(filtNam)>0){
                    #Need to think how can pass options to filterData...
                    x<-makeFilterStats(x,filterStat=filtNam, 
                        whichAssay = whichAssay,
                        transFun=transFun,isCount=isCount)
                }
            }
        }
        
        #----------------
        # Should come here ONLY if all methods needed have been calculated (i.e. built-in function)
        #Check inputs of reduceMethod slots
        ##NOTE: For now, IF there is a reducedDim slot, then will not try
        ##to patch in ones that are missing.
        ##This means can list some that want to be calculated.
        ##Either do all of them ahead of time or let all of them be done
        ##during call to clusterMany...
        #----------------
        doNone<-any(isNone)
        if(doNone) reduceMethod<-reduceMethod[-grep("none",reduceMethod)]
        if(length(reduceMethod)>0){
            if(any(!isReducedDims(x,reduceMethod) & !isFilterStats(x,reduceMethod))){
                stop("Internal Coding Error -- shouldn't have gotten to this point without catching from earlier error check that all necessary statistics are calculated.")
            }
            #check if nReducedDims values
            if(any(isReducedDims(x,reduceMethod))){
                maxDimValues<- 
                    ncolReducedDims(x)[isReducedDims(x,reduceMethod)]
                if(length(na.omit(nReducedDims))>0 && all(na.omit(nReducedDims) > max(maxDimValues)))
                    stop("The values of nReducedDims given are all higher than the maximum components stored in the reducedDims slot of the input object. Run 'makeReducedDims' to get larger number of components.")
                
            }
            
            #check if give nFilterDims if isFilterStats and no NA values
            if(any(isFilterStats(x,reduceMethod))){
                if(!missing(nFilterDims) && any(is.na(nFilterDims))){
                    warning("NA values have no meaning for the argument nFilterDims and will be ignored")
                    nFilterDims<-na.omit(nFilterDims)
                }
                if(length(nFilterDims)==0){
                    stop("no valid nFilterDims values given, but reduceMethod values given indicate a filterStat to be used.")
                }
            }
        }
        else{
            nReducedDims<-NA
            nFilterDims<-NA
            maxDimValues<-NA #indicates that only "none" will be done
        }
        #----------FINISH reduce methods section------
        
        ###############
        # Start creating the parameter combinations
        ###############
        if(is.null(paramMatrix)){
            if(is.list(clusterFunction)){
                .checkFunctionList(clusterFunction)
                clusterFunctionList<-clusterFunction
                clusterFunctionNames<-names(clusterFunction)
            }
            else{
                clusterFunctionList<-getBuiltInFunction(clusterFunction)
                if(length(clusterFunction)==1 & is(clusterFunctionList,"ClusterFunction")){
                    clusterFunctionList<-list(clusterFunctionList)
                    names(clusterFunctionList)<-clusterFunction
                }
                clusterFunctionNames<-clusterFunction
            }
            if(doNone) reduceMethod<-c(reduceMethod,"none")
            param <- expand.grid(clusterFunction=clusterFunctionNames,
                k=ks, 
                alpha=alphas, 
                beta=betas, 
                sequential=sequential, 
                subsample=subsample,
                reduceMethod=reduceMethod,
                nReducedDims=nReducedDims, 
                nFilterDims=nFilterDims,
                minSize=minSizes,
                findBestK=findBestK,
                removeSil=removeSil, 
                silCutoff=silCutoff,
                distFunction=distFunction
                )
            
            if(nrow(param)<=1) {
                stop("set of parameters imply only 1 combination. If you wish to run a single clustering, use 'clusterSingle'")
            }
            #-------------
            # Check param matrix:
            # don't vary them across ones that don't matter (i.e. 0-1 versus K);
            #   -> code sets to single value and then will do unique to remove 
            # also deals with just in case the user gave duplicated 
            #   values of something by mistake.
            #-------------
            
            # small function to pull the list of all functions 
            # from current param matrix            
            cf<-function(param){
                clusterFunctionList[param[,"clusterFunction"]]
            }
            paramAlgTypes<-algorithmType(cf(param))
            if(length(paramAlgTypes)!=nrow(param)) stop("Internal coding error in clusterMany: not getting right number of type of algorithms from param")
            #---
            #type K fixes
            #---
            typeK <- which( paramAlgTypes=="K")
            if(length(typeK)>0){
                param[typeK,"alpha"] <- NA #just a nothing value, because doesn't mean anything here
                #--------
                #if findBestK make sure other arguments make sense:
                #--------
                whFindBestK <- which(param[,"findBestK"])
                if(length(whFindBestK)>0){
                    #by default make kRange in mainClustering equal to the ks. Note this will be true of ALL
                    if(!"kRange" %in% names(mainClusterArgs)) {
                        mainClusterArgs[["kRange"]]<-ks
                    }
                    #if findBestK=TRUE, and sequential=FALSE, then need to set 'k'=NA
                    whNoSeq <- which(!param[,"sequential"])
                    if(length(intersect(whFindBestK,whNoSeq))>0){
                        param[intersect(whFindBestK,whNoSeq),"k"] <- NA
                    }
                    #and if subsample=TRUE, then user needs to set k via subsampleArgs
                    ##Might could handle this better by call to .checkArgs
                    whNoSeqSub <- which(!param[,"sequential"] & param[,"subsample"])
                    if(length(intersect(whFindBestK,whNoSeqSub))>0 &
                       is.null(subsampleArgs[["clusterArgs"]]) && is.null(subsampleArgs[["clusterArgs"]][["k"]])){
                        stop("must provide k in 'clusterArgs' element of 'subsampleArgs' because there are combinations of findBestK=TRUE, sequential=FALSE and subsample=TRUE. (Note this will set 'k' for all combinations that subsample, not just this parameter combinations)")
                    }
                }
            }
            #---
            # type01 combinations
            #---
            type01 <- which( paramAlgTypes=="01")
            if(length(type01)>0){
                param[type01,"findBestK"] <- FALSE
                param[type01,"removeSil"] <- FALSE
                param[type01,"silCutoff"] <- 0
            }
            #---
            #Turn off distFunction for some combinations
            #---
            #those that subsample, because will distance that of co-occurance
            whSubsample<-which(param[,"subsample"])
            if(length(whSubsample)>0){
                param[whSubsample,"distFunction"]<-NA
            }
            #those that use reducedDims will not use dist
            #but those that filter could use different distances...
						# FIXME: should there be some kind of warning if wind up completely ignoring 'distFunction' that user gave? Ditto on other parameters.
            whDimReduce<-which(param[,"reduceMethod"]!="none" & isReducedDims(x,param[,"reduceMethod"]) )
            if(length(whDimReduce)>0){
                param[whDimReduce,"distFunction"]<-NA
            }
            
            #---
            # deal with nReducedDims NA or larger than the size of the dataset
            # set it to the maximum value possible.
            #---
            whDimReduce<-which(isReducedDims(x,param[,"reduceMethod"]))
            if(length(whDimReduce)>0 && length(na.omit(maxDimValues[whDimReduce]))>0){
                #if NA, means do the largest possible dimension saved for that method
                whNADim<-intersect(which(is.na(param[,"nReducedDims"])),whDimReduce)
                maxDimValuesNA<-maxDimValues[param[whNADim,"reduceMethod"]]
                if(length(whNADim)>0){
                    param[whNADim,"nReducedDims"]<-maxDimValuesNA
                }
                if(anyNA(param[whDimReduce,"nReducedDims"])) stop("Internal coding error: didn't get rid of NA reduceMethod in checks")
                whAbove<-intersect(which(param[,"nReducedDims"] > maxDimValues[param[,"reduceMethod"]]), whDimReduce)
                maxDimValuesAbove<-maxDimValues[param[whAbove,"reduceMethod"]]
                if(length(whAbove)>0){
                    param[whAbove,"nReducedDims"]<-maxDimValuesAbove
                }
            }
            #now turn to NA is when reduceMethod a dim reduce
            whOther<-which(!isReducedDims(x,param[,"reduceMethod"]))
            if(length(whOther)>0){
                param[whOther,"nReducedDims"]<-NA
            }
            
            #---
            # deal with nFilterDims NA or larger than the size of the dataset
            # set it to the maximum value possible.
            #---
            whFilter<-which(isFilterStats(x,param[,"reduceMethod"]))
            whTooLarge<-intersect(which(param[,"nFilterDims"]>NROW(x)),whFilter)
            if(length(whTooLarge)>0){
                param[whTooLarge,"reduceMethod"]<-"none"
            }
            
            #now turn to NA is when reduceMethod a dim reduce
            whOther<-which(!isFilterStats(x,param[,"reduceMethod"]))
            if(length(whOther)>0){
                param[whOther,"nFilterDims"]<-NA
            }
           
            # get rid of duplicates
            param <- unique(param)
            
            #####################
            # Deal with those that are invalid combinations:
            # Also, if ever reinstate param option, then should apply these checks to that param
            #####################
            if(is.null(mainClusterArgs)) 
                mainClusterArgs<-list(clusterArgs=list())
            if(is.null(subsampleArgs)) 
                subsampleArgs<-list(clusterArgs=list())
            paramCheck<-function(paramRow, returnValue, warn){
                totalArgs<- .makeArgsFromParam(paramRow,
                    mainClusterArgs=mainClusterArgs,
                    seqArgs=seqArgs,
                    subsampleArgs=subsampleArgs,
                    clusterFunctionList=clusterFunctionList)
                checkOut<-.checkArgs(inputType="X",
                                 main=TRUE,
                                 subsample=totalArgs$subsample,
                                 sequential=totalArgs$sequential,
                                 mainClusterArgs=totalArgs$mainClusterArgs,
                                 subsampleArgs=totalArgs$subsampleArgs,
                                 seqArgs=totalArgs$seqArgs,
                                 allowMakeDiss=TRUE,
                                 warn = warn) #most of these are because have extra parameters 
                if(returnValue=="logical"){
                    if(is.character(checkOut)) return(FALSE)
                    else return(TRUE)                    
                }
                else{
                    return(checkOut)
                }
            
            }
            #-----
            # Do first run to get rid of invalid selections (i.e. silently)
            # Note! Cannot run apply on param if don't want all character values.
            # e.g. checks<-apply(param,1,paramCheck,returnValue="logical")
            #-----
            checks<-sapply(1:nrow(param), 
                function(i){paramCheck(param[i,],returnValue="logical",
                    warn=FALSE)})
            whInvalid<-which(!checks)
            if(length(whInvalid)>0) {
                if(length(whInvalid)==nrow(param)){
                    checksFull<-lapply(1:nrow(param), 
                        function(i){paramCheck(param[i,],
                            returnValue="full",warn=parameterWarnings)})
                    stop(sprintf("Set of parameters imply %s combinations, none of which are valid (note that if clustering matrix requires calculation of distance matrix, user must now set `makeMissingDiss=TRUE`).  Errors given are:\n\n %s",nrow(param),paste(checksFull[whInvalid],collapse="\n")))
                    
                }
                param <- param[-whInvalid, ,drop=FALSE]
            }
            #-----
            # Do second run to get full warnings that remain
            # & deal with creating distances
            #-----
            checks<-lapply(1:nrow(param), 
                function(i){paramCheck(param[i,],returnValue="full",
                    warn=parameterWarnings)})
            # doDiss means will send diss as inputMatrix
            # doDissPost means will pass to diss= in mainClusterArgs        
            doDiss<-sapply(checks,function(x){x$doDiss}) | !is.na(param$distFunction)
            doDissPost<-sapply(checks,function(x){x$doDissPost})
            #in case user asked for distFunction, don't do again:
            doDissPost[doDiss]<-FALSE 
            param$passDistToInput <- doDiss
            param$passDistToMain <- doDissPost
            # missDiss is all places where will need calculate distance
            missDiss<-doDiss | doDissPost
            if(any(missDiss) & !makeMissingDiss){
                stop("Parameter combinations requested require calculation of at least one distance matrix. User must now set `makeMissingDiss=TRUE` to have clusterMany calculated the necessary distances.")
            }
            if(any(missDiss) & makeMissingDiss){
                ## expand param matrix to include additional information for calculating/passing distances
                subCF<-sapply(checks,function(x){
                    cf<-x$subsampleArgs[["clusterFunction"]]
                    if(!is.null(cf)) algorithmType(cf) else NA})
                mainCF<-sapply(checks,function(x){
                    algorithmType(x$mainClusterArgs[["clusterFunction"]])})                
                # Need clustering algorithm for calculating distances
                param$distAlgType<-NA
                param$distAlgType[missDiss]<-
                    ifelse(param[missDiss,"subsample"],subCF,mainCF)
                param$distAlgType[!doDiss & doDissPost]<-
                    mainCF[!doDiss & doDissPost]

                # give "default" to those not assigned
                # make sure it is default that corresponds to algorithm.
                param[is.na(param$distFunction) & missDiss, "distFunction"] <-
                    param$distAlgType[is.na(param$distFunction) & missDiss]
            
            }
            
            if(any(!is.na(param[,"nFilterDims"]) &
             !is.na(param[,"nReducedDims"]))){
                stop("Internal error: failed to properly remove inconsistent nFilterDims, nReducedDims combination.")
                
            }
            if(any(is.na(param[,"nFilterDims"]) &
                is.na(param[,"nReducedDims"] & 
                !param[,"reduceMethod"] %in% "none"))) stop("Internal error: NA in both nFilterDims, nReducedDims combination without equal to 'none'")
            #####
            #require at least 2 combinations:
            #####
            if(nrow(param)<=1) {
                stop("set of parameters imply only 1 combination, after removing duplicates and invalid combinations. If you wish to run a single clustering, use 'clusterSingle'")
            }
            
            #####
            #give names to the parameter combinations.
            #####
            charParam<-as.matrix(param)
            ## Make default distance more interpretable
            whDefault<-which(charParam[,"distFunction"]=="default")
            if(length(whDefault)>0){
                charParam[whDefault,"distFunction"]<-
                    paste(charParam[whDefault,"distFunction"],
                        charParam[whDefault,"distAlgType"],sep="")           
            }
            ## Remove those that I added
            myAdditions<-c("distAlgType","passDistToMain","passDistToInput")
            wh<-which(names(param) %in% myAdditions)
            if(length(wh)>0) charParam<-charParam[,-wh]
            whVary <- which(apply(charParam,2,function(x){length(unique(x))>1}))
            if(length(whVary)>0) {
                makeLabel<- function(ii){
                    paste(colnames(charParam)[whVary],charParam[ii,whVary],sep="=",collapse=",")
                }
                cnames<-sapply(seq_len(nrow(charParam)),makeLabel)
            } else {
                stop("set of parameters imply only 1 combination. If you wish to run a single clustering, use 'clusterSingle'")
            }
            cnames <- gsub("dataset=","",cnames)
            cnames <- gsub("= ","=",cnames)
            cnames[param[,"sequential"]] <- gsub("k=", "k0=", cnames[param[,"sequential"]])
            #should I combine together nReducedDims and nFilterDims like they were before for the labels?
            rownames(param) <- cnames
            
        } else{ #if paramMatrix!=NULL, have killed off this code for now, because doesn't work.
            if(!run) {
                stop("If paramMatrix is given, run should be TRUE. Otherwise there is no effect.")
            }
            if(is.null(paramMatrix)) {
                stop("invalid input for paramMatrix; must be data.frame or matrix")
            }
            param <- paramMatrix
            if(is.null(rownames(paramMatrix))) {
                stop("input paramMatrix must have row names")
            }
            cnames<-rownames(paramMatrix)
        }
        
        if(verbose) {
            cat(nrow(param),"parameter combinations,",sum(param[,"sequential"]),"use sequential method,",sum(param[,"subsample"]),"use subsampling method\n")
        }
           
        

        ################
        ## Function that will call clusterSingle for each row of param matrix
        ################
        paramFun <- function(i){
            par <- param[i,]
            totalArgs<-.makeArgsFromParam(par,
                mainClusterArgs=mainClusterArgs,
                seqArgs=seqArgs,
                subsampleArgs=subsampleArgs,
                clusterFunctionList=clusterFunctionList)
            if(!is.null(random.seed)) {
                set.seed(random.seed)
            }
            reduceMethod<-as.character(par[["reduceMethod"]])
            if(reduceMethod=="none")
                dat<-transformData(x,transFun=transFun, whichAssay=whichAssay)
            else if(isReducedDims(x,reduceMethod))
                dat<-t(reducedDim(x,reduceMethod)[,seq_len(par[["nReducedDims"]])] )
            else if(isFilterStats(x,reduceMethod))
                dat<-transformData( filterData(x, 
                    filterStats=reduceMethod, 
                    percentile=par[["nFilterDims"]]),
                    transFun=transFun, 
                    whichAssay=whichAssay)
            else stop("Internal error: reduceMethod value that not in filtering statistics or reducedDimNames")
            #(Note, computational inefficiency: means reordering each time, even if same filter. But not recalculating filter.)
            distFunction<-totalArgs$distFunction
            
            if(!is.null(distFunction)){
                passInput<- as.logical(gsub(" ","",par["passDistToInput"]))
                passMain<- as.logical(gsub(" ","",par["passDistToMain"]))
                distParamMatrix<-cbind(distFunction=as.character(distFunction),
                    nFilterDims=as.character(par[["nFilterDims"]]),
                    reduceMethod=as.character(par[["reduceMethod"]]))
                dnm<-.createDistNames(distParamMatrix,returnNames=TRUE)
                if(!dnm %in% names(allDist)) stop("internal coding error -- created name that doesn't correspond to calculated distance")
                if(passInput){
                    out<-clusterSingle(inputMatrix=allDist[[dnm]],
                        inputType="diss",
                        sequential=totalArgs$sequential, 
                        subsample=totalArgs$subsample, 
                        reduceMethod="none",
                        mainClusterArgs=totalArgs$mainClusterArgs,
                        subsampleArgs=totalArgs$subsampleArgs, 
                        seqArgs=totalArgs$seqArgs,
                        transFun=function(x){x},
                        checkDiss=FALSE,
                        makeMissingDiss=FALSE,
                        warnings=parameterWarnings)
                    #because a distance, have to convert it back to a CE object.
                    #Using same internal function as clusterSingle
                    return(.convertOutListToCE(xMatrix=dat,
                            clustering=out$clustering,
                            clusterInfo=out$clusterInfo,
                            coClusterMatrix=NULL, 
                            clusterLabel="clusterSingle",
                            sequential=totalArgs$sequential, 
                            subsample=totalArgs$subsample,
                            transFun=function(x){x},
                            saveSubsamplingMatrix=FALSE, existingCE=NULL))
            
                
                }
                else{
                    if(passMain){
                        return(clusterSingle(inputMatrix=dat,
                            inputType="X",
                            sequential=totalArgs$sequential, 
                            subsample=totalArgs$subsample, 
                            reduceMethod="none",
                            mainClusterArgs=c(totalArgs$mainClusterArgs,
                                list(diss=allDist[[dnm]])),
                            subsampleArgs=totalArgs$subsampleArgs, 
                            seqArgs=totalArgs$seqArgs,
                            transFun=function(x){x},
                            checkDiss=FALSE,
                            makeMissingDiss=FALSE,
                            warnings=parameterWarnings))
                    }
                    else{
                        stop("Internal Error: didn't parse distance passing correctly.")
                    }
                    
                }

            }
            else
                return(clusterSingle(inputMatrix=dat, 
                    inputType="X", 
                    sequential=totalArgs$sequential, 
                    subsample=totalArgs$subsample,
                    reduceMethod="none",
                    mainClusterArgs=totalArgs$mainClusterArgs, 
                    subsampleArgs=totalArgs$subsampleArgs, 
                    seqArgs=totalArgs$seqArgs,
                    transFun=function(x){x},
                    checkDiss=FALSE,
                    makeMissingDiss=FALSE,
                    warnings=parameterWarnings))
        }
        
        ## Run this even if run=FALSE so see message.
        if(any(!is.na(param[,"distFunction"]))){
            ##Get the parameters that imply different datasets.
            uniqueId<-.createDistNames(param)
            ## Use to assume only take distances on original data (or filtered version of it). But in fact, it meant previously if needed dissimilarity, would silently calculate it to each call (e.g. if diss->X). So now can get it if makeMissingDiss=TRUE
            if(verbose)
                cat(sprintf("Calculating the %s Requested Distance Matrices needed to run clustering comparisions (if 'distFunction=NA', then needed because of choices of clustering algorithms and 'makeMissingDiss=TRUE').\n",length(uniqueId)))
        }

        if(run){
            ##------------
            ##Calculate distances necessary only once
            ##------------
            if(any(!is.na(param[,"distFunction"]))){
                ##Get the parameters that imply different datasets.
                distParam<-.createDistNames(param,returnNames=FALSE)
                allDist<-lapply(seq_len(nrow(distParam)),function(ii){
                    distFun<-as.character(distParam[ii,"distFunction"])
                    algCheckType<-distParam[ii,"distAlgType"]
                    redM<-as.character(distParam[ii,"reduceMethod"])
                    if(redM=="none"){
                        dat<-transformData(x,
                            transFun=transFun, whichAssay=whichAssay)
                        
                    }
                    else if(isFilterStats(x,redM)){
                        dat<-transformData( filterData(x, 
                            filterStats=redM,
                            percentile=distParam[ii,"nFilterDims"]),
                            transFun=transFun,whichAssay=whichAssay)
                    }
                    else if(makeMissingDiss & any(missDiss)){
                        ### FIXME: shouldn't be any(missDiss) -- should be only the one for this row!
                        dat<-t(reducedDim(x,type=redM))
                    }
                    else stop("Internal error: should have removed any combinations that included a dimensionality reduction and a distance unless due to makeMissingDiss=TRUE")
                    distMat<-.makeDiss(dat, distFunction=distFun, 
                        checkDiss=TRUE, algType=algCheckType)
                    return(distMat)
                })
            
                nms<-.createDistNames(distParam,returnNames=TRUE)
                names(allDist)<-nms
                
                if(verbose) cat("\tdone.\n\n")
            }
            
            ##------------
            ## Run clustering for each row of param
            ##------------
            if(verbose) {
                cat("Running Clustering on Parameter Combinations...\n")
            }
            
            if(ncores>1) {
                out <- mclapply(seq_len(nrow(param)), FUN=paramFun, mc.cores=ncores, ...)
                nErrors <- which(sapply(out, function(x){inherits(x, "try-error")}))
                if(length(nErrors)>0) {
                    stop(length(nErrors)," parameter values (of ",length(out),") hit an error. The first was:\n",out[nErrors[1]])
                }
            } else {
                out <- lapply(seq_len(nrow(param)),FUN=paramFun)
            }
            if(verbose) {
                cat("done.\n")
            }
            clMat <- sapply(out, function(x){primaryCluster(x)})
            
            colnames(clMat) <- unname(cnames)
            pList <- lapply(seq_len(nrow(param)), function(i){
                x <- param[i,]
                names(x) <- colnames(param)
                return(x)})
            clInfo <- mapply(pList, out, FUN=function(x, y){
                c(list(choicesParam=x), clusteringInfo(y))
            }, SIMPLIFY=FALSE)
            
            outval <- ClusterExperiment(x, clusters=clMat,
                                        transformation=transFun,
                                        clusterInfo=clInfo,			                                    clusterTypes="clusterMany",checkTransformAndAssay=FALSE)
        } else{
            if(verbose) {
                cat("Returning Parameter Combinations without running them (to run them choose run=TRUE)\n")
            }
            outval<-list(paramMatrix=param, mainClusterArgs=mainClusterArgs, seqArgs=seqArgs,subsampleArgs=subsampleArgs)
        }
        return(outval)
    }
)

#' @rdname clusterMany
#' @export
setMethod(
    f = "clusterMany",
    signature = signature(x = "ClusterExperiment"),
    definition = function(x, reduceMethod="none", nFilterDims=defaultNDims(x,reduceMethod,type="filterStats"), nReducedDims=defaultNDims(x,reduceMethod,type="reducedDims"),
                          eraseOld=FALSE, ...)
    {
        if(any(c("transFun","isCount") %in% names(list(...))))
            stop("The internally saved transformation function of a ClusterExperiment object must be used when given as input and setting 'transFun' or 'isCount' for a 'ClusterExperiment' is not allowed.")
        outval<-clusterMany(as(x,"SingleCellExperiment"), reduceMethod=reduceMethod, nFilterDims=nFilterDims,
                            nReducedDims=nReducedDims, transFun=transformation(x), ...)
        
        if(is(outval,"ClusterExperiment")) {
            
            #outval<-.addBackSEInfo(newObj=outval,oldObj=x) #added to '.addNewResult'
            ##Check if clusterMany already ran previously
            x<-.updateCurrentWorkflow(x,eraseOld,newTypeToAdd="clusterMany",newLabelToAdd=NULL)
            if(!is.null(x)){
                retval<-.addNewResult(newObj=outval,oldObj=x) #make decisions about what to keep.
                
            }
            else{
                retval<-.addBackSEInfo(newObj=outval,oldObj=x)
            }
            filterStats(retval)<-filterStats(outval)
            reducedDims(retval)<-reducedDims(outval)
            #both above check validity.
            return(retval)
        } else {
            return(outval)
        }
    }
)

#' @rdname clusterMany
#' @export
setMethod(
    f = "clusterMany",
    signature = signature(x = "SummarizedExperiment"),
    definition = function(x, ...){
        clusterMany(as(x,"SingleCellExperiment"),...)
    }
)


#' @export
#' @rdname clusterMany
setMethod(
    f = "clusterMany",
    signature = signature(x = "data.frame"),
    definition = function(x,...){clusterMany(data.matrix(x),...)}
)


.makeArgsFromParam<-function(paramRow,
    mainClusterArgs=NULL,
    seqArgs=NULL,
    subsampleArgs=NULL,
    clusterFunctionList=NULL){

    #The following fixes logical values that got messed up
    #otherwise adds a space before the TRUE and doesn't recognize.
    #well, sometimes. 
    removeSil <- as.logical(gsub(" ","",paramRow["removeSil"]))
    sequential <- as.logical(gsub(" ","",paramRow["sequential"]))
    subsample <- as.logical(gsub(" ","",paramRow["subsample"]))
    findBestK <- as.logical(gsub(" ","",paramRow["findBestK"]))
    mainClusterArgs[["findBestK"]] <- findBestK 
    mainClusterArgs[["removeSil"]] <- removeSil
    
    clusterFunctionName <- as.character(paramRow[["clusterFunction"]])
    distFunction <- if(!is.na(paramRow[["distFunction"]])) as.character(paramRow[["distFunction"]]) else NULL
    if(!is.null(clusterFunctionList)){
        clusterFunction<-clusterFunctionList[[clusterFunctionName]]        
    }
    else clusterFunction <- getBuiltInFunction(clusterFunctionName)
    reduceMethod<-as.character(paramRow[["reduceMethod"]])
    if(!is.na(paramRow[["k"]])){
        if(sequential) {
            seqArgs[["k0"]] <- paramRow[["k"]]
        } else{
            #to be safe, set both in case user set one.
            subsampleArgs[["clusterArgs"]][["k"]] <- paramRow[["k"]]
            mainClusterArgs[["clusterArgs"]][["k"]] <- paramRow[["k"]]
        }
    }
    mainClusterArgs[["clusterArgs"]][["alpha"]] <- paramRow[["alpha"]]
    seqArgs[["beta"]] <- paramRow[["beta"]]
    mainClusterArgs[["minSize"]] <- paramRow[["minSize"]]
    mainClusterArgs[["silCutoff"]] <- paramRow[["silCutoff"]]
    mainClusterArgs[["clusterFunction"]]<-clusterFunction
    if(!"verbose" %in% names(seqArgs)) seqArgs[["verbose"]]<-FALSE
    
    return(list(
        distFunction=distFunction,
        sequential=sequential,
        subsample=subsample,
        mainClusterArgs=mainClusterArgs,
        subsampleArgs=subsampleArgs,
        seqArgs=seqArgs
        ))
}

## Uniform way to create and access names of the list of the list of the distances
.createDistNames<-function(paramMatrix,returnNames=TRUE){
    paramNames<-c("reduceMethod", "nFilterDims",
                     "distFunction")
    if(!returnNames) paramNames<-c(paramNames,"distAlgType")
    if(any(! paramNames %in% colnames(paramMatrix)))
         stop("Internal error: must have following names: ", paste(paramNames,collapse=","))
    distParam<-unique(paramMatrix[, paramNames,drop=FALSE])
    distParam<-distParam[!is.na(distParam[,"distFunction"]), ,drop=FALSE]
    if(returnNames){
        uniqueId<-apply(distParam,1,paste,collapse=",")
        return(uniqueId)
    }
    else return(distParam)
    
}

setGeneric("combineMany", function(x,...) { standardGeneric("combineMany")})
setGeneric("removeUnclustered",function(x,...) {standardGeneric("removeUnclustered")})

#' @name clusterExperiment-deprecated
#' @aliases clusterExperiment-deprecated combineMany combineMany,ANY-method
#' @title Deprecated functions in package \sQuote{clusterExperiment}
#' @description These functions are provided for compatibility with older versions
#'  of \sQuote{clusterExperiment} only, and will be defunct at the next release.
#' @details The following functions are deprecated and will be made defunct; use
#'   the replacement indicated below:
#'  \itemize{
#'    \item{\code{combineMany}: \code{\link{makeConsensus}}}
#'    \item{\code{removeUnclustered}: \code{\link{removeUnassigned}}}
#'  }
#' @export
#' @param x any object
#' @param ... additional arguments
#' @rdname clusterExperiment-deprecated
setMethod(
  f = "combineMany",
  signature = signature(x = "ANY"),
  definition = function(x,...) {
		.Deprecated("makeConsensus") #06/2018 added in BioC 3.8
		makeConsensus(x,...)
	}
	)
setMethod(
  f = "removeUnclustered",
  signature = signature(x = "ANY"),
  definition = function(x,...) {
		.Deprecated("removeUnassigned") #06/2018 added in BioC 3.8
		removeUnassigned(x,...)
  }
)

.depricateArgument<-function(passedArgs,newArgName,oldArgName){
	if(oldArgName %in% names(passedArgs)){
		warning(paste0("the argument name '",oldArgName,"' has been changed to '",newArgName,"'. Please change future calls to reflect this, as the old argument will be depricated in future releases."))
		val<-passedArgs[[oldArgName]]
		passedArgs<-passedArgs[-which(names(passedArgs)==oldArgName)]
		return(list(passedArgs=passedArgs,val=val))
	}
	else return(NULL)
}

#' Find sets of samples that stay together across clusterings
#'
#' Find sets of samples that stay together across clusterings in order to define 
#' a new clustering vector.
#'
#' @name makeConsensus
#' @rdname makeConsensus
#' @param x a matrix with samples on the rows and different clusterings on the
#'   columns or \code{\link{ClusterExperiment}} object.
#' @param clusterFunction the clustering function to use (passed to 
#'   \code{\link{mainClustering}}); currently must be of type '01' and accept as
#'   input matrices of type "cat" (see details of ?ClusterFunction).
#' @param minSize minimum size required for a set of samples to be considered in 
#'  a cluster because of shared clustering, passed to
#'  \code{\link{mainClustering}}
#' @param proportion The proportion of times that two sets of samples should be 
#'  together in order to be grouped into a cluster (if <1, passed to
#'  mainClustering via alpha = 1 - proportion)
#' @param propUnassigned samples with greater than this proportion of assignments
#'  equal to '-1' are assigned a '-1' cluster value as a last step (only if
#'  proportion < 1)
#' @param ... arguments to be passed on to the method for signature 
#'  \code{matrix,missing}.
#' @param whenUnassign (provided for back compatibility with previous versions).
#'   Must be one of "before" or "after", indicating at what point are samples
#'   with a proportion of assignments of -1 greater than \code{propUnassigned}
#'   forced to have a '-1' value. If "before", then these samples are removed
#'   and not used for clustering. If "after", these samples are included in the
#'   clustering step, but then the cluster values they receive are assigned a
#'   '-1. These choices may result in different clusterings, because if these
#'   samples are included in the clustering (i.e. \code{whenUnassign="after"},
#'   then these samples may affect the cluster assignments of other samples. The
#'   default is currently "before", but previous to version 2.5.4, there was no
#'   such option and the code internally set to "after", so for reproducibility
#'   with older results, users may need to set this option.
#' @inheritParams clusterSingle
#' @inheritParams clusterMany
#' @param clusterArgs list of arguments to be passed to the call to 
#'  \code{\link{mainClustering}} that is used to cluster the proportion 
#'  overlap between samples. 
#' @inheritParams getClusterIndex
#' @details This function was previously called \code{combineMany} (versions <=
#'   2.0.0). \code{combineMany} is still available, but is considered defunct
#'   and users should update their code accordingly.
#' @details The function tries to find a consensus cluster across many different 
#'  clusterings of the same samples. It does so by creating a \code{nSamples} x 
#'  \code{nSamples} matrix of the percentage of co-occurance of each sample and 
#'  then calling mainClustering to cluster the co-occurance matrix. The function
#'  assumes that '-1' labels indicate clusters that are not assigned to a 
#'  cluster. Co-occurance with the unassigned cluster is treated differently 
#'  than other clusters. The percent co-occurance is taken only with respect to 
#'  those clusterings where both samples were assigned. Then samples with more 
#'  than \code{propUnassigned} values that are '-1' across all of the 
#'  clusterings are assigned a '-1' regardless of their cluster assignment.
#'@details The method calls \code{\link{mainClustering}} on the proportion
#'  matrix with \code{clusterFunction} as the 01 clustering algorithm,
#'  \code{alpha=1-proportion}, \code{minSize=minSize}, and
#'  \code{evalClusterMethod=c("average")}. See help of 
#'  \code{\link{mainClustering}} for more details.
#'@return If x is a matrix, a list with values \itemize{ 
#'  \item{\code{clustering}}{ vector of cluster assignments, with "-1" implying 
#'  unassigned}
#'  
#'  \item{\code{percentageShared}}{ a nSample x nSample matrix of the percent 
#'  co-occurance across clusters used to find the final clusters. Percentage is 
#'  out of those not '-1'} \item{\code{noUnassignedCorrection}{ a vector of 
#'  cluster assignments before samples were converted to '-1' because had 
#'  >\code{propUnassigned} '-1' values (i.e. the direct output of the 
#'  \code{mainClustering} output.)}} }
#'  
#' @return If x is a \code{\link{ClusterExperiment}}, a
#'  \code{\link{ClusterExperiment}} object, with an added clustering of
#'  clusterTypes equal to \code{makeConsensus} and the \code{percentageShared}
#'  matrix stored in the \code{coClustering} slot.
#'
#' @examples
#' \dontrun{
#' data(simData)
#'
#' cl <- clusterMany(simData,nReducedDims=c(5,10,50),  reduceMethod="PCA",
#' clusterFunction="pam", ks=2:4, findBestK=c(FALSE), removeSil=TRUE,
#' makeMissingDiss=TRUE, subsample=FALSE)
#'
#' #make names shorter for plotting
#' clMat <- clusterMatrix(cl)
#' colnames(clMat) <- gsub("TRUE", "T", colnames(clMat))
#' colnames(clMat) <- gsub("FALSE", "F", colnames(clMat))
#' colnames(clMat) <- gsub("k=NA,", "", colnames(clMat))
#'
#' #require 100% agreement -- very strict
#' clCommon100 <- makeConsensus(clMat, proportion=1, minSize=10)
#'
#' #require 70% agreement based on clustering of overlap
#' clCommon70 <- makeConsensus(clMat, proportion=0.7, minSize=10)
#'
#' oldpar <- par(no.readonly = TRUE)
#' par(mar=c(1.1, 12.1, 1.1, 1.1))
#' plotClusters(cbind("70%Similarity"=clCommon70, clMat,
#' "100%Similarity"=clCommon100), axisLine=-2)
#'
#' #method for ClusterExperiment object
#' clCommon <- makeConsensus(cl, whichClusters="workflow", proportion=0.7,
#' minSize=10)
#' plotClusters(clCommon)
#' par(oldpar)
#' }
#' @aliases makeConsensus,matrix-method
#' @export
setMethod(
    f = "makeConsensus",
    signature = signature(x = "matrix"),
    definition = function(x, proportion,
                          clusterFunction="hierarchical01",
                          minSize=5, propUnassigned=.5, 
                          whenUnassign=c("before","after"),
                          clusterArgs=NULL) {
        whenUnassign<-match.arg(whenUnassign)
        if(proportion >1 || proportion <0) stop("Invalid value for the 'proportion' parameter")
        if(propUnassigned >1 || propUnassigned <0) stop("Invalid value for the 'propUnassigned' parameter")
        N<-nrow(x)
        
        ## Now define as unassigned any samples with >= propUnassigned '-1' values in x
        ## Doing >= means that all -1 will be in -1 cluster, no matter what
        ## (protects against chance that they get assigned to a cluster)
        whUnassigned <- which(apply(x, 1, function(dat){
            sum(dat<0)/length(dat)>=propUnassigned}))
        if(length(whUnassigned)>0 && whenUnassign=="before") {           
            temp<-rep(-1,length=N)
            names(temp)<-rownames(x)
            x<-x[-whUnassigned, ,drop=FALSE]
        }
        if(length(whUnassigned)==N && whenUnassign=="after"){
            temp<-rep(-1,length=N)
            names(temp)<-rownames(x)
        }
        
        #Skip clustering altogether if just going to assign all to -1
        if(length(whUnassigned)!=N){
            if(proportion == 1) {
                #have to repeat from mainClustering because didn't
                if(!is.numeric(minSize) || minSize<0) 
                    stop("Invalid value for the 'minSize' parameter in determining the minimum number of samples required in a cluster.")
                else minSize<-round(minSize) #incase not integer.
                cl<-.uniqueCluster(inputMatrix=t(x), minSize=minSize)
            } else{
                if(is.character(clusterFunction) || is(clusterFunction,"ClusterFunction")){
                    typeAlg <- algorithmType(clusterFunction)
                    inputType<-inputType(clusterFunction)
                } 
                else{
                    stop("clusterFunction must be either a builtin clusterFunction name or a ClusterFunction object")
                }
                if(typeAlg!="01") {
                    stop("makeConsensus is only implemented for '01' type clustering functions (see ?ClusterFunction)")
                }
                if(!"cat" %in% inputType)
                    stop("makeConsensus is only implemented for clustering functions that permit inputType 'cat'.")
                #overwrites alpha if given
                clusterArgs[["alpha"]]<-1-proportion
                if(!"evalClusterMethod" %in% names(clusterArgs) &&
                    clusterFunction=="hierarchical01"){
                    clusterArgs<-c(clusterArgs,
                        list(evalClusterMethod=c("average")))
                }
                cl <- mainClustering(inputMatrix=t(x),
                                     inputType="cat",
                                     clusterFunction=clusterFunction,
                                     minSize=minSize, format="vector",
                                     clusterArgs=clusterArgs)
            
                if(is.character(cl)) {
                    stop("coding error -- mainClustering should return numeric vector")
                }
            }
            if(whenUnassign!="never" && length(whUnassigned)>0){
                if(whenUnassign=="after") cl[whUnassigned]<- -1
                if(whenUnassign=="before"){
                    #put back in the -1 values
                    temp[-whUnassigned]<-cl
                    cl<-temp
                }                    
            }
        }
        else{
            cl<-temp
        }
        return(cl)
    }

)

#' @rdname makeConsensus
#' @export
#' @param clusterLabel a string used to describe the type of clustering. By
#'   default it is equal to "makeConsensus", to indicate that this clustering is
#'   the result of a call to makeConsensus. However, a more informative label can
#'   be set (see vignette).
setMethod(
    f = "makeConsensus",
    signature = signature(x = "ClusterExperiment"),
    definition = function(x, whichClusters, eraseOld=FALSE,clusterLabel="makeConsensus",...){
        if(missing(whichClusters)){
            whichClusters <- getClusterIndex(x, 
                                whichClusters="clusterMany", 
                                noMatch="silentlyRemove")
            if(length(whichClusters)>0){
                .mynote("no clusters specified to combine, using results from clusterMany")
            }
            else{
                stop("no clusters specified to combine, please specify.")
            }
        }
        else{
            whichClusters <-getClusterIndex(x,
                whichClusters=whichClusters,noMatch="throwError")
        }
        outlist <- makeConsensus(clusterMatrix(x,whichClusters), ...)
        newObj <- ClusterExperiment(x, outlist,
                        transformation=transformation(x),
                        clusterTypes="makeConsensus",
                        checkTransformAndAssay=FALSE)
        #add "c" to name of cluster
        newObj<-.addPrefixToClusterNames(newObj,prefix="c",whCluster=1)
        clusterLabels(newObj) <- clusterLabel
        
        # ## FIXME: the function no longer returns this object.
        #  need to the index of clusterings used for the makeConsensus?
        # Or save it only if plot it (i.e. calculate it)?
        # if(!is.null(outlist$percentageShared)) {
        #   coClustering(newObj) <- Matrix::Matrix(outlist$percentageShared,sparse=TRUE)
        # }
        ##Check if pipeline already ran previously and if so increase
        x<-.updateCurrentWorkflow(x,eraseOld,
            newTypeToAdd="makeConsensus", newLabelToAdd=clusterLabel)
        if(!is.null(x)){
            coClustering(x) <- whichClusters
            retval<-.addNewResult(newObj=newObj, oldObj=x) #make decisions about what to keep.
        }
        else retval<-.addBackSEInfo(newObj=newObj,oldObj=x)
        return(retval)
    }
)

#Update here if change workflow values. Also defines the order of them.
.workflowValues<-c("final","mergeClusters","makeConsensus","clusterMany")

#' @title Methods for workflow clusters
#'
#' @description The main workflow of the package is made of
#'   \code{\link{clusterMany}}, \code{\link{makeConsensus}}, and
#'   \code{\link{mergeClusters}}. The clusterings from these functions (and not
#'   those obtained in a different way) can be obtained with the functions
#'   documented here.
#' @param x a \code{\link{ClusterExperiment}} object.
#' @param iteration numeric. Which iteration of the workflow should be used.
#' @return \code{workflowClusters} returns a matrix consisting of the
#'   appropriate columns of the \code{clusterMatrix} slot.
#' @aliases workflowClusters workflowClusterTable workflowClusterDetails
#'   workflowClusters,ClusterExperiment-method
#' @name workflowClusters
#' @export
#' @examples
#' \dontrun{
#' data(simData)
#'
#' cl <- clusterMany(simData,nReducedDims=c(5,10,50),  reduceMethod="PCA",
#' clusterFunction="pam", ks=2:4, findBestK=c(FALSE), removeSil=TRUE,
#' subsample=FALSE, makeMissingDiss=TRUE)
#'
#' clCommon <- makeConsensus(cl, whichClusters="workflow", proportion=0.7,
#' minSize=10)
#'
#' clCommon <- makeDendrogram(clCommon)
#'
#' clMerged <- mergeClusters(clCommon,mergeMethod="adjP", DEMethod="limma")
#'
#' head(workflowClusters(clMerged))
#' workflowClusterDetails(clMerged)
#' workflowClusterTable(clMerged)
#' }
setMethod(
  f = "workflowClusters",
  signature = signature("ClusterExperiment"),
  definition = function(x,iteration=0) {
    ppIndex<-workflowClusterDetails(x)
    if(is.na(iteration)) iteration<-unique(ppIndex[,"iteration"])
    if(!is.null(ppIndex)){
      whIteration<-which(ppIndex[,"iteration"]%in%iteration)
      if(length(whIteration)>0){
        index<-ppIndex[whIteration,"index"]
        return(clusterMatrix(x)[,index,drop=FALSE])
      }
      else return(NULL)
    }
    else return(NULL)
  }
)


#' @rdname workflowClusters
#' @return \code{workflowClusterDetails} returns a \code{data.frame} with some
#'   details on the clusterings, such as the type (e.g., `clusterMany`,
#'   `makeConsensus`) and iteration.
#' @export
setMethod(
  f = "workflowClusterDetails",
  signature = signature("ClusterExperiment"),
  definition = function(x) {

    if(length(clusterTypes(x))!=NCOL(clusterMatrix(x))) stop("Invalid ClusterExperiment object")
    #check if old iterations already exist; note assumes won't have previous iteration unless have current one.
    existingOld<-lapply(.workflowValues,function(ch){
      regex<-paste(ch,".",sep="")
      grep(regex,clusterTypes(x))

    })
    st<-strsplit(clusterTypes(x)[unlist(existingOld)],"[.]")
    oldValues<-data.frame(index=unlist(existingOld),type=sapply(st,.subset2,1),iteration=as.numeric(sapply(st,.subset2,2)),stringsAsFactors=FALSE)

    wh<-which(clusterTypes(x) %in% .workflowValues) #current iteration
    if(length(wh)>0){
      existingValues<-data.frame(index=wh,type=clusterTypes(x)[wh], iteration=0,stringsAsFactors=FALSE) #0 indicates current iteration
      if(nrow(oldValues)>0) existingValues<-rbind(oldValues,existingValues)
    }
    else{
      if(nrow(oldValues)>0) existingValues<-oldValues
      else   return(NULL)
    }
    if(nrow(existingValues)>0){ #just to make sure
        existingValues$label=clusterLabels(x)[existingValues$index]
        existingValues<-existingValues[order(existingValues$index),]
    }
    rownames(existingValues)<-NULL
    return(existingValues)

  }
)
#' @rdname workflowClusters
#' @return \code{workflowClusterTable} returns a table of how many of the
#'   clusterings belong to each of the following possible values: `final`,
#'   `mergeClusters`, `makeConsensus` and `clusterMany`.
#' @export
setMethod(
  f = "workflowClusterTable",
  signature = signature("ClusterExperiment"),
  definition = function(x){
    ppIndex<-workflowClusterDetails(x)
    table(Type=factor(ppIndex[,"type"],levels=.workflowValues),Iteration=factor(ppIndex[,"iteration"]))
  }
)


#' @rdname workflowClusters
#' @inheritParams clusterMany
#' @inheritParams getClusterIndex
#' @return \code{setToCurrent} returns a \code{ClusterExperiment} object where
#'   the indicated cluster of \code{whichCluster} has been set to the most
#'   current iteration in the workflow. Pre-existing clusters are appropriately
#'   updated.
#' @export
#' @aliases setToCurrent
setMethod(
  f = "setToCurrent",
  signature = signature("ClusterExperiment"),
  definition = function(x,whichCluster,eraseOld=FALSE){
    whCl<-getSingleClusterIndex(x,whichCluster=whichCluster)
    type<-strsplit(clusterTypes(x)[whCl],"[.]")[[1]][1]
    if(!type %in% .workflowValues[-1]) stop("Input cluster is not a workflow cluster. Must be of clustType: ",paste(.workflowValues[-1],sep=","))
			#not sure here if should have argument newLabelToAdd
    newX<-.updateCurrentWorkflow(x,eraseOld=eraseOld,newTypeToAdd=type,newLabelToAdd=NULL)
		
    clusterTypes(newX)[whCl]<-type
    primaryClusterIndex(newX)<-whCl
    
    if(strsplit(clusterLabels(x)[whCl],"[.]")[[1]][1]==type){
      clusterLabels(newX)[whCl]<-type
    }
    return(newX)
  }
)

#' @rdname workflowClusters
#' @param clusterLabel optional string value to give to cluster set to be "final"
#' @return \code{setToFinal} returns a \code{ClusterExperiment} object where the
#'   indicated cluster of \code{whichCluster} has clusterType set to "final".
#'   The primaryClusterIndex is also set to this cluster, and the clusterLabel,
#'   if given.
#' @export
#' @aliases setToFinal
setMethod(
  f = "setToFinal",
  signature = signature("ClusterExperiment"),
  definition = function(x,whichCluster,clusterLabel){
    whCl<-getSingleClusterIndex(x,whichCluster=whichCluster)
    clusterTypes(x)[whCl]<-"final"
    if(!missing(clusterLabel)) clusterLabels(x)[whCl]<-clusterLabel
    primaryClusterIndex(x)<-whCl
    return(x)
  }
)
#change current workflow to old iteration
# add number to it if eraseOld=FALSE
# delete ALL workflow if eraseOld=TRUE (not just the current iteration)
.updateCurrentWorkflow<-function(object,eraseOld,newTypeToAdd,newLabelToAdd){
  
  ppIndex<-workflowClusterDetails(object)
  origLabels<-clusterLabels(object)
  origTypes<-clusterTypes(object)
  if(!any(newTypeToAdd %in% .workflowValues[-1])) stop("error in internal coding: newTypeToAdd must be one of .workflowValues. Contact mantainer.")
  whNew<-max(match(newTypeToAdd, .workflowValues))
  downstreamType<-.workflowValues[2:whNew]
  if(!is.null(ppIndex)){ #there are pre-existing workflow results
    curr<-ppIndex[ppIndex[,"iteration"]==0,]
    if(any(curr[,"type"] %in% downstreamType) || any(ppIndex[,"iteration"]!=0)){
      if(eraseOld){
        #removes all past iterations, not just current, except for current iteration that upstream of new one
        whRm<- union(curr[curr[,"type"] %in% downstreamType, "index"],ppIndex[ppIndex[,"iteration"]!=0,"index"])
        if(length(whRm)==nClusterings(object)) return(NULL)
        else object<-removeClusterings(object,whRm) 
      }
      else{
        #otherwise, only current downstream ones that exist get updated number
        if(any(curr[,"type"] %in% downstreamType)){
          whDown<-which(ppIndex[,"type"] %in% downstreamType)
          maxDownstream<-max(ppIndex[whDown,"iteration"])
          newIteration<-maxDownstream+1
          #check if any upstream have iteration > maxDownstream
          whUp<-which(!ppIndex[,"type"] %in% downstreamType)
          if(length(whUp)>0){
            maxUpstream<-max(ppIndex[whUp,"iteration"])
            if(maxUpstream>newIteration) newIteration<-maxUpstream
          }
          whFix<-curr[curr[,"type"] %in% downstreamType, "index"]
          
          updateCluster<-origTypes
          updateCluster[whFix]<-paste(updateCluster[whFix],newIteration,sep=".")
          clusterTypes(object)<-updateCluster
          if(any(origLabels[whFix]%in%.workflowValues) | any(origTypes[whFix]=="clusterMany")){ 
            #only change those labels that haven't been manually fixed by the user
            updateLabel<-origLabels
            if(any(origLabels[whFix]%in%.workflowValues)){
              whUnedited<-which(updateLabel[whFix]%in%.workflowValues)
              updateLabel[whFix[whUnedited]]<-paste(updateLabel[whFix[whUnedited]],newIteration,sep=".")
            }
            if(any(origTypes[whFix]=="clusterMany")){
              #always update the clusterMany labels because otherwise get repetitive. Mainly for test checks...
              #should always be different set than the ones above.
              whCM<-which(origTypes[whFix]=="clusterMany")
              updateLabel[whFix[whCM]]<-paste(updateLabel[whFix[whCM]],newIteration,sep=".")
            }
            clusterLabels(object)<-updateLabel
          }
          else if(!is.null(newLabelToAdd) && any(newLabelToAdd %in% origLabels[whFix])){
            updateLabel<-origLabels
            whDuplicated<-which(updateLabel[whFix]%in%newLabelToAdd)
            updateLabel[whFix[whDuplicated]]<-paste(updateLabel[whFix[whDuplicated]],newIteration,sep=".")
            clusterLabels(object)<-updateLabel
            
          }
          
        }
      }
    }
  }
  object<-.unnameClusterSlots(object) #just to make sure didn't have names on labels or types
  ch<-.checkClusterTypes(object)
  if(!is.logical(ch)) stop(ch)
  ch<-.checkClusterLabels(object)
  if(!is.logical(ch)) stop(ch)
  return(object)
}

#' @name reduceFunctions
#' @title Filtering statistics and Dimensionality Reduction Functions
#' @description Functions for calculating and manipulating either filtering
#'   statistics, stored in rowData, or the dimensionality reduction results,
#'   stored in reducedDims.
#' @aliases reduceFunctions makeFilterStats
#'   makeFilterStats,SummarizedExperiment-method
#' @param filterStats character vector of statistics to calculate. Must be one
#'   of the character values given by \code{listBuildInFilterStats()}.
#' @param filterNames if given, defines the names that will be assigned to the 
#'  filtering statistics in the \code{rowData} of the object. If missing, will be 
#'  just the value of \code{filterStats} argument
#' @return \code{makeFilterStats} returns a
#'   \code{\link[SummarizedExperiment]{SummarizedExperiment}} object with the
#'   requested filtering statistics will be added to the \code{DataFrame} in the
#'   \code{rowData} slot and given names corresponding to the \code{filterStats}
#'   values. Warning: the function will overwrite existing columns in
#'   \code{rowData} with the same name. Columns in the \code{rowData} slot with
#'   different names should not be affected.
#' @inheritParams transformData
#' @rdname reduceFunctions
#'
#' @examples
#' data(simData)
#' listBuiltInFilterStats()
#' scf<-makeFilterStats(simData,filterStats=c("var","mad"))
#' scf
#' scfFiltered<-filterData(scf,filterStats="mad",percentile=10)
#' scfFiltered
#' assay(scfFiltered)[1:10,1:10]
#' @export
setMethod(
  f = "makeFilterStats",
  signature = "SummarizedExperiment",
  definition = function(object,filterStats=listBuiltInFilterStats(),transFun=NULL,isCount=FALSE,filterNames=NULL,whichAssay=1)
  {
    
    ###################
    ##Check user inputs
    ###################
    #check valid options for reducedDims
    filterStats<-unique(filterStats)
    if(!all(filterStats %in% listBuiltInFilterStats())){
      stop("Not all of the filterStats given are valid. Must be one of listBuiltInFilterStats().")
    }
    if(!is.null(filterNames)){
      if(length(unique(filterStats))!=length(filterStats)) stop("cannot set filterNames if filterStats not unique")
      if(!is.character(filterNames) || length(filterNames)!=length(filterStats)) stop("invalid values of filterNames")
      filterStatsOrig<-filterStats #preserve the original order of names
    }
    ###################
    ##Clean up data:
    ###################
    #transform data
    x<-transformData(object,transFun=transFun,isCount=isCount,whichAssay=whichAssay)
    
    ###################
    ##Do loop over filterStats values:
    ###################
    if('abscv' %in% filterStats){
      #origfilterStats<-filterStats
      #if abscv, add var and mean since calculating anyway and make abscv last one
      filterStats<-unique(c(filterStats,"var","mean"))
      doCV<-TRUE
      whCV<-grep('abscv',filterStats)
      filterStats<-c(filterStats[-whCV])
    }
    else doCV<-FALSE
    ### Problem here in hdf5
    filterStatData<-sapply(filterStats,function(statName){
      f<-.matchToStats(x)[[statName]]
      f(x)
    })
    if(doCV){
      filterStatData<-cbind(filterStatData, "abscv"=sqrt(filterStatData[,"var"])/abs(filterStatData[,"mean"]))
      #filterStatData<-filterStatData[,origfilterStats] #put it in order, though user shouldn't depend on it.
    }
    if(!is.null(filterNames)){
      m<-match(filterStatsOrig,colnames(filterStatData))
      if(any(is.na(m))) stop("error in assigning given filterNames to filterStats")
      colnames(filterStatData)[m]<-filterNames			
    }
    filterStats(object)<-filterStatData #should leave in place existing ones, update conflicting ones, and add new ones!
    return(object)
    
  }
)
#' @rdname reduceFunctions
#' @export
setMethod(
  f = "makeFilterStats",
  signature = "matrixOrHDF5",
  definition = function(object,...)
  {
    makeFilterStats(SummarizedExperiment(object),...)
  }
)

#' @rdname reduceFunctions
#' @export
#' @param whichClusterIgnoreUnassigned indicates clustering that should be used
#'   to filter out unassigned samples from the calculations. If \code{NULL} no
#'   filtering of samples will be done. See details for more information.
#' @details \code{whichClusterIgnoreUnassigned} is only an option when applied
#'   to a \code{ClusterExperiment} classs and indicates that the filtering
#'   statistics should be calculated based on samples that are unassigned by the
#'   designated clustering. The name given to the filter in this case is of the
#'   form \code{<filterStats>_<clusterLabel>}, i.e. the clustering label of the
#'   clustering is appended to the standard name for the filtering statistic.
#'
setMethod(
  f = "makeFilterStats",
  signature = "ClusterExperiment",
  definition = function(object,whichClusterIgnoreUnassigned=NULL,filterStats=listBuiltInFilterStats(),...)
  {
    if(any(c("transFun","isCount") %in% names(list(...)))) 
      stop("The internally saved transformation function of a ClusterExperiment object must be used when given as input and setting 'transFun' or 'isCount' for a 'ClusterExperiment' is not allowed.")  
    filterStats<-unique(filterStats)
    if(!is.null(whichClusterIgnoreUnassigned)){
      whCluster<-getClusterIndex(object,whichClusterIgnoreUnassigned,noMatch="silentlyRemove")
      if(length(whCluster)>1) warning("'whichClusterIgnoreUnassigned' corresponds to multiple clusterings. Ignoring argument")
      else if(length(whCluster)==0) warning("'whichClusterIgnoreUnassigned' does not correspond to a clustering. Ignoring argument.")
      else{
        #give new names to filters to indicate based on clustering.
        newNames<-.makeClusterFilterStats(filterStats,clusterLabels(object)[whCluster])
        whDo<-which(!isFilterStats(object,newNames))
        if(length(whDo)>0){
          whAssigned<-which(clusterMatrix(object)[,whCluster]>0)
          if(length(whAssigned)>0){
            out<-makeFilterStats(object[,whAssigned],filterStats=filterStats[whDo],filterNames=newNames[whDo],...)
          }
          else 
            stop("All samples are unassigned for clustering", clusterLabels(object)[whCluster])
          
        }
        else return(object) #already calculated all of the requested values
      }
    }
    else{
      out<-makeFilterStats(as(object,"SingleCellExperiment"),filterStats=filterStats,transFun=transformation(object),...)
    }
    #	
    filterStats(object)<-filterStats(out)
    return(object)
  }
)


#' @rdname reduceFunctions
#' @export
listBuiltInFilterStats<-function(){c('var', 'abscv', 'mad','mean','iqr','median')}

#' @importFrom matrixStats rowVars rowMeans2 rowMads rowMedians rowIQRs
setMethod(
  f = ".matchToStats",
  signature = "matrix",
  definition = function(object){
    S4Vectors::SimpleList(
      'var'=matrixStats::rowVars,
      'mad'=matrixStats::rowMads,
      'mean'=matrixStats::rowMeans2,
      'iqr'=matrixStats::rowIQRs,
      'median'=matrixStats::rowMedians)
  }
)
  
### need to go back to this and check what are the best options for these functions. 
#' @importFrom stats IQR
#' @importFrom S4Vectors SimpleList
setMethod(
  f = ".matchToStats",
  signature = "DelayedArray",
  definition = function(object){
    S4Vectors::SimpleList(
      'var'=function(x){apply(x,1,var)},
      'mad'=function(x){apply(x,1,mad)},
      'mean'=DelayedArray::rowMeans,
      'iqr'=function(x){apply(x,1,stats::IQR)},
      'median'=function(x){apply(x,1,median)})
    
  })
#' @rdname reduceFunctions
#' @aliases filterData
#' @param cutoff numeric. A value at which to filter the rows (genes) for the
#'   test statistic
#' @param percentile numeric. Either a number between 0,1 indicating what
#'   percentage of the rows (genes) to keep or an integer value indicated the
#'   number of rows (genes) to keep
#' @param absolute whether to take the absolute value of the filter statistic
#' @param keepLarge logical whether to keep rows (genes) with large values of
#'   the test statistic or small values of the test statistic.
#' @details Note that \code{filterData} returns a SingleCellExperiment object.
#'   To get the actual data out use either assay or \code{\link{transformData}}
#'   if transformed data is desired.
#' @return \code{filterData} returns a SingleCellExperiment object with the rows
#'   (genes) removed based on filters
#' @export
#' @importFrom stats quantile
setMethod( 
  f="filterData","SummarizedExperiment",
  function(object,filterStats,cutoff,percentile, absolute=FALSE,keepLarge=TRUE, whichAssay=1){
    stat<-filterStats(object,filterStats)
    if(!is.null(dim(stat))){
      if(NCOL(stat)==1) stat<-stat[,1]
      else stop("Attempting to filtering on more than one filter statistic")
    }
    if(absolute) stat<-abs(stat)
    if(missing(cutoff) & missing(percentile)) stop("must provide one of cutoff or percentile")
    if(!missing(cutoff) & !missing(percentile)) stop("can only provide one of cutoff or percentile")
    if(!missing(cutoff)){
      whKeep<- if(keepLarge) which(stat>cutoff) else which(cutoff > stat)
    }
    if(!missing(percentile)){
      if(0<percentile & percentile <1){
        quantile<- quantile(stat,probs=if(keepLarge) percentile else 1-percentile,na.rm=TRUE)
        whKeep<-if(keepLarge) which(stat>quantile) else which(stat<quantile)
      }
      else{
        if(is.na(percentile) || percentile>=1){
          if(is.na(percentile) || percentile>NROW(object)){
            warning("the number of most features requested after filtering is either missing or larger than the number of features. Will not do any filtering")
            whKeep<-seq_len(NROW(object))
          }
          else whKeep<- order(stat,decreasing=ifelse(keepLarge,TRUE,FALSE),na.last=NA)[seq_len(percentile)]
        }
        else stop("Invalid value for percentile. Must be either between 0,1 or a positive integer number to keep")
      }
    }
    object[whKeep,]
    
  }
)	


#' @rdname reduceFunctions
#' @return \code{filterNames} returns a vector of the columns of the rowData
#'   that are considered valid filtering statistics. Currently any numeric
#'   column in rowData is a valid filtering statistic.
#' @aliases filterNames
#' @export
setMethod( "filterNames","SummarizedExperiment",function(object){
  checkValid<-TRUE
  if(!checkValid || ncol(rowData(object))==0) colnames(rowData(object))
  else{
    whValid<- sapply(rowData(object),is.numeric)
    # #for now, do not allow NA values for valid filters.
    # whNa<-which(apply(rowData,2,anyNA))
    # whValid[whNA]<-FALSE
    return(names(rowData(object))[whValid])
    
  }
})	

#' @title Cluster distance matrix from subsampling
#'   
#' @description Given input data, this function will try to find the clusters
#'   based on the given ClusterFunction object.
#' @name mainClustering
#'   
#' @param orderBy how to order the cluster (either by size or by maximum alpha 
#'   value). If orderBy="size" the numbering of the clusters are reordered by 
#'   the size of the cluster, instead of by the internal ordering of the 
#'   \code{clusterFUN} defined in the \code{ClusterFunction} object (an internal
#'   ordering is only possible if slot \code{outputType} of the
#'   \code{ClusterFunction} is \code{"list"}).
#' @param minSize the minimum number of samples in a cluster. Clusters found 
#'   below this size will be discarded and samples in the cluster will be given 
#'   a cluster assignment of "-1" to indicate that they were not clustered.
#' @param format whether to return a list of indices in a cluster or a vector of
#'   clustering assignments. List is mainly for compatibility with sequential 
#'   part.
#' @param clusterArgs arguments to be passed directly to the \code{clusterFUN}
#'   slot of the \code{ClusterFunction} object
#' @param warnings logical as to whether should give warning if arguments given
#'   that don't match clustering choices given. Otherwise, inapplicable 
#'   arguments will be ignored without warning.
#' @param returnData logical as to whether to return the \code{diss} or \code{x}
#'   matrix in the output. If \code{FALSE} only the clustering vector is
#'   returned.
#' @param ... arguments passed to the post-processing steps of the clustering.
#'   The available post-processing arguments for a \code{ClusterFunction} object
#'   depend on it's algorithm type and can be found by calling
#'   \code{getPostProcessingArgs}. See details below for documentation.
#' @inheritParams subsampleClustering
#' @inheritParams clusterSingle
#' @details \code{mainClustering} is not meant to be called by the user. It is
#'   only an exported function so as to be able to clearly document the
#'   arguments for \code{mainClustering} which can be passed via the argument
#'   \code{mainClusterArgs} in functions like \code{\link{clusterSingle}} and
#'   \code{\link{clusterMany}}.
#'   
#' @return If \code{returnData=FALSE}, mainClustering returns a vector of cluster assignments (if
#'   format="vector") or a list of indices for each cluster (if format="list").
#'   Clusters less than minSize are removed. If \code{returnData=TRUE}, then mainClustering returns a list
#' \itemize{
#' \item{results}{The clusterings of each sample.}
#' \item{inputMatrix}{The input matrix given to argument \code{inputMatrix}. Useful if input is result of subsampling, in which case input is the set of clusterings found over subsampling.}
#' }
#'
#' @examples
#' data(simData)
#' cl1<-mainClustering(inputMatrix=simData, inputType="X", 
#'     clusterFunction="pam",clusterArgs=list(k=3))
#' #supply a dissimilarity, use algorithm type "01"
#' diss<-as.matrix(dist(t(simData),method="manhattan"))
#' cl2<-mainClustering(diss, inputType="diss", clusterFunction="hierarchical01",
#'     clusterArgs=list(alpha=.1))
#' cl3<-mainClustering(inputMatrix=diss, inputType="diss", clusterFunction="pam",
#'     clusterArgs=list(k=3))
#' 
#' # run hierarchical method for finding blocks, with method of evaluating
#' # coherence of block set to evalClusterMethod="average", and the hierarchical
#' # clustering using single linkage:
#' # (clustering function requires type 'diss'),
#' clustSubHier <- mainClustering(diss, inputType="diss",
#'     clusterFunction="hierarchical01", minSize=5,
#'     clusterArgs=list(alpha=0.1,evalClusterMethod="average", method="single"))
#'
#' #post-process results of pam -- must pass diss for silhouette calculation
#' clustSubPamK <- mainClustering(simData, inputType="X", clusterFunction="pam", 
#'     silCutoff=0, minSize=5, diss=diss, removeSil=TRUE, clusterArgs=list(k=3))
#' clustSubPamBestK <- mainClustering(simData, inputType="X", clusterFunction="pam", silCutoff=0,
#'     minSize=5, diss=diss, removeSil=TRUE, findBestK=TRUE, kRange=2:10)
#'
#' # note that passing the wrong arguments for an algorithm results in warnings
#' # (which can be turned off with warnings=FALSE)
#' clustSubTight_test <- mainClustering(diss, inputType="diss", 
#'    clusterFunction="tight", 
#'    clusterArgs=list(alpha=0.1), minSize=5, removeSil=TRUE)
#' clustSubTight_test2 <- mainClustering(diss, inputType="diss",
#'    clusterFunction="tight",
#'    clusterArgs=list(alpha=0.1,evalClusterMethod="average"))
#' @rdname mainClustering
#' @aliases mainClustering,character-method
#' @export
setMethod(
    f = "mainClustering",
    signature = signature(clusterFunction = "character"),
    definition = function(clusterFunction,...){
        mainClustering(getBuiltInFunction(clusterFunction),...)
        
    }
)
#' @rdname mainClustering
#' @export
setMethod(
    f = "mainClustering",
    signature = signature(clusterFunction = "ClusterFunction"),
    definition=function(clusterFunction,inputMatrix, inputType,
                        clusterArgs=NULL,
                        minSize=1, 
                        orderBy=c("size","best"),
                        format=c("vector","list"),
                        returnData=FALSE,
												warnings=TRUE,...){
        if(missing(inputType)) stop("Internal error: inputType was not passed to mainClustering step")
        orderBy<-match.arg(orderBy)
        format<-match.arg(format)
        #######################
        ### Check arguments.
        #######################
        postProcessArgs<-list(...)
        # remove those based added by checkArgs
        # from clusterMany/clusterSingle/seqCluster
		if("doKPostProcess" %in% names(postProcessArgs)) 
            postProcessArgs <- postProcessArgs[-grep("doKPostProcess", names(postProcessArgs))]
        mainArgs<-c(list(clusterFunction=clusterFunction,
                         clusterArgs=clusterArgs,
                         minSize=minSize, orderBy=orderBy,
						 extraArguments=names(postProcessArgs),
                         format=format),postProcessArgs)
        checkOut<-.checkArgs(inputType=inputType, main=TRUE, subsample=FALSE, sequential=FALSE, mainClusterArgs=mainArgs, subsampleArgs=NULL,warn=warnings)		
        if(is.character(checkOut)) stop(checkOut)
		mainClusterArgs<-checkOut$mainClusterArgs
		inputType<-mainClusterArgs[["inputType"]]
        doKPostProcess<-mainClusterArgs[["doKPostProcess"]]
        clusterFunction=mainClusterArgs[["clusterFunction"]]
        clusterArgs=mainClusterArgs[["clusterArgs"]]
        minSize=mainClusterArgs[["minSize"]] 
        orderBy=mainClusterArgs[["orderBy"]]
        format=mainClusterArgs[["format"]]
        postProcessArgs=mainClusterArgs[mainClusterArgs[["extraArguments"]]]
        
        #######################
        ####Run clustering:
        #######################
        N <- dim(inputMatrix)[2]
        argsClusterList<-c(clusterArgs, list("cluster.only"=TRUE,
					inputMatrix=inputMatrix,inputType=inputType))
        # if(doKPostProcess & inputType!="diss") stop("Cannot do the post-processing (findBestK or remove due to silhouette score) without input Matrix that is dissimilarity.")
        if(doKPostProcess) {
            res<-do.call(".postProcessClusterK", c(list(clusterFunction=clusterFunction, clusterArgs=argsClusterList, N=N, orderBy=orderBy), postProcessArgs))
            ###Note to self: .postProcessClusterK returns clusters in list form.
        }
        else{
            res<-do.call(clusterFunction@clusterFUN,argsClusterList)
        }
        
        #######################
        #Now format into desired output, order
        #######################
        ## FIXME: need to get rid of this, unless needed/requested (though it is requested by sequential)
        #this is perhaps not efficient. For now will do this, then consider going back and only converting when, where needed.
        if(clusterFunction@outputType=="vector" & !doKPostProcess){
            res<-.clusterVectorToList(res)
        }
        clusterSize<-sapply(res, length)
        if(length(res)>0) res <- res[clusterSize>=minSize]
        if(length(res)!=0 & orderBy=="size"){ #i.e. there exist clusters found that passed minSize
            clusterSize<-sapply(res, length) #redo because dropped small clusters earlier
            res <- res[order(clusterSize,decreasing=TRUE)]
        }
        if(format=="vector"){
            res<-.clusterListToVector(res,N)
			#FIXME can't I take colnames of a diss too?
            names(res)<-if(inputType!="diss") colnames(inputMatrix) else rownames(inputMatrix)
        }
        if(!returnData) return(res)
        else return(list(results=res,inputMatrix=inputMatrix))
    }
)



#' @rdname mainClustering
#' @aliases getPostProcessingArgs
#' @details Post-processing Arguments: For post-processing the clustering,
#'   currently only type 'K' algorithms have a defined post-processing.
#'   Specifically
#' \itemize{
#'  \item{"findBestK"}{logical, whether should find best K based on average
#'   silhouette width (only used if clusterFunction of type "K").}
#'  \item{"kRange"}{vector of integers to try for k values if findBestK=TRUE. If
#'  \code{k} is given in \code{clusterArgs}, then default is k-2 to k+20,
#'  subject to those values being greater than 2; if not the default is
#'  \code{2:20}. Note that default values depend on the input k, so running for
#'  different choices of k and findBestK=TRUE can give different answers unless
#'  kRange is set to be the same.}
#'  \item{"removeSil"}{logical as to whether remove the assignment of a sample
#'  to a cluster when the sample's silhouette value is less than
#'  \code{silCutoff}}
#'  \item{"silCutoff"}{Cutoff on the minimum silhouette width to be included in
#'   cluster (only used if removeSil=TRUE).}
#' }
#' @export
setMethod(
    f = "getPostProcessingArgs",
    signature = c("ClusterFunction"),
    definition = function(clusterFunction) {
        switch(algorithmType(clusterFunction),"01"=.argsPostCluster01,"K"=.argsPostClusterK)
    }
)

### Provide the post-processing args that can be provided by user
.argsPostCluster01<-c("")
.argsPostClusterK<-c("findBestK","kRange","removeSil","silCutoff","diss")

#' @importFrom cluster silhouette
.postProcessClusterK<-function(clusterFunction,findBestK=FALSE,  kRange,removeSil=FALSE,silCutoff=0,diss=NULL,clusterArgs,N,orderBy)
{
    if(is.null(diss)){
        if(clusterArgs[["inputType"]]!="diss") stop("Internal error: ran post-processing without a dissimilarity matrix being provided")
    }
    k<-clusterArgs[["k"]]
    if(!findBestK && is.null(k)) stop("If findBestK=FALSE, must provide k")
    if(!is.null(k)) clusterArgs<-clusterArgs[-which(names(clusterArgs)=="k")]
    if(findBestK){
        if(missing(kRange)){
            if(!is.null(k)) kRange<-(k-2):(k+20)
            else kRange<-2:20
        }
        if(any(kRange<2)){
            kRange<-kRange[kRange>=2]
            if(length(kRange)==0) stop("Undefined values for kRange; must be greater than or equal to 2")
        }
        ks<-kRange 
    }
    else ks<-k
    if(any(ks>= N)) ks<-ks[ks<N]
    clusters<-lapply(ks,FUN=function(currk){
        cl<-do.call(clusterFunction@clusterFUN,c(list(k=currk),clusterArgs))
        if(clusterFunction@outputType=="list") cl<-.clusterListToVector(cl,N=N)
        return(cl)
    })
    if(is.null(diss)) diss<-clusterArgs[["inputMatrix"]]
    silClusters<-lapply(clusters,function(cl){
        cluster::silhouette(cl,
            dmatrix=diss
        )
    })
    if(length(ks)>1){
        whichBest<-which.max(sapply(silClusters, mean))
        finalCluster<-clusters[[whichBest]]
        sil<-silClusters[[whichBest]][,"sil_width"]
    }
    else{
        finalCluster<-clusters[[1]]
        sil<-silClusters[[1]][,"sil_width"]
    }
    if(removeSil){
        cl<-as.numeric(sil>silCutoff)
        cl[cl==0]<- -1
        cl[cl>0]<-finalCluster[cl>0]
        sil[cl == -1] <- -Inf #make the -1 cluster the last one in order
    }
    else{
        cl<-finalCluster
    }
    
    #make list of indices and put in order of silhouette width (of positive)
    clList<-tapply(seq_along(cl),cl,function(x){x},simplify=FALSE)
    if(orderBy=="best"){
        clAveWidth<-tapply(sil,cl,mean,na.rm=TRUE)
        clList[order(clAveWidth,decreasing=TRUE)]
    }
    #remove -1 group
    if(removeSil){
        whNotAssign<-which(sapply(clList,function(x){all(cl[x]== -1)}))
        if(length(whNotAssign)>1) stop("Coding error in removing unclustered samples")
        if(length(whNotAssign)>0) clList<-clList[-whNotAssign]
    }    
    return(clList)
    
}

#' @useDynLib clusterExperiment, .registration = TRUE
#' @importFrom Rcpp sourceCpp
NULL

#' Cluster subsamples of the data
#'
#' Given input data, this function will subsample the samples, cluster the
#' subsamples, and return a \code{n x n} matrix with the probability of
#' co-occurance.
#' @name subsampleClustering
#' @param clusterFunction a \code{\link{ClusterFunction}} object that defines
#'   the clustering routine. See \code{\link{ClusterFunction}} for required
#'   format of user-defined clustering routines. User can also give a character
#'   value to the argument \code{clusterFunction} to indicate the use of
#'   clustering routines provided in package. Type
#'   \code{\link{listBuiltInFunctions}} at command prompt to see the built-in
#'   clustering routines. If \code{clusterFunction} is missing, the default is
#'   set to "pam".
#' @param clusterArgs a list of parameter arguments to be passed to the function
#'   defined in the \code{clusterFunction} slot of the \code{ClusterFunction}
#'   object. For any given \code{\link{ClusterFunction}} object, use function
#'   \code{\link{requiredArgs}} to get a list of required arguments for the
#'   object.
#' @param resamp.num the number of subsamples to draw.
#' @param samp.p the proportion of samples to sample for each subsample.
#' @param classifyMethod method for determining which samples should be used in
#'   calculating the co-occurance matrix. "All"= all samples, "OutOfSample"=
#'   those not subsampled, and "InSample"=those in the subsample.  See details
#'   for explanation.
#' @param ncores integer giving the number of cores. If ncores>1, mclapply will
#'   be called.
#' @param ... arguments passed to mclapply (if ncores>1).
#' @inheritParams mainClustering
#' @inheritParams clusterSingle
#'
#' @details \code{subsampleClustering} is not usually called directly by the
#'   user. It is only an exported function so as to be able to clearly document
#'   the arguments for \code{subsampleClustering}  which can be passed via the
#'   argument \code{subsampleArgs} in functions like \code{\link{clusterSingle}}
#'   and \code{\link{clusterMany}}.
#' @details \code{requiredArgs:} The choice of "All" or "OutOfSample" for
#'   \code{requiredArgs} require the classification of arbitrary samples not
#'   originally in the clustering to clusters; this is done via the classifyFUN
#'   provided in the \code{\link{ClusterFunction}} object. If the
#'   \code{\link{ClusterFunction}} object does not have such a function to
#'   define how to classify into a cluster samples not in the subsample that
#'   created the clustering then \code{classifyMethod} must be
#'   \code{"InSample"}. Note that if "All" is chosen, all samples will be
#'   classified into clusters via the classifyFUN, not just those that are
#'   out-of-sample; this could result in different assignments to clusters for
#'   the in-sample samples than their original assignment by the clustering
#'   depending on the classification function. If you do not choose 'All',it is
#'   possible to get NAs in resulting S matrix (particularly if when not enough
#'   subsamples are taken) which can cause errors if you then pass the resulting
#'   D=1-S matrix to \code{\link{mainClustering}}. For this reason the default is
#'   "All".
#' @return A \code{n x n} matrix of co-occurances, i.e. a symmetric matrix with
#'   [i,j] entries equal to the percentage of subsamples where the ith and jth
#'   sample were clustered into the same cluster. The percentage is only out of
#'   those subsamples where the ith and jth samples were both assigned to a
#'   clustering. If \code{classifyMethod=="All"}, this is all subsamples for all
#'   i,j pairs. But if \code{classifyMethod=="InSample"} or
#'   \code{classifyMethod=="OutOfSample"}, then the percentage is only taken on
#'   those subsamples where the ith and jth sample were both in or out of
#'   sample, respectively, relative to the subsample.
#'
#' @examples
#'\dontrun{
#' #takes a bit of time, not run on checks:
#' data(simData)
#' coOccur <- subsampleClustering( inputMatrix=simData, inputType="X",
#' clusterFunction="kmeans",
#' clusterArgs=list(k=3,nstart=10), resamp.n=100, samp.p=0.7)
#'
#' #visualize the resulting co-occurance matrix
#' plotHeatmap(coOccur)
#'}
#' @aliases subsampleClustering,character-method
#' @export
setMethod(
    f = "subsampleClustering",
    signature = signature(clusterFunction = "character"),
    definition = function(clusterFunction,...){
        subsampleClustering(getBuiltInFunction(clusterFunction),...)
        
    }
)

# #' @rdname subsampleClustering
# #' @export
# setMethod(
# f = "subsampleClustering",
# signature = signature(clusterFunction = "missing"),
# definition = function(clusterFunction,...){
# 	subsampleClustering(clusterFunction="pam",...)
# }
# )

#' @rdname subsampleClustering
#' @export
setMethod(
    f = "subsampleClustering",
    signature = signature(clusterFunction = "ClusterFunction"),
    definition=function(clusterFunction, inputMatrix,inputType, clusterArgs=NULL,
                        classifyMethod=c("All","InSample","OutOfSample"),
                        resamp.num = 100, samp.p = 0.7,
						ncores=1,warnings=TRUE,... )
    {
        
        classifyMethod<-match.arg(classifyMethod)
        ###########################
        ######## CHECKS
        ###########################
        if(missing(inputType)) stop("Internal error: inputType was not passed to subsampling step")
        moreArgs<-list(...)
        subsampleArgs<-c(list(clusterFunction=clusterFunction, clusterArgs=clusterArgs, classifyMethod=classifyMethod),moreArgs)
        checkOut<-.checkArgs(inputType=inputType, 
                        main=FALSE, subsample=TRUE, sequential=FALSE,
                        mainClusterArgs=NULL,
                        subsampleArgs=subsampleArgs, 
                        warn=warnings)		
        if(is.character(checkOut)) stop(checkOut)
        else{
            subsampleArgs<-checkOut$subsampleArgs
        }
        classifyMethod<-subsampleArgs[["classifyMethod"]]
        clusterFunction<-subsampleArgs[["clusterFunction"]]
        clusterArgs<-subsampleArgs[["clusterArgs"]]
        inputType<-subsampleArgs[["inputType"]]
        
        #-----
        # Basic parameters, subsamples
        #-----
        N <- dim(inputMatrix)[2]
        subSize <- round(samp.p * N)
        idx<-replicate(resamp.num,sample(seq_len(N),size=subSize))
        #each column a set of indices for the subsample.
        
        ###########################
        # Function that calls the clustering for each subsample
        # Called over a loop (lapply or mclapply)
        ###########################
        perSample<-function(ids){
            ##----
            ##Cluster subsample
            ##----
            #if doing InSample, do cluster.only because will be more efficient, e.g. pam and kmeans.
            argsClusterList <- list(inputType=inputType,
                "cluster.only"=(classifyMethod=="InSample"))
            if(inputType=="diss") 
                argsClusterList<-c(argsClusterList, 
                    list(inputMatrix=inputMatrix[ids,ids,drop=FALSE]))
            else
                argsClusterList<-c(argsClusterList, 
                    list(inputMatrix=inputMatrix[,ids,drop=FALSE]))
            result <- do.call(clusterFunction@clusterFUN,
                              c(argsClusterList,clusterArgs))
            
            ##----
            ##Classify subsample
            ##----
            if(classifyMethod=="All"){
                ##FIXME make this actually only recluster the out-of-sample, but otherwise use the clustering results from the method!
				classX <- do.call(clusterFunction@classifyFUN, list(inputMatrix=inputMatrix,inputType=inputType,clusterResult=result))
            }
            if(classifyMethod=="OutOfSample"){
                classElse <- do.call(clusterFunction@classifyFUN,
                                     list(clusterResult=result,
										 inputMatrix=if(inputType!="diss") inputMatrix[,-ids,drop=FALSE] else inputMatrix[-ids,-ids,drop=FALSE]))
                
                classX <- rep(NA,N)
                classX[-ids] <- classElse
            }
            if(classifyMethod=="InSample"){
                classX <- rep(NA,N)
                if(is.list(result)){
                    if(clusterFunction@outputType=="list"){
                        resultVec <- .clusterListToVector(result,N=length(ids))
                        classX[ids] <- resultVec
                    } else {
                        stop("The clusterFunction given to subsampleClustering returns a list when cluster.only=FALSE but does not have a named element 'clustering' nor outputType='list'")
                    }
                    #			  }
                } else{
                    classX[ids]<-result
                }
            }
            
            #classX is length N
            #classX has NA if method does not classify all of the data.
            return(classX)
        }
        
        if(ncores==1){
            DList<-apply(idx,2,perSample)
        }
        else{
            DList<-parallel::mclapply(seq_len(ncol(idx)), function(nc){ perSample(idx[,nc]) }, mc.cores=ncores,...)
            DList <- simplify2array(DList)
        }
        idnames<-colnames(inputMatrix)
        # DList is a NxB matrix
        return(DList)
    })

#' @name ClusterExperiment-methods
#' @title Helper methods for the ClusterExperiment class
#'
#' @description This is a collection of helper methods for the ClusterExperiment
#'   class.
#' @param ... For \code{addToColData}, arguments passed to
#'   \code{colDataClusters}.
#' @param value The value to be substituted in the corresponding slot. See the
#'   slot descriptions in \code{\link{ClusterExperiment}} for details on what
#'   objects may be passed to these functions.
#' @rdname ClusterExperiment-methods
#' @aliases show show,ClusterExperiment-method
#' @examples
#' # load data:
#' data(rsecFluidigm)
#' show(rsecFluidigm)
#' #Number of clusterings
#' nClusterings(rsecFluidigm)
#' # Number of clusters per clustering
#' nClusters(rsecFluidigm)
#' # Number of features/samples
#' nSamples(rsecFluidigm)
#' nFeatures(rsecFluidigm)
#' # retrieve all clustering assignments
#' # (either as cluster ids, cluster names or cluster colors)
#' head(clusterMatrix(rsecFluidigm)[,1:5])
#' head(clusterMatrixNamed(rsecFluidigm)[,1:5])
#' head(clusterMatrixColors(rsecFluidigm)[,1:5])
#' # clustering Types/Labels
#' clusterTypes(rsecFluidigm)
#' clusterLabels(rsecFluidigm)
#' # Add a clustering assignment to the colData of the object
#' # (useful if working with function that relies on colData)
#' colData(rsecFluidigm)
#' test<-addToColData(rsecFluidigm,whichCluster="primary")
#' colData(test)
#' @export
setMethod(
  f = "show",
  signature = "ClusterExperiment",
  definition = function(object) {
    cat("class:", class(object), "\n")
    cat("dim:", dim(object), "\n")
    cat("reducedDimNames:",if(anyValidReducedDims(object)) reducedDimNames(object) else "no reduced dims stored","\n")
    cat("filterStats:",if(anyValidFilterStats(object)) filterNames(object) else "no valid filtering stats stored","\n")
    cat("-----------\n")
    cat("Primary cluster type:", clusterTypes(object)[primaryClusterIndex(object)],"\n")
    cat("Primary cluster label:", clusterLabels(object)[primaryClusterIndex(object)],"\n")
    cat("Table of clusters (of primary clustering):")
    print(table(primaryClusterNamed(object)))
    cat("Total number of clusterings:", NCOL(clusterMatrix(object)),"\n")
    if(!is.na(dendroClusterIndex(object)) ) cat("Dendrogram run on '",clusterLabels(object)[dendroClusterIndex(object)],"' (cluster index: ", dendroClusterIndex(object),")\n",sep="") else cat("No dendrogram present\n")
    cat("-----------\n")
    cat("Workflow progress:\n")
    typeTab<-names(table(clusterTypes(object)))
    cat("clusterMany run?",if("clusterMany" %in% typeTab) "Yes" else "No","\n")
    cat("makeConsensus run?",if("makeConsensus" %in% typeTab) "Yes" else "No","\n")
    cat("makeDendrogram run?",if(!is.null(object@dendro_samples) & !is.null(object@dendro_clusters) ) "Yes" else "No","\n")
    cat("mergeClusters run?",if("mergeClusters" %in% typeTab) "Yes" else "No","\n")
  }
)

#' @rdname ClusterExperiment-methods
#' @return \code{transformation} prints the function used to transform the data
#' prior to clustering.
#' @export
#' @aliases transformation
setMethod(
  f = "transformation",
  signature = "ClusterExperiment",
  definition = function(x) {
    return(x@transformation)
  }
)

#' @rdname ClusterExperiment-methods
#' @export
#' @details Note that redefining the transformation function via
#'   \code{transformation(x)<-} will check the validity of the transformation on
#'   the data assay. If the assay is large, this may be time consuming. Consider
#'   using a call to ClusterExperiment, which has the option as to whether to
#'   check the validity of the transformation.
#' @aliases transformation<-
setReplaceMethod(
  f = "transformation",
  signature = signature("ClusterExperiment", "function"),
  definition = function(object, value) {
	checkValidity=TRUE
    object@transformation <- value
    if(checkValidity){
		ch<-.checkTransform(object)
    	if(ch) return(object) else stop(ch)
	}
	else return(object)
  }
)

#' @rdname ClusterExperiment-methods
#' @return \code{nClusterings} returns the number of clusterings (i.e., ncol of
#' clusterMatrix).
#' @export
#' @aliases nClusterings
setMethod(
  f = "nClusterings",
  signature = "ClusterExperiment",
  definition = function(x){
    return(NCOL(x@clusterMatrix))
  }
)


#' @rdname ClusterExperiment-methods
#' @return \code{nClusters} returns the number of clusters per clustering
#' @param ignoreUnassigned logical. If true, ignore the clusters with -1 or -2
#'   assignments in calculating the number of clusters per clustering.
#' @export
#' @aliases nClusters
setMethod(
  f = "nClusters",
  signature = "ClusterExperiment",
  definition = function(x,ignoreUnassigned=TRUE){
	  if(ignoreUnassigned){
		  return(apply(clusterMatrix(x),2,function(x){length(unique(x[x>0]))}))
	  }
	  else return(apply(clusterMatrix(x),2,function(x){length(unique(x))}))
  }
)

#' @rdname ClusterExperiment-methods
#' @return \code{nFeatures} returns the number of features (same as `nrow`).
#' @aliases nFeatures
#' @export
setMethod(
  f = "nFeatures",
  signature =  "ClusterExperiment",
  definition = function(x){
    return(NROW(assay(x)))
  }
)

#' @rdname ClusterExperiment-methods
#' @return \code{nSamples} returns the number of samples (same as `ncol`).
#' @aliases nSamples
#' @export
setMethod(
  f = "nSamples",
  signature = "ClusterExperiment",
  definition = function(x){
    return(NCOL(assay(x)))
  }
)

#' @rdname ClusterExperiment-methods
#' @return \code{clusterMatrixNamed} returns a matrix with cluster labels.
#' @export
#' @aliases clusterMatrixNamed
#' @param x,object a ClusterExperiment object.
setMethod(
  f = "clusterMatrixNamed",
  signature = "ClusterExperiment",
  definition = function(x, whichClusters="all") {
    convertClusterLegend(x,output="matrixNames",whichClusters=whichClusters)
  }
)

#' @rdname ClusterExperiment-methods
#' @return \code{clusterMatrixColors} returns the matrix with all the
#'   clusterings, using the internally stored colors for each cluster
#' @export
#' @aliases clusterMatrixColors
setMethod(
  f = "clusterMatrixColors",
  signature = c("ClusterExperiment"),
  definition = function(x,whichClusters="all") {
    convertClusterLegend(x,output="matrixColors",whichClusters=whichClusters)
  }
)

#' @rdname ClusterExperiment-methods
#' @return \code{clusterMatrix} returns the matrix with all the clusterings.
#' @export
#' @aliases clusterMatrix
setMethod(
  f = "clusterMatrix",
  signature = c("ClusterExperiment"),
  definition = function(x,whichClusters) {
	  if(missing(whichClusters)) mat<-x@clusterMatrix #note that 'all' changes the order of the clusterings...
	  else{
		  wh<-getClusterIndex(object=x,whichClusters=whichClusters,noMatch="silentlyRemove")
		  mat<-x@clusterMatrix[,wh,drop=FALSE]	  	
	  }
	  rownames(mat)<-colnames(x)
      return(mat)
  }
)


#' @rdname ClusterExperiment-methods
#' @return \code{primaryCluster} returns the primary clustering (as numeric).
#' @export
#' @aliases primaryCluster
setMethod(
  f = "primaryCluster",
  signature = "ClusterExperiment",
  definition = function(x) {
    return(x@clusterMatrix[,primaryClusterIndex(x)])
  }
)

#' @rdname ClusterExperiment-methods
#' @return \code{primaryClusterIndex} returns/sets the primary clustering index
#' (i.e., which column of clusterMatrix corresponds to the primary clustering).
#' @export
#' @aliases primaryClusterIndex
setMethod(
  f = "primaryClusterIndex",
  signature = "ClusterExperiment",
  definition = function(x) {
    return(x@primaryIndex)
  }
)

#' @rdname ClusterExperiment-methods
#' @return \code{primaryClusterIndex} returns/sets the primary clustering index
#' (i.e., which column of clusterMatrix corresponds to the primary clustering).
#' @export
#' @aliases primaryClusterLabel
setMethod(
  f = "primaryClusterLabel",
  signature = "ClusterExperiment",
  definition = function(x) {
    return(clusterLabels(x)[primaryClusterIndex(x)])
  }
)
#' @rdname ClusterExperiment-methods
#' @return \code{primaryClusterNamed} returns the primary cluster (using cluster
#' labels).
#' @export
#' @aliases primaryClusterNamed
setMethod(
  f = "primaryClusterNamed",
  signature = "ClusterExperiment",
  definition = function(x) {
    as.vector(clusterMatrixNamed(x,whichCluster="primary"))
  })

#' @rdname ClusterExperiment-methods
#' @return \code{primaryClusterIndex} returns/sets the primary clustering index
#' (i.e., which column of clusterMatrix corresponds to the primary clustering).
#' @export
#' @aliases primaryClusterType
setMethod(
  f = "primaryClusterType",
  signature = "ClusterExperiment",
  definition = function(x) {
    return(clusterTypes(x)[primaryClusterIndex(x)])
  }
)




#' @rdname ClusterExperiment-methods
#' @export
#' @aliases primaryClusterIndex<-
setReplaceMethod(
  f = "primaryClusterIndex",
  signature = signature("ClusterExperiment", "numeric"),
  definition = function(object, value) {
    object@primaryIndex <- value
    ch<-.checkPrimaryIndex(object)
    if(is.logical(ch) && ch) return(object) else stop(ch)
  }
)
#' @rdname ClusterExperiment-methods
#' @return \code{dendroClusterIndex} returns/sets the clustering index 
#' of the clusters used to create dendrogram
#' (i.e., which column of clusterMatrix corresponds to the clustering).
#' @export
#' @aliases dendroClusterIndex
setMethod(
  f = "dendroClusterIndex",
  signature = "ClusterExperiment",
  definition = function(x) {
    return(x@dendro_index)
  }
)



#' @rdname ClusterExperiment-methods
#' @return \code{coClustering} returns/sets the co-clustering matrix.
#' @export
#' @aliases coClustering
setMethod(
  f = "coClustering",
  signature = "ClusterExperiment",
  definition = function(x) {
    return(x@coClustering)
  }
)
#' @rdname ClusterExperiment-methods
#' @export
#' @aliases coClustering<-
setReplaceMethod(
  f = "coClustering",
  signature = signature(object="ClusterExperiment", value="matrix"),
  definition = function(object, value) {
    #This appears to detect if matrix is symmetric without any further effort
    object@coClustering <- Matrix::Matrix(value,sparse=TRUE)        
    ch<-.checkCoClustering(object)
    if(is.logical(ch) && ch) return(object) else stop(ch)
  }
)
#' @rdname ClusterExperiment-methods
#' @export
#' @aliases coClustering<-
setReplaceMethod(
  f = "coClustering",
  signature = signature(object="ClusterExperiment", value="dsCMatrix"),
  definition = function(object, value) {
    object@coClustering <- value
    ch<-.checkCoClustering(object)
    if(is.logical(ch) && ch) return(object) else stop(ch)
  }
)

#' @rdname ClusterExperiment-methods
#' @export
#' @aliases coClustering<-
setReplaceMethod(
  f = "coClustering",
  signature = signature(object="ClusterExperiment", value="numeric"),
  definition = function(object, value) {
    object@coClustering <- value
    ch<-.checkCoClustering(object)
    if(is.logical(ch) && ch) return(object) else stop(ch)
  }
)




#' @rdname ClusterExperiment-methods
#' @return \code{clusterTypes} returns/sets the clusterTypes slot.
#' @export
#' @aliases clusterTypes
setMethod(
  f = "clusterTypes",
  signature = "ClusterExperiment",
  definition = function(x) {
    out<-x@clusterTypes
    #names(out)<-clusterLabels(x)
    return(out)
  }
)

#' @rdname ClusterExperiment-methods
#' @return \code{clusteringInfo} returns the clusterInfo slot.
#' @aliases clusteringInfo
#' @export
setMethod(
  f = "clusteringInfo",
  signature = "ClusterExperiment",
  definition = function(x) {
    out<-x@clusterInfo
    names(out)<-clusterLabels(x)
    return(out)
  }
)


#' @rdname ClusterExperiment-methods
#' @return \code{clusterLabels} returns/sets the column names of the
#'   clusterMatrix slot.
#' @export
#' @aliases clusterLabels
setMethod(
  f = "clusterLabels",
  signature = signature(x = "ClusterExperiment"),
  definition = function(x){
    labels<-colnames(clusterMatrix(x))
    if(is.null(labels)) cat("No labels found for clusterings\n")
    return(labels)

  }
)
#' @export
#' @rdname ClusterExperiment-methods
#' @aliases clusterLabels<-
setReplaceMethod( 
  f = "clusterLabels",
  signature = signature(object="ClusterExperiment", value="character"),
  definition = function(object, value) {
    if(length(value)!=NCOL(clusterMatrix(object))) stop("value must be a vector of length equal to NCOL(clusterMatrix(object)):",NCOL(clusterMatrix(object)))
    colnames(object@clusterMatrix) <- value
    ch<-.checkClusterLabels(object)
    if(is.logical(ch) && ch) return(object) else stop(ch)
  }
)
#' @rdname ClusterExperiment-methods
#' @return \code{clusterLegend} returns/sets the clusterLegend slot.
#' @export
#' @aliases clusterLegend
setMethod(
    f = "clusterLegend",
    signature = "ClusterExperiment",
    definition = function(x) {
      out<-x@clusterLegend
      names(out)<-clusterLabels(x)
      return(out)
    }
)

#' @rdname ClusterExperiment-methods
#' @export
#' @aliases clusterLegend<-
setReplaceMethod( 
  f = "clusterLegend",
  signature = signature(object="ClusterExperiment", value="list"),
  definition = function(object, value) {
    object@clusterLegend<-unname(value)
    ch<-.checkClusterLegend(object)
    if(is.logical(ch) && ch) return(object) else stop(ch)
  }
)


.checkMatch<-function(clMat,value,matchTo){
	if(is.null(names(value))){
		clVals<-clMat[,"clusterIds"]
		if(length(value)== nrow(clMat)) names(value)<-clVals
		else if(length(value)==length(clVals[clVals>0])) names(value)<-clMat[clVals>0,"clusterIds"]
		else stop("length of argument 'value' not equal to number of clusters, nor does it have names to identify it to 'clusterIds' of this clustering.")
			matchTo<-"clusterIds"
	} 
	if(matchTo=="name"){
		if(!all(names(value) %in% clMat[,"name"])) stop("'value' must be vector with names that matches the 'name' column of the requested clusterLegend")
			m<-match(names(value),clMat[,"name"])
	}
	else{
		if(!all(names(value) %in% clMat[,"clusterIds"])) stop("'value' must be vector with names that matches the 'clusterIds' column of the requested clusterLegend")
			m<-match(names(value),clMat[,"clusterIds"])
		
	}
	return(m)
}


#' @rdname ClusterExperiment-methods
#' @return \code{orderSamples} returns/sets the orderSamples slot.
#' @export
#' @aliases orderSamples
setMethod(
  f = "orderSamples",
  signature = "ClusterExperiment",
  definition = function(x) {
    return(x@orderSamples)
  }
)
#' @rdname ClusterExperiment-methods
#' @export
#' @aliases orderSamples<-
setReplaceMethod( 
  f = "orderSamples",
  signature = signature(object="ClusterExperiment", value="numeric"),
  definition = function(object, value) {
    object@orderSamples<-value
    ch<-.checkOrderSamples(object) 
    if(is.logical(ch) && ch) return(object) else stop(ch)
    
  }
)

#' @rdname ClusterExperiment-methods
#' @export
#' @aliases clusterTypes<-
setReplaceMethod( 
  f = "clusterTypes",
  signature = signature(object="ClusterExperiment", value="character"),
  definition = function(object,value) {
    object@clusterTypes<-value
    object<-.unnameClusterSlots(object)
    ch<-.checkClusterTypes(object)
    if(is.logical(ch) && ch) return(object) else stop(ch)
    
  }
)


#' @rdname ClusterExperiment-methods
#' @export
#' @inheritParams plotClustersTable
#' @aliases addToColData
#' @return \code{addToColData} returns a \code{ClusterExperiment} object
#' with the clusterings in clusterMatrix slot added to the \code{colData} slot
setMethod(
	f="addToColData",
	signature="ClusterExperiment",
	definition=function(object,...){
		colData(object)<-colDataClusters(object,...)
		return(object)
	})
#' @rdname ClusterExperiment-methods
#' @export
#' @param makeFactor logical for \code{colDataClusters}. If TRUE the clustering
#'   will be added to the \code{colData} slot as a factor. If FALSE, the
#'   clustering will be added to the \code{colData} slot as a character vector
#'   if \code{useNames=TRUE} and as a numeric vector if \code{useNames=FALSE}.
#' @aliases colDataClusters
#' @return \code{colDataClusters} returns a \code{DataFrame} object
#' that has the clusterings in clusterMatrix slot added to the 
#' \code{DataFrame} in the \code{colData} slot
#' @importFrom S4Vectors DataFrame
setMethod(
	f="colDataClusters",
	signature="ClusterExperiment",
	definition=function(object,whichClusters="primary",useNames=TRUE,makeFactor=TRUE,...){
		if(useNames){
			cm<-clusterMatrixNamed(object,whichClusters=whichClusters)
		}
		else{
			cm<-clusterMatrix(object,whichClusters=whichClusters)			
		}
		if(makeFactor){
			cnames<-colnames(cm)
			cm<-do.call("DataFrame",c(lapply(seq_len(ncol(cm)),function(i){factor(cm[,i])}),list(check.names=FALSE))) 
			colnames(cm)<-cnames		
		}
		else cm<-DataFrame(cm,check.names=FALSE)
		return(cbind(colData(object),cm))
	})
	
#' @title Change assigned names or colors of clusters
#' @description Change the assigned names or colors of the clusters in a
#'   clustering stored in the clusterLegend slot of the object.
#' @name renameClusters
#' @return \code{renameClusters} changes the names assigned to clusters within a
#'   clustering
#' @export
#' @inheritParams subset
#' @inheritParams ClusterExperiment-methods
#' @aliases renameClusters,ClusterExperiment,character-method
#' @examples
#' #create CE object
#' data(simData)
#' cl1 <- clusterSingle(simData, subsample=FALSE,
#' sequential=FALSE, mainClusterArgs=list(clusterArgs=list(k=3), 
#' clusterFunction="pam"))
#' #Give names to the clusters
#' clusterLegend(cl1)
#' cl1<-renameClusters(cl1, c("1"="A","2"="B","3"="C"), matchTo="clusterIds")
#' clusterLegend(cl1)
#' # Change name of single one
#' cl1<-renameClusters(cl1, c("1"="D"), matchTo="clusterIds")
#' clusterLegend(cl1)
#' # Match to existing name, rather than clusterId
#' cl1<-renameClusters(cl1, c("B"="N"), matchTo="name")
#' clusterLegend(cl1)
#' # Change colors in similar way
#' cl1<-recolorClusters(cl1, c("N"="red"),matchTo=c("name"))
#' clusterLegend(cl1)
setMethod( 
  f = "renameClusters",
  signature = signature(object="ClusterExperiment", value="character"),
  definition = function(object, value,whichCluster="primary",matchTo=c("name","clusterIds")) {
		matchTo<-match.arg(matchTo)
		whCl<-getSingleClusterIndex(object,whichCluster)
		mat<-clusterLegend(object)[[whCl]]
		m<-.checkMatch(clMat=mat,value=value,matchTo=matchTo)
		mat[m,"name"]<-value
		clusterLegend(object)[[whCl]]<-mat
	
    ch<-.checkClusterLegend(object)
    if(is.logical(ch) && ch) return(object) else stop(ch)
  }
)

#' @rdname renameClusters
#' @return \code{recolorClusters} changes the colors assigned to clusters within
#'   a clustering
#' @export
#' @aliases recolorClusters
setMethod( 
  f = "recolorClusters",
  signature = signature(object="ClusterExperiment", value="character"),
  definition = function(object, value,whichCluster="primary",matchTo=c("name","clusterIds")) {
		matchTo<-match.arg(matchTo)
		whCl<-getSingleClusterIndex(object,whichCluster)
		mat<-clusterLegend(object)[[whCl]]
		m<-.checkMatch(clMat=mat,value=value,matchTo=matchTo)
		mat[m,"color"]<-value
		clusterLegend(object)[[whCl]]<-mat
	
    ch<-.checkClusterLegend(object)
    if(is.logical(ch) && ch) return(object) else stop(ch)
  }
)

#' Visualize cluster assignments across multiple clusterings
#'
#' Align multiple clusterings of the same set of samples and provide a
#' color-coded plot of their shared cluster assignments
#' @name plotClusters
#' @docType methods
#' @param object A matrix of with each column corresponding to a clustering
#'   and each row a sample or a \code{\link{ClusterExperiment}} object. If a
#'   matrix, the function will plot the clusterings in order of this matrix, and
#'   their order influences the plot greatly.
#' @param orderSamples A predefined order in which the samples will be plotted.
#'   Otherwise the order will be found internally by aligning the clusters
#'   (assuming \code{input="clusters"})
#' @param colData If \code{clusters} is a matrix, \code{colData} gives a 
#'   matrix of additional cluster/sample data on the samples to be plotted with 
#'   the clusterings given in clusters. Values in \code{colData} will be 
#'   added to the end (bottom) of the plot. NAs in the \code{colData} matrix
#'   will trigger an error. If \code{clusters} is a \code{ClusterExperiment}
#'   object, the input in \code{colData} refers to columns of the the
#'   \code{colData} slot of the \code{ClusterExperiment} object to be plotted
#'   with the clusters. In this case, \code{colData} can be TRUE (i.e. all
#'   columns will be plotted), or an index or a character vector that references
#'   a column or column name, respectively, of the \code{colData} slot of the
#'   \code{ClusterExperiment} object. If there are NAs in the \code{colData}
#'   columns, they will be encoded as 'unassigned' and receive the same color as
#'   'unassigned' samples in the clustering.
#' @param reuseColors Logical. Whether each row should consist of the same set
#'   of colors. By default (FALSE) each cluster that the algorithm doesn't
#'   identify to the previous rows clusters gets a new color.
#' @param matchToTop Logical as to whether all clusters should be aligned to the
#'   first row. By default (FALSE) each cluster is aligned to the ordered
#'   clusters of the row above it.
#' @param plot Logical as to whether a plot should be produced.
#' @param unassignedColor If ``-1'' in \code{clusters}, will be given this color
#'   (meant for samples not assigned to cluster).
#' @param missingColor If ``-2'' in clusters, will be given this color (meant
#'   for samples that were missing from the clustering, mainly when comparing
#'   clusterings run on different sets of samples)
#' @param minRequireColor In aligning colors between rows of clusters, require
#'   this percent overlap.
#' @param startNewColors logical, indicating whether in aligning colors between
#'   rows of clusters, should the colors restart at beginning of colPalette as
#'   long as colors are not in immediately proceeding row (the colors at the end 
#'   of \code{massivePalette} are all of \code{\link{colors}()} and many will be
#'   indistinguishable, so this option can be useful if you have a large cluster
#'   matrix).
#' @param colPalette a vector of colors used for the different clusters. Must be
#'   as long as the maximum number of clusters found in any single
#'   clustering/column given in \code{clusters} or will otherwise return an
#'   error.
#' @param input indicate whether the input matrix is matrix of integer assigned
#'   clusters, or contains the colors. If \code{input="colors"}, then the object
#'   \code{clusters} is a matrix of colors and there is no alignment (this
#'   option allows the user to manually adjust the colors and replot, for
#'   example).
#' @param clusterLabels names to go with the columns (clusterings) in matrix 
#'   \code{colorMat}. If \code{colData} argument is not \code{NULL}, the 
#'   \code{clusterLabels} argument must include names for the sample data too. 
#'   If the user gives only names for the clusterings, the code will try to 
#'   anticipate that and use the column names of the sample data, but this is
#'   fragile. If set to \code{FALSE}, then no labels will be plotted.
#' @param add whether to add to existing plot.
#' @param xCoord values on x-axis at which to plot the rows (samples).
#' @param ylim vector of limits of y-axis.
#' @param tick logical, whether to draw ticks on x-axis for each sample.
#' @param ylab character string for the label of y-axis.
#' @param xlab character string for the label of x-axis.
#' @param axisLine the number of lines in the axis labels on y-axis should be
#'   (passed to \code{line = ...} in the axis call).
#' @param box logical, whether to draw box around the plot.
#' @param ... for \code{plotClusters} arguments passed either to the method
#'   of \code{plotClusters} for matrices, or ultimately to \code{\link{plot}}
#'   (if \code{add=FALSE}).
#' @param existingColors how to make use of the exiting colors in the 
#'   \code{ClusterExperiment} object. 'ignore' will ignore them and assign new 
#'   colors. 'firstOnly' will use the existing colors of only the 1st 
#'   clustering, and then align the remaining clusters and give new colors for
#'   the remaining only. 'all' will use all of the existing colors.
#' @param resetNames logical. Whether to reset the names of the clusters in
#'   \code{clusterLegend} to be the aligned integer-valued ids from
#'   \code{plotClusters}.
#' @param resetColors logical. Whether to reset the colors in
#'   \code{clusterLegend} in the \code{ClusterExperiment} returned to be the
#'   colors from the \code{plotClusters}.
#' @param resetOrderSamples logical. Whether to replace the existing
#'   \code{orderSamples} slot in the \code{ClusterExperiment} object with the
#'   new order found.
#' @inheritParams getClusterIndex
#' @details All arguments of the matrix version can be passed to the
#'   \code{ClusterExperiment} version. As noted above, however, some arguments
#'   have different interpretations.
#' @details If \code{whichClusters = "workflow"}, then the workflow clusterings
#'   will be plotted in the following order: final, mergeClusters, makeConsensus,
#'   clusterMany.
#' @return If \code{clusters} is a \code{\link{ClusterExperiment}} Object, then
#'   \code{plotClusters} returns an updated \code{ClusterExperiment} object,
#'   where the \code{clusterLegend} and/or \code{orderSamples} slots have been
#'   updated (depending on the arguments).
#' @return If \code{clusters} is a matrix, \code{plotClusters} returns
#'   (invisibly) the orders and other things that go into making the matrix.
#'   Specifically, a list with the following elements.
#' \itemize{
#'
#' \item{\code{orderSamples}}{ a vector of length equal to 
#' \code{nrows(clusters)} giving the order of the samples (rows) to 
#'  use to get the original clusters matrix
#' into the order made by \code{plotClusters}.}
#'
#' \item{\code{colors}}{ matrix of color assignments for each element of
#' original clusters matrix. Matrix is in the same order as original clusters
#' matrix. The matrix \code{colors[orderSamples,]} is the matrix that can be 
#' given back to \code{plotClusters} to recreate the plot (see examples).}
#'
#' \item{\code{alignedClusterIds}}{ a matrix of integer valued cluster
#' assignments that match the colors. This is useful if you want to have cluster
#' identification numbers that are better aligned than that given in the
#' original clusters. Again, the rows/samples are in same order as original 
#' input matrix.}
#'
#' \item{\code{clusterLegend}}{ list of length equal to the number of columns of
#' input matrix. The elements of the list are matrices, each with three columns
#' named "Original","Aligned", and "Color" giving, respectively, the
#' correspondence between the original cluster ids in \code{clusters}, the
#' aligned cluster ids in \code{aligned}, and the color.}
#' 
#' \item{\code{origClusters}}{The original matrix of clusters given to
#' \code{plotClusters}}
#' }
#'
#' @author Elizabeth Purdom and Marla Johnson (based on the tracking plot in
#'   \link[ConsensusClusterPlus]{ConsensusClusterPlus} by Matt Wilkerson and Peter
#'   Waltman).
#'
#' @seealso The \code{\link[ConsensusClusterPlus]{ConsensusClusterPlus}} package.
#' @references Wilkerson, D. M, Hayes and Neil D (2010). "ConsensusClusterPlus: a class discovery tool with confidence assessments and item tracking." Bioinformatics, 26(12), pp. 1572-1573. 
#' @export
#'
#' @examples
#' \dontrun{
#' #clustering using pam: try using different dimensions of pca and different k
#' data(simData)
#'
#' cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reduceMethod="PCA",
#' clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
#' removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)
#'
#' clusterLabels(cl)
#'
#' #make names shorter for better plotting
#' x <- clusterLabels(cl)
#' x <- gsub("TRUE", "T", x)
#' x <- gsub("FALSE", "F", x)
#' x <- gsub("k=NA,", "", x)
#' x <- gsub("Features", "", x)
#' clusterLabels(cl) <- x
#'
#' par(mar=c(2,10,1,1))
#' #this will make the choices of plotClusters
#' cl <- plotClusters(cl, axisLine=-1, resetOrderSamples=TRUE, resetColors=TRUE)
#'
#' #see the new cluster colors
#' clusterLegend(cl)[1:2]
#'
#' #We can also change the order of the clusterings. Notice how this
#' #dramatically changes the plot!
#' clOrder <- c(3:6, 1:2, 7:ncol(clusterMatrix(cl)))
#' cl <- plotClusters(cl, whichClusters=clOrder, resetColors=TRUE,
#' resetOrder=TRUE, axisLine=-2)
#'
#' #We can manually switch the red ("#E31A1C") and green ("#33A02C") in the
#' #first cluster:
#'
#' #see what the default colors are and their names
#' showPalette(wh=1:5)
#'
#' #change "#E31A1C" to "#33A02C"
#' newColorMat <- clusterLegend(cl)[[clOrder[1]]]
#' newColorMat[2:3, "color"] <- c("#33A02C", "#E31A1C")
#' clusterLegend(cl)[[clOrder[1]]]<-newColorMat
#'
#' #replot by setting 'input="colors"'
#' par(mfrow=c(1,2))
#' plotClusters(cl, whichClusters=clOrder, orderSamples=orderSamples(cl),
#' existingColors="all")
#' plotClusters(cl, whichClusters=clOrder, resetColors=TRUE, resetOrder=TRUE,
#' axisLine=-2)
#' par(mfrow=c(1,1))
#'
#' #set some of clusterings arbitrarily to "-1", meaning not clustered (white),
#' #and "-2" (another possible designation getting gray, usually for samples not
#' #included in original clustering)
#' clMatNew <- apply(clusterMatrix(cl), 2, function(x) {
#'	wh <- sample(1:nSamples(cl), size=10)
#'	x[wh]<- -1
#'	wh <- sample(1:nSamples(cl), size=10)
#'	x[wh]<- -2
#'	return(x)
#'	})
#'
#' #make a new object
#' cl2 <- ClusterExperiment(assay(cl), clMatNew,
#' transformation=transformation(cl))
#' plotClusters(cl2)
#' }
#' @rdname plotClusters
#' @export
#' @importFrom grDevices gray
#' @importFrom stringr str_sort str_order
#' @aliases plotClusters,ClusterExperiment-method
setMethod(
  f = "plotClusters",
  signature = signature(object = "ClusterExperiment"),
  definition = function(object, whichClusters, 
  	existingColors=c("ignore","all","firstOnly"),
    resetNames=FALSE, resetColors=FALSE, resetOrderSamples=FALSE, colData=NULL, clusterLabels=NULL,...)
  {
	if(missing(whichClusters)){
	  if(!is.null(workflowClusterDetails(object))) whichClusters<-"workflow"
	  else whichClusters<-"all"	  	
	}
	whichClusters<-getClusterIndex(object,whichClusters=whichClusters,noMatch="throwError")
    existingColors<-match.arg(existingColors)
    args<-list(...)    
		checkIgnore<-.depricateArgument(passedArgs=args,"colData","sampleData") #06/2018 added in BioC 3.8
		if(!is.null(checkIgnore)){
			args<-checkIgnore$passedArgs
			colData<-checkIgnore$val
		}
    colData<-.pullColData(object,colData,fixNA="unassigned")
    if(is.null(clusterLabels)) clusterLabels<-clusterLabels(object)[whichClusters]
    if(existingColors!="ignore") useExisting<-TRUE else useExisting<-FALSE
    if(useExisting){ #using existing colors in some way:
      plotArg<-TRUE #the default
      if("plot" %in% names(args) ){
        whPlot<-which(names(args)=="plot")
        plotArg<-args[[whPlot]]
        if(length(args)>length(whPlot)){
          args<-args[-whPlot]
        }
        else{
          args<-NULL
        }
      }
      
      #if firstRow, add greys to the color palette for the first row so don't "use" up good colors on those
      colPaletteArg<-massivePalette #the default
      if("colPalette" %in% names(args) ){
        whPlot<-which(names(args)=="colPalette")
        colPaletteArg<-args[[whPlot]]
        if(length(args)>length(whPlot)){
          args<-args[-whPlot]
        }
        else{
          args<-NULL
        }
      }
      colPalGood<-colPaletteArg
      if(existingColors=="firstOnly"){
        nadd<-nrow(clusterLegend(object)[[whichClusters[[1]]]])
        addgreys<-gray(seq(0.01,.99,length=nadd))
        colPaletteArg<-c(addgreys,colPaletteArg)
      }
      
      #------------
      #align the samples, but don't plot them.
      #------------
      outval<-do.call(plotClusters, c(list(object=clusterMatrix(object)[,whichClusters,drop=FALSE], input="clusters", plot=FALSE, colData=colData,colPalette=colPaletteArg),args))
      #------------
      #make new color matrix with existing Colors
      #------------
      if(existingColors=="all"){
        newColorMat<-convertClusterLegend(object, whichClusters=whichClusters, output="matrixColors")
      }
      if(existingColors=="firstOnly"){
        #pre-existing colors for first clustering
        firstRow<-as.vector(convertClusterLegend(object, whichClusters=whichClusters[1], output="matrixColors"))
        #current assignment of colors all clusters:
        newColorMat<-outval$colors
        
        #integer values that match across the clusterings
        #note could skip this matrix, but just in case the colors are not unique, this guarantees don't change all of them....
        alignCl<-outval$alignedClusterIds[,1]
        firstCl<-clusterMatrix(object)[,whichClusters[1]]
        valsToChange<-unique(alignCl[firstCl>0]) #don't mess with assignment of -1/-2
        #match them and make them the existing color
        unusedColors<-colPalGood[!colPalGood %in% c(as.vector(newColorMat),firstRow)]
        ###Note: Need to update color legend in out val because used if resetColors
        ###Also, dimensions of outval matrices may be bigger than that of whichClusters because colData was added...
        for(val in valsToChange){
          wh<-which(alignCl==val)
          newCol<-unique(firstRow[wh])
          if(length(newCol)!=1) stop("internal coding error in existingColors='first'")
          #make sure not the same as existing color	
          if(any(newColorMat==newCol)){
            newColorMat[newColorMat==newCol]<-unusedColors[1]
            unusedColors<-tail(unusedColors,-1)
          }
          #alignedClusterIds aligns to a number even if -1/-2)
          newColorMat[outval$alignedClusterIds==val & !outval$origClusters%in% c("-1","-2")]<-newCol
        }
        outval$colors<-newColorMat
        ###make new clusterLegend from color matrix
        temp<-lapply(seq_len(ncol(outval$alignedClusterIds)),function(ii){
          oldMat<-outval$clusterLegend[[ii]]
          alignMat<-unique(cbind(alignedClusterIds=as.character(outval$alignedClusterIds[,ii]),color=newColorMat[,ii]))
          m<-match(oldMat[,"alignedClusterIds"],alignMat[,"alignedClusterIds"])
          if(any(is.na(m))) stop("internal coding error in make new clusterLegend")
          alignMat<-alignMat[m,]
          oldMat[,"color"]<-alignMat[,"color"]
          return(oldMat)
        })
        names(temp)<-names(outval$clusterLegend)
        outval$clusterLegend<-temp
        # $PublishedClusters
        #       clusterIds alignedClusterIds color     name
        #  [1,] "-2"       "1"               "black"   "1"
        #  [2,] "1"        "2"               "#161616" "2"
        #  [3,] "10"       "3"               "#292929" "3"
        #  [4,] "11"       "4"               "#3C3C3C" "4"
        
      }
      #------------
      #now plot them
      #------------
      if(plotArg) do.call(plotClusters,c(list(object=newColorMat[outval$orderSamples,], input="colors", clusterLabels=clusterLabels,colData=colData), args))
      
    }
    else{
			outval<-do.call(plotClusters,c(list(object=clusterMatrix(object)[,whichClusters,drop=FALSE],input="clusters",colData=colData,clusterLabels=clusterLabels), args))
    }
    if(resetColors | resetNames){
      ## recall, everything from outval is in the order of whichClusters!
      ## also includes values from colData, which are always at the bottom, so don't affect anything.
      oldClMat<-clusterMatrix(object)[,whichClusters]
      newClMat<-outval$alignedClusterIds
      #make both colors switch to aligned values (but keep name the same!)
      #can't convert the cluster ids because then wouldn't be consecutive numbers... but can give them names that match.
      convertAlignedColorLegend<-function(ii){
        oldColMat<-clusterLegend(object)[whichClusters][[ii]]
        newColMat<-outval$clusterLegend[[ii]]
        #weird space introduced in plotCluster to clusterIds
        newColMat[,"clusterIds"]<-as.character(as.numeric(newColMat[,"clusterIds"]))
        if(nrow(newColMat)!=nrow(oldColMat)) stop("error in aligning colorLegend") #just to make sure
        #update colors
        if(resetColors){
          m<-match(oldColMat[,"clusterIds"],newColMat[,"clusterIds"])
          oldColMat[,"color"]<-newColMat[m,"color"]
        }
        #convert from old to new names
        if(resetNames){
          
          newCl<-newClMat[,ii]
          oldCl<-oldClMat[,ii]
          oldNew<-unique(cbind(old=oldCl,new=newCl))
          if(nrow(oldNew)!=nrow(oldColMat)) stop("error in converting colorLegend from plotClusters output")
          #update clusterIds -- can't do
          m<-match(oldColMat[,"clusterIds"],oldNew[,"old"])
          oldColMat[,"name"]<-oldNew[m,"new"]
          if(any(oldColMat[,"clusterIds"] %in% c(-1,-2))){
            whNeg<-which(oldColMat[,"clusterIds"] %in% c(-1,-2))
            oldColMat[whNeg,"name"]<-as.character(oldColMat[whNeg,"clusterIds"])
          }
        }
        return(oldColMat)
      }
      newClLegend<-lapply(seq_len(NCOL(oldClMat)),convertAlignedColorLegend)
      names(newClLegend)<-colnames(oldClMat)
      clusterLegend(object)[whichClusters]<-newClLegend
      ch<-.checkClusterLegend(object)
      if(!is.logical(ch)) stop(ch)
    }
    if(resetOrderSamples) orderSamples(object)<-outval$orderSamples
    invisible(object)
    
  })



#' @rdname plotClusters
#' @export
setMethod(
  f = "plotClusters",
  signature = signature(object = "matrix"),
  definition = function(object, 
              orderSamples=NULL,colData=NULL,reuseColors=FALSE,matchToTop=FALSE,
              plot=TRUE,unassignedColor="white",missingColor="grey",
              minRequireColor=0.3,startNewColors=FALSE,colPalette=massivePalette,
              input=c("clusters","colors"),clusterLabels=colnames(object),
              add=FALSE,xCoord=NULL,ylim=NULL,tick=FALSE,ylab="",xlab="",
              axisLine=0,box=FALSE,...)
{
  if(!is.matrix(object)) stop("object must be a matrix")
	  
	  
  if(!is.null(orderSamples) && !all(orderSamples %in% seq_len(nrow(object))))
	  stop("invalid values for orderSamples")
  index<-orderSamples #match to old arguments
  input<-match.arg(input)

  #------------
  ###Add any additional colData to the bottom
  #------------
	if(!is.null(colData) & input=="clusters"){
	  if(!is.matrix(colData) && !is.data.frame(colData)){
      if(length(colData)!=nrow(object)){
        stop("if colData is a single vector, it must be the same length as nrow(object)")
        if(is.list(colData)) stop("colData cannot be a list, must be vector or matrix")
      }
	  }
	  else{
	    if(nrow(colData)!=nrow(object)) stop("colData must have same number of observations as clusterings")

	  }
	  
    clusters<-cbind(object,colData)
	}
	else clusters<-object
    
	#------------
	# deal with given clusterLabels 
    #------------
	clNames<-clusterLabels
	if(is.logical(clNames)){
		if(!clNames) clNames<-rep("",ncol(clusters))
		else clNames<-colnames(clusters)
	}
	
    if(length(clNames)!=ncol(clusters)){
		if(is.null(clNames)){
			clNames<-rep("",ncol(clusters))
		}
		else{
	    	if(length(clNames==ncol(clusters)-ncol(colData))){
	    		clNames<-c(clNames,colnames(colData))
			}
			else stop("number of cluster labels given in clusterLabels must be equal to the number of clusterings (or sample data columns plus clusterings)")
			
		}
    } 

	dnames<-dimnames(clusters)
	#------------
	# arguments to be passed at various calls (these are ones that do not change across the 
	#	different internal calls)
    #------------
	clusterPlotArgs<-list(clNames=clNames,add=add,xCoord=xCoord,ylim=ylim,tick=tick,ylab=ylab,xlab=xlab,axisLine=axisLine,box=box)
	plotTrackArgs<-list(index=index,reuseColors=reuseColors,matchToTop=matchToTop,minRequireColor=minRequireColor,startNewColors=startNewColors,colPalette=colPalette)

	if(input=="clusters"){
		clusters<-t(clusters) #original code had clusters in rows, rather than columns.
		if(any(apply(clusters,1,function(x){any(is.na(x))}))) stop("clusters should not have 'NA' values; non-clustered samples should get a '-1' or '-2' value depending on why they are not clustered.")
		if(any(as.character(clusters)%in%c("-1","-2"))){
			#------------
			# if have any -1/-2, have to do ordering without plotting, then fix it up by overwriting the -1/-2 colors
		    #------------
			### To Do: somehow reuse internal code of .makeColors to do this.		  	
			### To DO: do this ordering without -1/-2 so not lose a color to them.
			
			#1. align them, including "-1","-2" as a cluster. 
			out<-do.call(".plotClustersInternal",c(list(clusters=clusters,plot=FALSE),plotTrackArgs,clusterPlotArgs ,list(...)))
			#take out -1

			#2. replace colors of -1/-2 in clusterLegend (used to update clusterLegend if resetColors=TRUE)
			newColorLeg<-lapply(seq_len(nrow(clusters)),function(i){
				leg<-out$clusterLegend[[i]]
				if(any(wh<-leg[,"clusterIds"]== -1))
				leg[wh,"color"]<-unassignedColor
				if(any(wh<-leg[,"clusterIds"]== -2))
				leg[wh,"color"]<-missingColor
				return(leg)
			})
			names(newColorLeg)<-names(out$clusterLegend)
			#3. Update colors in colors matrix
			xColors<-do.call("cbind",lapply(seq_len(nrow(clusters)),function(i){
				out$colors[clusters[i,]=="-1",i]<-unassignedColor
				out$colors[clusters[i,]=="-2",i]<-missingColor
				return(out$colors[,i])
			}))
			#4. call .clusterTrackingPlot to make plot			
			if(plot) do.call(".clusterTrackingPlot",c(list(colorMat=xColors[out$orderSamples,]),clusterPlotArgs,list(...)))
			out$colors<-xColors
			dimnames(out$colors)<-dnames
			out$clusterLegend<-newColorLeg
		}
		else{
			out<-do.call(".plotClustersInternal",c(list(clusters=clusters,plot=plot),plotTrackArgs,clusterPlotArgs ,list(...)))
		}
		invisible(out)
	}
	else{
		do.call(".clusterTrackingPlot",c(list(colorMat=clusters),clusterPlotArgs,list(...)))
	}
})

.plotClustersInternal<-function(clusters, index, reuseColors, matchToTop, plot, minRequireColor, startNewColors, colPalette, ...){
	# clusters is a nclusterings x nsamples matrix.
  # The order of the rows (clusters) determines how the tracking will be done.
	# If given, index is the order of the samples (columns) for plotting. Otherwise determined by program
	# matchToTop allows that the comparison for color assignment be done, not row by row, but always back to the first row.

	# Check that no row has more than length(colPalette) number of unique entries
	colPalette<-unique(colPalette)
	tabVals<-apply(clusters,1,function(x){length(table(x))})
	if(any(tabVals> length(colPalette))) stop("Must give colPalette (i.e. distinct colors to be assigned to clustres) longer than number of clusters. Largest number of distinct clusters in the clusterings:",max(tabVals),". Length of colPalette:",length(colPalette))

	######
	#this for loop assigns colors to clusters
	######
	pastColorVector<-NULL
	colorM = rbind() #matrix of colors. gives colors for cluster assignments that give the 'right' color throughout the different rows
	for(i in seq_len(nrow(clusters))){
	  # Below calls the function:
		#.setClusterColors <- function(past_ct,ct,colorU,colorList,reuseColors,minRequireColor=0.5){
		# About this function:
  	  #description: sets common color of clusters between different K
			#takes previous row's colors and defines color assignments for next row.
			#assignments are done by:
				#start with largest cluster, assign to old cluster with largest overlap
			#returns a vector of colors

	#eap: rewrote a little, to make it easier to read. 5/27/2015
		currCl<-as.character(unlist(clusters[i,]))#as.numeric(as.character(unlist(clusters[i,]))) #think I've fixed it so don't need to convert to numeric.
		if(i == 1) pastCl<-NULL
		else{
			if(!matchToTop) pastCl<-as.character(unlist(clusters[i-1,]))#as.numeric(as.character(unlist(clusters[i-1,])))
			else pastCl<-as.character(unlist(clusters[1,]))#as.numeric(as.character(unlist(clusters[1,])))
		}
		newColorVector<- .setClusterColors(past_ct=pastCl,ct=currCl ,colorU=colPalette,past_colors=pastColorVector,reuseColors=reuseColors,minRequireColor=minRequireColor,startNewColors=startNewColors)

		if(i == 1) colorListTop<-newColorVector
		if(any(is.na(newColorVector))) stop("Error in internal code: NA values")
		if(!identical(sort(unname(table(newColorVector))),sort(unname(table(unlist(clusters[i,])))))) stop("Coding error, colors to do not have same number as original")
		crossTab<-paste(newColorVector,unlist(clusters[i,]),sep=";")
		if(!identical(sort(unname(table(crossTab))),sort(unname(table(unlist(clusters[i,])))))) stop("Coding error, colors to do not have same assignments as original")
		colorM = rbind(colorM,newColorVector)
		if(!matchToTop) pastColorVector<-newColorVector
			else pastColorVector<-colorM[1,]
	}
	dimnames(colorM)<-dimnames(clusters)

	###########
	#give integer values to alignCl
	###########
	allColors<-unique(as.vector(colorM))
	#make in order of input colPalette so that first row is names like 1,2,3 etc...
	tmp<-colPalette[colPalette %in% allColors]
	ord<-match(tmp,allColors)
	if(length(ord)!=length(allColors)) stop("coding error in the bowels of plotClusters. Contact mantainers.")
	allColors<-allColors[ord]
	#give numbers
	if(nrow(colorM)>1) alignCl<-apply(colorM,2,function(x){match(x,allColors)})
	else alignCl<-matrix(match(colorM[1,],allColors),nrow=1)
	dimnames(alignCl)<-dimnames(clusters)

	###########
	###Order the samples
	###########
	if(is.null(index)){
		tmp<-lapply(seq_len(nrow(alignCl)),function(i){unlist(alignCl[i,])})
		index<-do.call("order",tmp)
	}
	

	if(plot) .clusterTrackingPlot(t(colorM[,index,drop=FALSE]),...)

	###########
    # Make color legend
	###########
	clusterLegend<-lapply(seq_len(nrow(clusters)),function(ii){
		mat<-cbind("clusterIds"=unlist(clusters[ii,]),"alignedClusterIds"=unlist(alignCl[ii,]),"color"=unlist(colorM[ii,]))
		mat<-cbind(mat,"name"=mat[,"alignedClusterIds"])
		rownames(mat)<-NULL
		mat<-(unique(mat))
		mat<-mat[order(mat[,"clusterIds"]),,drop=FALSE]
        return(mat)
	})
	names(clusterLegend)<-rownames(clusters)
	invisible(list(orderSamples=index,colors=t(colorM),alignedClusterIds=t(alignCl),clusterLegend=clusterLegend,origClusters=t(clusters)))

}

.setClusterColors <- function(past_ct,ct,colorU,past_colors,reuseColors,minRequireColor,startNewColors){
	#description: sets common color of clusters between different K
	#takes previous row's colors and defines color assignments for next row.
	#assignments are done by:
		#start with largest cluster, assign to old cluster with largest overlap
	#past_ct is the past (i.e. previous row's) assigment to indices;
	#ct is the new cluster ids that need to be aligned to the past_ct
	#colorU is set of possible colors to take from
  #returns a vector of colors


	colorU<-rep(colorU,30) #so never run out of colors
	if(is.null(past_colors) | is.null(past_ct)){
		#map values of ct to 1:(# unique values)
		ncl<-length(unique(ct))
		v<-seq_len(ncl)
		names(v)<-stringr::str_sort(unique(ct),locale="en")
		clNum<-v[match(ct,names(v))]
		newColors = colorU[clNum]
	}
	else{
		#############
		#check that past_ct and past_colors line up
		#############
		pastTab<-table(past_ct,past_colors)
		if(nrow(pastTab)!=ncol(pastTab)) stop("past_ct and past_colors have different numbers of categories")
		pastTab<-t(apply(pastTab,1,sort,decreasing=TRUE)) #numeric sort, can use sort
		#each row should have only 1 non-zero category, so once order by row, then should all be zero except first
		if(any(colSums(pastTab[,-1,drop=FALSE])>0)) stop("past_ct and past_colors do not imply the same clustering of samples.")

		if(!startNewColors){
			whShared<-which(colorU %in% past_colors)
			m<-max(whShared)
			colorU<-c(colorU[m:length(colorU)],colorU[seq_len(m)]) #puts the used by previous rows colors at the end
		}
		colorU<-colorU[!colorU %in% past_colors]
		colorU<-unique(colorU) #make sure unique colors, at least per cluster
		if(length(colorU)==0) stop("not enough colors given -- all are in past_colors")

		colori<-0 #index which color will give next.
		newColors = rep(NA,length=length(past_colors)) #NULL #NA?

		####Tabulate the two clusters
		mo=table(past_ct,ct)
		#m=mo/apply(mo,1,sum) #for each old cluster, tells the percentage of the new cluster. Shouldn't it be the other way around?(EAP)
		m=mo #use raw number instead of percentage of cluster.
						# Browse[4]> m
						#        ct
						# past_ct  1  2  3  4  9 10
						#       1 18  7  0  0  0  0
						#       3  4  0 67 45  0  0
						#       9  0  0  1  0 21  1

		#EAP: change so do the one with the most overlap first, simplifies code.
		whichClusters<-order(apply(m,2,max),decreasing = TRUE)
		usedClusters<-vector("numeric") #index of m of used clusters (pci) if !reuseColors; if reuseColors, used slightly differently
		for(tci in whichClusters){ # for each cluster EAP: in order of size

			if(!reuseColors){
				#pci tells which old cluster has the greatest number of the new cluster
				vals<-unique(m[,tci])
				vals<-vals[vals>0] #remove zeros
				pvals<-vals/sum(m[,tci])
				maxvals<-max(pvals)
				vals<-vals[pvals >= min(c(minRequireColor,maxvals))]#require 30% overlap to take on new cluster value
				if(length(vals)>0){
					vals<-sort(vals,decreasing=TRUE) #numeric sort
					#for each value, determine its matching cluster (if any) then take highest
					matchPci<-sapply(vals,function(v){
						whMatch<-which(m[,tci]==v)
						whAv<-sapply(whMatch,function(x){!x %in% usedClusters})
						if(any(whAv)) return(whMatch[which(whAv)][1]) #pick the first of the available ones
						else return(NA)
					})
					newColor<-all(is.na(matchPci))
				}
				else{ newColor<-TRUE}
				if( newColor){ #new color
					colori=colori+1
					if(colori > length(colorU)) stop("not enough colors in color list (colorU)")
					newValue<-colorU[colori]
				}
				else{
					pci <- matchPci[!is.na(matchPci)][1] #take that with highest percentage that matches
					newValue<-unique(past_colors[which(as.character(past_ct)==rownames(m)[pci])])
					usedClusters<-c(usedClusters,pci)
				}
			}
			else{
				#pci tells which old cluster has the greatest number of the new cluster
				mCurr<-m
				if(length(usedClusters)>=nrow(m)){
					colori=colori+1
					newValue<-colorU[colori]
				}
				else{
					if(length(usedClusters)>0 & length(usedClusters)<nrow(m)){
						mCurr<-m[-usedClusters, , drop=FALSE]
					}
					maxC = max(mCurr[,tci])
					pci = row.names(mCurr)[which(mCurr[,tci] == maxC)]
					newValue<-unique(past_colors[which(as.character(past_ct)==pci)])
				}
				usedClusters<-c(usedClusters,grep(pci,row.names(m)))
			}
			newColors[which(as.character(ct)==colnames(m)[tci])] <-  newValue #EAP: fixed so no longer assumes cluster assignments were numeric integers with no gaps...
			tab<-table(newColors,ct)
			nPerColor<-apply(tab,1,function(x){sum(x!=0)})
			if(any(nPerColor!=1)) stop("color assigned twice within group.")
		}
	}
	if(any(is.na(newColors))) stop("Coding Error: some samples not given a color")
	tabOld<-table(ct)
	tabNew<-table(newColors)
	if(!identical(sort(unname(tabNew)),sort(unname(tabOld)))) stop("Coding error, didn't get same number of entries of each")
	#	if(length(tabNew)!=length(tabOld) || any(sort(tabNew)!=sort(tabOld))) stop("Coding error, didn't get same number of entries of each")
	return(newColors)
}

.clusterTrackingPlot <- function(colorMat, clNames, add, xCoord, ylim, tick, ylab, xlab, axisLine, box, ...){
  	m<-t(colorMat)
  #description: plots cluster tracking plot
  #input: m - matrix where rows are k, columns are samples, and entries are color assignments
  #names: names that correspond with rows of m
	if(is.null(xCoord)){
		xleft<-seq(0,1-1/ncol(m),by=1/ncol(m))
		xright<-seq(1/ncol(m),1,by=1/ncol(m))
	}
	else{
		d<-unique(round(diff(xCoord),6)) #distance between x values; had to round because got differences due to small numerical storage issues
		if(length(d)!=1) stop("invalid x values given -- not evenly spaced")
		xleft<-xCoord-d/2
		xright<-xCoord+d/2
	}
	if(is.null(ylim)){
		ylim<-c(0,1)
	}
	d<-(ylim[2]-ylim[1])/nrow(m)
	if(nrow(m)>1){ #for safety. theoretically works regardless, but if numerical tolerance problems...
		ybottom<-seq(ylim[2]-d,ylim[1],by=-d)
		ytop<-seq(ylim[2],ylim[1]+d,by=-d)
	}
	else{
		ybottom<-ylim[1]
		ytop<-ylim[2]
	}
	if(!add) graphics::plot(NULL, xlim=range(c(xleft,xright)), ylim=ylim,axes=FALSE, ylab=ylab, xlab=xlab, bty="n",...)

	for(i in seq_len(nrow(m))){
    rect(  xleft=xleft, xright=xright,  ybottom=rep(ybottom[i],ncol(m)) , ytop=rep(ytop[i],ncol(m)), col=m[i,],border=NA,xpd=NA)
  }
  #hatch lines to indicate samples
  if(tick){
	xl = (xleft+xright)/2
  	axis(1,at=xl,labels=FALSE,tick=TRUE)
#segments(  xl, rep(-0.1,ncol(m)) , xl, rep(0,ncol(m)), col="black")    #** alt white and black color?
  }
	if(!is.null(clNames)){
		y<-(ybottom+ytop)/2
	  	axis(2,at=y,labels=clNames,xpd=NA,las=2,tick=FALSE,line=axisLine)
	}
	#draw box around
	if(box){
		rect(xleft=min(xleft), ybottom=min(ybottom), xright=max(xright), ytop=max(ytop))
	}
}

.demethods<-c("edgeR","limma","limma-voom")
#' @name getBestFeatures
#' @aliases getBestFeatures getBestFeatures,matrixOrHDF5-method
#' @title Function for finding best features associated with clusters
#' @description Calls limma on input data to determine features most associated 
#'   with found clusters (based on an F-statistic, pairwise comparisons, or 
#'   following a tree that clusters the clusters).
#' @aliases getBestFeatures
#' @param x data for the test. Can be a numeric matrix or a 
#'   \code{\link{ClusterExperiment}}.
#' @param cluster A numeric vector with cluster assignments. ``-1'' indicates 
#'   the sample was not assigned to a cluster.
#' @param contrastType What type of test to do. `F' gives the omnibus 
#'   F-statistic, `Dendro' traverses the given dendrogram and does contrasts of 
#'   the samples in each side,  `Pairs' does pair-wise contrasts based on the 
#'   pairs given in pairMat (if pairMat=NULL, does all pairwise), and 
#'   `OneAgainstAll' compares each cluster to the average of all others. Passed 
#'   to \code{\link{clusterContrasts}}
#' @param contrastAdj What type of FDR correction to do for contrasts tests 
#'   (i.e. if contrastType='Dendro' or 'Pairs').
#' @param DEMethod character vector describing how the differential expression 
#'   analysis should be performed (replaces previous argument \code{isCount}. 
#'   See details.
#' @param weights weights to use in by edgeR. If \code{x} is a matrix, then
#'   weights should be a matrix of weights, of the same dimensions as \code{x}.
#'   If \code{x} is a \code{ClusterExperiment} object \code{weights} can be a
#'   either a matrix, as previously described, or a character or numeric index
#'   to an assay in \code{x} that contains the weights. We recommend that 
#'   weights be stored as an assay with name \code{"weights"} so that the
#'   weights will also be used with \code{\link{mergeClusters}}, and this is the
#'   default. Setting \code{weights=NULL} ensures that weights will NOT be used,
#'   and only the standard edgeR.
#' @param dgeArgs a list of arguments to pass to \code{\link[edgeR]{DGEList}}
#'   which is the starting point for both \code{edgeR} and \code{limma-voom}
#'   methods of DE. This includes normalization factors/total count values etc.
#' @param ... If \code{x} is a matrix, these are options to pass to 
#'   \code{\link{topTable}}  (see 
#'   \code{\link[limma]{limma}} package). If \code{x} is a 
#'   \code{ClusterExperiment} object, these arguments can also be those to pass 
#'   to the matrix version.
#' @inheritParams clusterContrasts
#' @inheritParams getClusterIndex
#' @details getBestFeatures returns the top ranked features corresponding to a 
#'   cluster assignment. It uses either limma or edgeR to fit the models, and 
#'   limma/edgeR functions \code{\link[limma]{topTable}} to find the best features. See the options 
#'   of this function to put better control on what gets returned (e.g. only 
#'   if significant, only if log-fc is above a certain amount, etc.). In 
#'   particular, set `number=` to define how many significant features to return
#'   (where number is per contrast for the `Pairs` or `Dendro` option)
#' @details  \code{DEMethod} triggers what type of differential expression 
#'   analysis will be performed. Three options are available: limma, edgeR, and 
#'   limma with a voom corrections. The last two options are only appropriate 
#'   for count data. If the input \code{x} is a \code{ClusterExperiment} object,
#'   and \code{DEMethod="limma"}, then the data analyzed for DE will be
#'   \emph{after} taking the transformation of the data (as given in the
#'   transformation slot of the object). For the options "limma-voom" and
#'   "edgeR", the transformation slot will be ignored and only the counts data
#'   (as specified by the \code{whichAssay} slot) will be passed to the
#'   programs. Note that for "limma-voom" this implies that the data will be
#'   transformed by voom with the function log2(x+0.5). If \code{weights} is not
#'   \code{NULL}, and \code{DEMethod="edgeR"}, then the function
#'   \code{glmWeightedF} from the \code{zinbwave} package is run; otherwise
#'   \code{glmLRT} from \code{edgeR}.
#' @details Note that the argument \code{DEMethod} replaces the previous option 
#'   \code{isCount}, to decide on the method of DE.
#' @details When `contrastType` argument implies that the best features should 
#'   be found via contrasts (i.e. 'contrastType' is `Pairs` or `Dendro`), then 
#'   then `contrastAdj` determines the type of multiple testing correction to 
#'   perform. `PerContrast` does FDR correction for each set of contrasts, and 
#'   does not guarantee control across all the different contrasts (so probably 
#'   not the preferred method). `All` calculates the corrected p-values based on
#'   FDR correction of all of the contrasts tested. `AfterF` controls the FDR 
#'   based on a hierarchical scheme that only tests the contrasts in those genes
#'   where the omnibus F statistic is significant. If the user selects `AfterF`,
#'   the user must also supply an option `p.value` to have any effect, and then 
#'   only those significant at that p.value level will be returned. Note that 
#'   currently the correction for `AfterF` is not guaranteed to control the FDR;
#'   improvements will be added in the future.
#' @details  Note that the default option for \code{\link[limma]{topTable}} is 
#'   to not filter based on adjusted p-values (\code{p.value = 1}) and return 
#'   only the top 10 most significant (\code{number = 10}) -- these are options 
#'   the user can change (these arguments are passed via the \code{...} in 
#'   \code{getBestFeatures}). In particular, it only makes sense to set 
#'   \code{requireF = TRUE} if \code{p.value} is meaningful (e.g. 0.1 or 0.05); 
#'   the default value of \code{p.value = 1} will not result in any effect on 
#'   the adjusted p-value otherwise.
#' @return A \code{data.frame} in the same format as 
#'   \code{\link[limma]{topTable}}, or \code{\link[edgeR]{topTags}}. The output
#'   differs between these two programs, mainly in the naming of columns.
#'   Furthermore, if weights are used, an additional column \code{padjFilter} is
#'   included as the result of running \code{\link[zinbwave]{glmWeightedF}} with
#'   default option \code{independentFiltering = TRUE}. The following column
#'   names are the same between all of the DE methods.
#' \itemize{
#'
#' \item{\code{Feature}}{ This is the column called 'ProbeID' by
#' \code{\link{topTable}}}
#'
#' \item{\code{IndexInOriginal}}{ Gives the index of the feature to the original
#' input dataset, \code{x}}
#'
#' \item{\code{Contrast}}{ The contrast that the results corresponds to (if
#' applicable, depends on \code{contrastType} argument)}
#'
#' \item{\code{ContrastName}}{ The name of the contrast that the results
#' corresponds to. For dendrogram searches, this will be the node of the tree of
#' the dendrogram. If \code{x} is a \code{ClusterExperiment} object, this name 
#' will make use of the user defined names of the cluster or node in \code{x}.}
#' 
#' \item{\code{InternalName}}{ Only present if \code{x} is a
#' \code{ClusterExperiment} object. In this case this column will give the name
#' of the contrast using the internal ids of the clusters and nodes, not the
#' user-defined names. This provides stability in matching the contrast if the
#' user has changed the names since running \code{getBestFeatures}}
#' 
#' \item{\code{P.Value}}{ The unadjusted p-value (changed from \code{PValue} in
#' \code{topTags})}
#' 
#' \item{\code{adj.P.Val}}{ The adjusted p-value (changed from \code{FDR} or
#' \code{FWER} in \code{topTags})}
#' }
#'
#' @references Ritchie, ME, Phipson, B, Wu, D, Hu, Y, Law, CW, Shi, W, and
#'   Smyth, GK (2015). limma powers differential expression analyses for
#'   RNA-sequencing and microarray studies. Nucleic Acids Research 43, e47.
#'   http://nar.oxfordjournals.org/content/43/7/e47
#' @references Law, CW, Chen, Y, Shi, W, and Smyth, GK (2014). Voom: precision
#'   weights unlock linear model analysis tools for RNA-seq read counts. Genome
#'   Biology 15, R29. http://genomebiology.com/2014/15/2/R29
#' @references Smyth, G. K. (2004). Linear models and empirical Bayes methods
#'   for assessing differential expression in microarray experiments.
#'   Statistical Applications in Genetics and Molecular Biology, Volume 3,
#'   Article 3. http://www.statsci.org/smyth/pubs/ebayes.pdf
#' @seealso \code{\link[edgeR]{glmLRT}} \code{\link[zinbwave]{glmWeightedF}}
#'   \code{\link[limma]{topTable}} \code{\link[edgeR]{topTags}}
#' @examples
#' data(simData)
#'
#' #create a clustering, for 8 clusters (truth was 4)
#' cl <- clusterSingle(simData, subsample=FALSE,
#' sequential=FALSE, mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=8)))
#'
#' #basic F test, return all, even if not significant:
#' testF <- getBestFeatures(cl, contrastType="F", number=nrow(simData),
#' DEMethod="limma")
#'
#' #Do all pairwise, only return significant, try different adjustments:
#' pairsPerC <- getBestFeatures(cl, contrastType="Pairs", contrastAdj="PerContrast",
#' p.value=0.05, DEMethod="limma")
#' pairsAfterF <- getBestFeatures(cl, contrastType="Pairs", contrastAdj="AfterF",
#' p.value=0.05, DEMethod="limma")
#' pairsAll <- getBestFeatures(cl, contrastType="Pairs", contrastAdj="All",
#' p.value=0.05, DEMethod="limma")
#'
#' #not useful for this silly example, but could look at overlap with Venn
#' allGenes <- paste("Row", 1:nrow(simData),sep="")
#' if(require(limma)){
#'  vennC <- vennCounts(cbind(PerContrast= allGenes %in% pairsPerC$Feature,
#'  AllJoint=allGenes %in% pairsAll$Feature, FHier=allGenes %in%
#'  pairsAfterF$Feature))
#'	vennDiagram(vennC, main="FDR Overlap")
#' }
#'
#' #Do one cluster against all others
#' oneAll <- getBestFeatures(cl, contrastType="OneAgainstAll", contrastAdj="All",
#' p.value=0.05,DEMethod="limma")
#'
#' #Do dendrogram testing
#' hcl <- makeDendrogram(cl)
#' allDendro <- getBestFeatures(hcl, contrastType="Dendro", contrastAdj=c("All"),
#' number=ncol(simData), p.value=0.05,DEMethod="limma")
#'
#' # do DE on counts using voom
#' # compare results to if used simData instead (not on count scale).
#' testFVoom <- getBestFeatures(simCount, primaryCluster(cl), contrastType="F",
#' number=nrow(simData), DEMethod="limma-voom")
#' plot(testF$P.Value[order(testF$Index)],
#' testFVoom$P.Value[order(testFVoom$Index)],log="xy")
#'
#' # do DE on counts using edgeR, compare voom
#' testFEdge <- getBestFeatures(simCount, primaryCluster(cl), contrastType="F",
#' n=nrow(simData), DEMethod="edgeR")
#' plot(testFVoom$P.Value[order(testFVoom$Index)],
#' testFEdge$P.Value[order(testFEdge$Index)],log="xy")
#' @export
#' @import limma
#' @rdname getBestFeatures
setMethod(f = "getBestFeatures",
signature = signature(x = "matrixOrHDF5"),
definition = function(x, cluster,
                      contrastType=c("F", "Dendro", "Pairs", "OneAgainstAll"),
                      dendro=NULL, pairMat=NULL, weights = NULL,
                      contrastAdj=c("All", "PerContrast", "AfterF"),
                      DEMethod=c("edgeR","limma","limma-voom"),
											dgeArgs=NULL,...) {
  
  
  #... is always sent to topTable/topTags, and nothing else
	DEMethod<-match.arg(DEMethod)
  cl<-cluster
  if(is.factor(cl)) {
    warning("cluster is a factor. Converting to numeric, which may not result in valid conversion")
    cl <- .convertToNum(cl)
  }
  
  dat <- data.matrix(x)
  contrastType <- match.arg(contrastType)
  contrastAdj <- match.arg(contrastAdj)
  returnType <- "Table" 
  if(is.null(rownames(dat))) {
    rownames(dat) <- paste("Row", as.character(seq_len(nrow(dat))), sep="")
  }
	
  #only use those assigned to a cluster to get good genes.
  tmp <- dat  
  if(any(cl<0)){ 
    whNA <- which(cl<0)
    tmp <- tmp[, -whNA]
    cl <- cl[-whNA]
    if(!is.null(weights)){
      weights<- weights[,-whNA]
    }
  }
	
  ###--------
  ### Fix up the names
  ###--------
  clPretty<-numericalAsCharacter(cl,prefix="Cl")
  clLevels<-unique(cl[order(clPretty)])
  clPrettyLevels<-unique(clPretty[order(clPretty)])
  #get them ordered nicely.
  clNumFac<-factor(cl,levels=clLevels)
  
  if(contrastType=="Dendro") {
    clPrettyFac<-factor(cl,levels=clLevels,labels=clLevels)
    if(is.null(dendro)) {
      stop("must provide dendro")
    }
    if(!inherits(dendro,"dendrogram") && !inherits(dendro,"phylo4")){
      stop("dendro must be of class 'dendrogram' or 'phylo4'")
    }
  }
  else{
    clPrettyFac<-factor(cl,levels=clLevels,labels=clPrettyLevels)
  }
	
  if(DEMethod %in% c("limma-voom","edgeR")){
  	dge<-do.call(edgeR::DGEList,c(list(counts=tmp),dgeArgs))
    if(!is.null(weights) & DEMethod=="edgeR") dge$weights <- weights
  }
  if(contrastType %in% c("Pairs", "Dendro", "OneAgainstAll")) {
    ###Create fit for running contrasts
    designContr <- model.matrix(~ 0 + clPrettyFac)
    colnames(designContr) <- make.names(levels(clPrettyFac))
    if(DEMethod == "limma-voom"){
      v <- voom(dge, design=designContr, plot=FALSE,
                normalize.method = "none")
      fitContr <- lmFit(v, designContr)
		}
		else if(DEMethod=="edgeR"){
      fitContr <- edgeR::estimateDisp(dge, designContr)
      fitContr <- edgeR::glmFit(fitContr,design = designContr)
		}
		else if(DEMethod=="limma"){
      fitContr <- lmFit(tmp, designContr)
    }
  }  
  if(contrastType=="F" || contrastAdj=="AfterF") {
    xdat<-data.frame("Cluster"=clPrettyFac)
    designF<-model.matrix(~Cluster,data=xdat)
    
    if(DEMethod == "limma-voom"){
      v <- voom(dge, design=designF, plot=FALSE,
                normalize.method = "none")
      fitF <- lmFit(v, designF)
    }
	else if(DEMethod=="edgeR"){
        fitF <- edgeR::estimateDisp(dge, design = designF)
        fitF <- edgeR::glmFit(fitF,design = designF)
    } else if(DEMethod=="limma") {
      fitF <- lmFit(tmp, designF)
    }
  } else {
    fitF <- NULL
  }
  	
  if(contrastType!="F"){
		contr.result<-clusterContrasts(clNumFac,contrastType=contrastType,dendro=dendro,pairMat=pairMat,outputType = "limma", removeUnassigned = TRUE)
  }
  if(contrastType=="F"){
    tops <- .getBestFGenes(fitF, DEMethod=DEMethod,...)
  }
  else{
    tops<-.testContrasts(contr.result$contrastMatrix,
			contrastNames=contr.result$contrastNames,
			fit=fitContr,fitF=fitF,DEMethod=DEMethod,
			contrastAdj=contrastAdj, ...)
  }
  if(contrastType=="Pairs"){
	  tops <- data.frame(ContrastName=tops$Contrast,tops)
  	  
  }
  tops <- data.frame(IndexInOriginal=match(tops$Feature, rownames(tmp)),tops)
  
  
  
  
  if(returnType=="Index") {
    whGenes <- tops$IndexInOriginal
    names(whGenes) <- tops$Feature
    return(whGenes)
  }
  if(returnType=="Table") {
    return(tops)
  }
}
)


#' @rdname getBestFeatures
#' @param whichAssay numeric or character specifying which assay to use. See
#'   \code{\link[SummarizedExperiment]{assay}} for details.
#' @export
setMethod(
  f = "getBestFeatures",
  signature = signature(x = "ClusterExperiment"),
  definition = 
    function(x, contrastType=c("F", "Dendro", "Pairs", "OneAgainstAll"), whichCluster="primary",
             whichAssay=1,DEMethod, weights=if("weights" %in% assayNames(x)) "weights" else NULL,...)
      {
    whCl<-getSingleClusterIndex(x,whichCluster,list(...))
      contrastType <- match.arg(contrastType)
    cl<-clusterMatrix(x)[,whCl]
    if(length(unique(cl[cl>0]))==1) stop("only single cluster in clustering -- cannot run getBestFeatures")
    if(contrastType=="Dendro") {
      if(is.null(x@dendro_clusters)) {
        stop("If `contrastType='Dendro'`, `makeDendrogram` must be run before `getBestFeatures`")
      } else {
        if(whCl!= dendroClusterIndex(x)){
          #check if merge from cluster that made dendro
          if(whCl==mergeClusterIndex(x) && x@merge_dendrocluster_index == dendroClusterIndex(x)){
            dendro<-.makeMergeDendrogram(x)
            if(is.null(dendro)) stop("Could not make merge dendrogram")
          }
          else stop("Cluster given in 'whichCluster' does not match either the cluster on which the dendrogram was made or the merge cluster from this dendrogram. Either replace the existing dendrogram with one based on using your preferred clustering (via 'makeDendrogram'), or set 'whichCluster' to be equal to index of 'dendo_index' or 'merge_index' slot")
        }
        else dendro <- x@dendro_clusters
      }
    }
    passedArgs<-list(...)
    if(DEMethod=="limma") dat<-transformData(x,whichAssay=whichAssay)
    else dat<-assay(x,whichAssay)
    if(!is.null(weights) && (is.character(weights) || (is.vector(weights) && is.numeric(weights)))  && length(weights)==1){
    		tops<-getBestFeatures(dat, cl, contrastType=contrastType, dendro=dendro, weights=assay(x, weights),DEMethod=DEMethod,...) 
    }
    else tops<-getBestFeatures(dat, cl, contrastType=contrastType, dendro=dendro, weights=weights,DEMethod=DEMethod,...)


    #### Fix up names
	#### Add column $InternalName -- 
	if(contrastType!="F"){
		wh<-which(colnames(tops)%in%c("IndexInOriginal", "ContrastName"))
		tops<-data.frame(tops[,wh],InternalName=tops$ContrastName,tops[,-wh])
		legMat<-clusterLegend(x)[[whCl]]
		#### Each contrastType needs different parsing
    if(contrastType=="Pairs"){
  	  #ContrastName in form of 'Cl01-Cl02' ...
  	  parseName<-gsub("Cl","",tops$InternalName)
  	  parseName<-strsplit(parseName,"-")
	    #note, incase there is padding in name, but not in clusterIds
  	  firstCl<-as.character(as.numeric(sapply(parseName,.subset2,1)))
  	  secondCl<-as.character(as.numeric(sapply(parseName,.subset2,2)))
  	  m1<-match(firstCl,as.numeric(legMat[,"clusterIds"]))
  	  m2<-match(secondCl,as.numeric(legMat[,"clusterIds"]))
	    if(any(is.na(m1))||any(is.na(m2))) stop("coding error -- cannot match parse cluster id from contrastName (no match to clusterLegend)")
  	  tops$ContrastName<-paste(legMat[m1,"name"],legMat[m2,"name"],sep="-")
    }
    if(contrastType=="OneAgainstAll"){
	    #ContrastName in form of Cl01
	  	parseName<-as.character(as.numeric(gsub("Cl","",tops$InternalName)))
  	  m1<-match(parseName,as.numeric(legMat[,"clusterIds"]))
  	  tops$ContrastName<-legMat[m1,"name"]
    }
    if(contrastType=="Dendro"){
	    #ContrastName in form of InternalNodeId5
	    m <- .matchToDendroData(inputValue=tops$InternalName, dendro, matchColumn="NodeId", returnColumn="NodeIndex")
	    tops$ContrastName<-phylobase::labels(dendro)[m]
    }
    #Removed this, because R 4.0.0 now does stringsAsFactor=FALSE by default. 
	#tops$ContrastName<-factor(tops$ContrastName) #make it consistent with the others and the results of matrix version
	}
	return(tops)
  }
)

#' @importFrom zinbwave glmWeightedF
.getBestFGenes<-function(fit, DEMethod, ...){
	if(DEMethod%in% c("limma","limma-voom")){
	  ## basic limma design
	  fit2 <- eBayes(fit)
      #browser()
	  tops <- suppressMessages(topTable(fit2,coef=NULL,
          genelist=rownames(fit$coef),...))
	  colnames(tops)[colnames(tops)=="ProbeID"]<-"Feature"
	}
	else if(DEMethod=="edgeR"){
    if(!is.null(fit$weights)){
      lrt <- zinbwave::glmWeightedF(fit, coef = c(2:ncol(fit$design)) )#Test if all the betas are zero i.e if the groups have the same gene expression
    }
    else{
      lrt <- edgeR::glmLRT(fit, coef = c(2:ncol(fit$design)))
    }
    tops<-edgeR::topTags(lrt,...)
		#Note, fit$coefficients are natural log, while output of topTags is log2 (see topTags documentation). Hence to add intercept, need to multiply by log2(exp(1))
    tops<- data.frame(Feature = rownames(tops), logFC.Intercept=log2(exp(1))*fit$coefficients[rownames(tops),"(Intercept)"],tops) 
	  colnames(tops)[colnames(tops)=="PValue"]<-"P.Value"
	  if("FDR" %in% colnames(tops)) colnames(tops)[colnames(tops)=="FDR"]<-"adj.P.Val"
		if("FWER" %in% colnames(tops)) colnames(tops)[colnames(tops)=="FWER"]<-"adj.P.Val"

  }
  
  return(tops)
}


.testContrasts<-function(contr.matrix, contrastNames=NULL, fit,fitF,contrastAdj, DEMethod,...){
  ncontr<-ncol(contr.matrix)
  if(DEMethod%in%c("limma","limma-voom")){
    fit2<-contrasts.fit(fit,contr.matrix)
    fit2<-eBayes(fit2)
  }
  else if(DEMethod=="edgeR"){
    fit2<-fit
  }
  args<-list(...)
  if("p.value" %in% names(args)){
    p.value<-args$p.value
  }
  else p.value<-1 #default of topTable
  
  if("number" %in% names(args)){
    nGenes<-args$number
  }
  else nGenes<-10 #default of topTable
  #get raw p-values for all(!)
  getRaw<-function(ii){
    if(DEMethod%in%c("limma","limma-voom")){
      if(contrastAdj%in%c("AfterF","All")) {
        tt<-topTable(fit2,coef=ii, number=length(rownames(fit2$coef)),p.value=1,adjust.method="none",genelist=rownames(fit2$coef))
      }
      else{
        tt<-topTable(fit2,coef=ii, genelist=rownames(fit2$coef),...)
      }
	    colnames(tt)[colnames(tt)=="ID"]<-"Feature"
    }
    if(DEMethod=="edgeR") {
      if(!is.null(fit2$weights)) fit2 <- zinbwave::glmWeightedF(fit2, contrast = contr.matrix[,ii])
      else fit2 <- edgeR::glmLRT(fit2, contrast = contr.matrix[,ii])
      if(contrastAdj%in%c("AfterF","All")) {
        tt<-edgeR::topTags(fit2, n=length(rownames(fit2$coef)),p.value=1,adjust.method="none")
      }
      else{
        tt<-edgeR::topTags(fit2,...)
      }
			if(is.null(rownames(tt))){rownames(tt)<- (seq_len(nGenes))}
      tt<- data.frame("Feature"=rownames(tt), tt)
			colnames(tt)[colnames(tt)=="PValue"]<-"P.Value"
			if("FWER" %in% colnames(tt)) colnames(tt)[colnames(tt)=="FWER"]<-"adj.P.Val"
			if("FDR" %in% colnames(tt)) colnames(tt)[colnames(tt)=="FDR"]<-"adj.P.Val"
    }
    if(nrow(tt)>0){
      tt<-data.frame("Contrast"=unname(colnames(contr.matrix)[ii]),tt,row.names=NULL)
      if(!is.null(contrastNames)){
        tt<-data.frame("ContrastName"=contrastNames[ii],tt,row.names=NULL)
      }
    }
    return(tt)
  }
  tops<-do.call("rbind",lapply(seq_len(ncontr),getRaw))
  if(contrastAdj=="AfterF" & p.value<1){
    #get p-value for F test for all genes, and only consider genes with significant F/LRT.
    if(DEMethod%in%c("limma","limma-voom")){
      fitF2<-eBayes(fitF)
      topsF<-topTable(fitF2,genelist=rownames(fit$coef),number=length(rownames(fit$coef)),adjust.method="BH",p.value=1)
      whGenesSigF<-topsF$ProbeID[which(topsF$adj.P.Val < p.value)]
    }
    else if(DEMethod %in% c("edgeR")) {
	    if(!is.null(fitF$weights)){
	      lrt <- zinbwave::glmWeightedF(fitF, coef = c(2:ncol(fitF$design)) )#Test if all the betas are zero i.e if the groups have the same gene expression
	    }
	    else{
	      lrt <- edgeR::glmLRT(fitF, coef = c(2:ncol(fitF$design)))
	    }
			topsF<-edgeR::topTags(lrt,genelist=rownames(fitF$coef),n=length(rownames(fitF$coef)),adjust.method="BH",p.value=1)
      whGenesSigF<-rownames(topsF)[which(topsF$FDR < p.value)]
    }
    tops<-tops[tops$Feature %in% whGenesSigF,]
  }
  #do FDR correction on all raw p-values (that remain)
  if(contrastAdj%in%c("AfterF","All")) {
    tops$adj.P.Val<-p.adjust(tops$P.Value,method="BH")
    if(p.value<1) tops<-tops[tops$adj.P.Val<p.value,,drop=FALSE]
    if(nGenes<length(rownames(fit$coef))){
      #just return relevant number per contrast
      tops<-do.call("rbind",by(tops,tops$Contrast,function(x){x[seq_len(min(c(nGenes,nrow(x)))),,drop=FALSE]}))
    }
  }
  row.names(tops)<-NULL  
  return(tops)

}

.makeMergeDendrogram<-function(object){
	#this function returns a dendrogram where the tips are the merged clusters, rather than the clusters of the original dendrogram.
  if(is.na(object@dendro_index)) stop("no dendrogram for this ClusterExperiment Object")
  #should this instead just silently return the existing?
  if(is.na(object@merge_index)) stop("no merging was done for this ClusterExperiment Object")
  if(object@merge_dendrocluster_index != object@dendro_index) stop("dendrogram of this object was made from different cluster than that of merge cluster.")
  
  #-----
  #test mergeClusters actually subset of the cluster says merged -- 
  # -- should go back to this with new phylo4d object
  #-----
  whClusterNode<-which(!is.na(object@merge_nodeMerge[,"mergeClusterId"]))
  clusterNode<-object@merge_nodeMerge[whClusterNode,"NodeId"]
  nodePruneIndex <- .matchToDendroData(inputValue=clusterNode,dendro=object@dendro_clusters,matchColumn="NodeId",returnColumn="NodeIndex")
  mergePhylo<-.pruneToNodes(phylo4=object@dendro_clusters,nodesPruned=nodePruneIndex)
  ##################
  #Now need to change tip labels to be that of the merge cluster ids
  ##################  
  tipIndex<-phylobase::getNode(mergePhylo,type="tip")
  clusterIds <- .matchToDendroData(inputValue=tipIndex, dendro=mergePhylo, matchColumn="NodeIndex", returnColumn="ClusterIdMerge")
  phylobase::labels(mergePhylo,type="tip")<-gsub("ClusterId","",clusterIds)
  return(mergePhylo)

}
  # old .makeMergeDendrogram code:
  #
  # phylo4Obj <- .convertToPhyClasses(object@dendro_clusters,"phylo4",convertNodes=TRUE,convertTips=TRUE)
  # newPhylo4<-phylo4Obj
  # if(names(rootNode(phylo4Obj)) %in% clusterNode){
  #   stop("coding error -- trying to make dendrogram from merge cluster when only 1 cluster in the clustering.")
  # }
  #
  #
  # #-----
  # # remove tips
  # # Note that subset function only removes specified tips (i.e. give tips to exclude)
  # # Doesn't allow you to give nodes you want to keep...
  # #-----
  # #After have updated merge, should come back to this...
  # for(node in clusterNode){
  #   #first remove tips of children nodes so all children of node in question are tips
  #
  #   desc<-phylobase::descendants(newPhylo4, node, type = c("all")) #names are names
  #   whDescNodes<-which(names(desc) %in% phylobase::nodeLabels(newPhylo4))
  #   while(length(whDescNodes)>0){
  #     tipNodeDesc<-unique(unlist(phylobase::descendants(newPhylo4, desc[whDescNodes], type = c("tips")))) #internal ids, not names, and no names to it
  #     newPhylo4<-phylobase::subset(newPhylo4,tips.exclude=tipNodeDesc,trim.internal =FALSE)
  #     #redo to check fixed problem
  #     desc<-phylobase::descendants(newPhylo4, node, type = c("all"))
  #     whDescNodes<-which(names(desc) %in% phylobase::nodeLabels(newPhylo4))
  #   }
  #   #should only have tips now
  #   tipsRemove<-phylobase::descendants(newPhylo4, node, type = c("tips"))
  #   newPhylo4<-.safePhyloSubset(newPhylo4,tipsRemove=tipsRemove,nodeName=node) #use instead of subset, because run into problems in phylobase in subset when small tree.
  # }
  #
  # ##################
  # #Now need to change tip name to be that of the merge cluster
  # #Currently tips should be either
  # #1) Name of makeConsensus cluster (i.e. integer) which needs to translate to a merge cluster
  # #2) Node name which now should be a merge cluster id
  # ##################
  # newTips<-currTips<-phylobase::tipLabels(newPhylo4) #has *names* as entries
  # #
  #
  # #Solve 1) First:
  # #Find the correspondence between old and new
  # #replace the old (i.e. non-nodes) with the new
  # corrsp<-getMergeCorrespond(object,by="original") #should be vector with names corresponding to original clusters, entries to merge clusters
  # whOldCl<-which(currTips %in% names(corrsp)) #which are cluster names of the original; these are ones that should be
  # if(length(whOldCl)>0){
  #   newTips[whOldCl]<-corrsp[currTips[whOldCl]]
  # }
  # ## Solve 2) Now:
  # ## should all be clusterNode
  # if(!all(clusterNode %in% currTips)) stop("coding error -- some cluster nodes didn't wind up as tips of new tree")
  # mClusterNode<-match(clusterNode, currTips)
  # newTips[mClusterNode]<-as.character(clusterId)
  # names(newTips)<-names(currTips) #this is the internal numbering of the nodes
  # phylobase::tipLabels(newPhylo4)<-newTips
  #
  # #--------
  # #just some checks didn't screw up
  # #--------
  # whMergeNode<-which(currTips %in% clusterNode)
  # if(length(intersect(whMergeNode,whOldCl))) stop("coding error -- should be no overlap bwettween merged node in tree tips and old clusters")
  # if(length(union(whMergeNode,whOldCl))!= length(currTips)) stop("coding error -- all tips should be either old clusters of merged nodes")
  # mCl<-clusterMatrix(object)[,object@merge_index]
  # mCl<-unique(mCl[mCl>0])
  # if(length(currTips)!= length(mCl)) stop("coding error -- number of tips of new tree not equal to the number of clusters in merged cluster")
  # if(!identical(sort(unname(as.character(mCl))),sort(unname(phylobase::tipLabels(newPhylo4))))){
  #   stop("coding error -- names of new tips of tree do not match cluster ids")
  # }
  # return(newPhylo4)

.makeSampleNames<-function(x){paste("Sample",as.character(x))}

.clusterDendroColumns<-c("Position","ClusterIdDendro","ClusterIdMerge","NodeId")
.clusterSampleColumns<-c("Position","NodeId","SampleIndex")

.positionLevels<-c("cluster hierarchy node","cluster hierarchy tip","tip hierarchy","assigned tip","outbranch hierarchy node","unassigned tip","outbranch root")

.hasOutBranch<-function(object){
	"outbranch root" %in% phylobase::tdata(object@dendro_samples)$Position
}

#' @description internal functions used by the package to manipulate dendrograms
#' @param inputValue a vector of values to match to dendro data
#' @param dendro a phylo4d object
#' @param matchColumn a character value giving the name of a column in the \code{tdata} of the dendro to match to. If "NodeIndex", means that will match to the row number (i.e. inputValue is a integer giving the row index); in this case it is the same as doing tdata(dendro)[inputValue,returnColumn]
#' @param returnColumn a character value giving the name of a column to return -- i.e. after matching to the matchColumn, return the corresponding values in the returnColumn. If returnColumn="NodeIndex", then just the index of the match is returned.
#' @noRd
.matchToDendroData<-function(inputValue,dendro,matchColumn="NodeId",returnColumn){
	#Note that matchColumn="NodeIndex" means inputs give rows of the df. returnColumn="NodeIndex" means to return the index that matches (i.e. nodeIndex too)
	df<-phylobase::tdata(dendro,type="all")
	if(!is.character(returnColumn) || !is.character(matchColumn)) stop("coding error -- returnColumn and matchColumn must be character valued")
	if(!returnColumn=="NodeIndex" && !returnColumn %in% names(df)) stop("coding error -- invalid value of returnColumn ")
	if(!matchColumn=="NodeIndex" && !matchColumn %in% names(df)) stop("coding error -- invalid value of matchColumn")
	if(!matchColumn=="NodeIndex") m<-match(inputValue, df[,matchColumn]) else m<-inputValue
	if(any(is.na(m))) stop("coding error -- invalid value in inputValue (not in dendro)")
	if(returnColumn=="NodeIndex") return(m)
	else return(df[m,returnColumn])
}

.getClusterIds<-function(tipIndex,clusterDendro,returnValue=c("ClusterIdDendro","ClusterIdMerge")){
	returnValue<-match.arg(returnValue)
  tipNames <- .matchToDendroData(tipIndex,clusterDendro,matchColumn="NodeIndex",returnColumn=returnValue)
  tipNames<-gsub("ClusterId","",as.character(tipNames))
  return(tipNames)
}


#' @description phylo4d objects for cluster and sample dendrograms with pretty (and matched!) node and tip labels.
#' @param object a ClusterExperiment object
#' @param labelType. If 'id' returns the NodeId value for all the internal nodes that match the cluster dendrogram (cluster and sample dendrograms) and the cluster id for the tips (cluster dendrogram) or sample index for the tips (sample dendrogram). If "name", then the cluster ids and sample indices are converted into the cluster names (in the clusterMat) and the sample names (in colnames)
#' @param useMergeClusters if TRUE and there is an active merge, will remove the dendrogram cluster ids, and instead use merge cluster ids (which means will nave no label for dendrogram cluster merged)
#' @return Returns list of the two dendrograms with nodes that have been updated with names 'dendro_clusters' and 'dendro_samples'. Note they do not match requirement of the clusterExperiment object because have labels for nodes they "shouldn't". 	 
#' @details  Different from convertToPhyClasses, which is trying to get the needed info into the phylo or phylo4 class that doesn't have tdata. Calls that function internally
#' @noRd
.setNodeLabels<-function(object,labelType=c("name","id"),useMergeClusters=FALSE,overrideExistingNode=FALSE,singletonCluster=c("sample","cluster"),...){
	labelType<-match.arg(labelType)
	singletonCluster<-match.arg(singletonCluster)
	 if(!inherits(object,"ClusterExperiment")) stop("coding error -- function should be used for ClusterExperiment objects")
	 if(is.na(object@merge_index)|| all(is.na(phylobase::tdata(object@dendro_clusters)$ClusterIdMerge))) useMergeClusters<-FALSE
	 if(is.null(object@dendro_clusters)) return(NULL)
	 #internal option returns same as the .convertPhyClasses with both =TRUE
	 internalCluster<-.convertToPhyClasses(object@dendro_clusters, returnClass=c("phylo4d"), convertNodes=TRUE, convertTips=TRUE)
	 internalSamples<-.convertToPhyClasses(object@dendro_samples, returnClass=c("phylo4d"), convertNodes=TRUE, convertTips=TRUE)
	 
	 
	 if(useMergeClusters){
		 #erase all the dendro cluster Ids, which are only on the tips
		 whConvert<-which(!is.na(phylobase::tdata(internalCluster,type="tip")$ClusterIdDendro))
		 phylobase::tipLabels(internalCluster)[whConvert]<-NA 
		 #now add merge cluster ids.
		 whConvert<-which(!is.na(phylobase::tdata(internalCluster,type="all")$ClusterIdMerge))
		 phylobase::labels(internalCluster)[whConvert] <- phylobase::tdata(internalCluster,type="all")$ClusterIdMerge[whConvert]
		 
	 }
	 if(!overrideExistingNode){
		 #located here so merge labels won't be displayed if user changed that node name manually.
		 existingNodeLabels<-phylobase::nodeLabels(object@dendro_clusters)
		 whKeep<-which(!is.na(existingNodeLabels) & existingNodeLabels!=phylobase::tdata(internalCluster,type="internal")$NodeId)
		 phylobase::nodeLabels(internalCluster)[whKeep]<-existingNodeLabels[whKeep]
	 }
	 	 
	 
	 if(labelType=="name"){
		 ###Convert to the name versions
		 if(useMergeClusters){
			 legMat<-clusterLegend(object)[[object@merge_index]]
			 labs<-phylobase::labels(internalCluster)[whConvert]
			 labs<-as.character(as.numeric(gsub("ClusterId","",labs)))
			 m<-match(labs,legMat[,"clusterIds"])	
			 phylobase::labels(internalCluster)[whConvert]<-legMat[m,"name"]
		 }
		 else{
			 legMat<-clusterLegend(object)[[object@dendro_index]]
		 	 labs<-phylobase::tipLabels(internalCluster)
			 labs<-as.character(as.numeric(gsub("ClusterId","",labs)))
			 m<-match(labs,legMat[,"clusterIds"])	
			 phylobase::tipLabels(internalCluster)<-legMat[m,"name"]
		 }
		 
		 nms<-if(!is.null(colnames(object))) colnames(object) else .makeSampleNames(seq_len(ncol(object)))
		 phylobase::tipLabels(internalSamples) <- nms[as.numeric(phylobase::tipLabels(internalSamples))]

			 
	 }
	 #Make sample node names match
	 type<-switch(singletonCluster,"sample"="internal","cluster"="all")
	 vals<-phylobase::tdata(internalSamples,type=type)$NodeId
	 wh<-which(!is.na(vals))
	 mToSample <- .matchToDendroData(inputValue=vals[wh], dendro=internalCluster, matchColumn="NodeId", returnColumn="NodeIndex")
	 phylobase::labels(internalSamples,type=type)[wh]<- phylobase::labels(internalCluster)[mToSample]
	 
	 
	 return(list(dendro_clusters=internalCluster,dendro_samples=internalSamples))
}






#' @param convertNodes logical. If true, the returned dendrogram will have the node labels change to be NodeId
#' @param convertTips logical. If true, the returned dendrogram will have the tip labels changed. If 'ClusterIdDendro' is in tdata (i.e. its a cluster dendrogram) then they are converted to Dendro cluster id (if no NAs) or if there are NAs, then merge cluster id (if dendros are NA in tips). If 'SampleIndex' is column name in tdata (i.e. is a samples dendrogram), then returns tip labels that are the SampleIndex value
#' @importFrom stats as.hclust
#' @importFrom ape as.phylo.hclust
#' @details phylo is class in ape package (S3). phylo4 and phylo4d are classes from  phylobase package -- S4 version of phylo. 
#' @noRd
.convertToPhyClasses<-function(x,returnClass=c("phylo4","phylo","phylo4d"),convertNodes=FALSE,convertTips=FALSE){
	returnClass<-match.arg(returnClass)
	if(inherits(x,"phylo4d") ){
		if(returnClass %in% c("phylo","phylo4","phylo4d")){
			#------
			#Before convert,
			#make internal node and cluster ids 
			#the node and tip labels (i.e. erase existing)
			#------
			if(convertNodes) 
                phylobase::nodeLabels(x)<-as.character(
                    phylobase::tdata(x,type="internal")$NodeId)
	  
			if(convertTips){
				if("ClusterIdDendro" %in%
                    names(phylobase::tdata(x,type="all"))){
					phylobase::tipLabels(x)<-as.character(
                        phylobase::tdata(x,type="tip")$ClusterIdDendro)
					if(any(is.na(phylobase::tipLabels(x)))){
						#this should mean its the cluster dendrogram limited to the merge ids.
						phylobase::tipLabels(x)<-as.character(phylobase::tdata(x,type="tip")$ClusterIdMerge)
					}	
					if(any(is.na(phylobase::tipLabels(x)))) stop("coding error -- even after accounting for merge id, still have NA values")
				}
				else if("SampleIndex" %in% names(phylobase::tdata(x,type="all")))
					phylobase::tipLabels(x)<-as.character(phylobase::tdata(x,type="tip")$SampleIndex)
				else stop("coding error -- tree should have either 'ClusterIdDendro' or 'SampleIndex' as names in tdata(x)")
			} 			
		}
		if(returnClass %in% c("phylo4","phylo4d")) return(x)
	}
	if(!inherits(x,"phylo4d") & returnClass=="phylo4d") stop("coding error -- can't convert other classes to phylo4d at this time and still retain all correct information")
	if(inherits(x,"dendrogram")){
		x<-try(stats::as.hclust(x),TRUE)
		if(inherits(x, "try-error")) stop("coding error -- could not convert from dendrogram to hclust object. Reported error:",x)
	}
	if(inherits(x,"hclust")){
		x<-try(ape::as.phylo.hclust(x),TRUE)
		if(inherits(x, "try-error")) stop("coding error -- could not convert from hclust to phylo object. Reported error:",x)
	}

	if(inherits(x,"phylo")){
		if(returnClass=="phylo") return(x)
		else{
            ## FIXME: getting warning here in test of plotCoClustering
            ## Commented out test for now, need to go back to it.
			x<-try(as(x,"phylo4"),TRUE)
			if(inherits(x, "try-error")) stop("coding error -- could not convert from phylo to phylo4 object. Reported error:",x)
			return(x)
		}
	}
	if(inherits(x,"phylo4")){
		if(returnClass=="phylo4") return(x)
		else{
			#phylobase warnings that trees with unknown edge order may be unsafe. This is probably because of this problem they ran into: http://lists.r-forge.r-project.org/pipermail/phylobase-devl/2009-January/000353.html
			#but the problem has probably been fixed by now in ape!
			x<-try(suppressWarnings(as(x,"phylo")),TRUE)
			if(inherits(x, "try-error")) stop("coding error -- could not convert from phylo4 to phylo object. Reported error:",x)
            #checkValidPhylo(x) returns NULL whether true or not. 
			return(x)
		}
	}
	else{ stop("input x is not of hclust, dendrogram, phylo or phylo4 class")}
	
}

.pruneToNodes<-function(phylo4,nodesPruned){
	##nodesPruned should be *the node numerical index* of those nodes that need "pruning" -- i.e. drop their children; other nodes kept the same.
	tipsPruned<-unlist(phylobase::descendants(phylo4,node=nodesPruned,type="tip"),use.names=FALSE)
	tipsKept<-phylobase::getNode(phylo4,type="tip")
	tipsKept<-tipsKept[!tipsKept %in% tipsPruned]
	nodesKeep<-c(nodesPruned,tipsKept)
	
	allNodes<-phylobase::getNode(phylo4,type="all")
	if(!all(nodesKeep %in% allNodes)) 
		stop("nodes specified are not valid")
	nodesKeep<-unique(unlist(phylobase::ancestors(phylo4,nodesKeep,"ALL"),use.names=FALSE)) #ALL means include self
	names(nodesKeep)<-names(allNodes)[nodesKeep]
	whEdgesKeep<-which(phylobase::edges(phylo4)[,2] %in% nodesKeep) 
	
	newEdge<-phylobase::edges(phylo4)[whEdgesKeep,]
	
	#need to get rid of singleton nodes -- i.e. nodes with only 1 descendant
	#this won't happen I think in my merge tree example, so make it error...
	singletonNode<-if(length(which(table(newEdge[-which(newEdge[,1]==0),1])==1))>0) stop("coding error -- removing these nodes creates internal nodes that are singletons. Should be only choice of nodes that 'prunes' back the tree, not actually subsetting.")
	
	#----------
	#have to renumber everything so consecutive... technically documentation doesn't say that, but actually do (error otherwise)
	#----------
	#get the current numbers of tips, internal nodes, and root
	#this is so know how to match the names, etc. back to them
	newTips<-newEdge[which(!newEdge[,2] %in% newEdge[,1]),2]
	newNodes<-nodesKeep
	root<-newEdge[newEdge[,1] ==0,2]
	whRoot<-which(newEdge[,1] ==0)

	#tips: make them 1:(#tips)
	#tips should be in edge matrix once:
	whEdgeTips<-match(newTips,newEdge) #find edges that contain tips
	consecutiveTips<-match(newTips,sort(newTips)) #give them consecutive numbers in same order as previous
	newEdge[whEdgeTips]<-consecutiveTips
	whNodeTips<-match(newTips,newNodes) #this is for reordering
	newNodes[whNodeTips]<-consecutiveTips
	
	#internal nodes: make them consecutive but in same order as previously, then add root number + nTips
	consecutiveNodes<-match(newNodes[-whNodeTips],sort(newNodes[-whNodeTips])) #doesn't include the 0 edge to root, so root is #1
	mNodesToEdge<-match(newEdge[-whEdgeTips],newNodes[-whNodeTips]) #get one NA from 0
	if(sum(is.na(mNodesToEdge))!=1) stop("coding error -- should have one na because of root")
	newEdge[-whEdgeTips]<-consecutiveNodes[mNodesToEdge]+length(newTips)
	newNodes[-whNodeTips]<-consecutiveNodes+length(newTips)	
	newEdge[whRoot,1]<-0
	if(any(sort(newNodes)!=seq_along(newNodes))) stop("coding error -- did not result in consecutive node numbers")
		
	#now need to make sure everything in right order with new edges:
	newdata<-phylo4@data[nodesKeep,]
	row.names(newdata)<-as.character(newNodes)
	newdata<-newdata[order(newNodes),]
	
	tipLabels<-names(newNodes)[whNodeTips]
	tipLabels<-tipLabels[order(newNodes[whNodeTips])]
	nodeLabels<-names(newNodes)[-whNodeTips]
	nodeLabels<-nodeLabels[order(newNodes[-whNodeTips])]

	##edge length slots: edge, edge.length, edge.label
	##node length slots: data (rows), label
	##Don't change: order, metadata, annote
	return(phylobase::phylo4d(x=newEdge, 
		edge.length = phylo4@edge.length[whEdgesKeep], 
		tip.label = tipLabels,
		node.label = nodeLabels, 
		edge.label = phylo4@edge.label[whEdgesKeep], 
		order = phylo4@order,
		annote = phylo4@annote,
		all.data=newdata,
		metadata=phylo4@metadata)
	)	
	
}

.safePhyloSubset<-function(phylo4,tipsRemove,nodeName){
  if(length(phylobase::tipLabels(phylo4))-length(tipsRemove)<2){
    ###Check that would have >1 tips left after remove (otherwise gives an error, not sure why with trim.internal=FALSE; should report it)
    ###Remove all but 1 tip seems to work -- collapse down desptie trim.internal=FALSE. Very weird.
    keptTip<-TRUE
    tipKeep<-names(tipsRemove)[1] #label of the tip removed (tipsRemove has internal names as value)
    tipsRemove<-tipsRemove[-1] 
  }
  else keptTip<-FALSE
  phylo4<-phylobase::subset(phylo4,tips.exclude=tipsRemove,trim.internal =FALSE)
  #have to give that 
  if(keptTip){
    labs<-phylobase::tipLabels(phylo4)
    wh<-which(labs==tipKeep)
    labs[wh]<-nodeName
    phylobase::tipLabels(phylo4)<-labs
  }
  return(phylo4)
}


###Note, cluster nodes DEFINED as those whose descendants are of length>0. So edge from root of these fake binary trees needs to be > 0, and rest =0 
### But also use this to make fake binary when n<5 samples and need it to stay ultrametric. Here, the edgeLength>0 will make it not ultrametric! Sigh. I fix that issue after returned (i.e. not here)
### Note that if n==1 (i.e. only 1 group), will return INVALID phylo object, because the edge matrix is matrix(NA,nrow=0,ncol=2) and Nnode=0
.makeFakeBinary<-function(tipNames,rootEdgeLength=0,edgeLength=0){
	newPhylo<-list()
	n<-length(tipNames)
	if(n>1){
		if(n>2){
			newPhylo$edge<-cbind(seq(from=n+1,to=n+(n-1)-1,by=1),seq(from=n+2,to=n+(n-1),by=1))
			newPhylo$edge<-rbind(newPhylo$edge,cbind(n+seq_len(n-1),seq_len(n-1)))
			newPhylo$edge<-rbind(newPhylo$edge,c(n+n-1,n))
		}
		else if(n==2){
			newPhylo$edge<-rbind(c(3,1),c(3,2))		
		}
		newPhylo$tip.label<-tipNames
		newPhylo$Nnode<-as.integer(n-1)
		newPhylo$edge.length<-rep(edgeLength,length=nrow(newPhylo$edge))
		whRoot<-which(newPhylo$edge==n+1,arr.ind = TRUE)
		if(nrow(whRoot)>2) stop("coding error -- found more than two descendants of root")
		if(any(whRoot[,"col"]==2)) stop("coding error -- found root as a descendant")
		newPhylo$edge.length[whRoot[,"row"]]<-rootEdgeLength
	}
	else{
		if(n==1){
			newPhylo<-list()
			newPhylo$edge<-matrix(NA,nrow=0,ncol=2)
			newPhylo$tip.label<-tipNames
			newPhylo$edge.length<-rootEdgeLength
			newPhylo$Nnode<-as.integer(0)
		}
		
		else stop("coding error -- zero or less length tipNames")
	}
	newPhylo$edge<-.makeIntegerMatrix(newPhylo$edge)
	
	class(newPhylo)<-"phylo"
	return(newPhylo)
}


.makeIntegerMatrix<-function(mat){
	#make the entries integer valued.
	matrix(as.integer(mat),ncol=ncol(mat),byrow=FALSE)
}


#' @importFrom ape node.depth.edgelength
.mergePhylo<-function(tree1,tree2,mergeEdgeLength,balanceLength=TRUE){
	#if balanceLength==TRUE, want to rebalance so that the two trees have same node height -- without adding to 0-length edges!
	n1<-length(tree1$tip.label)
	m1<-tree1$Nnode
	isTree<-is.list(tree2) & is(tree2,"phylo") #whether tree2 merging is singleton or actual tree (different protocal from .addTipsToTrees...)
	mergeEdgeLength1<-mergeEdgeLength2<-mergeEdgeLength
	depth1<-max(ape::node.depth.edgelength(tree1))	
	if(isTree){
		if(is.null(tree1$edge.length) || is.null(tree1$edge.length) ) stop("must have edge length on both trees")
		n2<-length(tree2$tip.label)
		m2<-tree2$Nnode
		if(balanceLength==TRUE){
			depth2<-max(ape::node.depth.edgelength(tree2))	
			finalDepth<-max(depth1,depth2)
			
			if(depth1==0){
				depth1<-1
				mergeEdgeLength1<-finalDepth+mergeEdgeLength
			}
			if(depth2==0){
				depth2<-1
				mergeEdgeLength2<-finalDepth+mergeEdgeLength	
			}
			tree1$edge.length<-tree1$edge.length/depth1*finalDepth
			tree2$edge.length<-tree2$edge.length/depth2*finalDepth
		}
	}else{
		if(is.null(tree1$edge.length)) stop("must have edge length to merge to tree")
		if(!is.vector(tree2) || length(tree2)!=1) stop("coding error -- trying to merge add more than 1 tip to tree")
		n2<-1
		m2<-0
		mergeEdgeLength2<-depth1+mergeEdgeLength
	}
	newPhylo<-list()
	
	#1) Tree 1 inner nodes: add n2+1 (because now will have new root)
	whinner1<-which(tree1$edge[,2] > n1) #including root of tree 1
	if(length(whinner1)>0) tree1$edge[whinner1,2]<-tree1$edge[whinner1,2]+n2+1
	tree1$edge[,1]<-tree1$edge[,1]+n2+1
	#2) Number of internal nodes
	newPhylo$Nnode<-m1+m2+1
	#3) Rescue node labels in tree1:
	#were 1:tree1$Nnode
	#in edge were +n1
	#moved to +n2+1 in new edge matrix
	#in node index, subtract n1+n2 -> +1
	newPhylo$node.label<-rep(NA,times=newPhylo$Nnode)
	if(!is.null(tree1$node.label)) newPhylo$node.label[seq_len(m1)+1]<-tree1$node.label

	if(isTree){
		#4) Tree 2 tips: add n1
		whtip2<-which(tree2$edge[,2]<=n2)
		tree2$edge[whtip2,2]<-tree2$edge[whtip2,2]+n1
	
		#5) Tree 2 inner nodes: -n2 to get to 1:... then add n2+n1+m1+1 -> +n1+m1+1
		if(nrow(tree2$edge)!=length(whtip2)) tree2$edge[-whtip2,2]<-tree2$edge[-whtip2,2]+n1+m1+1
		tree2$edge[,1]<-tree2$edge[,1]+n1+m1+1
	
		#6) Add root edge (n2+n1+1) to (n1+n2+2) and (n2+m1+n2+2) to bottom of edge matrix:
		newPhylo$edge<-rbind(tree1$edge,tree2$edge)
		newPhylo$edge<-rbind(newPhylo$edge, rbind(c(n2+n1+1,n1+n2+2),c(n2+n1+1,n1+m1+n2+2)))
		#7) tip.label
		newPhylo$tip.label<-c(tree1$tip.label,tree2$tip.label)
		#8) 		#should there be check that there is edge length?
		newPhylo$edge.length<-c(tree1$edge.length,tree2$edge.length,c(mergeEdgeLength1,mergeEdgeLength2))
		#9) Rescue node labels in tree 2:
		#were 1:tree2$Nnode
		#in edge were +n2
		#moved to +n1+m1+1 in new edge matrix
		#in node index, subtract n1+n2 -> +m1+1
			if(!is.null(tree2$node.label)) newPhylo$node.label[seq_len(m2)+m1+1]<-tree2$node.label

	}
	else{
		#6) Add root edge (n2+n1+1) to tree 1 (n1+n2+2) and root to tip to bottom of edge matrix:
		newPhylo$edge<-tree1$edge
		newPhylo$edge<-rbind(newPhylo$edge, rbind(c(n2+n1+1,n1+n2+2),c(n2+n1+1,n1+n2)))
		#7) tip.label: c(tree1$tip.label,tree2$tip.label)
		newPhylo$tip.label<-c(tree1$tip.label,tree2)		
		#8) 
		newPhylo$edge.length<-c(tree1$edge.length,c(mergeEdgeLength1,mergeEdgeLength2))
	}
	newPhylo$edge<-.makeIntegerMatrix(newPhylo$edge)
	class(newPhylo)<-"phylo"
	return(newPhylo)
	
}


#Input:
#' @param tipTrees should be list of trees in same order as tips in mainTree
#' @param mainTree a phylo object, assumed to be rooted tree
#' @details single values to be added to tree should have $edge=0-row matrix, $Nnode=0, $tip.label name of cluster, $edge.length=value in the tipTrees list to be added to length of cluster node
#' @return
#' noRd
.addTreesToTips<-function(mainTree,tipTrees){
	N<-length(mainTree$tip.label)
	M<-mainTree$Nnode
	
	if(length(tipTrees)!=N) stop("coding error -- length of list of trees must match number of tips in main Tree")
	
	#check valid phylo objects
	isPhylo<-sapply(tipTrees,class)=="phylo"
	if(!all(isPhylo)) stop("coding error -- each of the new trees must be of class phylo")
	# check<-sapply(tipTrees,.testPhyloObject,verbose=FALSE)
	# check<-.testPhyloObject(mainTree)
	
	
	nVec<-sapply(tipTrees,function(x){length(x$tip.label)})
	mVec<-sapply(tipTrees,function(x){x$Nnode})
	isSingle<-sapply(tipTrees,function(x){nrow(x$edge)==0})
	cumN<-c(0,cumsum(nVec))
	cumM<-c(0,cumsum(pmax(mVec-1,0))) #cumulative number of NON-ROOT internal
    #cumSingle[i] counts how many clusters/tips on main tree were singleton before (main) tip i 
	cumSingle<-c(0,cumsum(as.numeric(isSingle))) 
	#don't need the last entry of any of these.
	cumN<-head(cumN,-1) 
	cumM<-head(cumM,-1) 
	cumSingle<-head(cumSingle,-1) 
	
	##############
	#change mainTree edge matrix in anticipation of adding trees to tips
	##############
	whRoot<-which(mainTree$edge==N+1)
	whInternal<-which(mainTree$edge>N) #includes root
	#1) internal nodes: add sum(nVec)-N (including root)
	# note root becomes sum(nVec)+1, and the last internal edge should be sum(nVec)+M
	mainTree$edge[whInternal]<-mainTree$edge[whInternal]-N+sum(nVec)
    if(max(mainTree$edge[whInternal])!= sum(nVec)+M) stop("internal coding error in adding trees")
      
    #     whInternalIndex<-which(mainTree$edge>N,arr.ind=TRUE)
    # whTipsIndex<-which(mainTree$edge<=N & !mainTree$edge %in% which(isSingle),arr.ind=TRUE)
    
	#2) tips add sum(nVec)+M so now internal edges  -- except those that will be continue being tip because singleton added!
	whTips<-which(mainTree$edge<=N & !mainTree$edge %in% which(isSingle))
	# main tree tips that are not single get consecutive numbers
	mainTree$edge[whTips]<-mainTree$edge[whTips]+sum(nVec)+M-cumSingle[mainTree$edge[whTips]]
	if(sum(isSingle)>0){
		whSingle<-match(which(isSingle),mainTree$edge[,2])
		mainTree$edge[whSingle,2]<-cumN[isSingle]+1
	
	# Note that singletons, unlike others, short value of rootEdgeLength to their edge length, comparatively, when made with fakeBinaryTrees function. (and rootEdgeLength was added to deal with identification of cluster problem)
	# Need to add to those
		addValues<-sapply(tipTrees[isSingle],.subset2,"edge.length")
		mainTree$edge.length[whSingle]<-mainTree$edge.length[whSingle]+addValues
	}
    #######
    ### Check valid edge matrix:
    #######
    # check no duplicated edges:
    if(any(duplicated(mainTree$edge[,2]))) stop("internal coding error -- adding tips to tree led to duplicate edges")
    
    
	newPhylo<-list()
	newPhylo$Nnode<-mainTree$Nnode+sum(sapply(tipTrees,.subset2,"Nnode"))
	
	#------
	#Add back original node labels of the tree:
	#------
	newPhylo$node.label<-rep(NA,length=newPhylo$Nnode)
	#previously internal in main were 1:mainTree$Nnode+N in edge branch; 
	#Moved to -N +sum(nVec)
	#Then to move to nodes index by subtracting off length(newPhylo$tip.label)=sum(nVec)
	#so same index...
	newPhylo$node.label[seq_len(mainTree$Nnode)]<-mainTree$node.label

	# main tree tips that are not single become nodes -- same formula as before
	# were c(1:N)[!isSingle] in edge matrix (and tip.label)
	# Then moved by +sum(nVec)+M-cumSingle[!isSingle] in edge matrix
	# Then subtract of sum(nVec) to get to node index
	notSingleTips<-c(seq_len(N))[!isSingle]
	newPhylo$node.label[notSingleTips+M-cumSingle[!isSingle]]<-mainTree$tip.label[notSingleTips]
	##############
	#change treeList edgeMatrix
	##############
	tipTrees<-lapply(seq_along(tipTrees),function(ii){
		n<-nVec[[ii]]
		x<-tipTrees[[ii]]
		if(nrow(x$edge)>0){
			whInternal<-which(x$edge>n+1)
			whRoot<-which(x$edge==n+1)
			whTips<-which(x$edge<=n)
			x$edge[whRoot]<-ii+sum(nVec)+M-cumSingle[ii]
			# give new numbers to internal:
			# 1) subtract n[i]-1 to non-root internal nodes go from 1:(m[i]-1)
			# 2) + no. new tips (sum(nVec))
			# 3) + no. internal of main tree (M) 
			# 4) + total no. tips of main that become internal (i.e. not singletons) (sum(as.numeric(!isSingle)))
			# 5) + no. non-root internal of the ii-1 trees already added (cumM[ii])
			x$edge[whInternal]<-x$edge[whInternal]-(n+1)+ sum(nVec) + M  + sum(as.numeric(!isSingle)) + cumM[ii]
			x$edge[whTips]<-x$edge[whTips]+ cumN[ii]
		}
		return(x)
	})
	
    
	newPhylo$edge<-do.call("rbind",c(lapply(tipTrees[!isSingle],.subset2,"edge"),list(mainTree$edge)))
	newPhylo$tip.label<-do.call("c",lapply(tipTrees,.subset2,"tip.label"))
	newPhylo$edge.length<-do.call("c",c(lapply(tipTrees[!isSingle],.subset2,"edge.length"),list(mainTree$edge.length)))
	newPhylo$edge<-.makeIntegerMatrix(newPhylo$edge)

	#singleton clusters stay tips... make sure they get the right label (namely cluster, not sample -- sort of arbitrary choice)
	# were tip.label/edge values: which(isSingle)
	# move to = cumN[isSingle]+1 (still tip values don't subtract off
	newPhylo$tip.label[cumN[isSingle]+1]<-mainTree$tip.label[which(isSingle)]
	
	class(newPhylo)<-"phylo"
	return(newPhylo)	
}

.testPhyloObject<-function(phyloObj,verbose=FALSE){
	if(verbose) cat("Tests (should all be yes):\n")
	.testPrint("Required elements with right name:",all(c("tip.label","edge","Nnode") %in% names(phyloObj)),verbose=verbose)
	ntip<-length(phyloObj$tip.label)
	ninterior<-phyloObj$Nnode
	allNodes<-unique(as.numeric(phyloObj$edge))

	.testPrint("Edge matrix has two columns",dim(phyloObj$edge)[2]==2,verbose=verbose)

	#no gaps in series
	.testPrint("No gaps", all(sort(allNodes)==seq_along(allNodes) ),verbose=verbose)

	#matches expected range
	.testPrint("Range of series matches expected", length(allNodes)== ntip+ninterior ,verbose=verbose)
	
	#All elements, except the root
	#n+1, appear once in the second column.
	rootVal<-ntip+1
	tab<-table(phyloObj$edge[,2])
	.testPrint("All elements except root once in second column",all(tab[names(tab)!= as.character(rootVal)]==1),verbose=verbose)
	
	if("edge.length" %in% names(phyloObj)){
		.testPrint("edge length matches number edges",nrow(phyloObj$edge)==length(phyloObj$edge.length),verbose=verbose)
	}
	if(verbose){
		cat("-----------\n")
		cat("Number of tips:",ntip,"\n")
		cat("Number of interior:",ninterior,"\n")
		cat("Tips labels:",paste(phyloObj$tip.label,collapse=","),"\n")
		cat("Range of edge matrix values:",paste(range(allNodes),collapse=","),"\n")		
	}

}
.testPrint<-function(name,logic,verbose=FALSE){
	if(!is.logical(logic)) stop("logic input is not logical")
	if(!verbose){
		if(!logic)stop(paste("coding error -- the following check for phylo class did not pass",name))
	}
	else{
		cat(paste0(name,":"))
		if(logic) cat("Yes\n") else cat("No\n")		
	}
	
}

.force.ultrametric<-function(tree){
	##From http://blog.phytools.org/2017/03/forceultrametric-method-for-ultrametric.html
	#calculates, per tip, the amount missing and adds it to the tips.
	if(inherits(tree,"phylo")){
		depth<-ape::node.depth.edgelength(tree) #in order of all nodes
		ntips<-length(tree$tip.label)
		maxD<-max(depth[seq_len(ntips)])
		addValue<-maxD-depth[seq_len(ntips)]
		whTips<-match(seq_len(ntips),tree$edge[,2])
		tree$edge.length[whTips]<-tree$edge.length[whTips]+addValue
		return(tree)
	}
	else if(!inherits(tree,"phylo4")){
		stop("tree must be of class phylo4")
	} else{
		allTips<-phylobase::tipLabels(tree)
		depthToTips<-phylobase::nodeHeight(tree,allTips,from="root")
		maxD<-max(depthToTips)
		addValue<-maxD-depthToTips
		allLen<-phylobase::edgeLength(tree)
	  edgeMat<-phylobase::edges(tree)
  
	  #add 'addValue' to the tips so ultrametric
	  tipIds<-as.numeric(names(allTips))
	  m<-match(tipIds,edgeMat[,2])
	  edgeIds<-paste(edgeMat[m,1],edgeMat[m,2],sep="-")

	  #check didn't do something stupid:
	  checkTipEdges<-phylobase::edgeId(tree,type="tip")
	  if(!identical(sort(unname(checkTipEdges)),sort(unname(edgeIds)))) stop("coding error -- didn't correctly get edge ids for tips")

	  #replace with new edges:
		allLen[edgeIds]<-allLen[edgeIds]+addValue
		phylobase::edgeLength(tree)<-allLen
		return(tree) #returns phylo4 tree
	}

}



# #---------
# 		  #old code that made non-binary tree by hand.
# #---------
# 		  tab<-table(cl[whPos])
# 		  newPhylo<-list()
#
# 		  if(any(tab==1)){
# 	cluster1<-names(tab)[tab==1]
# 	nCluster1<-length(cluster1)
#   mClToTip1<-match(cluster1,phyloObj$tip.label) #find their tip number in matrix
# 	tipEdges1<-phyloObj$edge[phyloObj$edge[,2]==mClToTip1, ,drop=FALSE] #get the edge matrix for them
# 	mToPositiveCl<-match(tipEdges1[,2],cl[whPos])
# 	tipEdges1[,2]<-mToPositiveCl #change their tip number to their index in positive cluster vector
#
# }
# else{
# 	nCluster1<-0
# 	tipEdges1<-matrix(0,nrow=0,ncol=2)
# }
# if(any(tab>1)){
# 	clusterGr1<-names(tab)[tab>1]
# 	mGr1<-which(as.character(cl[whPos]) %in% clusterGr1)
#   newPhylo$edge<-cbind(cl[whPos][mGr1],mGr1)
#
#   mClToTip<-match(as.character(newPhylo$edge[,1]),phyloObj$tip.label) #gives for each cluster, tip number in old cluster tree
# 	if(any(is.na(mClToTip))) stop("coding error -- some of tip labels in phylo object do not match cluster names")
#   newPhylo$edge[,1]<-mClToTip
# 	newPhylo$edge<-rbind(newPhylo$edge,tipEdges1)
# }
# #---------
# ##Some tests -- might drop them for speed later
# #---------
# if(any(newPhylo$edge[,2]>length(cl[whPos]))) stop("coding error -- gave tip values greater than number of assigned samples")
# if(nrow(newPhylo$edge)!=length(cl[whPos])) stop("coding error -- did not assign all assigned samples to tip of tree")
# if(length(newPhylo$edge[,2])!=length(unique(newPhylo$edge[,2]))) stop("coding error -- did not give unique ids to tips of tree")
#
# #-------
# #make tips have values larger than root
# #-------
# internalEdges<-phyloObj$edge
# nOrigTips<-length(phyloObj$tip.label)
# maxOrig<-nOrigTips+phyloObj$Nnode
# whTipInternal<-internalEdges[,2]<=nOrigTips
# internalEdges[whTipInternal,2]<-internalEdges[whTipInternal,2]+maxOrig
# #fix up the newPhylo ones to have same number (again, for speed if fix to begin with?)
# whTipNew<-newPhylo$edge[,1]<=nOrigTips
# newPhylo$edge[whTipNew,1]<-newPhylo$edge[whTipNew,1]+maxOrig
#
#
# if(any(tab==1)){
# 	#have to remove those edges that went to clusters of size 1, because won't be internal edges.
# 	internalEdges<-internalEdges[ phyloObj$edge[,2]!=mClToTip1, ,drop=FALSE]
# }
# #also need to renumber everything, because the tip lost will leave gap... Need to keep the same order.
# oldValues<-sort( unique( as.numeric( internalEdges )))
# newValues<-1:length(oldValues)
# m1<-match(internalEdges[,1],oldValues)
# m2<-match(internalEdges[,2],oldValues)
# internalEdges<-cbind(newValues[m1],newValues[m2])
#
# #now fix up the edge matrix too; speed wise, is it faster to do it before? More complicated to code it before...
# newPhylo$edge[,1]<-newValues[match(newPhylo$edge[,1],oldValues)]
# internalEdges<-internalEdges+length(cl[whPos])
# newPhylo$edge[,1]<-newPhylo$edge[,1]+length(cl[whPos])
# newPhylo$edge<-rbind(internalEdges,newPhylo$edge)
#
# 		  if(!is.null(sampleNames)) newPhylo$tip.label<-sampleNames[whPos]
# 		  else{
# 		      newPhylo$tip.label<-paste("Sample",whPos)
# 		    }
# 		  newPhylo$Nnode<-phyloObj$Nnode+length(phyloObj$tip.label)-nCluster1
# if("edge.length" %in% names(phyloObj)){
# 	oldEdgeLength<-phyloObj$edge.length
# 	if( any(tab==1) ) oldEdgeLength<-oldEdgeLength[phyloObj$edge[,2]!=mClToTip1]
# 	newPhylo$edge.length<-rep(sampleEdgeLength,length=length(newPhylo$tip.label))
# 	if(any(tab==1)){ #need get back length to single-sample cluster
# 		oldEdgeLength1<-phyloObj$edge.length[phyloObj$edge[,2]==mClToTip1]
# 		newPhylo$edge.length[(length(newPhylo$edge.length)-nCluster1+1):length(newPhylo$edge.length)]<-oldEdgeLength1
# 	}
# 	newPhylo$edge.length<-c(oldEdgeLength,newPhylo$edge.length)
# }
# 		  class(newPhylo)<-"phylo"

## Construct ClusterExperiment with multiple named assays
suppressMessages(multi_se <- SummarizedExperiment(assays = list(counts = simCount,
                                               logcounts = log1p(simCount))))
suppressMessages(multi_cc <- ClusterExperiment(multi_se, trueCluster))
suppressMessages(ccTrue <- ClusterExperiment(simCount, trueCluster))
suppressMessages(ccTrue2 <- ClusterExperiment(simCount, trueCluster, transformation = log1p))
seedValue<-01875 #works for sample.kind="Rejection"

test_that("RSEC works wih non default assays", {
  expect_message(out1<-RSEC(x=multi_cc, reduceMethod="none",
                            k0s=4:5, clusterFunction="hierarchical01", alphas=0.1,
                            betas=0.9, dendroReduce="none", minSizes=1,
                            subsampleArgs=list(resamp.num=5),   
                            seqArgs=list(top.can=0),
                            random.seed=seedValue, whichAssay = "counts"),
                 "Merging will be done on")
  
  expect_message(out2<-RSEC(x=multi_cc, reduceMethod="none",
                            k0s=4:5, clusterFunction="hierarchical01", alphas=0.1,
                            betas=0.9, dendroReduce="none", minSizes=1,
                            subsampleArgs=list(resamp.num=5),
                            seqArgs=list(top.can=0),
                            random.seed=seedValue, whichAssay = "logcounts"),
                 "Merging will be done on")
  
  expect_message(out3<-RSEC(x=ccTrue, reduceMethod="none",
                            k0s=4:5, clusterFunction="hierarchical01", alphas=0.1,
                            betas=0.9, dendroReduce="none", minSizes=1,
                            subsampleArgs=list(resamp.num=5),
                            seqArgs=list(top.can=0),
                            random.seed=seedValue),
                 "Merging will be done on")
  
  expect_message(out4<-RSEC(x=ccTrue2, reduceMethod="none",
                            k0s=4:5, clusterFunction="hierarchical01", alphas=0.1,
                            betas=0.9, dendroReduce="none", minSizes=1,
                            subsampleArgs=list(resamp.num=5),
                            seqArgs=list(top.can=0),
                            random.seed=seedValue),
                 "Merging will be done on")
  
  expect_false(all(primaryCluster(out1) == primaryCluster(out2)))
  
  expect_equivalent(out1, out3)
  expect_equivalent(out2, out4)
})

test_that("RSEC works independent of assay order", {
  local_edition(3) # allows to ignore function environment; for some reason this wasn't problem before, but now it is
  #create two with different order of the assays
  multi_se <- SummarizedExperiment(assays = list(counts = simCount,
                                                 logcounts = log1p(simCount)))
  multi_se2 <- SummarizedExperiment(assays = list(logcounts = log1p(simCount),
                                                  counts = simCount))
  multi_cc <- ClusterExperiment(multi_se, trueCluster)
  multi_cc2 <- ClusterExperiment(multi_se2, trueCluster)
  
  #use character, logcounts on both 
  expect_message(out1<-RSEC(x=multi_cc, reduceMethod="none",
                            k0s=4:5, clusterFunction="hierarchical01", alphas=0.1,
                            betas=0.9, dendroReduce="none", minSizes=1,
                            subsampleArgs=list(resamp.num=5),
                            seqArgs=list(top.can=0),
                            random.seed=seedValue, whichAssay = "logcounts"),
                 "Merging will be done on")
  
  expect_message(out2<-RSEC(x=multi_cc2, reduceMethod="none",
                            k0s=4:5, clusterFunction="hierarchical01", alphas=0.1,
                            betas=0.9, dendroReduce="none", minSizes=1,
                            subsampleArgs=list(resamp.num=5),
                            seqArgs=list(top.can=0),
                            random.seed=seedValue, whichAssay = "logcounts"),
                 "Merging will be done on")
  
  # had to change the tests, because with new version of bioconductor, 
  # order of assays being different is caught, and for some reason wasn't caught before. 
  # Very strange passed previously.
  # Want to test everything same but assays
  out1@assays<-out2@assays
  expect_equal(out1, out2,ignore_function_env=TRUE)
  
  #use numeric
  expect_message(out1<-RSEC(x=multi_cc, reduceMethod="none",
                            k0s=4:5, clusterFunction="hierarchical01", alphas=0.1,
                            betas=0.9, dendroReduce="none", minSizes=1,
                            subsampleArgs=list(resamp.num=5),
                            seqArgs=list(top.can=0),
                            random.seed=seedValue, whichAssay = 2),
                 "Merging will be done on")
  
  expect_message(out2<-RSEC(x=multi_cc2, reduceMethod="none",
                            k0s=4:5, clusterFunction="hierarchical01", alphas=0.1,
                            betas=0.9, dendroReduce="none", minSizes=1,
                            subsampleArgs=list(resamp.num=5),
                            seqArgs=list(top.can=0),
                            random.seed=seedValue, whichAssay = 1),
                 "Merging will be done on")
  # Want to test everything same but assays
  out1@assays<-out2@assays
  expect_equal(out1, out2,ignore_function_env=TRUE)
  
  #use character, counts on both with PCA reduce
  # In 2.7.2-9001 had to change this to 60 dims -- no longer ran, probably because changed the clustering slightly?
  expect_message(out1<-RSEC(x=multi_cc, reduceMethod="PCA", nReducedDims = 60,
                            k0s=4:5, clusterFunction="hierarchical01", alphas=0.1,
                            betas=0.9, dendroReduce="none", minSizes=1,
                            subsampleArgs=list(resamp.num=5, clusterFunction="pam"),
                            seqArgs=list(top.can=0),
                            random.seed=seedValue, whichAssay = "counts"),
                 "Merging will be done on")
  
  expect_message(out2<-RSEC(x=multi_cc2, reduceMethod="PCA", nReducedDims = 60,
                            k0s=4:5, clusterFunction="hierarchical01", alphas=0.1,
                            betas=0.9, dendroReduce="none", minSizes=1,
                            subsampleArgs=list(resamp.num=5, clusterFunction="pam"),
                            seqArgs=list(top.can=0),
                            random.seed=seedValue, whichAssay = "counts"),
                 "Merging will be done on")
  
  # Want to test everything same but assays
  out1@assays<-out2@assays
  expect_equal(out1, out2,ignore_function_env=TRUE)
  
  #use character, counts on both with var reduce
  expect_message(out1<-RSEC(x=multi_cc, reduceMethod="var",
                            k0s=4:5, clusterFunction="hierarchical01", alphas=0.1,
                            betas=0.9, dendroReduce="none", minSizes=1,
                            subsampleArgs=list(resamp.num=5),
                            seqArgs=list(top.can=0),
                            random.seed=seedValue, whichAssay = "logcounts"),
                 "Merging will be done on")
  
  expect_message(out2<-RSEC(x=multi_cc2, reduceMethod="var",
                            k0s=4:5, clusterFunction="hierarchical01", alphas=0.1,
                            betas=0.9, dendroReduce="none", minSizes=1,
                            subsampleArgs=list(resamp.num=5),
                            seqArgs=list(top.can=0),
                            random.seed=seedValue, whichAssay = "logcounts"),
                 "Merging will be done on")
  
  # Want to test everything same but assays
  out1@assays<-out2@assays
  expect_equal(out1, out2,ignore_function_env=TRUE)
  
})

test_that("`RSEC` works through whole series of steps",{
  #bigger example where actually goes through all the steps, takes some time:
  expect_message(rsecOut<-RSEC(
    x=assay(seSimCount), isCount=TRUE,reduceMethod="none",
    k0s=4:5,clusterFunction="hierarchical01", alphas=0.1, 
    betas=0.9,minSizes=1,
    subsampleArgs=list(resamp.num=5), seqArgs=list(top.can=0),
    random.seed=seedValue, makeMissingDiss=TRUE,
    consensusProportion=0.7, consensusMinSize=5,
    dendroReduce="none", stopOnErrors = FALSE,
    mergeMethod = "adjP", 
    mergeDEMethod="edgeR",mergeCutoff = 0.05),
    "Merging will be done on")
  expect_silent(ceOut<- clusterMany(x=assay(seSimCount), isCount=TRUE,
                                    reduceMethod="none", 
                                    ks=4:5, clusterFunction="hierarchical01", alphas=0.1, 
                                    betas=0.9, minSizes=1,
                                    transFun = NULL,
                                    sequential=TRUE,removeSil=FALSE,subsample=TRUE,
                                    silCutoff=0,distFunction=NA,
                                    nFilterDims=NA,nReducedDims=NA,
                                    mainClusterArgs=NULL,subsampleArgs=list(resamp.num=5),
                                    ncores=1,run=TRUE, verbose=FALSE,stopOnErrors = TRUE,
                                    seqArgs=list(verbose=FALSE,top.can=0),random.seed=seedValue
  ))
  expect_equal(clusterMatrix(rsecOut,
                             whichClusters="clusterMany"),clusterMatrix(ceOut))
  expect_message(combOut<-makeConsensus(ceOut, 
                                        proportion = 0.7,minSize = 5),
                 "no clusters specified to combine")
  expect_equal(clusterMatrix(rsecOut,
                             whichClusters="makeConsensus"),
               clusterMatrix(combOut,
                             whichClusters="makeConsensus"))
  expect_equal(clusterMatrix(rsecOut,which=coClustering(rsecOut)),
               clusterMatrix(combOut,which=coClustering(combOut)))
  
  expect_silent(dendOut<-makeDendrogram(combOut,
                                        reduceMethod="none",nDims=NA))
  #they differ in tdata:
  expect_equal(as(dendOut@dendro_clusters,"phylo4"), 
               as(rsecOut@dendro_clusters,"phylo4"))
  tdRsec<-phylobase::tdata(rsecOut@dendro_clusters)
  tdRsec<-tdRsec[,-grep("ClusterIdMerge",colnames(tdRsec))]
  td<-phylobase::tdata(dendOut@dendro_clusters)
  td<-td[,-grep("ClusterIdMerge",colnames(td))]
  expect_equal(td, tdRsec)
  expect_equal(clusterExperiment:::.hasOutBranch(dendOut),
               clusterExperiment:::.hasOutBranch(rsecOut))
  #now should be the same, check all objects except dendro_samples because very big:
  expect_message(mergeOut<-mergeClusters(dendOut,mergeMethod = "adjP", 
                                         DEMethod="edgeR",cutoff = 0.05),
                 "Merging will be done on")
  expect_equal(dendroClusterIndex(mergeOut),dendroClusterIndex(rsecOut))
  expect_equal(mergeOut@dendro_clusters,rsecOut@dendro_clusters)
  expect_equal(clusterExperiment:::.hasOutBranch(mergeOut),
               clusterExperiment:::.hasOutBranch(rsecOut))
  expect_equal(coClustering(mergeOut),coClustering(rsecOut))
  expect_equal(clusterMatrix(rsecOut,whichClusters="mergeClusters"),
               clusterMatrix(mergeOut,whichClusters="mergeClusters"))
  expect_equal(clusterTypes(rsecOut),clusterTypes(mergeOut))
})

test_that("`RSEC` works with hdf5",{
  
  expect_message(rsecOut2<-RSEC(hdfObj, isCount=FALSE,k0s=4:5,
                                reduceMethod="PCA",
                                clusterFunction="hierarchical01", alphas=0.1, nReducedDims=3,
                                subsampleArgs=list(resamp.num=5),
                                seqArgs=list(top.can=0),
                                random.seed=seedValue),
                 "Merging will be done on"
  )
  
  expect_message(rsecOut3<-RSEC(assay(hdfObj), isCount=FALSE,
                                k0s=4:5, reduceMethod="PCA",
                                clusterFunction="hierarchical01", alphas=0.1, nReducedDims=3,
                                subsampleArgs=list(resamp.num=5),
                                seqArgs=list(top.can=0), random.seed=seedValue),
                 "Merging will be done on"
  )
  
  expect_equal(clusterMatrix(rsecOut2),clusterMatrix(rsecOut3))
  
  #no reduce method, do everything on raw data
  #requires numeric/complex matrix/vector arguments
  expect_message(rsecOut1<-RSEC(hdfObj, isCount=FALSE,
                                k0s=4:5,reduceMethod="none",
                                clusterFunction="hierarchical01", alphas=0.1, 
                                seqArgs=list(top.can=0), 
                                subsampleArgs=list(resamp.num=5,clusterFunction="pam"),
                                random.seed=seedValue),
                 "Merging will be done on"
  )
})

#' @name plotFeatureBoxplot
#' @title Plot boxplot of feature values by cluster
#' @description Plot a boxplot of the (transformed) values for a particular
#'   gene, separated by cluster
#' @aliases plotFeatureBoxplot
#'   plotFeatureBoxplot,ClusterExperiment,character-method
#' @rdname plotFeatureBoxplot
#' @export
setMethod(
  f = "plotFeatureBoxplot",
  signature = signature(object = "ClusterExperiment",feature="character"),
  definition = function(object, feature,whichCluster="primary",...)
  {
    m<-match(feature,rownames(object))
    if(is.na(m)) stop("feature does not match one of the rownames of this object")
    else invisible(plotFeatureBoxplot(object,whichCluster=whichCluster,feature=m,...))
  })



#' @param feature identification of feature to plot, either row name or index
#' @param unassignedColor If not NULL, should be character value giving the
#'   color for unassigned (-2) samples (overrides \code{clusterLegend}) default.
#' @param missingColor If not NULL, should be character value giving the color
#'   for missing (-2) samples (overrides \code{clusterLegend}) default.
#' @param main title of plot. If NULL, given default title.
#' @param plotUnassigned whether to plot the unassigned samples as a cluster
#'   (either -1 or -2)
#' @param whichAssay numeric or character specifying which assay to use. See
#'   \code{\link[SummarizedExperiment]{assay}} for details.
#' @param ... arguments passed to \code{\link{boxplot}}
#' @inheritParams plotReducedDims
#' @seealso \code{\link{boxplot}}
#' @rdname plotFeatureBoxplot
#' @return A plot is created. The output of boxplot is returned (see
#' \code{\link{boxplot}}), with additional elements \code{colors} and 
#'  \code{clusterIds} that gives the colors and internal ids that match 
#'  each boxplot (pulled from \code{clusterLegend} but in the order of plot)
#' @examples
#' data(simData)
#' #Create a ClusterExperiment object
#' cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reducedDim="PCA",
#'    clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
#'    removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)
#' #give names to the clusters
#' cl<-renameClusters(cl, whichCluster=1, 
#'    value=letters[1:nClusters(cl)[1]])
#' plotFeatureBoxplot(cl,feature=1)
#' @importFrom graphics boxplot
#' @export
setMethod(
  f = "plotFeatureBoxplot",
  signature = signature(object = "ClusterExperiment",feature="numeric"),
  definition = function(object,  feature,whichCluster="primary",plotUnassigned=FALSE,unassignedColor=NULL,missingColor=NULL,main=NULL,whichAssay=1,...)
  {
    whCl<-getSingleClusterIndex(object,whichCluster,list(...))
    #get data:
    dat<-transformData(object, whichAssay=whichAssay)[feature,]
    clLegend<-clusterLegend(object)[[whCl]]
    uniqueNames<-length(unique(clLegend[,"name"]))==nrow(clLegend)
    #-----
		#put in alpha order by cluster name / id (except for the missing ones)
    #-----
    whNotMissing<-which(as.numeric(clLegend[,"clusterIds"])>0)
    if(length(whNotMissing)>0){
      orderedLegend<-clLegend[whNotMissing,,drop=FALSE]
      if(uniqueNames) orderedLegend<-orderedLegend[order(orderedLegend[,"name"]), ,drop=FALSE]
      else orderedLegend<-orderedLegend[order(orderedLegend[,"name"],orderedLegend[,"clusterIds"]), ,drop=FALSE]
    }
		else orderedLegend<-clLegend
    #-----
		#if plot unassigned, then add unassigned back to the end of group
    #-----
    whMissing<-which(as.numeric(clLegend[,"clusterIds"])<0)
    if(length(whMissing)>0 & plotUnassigned){
      if(length(whNotMissing)>0) orderedLegend<-rbind(orderedLegend,clLegend[whMissing, ,drop=FALSE])
      else orderedLegend<-clLegend[whMissing, ,drop=FALSE]
      if(!is.null(unassignedColor) & any(orderedLegend[,"clusterIds"]=="-1"))
        orderedLegend[orderedLegend[,"clusterIds"]=="-1","color"]<-unassignedColor
      if(!is.null(missingColor) & any(orderedLegend[,"clusterIds"]=="-2"))
        orderedLegend[orderedLegend[,"clusterIds"]=="-2","color"]<-missingColor
    }
    clLegend<-orderedLegend
    if(uniqueNames){
      cl<-clusterMatrixNamed(object)[,whCl]
      cl<-factor(cl,levels=orderedLegend[,"name"])
    }
    else{
      warning("Non-unique names for the clusters. Will order them by internal cluster ids")
      cl<-clusterMatrix(object)[,whCl,drop=FALSE]
    }
    if(!is.null(dim(cl))){
      if(ncol(cl)>1) stop("only a single cluster may be used in whichCluster")
      else cl<-cl[,1]
    }
    if(is.null(main)){
      if(!is.null(rownames(object))) main<-paste("Gene expression of",rownames(object)[feature])
      else paste("Gene expression of feature, index number",feature)
    }
    cl<-factor(cl,levels=if(uniqueNames) clLegend[,"name"] else clLegend[,"clusterIds"])
    bxpOut<-graphics::boxplot(as.vector(dat) ~ cl, names=clLegend[,"name"],main=main,col=clLegend[,"color"],...)
		bxpOut<-c(bxpOut,list(colors=clLegend[,"color"],clusterIds=clLegend[,"clusterIds"]))
		invisible(bxpOut)
  })

#'@rdname plottingFunctions
#'@name plottingFunctions
#'@aliases makeBlankData
#'@param data matrix with samples on columns and features on rows.
#'@param groupsOfFeatures list, with each element of the list containing a
#'  vector of numeric indices of features (rows).
#'@param groupsOfSamples list, with each element of the list containing a vector
#'  of numeric indices of samples (columns).
#'@param nBlankFeatures the number of blank lines to add in the data matrix to
#'  separate the groups of feature indices (will govern the amount of white
#'  space if data is then fed to heatmap.)
#'@param nBlankSamples the number of blank lines to add in the data matrix to
#'  separate the groups of sample indices (will govern the amount of white space
#'  if data is then fed to heatmap.)
#'@details \code{makeBlankData} pulls the data corresponding to the row indices
#'  in \code{groupsOfFeatures} adds lines of NA values into data between these
#'  groups. When given to heatmap, will create white space between these groups
#'  of features.
#'@seealso \code{\link{plotHeatmap}}
#'@return \code{makeBlankData} returns a list with items 
#'\itemize{
#'  \item{"dataWBlanks"}{ The data with the rows of NAs separating the given
#'  indices.} 
#'  \item{"rowNamesWBlanks"}{ A vector of characters giving the
#'  rownames for the data, including blanks for the NA rows. These are not given
#'  as rownames to the returned data because they are not necessarily unique.
#'  However, they can be given to the \code{labRow} argument of
#'  \code{\link[NMF]{aheatmap}} or \code{\link{plotHeatmap}}.}
#'  \item{"colNamesWBlanks"}{ A vector of characters giving the colnames for the
#'  data, including blanks for the NA rows. They can be given to the
#'  \code{labCol} argument of \code{\link[NMF]{aheatmap}} or
#'  \code{\link{plotHeatmap}}.} 
#'  \item{"featureGroupNamesWBlanks"}{ A vector of
#'  characters of the same length as the number of rows of the new data (i.e.
#'  with blanks) giving the group name for the data, indicating which group
#'  (i.e. which element of \code{groupsOfFeatures} list) the feature came from.
#'  If \code{groupsOfFeatures} has unique names, these names will be used, other
#'  wise "Feature Group1", "Feature Group2", etc. The NA rows are given NA
#'  values. } 
#'  \item{"sampleGroupNamesWBlanks"}{ A vector of characters of the
#'  same length as the number of columns of the new data (i.e. with blanks)
#'  giving the group name for the data, indicating which group (i.e. which
#'  element of \code{groupsOfFeatures} list) the feature came from. If
#'  \code{groupsOfFeatures} has unique names, these names will be used, other
#'  wise "SampleGroup1", "Group2", etc. The NA rows are given NA values. } }
#'  
#' @export
#'
#' @examples
#' data(simData)
#'
#' x <- makeBlankData(simData[,1:10], groupsOfFeatures=list(c(5, 2, 3), c(20,
#' 34, 25)))
#' plotHeatmap(x$dataWBlanks,clusterFeatures=FALSE)
makeBlankData <- function(data,groupsOfFeatures=NULL,groupsOfSamples=NULL,nBlankFeatures = 1, nBlankSamples=1) {
	###Checks on input arguments
	if(is.null(groupsOfFeatures)& is.null(groupsOfSamples))
		stop("One of 'groupsOfFeatures' and 'groupsOfSamples' must be non-NULL")
  if (!is.null(groupsOfFeatures) && !is.list(groupsOfFeatures))
    stop("groupsOfFeatures must be a list")
  if (!is.null(groupsOfSamples) && !is.list(groupsOfSamples))
    stop("groupsOfSamples must be a list")

  if(!is.null(groupsOfFeatures)){
		testFeatureIndices <- sapply(groupsOfFeatures,function(x) {
    	is.numeric(x) && all(x %in% seq_len(NROW(data)))
  	})
	  if (!all(testFeatureIndices))
	    stop("Invalid list of indices in groupsOfFeatures.")
	}
	
  if(!is.null(groupsOfSamples)){
		testSampleIndices <- sapply(groupsOfSamples,function(x) {
	    is.numeric(x) && all(x %in% seq_len(NCOL(data)))
	  })
	  if (!all(testSampleIndices))
	    stop("Invalid list of indices in groupsOfSamples.")
	}

  if(is.null(rownames(data))) row.names(data)<-as.character(seq_len(nrow(data)))
	if(is.null(colnames(data))) colnames(data)<-as.character(seq_len(ncol(data)))
	rnames<-rownames(data)
	cnames<-colnames(data)
	####
	#Features
	######
	if(!is.null(groupsOfFeatures)){
	  #make list of data of feature groups
	  dataList <- lapply(groupsOfFeatures,function(ii){data[ii,,drop=FALSE]})
	  rnames <- lapply(groupsOfFeatures,function(ii){rnames[ii]})
	  #remove rownames to avoid warning
	  dataList<-lapply(dataList,function(x){
		  row.names(x)<-NULL
		  return(x)
	  })
	  #add NA rows between groups
	  naData <- matrix(NA,nrow = nBlankFeatures,ncol = ncol(data))
	  dataListMinus <- lapply(dataList[-length(dataList)],function(x) {
		  rbind(x,naData)
	  })  
	  newData <-
	    data.frame(do.call("rbind",c(dataListMinus,dataList[length(dataList)])))

	  #make names for this
	  rnamesMinus <-
	    lapply(head(rnames,-1),function(x) {
	      c(x,rep("",nBlankFeatures))
	    })
	  rnames <- unname(c(unlist(rnamesMinus),rnames[[length(rnames)]]))
  
  
	  if(!is.null(names(groupsOfFeatures)) && length(unique(names(groupsOfFeatures)))==length(names(groupsOfFeatures)) ){
		  gNames<-names(groupsOfFeatures)
	  }
	  else gNames<-paste("FeatureGroup",seq_along(groupsOfFeatures),sep="")
	  groupNames<- lapply(seq_along(dataList),function(i){
		  gname<-rep(gNames[i],times=length(groupsOfFeatures[[i]]))
		  if(i!=length(dataList)) gname<-c(gname,rep(NA,nBlankFeatures))
		  return(gname)
	  })
	  featureGroupNames<-unname(unlist(groupNames))
		data<-newData
		
	}
	else{
		rnames<-NULL
		featureGroupNames<-NULL
	}
	####
	#Samples
	######
	if(!is.null(groupsOfSamples)){
	  #make list of data of feature groups
	  dataList <- lapply(groupsOfSamples,function(ii){data[,ii,drop=FALSE]})
	  cnames <- lapply(groupsOfSamples,function(ii){cnames[ii]})
	  #remove rownames to avoid warning
	  dataList<-lapply(dataList,function(x){
			colnames(x)<-NULL
					  return(x)
	  })
	  #add NA columns between groups
	  naData <- matrix(NA,ncol = nBlankSamples,nrow = nrow(data),dimnames=NULL)
	  dataListMinus <- lapply(dataList[-length(dataList)],function(x) {
		  cbind(x,naData)
	  })  
	  newData <- unname(data.frame(do.call("cbind",c(dataListMinus,dataList[length(dataList)]))))
	  #make names for this
	  cnamesMinus <-
	    lapply(head(cnames,-1),function(x) {
	      c(x,rep("",nBlankSamples))
	    })
	  cnames <- unname(c(unlist(cnamesMinus),cnames[[length(cnames)]]))
  
  
	  if(!is.null(names(groupsOfSamples)) && length(unique(names(groupsOfSamples)))==length(names(groupsOfSamples)) ){
		  gNames<-names(groupsOfSamples)
	  }
	  else gNames<-paste("SampleGroup",seq_along(groupsOfSamples),sep="")
	  groupNames<- lapply(seq_along(dataList),function(i){
		  gname<-rep(gNames[i],times=length(groupsOfSamples[[i]]))
		  if(i!=length(dataList)) gname<-c(gname,rep(NA,nBlankSamples))
		  return(gname)
	  })
	  sampleGroupNames<-unname(unlist(groupNames))
		data<-newData
		
	}
	else{
		cnames<-NULL
		sampleGroupNames<-NULL
	}

  #can't set rownames because not unique values!
  #rownames(newData)<-rnames
  return(list(dataWBlanks = data,rowNamesWBlanks = rnames,colNamesWBlanks=cnames, featureGroupNamesWBlanks=featureGroupNames,sampleGroupNamesWBlanks=sampleGroupNames))

}

#' @name addClusterings
#' @title Add clusterings to ClusterExperiment object
#' @description Function for adding new clusterings in form of vector (single
#'   cluster) or matrix (multiple clusterings) to an existing ClusterExperiment
#'   object
#' @param x a ClusterExperiment object
#' @param y additional clusters to add to x. Can be a ClusterExperiment object
#'   or a matrix/vector of clusters.
#' @param clusterLabels label(s) for the clusters being added. If \code{y} a
#'   matrix, the column names of that matrix will be used by default, if
#'   \code{clusterLabels} is not given.
#' @param clusterLegend a list giving the cluster legend for the clusters added.
#' @aliases addClusterings removeClusterings
#'   addClusterings,ClusterExperiment,matrix-method
#' @inheritParams ClusterExperiment-class
#' @return A \code{ClusterExperiment} object.
#' @details addClusterings adds y to x, and is thus not symmetric in the two
#'   arguments. In particular, the \code{primaryCluster}, all of the dendrogram
#'   information, the merge information, \code{coClustering}, and 
#'   \code{orderSamples} are all kept from
#'   the x object, even if y is a ClusterExperiment.
#' @export
#' @examples
#' data(simData)
#'
#' cl1 <- clusterSingle(simData, subsample=FALSE,
#' sequential=FALSE, mainClusterArgs=list(clusterArgs=list(k=3), 
#' clusterFunction="pam"))

#' cl2 <- clusterSingle(simData, subsample=FALSE,
#' sequential=FALSE, mainClusterArgs=list(clusterArgs=list(k=3), 
#' clusterFunction="pam"))
#'
#' addClusterings(cl1, cl2)
setMethod(
  f = "addClusterings",
  signature = signature("ClusterExperiment", "matrix"),
  definition = function(x, y, clusterTypes="User",clusterLabels=NULL,clusterLegend=NULL) {
    if(!is.null(clusterLabels)){
      if(length(clusterLabels)!=ncol(y)) stop("clusterLabels must vector of length equal to the number of clusterings (columns of y)")
      colnames(y)<-clusterLabels
    }
    ccObj<-ClusterExperiment(x,
                             clusters=y,
                             transformation=transformation(x),
                             clusterTypes=clusterTypes,
                             checkTransformAndAssay=FALSE,
                             clusterLegend=clusterLegend)
    addClusterings(x,ccObj)
  }
)

#' @rdname addClusterings
#' @param transferFrom If x and y are both \code{ClusterExperiment} objects
#'   indicates from which object the clustering info should be taken (regarding merging, dendrogram, etc). Does not affect the order of the clusterings, which will always be the clusterings of x, followed by those of y (along with slots `clusterType`, `clusterInfo`, `clusterLegend`)
#' @param mergeCEObjects logical If x and y are both \code{ClusterExperiment} objects indicates as to whether should try to grab in the information missing from x from y (or vice versa if transferFrom=y).

#' @export
setMethod(
    f = "addClusterings",
    signature = signature("ClusterExperiment", "ClusterExperiment"),
    definition = function(x, y, transferFrom=c("x","y"),mergeCEObjects=FALSE) 
{
    transferFrom<-match.arg(transferFrom)
    if(!all(dim(assay(y)) == dim(assay(x))) || !all(assay(y) == assay(x))) {
      stop("Cannot merge clusters from different data.")
    }
    ## FIXME: does make copy -- could be much more memory than before. 
    if(transferFrom=="x") retval<-x
    else retval<-y
    retval@clusterMatrix <- cbind(x@clusterMatrix, y@clusterMatrix)
    retval@clusterTypes <- c(x@clusterTypes, y@clusterTypes)
    retval@clusterInfo<-c(x@clusterInfo,y@clusterInfo)
    retval@clusterLegend<-c(x@clusterLegend,y@clusterLegend)
    if(any(duplicated(colnames(retval@clusterMatrix)))){
      colnames(retval@clusterMatrix)<-
          make.names(colnames(retval@clusterMatrix),unique=TRUE)
    }
    if(transferFrom=="y"){
        retval@dendro_index<-y@dendro_index+nClusterings(x)
        retval@merge_index<-y@merge_index+nClusterings(x) #update index to where merge from
        retval@merge_dendrocluster_index <- 
            y@merge_dendrocluster_index + nClusterings(x)
        if(.typeOfCoClustering(y)=="indices")
            retval@coClustering<-y@coClustering+nClusterings(x)
    }

    if(mergeCEObjects){
        ### If missing in transfer object, pull from the other one
        if(transferFrom=="x"){
            addedObj<-y
            defaultObj<-x
        }
        else{
            addedObj<-x
            defaultObj<-y
        }
        if(is.na(retval@dendro_index) & !is.na(addedObj@dendro_index)){
            retval@dendro_samples<-addedObj@dendro_samples
            retval@dendro_clusters<-addedObj@dendro_clusters
            if(transferFrom=="x") 
                retval@dendro_index <- 
                    addedObj@dendro_index+ nClusterings(defaultObj) 
        }
        if(is.na(retval@merge_index) & !is.na(addedObj@merge_index)){
            if(transferFrom=="x") 
                retval@merge_index<-addedObj@merge_index+nClusterings(defaultObj) 
            retval@merge_nodeMerge<-addedObj@merge_nodeMerge
            retval@merge_cutoff<-addedObj@merge_cutoff
            retval@merge_method<-addedObj@merge_method
            retval@merge_demethod<-addedObj@merge_demethod
        }
        if(is.null(retval@merge_nodeProp) & !is.null(addedObj@merge_nodeProp)){
            retval@merge_nodeProp<-addedObj@merge_nodeProp
            retval@merge_dendrocluster_index<-
                addedObj@merge_dendrocluster_index+nClusterings(defaultObj)                }
        #put back orderSamples
        if(all(retval@orderSamples==seq_len(nSamples(retval))) & 
            !all(addedObj@orderSamples==seq_len(nSamples(retval)))) 
                retval@orderSamples<-addedObj@orderSamples
        if(is.null(retval@coClustering)){
            if(transferFrom=="x" & .typeOfCoClustering(addedObj)=="indices")    
                retval@coClustering<-addedObj@coClustering + nClusterings(defaultObj)
            else retval@coClustering<-addedObj@coClustering
        }
    }
    retval<-.unnameClusterSlots(retval) #just gets rid of the names of objects that shouldn't have them

    ch<-.checkClusterMatrix(retval)
    if(!is.logical(ch)) stop(ch)
    ch<-.checkClusterTypes(retval)
    if(!is.logical(ch)) stop(ch)
    ch<-.checkClusterLegend(retval)
    if(!is.logical(ch)) stop(ch)
    #would it be less memory to do a call to "new"? What is difference versus having to check dendrogram, coClustering, etc if they exists? Should do checks on large data.
    return(retval)
  }
)

#' @rdname addClusterings
#' @export
#' @param ... For \code{addClusterings}, passed to signature
#'   \code{ClusterExperiment,matrix}. For \code{[} (subsetting), passed to
#'   \code{SingleCellExperiment} subsetting function.
#' @param makePrimary whether to make the added cluster the primary cluster
#'   (only relevant if \code{y} is a vector)
setMethod(
  f = "addClusterings",
  signature = signature("ClusterExperiment", "vector"),
  definition = function(x, y, makePrimary=FALSE,...) {
    mat<-matrix(y,ncol=1)
    x<-addClusterings(x,mat,...)
    if(makePrimary){
      x@primaryIndex<-ncol(clusterMatrix(x))
    }
    return(x)
  }
)

#' @title Assign unassigned samples to nearest cluster
#' @description Assigns the unassigned samples in a cluster to the nearest
#'   cluster based on distance to the medians of the clusters.
#' @name assignUnassigned
#' @rdname assignUnassigned
#' @aliases assignUnassigned,ClusterExperiment-method
#' @param object A Cluster Experiment object
#' @param whichAssay which assay to use to calculate the median per cluster and
#'   take dimensionality reduction (if requested)
#' @param clusterLabel if missing, the current cluster label of the cluster will
#'   be appended with the string "_AllAssigned".
#' @param ... arguments passed to \code{\link{getReducedData}} specifying the
#'   dimensionality reduction (if any) to be taken of the data for calculating
#'   the medians of the clusters
#' @inheritParams getClusterIndex
#' @details The function \code{assignUnassigned} calculates the median values of
#'   each variable for each cluster, and then calculates the euclidean distance
#'   of each unassigned sample to the median of each cluster. Each unassigned
#'   sample is assigned to the cluster for which it closest to the median.
#' @details All unassigned samples in the cluster are given a clustering,
#'   regardless of whether they are classified as -1 or -2.
#' @return The function \code{assignUnassigned} returns a
#'   \code{ClusterExperiment} object with the unassigned samples assigned to one
#'   of the existing clusters.
#' @seealso \code{\link{getReducedData}}
#' @examples
#' #load CE object
#' \dontrun{
#' data(rsecFluidigm)
#' smallCE<-rsecFluidigm[,1:50]
#' #assign the unassigned samples
#' assignUnassigned(smallCE, makePrimary=TRUE)
#' 
#' #note how samples are REMOVED:
#' removeUnassigned(smallCE)
#' }
#' @inheritParams addClusterings
#' @inheritParams reduceFunctions
#' @export
setMethod(
  f = "assignUnassigned",
  signature = signature("ClusterExperiment"),
  definition = function(object,whichCluster="primary",clusterLabel,
                        makePrimary=TRUE,whichAssay=1,reduceMethod="none",...){
		
    whCl<-getSingleClusterIndex(object,whichCluster,list(...))
    cl<-clusterMatrix(object)[,whCl]
		if(missing(clusterLabel)) clusterLabel<-paste0(clusterLabels(object)[whCl],"_AllAssigned")
		whichUnassigned<-which(cl<0)
		if(length(whichUnassigned)>0){
				if(length(whichUnassigned)< length(cl)){
			    ########
			    ##Transform the data
			    ########
			    datList<-getReducedData(object,reduceMethod=reduceMethod,returnValue="list",...)
			    object<-datList$objectUpdate
			    dat<-datList$dat
					###############
					#find centers of clusters based on assigned samples:
					###############
			    clFactor <- factor(cl[-whichUnassigned])
					medoids <- do.call("rbind", by(t(dat[,-whichUnassigned]), clFactor, function(z){apply(z, 2, median)}))
			    rownames(medoids) <- levels(clFactor)
			
					classif<-.genericClassify(inputMatrix=dat[,whichUnassigned],centers=medoids)
			
					###############
					#check reasonable results:
					###############
					if(any(!unique(classif)%in%unique(cl[cl>0]))) stop("programming error in classifying -- clusters not seen before given as assignment")
					if(	length(which(cl<0))!=length(classif)) stop("programming error in classifying -- not right length")
					###############
					# Assign cluster to object
					###############
					cl[cl<0]<-classif
					object<-addClusterings(object,cl, clusterLabel= clusterLabel,makePrimary=makePrimary)
					return(object)
			}
			else{
				stop("All cells are unassigned, cannot assign them")
			}
		}
		else stop("No cells are unassigned in the designated cluster")

  }
)


#' @rdname assignUnassigned
#' @aliases removeUnassigned
#' @details \code{removeUnclustered} removes all samples that are unclustered
#'   (i.e. -1 or -2 assignment) in the designated cluster of \code{object} (so
#'   they may be unclustered in other clusters found in
#'   \code{clusterMatrix(object)}).
#' @return The function \code{removeUnassigned} returns a
#'   \code{ClusterExperiment} object with the unassigned samples removed.
#' @export
setMethod(
  f = "removeUnassigned",
  signature = "ClusterExperiment",
  definition = function(object,whichCluster="primary") {
    whCl<-getSingleClusterIndex(object,whichCluster)
		cl<-clusterMatrix(object)[,whCl]
		return(object[,which(cl>= 0)])
  }
)

.onUnload <- function (libpath) {
  library.dynam.unload("clusterExperiment", libpath)
}

.mynote<-function(x){
	message(paste("Note:",x))
}

# Small function to identify what type of CoClustering information is stored in co-clustering slot
# (might make sense to make it a slot, but then have to update object, for something not so important...)
.typeOfCoClustering<-function(ceObj){
	if(is.null(ceObj@coClustering)) return("null")
    if(is.null(dim(ceObj@coClustering))) return("indices")
        else if(!isSymmetric(ceObj@coClustering)) return("clusterings")
            else return("distance")
}

.addPrefixToClusterNames<-function(ceObj,prefix,whCluster){
  ceLegend<-clusterLegend(ceObj)[[whCluster]]
  cl<-ceLegend[,"clusterIds"]
  ceLegend[,"name"]<-numericalAsCharacter(values=cl,prefix=prefix)
  clusterLegend(ceObj)[[whCluster]]<-ceLegend
  return(ceObj)
}
#' @title Convert numeric values to character that sort correctly
#' @description Small function that takes as input integer values (or values
#'   that can be converted to integer values) and converts them into character
#'   values that are 'padded' with zeros at the beginning of the numbers so that
#'   they will sort correctly.
#' @param values vector of values to be converted into sortable character values
#' @param prefix optional character string that will be added as prefix to the
#'   result
#' @details The function determines the largest value and adds zeros to the
#'   front of smaller integers so that the resulting characters are the same
#'   number of digits. This allows standard sorting of the values to correctly
#'   sort.
#' @details The maximum number of zeros that will be added is 3. Input integers
#'   beyond that point will not be correctly fixed for sorting.
#' @details Negative integers will not be corrected, but left as-is
#' @return A character vector
#' @seealso \code{\link[stringr]{str_pad}}
#' @examples
#' numericalAsCharacter(c(-1, 5,10,20,100))
#' @importFrom stringr str_pad
#' @export
numericalAsCharacter<-function(values,prefix=""){
	if(is.factor(values)) values<-as.character(values)
	values<-suppressWarnings(as.integer(values))
	if(any(is.na(values))) stop("input must convert to numeric (integer) vector")
	whPos<-which(values >0)
	if(length(whPos)>0) largestLength<-max(values[whPos])
	values<-as.character(values)
	if(length(whPos)>0){
		pad<-if(largestLength<100) 2 else if(largestLength<1000) 3 else 4			
		values[whPos]<-paste(prefix,stringr::str_pad(values[whPos],width=pad,pad="0"),sep="")
  	} 
	return(values)
}

## FIXME: Some day, if update addClusterings so could choose where to put new clusterings, could just use that so don't replicate code here.
.addNewResult<-function(newObj,oldObj){
    # want most recent addition on top of clusterMatrix
    # mergeCEObjects=TRUE means will keep any CE info in newObj, but add CE info in oldObj if not in new object (used to be done by this function, but updated `addClusterings` to do that for me so all in one place). 
    retval<-addClusterings(x=newObj, y= oldObj, 
        transferFrom="x", mergeCEObjects=TRUE) 
    retval<-.addBackSEInfo(newObj=retval,oldObj=oldObj) #make sure keeps SE info
    #   Note: .addBackSEInfo calls ClusterExperiment (i.e. validates)
    return(retval)
}

#this function keeps everything from new, except grabs SE info from old
.addBackSEInfo<-function(newObj,oldObj){
  retval<-ClusterExperiment(as(oldObj,"SingleCellExperiment"),
                            clusters=clusterMatrix(newObj),
                            transformation=transformation(newObj),
                            clusterTypes=clusterTypes(newObj),
                            clusterInfo=clusteringInfo(newObj),
                            orderSamples=orderSamples(newObj),
                            coClustering=coClustering(newObj),
                            dendro_samples=newObj@dendro_samples,
                            dendro_clusters=newObj@dendro_clusters,
                            dendro_index=newObj@dendro_index,
                            merge_index=newObj@merge_index,
                            merge_cutoff=newObj@merge_cutoff,
                            merge_dendrocluster_index=newObj@merge_dendrocluster_index,
                            merge_nodeProp=newObj@merge_nodeProp,
                            merge_nodeMerge=newObj@merge_nodeMerge,
                            merge_method=newObj@merge_method,
                            merge_demethod=newObj@merge_demethod,
                            primaryIndex=primaryClusterIndex(newObj),
                            clusterLegend=clusterLegend(newObj),
                            checkTransformAndAssay=FALSE
  )
  return(retval)
}



#Returns NULL if no sample data
.pullColData<-function(ce,wh,fixNA=c("keepNA","unassigned","missing")){
  fixNA<-match.arg(fixNA)
  if(!is.null(wh)){
    sData<-colData(ce)
    if(!is.logical(wh)){
      
      if(NCOL(sData)==0) stop("no colData for object data, so cannot pull sample data")
      if(is.character(wh)){
        if(all(wh=="all")) wh<-seq_len(NCOL(sData))
        else{
          if(!all(wh %in% colnames(sData))) stop("Invalid names for pulling sample data (some do not match names of colData)")
          else wh<-match(wh,colnames(sData))
        }
      }
      else if(is.numeric(wh)){
        if(!all(wh %in% seq_len(NCOL(sData)))) stop("Invalid indices for for pulling sample data (some indices are not in 1:NCOL(colData)")
      }
      else stop("invalid values for pulling sample data from colData of object")
      sData<-as.data.frame(sData[,wh,drop=FALSE])
    }
    else{ #if 
      if(wh) sData<- colData(ce)
      else sData<-NULL
    }
  }
  else sData<-NULL
  if(!is.null(sData) && fixNA!="keepNA"){
    newValue<-switch(fixNA,"unassigned"=-1,"missing"=-2)
    fixNAFunction<-function(x,newValue){
      if(is.factor(x)){ #change to character
        waslevels<-levels(x)
        wasFactor<-TRUE
        x<-as.character(x)
      }
      else wasFactor<-FALSE
      if(is.character(x)){
        x[which(is.na(x))]<-as.character(newValue)
        if(wasFactor) x<-factor(x,levels=c(waslevels,as.character(newValue))) #keeps order of previous factors
      }
      else x[which(is.na(x))]<- newValue #assume numeric if not character/factor
      return(x)
    }
    #have to do this; otherwise makes them all characters if use apply...
    cnames<-colnames(sData)
    sData<-do.call("data.frame",lapply(seq_len(ncol(sData)),function(ii){fixNAFunction(sData[,ii],newValue=newValue)}))
    colnames(sData)<-cnames
    
  }
  return(sData)
}

.unnameClusterSlots<-function(ce){
  names(ce@clusterLegend)<-names(ce@clusterInfo)<-names(ce@clusterTypes)<-NULL
  return(ce)
}

#.convertToNum works on vector and is basically as.numeric, so that if character valued numbers, will get them, but other wise, arbitrary values based on making input x a factor (so alphabetical).
.convertToNum<-function(x){
  nms<-names(x)
  if(is.factor(x)){
    x<-as.character(x)
  }
  # FIXME what is this doing???
  if(is.character(x)){
    op <- options(warn=2)
    #if character values convert to numeric, will use that, otherwise convert to factor first
    test <- try(as.numeric(x) ,silent=TRUE)
    if(inherits(test,"try-error")) x<-as.numeric(factor(x))
    else x<-test
    options(op)
    
  }
  names(x)<-nms
  return(x)
}
##Universal way to convert matrix of clusters (of any value) into integers, preserving -1, -2 values. Makes them adjacent values. NAs are converted into -1 values.
.makeIntegerClusters<-function(clMat){
  if(!is.matrix(clMat)) stop("must give matrix input") #means must be either character or numeric values.
  fun<-function(x){ #make it numbers from 1:length(x), except for -1,-2
    #id special values of -1,-2
    isChar<-as.character(is.character(x))
		
		#make NA values NA
		if(any(is.na(x))){
			if(is.character(x)) x[is.na(x)]<-"-1" 
			else x[is.na(x)]<- -1 
		}
    wh1<-switch(isChar,"TRUE"= x =="-1","FALSE"= x ==-1)
    wh2<-switch(isChar,"TRUE"= x =="-2","FALSE"= x ==-2)
    wh<-wh1 | wh2
    vals<-sort(unique(x[!wh])) #if input was 1:n will hopefully make output 1:n matching
    y<-match(x,vals)
    y[wh1]<- -1
    y[wh2]<- -2
    return(y)
  }
  
  if(!is.null(dim(clMat)) && ncol(clMat)>1){
    x<-apply(clMat,2,fun)
    if(is.null(dim(x))) x<-matrix(x,nrow=1) #in case clMat was matrix with 1 row
    return(x  )
  } 
  else{
    if(is.matrix(clMat)) clMat<-clMat[,1]
    return(matrix(fun(clMat),ncol=1))
  }
}
## Universal way to convert matrix of clusters into default colorLegend
## returns  list(colorList=colorList,numClusters=clMat,facClusters=clMat))
## If giving clNumMat, then will not check for consecutive integers, but will use entries in clNumMat as the clusterIds
## If matchClusterLegend given will use names and colors of matchClusterLegend
## if matchTo="numIds", assume that 'clusterIds' column matches clNumMat (so only makes sense if giving clNumMat); will keep clusterId and color of clusterLegend -- NOT name
## if matchTo="givenIds", assume 'clusterIds' matches entries of clMat; will keep name and color of clusterLegend
## if matchTo="name" assume "name" matches entries of clMat; will keep name and color of clusterLegend
## (will only match if finds name or clusterIds in matchClusterLegend, depending on value of matchTo)
## Assumes that clMat has already had continuous columns removed
## Note, does drop levels, so returned datasets as well as color legend doesn't include missing factor levels
.makeColors<-function(clMat, colors,clNumMat=NULL,unassignedColor="white",missingColor="grey", distinctColors=FALSE,
                      matchClusterLegend=NULL,matchTo=c("givenIds","numIds","name")){ 
  
  matchTo<-match.arg(matchTo)
  if(!is.null(matchClusterLegend)){
    if(!is.list(matchClusterLegend) ) matchClusterLegend<-.convertToClusterLegend(matchClusterLegend)
    if("matchTo"%in% c("numIds","givenIds")) reqNames<-c("color","clusterIds") else reqNames<-c("color","name")
    ch<-.checkClusterLegendList(matchClusterLegend,allowNames=TRUE,reqNames=reqNames)
    if(!is.logical(ch)){
      #try again in case in aheatmap format
      if(!all(sapply(matchClusterLegend, function(x) {!is.null(dim(x))}))) matchClusterLegend<-.convertToClusterLegend(matchClusterLegend)
      ch<-.checkClusterLegendList(matchClusterLegend,allowNames=TRUE,reqNames=reqNames)
      if(!is.logical(ch)) stop(ch)
      
    }
		if(is.null(names(matchClusterLegend)) && !is.null(colnames(clMat)) && length(matchClusterLegend)==ncol(clMat)){
			names(matchClusterLegend)<-colnames(clMat)
		}
  }
  if(ncol(clMat)==1) distinctColors<-FALSE
  if(any(apply(clMat,2,function(x){length(unique(x))})>length(colors))) warning("too many clusters to have unique color assignments")
    ###(not sure why this simpler code doesn't give back data.frame with factors: annCol<-apply(clMat,2,function(x){factor(x)}))
		
	cNames<-colnames(clMat)
	#if give clNumMat, can skip this step, save time
	if(is.null(clNumMat)) clNumMat<-.makeIntegerClusters(as.matrix(clMat)) #require to not skip a integer value
	allFactors<-all(apply(clMat,2,function(x){is.factor(x)}))
	if(!allFactors){
		clMat<-do.call("data.frame", lapply(seq_len(ncol(clMat)), function(ii){ 
		  x<-clMat[,ii]
		  if(!is.factor(x)) return(factor(clMat[,ii]))
		  else return(droplevels(x))
		  }))
  	names(clMat)<-cNames
	}
	
  maxPerCol<-apply(clNumMat,2,max) #max cluster value (not including -1,-2)
  currcolors<-rep(colors,length= max(c(length(colors),sum(maxPerCol)))) #make sure don't run out of colors. Put max in case none are >0
  if(distinctColors) maxPreviousColor<-c(0,head(cumsum(maxPerCol),-1)) 
		
	#make a clusterLegend list
	perColumnFunction<-function(ii){
    facInt<-clNumMat[,ii] #assumes adjacent numbers
    nVals<-max(c(facInt,0))
    facOrig<-clMat[,ii] #assumes factors
    matchName<-cNames[[ii]]
		if(!is.null(matchClusterLegend[[matchName]])){
	    colMat<-matchClusterLegend[[matchName]]
			if(matchTo=="numIds"){
			  m<-match(colMat[,"clusterIds"],as.character(facInt))
			  cols<-cbind(colMat[,c("clusterIds","color"),drop=FALSE],"name"=as.character(facOrig)[m])
			}
			else if(matchTo=="givenIds"){
			  m<-match(colMat[,"clusterIds"],as.character(facOrig))
			 cols<-cbind("clusterIds"=facInt[m],colMat[,c("color","name"),drop=FALSE]) 			}
			else{
			  m<-match(colMat[,"name"],as.character(facOrig))
			  cols<-cbind("clusterIds"=facInt[m],colMat[,c("color","name"),drop=FALSE])
			}
	    if(any(is.na(m))) cols<-cols[!is.na(m),,drop=FALSE] #incase given legend has names/Ids not found in data
	    #in case given legend doesn't have values found in data
	    whMissing<-which(!as.character(facInt)[facInt>0] %in% cols[,"clusterIds"])
	    if(length(whMissing)>0){ 
	      #remove colors already in colMat
	      colors<-currcolors
	      mColors<-.matchColors(colors,cols[,"color"])
	      if(any(!is.na(mColors))) colors<-colors[-!is.na(mColors)]
	      mAdd<-match(unique(facInt[facInt>0][whMissing]),facInt) #make unique value
	      addMat<-cbind("clusterIds"=as.character(facInt[mAdd]),"color"=colors[seq_along(mAdd)],"name"=as.character(facOrig[mAdd]))
        cols<-rbind(cols,addMat)
	     }
	    
		}
		else{
		  if(nVals>0){
		    if(distinctColors){
		      add<-maxPreviousColor[[ii]]
		      colors<-currcolors[seq_len( nVals)+add]
		    }
		    else colors<-currcolors[seq_len( nVals)]
        #make matrix for non -1 values
		    clIds<-sort(unique(facInt[facInt>0]))
        m<-match(clIds,facInt)
		    cols<-cbind(
		      "clusterIds"=as.character(clIds),
		      "color"=colors,
		      "name"=as.character(facOrig[m])
		    )
		  }
		  else{ #all <0 cluster values
		    cols<-matrix(nrow=0,ncol=3)
		    colnames(cols)<-c("clusterIds","color","name")
		  }
	    if(any(facInt== -1)) 
				cols<-rbind(cols,c("clusterIds"="-1","color"=unassignedColor,"name"="-1") )
	    if(any(facInt== -2)) 
				cols<-rbind(cols,c("clusterIds"="-2","color"=missingColor,"name"="-2") )
		}
    cols<-cols[order(cols[,"name"]),,drop=FALSE]
		rownames(cols)<-NULL
    return(cols)
	}
	colorList<-lapply(seq_len(ncol(clNumMat)),FUN=perColumnFunction)
  names(colorList)<-cNames
  return(list(colorList=colorList,numClusters=clNumMat,facClusters=clMat))
}

#' @importFrom grDevices col2rgb 
.matchColors<-function(x,y){
  #match x colors to y colors, only with rgb definitions
  if(!is.character(x) || !is.character(y)) stop("colors must be character values")
  xrgb<-col2rgb(x)
  yrgb<-col2rgb(y)
  match(data.frame(xrgb), data.frame(yrgb))
}

#' @title Make hierarchy of set of clusters
#'
#' @aliases makeDendrogram,ClusterExperiment-method
#' @description Makes a dendrogram of a set of clusters based on hclust on the
#'   medoids of the cluster.
#' @param x data to define the medoids from. Matrix and
#'   \code{\link{ClusterExperiment}} supported.
#' @param cluster A numeric vector with cluster assignments. If x is a
#'   ClusterExperiment object, cluster is automatically the primaryCluster(x).
#'   ``-1'' indicates the sample was not assigned to a cluster.
#' @param unassignedSamples how to handle unassigned samples("-1") ; only
#'   relevant for sample clustering. See details.
#' @param reduceMethod character A character identifying what type of
#'   dimensionality reduction to perform before clustering. Can be either a
#'   value stored in either of reducedDims or filterStats slot or a built-in
#'   diminsionality reduction/filtering. The option "coCluster" will use the
#'   co-Clustering matrix stored in the 'coClustering' slot of the
#'   \code{ClusterExperiment} object
#' @param calculateSample only relevant for \code{matrix} or \code{dist}
#'  version of function. Indicates whether to calculate the sample dendrogram.
#' @param nDims The number of dimensions to keep from \code{reduceMethod}. If
#'   missing calls \code{\link{defaultNDims}}.
#' @param ... for makeDendrogram, if signature \code{matrix}, arguments passed
#'   to hclust; if signature \code{ClusterExperiment} passed to the method for
#'   signature \code{matrix}. For plotDendrogram, passed to
#'   \code{\link{plot.dendrogram}}.
#' @inheritParams clusterSingle
#' @inheritParams reduceFunctions
#' @inheritParams getClusterIndex
#' @details The function returns two dendrograms (as a list if x is a matrix or
#'   in the appropriate slots if x is ClusterExperiment). The cluster dendrogram
#'   is created by applying \code{\link{hclust}} to the medoids of each cluster.
#'   In the sample dendrogram the clusters are again clustered, but now the
#'   samples are also part of the resulting dendrogram. This is done by giving
#'   each sample the value of the medoid of its cluster.
#' @details The argument \code{unassignedSamples} governs what is done with
#'   unassigned samples (defined by a -1 cluster value). If
#'   unassigned=="cluster", then the dendrogram is created by hclust of the
#'   expanded medoid data plus the original unclustered observations. If
#'   \code{unassignedSamples} is "outgroup", then all unassigned samples are put
#'   as an outgroup. If the \code{x} object is a matrix, then
#'   \code{unassignedSamples} can also be "remove", to indicate that samples
#'   with "-1" should be discarded. This is not a permitted option, however,
#'   when \code{x} is a \code{ClusterExperiment} object, because it would return
#'   a dendrogram with less samples than \code{NCOL(x)}, which is not permitted
#'   for the \code{@dendro_samples} slot.
#' @details If any merge information is stored in the input object, it will be
#'   erased by a call to makeDendrogram.
#' @return If x is a matrix, a list with two dendrograms, one in which the
#'   leaves are clusters and one in which the leaves are samples. If x is a
#'   ClusterExperiment object, the dendrograms are saved in the appropriate
#'   slots.
#'
#' @export
#' @seealso makeFilterStats, makeReducedDims
#' @examples
#' data(simData)
#'
#' #create a clustering, for 8 clusters (truth was 3)
#' cl <- clusterSingle(simData, subsample=FALSE,
#' sequential=FALSE, mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=8)))
#'
#' #create dendrogram of clusters:
#' hcl <- makeDendrogram(cl)
#' plotDendrogram(hcl)
#' plotDendrogram(hcl, leafType="samples",plotType="colorblock")
#'
#' @name makeDendrogram
#' @rdname makeDendrogram
setMethod(
    f = "makeDendrogram",
    signature = "ClusterExperiment",
    definition = function(x, whichCluster="primaryCluster",reduceMethod="mad",
                          nDims=defaultNDims(x,reduceMethod),filterIgnoresUnassigned=TRUE,
                          unassignedSamples=c("outgroup", "cluster"),
                          whichAssay=1,...)
    {
        passedArgs<-list(...)
        
        checkIgnore<-.depricateArgument(passedArgs=passedArgs,"filterIgnoresUnassigned","ignoreUnassignedVar") #06/2018 added in BioC 3.8
        if(!is.null(checkIgnore)){
            passedArgs<-checkIgnore$passedArgs
            filterIgnoresUnassigned<-checkIgnore$val
        }
        unassignedSamples<-match.arg(unassignedSamples)
        whCl<-getSingleClusterIndex(x,whichCluster,passedArgs)
        cl<-clusterMatrix(x)[,whCl]
        ##erase merge information
        if(!is.na(mergeClusterIndex(x)) || !is.na(x@merge_dendrocluster_index)) x<-.eraseMerge(x)
        
        ########
        ##Transform the data
        ########
        if(length(reduceMethod)>1) stop('makeDendrogram only takes one choice of "reduceMethod" as argument')
        if(reduceMethod!="coCluster"){
            #need to change name of reduceMethod to make it match the
            #clustering information if that option chosen.
            datList<-getReducedData(object=x,whichCluster=whCl,reduceMethod=reduceMethod,
                                    nDims=nDims,filterIgnoresUnassigned=TRUE,  whichAssay=whichAssay,returnValue="list")
            x<-datList$objectUpdate
            dat<-datList$dat
            
            outlist <- do.call("makeDendrogram",c(list(
                x=dat, 
                cluster=cl,calculateSample=TRUE,
                unassignedSamples=unassignedSamples),
                passedArgs))
        }
        else{
            if(is.null(x@coClustering)) stop("Cannot choose 'coCluster' if 'coClustering' slot is empty. Run makeConsensus before running 'makeDendrogram' or choose another option for 'reduceMethod'")
            if(is.null(dimnames(x@coClustering))) stop("This ClusterExperiment object was made with an old version of clusterExperiment and did not give dimnames to the coClustering slot.")
            outlist<-do.call("makeDendrogram",c(list(
                x=as.dist(1-x@coClustering),
                cluster=cl,calculateSample=TRUE,
                unassignedSamples=unassignedSamples),
                passedArgs)) 
        }
        
        #Add clusterNames as Ids to cluster and sample dendrogram.
		x@dendro_clusters <- outlist$clusters
		phylobase::tipLabels(x@dendro_clusters)<-NA #erase any labels of the tips, internal nodes already have the defaults.
		x@dendro_samples <- outlist$samples #labels should have been erased already
        x@dendro_index<-whCl
        ch<-.checkDendrogram(x)
        if(!is.logical(ch)) stop(ch)
        return(x)
})



#' @rdname makeDendrogram
#' @export
setMethod(
    f = "makeDendrogram",
    signature = "dist",
    definition = function(x, cluster,
                          unassignedSamples=c("outgroup", "cluster", "remove"),
                          calculateSample=TRUE,...) {
        unassigned <- match.arg(unassignedSamples)
        if(is.null(attributes(x)$Labels)) {
            attributes(x)$Labels <- .makeSampleNames(seq_len(nSamples))
        }
		if(!all(is.na(suppressWarnings(as.numeric(attributes(x)$Labels ))))){
			warning("Cannot use the attributes(x)$Labels because they are numbers. Making sample names")
			sampleNames<-.makeSampleNames(seq_len(nSamples))
		}
		
        clusterD<-.makeClusterDendro(x,cluster,type="dist",...)  
        fullD<-.makeSampleDendro(x,clusterDendro=clusterD, cl=.convertToNum(cluster), type=c("dist"), unassignedSamples=unassigned,sampleEdgeLength=0,  outbranchLength=1,calculateSample=calculateSample)
        return(list(samples=fullD,clusters=clusterD))
    })

#' @rdname makeDendrogram
#' @export
setMethod(
    f = "makeDendrogram",
    signature = "matrixOrHDF5",
    definition = function(x, cluster,
                          unassignedSamples=c("outgroup", "cluster", "remove"),
                          calculateSample=TRUE,...) {
		
        unassigned <- match.arg(unassignedSamples)
        if(is.null(colnames(x))) {
            colnames(x) <- .makeSampleNames(seq_len(ncol(x)))
        }
		if(!all(is.na(suppressWarnings(as.numeric(colnames(x) ))))){
			warning("Cannot use the colnames(x) because they are numbers. Making sample names")
			sampleNames<-.makeSampleNames(seq_len(ncol(x)))
		}
        clusterD<-.makeClusterDendro(x,cluster,type="mat",...)
        fullD<- .makeSampleDendro(x,clusterDendro=clusterD, cl=.convertToNum(cluster), type=c("mat"), unassignedSamples=unassigned, sampleEdgeLength=0,  outbranchLength=1,calculateSample=calculateSample)
        
        return(list(samples=fullD,clusters=clusterD))
    })

#' @param x a matrix or hdf5 or distance
#' @param cl vector of cluster info
#' @param type whether input x is a matrix or distance.
#' @noRd
.makeClusterDendro<-function(x, cl,type=c("mat","dist"),...){
    type<-match.arg(type)
    if(type=="dist"){
        nSamples<-  attributes(x)$Size
        if(nSamples != length(cl)) {
            stop("cluster must be the same length as the number of samples")
        }
    }
    if(type=="mat"){
        if(ncol(x) != length(cl)) {
            stop("cluster must be the same length as the number of samples")
        }
     }
    
    clNum<-.convertToNum(cl)
    
    #############
    # Cluster dendrogram
    #############
    whKeep <- which(clNum >= 0) #remove -1, -2
    if(length(whKeep) == 0) stop("all samples have clusterIds<0")
    if(length(unique(cl[whKeep]))==1) stop("Only 1 cluster given. Can not make a dendrogram.")
    clFactor <- factor(cl[whKeep])
    
    #each pair of clusters, need to get median of the distance values
    #do a double by, just returning the values as a vector, and then take the median
    if(type=="dist"){
        medoids<-do.call("rbind", by(as.matrix(x)[whKeep,whKeep], clFactor, function(z){
            out<-as.vector(by(t(z),clFactor,function(y){median(as.vector(unlist(y)))}))
            names(out)<-levels(clFactor)
            return(out)
        }))
        diag(medoids)<-0 #incase the median of within is not zero...
        rownames(medoids) <- levels(clFactor)
        colnames(medoids) <- levels(clFactor)
    }
    if(type=="mat"){
        medoids <- do.call("rbind", by(t(x[,whKeep]), clFactor, function(z){apply(z, 2, median)}))
        rownames(medoids) <- levels(clFactor)
    }
    nPerCluster <- table(clFactor)
    clusterD<-if(type=="dist").convertToPhyClasses(stats::hclust(as.dist(medoids),members=nPerCluster,...),returnClass=c("phylo4")) else .convertToPhyClasses(stats::hclust(dist(medoids)^2,members=nPerCluster,...),returnClass=c("phylo4"))
       
   #create data for phylo4d object
   clusterNodes<-phylobase::getNode(clusterD,type="all") 
   clusterIdDendro<-paste("ClusterId",names(clusterNodes),sep="")
   clusterIdDendro[is.na(names(clusterNodes))]<-NA
   
   #make permanent node ids, starting with internal nodes.
   justNodes<-phylobase::getNode(clusterD,type="internal")
   justTips<-phylobase::getNode(clusterD,type="tip")
   nodeId<-rep(NA,length(clusterNodes))  #want tips last, and internal nodes first
   nodeId[match(justNodes,clusterNodes)]<-seq_along(justNodes)
   nodeId[match(justTips,clusterNodes)]<-seq_along(justTips)+length(justNodes)
   if(!all(sort(nodeId)==sort(seq_along(nodeId)))) stop("coding error in giving node names")
   
   data.cl <- data.frame(NodeId = paste("NodeId",nodeId,sep=""), ClusterIdDendro = clusterIdDendro, ClusterIdMerge= rep(NA,length(clusterNodes)),stringsAsFactors=FALSE)
	data.cl$Position<-factor(rep("cluster hierarchy node",nrow(data.cl)), levels=.positionLevels)
	row.names(data.cl)<-as.character(unname(clusterNodes))
    data.cl$Position[phylobase::getNode(clusterD,type="tip")]<-"cluster hierarchy tip"
	  
	#give default node labels:
	phylobase::labels(clusterD)[justNodes]<-data.cl$NodeId[justNodes]
	clusterD<-phylobase::phylo4d(x=clusterD, all.data = data.cl)


	# -- Position: one of either "cluster hierarchy node","cluster hierarchy tip","tip hierarchy","assigned tip","outbranch hierarchy node","unassigned tip","outbranch root"). This column should be internal and validity check that numbers correspond to clustering from @dendro_index. Needs to be a check on "ClusterExperiment", not the "clusterPhylo4d"
    # -- ClusterIdDendro (only for cluster): cluster Id in @dendro_index that corresponds to node (NA otherwise)
    # -- ClusterIdMerge  (only for cluster): cluster Id in @merge_index that corresponds to node (NA otherwise)
    # -- MatchToClusterHier (only for sample tree): for those that are part of the cluster hierarchy, the (permanent) node id from cluster hierarchy -- ie not the name but the number so can always link them. Use this to create checks that the node names are the same, grab the default names, etc. 
    # -- SampleIndex (only for sample tree): index to the columns in the assay
    
    
    return(clusterD)
}



#' @param x a matrix or hdf5 or distance
#' @param cl a clustering of samples in x
#' @param clusterDendro the cluster dendrogram (a phylo4d class)-- will be converted internally to type 'phylo'
#' @param type whether input x is a matrix or distance.
#' @noRd
.makeSampleDendro<-function(x,clusterDendro, cl,type=c("mat","dist"), unassignedSamples=c("remove","outgroup","cluster"),sampleEdgeLength=0, outbranchLength=0,calculateSample=TRUE){
    unassignedSamples<-match.arg(unassignedSamples)
    type<-match.arg(type)
    sampleNames<-if(type=="mat") colnames(x) else attributes(x)$Labels
	
    if(calculateSample){
        whPos<-which(cl>0) #this is copy close to length of n
        if(!is.null(sampleNames) && length(sampleNames)!=length(cl)) stop("sampleNames must be same length as cluster vector")
					#converts internal node id and cluster id to the node, tip labels respectively.
        phyloObj <- .convertToPhyClasses(clusterDendro,"phylo",convertNodes=TRUE,convertTips=TRUE)
        if(!is.ultrametric(phyloObj)) stop("coding error -- the cluster dendrogram is not ultrametric")
        nSamples<-switch(type,"mat"=ncol(x),"dist"=attributes(x)$Size)
        
        ###########################
        ## I. Make outlier tree if needed
        ###########################
        outbranchHclust<-NULL
        whNeg<-which(cl<0) #technically should be able to do with whPos, but a pain
        if(length(whNeg)>0){
	        outNames<- if(!is.null(sampleNames)) sampleNames[whNeg] else paste("OutSample",whNeg)
        	if(unassignedSamples=="outgroup" | length(whPos)==0){
                #--------
                # 1. Make outlier tree 
                #--------
                if(length(whNeg) > 5 | length(whPos)==0){ 
                    outlierDat <- if(type=="mat") x[,whNeg,drop=FALSE] else as.matrix(x)[whNeg,whNeg,drop=FALSE]
                    outbranchHclust <- if(type=="mat") stats::hclust(dist(t(outlierDat))^2) else  stats::hclust(as.dist(outlierDat))
                    outTree<- .convertToPhyClasses(x=outbranchHclust,"phylo") 
                    if(length(outTree$tip.label)!=length(whNeg)) stop("coding error - given hclust doesn't have correct number of tips.")
                }
                else{ #construct tree with just root and tips:
                    outTree<-.makeFakeBinary(tipNames=outNames,rootEdgeLength=1,edgeLength=.1)
                    #have to make it ultrametric -- since arbitrary doesn't matter.
                    if(length(outNames)>1) outTree<-.force.ultrametric(outTree)
                }
            }
            if(unassignedSamples=="cluster"){
                if(type=="dist")stop("cannot use unassigned='cluster' if input is a distance matrix")
                #code from assignUnassigned
                clFactor <- factor(cl[-whNeg])
                medoids <- do.call("rbind", by(t(x[,-whNeg]), clFactor, function(z){apply(z, 2, median)}))
                rownames(medoids) <- levels(clFactor)
                classif<-.genericClassify(inputMatrix=x[,whNeg],centers=medoids) 
                cl[whNeg]<-classif
                whPos<-which(cl>0)
                whNeg<-which(cl<0)
                if(length(whPos)!=length(cl)) stop("coding error -- predicting unassigned missed some")
                unassignedSamples<-"remove"
            }			
        }      
        
        ###################
        ## II. Make tree with main samples:
        ###################
        if(length(whPos)>0){
            #---------
            #make list of phylo trees for each cluster:
            #---------
            fakePhyloList <- tapply(sampleNames[whPos], as.factor(paste("ClusterId",cl[whPos],sep="")), .makeFakeBinary, simplify=FALSE)
            
            #need to reorder so in order of the tips of phyloObj
            m<-match(phyloObj$tip.label,names(fakePhyloList))
            if(any(is.na(m))) stop("coding error -- names in dendrogram of clusters don't match cluster ids")
            fakePhyloList<-fakePhyloList[m]
            newPhylo<-.addTreesToTips(mainTree=phyloObj,tipTrees=fakePhyloList)
        }
        else newPhylo<-outTree 
        
        
        ###################
        ## III. Add outlier samples if unassignedSamples=="outgroup"
        ###################
        if(unassignedSamples == "outgroup" & length(whNeg)>0 & length(whPos)>0){
            if(length(whNeg)>1){
                newPhylo<-.mergePhylo(tree1=newPhylo,tree2=outTree,mergeEdgeLength=outbranchLength)
            }
            else{
                newPhylo<-.mergePhylo(tree1=newPhylo, tree2=outNames, mergeEdgeLength=outbranchLength)
            }
        }
        
        ###################
        ## IV. Convert to return format
        ###################
        newPhylo<-.convertToPhyClasses(newPhylo,"phylo4")
				
		#create data for phylo4d object
		nodeLabs<-phylobase::getNode(newPhylo,type="all") 
		mClusterHier<-rep(NA,length(nodeLabs))
		position<-rep(NA,length(nodeLabs))
		whInternal<-grep("NodeId",names(nodeLabs))
		mClusterHier[whInternal]<-as.character(.matchToDendroData(inputValue=names(nodeLabs)[whInternal], dendro=clusterDendro, returnColumn="NodeId"))
		position[whInternal]<-as.character(.matchToDendroData(inputValue=names(nodeLabs)[whInternal], dendro=clusterDendro, returnColumn="Position"))

		whCluster<-grep("ClusterId",names(nodeLabs))
		mClusterHier[whCluster]<-as.character(.matchToDendroData(inputValue=names(nodeLabs)[whCluster], dendro=clusterDendro, matchColumn="ClusterIdDendro",returnColumn="NodeId"))
		position[whCluster]<-as.character(.matchToDendroData(inputValue=names(nodeLabs)[whCluster], dendro=clusterDendro, matchColumn="ClusterIdDendro",returnColumn="Position"))

				#rather than keep track of position as make trees (which require making it a phylo4d class much more frequently), figure it out from here.
				
		rootNode<-phylobase::rootNode(newPhylo)
		rootChildren <- phylobase::descendants(newPhylo,node=rootNode,type="children")
			 
		 whNAChild<-which(is.na(names(rootChildren)))
		 if(length(whNAChild)==0){
			 #all are part of cluster hierarchy
			 #all NA nodes -> "tip hierarchy"
			 #all tips -> "assigned tip"
			 position[is.na(names(nodeLabs))]<-"tip hierarchy"
			 position[phylobase::getNode(newPhylo,type="tip")]<-"assigned tip"
			 #check if there is a singleton sample for outlier:
			 whSingletonOutlier<-which(is.na(mClusterHier[rootChildren]))
			 if(length(whSingletonOutlier)>0){
			 	position[rootChildren[whSingletonOutlier]]<-"unassigned tip"
			 }
	 
		 }
		 else if(length(whNAChild)==1){
			 #all nodes descending from child of NA -> "outbranch hierarchy node"
			 #all tips descending from child of NA -> "unassigned tip"
			 #all tips descending from non-NA -> "assigned tip"
			 #all NA nodes descending from non-NA -> "tip hierarchy"
			 #root -> "outbranch root"
			 position[rootNode]<-"outbranch root"
			 naNode<-rootChildren[which(is.na(names(rootChildren)))]
			 position[phylobase::descendants(newPhylo,node=naNode,type="ALL")] <- "outbranch hierarchy node" #includes tips, but will overwrite this in next line...
			 position[phylobase::descendants(newPhylo,node=naNode,type="tip")] <- "unassigned tip"
			 nonNANode<-rootChildren[which(!is.na(names(rootChildren)))]
			  whDescendantNonNA <- phylobase::descendants(newPhylo,node=nonNANode,type="all")
				whDescendantNonNA<-whDescendantNonNA[is.na(names(whDescendantNonNA))]
				position[whDescendantNonNA]<-"tip hierarchy"
			  whTipsDescendantNonNA <- phylobase::descendants(newPhylo,node=nonNANode,type="tip")
				position[whTipsDescendantNonNA]<-"assigned tip"
		
		 }
		else stop("coding error -- neither child of root is part of cluster hierarchy")
		if(any(is.na(position))) stop("coding error -- not all nodes assigned a position value")
		position<-factor(position,levels=.positionLevels)

		mSamples<-match(names(nodeLabs),sampleNames)
		#this will miss singleton clusters (which will have name ClusterIdX instead of sample name)
		whMissed<-intersect(which(is.na(mSamples)),grep("assigned tip",position))
		if(length(whMissed)>0){
			clusters<-.matchToDendroData(inputValue=mClusterHier[whMissed],clusterDendro,returnColumn="ClusterIdDendro",matchColumn="NodeId")
			if(any(is.na(clusters))) stop("coding error -- didn't find singleton cluster")
			whSamplesSingle<-match(gsub("ClusterId","",clusters),as.character(cl))
			mSamples[whMissed]<-whSamplesSingle
		}
		if(any(is.na(mSamples[position %in% c("unassigned tip","assigned tip")]))) stop("coding error -- finding sample index failed")
		data.cl <- data.frame(NodeId=mClusterHier, Position=position, SampleIndex=mSamples, stringsAsFactors=FALSE)				
		row.names(data.cl)<-as.character(nodeLabs)

		# #Need to fix back up the nodeLabels so match that of clusterDendro (in conversion made them the internal names)
		# whMatchCluster<-which(!is.na(data.cl$NodeId))
		# mToCluster<-.matchToDendroData(inputValue=data.cl$NodeId[whMatchCluster],clusterDendro,returnColumn="NodeIndex",matchColumn="NodeId")
		# phylobase::labels(newPhylo,type="all")[whMatchCluster]<-phylobase::labels(clusterDendro,type="all")[mToCluster]
		
		phylobase::labels(newPhylo,type="all")<-NA
		return(phylobase::phylo4d(x=newPhylo, all.data = data.cl))

    }
    else return(NULL)
}

#'@title Plot dendrogram of ClusterExperiment object
#'  
#'@description Plots the dendrogram saved in a ClusterExperiment object
#'  
#'@param x a \code{\link{ClusterExperiment}} object.
#'@param leafType if "samples" the dendrogram has one leaf per sample, otherwise
#'  it has one per cluster.
#'@param main passed to the \code{plot.phylo} function to set main title.
#'@param sub passed to the \code{plot.phylo} function to set subtitle.
#'@param plotType one of 'name', 'colorblock' or 'id'. If 'Name' then dendrogram
#'  will be plotted, and name of cluster or sample (depending on type of value 
#'  for \code{leafType}) will be plotted next to the leaf of the dendrogram. If 
#'  'colorblock', rectangular blocks, corresponding to the color of the cluster 
#'  will be plotted, along with cluster name legend. If 'id' the internal 
#'  clusterIds value will be plotted (only appropriate if 
#'  \code{leafType="clusters"}).
#'@param ... arguments passed to the \code{\link{plot.phylo}} function of 
#'  \code{ape} that plots the dendrogram.
#'@param removeOutbranch logical, only applicable if there are missing samples 
#'  (i.e. equal to -1 or -2), \code{leafType="samples"} and the dendrogram for 
#'  the samples was made by putting missing samples in an outbranch. In which 
#'  case, if this parameter is TRUE, the outbranch will not be plotted, and if 
#'  FALSE it will be plotted.
#'@param legend character, only applicable if \code{plotType="colorblock"}. 
#'  Passed to \code{\link{phydataplot}} in \code{\link{ape}} package that is 
#'  used to draw the color values of the clusters/samples next to the 
#'  dendrogram. Options are 'none', 'below', or 'side'. (Note 'none' is only 
#'  available for 'ape' package >= 4.1-0.6).
#'@param nodeColors named vector of colors to be plotted on a node in the 
#'  dendrogram (calls \code{\link[ape]{nodelabels}}). Names should match the 
#'  \emph{internal} name of the node (the "NodeId" value, see 
#'  \code{\link{clusterDendrogram}}).
#'@param clusterLabelAngle angle at which label of cluster will be drawn. Only 
#'  applicable if \code{plotType="colorblock"}.
#'@param mergeInfo What kind of information about merge to plot on dendrogram. 
#'  If not equal to "none", will replicate the kind of plot that 
#'  \code{\link{mergeClusters}} creates, and the input to \code{mergeInfo} 
#'  corresponds to that of \code{plotInfo} in \code{mergeClusters}.
#' @param colData index (by integer or name) the sample data stored as a 
#'   \code{DataFrame} in \code{colData} slot of the object. Only discrete valued
#'   ("character" or "factor" variables) will be plotted; indexing of continous 
#'   variables will be ignored. Whether that data is continuous or not will be 
#'   determined by the properties of \code{colData} (no user input is needed).
#'   This argument is only relevant if \code{plotType=="colorblock"} and
#'   \code{leafType=="samples"}
#' @param clusterLegend Assignment of colors to the clusters or sample data (as
#'   designated by \code{colData} argument) plotted with the dendrogram . If
#'   \code{NULL} or a particular variable/cluster are not assigned a color, 
#'   colors will be assigned internally for sample data and pull from the
#'   \code{clusterLegend} slot of the x for the clusters.
#' @inheritParams getClusterIndex
#' @return A dendrogram is plotted. Returns (invisibly) a list with elements
#' \itemize{
#' \item{\code{plottedObject}}{ the \code{phylo} object that is plotted.}
#' \item{\code{originalObject}}{ the \code{phylo} object before adjusting the
#' node/tip labels. }
#' }
#' @aliases plotDendrogram
#' @details If \code{leafType="clusters"}, the plotting function will work best
#'   if the clusters in the dendrogram correspond to the primary cluster. This
#'   is because the function colors the cluster labels based on the colors of
#'   the clusterIds of the primaryCluster
#' @importFrom ape plot.phylo nodelabels
#' @seealso
#'   \code{\link{mergeClusters}},\code{\link[ape]{plot.phylo}},
#'   \code{\link[ape]{nodelabels}},\code{\link[ape]{tiplabels}}
#'   
#' @export
#' 
#' @examples
#' data(simData)
#' 
#' #create a clustering, for 8 clusters (truth was 3) 
#' cl <-clusterSingle(simData, subsample=FALSE, 
#' sequential=FALSE, 
#' mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=8)))
#' 
#' #create dendrogram of clusters and then 
#' # merge clusters based ondendrogram: 
#' cl <- makeDendrogram(cl) 
#' cl <- mergeClusters(cl,mergeMethod="adjP",DEMethod="limma",
#'    cutoff=0.1,plot=FALSE) 
#' plotDendrogram(cl) 
#' plotDendrogram(cl,leafType="samples",whichClusters="all",plotType="colorblock")
#' 
#' @export
#' @rdname plotDendrogram
setMethod(
  f = "plotDendrogram",
  signature = "ClusterExperiment",
  definition = function(x,whichClusters="dendro",leafType=c("samples","clusters" ),  plotType=c("colorblock","name","ids"), mergeInfo="none", main, sub, clusterLabelAngle=45, removeOutbranch=TRUE, legend=c("side","below", "none"),nodeColors=NULL,colData=NULL,clusterLegend=NULL,...)
  {
    if(is.null(x@dendro_samples) || is.null(x@dendro_clusters)) stop("No dendrogram is found for this ClusterExperiment Object. Run makeDendrogram first.")
    leafType<-match.arg(leafType)
    plotType<-match.arg(plotType)
    
    possibleMergeValues<-c("none", "all","mergeMethod",.availMergeMethods)
    if(!is.null(x@merge_nodeProp)){
      otherVals<-colnames(x@merge_nodeProp)[!colnames(x@merge_nodeProp)%in%c("NodeId","Contrast")]
      possibleMergeValues<-unique(c(possibleMergeValues,otherVals))
      
    }
    mergeInfo<-match.arg(mergeInfo,possibleMergeValues)
    
    legend<-match.arg(legend)
    whCl<-getClusterIndex(x,whichClusters=whichClusters,noMatch="throwError")
    if(leafType=="clusters" && whCl!=x@dendro_index){
      warning("if leafType=='clusters', 'whichClusters' must match the clusters that created the dendrogram. Changing 'leafType' to 'samples'")
      leafType<-"samples"
    }
    if(leafType=="clusters" & length(whCl)>1) stop("If leafType equal to 'clusters' 'whichClusters' must be of length 1 (i.e. single cluster).")
    if(missing(main)) main<-ifelse(leafType=="samples","Dendrogram of samples", "Dendrogram of clusters")
    if(missing(sub)) sub<-paste("Dendrogram made with '",clusterLabels(x)[dendroClusterIndex(x)],"', cluster index ",dendroClusterIndex(x),sep="")

	#This grabs the labels and moves them to node and tip labels
	convertedDends<-.setNodeLabels(x,labelType="name",useMergeClusters=FALSE,overrideExistingNode=FALSE,singletonCluster=c("sample"))
	dend<- switch(leafType,"samples"=convertedDends$dendro_samples,"clusters"=convertedDends$dendro_clusters)
    
    #---
    #make color matrix
    #---
    cl<-switch(leafType,"samples"=clusterMatrix(x)[,whCl,drop=FALSE],"clusters"=NULL)
		
		if(leafType=="samples" & plotType=="colorblock"){
	    sData<-.pullColData(x,colData) #returns data.frame
	    #identify which numeric and remove
	    if(!is.null(sData)){
				whCont<-which(sapply(seq_len(ncol(sData)),function(ii){is.numeric(sData[,ii])}))
				if(length(whCont)>0){
					warning("argument 'colData' implies using columns of colData that are continuous, which is not handled by plotDendrogram. Those columns will be ignored")
					if(length(whCont)< ncol(sData)) sData<-sData[,-whCont,drop=FALSE]
					else sData<-NULL
				}
	    	
	    }
		}
		else{
			if(!is.null(colData)) 
				warning("argument colData only used if leafType='samples' and plotType='colorblock'. Ignoring input to colData.")
			sData<-NULL
		}
		if(!is.null(sData)){
		  sClusterLegend<-.makeColors(sData,colors=massivePalette,distinctColors=TRUE,matchClusterLegend = clusterLegend,matchTo="name")
			sNames<-colnames(sData)
			sData<-sClusterLegend$numClusters
      colnames(sData)<-sNames
			cl<-cbind(cl,sData)
			
		}
    if(!is.null(clusterLegend) || !is.null(sData)){
      #preserve those in given clusterLegend that don't match colData (could go with features/rows)
      if(is.list(clusterLegend)){ #could be single vector, but in that case, will loose them
        whKeep<-names(clusterLegend)[which(!names(clusterLegend)%in% names(sClusterLegend$colorList  ))]
        clusterLegend<-c(sClusterLegend$colorList,clusterLegend[whKeep])
      }
      else clusterLegend<-sClusterLegend$colorList
    }
		
    if(leafType=="samples") rownames(cl)<-if(!is.null(colnames(x))) colnames(x) else
		.makeSampleNames(seq_len(ncol(x)))
    if(length(whCl)==1 & is.null(sData)){
      leg<-clusterLegend(x)[[whCl]]
      if(plotType=="id") leg[,"name"]<-leg[,"clusterIds"]		
    }
    else{
      leg<-clusterLegend(x)[whCl]
      if(plotType=="id") leg<-lapply(leg,function(x){x[,"name"]<-x[,"clusterIds"]; return(x)})	
			if(!is.null(sData)){
				leg<-c(leg,clusterLegend)
			}
    }
    label<-switch(plotType,"name"="name","colorblock"="colorblock","ids"="name")
    #   mergePlotType=NULL,mergeMethod=NULL,mergeOutput=NULL, 
    
    
    if(is.na(x@merge_dendrocluster_index)) mergeInfo<-"none"
    if(mergeInfo=="none"){
      mergeInfo<-NULL
      mergeMethod<-NULL
    }
    else if(!is.na(x@merge_dendrocluster_index)){ #has node prop info, even if no merge cluster
      if(mergeInfo=="mergeMethod"){
        if(is.na(x@merge_index)){
          warning("Cannot plot merge method because there is none. Plotting all")
          mergeInfo<-"all"
        }
        else{
          mergeMethod<-x@merge_method
        }
      }
      else if(mergeInfo==x@merge_method ) mergeMethod<-x@merge_method #only do dotted lines if matches saved merged method -- is this good default or no? not sure .plotDendro works otherwise from ClusterExperiment object
      else mergeMethod<-"none"
    } else{
      warning("There is no information about merging -- will ignore input to 'mergeInfo'")
    }
    
    phyloOut<-.plotDendro(dendro=dend,leafType=leafType,mergeMethod=mergeMethod,mergePlotType=mergeInfo,mergeOutput=nodeMergeInfo(x),clusterLegendMat=leg,clObj=cl,plotType=label,main=main,sub=sub,removeOutbranch=removeOutbranch,legend=legend,clusterLabelAngle=clusterLabelAngle,...)
    
    if(!is.null(nodeColors)){
      if(is.null(names(nodeColors))) warning("Must give names to node colors, ignoring argument nodeColors")
      m<-match(names(nodeColors),phyloOut$originalObject$node.label)
      nodeColors<-nodeColors[!is.na(m)]
      m<-m[!is.na(m)]
      if(length(m)>0){
        ape::nodelabels(rep("",length(nodeColors)),m+length(phyloOut$originalObject$tip.label), frame = "c", bg = nodeColors,cex=1/par("cex"))
      }
      else{warning("No names of node colors match node name, ignoring argument nodeColors")}
    }
    invisible(phyloOut)
    
  })




########
# Internal plotting function used by both mergeClusters and plotDendrogram
#' @importFrom graphics plot
#' @importFrom ape plot.phylo phydataplot
.plotDendro<-function(dendro, leafType="clusters",mergePlotType=NULL,mergeMethod=NULL,mergeOutput=NULL,clusterLegendMat=NULL,clObj=NULL,plotType=c("name","colorblock"),removeOutbranch=FALSE,legend="below",clusterLabelAngle=45,...){
  plotType<-match.arg(plotType)
  outbranch<- "outbranch root" %in% phylobase::tdata(dendro)$Position
  
  ############
  ## Everything in this code assumes that phylobase::getNode(dend@dendro_samples,type="tip") is in same order as phyloObj$tip.label
  ############
    
  #---
  #remove the outbranch from the dendrogram and update clObj and mTipsToSamples
  #(note this is using phylo4 obj)
  #---	
  if(outbranch & removeOutbranch & leafType=="samples"){
	  ##Find node that is cluster child of root	  
    rootNode<-phylobase::rootNode(dendro)
    rootChild<-phylobase::descendants(dendro,node=rootNode,type="children")
	position<-.matchToDendroData(inputValue=rootChild,dendro=dendro,matchColumn="NodeIndex",returnColumn="Position")
	if(!any(position=="cluster hierarchy node")) stop("coding error -- child of root with outbranch isn't cluster hierarchy node")
	if(all(position=="cluster hierarchy node")) stop("coding error -- both child of root are 'cluster hierarchy node', but also have root is 'outbranchroot'")	
	clusterNode<-rootChild[which(position=="cluster hierarchy node")]
	
	#remove from clObj and update mTipsToSamples
	nSamples<-nTips(dendro) #save so have after changed dendro
    clusterTips<-phylobase::descendants(dendro,node=clusterNode,type="tip")
    if(length(clusterTips)==0) stop("Internal coding error: no unassigned samples in tree")
	whKeep<-.matchToDendroData(inputValue=clusterTips, dendro, matchColumn="NodeIndex", returnColumn="SampleIndex")
#	all(tipLabels(dendro)==row.names(clObj)[tdata(dendro,type="tip")$SampleIndex])

	####Subset dendro
	dendro<-phylobase::subset(dendro, node.subtree=clusterNode)
	ch<-.checkDendroSamplesFormat(dendro,checkLabels=FALSE)
	if(!is.logical(ch)) stop(ch)

	#need to create mTipsToSamples -- match of tips to samples ("SampleIndex" is index to full data)
	mTipsToSamplesOld <- .matchToDendroData(inputValue=phylobase::getNode(dendro,type="tip"), dendro, matchColumn="NodeIndex", returnColumn="SampleIndex")
	#these are still indices in the full sample clObj. Now need to get their indices in the subsetted one:
	mToSubset<-match(seq_len(nSamples),whKeep) #gives where each of old sample indices (1:n) map to in new order of future clObj (with NA for those that not in new clObj)
	mTipsToSamples<-mToSubset[mTipsToSamplesOld] #use that to map mTipsToSamplesNew to new order
	if(any(is.na(mTipsToSamples))) stop("coding error -- didn't update mTipsToSamples correctly")
		#these check against names, but doesn't always have names, etc.
	# if(!all(tipLabels(dendro)==row.names(clObj)[mTipsToSamplesOld])) stop("coding error -- names no longer match")
	# if(!all(sort(tipLabels(dendro))==sort(names(clObj[whKeep,])))) stop("coding error -- don't have the same set of tips after pruning outbranch")
	# if(!all(tipLabels(dendro)==row.names(clObj[whKeep,,drop=FALSE])[mTipsToSamples])) stop("coding error -- names no longer match")
    if(is.matrix(clObj)) clObj<-clObj[whKeep,,drop=FALSE] else clObj<-clObj[whKeep]		
    #set outbranch=FALSE because now doesn't exist in tree...
    outbranch<-FALSE
  }
  else{
	  if(leafType=="samples") mTipsToSamples <- .matchToDendroData(inputValue=phylobase::getNode(dendro,type="tip"), dendro, matchColumn="NodeIndex", returnColumn="SampleIndex")
	
  }
  
  #-------
  #convert to phylo object...
  #-------
  phyloObj <- .convertToPhyClasses(dendro, "phylo",convertNodes=TRUE,convertTips=(leafType!="samples")) 
  if(leafType=="samples"){
	  #not clear need this now that convert dendro it before send to .plotDendro
	  #these are in the order of dendro tips. To 
	  clNames<-phylobase::tipLabels(dendro)
  }
  plotArgs<-list(...)
  if(plotType=="colorblock" && is.null(clObj) && leafType=="samples") stop("Internal coding error: must provide a clustering if plotType='colorblock'")
  origPhylo<-phyloObj #so can return and get any information that gets overwritten in phyloObj
  
  #---------------
  ### For plotting of dendrogram for the merging
  ### Add information about the merging as node labels and change edge type
  ### Note: could probably have used nodelabels function and avoided some of this
  #---------------
  if(!is.null(mergeOutput)){
    annotNames<-c("NodeId","Contrast","isMerged", "mergeClusterId")
    methods<-colnames(mergeOutput)[!colnames(mergeOutput)%in%annotNames] #possible for which have proportion saved
  }
  doMerge<-!is.null(mergePlotType) && !is.null(mergeOutput) && mergePlotType %in% c("all",methods,"mergeMethod")
  if(doMerge){
    #####
    #convert names of internal nodes for plotting
    #####
    #match to order of tree
    whToMerge<-which(mergeOutput$isMerged)
    nodesToMerge<-as.character(mergeOutput$Node[whToMerge])
    m <- match( as.character(mergeOutput$Node),phyloObj$node.label)
    if(any(is.na(m))) stop("some nodes in merge node info not in the given dendrogram")
    edgeLty <- rep(1, nrow(phyloObj$edge))
    if(mergeMethod != "none" && length(whToMerge) > 0){
      #which of nodes merged
      whMerge <- which(phyloObj$node.label %in% nodesToMerge) 
      nodeNumbers <- (length(phyloObj$tip) + 1):max(phyloObj$edge)
      whEdge <- which(phyloObj$edge[,1] %in% nodeNumbers[whMerge])
      edgeLty[whEdge] <- 2
    }
    if(mergePlotType == "mergeMethod"){
      if(!mergeMethod %in% methods) stop("mergeMethod not in methods of output")
      valsNodes<-as.character(signif(mergeOutput[,mergeMethod],2))
      valsNodes[is.na(valsNodes)]<-"NA" #make them print out as NA -- otherwise doesn't plot
      phyloObj$node.label[m] <- valsNodes
      # offsetDivide<-3
      # dataPct<-.7
    }
    else if(all(mergePlotType %in% c("all",methods))) {
      meth<-if(mergePlotType=="all") methods else methods[methods%in%mergePlotType]
      phyloObj$node.label[m] <- apply(mergeOutput[,meth,drop=FALSE],1, 
                                      function(x){
                                        whKp<-which(!is.na(x))
                                        #fix up name of FC methods:
                                        mm<-meth[whKp]
                                        mm<-sapply(strsplit(mm,"_"),function(u){if(length(u)==2) return(sprintf("%s(>%s)",u[1],u[2])) else return(u)})
                                        vals<-paste(paste(mm, signif(x[whKp],2), sep=":"), collapse="\n")
                                        vals[is.na(vals)]<-"NA"
                                        return(vals)
                                      })
      if(mergePlotType!="all"){
        # offsetDivide<-3
        # dataPct<-.7
      }
      else{
        # offsetDivide<-2.5
        # dataPct<-.7
        
      }
    }
    
    phyloObj$node.label[-m]<-""
    ###Only set these if user doesn't...
    if(!"show.node.label" %in% names(plotArgs)) plotArgs$show.node.label<-TRUE
    if(!"edge.lty" %in% names(plotArgs)) plotArgs$edge.lty<-edgeLty
  }
  else{
	  phyloObj$node.label<-phylobase::nodeLabels(dendro) #use the user-given/assigned node labels
	
  }
  ###############
  ### Deal with clusterLegend object: 
  ### - Make default if not provided and 
  ### - If # of clusterings>1 make clusterLegend and clObj matrix appropriate
  ###############
  
  if(plotType=="colorblock"){
	  
    clusterLegend<-TRUE #doesn't do anything right now because phydataplot doesn't have option of no legend...
    if(is.null(clusterLegendMat)){ 
      #----
      #make default colors, works for vector or matrix clObj
      #----
      clusterIds<-sort(unique(as.vector(clObj)))
      clusterLegendMat <- cbind("clusterIds"=clusterIds, "name"=clusterIds, "color"=bigPalette[seq_along(clusterIds)])
    }
    else{
      if(is.matrix(clObj) && ncol(clObj)>1){
        #if not provide list of cluster legends, do only 1st clustering provided (temporary while fixing so works for matrix)
        if(!is.list(clusterLegendMat) ) clObj<-clObj[,1,drop=FALSE]
        else{
          #----
          #create one big clObj/clusterLegendMat object that will allow for coloring that is okay.
          #----
          nclusters<-ncol(clObj)
          if(length(clusterLegendMat)!=nclusters) stop("Internal coding error -- wrong length of colors for clustering")
          newClusterLegendMat<-clusterLegendMat[[1]]
          newCl<-clObj[,1]
          
          #make it general in case some day want more than just 2 clusterings
          for(ii in 2:nclusters){
            currMat<-clusterLegendMat[[ii]]
            currCl<-clObj[,ii]
            
            #note that because subset to those samples that are not -1/-2 on
            # clObj[,1], may have entire clusters in other columns of clObj that disappear but still in color matrix with no entry in clObj
            #reduce down the currMat to accomodate that
            whExist<-which(as.numeric(currMat[,"clusterIds"]) %in% currCl)
            currMat<-currMat[whExist, ,drop=FALSE]
            
            whExistingColor<-which(currMat[,"color"] %in% newClusterLegendMat[,"color"])
            updatedCurrCl<-currCl
            if(length(whExistingColor)>0){
              #-----------
              #reassign the cluster id to the one matching existing color id.
              # only should affect names, not color of newClusterLegendMat
              #-----------
              matchNew<-match(currMat[whExistingColor,"color"],newClusterLegendMat[,"color"])
              oldId<-currMat[whExistingColor,"clusterIds"]
              newId<-newClusterLegendMat[matchNew,"clusterIds"]
              mexist<-match(currCl,oldId)
              newFullId<-as.numeric(newId[mexist])
              updatedCurrCl[!is.na(mexist)]<-newFullId[!is.na(mexist)]
              
              #change name so combination, if not already the same
              whDiff<-which(newClusterLegendMat[matchNew,"name"]!=currMat[whExistingColor,"name"])
              if(length(whDiff)>0){
                combName<-paste(newClusterLegendMat[matchNew,"name"],currMat[whExistingColor,"name"],sep="/")
                newClusterLegendMat[matchNew[whDiff],"name"]<-combName[whDiff]
                
              }
              
              #remove from current color scheme
              currMat<-currMat[-whExistingColor,,drop=FALSE]
            }
            
            if(nrow(currMat)>0){
              #-----------
              # for remaining, add to newClusterLegendMat 
              # with ids increased to be unique
              #-----------
              maxNew<-max(as.vector(newCl))
              oldId2<-currMat[,"clusterIds"]
              newId2<-seq(from=maxNew+1,by=1,length=length(oldId2)) #replace with this in legend
              mexist2<-match(currCl,oldId2) #match old ids to the clusterings vector
              newFullId2<-as.numeric(newId2[mexist2]) #will get NAs for those that don't match (e.g. have been changed by previous step)
              updatedCurrCl[!is.na(mexist2)]<-newFullId2[!is.na(mexist2)]
              
              ## change ids in currMat
              currMat[,"clusterIds"]<-newId2
              ## test correct that no overlap in ids or names or colors:
              if(any(currMat[,"clusterIds"] %in% newClusterLegendMat[,"clusterIds"])){
                stop("Internal coding error: still overlap in cluster Ids")
              }
              if(any(currMat[,"color"] %in% newClusterLegendMat[,"color"])){
                stop("Internal coding error: still overlap in color")
              } 
              
              ## add to new cluster color legend
              newClusterLegendMat<-rbind(newClusterLegendMat,currMat)
            }
            #change them all here (before changed)
            currCl<-updatedCurrCl
            newCl<-cbind(newCl,currCl)
            
          }
          clusterLegendMat<-newClusterLegendMat
          colnames(newCl)<-colnames(clObj)
          rownames(newCl)<-rownames(clObj)
          clObj<-newCl
          clusterLegend<-FALSE
          
        }
        
      }
    }
  } 
  ###############
  ### Deal with clusterLegend object Part II: 
  ### - Add color of cluster and cluster/sample name to tip labels if plotType=="name"
  ### - Make colorMat matrix if plotType=="colorblock"
  ###############
  edge.width=1
  if(!is.null(clusterLegendMat)){
    if(leafType=="clusters"){
      #get rid of matching string 
	  #don't need this now that convert before send to .plotDendro
      m<-match(gsub("ClusterId","",phyloObj$tip.label),clusterLegendMat[,"clusterIds"])
      if(any(is.na(m))) stop("clusterIds in clusterLegend do not match dendrogram labels")
      tip.color<-clusterLegendMat[m,"color"]
	  if(plotType=="colorblock"){
        clusterLegendMat<-clusterLegendMat[!clusterLegendMat[,"clusterIds"]%in%c(-1,-2),]
        colorMat<-matrix(clusterLegendMat[,"name"],ncol=1)
        row.names(colorMat)<-clusterLegendMat[,"name"]
        cols<-clusterLegendMat[,"color"]
        names(cols)<-clusterLegendMat[,"name"]
        
        
      }
      
    }
    if(leafType=="samples"){
  	  if(is.matrix(clObj) && ncol(clObj)>1){
        if(plotType=="colorblock"){
          colorMat<-apply(clObj,2,function(x){
            m<-match(x,clusterLegendMat[,"clusterIds"])
            clusterLegendMat[m,"name"]
          })
          if(any(dim(colorMat)!=dim(clObj))) stop("Internal coding error: dimensions of colorMat don't match input")
          dimnames(colorMat)<-dimnames(clObj)
          cols<-clusterLegendMat[,"color"]
          names(cols)<-clusterLegendMat[,"name"]
		  #match samples to order of the tree:
          colorMat<-colorMat[mTipsToSamples,]
        }
        tip.color<-"black"
      }
      else{
        if(is.matrix(clObj)) clObj<-clObj[,1]
        m<-match(clObj,clusterLegendMat[,"clusterIds"])
        tip.color<-clusterLegendMat[m,"color"]		
        if(plotType=="colorblock"){
		  colorMat<-matrix(clusterLegendMat[m,"name"],ncol=1)
		  colorMat<-colorMat[mTipsToSamples,,drop=FALSE]
          rownames(colorMat)<-clNames
          cols<-clusterLegendMat[,"color"]
          names(cols)<-clusterLegendMat[,"name"]
        }	
		else tip.color<-tip.color[mTipsToSamples]		
      }
      if(plotType=="colorblock"){
        #make only edges going to/from nodes in cluster hierarchy have edge.width>0
		#Note that some of origPhylo$node.label have NA for value; won't get unique value for these; but these aren't part of cluster hierarchy anyway (in fact could probably just pick those that are NA and use them, but this is safer(?))
		ntips<-length(phyloObj$tip.label)
		positionValue<-.matchToDendroData(inputValue=origPhylo$node.label, dendro=dendro, matchColumn="NodeId", returnColumn="Position")
		whClusterNode<-which(as.character(positionValue)%in% c("cluster hierarchy node","cluster hierarchy tip")) + ntips 
        whEdgePlot<-which(apply(phyloObj$edge,1,function(x){any(x %in% whClusterNode)}))
        edge.width<-rep(0,nrow(phyloObj$edge))
        edge.width[whEdgePlot]<-1
      }
    }
  }
  else tip.color<-"black"
	 
  #Couldn't do this before, because needed cluster id to safely match to clusterLegendMat. 
  if(leafType=="clusters") phyloObj$tip.label<-phylobase::tipLabels(dendro)

  #this next code is hack to deal with error sometimes get if very long edge length -- usually due to unusual distance, etc.
  # Divides edge lengths so not too large.
  if(max(phyloObj$edge.length)>1e6) phyloObj$edge.length <- phyloObj$edge.length / max(phyloObj$edge.length) 
  
  #---------------
  # PLOTTING
  #-------------
  #the percentage of total width will be dataPct/(1+dataPct+offsetPct)
  #tree=64%
  #data=32%
  #offset=3%
  dataPct<-0.5 
  offsetPct<-0.01 
  
  .pctCalculation<-function(data,offset){
	  denom<-1+data+offset
	  return(c("tree"=1/denom,data=data/denom,offset=offset/denom))
  }
  # > .pctCalculation(.5,.01)
#          tree        data      offset
#   0.662251656 0.331125828 0.006622517
  
  prohibitOptions<-c("tip.color","node.pos","edge.width","horizontal","direction","type")
  if(any(prohibitOptions %in% names(plotArgs))) stop("User cannot set following options to plot.phylo:",paste(prohibitOptions, collapse=","))
  plotArgs<-c(plotArgs,list(tip.color=tip.color,node.pos=2,edge.width=edge.width))	
  if(plotType=="name") do.call(ape::plot.phylo,c(list(phyloObj),plotArgs))
  else{
    #if colorblock
    if(length(grep("show.tip",names(plotArgs)))==0) plotArgs$show.tip.label<-FALSE
	sn<-grep("show.node",names(plotArgs))
	if(length(sn)>0 && plotArgs[[sn]] && !doMerge){
		offsetPct<-1/8	
		dataPct<-0.6
		# > .pctCalculation(.6,1/6)
# 		      tree       data     offset
# 		0.56603774 0.33962264 0.09433962
	}
			
	#just calculate needed width for tree...
    phyloPlotOut<-do.call(.calculatePlotPhylo,c(list(phyloObj,plot=FALSE),plotArgs))
    treeWidth<-phyloPlotOut$x.lim[2]
    #plot dendrogram:
    do.call(ape::plot.phylo,c(list(phyloObj,x.lim=treeWidth*(1+dataPct+offsetPct)),plotArgs))
    
    nclusters<-ncol(colorMat)
    colnames(colorMat)<-NULL		    
    colInput<-function(n){cols}
    width<-treeWidth*dataPct/(nclusters+.5)
	ape::phydataplot(x=colorMat, phy=phyloObj, style="mosaic",offset=treeWidth*offsetPct, width = width, border = NA, lwd = 3,legend = legend, funcol = colInput)
    
    if(nclusters>1 & !is.null(colnames(clObj))){
      xloc<-treeWidth+treeWidth*offsetPct+seq(from=0,by=width,length=nclusters)
      xloc<-xloc+width/2
      ypos<-par("usr")[4]-0.025*diff(par("usr")[3:4])	
      adj<-c(0,0)		
      if("cex" %in% names(list(...))) labcex<-list(...)[["cex"]]
      else labcex<-1	
      text(x=xloc,y=ypos,labels=colnames(clObj),srt=clusterLabelAngle,xpd=NA,adj=adj,cex=labcex)
      
    }
    
  }
  
  invisible(list(plottedObject=phyloObj,originalObject=origPhylo))
}


#' @importFrom ape node_height_clado node_height node_depth node_depth_edgelength is.ultrametric unrooted.xy
###Copy code from ape:plot.phylo so get how calculates x.lim. Just removed the plot parts.
.calculatePlotPhylo<-function (x, type = "phylogram", use.edge.length = TRUE, node.pos = NULL, 
                               show.tip.label = TRUE, show.node.label = FALSE, edge.color = "black", 
                               edge.width = 1, edge.lty = 1, font = 3, cex = par("cex"), 
                               adj = NULL, srt = 0, no.margin = FALSE, root.edge = FALSE, 
                               label.offset = 0, underscore = FALSE, x.lim = NULL, y.lim = NULL, 
                               direction = "rightwards", lab4ut = NULL, tip.color = "black", 
                               plot = TRUE, rotate.tree = 0, open.angle = 0, node.depth = 1, 
                               align.tip.label = FALSE, ...) 
{
  Ntip <- length(x$tip.label)
  if (Ntip < 2) {
    warning("found less than 2 tips in the tree")
    return(NULL)
  }
  .nodeHeight <- function(edge, Nedge, yy) .C(ape::node_height, 
                                              as.integer(edge[, 1]), as.integer(edge[, 2]), as.integer(Nedge), 
                                              as.double(yy))[[4]]
  .nodeDepth <- function(Ntip, Nnode, edge, Nedge, node.depth) .C(ape::node_depth, 
                                                                  as.integer(Ntip), as.integer(edge[, 1]), as.integer(edge[, 
                                                                                                                           2]), as.integer(Nedge), double(Ntip + Nnode), as.integer(node.depth))[[5]]
  .nodeDepthEdgelength <- function(Ntip, Nnode, edge, Nedge, 
                                   edge.length) .C(ape::node_depth_edgelength, as.integer(edge[, 
                                                                                               1]), as.integer(edge[, 2]), as.integer(Nedge), as.double(edge.length), 
                                                   double(Ntip + Nnode))[[5]]
  Nedge <- dim(x$edge)[1]
  Nnode <- x$Nnode
  if (any(x$edge < 1) || any(x$edge > Ntip + Nnode)) 
    stop("tree badly conformed; cannot plot. Check the edge matrix.")
  ROOT <- Ntip + 1
  type <- match.arg(type, c("phylogram", "cladogram", "fan", 
                            "unrooted", "radial"))
  direction <- match.arg(direction, c("rightwards", "leftwards", 
                                      "upwards", "downwards"))
  if (is.null(x$edge.length)) {
    use.edge.length <- FALSE
  }
  else {
    if (use.edge.length && type != "radial") {
      tmp <- sum(is.na(x$edge.length))
      if (tmp) {
        warning(paste(tmp, "branch length(s) NA(s): branch lengths ignored in the plot"))
        use.edge.length <- FALSE
      }
    }
  }
  if (is.numeric(align.tip.label)) {
    align.tip.label.lty <- align.tip.label
    align.tip.label <- TRUE
  }
  else {
    if (align.tip.label) 
      align.tip.label.lty <- 3
  }
  if (align.tip.label) {
    if (type %in% c("unrooted", "radial") || !use.edge.length || 
        is.ultrametric(x)) 
      align.tip.label <- FALSE
  }
  if (type %in% c("unrooted", "radial") || !use.edge.length || 
      is.null(x$root.edge) || !x$root.edge) 
    root.edge <- FALSE
  phyloORclado <- type %in% c("phylogram", "cladogram")
  horizontal <- direction %in% c("rightwards", "leftwards")
  xe <- x$edge
  if (phyloORclado) {
    phyOrder <- attr(x, "order")
    if (is.null(phyOrder) || phyOrder != "cladewise") {
      x <- ape::reorder.phylo(x)
      if (!identical(x$edge, xe)) {
        ereorder <- match(x$edge[, 2], xe[, 2])
        if (length(edge.color) > 1) {
          edge.color <- rep(edge.color, length.out = Nedge)
          edge.color <- edge.color[ereorder]
        }
        if (length(edge.width) > 1) {
          edge.width <- rep(edge.width, length.out = Nedge)
          edge.width <- edge.width[ereorder]
        }
        if (length(edge.lty) > 1) {
          edge.lty <- rep(edge.lty, length.out = Nedge)
          edge.lty <- edge.lty[ereorder]
        }
      }
    }
    yy <- numeric(Ntip + Nnode)
    TIPS <- x$edge[x$edge[, 2] <= Ntip, 2]
    yy[TIPS] <- seq_len(Ntip)
  }
  z <- ape::reorder.phylo(x, order = "postorder") ##
  if (phyloORclado) {
    if (is.null(node.pos)) 
      node.pos <- if (type == "cladogram" && !use.edge.length) 
        2
    else 1
    if (node.pos == 1) 
      yy <- .nodeHeight(z$edge, Nedge, yy)
    else {
      ans <- .C(ape::node_height_clado, as.integer(Ntip), as.integer(z$edge[, 
                                                                            1]), as.integer(z$edge[, 2]), as.integer(Nedge), 
                double(Ntip + Nnode), as.double(yy))
      xx <- ans[[5]] - 1
      yy <- ans[[6]]
    }
    if (!use.edge.length) {
      if (node.pos != 2) 
        xx <- .nodeDepth(Ntip, Nnode, z$edge, Nedge, 
                         node.depth) - 1
      xx <- max(xx) - xx
    }
    else {
      xx <- .nodeDepthEdgelength(Ntip, Nnode, z$edge, Nedge, 
                                 z$edge.length)
    }
  }
  else {
    twopi <- 2 * pi
    rotate.tree <- twopi * rotate.tree/360
    if (type != "unrooted") {
      TIPS <- x$edge[which(x$edge[, 2] <= Ntip), 2]
      xx <- seq(0, twopi * (1 - 1/Ntip) - twopi * open.angle/360, 
                length.out = Ntip)
      theta <- double(Ntip)
      theta[TIPS] <- xx
      theta <- c(theta, numeric(Nnode))
    }
    switch(type, fan = {
      theta <- .nodeHeight(z$edge, Nedge, theta)
      if (use.edge.length) {
        r <- .nodeDepthEdgelength(Ntip, Nnode, z$edge, 
                                  Nedge, z$edge.length)
      } else {
        r <- .nodeDepth(Ntip, Nnode, z$edge, Nedge, node.depth)
        r <- 1/r
      }
      theta <- theta + rotate.tree
      if (root.edge) r <- r + x$root.edge
      xx <- r * cos(theta)
      yy <- r * sin(theta)
    }, unrooted = {
      nb.sp <- .nodeDepth(Ntip, Nnode, z$edge, Nedge, node.depth)
      XY <- if (use.edge.length) ape::unrooted.xy(Ntip, Nnode, 
                                                  z$edge, z$edge.length, nb.sp, rotate.tree) else unrooted.xy(Ntip, 
                                                                                                              Nnode, z$edge, rep(1, Nedge), nb.sp, rotate.tree)
      xx <- XY$M[, 1] - min(XY$M[, 1])
      yy <- XY$M[, 2] - min(XY$M[, 2])
    }, radial = {
      r <- .nodeDepth(Ntip, Nnode, z$edge, Nedge, node.depth)
      r[r == 1] <- 0
      r <- 1 - r/Ntip
      theta <- .nodeHeight(z$edge, Nedge, theta) + rotate.tree
      xx <- r * cos(theta)
      yy <- r * sin(theta)
    })
  }
  if (phyloORclado) {
    if (!horizontal) {
      tmp <- yy
      yy <- xx
      xx <- tmp - min(tmp) + 1
    }
    if (root.edge) {
      if (direction == "rightwards") 
        xx <- xx + x$root.edge
      if (direction == "upwards") 
        yy <- yy + x$root.edge
    }
  }
  if (no.margin) 
    par(mai = rep(0, 4))
  if (show.tip.label) 
    nchar.tip.label <- nchar(x$tip.label)
  max.yy <- max(yy)
  getLimit <- function(x, lab, sin, cex) {
    s <- strwidth(lab, "inches", cex = cex)
    if (any(s > sin)) 
      return(1.5 * max(x))
    Limit <- 0
    while (any(x > Limit)) {
      i <- which.max(x)
      alp <- x[i]/(sin - s[i])
      Limit <- x[i] + alp * s[i]
      x <- x + alp * s
    }
    Limit
  }
  if (is.null(x.lim)) {
    if (phyloORclado) {
      if (horizontal) {
        xx.tips <- xx[seq_len(Ntip)]
        if (show.tip.label) {
          pin1 <- par("pin")[1]
          tmp <- getLimit(xx.tips, x$tip.label, pin1, 
                          cex)
          tmp <- tmp + label.offset
        }
        else tmp <- max(xx.tips)
        x.lim <- c(0, tmp)
      }
      else x.lim <- c(1, Ntip)
    }
    else switch(type, fan = {
      if (show.tip.label) {
        offset <- max(nchar.tip.label * 0.018 * max.yy * 
                        cex)
        x.lim <- range(xx) + c(-offset, offset)
      } else x.lim <- range(xx)
    }, unrooted = {
      if (show.tip.label) {
        offset <- max(nchar.tip.label * 0.018 * max.yy * 
                        cex)
        x.lim <- c(0 - offset, max(xx) + offset)
      } else x.lim <- c(0, max(xx))
    }, radial = {
      if (show.tip.label) {
        offset <- max(nchar.tip.label * 0.03 * cex)
        x.lim <- c(-1 - offset, 1 + offset)
      } else x.lim <- c(-1, 1)
    })
  }
  else if (length(x.lim) == 1) {
    x.lim <- c(0, x.lim)
    if (phyloORclado && !horizontal) 
      x.lim[1] <- 1
    if (type %in% c("fan", "unrooted") && show.tip.label) 
      x.lim[1] <- -max(nchar.tip.label * 0.018 * max.yy * 
                         cex)
    if (type == "radial") 
      x.lim[1] <- if (show.tip.label) 
        -1 - max(nchar.tip.label * 0.03 * cex)
    else -1
  }
  if (phyloORclado && direction == "leftwards") 
    xx <- x.lim[2] - xx
  if (is.null(y.lim)) {
    if (phyloORclado) {
      if (horizontal) 
        y.lim <- c(1, Ntip)
      else {
        pin2 <- par("pin")[2]
        yy.tips <- yy[seq_len(Ntip)]
        if (show.tip.label) {
          tmp <- getLimit(yy.tips, x$tip.label, pin2, 
                          cex)
          tmp <- tmp + label.offset
        }
        else tmp <- max(yy.tips)
        y.lim <- c(0, tmp)
      }
    }
    else switch(type, fan = {
      if (show.tip.label) {
        offset <- max(nchar.tip.label * 0.018 * max.yy * 
                        cex)
        y.lim <- c(min(yy) - offset, max.yy + offset)
      } else y.lim <- c(min(yy), max.yy)
    }, unrooted = {
      if (show.tip.label) {
        offset <- max(nchar.tip.label * 0.018 * max.yy * 
                        cex)
        y.lim <- c(0 - offset, max.yy + offset)
      } else y.lim <- c(0, max.yy)
    }, radial = {
      if (show.tip.label) {
        offset <- max(nchar.tip.label * 0.03 * cex)
        y.lim <- c(-1 - offset, 1 + offset)
      } else y.lim <- c(-1, 1)
    })
  }
  else if (length(y.lim) == 1) {
    y.lim <- c(0, y.lim)
    if (phyloORclado && horizontal) 
      y.lim[1] <- 1
    if (type %in% c("fan", "unrooted") && show.tip.label) 
      y.lim[1] <- -max(nchar.tip.label * 0.018 * max.yy * 
                         cex)
    if (type == "radial") 
      y.lim[1] <- if (show.tip.label) 
        -1 - max(nchar.tip.label * 0.018 * max.yy * cex)
    else -1
  }
  if (phyloORclado && direction == "downwards") 
    yy <- y.lim[2] - yy
  if (phyloORclado && root.edge) {
    if (direction == "leftwards") 
      x.lim[2] <- x.lim[2] + x$root.edge
    if (direction == "downwards") 
      y.lim[2] <- y.lim[2] + x$root.edge
  }
  asp <- if (type %in% c("fan", "radial", "unrooted")) 1 else NA
  L <- list(type = type, use.edge.length = use.edge.length, 
            node.pos = node.pos, node.depth = node.depth, show.tip.label = show.tip.label, 
            show.node.label = show.node.label, font = font, cex = cex, 
            adj = adj, srt = srt, no.margin = no.margin, label.offset = label.offset, 
            x.lim = x.lim, y.lim = y.lim, direction = direction, 
            tip.color = tip.color, Ntip = Ntip, Nnode = Nnode, root.time = x$root.time, 
            align.tip.label = align.tip.label)
  invisible(L)
}

#' @name plotReducedDims
#' @title Plot 2-dimensionsal representation with clusters
#' @description Plot a 2-dimensional representation of the data, color-code by a
#'   clustering.
#' @param object a ClusterExperiment object
#' @param reducedDim What dimensionality reduction method to use. Should match
#'   either a value in \code{reducedDimNames(object)} or one of the built-in
#'   functions of \code{\link{listBuiltInReducedDims}()}
#' @param whichDims vector of length 2 giving the indices of which dimensions to
#'   show. The first value goes on the x-axis and the second on the y-axis.
#' @param clusterLegend matrix with three columns and colnames
#'   'clusterIds','name', and 'color' that give the color and name of the
#'   clusters in whichCluster. If NULL, pulls the information from
#'   \code{object}.
#' @param legend either logical, indicating whether to plot legend, or character
#'   giving the location of the legend (passed to \code{\link{legend}})
#' @param xlab Label for x axis
#' @param ylab Label for y axis
#' @param unassignedColor If not NULL, should be character value giving the
#'   color for unassigned (-1) samples (overrides \code{clusterLegend}) default.
#' @param missingColor If not NULL, should be character value giving the color
#'   for missing (-2) samples (overrides \code{clusterLegend}) default.
#' @param plotUnassigned logical as to whether unassigned (either -1 or -2
#'   cluster values) should be plotted.
#' @param pch the point type, passed to \code{plot.default}
#' @param legendTitle character value giving title for the legend. If NULL, uses
#'   the clusterLabels value for clustering.
#' @param nColLegend The number of columns in legend. If missing, picks number 
#'   of columns internally. 
#' @param ... arguments passed to \code{\link{plot.default}}
#' @inheritParams getClusterIndex
#' @details If \code{plotUnassigned=TRUE}, and the color for -1 or -2 is set to
#'   "white", will be coerced to "lightgrey" regardless of user input to
#'   \code{missingColor} and \code{unassignedColor}. If \code{plotUnassigned=FALSE},
#'   the samples with -1/-2 will not be plotted, nor will the category show up in the
#'   legend.
#' @details If the requested \code{reducedDim} method has not been created yet,
#'   the function will call \code{\link{makeReducedDims}} on the FIRST assay of
#'   \code{x}. The results of this method will be saved as part of the object
#'   and returned INVISIBLY (meaning if you don't save the output of the
#'   plotting command, the results will vanish). To pick another assay, you
#'   should call `makeReducedDims` directly and specify the assay.
#' @seealso \code{\link{plot.default}}, \code{\link{makeReducedDims}}, \code{\link{listBuiltInReducedDims}()}
#' @rdname plotReducedDims
#' @return A plot is created. Nothing is returned.
#' @examples
#' #clustering using pam: try using different dimensions of pca and different k
#' data(simData)
#'
#' cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reducedDim="PCA",
#' clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
#' removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)
#'
#' plotReducedDims(cl,legend="bottomright")
#' @export
#' @aliases plotReducedDims plotReducedDims,ClusterExperiment-method
setMethod(
  f = "plotReducedDims",
  signature = signature(object = "ClusterExperiment"),
  definition = function(object, whichCluster="primary",
                        reducedDim="PCA",whichDims=c(1,2),plotUnassigned=TRUE,legend=TRUE,legendTitle="",nColLegend=6,
                        clusterLegend=NULL,unassignedColor=NULL,missingColor=NULL,pch=19,xlab=NULL,ylab=NULL,...)
  {
	whichCluster<-getSingleClusterIndex(object,whichCluster,passedArgs=list(...))
    cluster<-clusterMatrix(object)[,whichCluster]
    if("col" %in% names(list(...))) stop("plotting parameter 'col' may not be passed to plot.default. Colors must be set via 'clusterLegend' argument.")
    if(is.null(clusterLegend)){
      clusterLegend<-clusterLegend(object)[[whichCluster]]
    }
    else{
      if(is.null(dim(clusterLegend)) || ncol(clusterLegend)!=3 || !all(c("clusterIds","name","color") %in% colnames(clusterLegend))) stop("clusterLegend must be a matrix with three columns and names 'clusterIds','name', and 'color'")
      if(!all(as.character(unique(cluster)) %in% clusterLegend[,"clusterIds"])) stop("'clusterIds' in 'clusterLegend' do not match clustering values")
    }
    clColor<-clusterLegend[,"color"]
    names(clColor)<-clusterLegend[,"name"]
    wh1<-which(clusterLegend[,"clusterIds"]=="-1")
    wh2<-which(clusterLegend[,"clusterIds"]=="-2")
    if(!is.null(unassignedColor) && is.character(unassignedColor)){
      if(length(wh1)>0) clColor[wh1]<-unassignedColor
    }
    if(!is.null(missingColor) && is.character(missingColor)){
      if(length(wh2)>0) clColor[wh2]<-missingColor
    }
    if(length(wh1)>0){
      if(plotUnassigned && clColor[wh1]=="white") clColor[wh1]<-"lightgrey"
      else if(!plotUnassigned) clColor[wh1]<-NA
      names(clColor)[wh1]<-"Unassigned"
      clusterLegend[wh1,"name"]<-"Unassigned"
      clColor<-c(clColor[-wh1],clColor[wh1])
    }
    if(length(wh2)>0){
      if(plotUnassigned && clColor[wh2]=="white") clColor[wh2]<-"lightgrey"
      else if(!plotUnassigned) clColor[wh2]<-NA
      names(clColor)[wh2]<-"Missing"
      clusterLegend[wh2,"name"]<-"Missing"
      clColor<-c(clColor[-wh2],clColor[wh2])
    }
    
    clFactor<-factor(as.character(cluster),levels=clusterLegend[,"clusterIds"], labels=clusterLegend[,"name"])
    
    #################
    ####Dim reduction stuff:
    #################
    if(length(whichDims)<2)
      stop("whichDims must be a vector of length at least 2 giving the which dimensions of the dimensionality reduction to plot")
    redoDim<-FALSE
    if(!reducedDim %in% reducedDimNames(object) & reducedDim %in% listBuiltInReducedDims()) redoDim<-TRUE
    if(reducedDim %in% reducedDimNames(object)){
      #check if ask for higher dim than available
      if(max(whichDims)>NROW(object) || max(whichDims)>NCOL(object)) stop("Invalid value for whichDims: larger than row or column")
      if(max(whichDims)>ncol(reducedDim(object,type=reducedDim))) redoDim<-TRUE
    }
    
    if(redoDim){
     object<-makeReducedDims(object,reducedDims=reducedDim,maxDims=max(whichDims),whichAssay=1)
		 warning(paste("'makeReducedDims' is being called because requested method",reducedDim,"has not been created yet for this object. 'makeReducedDims' will be run on the FIRST assay; for a different assay call 'makeReducedDims' directly."))	
    }
    if(reducedDim %in% reducedDimNames(object)){
      
      dat<-reducedDim(object,type=reducedDim)[,whichDims]
    }
    else stop("'reducedDim' does not match saved dimensionality reduction nor built in methods.")
    
    if(length(whichDims)==2){
      if(is.null(ylab)) ylab<-paste("Dimension",whichDims[2])
      if(is.null(xlab)) xlab<-paste("Dimension",whichDims[1])
      
      graphics::plot(dat,col=clColor[as.character(clFactor)],pch=pch,xlab=xlab,ylab=ylab,...)
      
      doLegend<-FALSE
      
      if(is.logical(legend)) {
        if(legend) {
          doLegend<-TRUE
          legend<-"topright"
        }
      } else{
        if(is.character(legend)) doLegend<-TRUE
        else stop("legend must either be logical or character value.")
      }
      
      if(doLegend){
        if(!plotUnassigned){
          wh1<-which(names(clColor)=="Unassigned")
          wh2<-which(names(clColor)=="Missing")
          if(length(wh1)>0) clColor<-clColor[-wh1]
          if(length(wh2)>0) clColor<-clColor[-wh2]
        }
        if(is.null(legendTitle)) legendTitle<-clusterLabels(object)[whichCluster]
			if(missing(nColLegend)){
				nPerColLegend<-6
				nColLegend<-length(clColor) %/% nPerColLegend					
				if(length(clColor)%% nPerColLegend > 0) nColLegend<-nColLegend+1
			}
			graphics::legend(x=legend, legend=names(clColor), fill=clColor,title=legendTitle,ncol=nColLegend)
      }
      
    }
    else if(length(whichDims)>2){
      colnames(dat)<-paste("Dim.",whichDims,sep="")
      graphics::pairs(data.frame(dat),col=clColor[as.character(clFactor)],pch=pch,...)
    }
    
    invisible(object)
    
  })

#######
#Internal algorithms for cluster functions
#######

.clusterVectorToList <- function(vec) {
    clList <- tapply(seq_along(vec), vec, function(x) {
        x
    }, simplify = FALSE)
    whNotAssign <- which(sapply(clList, function(x) {
        all(vec[x] == -1)
    }))
    if (length(whNotAssign) > 1)
        stop("Internal coding error in removing unclustered samples")
    if (length(whNotAssign) > 0)
        clList <- clList[-whNotAssign]
    return(clList)
}
.clusterListToVector <- function(clusterList, N)
{
    clust.id <- rep(-1, N)
    nfound <- length(clusterList)
    if (nfound > 0) {
        #make cluster ids in order of when found
        for (i in seq_along(clusterList))
            clust.id[clusterList[[i]]] <- i
    }
    return(clust.id)
}
.checkDissFunction <- function(D, algType = NA) {
    if (anyNA(D))
        stop(
            "NA values found in dissimilarity matrix (could be from too small of subsampling if classifyMethod!='All', see documentation of subsampleClustering)"
        )
    if (any(is.nan(D) |
            is.infinite(D)))
        stop("Dissimilarity matrix contains either NAs, NANs or Infinite values.")
    if (any(D < 0))
        stop("Dissimilarity matrix must have strictly positive values")
    if (any(diag(D) != 0))
        stop("Dissimilarity matrix must have zero values on the diagonal")
    if (!all(D == t(D)))
        stop("Dissimilarity matrix must result in a symmetric matrix")
    if (algType == "01" &
        any(D > 1))
        stop(
            "distance function must give values between 0 and 1 when algorithm type of the ClusterFunction object is '01'"
        )
}



.defaultDiss<-c("euclidean", "maximum", "manhattan", "canberra", "binary" ,"minkowski")
.myDefaultDiss<-.algTypes
.diss01<-function(x) {
    (1 - cor(t(x))) / 2
}
.dissK<-function(x) {
    dist(x) #distance finds between rows of matrix
}

## x is a nfeatures x nsamples matrix
.makeDiss <- function(x, distFunction, algType, checkDiss) {
    if (!is.function(distFunction)) {
        if (length(distFunction) > 1)
            stop("if distFunction is not a function, it must be of length 1")
        if (is.character(distFunction) & distFunction!="default") {
            ## Handle defaults in `dist` function
            if(distFunction %in% .defaultDiss){
                distMethod<-distFunction
                distFunction<-function(x){dist(x,method=distMethod)}
            }
            else{
                if(distFunction %in% .myDefaultDiss){
                    if(distFunction=="01") distFunction<-.diss01
                    else if(distFunction=="K") distFunction<-.dissK
                }
                else distFunction <- get(distFunction, envir = globalenv())
            }
        } else if (is.na(distFunction) || distFunction=="default") {
            distFunction <-
                switch(
                    algType,
                    "01" = .diss01,
                    "K" = .dissK
                )
        } else
            stop("if distFunction is not a function, it must be either NA or a character")
    }
    ### FIXME: HDF5 optimization could be improved?
    ### Add data.matrix here for HDF5, not optimized.
    D <-try(as.matrix(distFunction(data.matrix(t(x)))))
    #distances assumed to be of observations on rows
    if (inherits(D, "try-error"))
        stop("given distance function gives error when applied to x")
    if (!all(dim(D) == c(ncol(x), ncol(x))))
        stop(
            "given distance function must result in a ",
            ncol(x),
            "by",
            ncol(x),
            "matrix of distances"
        )
    if (checkDiss)
        .checkDissFunction(D, algType = algType)
    return(D)
    
    
}



.orderByAlpha <- function(res, S)
{
    if (length(res) > 0) {
        alphaMax <- unlist(lapply(res, function(x) {
            vals <- lower.tri(S[x, x]) #don't grab diag
            1 - min(vals) #max(alpha)=1-min(S)
        }))
        res <- res[order(alphaMax, decreasing = TRUE)]
        
    }
    else
        return(res)
}


# if inputType is character value, returns if matches desiredInputType, otherwise throws appropriate errors. 

.checkCFInput <- function(inputType, desiredInputType) {
    if (!all(desiredInputType %in% .inputTypes))
        stop(sprintf(
            "not all values in 'desiredInputType' match allowable values (%s)",
            paste(.inputTypes, ", ")
        ))
    intersect<-intersect(desiredInputType,inputType)
    if(length(intersect)==0){
        stop(sprintf("given clusterFunction/classifyFunction does not take as input matrices of type %s",paste(inputType,collapse=" or ")))
    }
    return(inputType)
}

###This function checks the mainClusterArgs and subsampleArgs to make sure makes sense with combination of sequential, subsample, x, and diss given by the user.
#' @return If there is error, returns a character string describing error, otherwise returns list with necessary information:
#' mainClusterArgs
#' subsampleArgs
#' makeDiss (if allowMakeDiss=FALSE, will always be FALSE)
#' Note that warnings will be printed if warn=TRUE (different than errors...)
.checkArgs <-
    function(inputType,
             main=TRUE,
             subsample,
             sequential,
             mainClusterArgs,
             subsampleArgs,
             seqArgs=NULL,
             allowMakeDiss=FALSE,
             warn = TRUE) {
        ########
        #checks for mainClustering stuff
        ########
        inputType<-match.arg(inputType,.inputTypes)
        doDiss<-FALSE #If returns TRUE, then was a mismatch, but could be fixed by X -> Diss
        doDissPost<-FALSE
        if (main) {
            if ("clusterFunction" %in% names(mainClusterArgs)) {
                #get clusterFunction for cluster D
                clusterFunction <- mainClusterArgs[["clusterFunction"]]
                if (is.character(clusterFunction))
                    clusterFunction <- getBuiltInFunction(clusterFunction)
                
                if(subsample){
                    # Check that mainClustering cluster function takes 
                    # input that is cat
                    # Reason: if subsampling, then the results from subsampling 
                    # sent to the clusterFunction.
                    # This would be caught below anyway, 
                    # but this is a more informative error
                    mainInputType<-"cat"
                    if (!"cat" %in% inputType(clusterFunction))
                        return(
                            "If choosing subsample=TRUE, the clusterFunction used in the mainClustering step must take input that is a categorical (inputType='cat') matrix."
                        )
                }
                else mainInputType<-inputType
                inputMain <- try(.checkCFInput(
                    inputType=mainInputType,
                    desiredInputType = inputType(clusterFunction)),
                    silent=TRUE)
                if(inherits(inputMain, "try-error")){
                    ##In clusterSingle, can build diss if should.
                    mess<-paste("In mainClustering step,",inputMain)
                    if(allowMakeDiss){
                        ## Reason: if subsample=TRUE, then don't want to calculate dissimilarity matrix or anything. Just want check that it is categorical. 
                       if(!subsample){
                           if(mainInputType=="X" & "diss" %in% inputType(clusterFunction)){
                               doDiss<-TRUE 
                               inputMain<-"diss"
                           }
                           else return(mess) 
                       }
                       else return(mess) 
                    }
                    else{
                        return(mess) 
                    }
                } 
                algType <- algorithmType(clusterFunction)
            }
            else{
                return(
                    "Must provide 'clusterFunction' for the mainClustering step to be able to run (give 'clusterFunction' argument via 'mainClusterArgs')"
                )
            }
            #-----
            # removes certain options based on others
            #-----
            reqArgs <- requiredArgs(clusterFunction)
            # remove 'k' if sequential is given
            if (sequential & length(reqArgs) > 0)
                reqArgs <- reqArgs[-which(reqArgs == "k")]
            # remove 'k' if choose 'findBestK=TRUE'
            if (length(reqArgs) > 0 &
                algorithmType(clusterFunction) == "K" &
                "findBestK" %in% names(mainClusterArgs)) {
                if (mainClusterArgs[["findBestK"]])
                    reqArgs <- reqArgs[-which(reqArgs == "k")]
            }
            #------
            # Check have required args for mainClustering
            #------
            if (length(reqArgs) > 0) {
                if (("clusterArgs" %in% names(mainClusterArgs) &
                     !all(reqArgs %in% names(mainClusterArgs[["clusterArgs"]]))) ||
                    !("clusterArgs" %in% names(mainClusterArgs)))
                    return(
                        paste(
                            "For the clusterFunction algorithm type ('",
                            algorithmType(clusterFunction),
                            "') given in 'mainClusterArgs', must supply arguments:",
                            reqArgs,
                            "These must be supplied as elements of the list of 'clusterArgs' given in 'mainClusterArgs'"
                        )
                    )
                    #----------
                    # check clusterArgs alpha and k are valid
                    #----------
                    if("clusterArgs" %in% names(mainClusterArgs)){
                        clArgs<-mainClusterArgs[["clusterArgs"]]
                        if("alpha" %in% names(clArgs) & algType=="01"){
                            if(is.na(clArgs[["alpha"]]) ||
                                clArgs[["alpha"]]<0 || 
                                clArgs[["alpha"]]>1){
                                return("alpha parameter for type 01 clustering functions must be in (0,1)")
                            }
                        }
                        if("k" %in% names(clArgs) & algType=="K"){
                            if(clArgs[["k"]]<0 || 
                                !is.whole(clArgs[["k"]])){
                                return("k parameter for type K clustering functions must be positive whole number")
                            }
                        }
                    }
            }
            
            #--------
            #Check that minSize is valid
            #--------
            if("minSize" %in% names(mainClusterArgs)){
                if (!is.numeric(mainClusterArgs[["minSize"]]) ||
                    mainClusterArgs[["minSize"]] < 0)
                    return(
                        "Invalid value for the 'minSize' parameter in determining the minimum number of samples required in a cluster (in main clustering step)."
                    )
                else
                    mainClusterArgs[["minSize"]] <-
                        round(mainClusterArgs[["minSize"]]) #in case not integer.                
            }
            
            #-------
            ## Check post-processing arguments
            #-------
            doKPostProcess <- FALSE
            
            # whPostProcess args are which of given minClusterArgs are user options to give to .postProcess
            # They will ultimately be the only ones returned back to mainClustering to pass along.
            whPostProcessArgs <-
                which(names(mainClusterArgs) %in% getPostProcessingArgs(clusterFunction))
            if (length(whPostProcessArgs) > 0 | !is.null(mainClusterArgs[["extraArguments"]])) {
                #get rid of wrong args passed because of user confusion between the two
                
                postProcessArgs <- mainClusterArgs[whPostProcessArgs]
                if (clusterFunction@algorithmType == "K") {
                    if ("findBestK"  %in% names(postProcessArgs) &&
                        postProcessArgs[["findBestK"]])
                        doKPostProcess <- TRUE
                    if ("removeSil"  %in% names(postProcessArgs) &&
                        postProcessArgs[["removeSil"]])
                        doKPostProcess <- TRUE
                    if(doKPostProcess){
                        if(inputMain !="diss"){
                            if(!"diss" %in% names(mainClusterArgs)){
                                if(allowMakeDiss & mainInputType=="X"){
                                    doDissPost<-TRUE
                                }
                                else{
                                    return("Cannot do requested post processing (e.g. from arguments 'findBestK' or 'removeSil') without calculating distance matrix")
                                }
                            }
                        }
                    }
                }
                
                
                # remove existing ones, and only keep usable ones.
                # Only relevant in the check run in mainClustering, 
                # where need get rid of extra ones.
                if(!is.null(mainClusterArgs[["extraArguments"]])){
                    exArgsNames<-mainClusterArgs[["extraArguments"]]
                    
                    if (length(whPostProcessArgs) !=
                         length(exArgsNames) & warn){
                             
                             whNotMatch<- !exArgsNames %in%
                                  names(mainClusterArgs)[whPostProcessArgs]
                             warning(
                                 sprintf("Some arguments passed via mainClusterArgs in mainClustering step do not match the algorithmType of the given ClusterFunction object: %s",paste(exArgsNames[whNotMatch],collapse=",")))
                         }
                        
                    whRm<-which(names(mainClusterArgs) %in% exArgsNames)
                    if(length(whRm)>0){
                        mainClusterArgs<-mainClusterArgs[-whRm]
                        mainClusterArgs<-c(mainClusterArgs,postProcessArgs)           
                    }
                    mainClusterArgs[["extraArguments"]] <- names(postProcessArgs)     
                }
            }
            mainClusterArgs[["doKPostProcess"]] <- doKPostProcess
            


            mainClusterArgs[["inputType"]]<-inputMain
			if(!"checkArgs" %in% names(mainClusterArgs[["clusterArgs"]]))
                mainClusterArgs[["clusterArgs"]][["checkArgs"]]<-warn
            mainClusterArgs[["warnings"]]<-warn
            #----------
            # Adapt MainClusteringArgs if using Sequential
            #----------
            if (sequential) {
                # Remove argument 'k'
                # Reason: if subsample=FALSE, then need to change k of the mainClustering step for sequential. If subsample=TRUE, similarly set the k of mainClustering step to match that used in subsample. Either way, can't be predefined by user
                if ("clusterArgs" %in% names(mainClusterArgs)) {
                    if ("k" %in% names(mainClusterArgs[["clusterArgs"]])) {
                        #remove predefined versions of k from both.
                        whK <- which(names(mainClusterArgs[["clusterArgs"]]) == "k")
                        if (warn)
                            warning(
                                "Setting 'k' in mainClusterArgs when sequential clustering is requested will have no effect."
                            )
                        mainClusterArgs[["clusterArgs"]] <-
                            mainClusterArgs[["clusterArgs"]][-whK]
                    }
                } else{
                    #make it exist... not sure if I need this.
                    mainClusterArgs[["clusterArgs"]] <- NULL 
                }
                if(!subsample){
                    # Reason: if subsample=FALSE, and sequential=TRUE 
                    # then need to adjust K in the mainClustering step, so need algorithm of type K
                    if (algorithmType(clusterFunction) != "K") {
                        return(
                            "if subsample=FALSE, sequentical clustering can only be implemented with a clusterFunction for mainClustering step with algorithmType 'K'. See documentation of seqCluster."
                        )
                        #Not sure why this was here!
                        # subsampleArgs <-
                        #                         subsampleArgs[-which(names(subsampleArgs) == "clusterFunction")]
                    }
                    # Reason: subsample=FALSE can't do sequential clustering 
                    # and findBestK=TRUE because need to remove cluster based on
                    # testing many k and finding stable, and if not doing it over 
                    # subsample, then do it over actual clustering
                    if ("findBestK" %in% names(mainClusterArgs)) {
                        if (mainClusterArgs[["findBestK"]])
                            return(
                                "Cannot do sequential clustering where subsample=FALSE and 'findBestK=TRUE' is passed to the mainClustering step via mainClusterArgs. See help documentation of seqCluster."
                            )
                    }     
                }  
            }

        }
        
        

        
        #########
        # Checks related to subsample=TRUE
        #########
        if (subsample) {
            if(is.null(subsampleArgs)){
                if(main) subsampleArgs<-list()
                else{
                    #I don't think this can ever actually happen:
                    return("if subsample=TRUE and mainClusterArgs not given, must give argument subsampleArgs so as to identify the clusterFunction")}
            }
            
            # Note if subsample=TRUE, then inputType refers to the subsampling, not the main step.
            inputSub<-inputType
            # Set default for when clusterFunction for subsampling 
            # not given or needs to be overridden (only used if main=TRUE)
            default <- switch(inputSub, 
                              X = "kmeans",
                              cat = "hier01",
                              diss = "pam")
            #---
            # If no cluster function given for subsampling...
            # set some defaults if at top level 
            #---
            if(main & !"clusterFunction" %in% names(subsampleArgs)){
                cfInp<-inputType(mainClusterArgs[["clusterFunction"]])
                if (inputSub %in% cfInp) {
                    mess <-
                        "a clusterFunction was not given for subsampleClustering -- set to be the same as the mainClustering step"
                    if (is.character(mainClusterArgs[["clusterFunction"]]))
                        mess <- sprintf("%s (%s)", mess, mainClusterArgs[["clusterFunction"]])
                    if (warn)
                        .mynote(mess)
                    subsampleArgs[["clusterFunction"]] <- clusterFunction
                    diffSubsampleCF <- FALSE
                }
                else{
                    if(doDiss & inputSub=="X" & "diss" %in% cfInp){
                        # If already going to be calculating 
                        # the dissimilarity matrix for the main step
                        # go ahead and reuse for the subsample step. 
                        mess<-sprintf("a clusterFunction was not given for subsampleClustering -- set to be the same as the mainClustering step")
                        if(warn) .mynote(mess)
                        subsampleArgs[["clusterFunction"]] <- clusterFunction
                        diffSubsampleCF <- FALSE
                    }
                    else{
                        if(warn){
                            mess<-sprintf("a clusterFunction was not given for subsampleClustering (and inputMatrix is not of type allowed by clusterFunction of mainClustering step). The clusterFunction for subsampling was set to the default of %s",
                                default)
                            .mynote(mess)
                        }
                        subsampleArgs[["clusterFunction"]] <- default
                        diffSubsampleCF <- TRUE
                    }
                }
            }
            
            #--------
            # Evaluate subsample clusterFunction
            #--------
            if ("clusterFunction" %in% names(subsampleArgs)) {
                #---
                #Checks for when cluster function set by user for subsampling..
                #---
                subsampleCF <- subsampleArgs[["clusterFunction"]]
                if (is.character(subsampleCF))
                    subsampleCF <- getBuiltInFunction(subsampleCF)
                subsampleAlgType <- algorithmType(subsampleCF)
                inputSub<-try(.checkCFInput(inputType=inputSub, 
                    desiredInputType=inputType(subsampleCF)),
                    silent=TRUE)
                if(inherits(inputSub, "try-error")){
                    mess<-paste("In subsampling clustering step,",inputSub)
                    ##In clusterSingle, can build diss if should.
                    if(allowMakeDiss){
                       if(inputType=="X" & "diss" %in% inputType(subsampleCF)){
                           doDiss<-TRUE 
                           inputSub<-"diss"
                       }
                       else return(mess) 
                    }
                    else{
                        return(mess) 
                    }
                } 
                
                # Check that subsample clustering function is of type 'k' if sequential=TRUE
                # Reason: seqCluster requires subsampling cluster function to be of type "K"
                if (sequential & algorithmType(subsampleCF) != "K") {
                    if (warn)
                        warning(
                            sprintf(
                                "If subsample=TRUE, sequentical clustering can only be implemented with a clusterFunction for subsampling that has algorithmType 'K'. See documentation of seqCluster. Will ignore 'clusterFunction' argument of subsampleArgs and set to default of %s",
                                default
                            )
                        )
                    subsampleArgs[["clusterFunction"]] <- default
                    diffSubsampleCF <- TRUE
                }
            }
            else return("Must give 'clusterFunction' value to 'subsampleClustering'.")
            #-----
            #Check the classifyMethod argument
            #-----
            if(!is.null(subsampleArgs[["classifyMethod"]])){
                if (is.null(subsampleCF@classifyFUN)) {
                    if ("classifyMethod" %in% names(subsampleArgs) &&
                        subsampleArgs[["classifyMethod"]] != "InSample")
                        if(warn) warning(
                            "Cannot set 'classifyMethod' to anything but 'InSample' if do not specify a clusterFunction in subsampleArgs that has a non-null classifyFUN slot. Changing classifyMethod to 'InSample'."
                        )
                    subsampleArgs[["classifyMethod"]] <- "InSample"
                }
                if(subsampleArgs[["classifyMethod"]]!="InSample"){
                    if(!inputSub %in% subsampleCF@inputClassifyType){
                        if(warn) warning(sprintf("The clusterFunction for subsampling chosen does not provide ability to classify new samples from input type %s to clusters in the subsampling step (needed by classifyMethod=%s in 'subsampleArgs'). Changing classifyMethod to 'InSample'.", inputSub, subsampleArgs[["classifyMethod"]]))
                        subsampleArgs[["classifyMethod"]]<-"InSample"
                    }
                }
            }

            
            ##------
            ## Check have required args for subsample. 
            ## If missing, 'borrow' those args from mainClusterArgs.
            ##------
            reqSubArgs <- requiredArgs(subsampleCF)
            
            #Reason: sequential sets k for the subsampling via k0
            if (sequential & length(reqSubArgs) > 0)
                reqSubArgs <- reqSubArgs[-which(reqSubArgs == "k")]
            if (length(reqSubArgs) > 0){
                if(main){
                    #check if can borrow...
                    if ("clusterArgs" %in% names(mainClusterArgs)) {
                        mainReqArgs <- requiredArgs(clusterFunction)
                        mainReqArgs <-
                            mainReqArgs[mainReqArgs %in%
                            names(mainClusterArgs[["clusterArgs"]])]
                        if (!is.null(subsampleArgs) &&
                            "clusterArgs" %in% names(subsampleArgs)) {
                            # check if existing clusterArgs has required names already
                            # Define missingArgs to be those that are needed and available from main 
                            if (!all(reqSubArgs %in%
                                names(subsampleArgs[["clusterArgs"]]))) {
                                missingArgs <-
                                    reqSubArgs[!reqSubArgs %in%
                                    names(subsampleArgs[["clusterArgs"]])]
                                missingArgs <- missingArgs[missingArgs %in% mainReqArgs]
                                
                            }
                            else
                                missingArgs <- vector("character")
                        }
                        else{
                            missingArgs <- reqSubArgs[reqSubArgs %in% mainReqArgs]
                        }
                        if (length(missingArgs) > 0) {
                            subsampleArgs[["clusterArgs"]][missingArgs] <-
                                mainClusterArgs[["clusterArgs"]][missingArgs]
                            if (warn)
                                warning(
                                    "missing arguments ",
                                    missingArgs,
                                    " provided from those in 'mainClusterArgs'"
                                )
                        }
                    }
                }
                #now check if got everything needed...
                if (("clusterArgs" %in% names(subsampleArgs) &
                     !all(reqSubArgs %in% names(subsampleArgs[["clusterArgs"]]))) ||
                    !("clusterArgs" %in% names(subsampleArgs)))
                    return(
                        paste(
                            "For the clusterFunction algorithm type ('",
                            algorithmType(subsampleCF),
                            "') given in 'subsampleArgs', must supply arguments:",
                            reqSubArgs,
                            ". These must be supplied as elements of the list of 'clusterArgs' given in 'subsampleArgs'"
                        )
                    )
                
            }
            
            #---
            #Check don't give distance function and subsample=TRUE
            #Reason, if subsample=TRUE, user can't set distance function because use diss provided from subsampling.
            #---
            if(main &&
               "distFunction" %in% names(mainClusterArgs) &&
               !is.na(mainClusterArgs[["distFunction"]])) {
                if (warn)
                    warning(
                        "if 'subsample=TRUE', 'distFunction' argument in mainClusterArgs is ignored."
                    )
                mainClusterArgs[["distFunction"]] <- NA
            }
            if (sequential) {
                #Reason: if subsampling, sequential goes over different k values, so user can't set k
                if ("clusterArgs" %in% names(subsampleArgs) &&
                    "k" %in% names(subsampleArgs[["clusterArgs"]])) {
                    #remove predefined versions of k from both.
                    whK <- which(names(subsampleArgs[["clusterArgs"]]) == "k")
                    if (warn)
                        warning(
                            "Setting 'k' in subsampleArgs when sequential=TRUE is called will have no effect."
                        )
                    subsampleArgs[["clusterArgs"]] <-
                        subsampleArgs[["clusterArgs"]][-whK]
                }
                if (!"clusterArgs" %in% names(subsampleArgs)) {
                    subsampleArgs[["clusterArgs"]] <- list() #make it if doesn't exist
                }
            }
            subsampleArgs[["inputType"]]<-inputSub
            if(!"checkArgs" %in% names(subsampleArgs[["clusterArgs"]]))
                subsampleArgs[["clusterArgs"]][["checkArgs"]]<-warn
            subsampleArgs[["warnings"]]<-warn
						
        }    
        
        #########
        # Check seqArgs
        #########
        if(sequential){
            if(is.null(seqArgs))
                return("if sequential=TRUE, must give seqArgs so as to identify k0 and beta")
            
            ## Test parameters that need to be whole positive numbers
            testFUN<-function(name){
                if(name %in% names(seqArgs)){
                    if(is.na(seqArgs[[name]]) || !is.whole(seqArgs[[name]]) || seqArgs[[name]]<0) return(sprintf("%s argument of seqArgs must be a positive whole number",name))
                    else return(TRUE)                    
                }
                else return(TRUE)
            }
            if(!"k0"%in%names(seqArgs)) {
                return("required argument 'k0' is missing for the sequential clustering step")
            }
            else{
                out<-testFUN("k0")
                if(!is.logical(out)) return(out)
            }
            if(!"beta" %in% names(seqArgs))
                return("required argument 'beta' is missing for the sequential clustering step")
            else{
                if(is.na(seqArgs[["beta"]]) || seqArgs[["beta"]]<=0 || seqArgs[["beta"]]>=1 ) return("beta parameter for sequential clustering step must be in (0,1)")
            }
            for(nam in c("remain.n","k.min","k.max")){
                out<-testFUN(nam)
                if(!is.logical(out)) return(out)
            }
            if("top.can" %in% names(seqArgs)){
                if(is.na(seqArgs[["top.can"]]) || seqArgs[["top.can"]]<0) 
                    return("top.can parameter of seqArgs cannot be NA or negative value")             
                if(!is.whole(seqArgs[["top.can"]])){
                    if(seqArgs[["top.can"]]>1) return("If top.can parameter of seqArgs is not a whole number, must be between 0 and 1.")
                }
            }
            seqArgs[["warnings"]]<-warn
        }    
        return(
            list(
                mainClusterArgs = mainClusterArgs,
                subsampleArgs = subsampleArgs,
                seqArgs = seqArgs,
                doDiss=doDiss,
                doDissPost=doDissPost
            )
        )
        
    }


# from Martin Maechler  to test if whole number https://stat.ethz.ch/pipermail/r-help/2003-April/032471.html
is.whole <- function(a, tol = 1e-7) { 
   is.eq <- function(x,y) { 
	 r <- all.equal(x,y, tol=tol)
	 is.logical(r) && r 
   }
   (is.numeric(a) && is.eq(a, floor(a))) ||
   (is.complex(a) && {ri <- c(Re(a),Im(a)); is.eq(ri, floor(ri))})
}

#####Function to calculate the reducedDim matrices.


#' @rdname reduceFunctions
#' @param reducedDims a vector of character values indicating the methods of
#'   dimensionality reduction to be performed. Currently only "PCA" is
#'   implemented.
#' @param maxDims Numeric vector of integer giving the number of PC dimensions
#'   to calculate. \code{maxDims} can also take values between (0,1) to indicate
#'   keeping the number of dimensions necessary to account for that proportion
#'   of the variance. \code{maxDims} should be of same length as
#'   \code{reducedDims}, indicating the number of dimensions to keep for each
#'   method (if \code{maxDims} is of length 1, the same number of dimensions
#'   will be used for each).
#' @param ... Values passed on the the 'SingleCellExperiment' method.
#' @details The PCA method uses either \code{prcomp} from the \code{stats}
#'   package or  \code{svds} from the \code{RSpectra} package to perform PCA.
#'   Both are called on \code{t(assay(x))} with \code{center=TRUE} and
#'   \code{scale=TRUE} (i.e. the feature are centered and scaled), so that it is
#'   performing PCA on the correlation matrix of the features.
#' @details Note that this function does not check if such a reduceDim value already exists, and will recalculate (and overwrite) if it does.
#' @return \code{makeReducedDims} returns a \code{\link{SingleCellExperiment}}
#'   containing the calculated dimensionality reduction in the \code{reduceDims}
#'   with names corresponding to the name given in \code{reducedDims}.
#' @examples
#' data(simData)
#' listBuiltInReducedDims()
#' scf<-makeReducedDims(simData, reducedDims="PCA", maxDims=3)
#' scf
#' @export
#' @aliases makeReducedDims,SingleCellExperiment-method makeReducedDims
#' @importFrom matrixStats rowVars
setMethod(
  f = "makeReducedDims",
  signature = "SingleCellExperiment",
  definition = function(object,reducedDims="PCA",maxDims=500,transFun=NULL,isCount=FALSE,whichAssay=1)
  {
    
    ###################
    ##Check user inputs
    ###################
    #check valid options for reducedDims
    validDim<-listBuiltInReducedDims()
    reducedDims<-unique(reducedDims)
    if(length(maxDims)==1) maxDims<-rep(maxDims,length=length(reducedDims))
    if(length(maxDims)!=length(reducedDims)) stop("'maxDims' must be of same length as 'reducedDims'")
    
    ######Check dimensions and valid argument
    for(dr in reducedDims){
      dr<-match.arg(dr,validDim) 
      if(is.na(maxDims) || maxDims>NROW(object) || maxDims > NCOL(object)){
        if(!is.na(maxDims) & (maxDims>NROW(object) || maxDims > NCOL(object)))
          warning("User requested more dimensionality reduction dimensions than the minimimum of number of rows and columns. Will return all dimensions.")
        maxDims<-min(c(NROW(object),NCOL(object)))
      }
      if(maxDims<=0)  stop("the number of reducedDims dimensions must be a value strictly greater than 0")
      
      
    }
    ###################
    ##Clean up data:
    ###################
    #transform data
    x<-transformData(object,transFun=transFun,isCount=isCount,whichAssay=whichAssay)
    #---------
    #Check zero variance genes before doing reducedDims:
    #---------
    varFun<-.matchToStats(x)[["var"]] #does matrixStats::rowVars if x is matrix, otherwise apply with var across rows.
    rowvars <- varFun(x) 
    if(any(rowvars==0)) {
      if(all(rowvars==0)) {
        stop("All features have zero variance.")
      }
      warning("Found features with zero variance.\nMost likely these are features with 0 across all samples.\nThey will be removed from dimensionality reduction step.")
    }
    
    ###################
    ##Do loop over reducedDims values:
    ###################
    currErrors<-vector("character")
    for(kk in seq_along(reducedDims)){
      dr<-reducedDims[[kk]]
      md<-maxDims[[kk]]
      isPct <- md < 1
      #check if already calculated
      #note, currently no way to check if have already done if md<1
      if(dr %in% reducedDimNames(object)){
        if(!isPct & md<=ncol(reducedDim(object,type=dr))) next
      }
      #-------------
      # if add other functions, add if statements here
      if(dr=="PCA") out<-try(.pcaDimRed(x,md=md,isPct=isPct,rowvars=rowvars))
      ##-------
      
      if(!inherits(out,"try-error")) reducedDim(object,reducedDims) <- out
      else{
        currErrors<-c(currErrors,paste("\t",dr,":",out,sep=""))
      }	  
    }
    if(length(currErrors)>0){
      if(length(currErrors)==length(reducedDims)) 
        stop(paste("No dimensionality reduction techniques were successful:",currErrors,sep="\n"))
      else{
        warning(paste("The following dimensionality reduction techniques hit errors:",currErrors,sep="\n"))
      }
    }
    return(object)
    
  }
)
#' @rdname reduceFunctions
#' @export
setMethod(
  f = "makeReducedDims",
  signature = "matrixOrHDF5",
  definition = function(object,...)
  {
    makeReducedDims(SummarizedExperiment(object),...)
  }
)
#' @rdname reduceFunctions
#' @export
setMethod(
  f = "makeReducedDims",
  signature = "SummarizedExperiment",
  definition = function(object,...)
  {
    makeReducedDims(as(object,"SingleCellExperiment"),...)
  }
)
#' @rdname reduceFunctions
#' @export
setMethod(
  f = "makeReducedDims",
  signature = "ClusterExperiment",
  definition = function(object,...)
  {
    if(any(c("transFun","isCount") %in% names(list(...)))) 
      stop("The internally saved transformation function of a ClusterExperiment object must be used when given as input and setting 'transFun' or 'isCount' for a 'ClusterExperiment' is not allowed.")  
    out<-makeReducedDims(as(object,"SingleCellExperiment"),transFun=transformation(object),...)
    return(.addBackSEInfo(newObj=object,oldObj=out))
  }
)

#' @rdname reduceFunctions
#' @export
listBuiltInReducedDims<-function(){c("PCA")}

#' @importFrom BiocSingular runPCA
.pcaDimRed<-function(x,md,isPct,rowvars){	
  tempX <- t(x[which(rowvars>0),])
  if(isPct) {
    prcObj<-BiocSingular::runPCA(tempX,rank=min(dim(tempX)),center=TRUE,scale=TRUE)
    prvar<-prcObj$sdev^2 #variance of each component
    prvar<-prvar/sum(prvar)
    prc<-prcObj$x
    if(NROW(prc) != NCOL(x)) stop("Internal error in coding of principal components.")
    md <- which(cumsum(prvar)>md)[1] #pick first pca coordinate with variance > value
    prc <- prc[,seq_len(md)]
  }
  else {
    prcObj <- BiocSingular::runPCA(tempX, rank=md, center=TRUE, scale=TRUE)
    prc<-prcObj$x
    if(any(md > NROW(prc)))
      stop("Internal error in coding of principal components.")
  }
  return(prc)
}

#' @title Update old ClusterExperiment object to current class definition
#' @name updateObject
#' @description This function updates ClusterExperiment objects from previous
#'   versions of package into the current definition
#' @param object a \code{ClusterExperiment} (or \code{clusterExperiment} from 
#'   older versions). Must have at a minimum a slot \code{clusterMatrix}.
#' @inheritParams BiocGenerics::updateObject 
#' @inheritParams ClusterExperiment-class
#' @details The function creates a valid \code{ClusterExperiment} object by 
#'   adding the default values of missing slots. It does so by calling the 
#'   \code{\link{ClusterExperiment}} function, which imputs default (empty) 
#'   values for missing slots.
#' @details The object is required to have minimal components to be updated. 
#'   Specifically, it must have all the required elements of a Summarized 
#'   Experiment as well as the basic slots of a ClusterExperiment object which 
#'   have not changed over time. These are: \code{clusterMatrix},
#'   \code{primaryIndex}, \code{clusterInfo}, \code{transformation}, 
#'   \code{clusterTypes}, \code{clusterLegend}, \code{orderSamples}.
#' @details If \emph{any} of the dendrogram-related slots are missing, ALL of 
#'   the dendrogram \emph{and} merge related slots will be cleared to default 
#'   values. Similarly, if \emph{any} of the merge-related slots are missing, 
#'   ALL of the merge-related slots will be cleared to the default values.
#' @details Cluster and Sample dendrograms of the class \code{dendrogram} will
#'   be updated to the \code{\link[phylobase]{phylo4d}} class now used in
#'   \code{ClusterExperiment} objects; the merge information on these nodes will
#'   be updated to have the correct format (i.e. match to the internal node id
#'   names in the new dendrogram). The previous identification of nodes that was
#'   previously created internally by plotDendrogram and the merging (labels in
#'   the form of 'Node1','Node2'), will be kept as
#'   \code{\link[phylobase]{nodeLabels}} in the new dendrogram class.
#' @details The function currently only works for object of 
#'   \code{ClusterExperiment}, not the older name \code{clusterExperiment}.
#' @return A valid \code{ClusterExperiment} object based on the current 
#'   definition of ClusterExperiment.
#' @seealso \code{\link{ClusterExperiment}}
#' @aliases updateObject,ClusterExperiment-method
#' @rdname updateObject
#' @export
#' @importFrom BiocGenerics updateObject
setMethod(
  f = "updateObject",
  signature = signature(object = "ClusterExperiment"),
  definition = function(object, checkTransformAndAssay=FALSE,...,verbose=FALSE){
    #create snames, which is the slots the object actually has
	#and will eventually be narrowed down to only those slots will pass to `ClusterExperiment`
	#list names of all current required slots
	ceSlots<-slotNames(object)
	testSnames<-sapply(ceSlots,.hasSlot,object=object)
	snames<-ceSlots[testSnames]

	#--------
	#check has at least the required slots of SummarizedExperiment class
	#--------
	if(!all(slotNames("SummarizedExperiment") %in% snames)){
		missSE<-which(!slotNames("SummarizedExperiment") %in% snames)
		stop("given object does not have the basic slots of SummarizedExperiment, cannot be updated (missing:",paste(slotNames("SummarizedExperiment")[missSE],collapse=","),"). To construct a ClusterExperiment object from its original components, use the function 'ClusterExperiment'")
	}

	#--------
	#check has minimal required slots of a clusterExperiment object of any version
	#--------
  requiredSlots<-c("clusterMatrix","primaryIndex", "clusterInfo","transformation","clusterTypes","clusterLegend","orderSamples")
	if(!all(requiredSlots %in% snames)){
		missCE<-which(!requiredSlots %in% snames)
		stop("given object does not have the basic slots of ClusterExperiment, cannot be updated (missing:",paste(requiredSlots[missCE],collapse=","),"). To construct a ClusterExperiment object from its original components, use the function 'ClusterExperiment'")
	}

	#--------
	#extract either SE or SCE object
	#--------
	# if(canCoerce(object,"SummarizedExperiment")) se<-updateObject(as(object,"SummarizedExperiment"))
    se<-callNextMethod(object)
    # if(canCoerce(object,"SingleCellExperiment")){
    #     callNextMethod(se)
    #         #se<-updateObject(as(object,"SingleCellExperiment"))
    # }
    # else{
    #     #if object was from before SCE requirement (2.0.0)
    #         if(canCoerce(object,"SummarizedExperiment")) se<-updateObject(as(object,"SummarizedExperiment"))
    #     else stop("cannot coerce object to SummarizedExperiment")
    # }

	#--------
	# Ignore slots that have to come together, with warnings
	#--------

	dendroSlots<-c("dendro_samples", "dendro_clusters",
  "dendro_index")

	mergeSlots<-c("merge_index",
"merge_dendrocluster_index",
"merge_method", "merge_demethod", "merge_cutoff",
"merge_nodeProp", "merge_nodeMerge")

	if(any(!dendroSlots %in% snames)& any(dendroSlots %in% snames)){
		warning("'object' does not contain ALL required slots saving the dendro-related information. Updated object will remove all dendro AND merge related slots")
		snames<-snames[-which(snames %in% dendroSlots)]
		snames<-snames[-which(snames %in% mergeSlots)]
	}

	if(any(!mergeSlots %in% snames) & any(mergeSlots %in% snames)){
		warning("'object' does not contain ALL required slots saving the merge-related information.  Updated object will remove all merge related slots")
		snames<-snames[-which(snames %in% mergeSlots)]
	}

	#--------
	# Get included slots of ones I create 
	#--------
	myslots<- c("transformation",
  	"primaryIndex", "clusterInfo",
  "clusterTypes", "dendro_samples", "dendro_clusters",
  "dendro_index", "coClustering",
  "clusterLegend", "orderSamples", "merge_index",
"merge_dendrocluster_index",
"merge_method", "merge_demethod", "merge_cutoff",
"merge_nodeProp", "merge_nodeMerge")
	snames<-snames[snames %in% myslots]

	## Fix class of the dendrograms:
	success<-TRUE
	successMerge<-TRUE
	if("dendro_samples" %in% snames){
		if(is(object@dendro_clusters,"dendrogram")){
			newPhyloCluster<-.makePhylobaseTree(object@dendro_clusters,type="dendro",isSamples=FALSE)
			#-----
			#Add necessary information: Position, NodeId, ClusterIdDendro, ClusterIdMerge
			#-----
			nTotal<-length(phylobase::getNode(newPhyloCluster,type="all"))
			clIds<-phylobase::tipLabels(newPhyloCluster)
			clIdsDendro<-rep(NA,length= nTotal)
			clIdsDendro[phylobase::getNode(newPhyloCluster,type="tip")]<-paste("ClusterId",clIds,sep="")
			clIdsMerge<-rep(NA,length= nTotal)
			mergeCorr<-getMergeCorrespond(object,by="original")
			clIdsMerge[phylobase::getNode(newPhyloCluster,type="tip")]<-paste("ClusterId",mergeCorr[clIds],sep="")
			maxInternal<-max(as.numeric(gsub("Node","",phylobase::nodeLabels(newPhyloCluster))))
			tipIds<-paste("NodeId",seq(from=maxInternal+1,length=nTips(newPhyloCluster)),sep="")
			phylobase::tipLabels(newPhyloCluster)<-tipIds
			labs<-phylobase::labels(newPhyloCluster,type="all")
			labs<-gsub("Node","NodeId",labs)
			pos<-factor(rep(NA,length= nTotal),levels=.positionLevels)
			pos[phylobase::getNode(newPhyloCluster,type="tip")]<-"cluster hierarchy tip"
			pos[phylobase::getNode(newPhyloCluster,type="internal")]<-"cluster hierarchy node"
			data.cl<-data.frame(NodeId=labs, ClusterIdDendro=clIdsDendro, ClusterIdMerge=clIdsMerge,              Position=pos,stringsAsFactors=FALSE)
			#get rid of node labels of tips (keep internal node labels)
			phylobase::tipLabels(newPhyloCluster)<-NA
			newPhyloCluster<-phylobase::phylo4d(x=newPhyloCluster, all.data = data.cl)
			ch<-.checkDendroClusterFormat(newPhyloCluster,checkLabels=TRUE)
			if(!is.logical(ch)) success<-success<-paste("Error in converting to new format for dendro_clusters slot:",ch)
			#make tip labels match cluster names
			object@dendro_clusters<-newPhyloCluster
			
			###Need to match up the merge information with NodeIds:
			if(!is.na(object@merge_nodeMerge) && !"NodeId" %in% colnames(object@merge_nodeMerge) && "Node" %in% colnames(object@merge_nodeMerge)){
				nodes<-object@merge_nodeMerge[,"Node"]
				m<-match(nodes,phylobase::nodeLabels(object@dendro_clusters))
				if(any(is.na(m))) successMerge<-"Error in converting merge_nodeMerge information -- doesn't match node labels of dendro_clusters"
				nodeId<-phylobase::tdata(object@dendro_clusters,type="internal")[m,"NodeId"]
				object@merge_nodeMerge[,"Node"]<-nodeId
				colnames(object@merge_nodeMerge)[grep("Node",object@merge_nodeMerge)]<-"NodeId"
			}
			if(!is.na(object@merge_nodeProp)&& !"NodeId" %in% colnames(object@merge_nodeProp) && "Node" %in% colnames(object@merge_nodeProp)){
				nodes<-object@merge_nodeProp[,"Node"]
				m<-match(nodes,phylobase::nodeLabels(object@dendro_clusters))
				if(any(is.na(m))) successMerge<-"Error in converting merge_nodeProp information -- doesn't match node labels of dendro_clusters"
				nodeId<-phylobase::tdata(object@dendro_clusters,type="internal")[m,"NodeId"]
				object@merge_nodeProp[,"Node"]<-nodeId
				colnames(object@merge_nodeProp)[grep("Node",object@merge_nodeProp)]<-"NodeId"
				
			}
		}
		if(is(object@dendro_samples,"dendrogram")){
			#requires old slot name @dendro_outbranch too.
			dataNode<-phylobase::tdata(newPhyloCluster,type="internal")
			data.cl<-phylobase::tdata(newPhyloCluster,type="all")
			newPhyloSample<-.makePhylobaseTree(object@dendro_samples,type="dendro",isSamples=TRUE,outbranch=object@dendro_outbranch)
			
			#-------
			#Add necessary information: Position, NodeId, SampleIndex
			#-------
			origLabs<-phylobase::labels(newPhyloSample,type="all")
			tipNodes<-phylobase::getNode(newPhyloSample,type="tip")
			nTotal<-length(origLabs)
			## SampleIndex
			#Note: Previous naming convention of samples was to give names if not null, and if null, give values "1","2", etc. corresponding to index.
			sampIndex<-rep(NA,length=nTotal)
			if(!is.null(colnames(object))){
				mSample<-match(origLabs,colnames(object))
				if(any(is.na(mSample[tipNodes]))) success<-"error in converting tip labels in the dendro_samples slot: the tip labels do not all match colnames of object"
				sampIndex<-mSample
			}
			else{
				tipLabs<-as.numeric(phylobase::tipLabels(newPhyloSample))
				if(any(is.na(tipLabs))) success<-"error in converting tip labels in the dendro_samples slot: the object has no colnames and tip labels are not numeric index"
				sampIndex[tipNodes]<-tipLabs
			}
			
			## NodeId
			mNode<-match(origLabs,gsub("Id","",dataNode$NodeId))
			newNodeLabs<-rep(NA,length=length(mNode))
			newNodeLabs[!is.na(mNode)]<-as.character(dataNode$NodeId)[mNode[!is.na(mNode)]]
			#Now need to match nodes that correspond to cluster id... they don't have any reasonable naming scheme...
			clusterNodes<-phylobase::getNode(newPhyloSample,type="all")
			clusterNodes<-clusterNodes[which(!is.na(newNodeLabs))]
			getClusterDescendants<-function(node){
				dd<-phylobase::descendants(phy=newPhyloSample,node=node,type="tip")
				if(any(is.na(sampIndex[dd]))) success<<-"Not all descendants have sample ids; unable to identify clusters to cluster ids in sample dendrogram"
				else{
					cl<-clusterMatrix(object)[sampIndex[dd],object@dendro_index]
					tab<-table(clusterMatrix(object)[,object@dendro_index])
					tabNode<-table(cl)
					tab<-tab[names(tabNode)]
					clusterVal<-as.numeric(names(tabNode))
					allCluster<-tabNode==tab	
					return(list(clusters=clusterVal,completeCluster=allCluster))
				}
			}
			#of cluster hierarchy nodes, figure out which of them have child that == cluster
			#returns a character matrix where column "Node" is numberic node index and "Cluster" is numeric
			clusterLink<-do.call("rbind",lapply(clusterNodes,function(x){
				cc<-phylobase::descendants(phy=newPhyloSample,node=x,type="children")
				if(!all(cc %in% clusterNodes)){
					cc<-cc[!cc%in%clusterNodes]
					out<-sapply(cc,function(node){
						ret<-NA
						clVal<-getClusterDescendants(node)
						if(length(clVal$clusters)==1){
							if(clVal$completeCluster) ret<-clVal$clusters
						}
						return(ret)
					})
					names(out)<-cc
					out<-out[!is.na(out)]
					return(cbind("Node"=as.numeric(names(out)),"Cluster"=as.numeric( out)))
				}
				else return(NULL)
				})
			)
			#check got all of them
			if(any(is.na(match(na.omit(data.cl$ClusterIdDendro),paste("ClusterId",clusterLink[,"Cluster"],sep=""))))) success<-"error in updating sample dendrogram -- not all clusters found in cluster dendro find match to node in samples dendro"
			nodeIdMatch<-match(paste("ClusterId",clusterLink[,"Cluster"],sep=""),data.cl$ClusterIdDendro)
			newNodeLabs[clusterLink[,"Node"]]<-data.cl$NodeId[nodeIdMatch]
			## Postion
			# .positionLevels<-c("cluster hierarchy node","cluster hierarchy tip","tip hierarchy","assigned tip","outbranch hierarchy node","unassigned tip","outbranch root")
			#
			pos<-factor(rep(NA,length= nTotal),levels=.positionLevels)
			pos[grep("Missing",origLabs)]<-"outbranch hierarchy node"
			pos[grep("Root",origLabs)]<-"outbranch root"
			tipUnassigned<-unlist(phylobase::descendants(phy=newPhyloSample,node=grep("Missing",origLabs),type="tip"))
			pos[tipUnassigned]<-"unassigned tip"
			pos[!is.na(match(newNodeLabs,data.cl$NodeId))]<-"cluster hierarchy tip"
			pos[!is.na(mNode)]<-"cluster hierarchy node"
			tipAssigned<-unlist(phylobase::descendants(phy=newPhyloSample,node=which(pos=="cluster hierarchy tip"),type="tip"))
			pos[tipAssigned]<-"assigned tip"
			pos[is.na(pos)]<-"tip hierarchy"
			sample.data<-data.frame(SampleIndex=sampIndex,NodeId=newNodeLabs,Position=pos,stringsAsFactors=FALSE)
			#get rid of labels
			phylobase::labels(newPhyloSample)<-NA
			newPhyloSample<-phylobase::phylo4d(x=newPhyloSample, all.data = sample.data)
			ch<-.checkDendroSamplesFormat(newPhyloSample,checkLabels=TRUE)
			if(!is.logical(ch)) success<-paste("Error in converting to new format for dendro_samples slot:",ch)
			object@dendro_samples<-newPhyloSample
			
		}
		if(!is.logical(success)){
			warning("Could not successfully convert dendrogram to class 'phylo4d'. Updated object will remove all dendro AND merge related slots. Error:",success)
			snames<-snames[-which(snames %in% dendroSlots)]
			snames<-snames[-which(snames %in% mergeSlots)]
		}
		else if(!is.logical(successMerge)){
			warning("Could not successfully convert merge information to match the nodes in updated dendrogram object. Updated object will remove ALL merge related slots. Error:",successMerge)
			snames<-snames[-which(snames %in% mergeSlots)]
		}
	}

	##Fix class of coClustering -- have turned this off for now after changed coClustering.
	if(!is.null(object@coClustering) &&
        inherits(object@coClustering,"matrix")){
		coClustering(object)<-object@coClustering
	}
    passedSlots<-attributes(object)[snames]
    ### NULL values will be turned into `\001NULL\001` (class 'name', but appears to work to compare to character value)
    whNotNULL<-which(!sapply(passedSlots,function(x){inherits(x,"name") && 
            x=="\001NULL\001"}))
    object<-try(do.call("ClusterExperiment",c(list(
        object=se,
        clusters=object@clusterMatrix,
        checkTransformAndAssay=checkTransformAndAssay), 
        passedSlots[whNotNULL])
        ),silent=TRUE)
	if(!inherits(object,"try-error")){
		return(object)
	} 
		else stop("Attempt to convert did not result in valid object. Here is the error from 'ClusterExperiment':\n",object)
}
)


#---
#remove the outbranch from the dendrogram and from cl
#(note this is using phylo4 obj)
#---
.removeOutbranch<-function(phylo4Obj){
  rootNode<-phylobase::rootNode(phylo4Obj)
  rootChild<-phylobase::descendants(phylo4Obj,node=rootNode,type="children")
  tips<-phylobase::getNode(phylo4Obj,type="tip")
  whMissingNode<-grep("MissingNode",names(rootChild))
  if(length(whMissingNode)==0){
    #check not a single -1 sample from root:
    if(any(rootChild %in% tips)){
      #which ever rootChild is in tips must be single missing sample 
      #because can't make dendrogram with only 1 cluster so couldn't run plot or mergeClusters. 
	#Note only true because outbranch=TRUE
      clusterNode<-rootChild[!rootChild %in% tips]
      #stop("Internal coding error: need to fix .plotDendro to deal with when single missing sample")
    }
    else stop("Internal coding error: no outbranch nodes")	
  } 
  else clusterNode<-rootChild[-whMissingNode]
  if(length(clusterNode)!=1) stop("Internal coding error: removing missing node does not leave exactly 1 descendent of root")
  clusterTips<-phylobase::descendants(phylo4Obj,node=clusterNode,type="tip")
  if(length(clusterTips)==0) stop("Internal coding error: no none missing samples in tree")
  namesClusterTips<-names(clusterTips)
  #
  
  if(is.matrix(cl)) cl<-cl[namesClusterTips,] else cl<-cl[namesClusterTips]
  phylo4Obj<-phylobase::subset(phylo4Obj, node.subtree=clusterNode)
  return(phylo4Obj)
}



####
#Convert dendrogram class slots to class used by phylobase (phylo4) so can navigate easily. Does so by first converting to class of ape (phylo)
#' @importFrom ape as.phylo
#' @importFrom stats as.hclust
#' @importClassesFrom phylobase phylo4 
####
#Convert to object used by phylobase so can navigate easily 
.makePhylobaseTree<-function(x,type,isSamples=FALSE,outbranch=FALSE){
  type<-match.arg(type,c("hclust","dendro"))
  if(type=="dendro"){
    tempPhylo<-try(stats::as.hclust(x),FALSE)
    if(inherits(tempPhylo, "try-error")) stop("the dendrogram object cannot be converted to a hclust class with 'as.hclust.dendrogram'. Check that you gave simple hierarchy of clusters, and not one with fake data per sample")
  }
  if(type=="hclust"){
    #first into phylo from ape package
    tempPhylo<-try(ape::as.phylo(x),FALSE)
    if(inherits(tempPhylo, "try-error")) stop("the hclust object cannot be converted to a phylo class with the methods of the 'ape' package.")
  }

  phylo4Obj<-try(as(tempPhylo,"phylo4"),FALSE) 
  if(inherits(phylo4Obj, "try-error")) stop("the internally created phylo object cannot be converted to a phylo4 class. Check that you gave simple hierarchy of clusters, and not one with fake data per sample")
  
  if(isSamples){
    #NOTE: clusterNodes are found by those with non-zero edge-length between them and their decendents
    nonZeroEdges<-phylobase::edgeLength(phylo4Obj)[which(phylobase::edgeLength(phylo4Obj)>0)] #doesn't include root
    trueInternal<-sort(unique(as.numeric(sapply(strsplit(names(nonZeroEdges),"-"),.subset2,1)))) #this also picks up the outbranch between -1,-2
    #old way of doing it:
    #clusterNodes<-sort(unique(unlist(phylobase::ancestors(phylo4Obj,node=phylobase::getNode(phylo4Obj,type="tip"),type="parent"),recursive=FALSE,use.names=FALSE)))
    if(outbranch){#remove root from labeling if -1 outbranch
      #######
      #remove root
      #######
      rootNode<-phylobase::rootNode(phylo4Obj)
      trueInternal<-trueInternal[!trueInternal%in%rootNode]
      
      #######
      #find the -1/-2 internal node (if it exists)
      #determine it as the one without 0-length tip edges.
      #assumes all tips in the non-outbranch have 0-length (so max value is zero)
      #######
      rootChild<-phylobase::descendants(phylo4Obj,node=rootNode,type="children")
      #find tip descendants of each of these:
      rootChildDesc<-lapply(rootChild,phylobase::descendants,phy=phylo4Obj,type="tip")
      rootChildLeng<-lapply(rootChildDesc,phylobase::edgeLength,x=phylo4Obj)
      
      #Problem here!!! if there is single sample in a cluster, then could be a tip with length not equal to zero. Need to ignore these....how? If take the min, then a single zero length in outbranch will result in both having zeros...
      #maybe should change function so have to provide a single name of a sample that is in outbranch so as to identify it that way. 
      #for now, lets hope that never happens! i.e. that BOTH a single sample in a cluster and that outbranch has a zero length
      rootChildNum<-sapply(rootChildLeng,min) #minimum length 
      
      #indicator of which child node is the 
      whKeep<-sapply(rootChildNum,function(x){isTRUE(all.equal(x,0))}) #just incase not *exactly* 0
      if(sum(whKeep)!=1){
        #if both sides have a zero, then use max instead. 
        rootChildNum<-sapply(rootChildLeng,max) #maximum length 
        whKeep<-sapply(rootChildNum,function(x){isTRUE(all.equal(x,0))}) 
      }
      if(sum(whKeep)!=1) stop("Internal coding error in finding which is the outbranch in the dendro_samples slot. Please report to git repository!")
      outbranchNode<-rootChild[!whKeep]
      
      if(outbranchNode %in% trueInternal){
        outbranchIsInternal<-TRUE
        outbranchNodeDesc<-phylobase::descendants(phylo4Obj,node=outbranchNode,type="ALL") #includes itself
        trueInternal<-trueInternal[!trueInternal%in%outbranchNodeDesc]
        outbranchNodeDesc<-outbranchNodeDesc[outbranchNodeDesc %in% phylobase::getNode(phylo4Obj,type="internal")]
      }
      else outbranchIsInternal<-FALSE
      
    }
    #trueInternal<-allInternal[!allInternal%in%clusterNodes]
    
    phylobase::nodeLabels(phylo4Obj)[as.character(trueInternal)]<-paste("Node",seq_along(trueInternal),sep="")
    #add new label for root 
    if(outbranch){
      phylobase::nodeLabels(phylo4Obj)[as.character(rootNode)]<-"Root"
      if(outbranchIsInternal) phylobase::nodeLabels(phylo4Obj)[as.character(outbranchNodeDesc)]<-paste("MissingNode",seq_along(outbranchNodeDesc),sep="")
    }
  }
  else phylobase::nodeLabels(phylo4Obj)<-paste("Node",seq_len(phylobase::nNodes(phylo4Obj)),sep="")
  
  return(phylo4Obj)
}

#' @rdname getClusterManyParams
#' @title Get parameter values of clusterMany clusters
#'   
#' @description Takes an input a ClusterExperiment object and returns the
#'   parameter values used in creating the clusters that were created by
#'   'clusterMany'
#'   
#' @aliases getClusterManyParams
#'   
#' @param x a ClusterExperiment object that contains clusterings from running
#'   \code{\link{clusterMany}}.
#' @param searchAll logical, indicating whether all clusterings with 
#'   \code{clusterMany} label should be allowed (i.e. including those from
#'    previous ones with labels like \code{clusterMany.1}), or only limited 
#'    to those in most recent workflow (default).
#' @param simplify logical. Whether to simplify the output so as to remove 
#'   features that do not change across the clusterings.
#' @details The method simply parses the \code{clusterLabels} of the indicated
#'   clusterings, relying on the specific format used by \code{clusterMany} to
#'   create labels. The function will only allow the parsing to be performed on
#'   those clusterings with a 'clusterMany' clusterType. If the user has
#'   manipulated the clusterLabels manually or manually identified the
#'   clusterType of a clustering as 'clusterMany', this function may create
#'   unexpected results or errors. Similarly, it cannot be used on 'clusterMany'
#'   results from an old iteration (e.g. that have type 'clusterMany.1')
#' @details Specifically, it splits the label of each clustering by the
#'   character ",", as indicating the different parameters; this should return a
#'   value of form "ABC=123". The function then pulls out the numeric value
#'   ('123') and associates that value as the value of the parameter ('ABC')
#' @return Returns a data.frame where the column names are the parameter names,
#'   and the entries are the values of the parameter for the indicated
#'   clustering. The column 'clusteringIndex' identifies the index of the
#'   clustering in the full set of clusterings of the given ClusterExperiment
#'   object.
#' @inheritParams ClusterExperiment-methods
#' @examples
#' \dontrun{
#' data(simData)
#'
#' cl <- clusterMany(simData, nReducedDims=c(5, 10), reduceMethod="PCA",
#'      clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE), 
#'      makeMissingDiss=TRUE, removeSil=c(TRUE,FALSE))
#' getClusterManyParams(cl)
#' }
#' @export
setMethod(
  f = "getClusterManyParams",
  signature = signature(x = "ClusterExperiment"),
  function(x,whichClusters="clusterMany",searchAll=FALSE,simplify=TRUE) {
    
    allClTypes<-clusterTypes(x)
	if(searchAll) whCM<-grep("clusterMany",allClTypes)
	else whCM<-which(allClTypes=="clusterMany")
    if(length(whCM)==0) stop("x does not have any clusterings that have clusterType 'clusterMany' ")	
	wh <- getClusterIndex(x, whichClusters=whichClusters,noMatch="silentlyRemove")
    if(length(wh)==0){
      warning("argument whichClusters did not return any clusters; using all clusterMany clusters")
      wh<-whCM
    }
    if(!any(wh %in% whCM)){
      warning("argument whichClusters did not return any clusters of type 'clusterMany'; using all clusterMany clusters")
      wh<-whCM
    }
    else{
      if(any(!wh %in%  whCM)){
        warning("some clusters indicated in 'whichClusters' do not have type 'clusterMany' and will be ignored.") 
        wh<-wh[wh %in% whCM]
      }
      
    }
    infoList<-clusteringInfo(x)[wh]
    params<-do.call("rbind",lapply(infoList,.subset2,"choicesParam"))
    if(simplify){
      # notAllNA<-which(apply(params,2,function(x){!all(is.na(x))}))
      # params<-params[,notAllNA]
      
      notAllSame<-which(apply(params,2,function(z){length(unique(z))>1}))
      params<-params[,notAllSame,drop=FALSE]
    }
	row.names(params)<-clusterLabels(x)[wh]
    params<-data.frame("clusteringIndex"=wh,params)
    return(params)
  }
)

#' @name plotHeatmap
#' @title Heatmap for showing clustering results and more
#' @description Make heatmap with color scale from one matrix and hiearchical
#'   clustering of samples/features from another. Also built in functionality
#'   for showing the clusterings with the heatmap. Builds on
#'   \code{\link[NMF]{aheatmap}} function of \code{NMF} package.
#' @docType methods
#' @param colData If input to \code{data} is either a
#'   \code{\link{ClusterExperiment}},or \code{SummarizedExperiment} object or
#'   \code{SingleCellExperiment}, then \code{colData} must index the
#'   colData stored as a \code{DataFrame} in \code{colData} slot of the
#'   object. Whether that data is continuous or not will be determined by the
#'   properties of \code{colData} (no user input is needed). If input to
#'   \code{data} is matrix, \code{colData} is a matrix of additional data on
#'   the samples to show above heatmap. In this case, unless indicated by
#'   \code{whColDataCont}, \code{colData} will be converted into factors,
#'   even if numeric. ``-1'' indicates the sample was not assigned to a cluster
#'   and gets color `unassignedColor' and ``-2`` gets the color 'missingColor'.
#' @param data data to use to determine the heatmap. Can be a matrix,
#'   \code{\link{ClusterExperiment}},
#'   \code{\link[SingleCellExperiment]{SingleCellExperiment}} or
#'   \code{\link[SummarizedExperiment]{SummarizedExperiment}} object. The
#'   interpretation of parameters depends on the type of the input to
#'   \code{data}.
#' @param whColDataCont Which of the \code{colData} columns are continuous
#'   and should not be converted to counts. \code{NULL} indicates no additional
#'   \code{colData}. Only used if \code{data} input is matrix.
#' @param visualizeData either a character string, indicating what form of the
#'   data should be used for visualizing the data (i.e. for making the
#'   color-scale), or a data.frame/matrix with same number of samples as
#'   \code{assay(data)}. If a new data.frame/matrix, any character arguments to
#'   clusterFeaturesData will be ignored.
#' @param clusterSamplesData If \code{data} is a matrix,
#'   \code{clusterSamplesData} is either a matrix that will be used by
#'   \code{hclust} to define the hiearchical clustering of samples (e.g.
#'   normalized data) or a pre-existing dendrogram (of class
#'  \code{\link[stats]{dendrogram}}) that clusters the samples. If
#'   \code{data} is a \code{ClusterExperiment} object, \code{clusterSamplesData}
#'   should be either character or integers or logical which indicates how (and
#'   whether) the samples should be clustered (or gives indices of the order for
#'   the samples). See details.
#' @inheritParams getClusterIndex
#' @param clusterFeaturesData  If \code{data} is a matrix, either a matrix that
#'   will be used in \code{hclust} to define the hiearchical clustering of
#'   features (e.g. normalized data) or a pre-existing dendrogram that clusters
#'   the features. If \code{data} is a \code{ClusterExperiment} object, the
#'   input should be either character or integers indicating which features
#'   should be used (see details).
#' @param clusterSamples Logical as to whether to do hierarchical clustering of
#'   cells (if FALSE, any input to clusterSamplesData is ignored).
#' @param clusterFeatures Logical as to whether to do hiearchical clustering of
#'   features (if FALSE, any input to clusterFeaturesData is ignored).
#' @param showSampleNames Logical as to whether show sample names.
#' @param showFeatureNames Logical as to whether show feature names.
#' @param colorScale palette of colors for the color scale of the heatmap.
#' @param clusterLegend Assignment of colors to the clusters. If \code{NULL},
#'   \code{colData} columns will be assigned colors internally. See details
#'   for more.
#' @param alignColData Logical as to whether should align the colors of the
#'   \code{colData} (only if \code{clusterLegend} not given and
#'   \code{colData} is not \code{NULL}).
#' @param breaks Either a vector of breaks (should be equal to length 52), or a
#'   number between 0 and 1, indicating that the breaks should be equally spaced
#'   (based on the range in the data) upto the `breaks' quantile, see
#'   \code{\link{setBreaks}}
#' @param unassignedColor color assigned to cluster values of '-1'
#'   ("unassigned").
#' @param missingColor color assigned to cluster values of '-2' ("missing").
#' @param ... for signature \code{matrix}, arguments passed to \code{aheatmap}.
#'   For the other signatures, passed to the method for signature \code{matrix}.
#'   Not all arguments can be passed to \code{aheatmap} effectively, see
#'   details.
#' @param nFeatures integer indicating how many features should be used (if
#'   \code{clusterFeaturesData} is 'var' or 'PCA').
#' @param isSymmetric logical. if TRUE indicates that the input matrix is
#'   symmetric. Useful when plotting a co-clustering matrix or other sample by
#'   sample matrices (e.g., correlation).
#' @param overRideClusterLimit logical. Whether to override the internal limit
#'   that only allows 10 clusterings/annotations. If overridden, may result in
#'   incomprehensible errors from \code{aheatmap}. Only override this if you
#'   have a very large plotting device and want to see if \code{aheatmap} can
#'   render it.
#' @param plot logical indicating whether to plot the heatmap. Mainly useful for
#'   package mantaince to avoid calls to aheatmap on unit tests that take a long
#'   time.
#' @param symmetricBreaks logical as to whether the breaks created for the color
#'   scale should be symmetrical around 0
#' @param capBreaksLegend logical as to whether the legend for the breaks should
#'   be capped. Only relevant if \code{breaks} is a value < 1, in which case if
#'   \code{capBreaksLegend=TRUE}, only the values between the quantiles
#'   requested will show in the color scale legend.
#' @param whichAssay numeric or character specifying which assay to use. See
#'   \code{\link[SummarizedExperiment]{assay}} for details.
#' @param labelTracks logical, whether to put labels next to the color tracks 
#' corresponding to the colData. 
#' @inheritParams clusterSingle
#' @details The plotHeatmap function calls \code{\link[NMF]{aheatmap}} to draw
#'   the heatmap. The main points of \code{plotHeatmap} are to 1) allow for
#'   different matrix inputs, separating out the color scale visualization and
#'   the clustering of the samples/features. 2) to visualize the clusters and
#'   meta data with the heatmap. The intended use case is to allow the user to
#'   visualize the original count scale of the data (on the log-scale), but
#'   create the hierarchical clustering on another, more appropriate dataset for
#'   clustering, such as normalized data. Similarly, some of the palettes in the
#'   package were developed assuming that the visualization might be on
#'   unscaled/uncentered data, rather than the residual from the mean of the
#'   gene, and thus palettes need to take on a greater range of relevant values
#'   so as to show meaningful comparisons with genes on very different scales.
#' @details If \code{data} is a \code{ClusterExperiment} object,
#'   \code{visualizeData} indicates what kind of transformation should be done
#'   to \code{assay(data)} for calculating the color scale. The features will be
#'   clustered based on these data as well. A different data.frame or matrix can
#'   be given for the visualization. For example, if the
#'   \code{ClusterExperiment} object contains normalized data, but the user
#'   wishes that the color scale be based on the log-counts for easier
#'   interpretation, \code{visualizeData} could be set to be the
#'   \code{log2(counts + 1)}.
#' @details If \code{data} is a \code{ClusterExperiment} object,
#'   \code{clusterSamplesData} can be used to indicate the type of clustering
#'   for the samples. If equal to `dendrogramValue` the dendrogram stored in
#'   \code{data} will be used; if dendrogram is missing, a new one will be
#'   created based on the \code{primaryCluster} of data using
#'   \code{\link{makeDendrogram}}, assuming no errors are created (if errors are
#'   created, then \code{clusterSamplesData} will be set to "primaryCluster").
#'   If \code{clusterSamplesData} is equal to "hclust", then standard
#'   hierachical clustering of the transformed data will be used. If
#'   \code{clusterSamplesData} is equal to 'orderSamplesValue' no clustering of
#'   the samples will be done, and instead the samples will be ordered as in the
#'   slot \code{orderSamples} of \code{data}. If \code{clusterSamplesData} is
#'   equal to 'primaryCluster', again no clustering will be done, and instead
#'   the samples will be ordered based on grouping the samples to match the
#'   primaryCluster of \code{data}; however, if the primaryCluster of
#'   \code{data} is only one cluster or consists soley of -1/-2 values,
#'   \code{clusterSamplesData} will be set to "hclust". If
#'   \code{clusterSamplesData}  is not a character value,
#'   \code{clusterSamplesData} can be a integer valued vector giving the order
#'   of the samples.
#' @details If \code{data} is a matrix, then \code{colData} is a data.frame
#'   of annotation data to be plotted above the heatmap and
#'   \code{whColDataCont} gives the index of the column(s) of this dataset
#'   that should be consider continuous. Otherwise the annotation data for
#'   \code{colData} will be forced into a factor (which will be nonsensical
#'   for continous data). If \code{data} is a \code{ClusterExperiment} object,
#'   \code{colData} should refer to a index or column name of the
#'   \code{colData} slot of \code{data}. In this case \code{colData} will be
#'   added to any choices of clusterings chosen by the \code{whichClusters}
#'   argument (if any). If both clusterings and sample data are chosen, the
#'   clusterings will be shown closest to data (i.e. on bottom).
#' @details If \code{data} is a \code{ClusterExperiment} object,
#'   \code{clusterFeaturesData} is not a dataset, but instead indicates which
#'   features should be shown in the heatmap. In this case
#'   \code{clusterFeatures} can be one of the following: \itemize{ \item{"all"}{
#'   All rows/genes will be shown} \item{character giving dimensionality
#'   reduction}{Should match one of values saved in \code{reducedDims} slot or a
#'   builtin function in \code{listBuiltInReducedDims()}. \code{nFeatures} then
#'   gives the number of dimensions to show. The heatmap will then be of the
#'   dimension reduction vectors} \item{character giving filtering}{ Should
#'   match one of values saved in \code{filterStats} slot or a builtin function
#'   in \code{listBuiltInFilterStats()}. \code{nFeatures} gives the number of
#'   genes to keep after filtering.} \item{character giving gene/row names}{ }
#'   \item{vector of integers giving row indices}{ } \item{a list of indices or
#'   rownames}{This is used to indicate that the features should be grouped
#'   according to the elements of the list, with blank (white) space between
#'   them (see \code{\link{makeBlankData}} for more details). In this case, no
#'   clustering is done of the features.} }
#' @details If \code{breaks} is a numeric value between 0 and 1, then
#'   \code{breaks} is assumed to indicate the upper quantile (on the log scale)
#'   at which the heatmap color scale should stop. For example, if
#'   \code{breaks=0.9}, then the breaks will evenly spaced up until the 0.9
#'   upper quantile of \code{data}, and then all values after the 0.9 quantile
#'   will be absorbed by the upper-most color bin. This can help to reduce the
#'   visual impact of a few highly expressed genes (features).
#' @details Note that plotHeatmap calls \code{\link[NMF]{aheatmap}} under the
#'   hood. This allows you to plot multiple heatmaps via
#'   \code{par(mfrow=c(2,2))}, etc. However, the dendrograms do not resize if
#'   you change the size of your plot window in an interactive session of R
#'   (this might be a problem for RStudio if you want to pop it out into a large
#'   window...). Also, plotting to a pdf adds a blank page; see help pages of
#'   \code{\link[NMF]{aheatmap}} for how to turn this off.
#' @details \code{clusterLegend} takes the place of argument \code{annColors}
#'   from \code{aheatmap} for giving colors to the annotation on the heatmap.
#'   \code{clusterLegend} should be list of length equal to
#'   \code{ncol(colData)} with names equal to the colnames of
#'   \code{colData}. Each element of the list should be a either the format
#'   requested by \code{\link[NMF]{aheatmap}} (a vector of colors with names
#'   corresponding to the levels of the column of \code{colData}), or should
#'   be format of the \code{clusterLegend} slot in a \code{ClusterExperiment}
#'   object. Color assignments to the rows/genes should also be passed via
#'   \code{clusterLegend} (assuming \code{annRow} is an argument passed to
#'   \code{...}). If \code{clusterFeaturesData} is a \emph{named} list
#'   describing groupings of genes then the colors for those groups can be given
#'   in \code{clusterLegend} under the name "Gene Group".
#' @details If you have a factor with many levels, it is important to note that
#'   \code{\link[NMF]{aheatmap}} does not recycle colors across factors in the
#'   \code{colData}, and in fact runs out of colors and the remaining levels
#'   get the color white. Thus if you have many factors or many levels in those
#'   factors, you should set their colors via \code{clusterLegend}.
#' @details Many arguments can be passed on to \code{aheatmap}, however, some are set
#'   internally by \code{plotHeatmap.} In particular, setting the values of
#'   \code{Rowv} or \code{Colv} will cause errors. \code{color} in
#'   \code{aheatmap} is replaced by \code{colorScale} in \code{plotHeatmap.} The
#'   \code{annCol} to give annotation to the samples is replaced by the
#'   \code{colData}; moreover, the \code{annColors} option in \code{aheatmap}
#'   will also be set internally to give more vibrant colors than the default in
#'   \code{aheatmap} (for \code{ClusterExperiment} objects, these values can
#'   also be set in the \code{clusterLegend} slot ). Other options should be
#'   passed on to \code{aheatmap}, though they have not been all tested. Useful options
#'  include \code{treeheight=0} to suppress plotting of the dendrograms, 
#'  \code{annLegend=FALSE} to suppress the legend of factors shown beside columns/rows, 
#'  and \code{cexRow=0} or \code{cexCol=0} to suppress plotting of row/column labels.
#'
#' @return Returns (invisibly) a list with elements
#' \itemize{
#' \item{\code{aheatmapOut}}{ The output from the final call of
#' \code{\link[NMF]{aheatmap}}.}
#' \item{\code{colData}}{ the annotation data.frame given to the argument
#' \code{annCol} in \code{aheatmap}.}
#' \item{\code{clusterLegend}}{ the annotation colors given to the argument
#' \code{annColors} \code{aheatmap}.}
#' \item{\code{breaks}}{ The breaks used for \code{aheatmap}, after adjusting
#' for quantile.}
#' }
#' @author Elizabeth Purdom
#' @seealso \code{\link[NMF]{aheatmap}}, \code{\link{makeBlankData}}, \code{\link{showHeatmapPalettes}}, \code{\link{makeDendrogram}}, \code{\link[stats]{dendrogram}}
#' @export
#' @examples
#' \dontrun{
#' data(simData)
#'
#' cl <- rep(1:3,each=100)
#' cl2 <- cl
#' changeAssign <- sample(1:length(cl), 80)
#' cl2[changeAssign] <- sample(cl[changeAssign])
#' ce <- ClusterExperiment(simCount, cl2, transformation=function(x){log2(x+1)})
#'
#' #simple, minimal, example. Show counts, but cluster on underlying means
#' plotHeatmap(ce)
#'
#' #assign cluster colors
#' colors <- bigPalette[20:23]
#' names(colors) <- 1:3
#' plotHeatmap(data=simCount, clusterSamplesData=simData,
#' colData=data.frame(cl), clusterLegend=list(colors))
#'
#' #show two different clusters
#' anno <- data.frame(cluster1=cl, cluster2=cl2)
#' out <- plotHeatmap(simData, colData=anno)
#'
#' #return the values to see format for giving colors to the annotations
#' out$clusterLegend
#'
#' #assign colors to the clusters based on plotClusters algorithm
#' plotHeatmap(simData, colData=anno, alignColData=TRUE)
#'
#' #assign colors manually
#' annoColors <- list(cluster1=c("black", "red", "green"),
#' cluster2=c("blue","purple","yellow"))
#'
#' plotHeatmap(simData, colData=anno, clusterLegend=annoColors)
#'
#' #give a continuous valued -- need to indicate columns
#' anno2 <- cbind(anno, Cont=c(rnorm(100, 0), rnorm(100, 2), rnorm(100, 3)))
#' plotHeatmap(simData, colData=anno2, whColDataCont=3)
#'
#' #compare changing breaks quantile on visual effect
#' par(mfrow=c(2,2))
#' plotHeatmap(simData, colorScale=seqPal1, breaks=1, main="Full length")
#' plotHeatmap(simData,colorScale=seqPal1, breaks=.99, main="0.99 Quantile Upper
#' Limit")
#' plotHeatmap(simData,colorScale=seqPal1, breaks=.95, main="0.95 Quantile Upper
#' Limit")
#' plotHeatmap(simData, colorScale=seqPal1, breaks=.90, main="0.90 Quantile
#' Upper Limit")
#' }
#'
#' @rdname plotHeatmap
#' @aliases plotHeatmap,SingleCellExperiment-method
#' @importFrom stats hclust dist
#' @importFrom NMF aheatmap
setMethod(
  f = "plotHeatmap",
  signature = signature(data = "SingleCellExperiment"),
  definition = function(data, isCount=FALSE,transFun=NULL,...
  ){
    #get transformation function
    transformation<-.makeTransFun(transFun=transFun,isCount=isCount)
    fakeCL<-sample(c(1,2),size=NCOL(data),replace=TRUE)
    fakeCE<-ClusterExperiment(data, fakeCL,transformation=transformation,
                              checkTransformAndAssay=FALSE)
    if("whichClusters" %in% names(list(...))) stop("cannot provide argument 'whichClusters' for input data not of class 'ClusterExperiment'")
    plotHeatmap(fakeCE,whichClusters="none",...)
  })

#' @rdname plotHeatmap
#' @export
setMethod(
  f = "plotHeatmap",
  signature = signature(data = "SummarizedExperiment"),
  definition = function(data, isCount=FALSE,transFun=NULL,...
  ){
    plotHeatmap(as(data,"SingleCellExperiment"),...)
  })


#' @rdname plotHeatmap
#' @export
setMethod(
  f = "plotHeatmap",
  signature = signature(data = "table"),
  definition = function(data,...
  ){
    plotHeatmap(unclass(data),...)
  })

#' @rdname plotHeatmap
#' @param nBlankLines Only applicable if input is \code{ClusterExperiment} object. Indicates the number of lines to put between groups of features if \code{clusterFeaturesData} gives groups of genes (see details and \code{\link{makeBlankData}}).
setMethod(
  f = "plotHeatmap",
  signature = signature(data = "ClusterExperiment"),
  definition = function(data,
                        clusterSamplesData=c("dendrogramValue", "hclust", "orderSamplesValue", "primaryCluster"),
                        clusterFeaturesData="var", nFeatures=NA,
                        visualizeData=c("transformed","centeredAndScaled","original"),
                        whichClusters= c("primary","workflow","all","none"),
                        colData=NULL,clusterFeatures=TRUE, nBlankLines=2,
                        colorScale, whichAssay=1,
                        ...
  ){

    .convertTry<-function(x,tryResult){if(!inherits(tryResult,"try-error")) return(tryResult) else return(x)}
    userList<-list(...)    
		checkIgnore<-.depricateArgument(passedArgs=userList,"colData","sampleData") #06/2018 added in BioC 3.8
		if(!is.null(checkIgnore)){
			userList<-checkIgnore$passedArgs
			colData<-checkIgnore$val
		}
    #########
    ##Determine visualization data and default colorScale based on that
    #########
    externalData<-FALSE
    visualizeData <- .convertTry(visualizeData,
                                 try(match.arg(visualizeData), silent=TRUE))
    if(is.character(visualizeData)){
      if(!visualizeData %in% c("transformed","centeredAndScaled","original")) stop("visualizeData value, '",visualizeData,"',is invalid option")
    }
    else{
      if(!is.data.frame(visualizeData) && !is.matrix(visualizeData)) stop("if visualizeData is not character, must be either data frame or matrix")
      externalData<-TRUE
      if(!ncol(visualizeData)==ncol(assay(data))) stop("if give separate visualizeData, must have same number of sample (columns) as assay(data)")
    }
    if(missing(colorScale)) {
      colorScale <- seqPal5
      if(is.character(visualizeData)) {
        if (visualizeData == "centeredAndScaled") {
          colorScale <- seqPal4
        }
      }
    }

    ####
    ##Transform data and determine which features to use
    ####
    # clusterFeaturesData <- .convertTry(clusterFeaturesData,
    #                                    try(match.arg(clusterFeaturesData),
    #                                        silent=TRUE))

    if(is.list(clusterFeaturesData)){
      groupFeatures<-clusterFeaturesData
      clusterFeaturesData<-unlist(clusterFeaturesData)
    }
    else groupFeatures<-NULL
    if(!externalData){
      if(!clusterFeatures && visualizeData=="original"){
        heatData<-assay(data, whichAssay)
      }
      else{
        if(length(clusterFeaturesData)==1 && isPossibleReducedDims(data,clusterFeaturesData)){
          ##### Dimensionality reduction ####
          if(!isReducedDims(data,clusterFeaturesData)){
            data<-makeReducedDims(data,reducedDims=clusterFeaturesData,maxDims=nFeatures,whichAssay=whichAssay)
          }
          heatData<-t(reducedDim(data,type=clusterFeaturesData))
        }
        else{
          #remaining options subset the data,
          #either by filtering or by user-specified genes
          #These operations will filter the input data object to only the relevant genes
          if(length(clusterFeaturesData)==1 && isPossibleFilterStats(data, clusterFeaturesData) ){
            ##### Filter ####
            if(!isFilterStats(data,clusterFeaturesData)){
              data<-makeFilterStats(data,filterStats=clusterFeaturesData,whichAssay=whichAssay)
            }
            if(is.na(nFeatures)) nFeatures<-min(NROW(data),500)
            data<-filterData(data,filterStats=clusterFeaturesData,percentile=nFeatures)
          }
          else{
            ### Other character values ####
            if(is.character(clusterFeaturesData)){#gene names
              if(length(clusterFeaturesData)==1 && clusterFeaturesData=="all")
                whRows<-seq_len(NROW(data))
              else{
                ### Give specific genes to use ####
                if(is.null(rownames(data))) stop("Cannot give feature names in clusterFeaturesData unless assay(data) has rownames")
                else{
                  whRows<-match(clusterFeaturesData,rownames(data))
                  if(all(is.na(whRows))) stop("None of the feature names in clusterFeaturesData match rownames(assay(data))")
                  if(any(is.na(whRows))){
                    warning("Not all of the feature names in clusterFeaturesData match rownames(assay(data))")
                    whRows<-na.omit(whRows)
                  }
                }
              }
            }
            else{
              ### Numeric row indices ####
              if(any(!clusterFeaturesData %in% seq_len(NROW(data)))) stop("invalid indices for clusterFeaturesData")
              whRows<-clusterFeaturesData
            }
            data<-data[whRows,]
          }
          heatData<-switch(visualizeData,
                           "original"=assay(data, whichAssay),
                           "transformed"=transformData(data, whichAssay),
                           "centeredAndScaled"=t(scale(t(transformData(data, whichAssay)), center=TRUE, scale=TRUE))
          )
        }
      }
    }
    else{
      heatData<-visualizeData
    }


    ######
    #Make colData based on clusterings and columns of colData
    ######
    #---
    #Get clusterings
    #---
    whCl<-getClusterIndex(data,whichClusters=whichClusters,noMatch="silentlyRemove")
    #
    if(length(whCl)>0){
      clusterData<-clusterMatrixNamed(data,whichClusters=whCl)
    }
    else{
      if(any( whichClusters!="none")) warning("given whichClusters value does not match any clusters, none will be plotted")
      clusterData<-NULL
    }
    #---
    #get colData values and subset to those asked for by user
    #---
    sData<-.pullColData(data,colData)
    #identify which numeric
    if(!is.null(sData)) whCont<-which(sapply(seq_len(ncol(sData)),function(ii){is.numeric(sData[,ii])}))
    whColDataCont<-NULL

    if(!is.null(clusterData) & !is.null(sData)){
      colData<-data.frame(clusterData,sData,stringsAsFactors=FALSE,check.names=FALSE)
      if(length(whCont)>0)  whColDataCont<-whCont+ncol(clusterData)
    }
    else{
      if(!is.null(clusterData)) colData<-clusterData
      if(!is.null(sData)){
        colData<-sData
        if(length(whCont)>0) whColDataCont<-whCont
      }
      if(is.null(sData) & is.null(clusterData)) colData<-NULL
    }

    #------
    #check user didn't give something different for colors
    #------
    clLegend<-clusterLegend(data)[whCl] #note, clusterLegend gives names even though not stored internally with @clusterLegend so will match, which plotHeatmap needs
    if(length(clLegend)==0) clLegend<-NULL

    if(!"symmetricBreaks" %in% names(userList) && !externalData && visualizeData %in% c("centeredAndScaled","centered")){
      userList$symmetricBreaks<-TRUE
    }
    userAlign<-"alignColData" %in% names(userList) & !is.null(userList$alignColData)
    userLegend<-"clusterLegend" %in% names(userList) & !is.null(userList$clusterLegend)
    if(userAlign | userLegend){ #if user asks for alignment, don't assign clusterLegend
      if(userLegend){
        userClLegend<-userList[["clusterLegend"]]
        annotNames<-colnames(colData)
        if("annRow" %in% names(userList)) annotNames<-c(annotNames,colnames(userList$annRow))
        if(!is.null(groupFeatures)) annotNames<-c(annotNames,"Gene Group")
        userClLegend<-userClLegend[names(userClLegend) %in% annotNames]
        if(length(userClLegend)==0){
          warning("names of list given by user in clusterLegend do not match clusters nor colData chosen. Will be ignored.")
        }
        else{
          #keep existing clLegend from ClusterExperiment object if not conflict with user input:
          whNotShared<-which(!names(clLegend)%in% names(userClLegend))
          if(length(whNotShared)>0) clLegend<-c(userClLegend,clLegend[whNotShared]) else clLegend<-userClLegend
          clLegend<-.convertToAheatmap(clLegend, names=TRUE)

        }
        userList<-userList[-grep("clusterLegend",names(userList))]
      }
      else{
        if(userAlign){
          al<-userList[["alignColData"]]
          if(al) clLegend<-NULL
        }
      }
    }
    else clLegend<-.convertToAheatmap(clLegend,names=TRUE)
    ######
    #Create clusterSamplesData
    ######
    clusterSamplesData<-.convertTry(clusterSamplesData, try(match.arg(clusterSamplesData), silent=TRUE))
    if(is.logical(clusterSamplesData)) clusterSamples<-clusterSamplesData
    else{
      clusterSamples<-TRUE
      if(is.numeric(clusterSamplesData)){
        heatData<-heatData[,clusterSamplesData,drop=FALSE]
        if(!is.null(colData)) colData<-colData[clusterSamplesData,,drop=FALSE]
        clusterSamplesData<-heatData
        clusterSamples<-FALSE
      }
      else if(is.character(clusterSamplesData)){
        if(clusterSamplesData=="orderSamplesValue"){
          heatData<-heatData[,orderSamples(data),drop=FALSE]
          if(!is.null(colData)) colData<-colData[orderSamples(data), ,drop=FALSE]
          clusterSamplesData<-heatData
          clusterSamples<-FALSE
        }
        else if(clusterSamplesData=="dendrogramValue"){
					if(is.null(data@dendro_samples)){
			      clusterSamplesData <- try( convertToDendrogram(makeDendrogram(data)) ,silent = TRUE) 
	          if(inherits(clusterSamplesData, "try-error")){
	            warning("cannot make dendrogram from 'data' with default makeDendrogram options. Ordering by primary cluster without dendrogram")
	            clusterSamplesData<-"primaryCluster"
	          }
          }
          else{
						#make sure get the sample ids as labels of the tips:
            clusterSamplesData<-try(convertToDendrogram(data),silent=TRUE)
						if(inherits(clusterSamplesData, "try-error")){
	            warning("cannot make dendrogram class from stored dendrograms. Ordering by primary cluster without dendrogram")
	            clusterSamplesData<-"primaryCluster"
	          }
          }
        }
        if(is.character(clusterSamplesData) && clusterSamplesData=="primaryCluster"){
          whUnassign<-which(primaryCluster(data) %in% c(-1,-2))
          if(length(whUnassign)==nSamples(data) || length(unique(primaryCluster(data)[-whUnassign]))==1){
            #in this case, all -1/-2 or same cluster, just do heatmap with hclust
            warning("Cannot order by primary cluster because all one cluster and/or all clustering values are -1/-2. Using standard hiearchical clustering.")
            clusterSamplesData<-"hclust"
          }
          else{
            heatData<-heatData[,order(primaryCluster(data))]
            if(!is.null(colData)) colData<-colData[order(primaryCluster(data)),,drop=FALSE]
            clusterSamplesData<-heatData
            clusterSamples<-FALSE
          }
        }
        if(is.character(clusterSamplesData) && clusterSamplesData=="hclust"){
          #if hclust, then use the visualizeData data
          #unless visualizeData data is original, in which case use transformed (and possibly filtered)
          if(is.character(visualizeData) && visualizeData=="original")
            clusterSamplesData<-transformData(data,whichAssay=whichAssay)
          else clusterSamplesData<-heatData
        }
      }
      else stop("clusterSamplesData must be either character, or vector of indices of samples")
    }

    #################
    ###Deal with grouping of genes
    #################
    if(!is.null(groupFeatures)){
      #convert groupFeatures to indices on new set of data.
      groupFeatures<-lapply(groupFeatures,function(x){match(x,whRows)})
      blankData<-makeBlankData(data=heatData,groupsOfFeatures=groupFeatures,nBlankFeatures=nBlankLines)
      #replace heatData with one with blanks -- won't cluster them now...
      heatData<-data.matrix(blankData$dataWBlanks)
      labRow<-blankData$rowNamesWBlanks
      clusterFeatures<-FALSE

      if(!is.null(names(groupFeatures))){
        annRow<-list("Gene Group"=factor(blankData$featureGroupNamesWBlanks,levels=names(groupFeatures)))
        #show color-coding of gene groupings:
        if("annRow" %in% names(userList)){
          stop("Cannot provide 'annRow' if grouping features")
          # if(is.list(userList$annRow)) userList$annRow<-c(userList$annRow,annRow)
          # else userList$annRow<-c(list(userList$annRow),annRow)
        }
        else userList$annRow<-annRow
        if(!"Gene Group" %in% names(clLegend)){
          nGroups<-length(groupFeatures)
          groupColors<-bigPalette[seq_len(nGroups)]
          names(groupColors)<-levels(annRow[["Gene Group"]])
          clLegend<-c(clLegend, "Gene Group"=list(groupColors))
        }
      }
    }
    else{
      labRow<-rownames(heatData)
    }
    do.call("plotHeatmap",c(list(data=heatData,
                                 clusterSamplesData=clusterSamplesData,
                                 clusterFeaturesData=heatData, #set it so user doesn't try to pass it and have something weird happen because dimensions wrong, etc.
                                 colData=colData,whColDataCont=whColDataCont,
                                 clusterSamples=clusterSamples,labRow=labRow,
                                 clusterLegend=clLegend,clusterFeatures=clusterFeatures,
                                 colorScale=colorScale),userList))


  })


#' @rdname plotHeatmap
setMethod(
  f = "plotHeatmap",
  signature = signature(data = "data.frame"),
  definition = function(data,...){
	  plotHeatmap(data.matrix(data),...)
  }
)
#' @rdname plotHeatmap
setMethod(
  f = "plotHeatmap",
  signature = signature(data = "ExpressionSet"),
  definition = function(data,...){
	  plotHeatmap(exprs(data),...)
  }
)

#' @rdname plotHeatmap
setMethod(
  f = "plotHeatmap",
  signature = signature(data = "matrixOrHDF5"),
  definition = function(data,colData=NULL,
                        clusterSamplesData=NULL,
                        clusterFeaturesData=NULL,
                        whColDataCont=NULL,
                        clusterSamples=TRUE,showSampleNames=FALSE,
                        clusterFeatures=TRUE,showFeatureNames=FALSE,
                        colorScale=seqPal5,
                        clusterLegend=NULL,alignColData=FALSE,
                        unassignedColor="white",missingColor="grey",
                        breaks=NA,symmetricBreaks=FALSE,capBreaksLegend=FALSE,
                        isSymmetric=FALSE, overRideClusterLimit=FALSE,
						plot=TRUE,labelTracks=TRUE,...
  ){
    
    aHeatmapArgs<-list(...)  
		checkIgnore<-.depricateArgument(passedArgs=aHeatmapArgs,"colData","sampleData") #06/2018 added in BioC 3.8
		if(!is.null(checkIgnore)){
			aHeatmapArgs<-checkIgnore$passedArgs
			colData<-checkIgnore$val
		}
    
    ##########
    ##Deal with numeric matrix for heatmap ...
    ##########
    heatData<-data.matrix(data)
    aHeatmapDefaultArgs<-as.list(args(NMF::aheatmap))
    getHeatmapValue<- function(string,value=NULL){ #note, doesn't work for pulling function 'reorder' so put in manually
      if(string %in% names(aHeatmapArgs)) val<-aHeatmapArgs[[string]]
      else{
        if(is.null(value)) val<-aHeatmapDefaultArgs[[string]]
        else val<-value
      }
      return(val)
    }
    badValues<-c("Rowv","Colv","color","annCol","annColors")
    replacedValues<-c("clusterSamplesData","clusterFeaturesData","colorScale","colData","clusterLegend")
    if(any(badValues %in% names(aHeatmapArgs))) stop("The following arguments to aheatmap cannot be set by the user in plotHeatmap:",paste(badValues,collapse=","),". They are over-ridden by: ",paste(replacedValues,collapse=","))

    ##########
    ###Create the clustering dendrogram (samples):
    ##########
    if(clusterSamples){
			if(inherits(clusterSamplesData,"dendrogram")){
        if(nobs(clusterSamplesData)!=ncol(heatData)) stop("clusterSamplesData dendrogram is not on same number of observations as heatData")
        dendroSamples<-clusterSamplesData
      }
      else{
        if(is.null(clusterSamplesData)){
          dendroSamples<-NULL
        }
        else{
          ##Call NMF:::cluster_mat so do the same thing:
          
          if(!is.data.frame(clusterSamplesData) & !is.matrix(clusterSamplesData) & !inherits(clusterSamplesData,"DelayedArray")) stop("clusterSamplesData must either be dendrogram, or data.frame/matrix/DelayedArray class")
          clusterSamplesData<-data.matrix(clusterSamplesData)
          #check valid
          if(ncol(clusterSamplesData)!=ncol(heatData)) stop("clusterSamplesData matrix does not have on same number of observations as heatData")
          dendroSamples<-NMF:::cluster_mat(t(clusterSamplesData),param=TRUE,distfun=getHeatmapValue("distfun"),hclustfun=getHeatmapValue("hclustfun"),reorderfun=getHeatmapValue("reorderfun",value=function(d, w) reorder(d, w)))$dendrogram
          
          #dendroSamples<-as.dendrogram(stats::hclust(stats::dist(t(clusterSamplesData)))) #dist finds distances between rows
          
        }
      }
    }
    else{
      clusterSamples<-NA
    }
    if(!is.na(clusterSamples) && clusterSamples && is.null(dendroSamples)) Colv<-TRUE #then just pass the data
    else Colv<-if(!is.na(clusterSamples) && clusterSamples) dendroSamples else clusterSamples
    
    ##########
    ###Create the clustering dendrogram (features):
    ##########
    if(isSymmetric){
      Rowv<-Colv
      Colv<-"Rowv"
    }
    else{
      if(clusterFeatures){
        if(inherits(clusterFeaturesData, "dendrogram")){
					if(nobs(clusterFeaturesData)!=nrow(heatData)) stop("clusterFeaturesData dendrogram is not on same number of observations as heatData")
          dendroFeatures<-clusterFeaturesData
        }
        else{
          if(is.null(clusterFeaturesData)){
            dendroFeatures<-NULL
          }
          else{
            ##Call NMF:::cluster_mat so do the same thing:
            
            if(!is.data.frame(clusterFeaturesData) & !is.matrix(clusterFeaturesData) &  !inherits(clusterFeaturesData,"DelayedArray")) stop("clusterFeaturesData must either be dendrogram, or data.frame/matrix")
            clusterFeaturesData<-data.matrix(clusterFeaturesData)
            #check valid
            if(ncol(clusterFeaturesData)!=ncol(heatData)) stop("clusterFeaturesData matrix not have on same number of observations as heatData")
            dendroFeatures<-NMF:::cluster_mat(clusterFeaturesData,param=TRUE,distfun=getHeatmapValue("distfun"),hclustfun=getHeatmapValue("hclustfun"),reorderfun=getHeatmapValue("reorderfun",value=function(d, w) reorder(d, w)))$dendrogram
            #                dendroFeatures<-as.dendrogram(stats::hclust(stats::dist(clusterFeaturesData))) #dist finds distances between rows
            
          }
          
        }
      }
      else{
        clusterFeatures<-NA
      }
      if(!is.na(clusterFeatures) && clusterFeatures && is.null(dendroFeatures)) Rowv<-TRUE #then just pass the data
      else Rowv<-if(!is.na(clusterFeatures) && clusterFeatures) dendroFeatures else clusterFeatures
      
      
    }
    
    
    ##########
    ##Deal with annotation of samples (colData) ...
    ##########
    if(!is.null(colData)){
      #----
      #check colData input:
      #----
      if(!is.matrix(colData) & !is.data.frame(colData)) stop("colData must be a either a matrix or a data.frame")
      if(NCOL(data) != NROW(colData)) stop("colData must have same number of rows as columns of heatData")
      if(NCOL(colData)>10){
        if(overRideClusterLimit) warning("More than 10 annotations/clusterings can result in incomprehensible errors in aheamap. You have >10 but have chosen to override the internal stop by setting overRideClusterLimit=TRUE.")
        else stop("More than 10 annotations/clusterings cannot be reliably shown in plotHeatmap. To override this limitation and try for yourself, set overRideClusterLimit=TRUE.")
      }
      
      #----
      # check that no ordered factors...
      #----
      anyOrdered<-sapply(seq_len(ncol(colData)),function(ii){is.ordered(colData[,ii])})
      if(any(anyOrdered)) stop("The function aheatmap in the NMF package that is called to create the heatmap does not currently accept ordered factors (https://github.com/renozao/NMF/issues/83)")
      
      #-------------------
      # run .makeColors. This will:
      # 1) Make colData explicitly factors (except for whColDataCont variables which are not given to function)
      # 2) Make default clusterLegend, including incorporating and checking user-given clusterLegend
      # 3) Make a numeric summary of factors (for if alignSamples==TRUE)
      #-------------------
      if(is.data.frame(colData)) colData<-droplevels(colData)
      if(!is.null(whColDataCont)){
        if(any(logical(whColDataCont))) whColDataCont<-which(whColDataCont)
      }
      if(length(whColDataCont)>0) tmpDf<-colData[,-whColDataCont,drop=FALSE]
      else tmpDf<-colData
      defaultColorLegend<-.makeColors(tmpDf,colors=massivePalette,unassignedColor=unassignedColor,missingColor=missingColor, distinctColors=TRUE, matchClusterLegend = clusterLegend, matchTo="name") 
      tmpDfNum<-defaultColorLegend$numClusters
      colnames(tmpDfNum)<-colnames(tmpDf)
      #so that annCol has them as factors.
      tmpDf<-defaultColorLegend$facClusters
      if(length(whColDataCont)>0){
        annCol<-colData
        annCol[,-whColDataCont]<-tmpDf
      }			  
      else annCol<-tmpDf 
      
      #-----
      #final update of clusterLegend
      #-----
      if(is.null(clusterLegend) & alignColData & (is.null(whColDataCont) || length(whColDataCont)<ncol(annCol))){
            #align the clusters and give them colors
            alignObj<-plotClusters(tmpDfNum ,plot=FALSE,unassignedColor=unassignedColor, missingColor=missingColor)
            defaultColorLegend<-.makeColors(tmpDf,clNumMat=tmpDfNum,colors=massivePalette,unassignedColor=unassignedColor,missingColor=missingColor, matchClusterLegend=alignObj$clusterLegend,matchTo="numIds")
      }
      #preserve those in given clusterLegend that don't match colData (could go with features/rows)
			
      if(is.list(clusterLegend)){ #could be single vector, but in that case, will loose them
        whKeep<-names(clusterLegend)[which(!names(clusterLegend)%in% names(defaultColorLegend$colorList))]
        clusterLegend<-c(defaultColorLegend$colorList,clusterLegend[whKeep])
      }
      else clusterLegend<-defaultColorLegend$colorList

      # Convert to aheatmap format, if needed
      annColors<-.convertToAheatmap(clusterLegend, names=TRUE)
      
      ##########################
      # remove any unused level colors to clean up legend,
      # give names to colors if not given, and
      # make them in same order as in annCol factor
      ##########################
      whInAnnColors<-which(names(annColors)%in% colnames(annCol))
      if(!is.null(whColDataCont) & length(whColDataCont)>0){
        whInAnnColors<-setdiff(whInAnnColors,whColDataCont)
      }
      
      #-----
      # Deal with extra levels, only a single level, etc. 
      # These are all problems with NMF. Need to check on development version.
			# Also, doesn't deal with if in rowData...
      #-----
      prunedList<-lapply(whInAnnColors,function(ii){
        nam<-names(annColors)[[ii]] #the name of variable
        x<-annColors[[ii]] ##list of colors
        levs<-levels(annCol[,nam])
        if(length(x)<length(levs)) stop("number of colors given for ",nam," is less than the number of levels in the data")
        #Note to self:
        #It appears that if there is only 1 level of a factor, aheatmap
        #doesn't plot it unless there are colors longer than 1
        #But appears only problem with extra colors are just that the first ones must match the levels -- not being matched (or at least not well) to names of the colors.
        #So going to leave "extra" colors in place, but put them at the end.
        if(is.null(names(x))){
          #if user didn't give names to colors, assign them in order of levels
          #x<-x[1:length(levs)] #shorten if needed
          names(x)<-levs
        }
        else{
          if(any(!levs %in% names(x))) stop("colors given for ",nam," do not cover all levels in the data")
          whlevs<-match(levs,names(x))
          x<-c(x[whlevs],x[-whlevs])
        }
        return(x)
      })
      names(prunedList)<-names(annColors)[whInAnnColors]
      annColors[whInAnnColors]<-prunedList
      
    }
    else{ #no colData provided -- just a heatmap with no annotation
      annCol<-NA
      annColors<-NA
    }
    
    #############
    # put into aheatmap
    #############
    capBreaks<-length(breaks)==1 & capBreaksLegend
    breaks<-setBreaks(data=heatData, breaks=breaks, makeSymmetric=symmetricBreaks,returnBreaks=!capBreaks)
    if(capBreaks){ #so the legend is not so weird
      if(length(breaks)!=2)
        stop("coding error in new breaks function")
      heatData[which(heatData<breaks[1])]<-breaks[1]
      heatData[which(heatData>breaks[2])]<-breaks[2]
      breaks<-seq(breaks[1],breaks[2],length=52)
    }
    if(plot){
	    out <-NMF::aheatmap(heatData,
                         Rowv =Rowv,Colv = Colv,
                         color = colorScale, scale = getHeatmapValue("scale","none"),
                         annCol = annCol,annColors=annColors,breaks=breaks,...)
      
      #############
      # add labels to clusters at top of heatmap
      #############
      if(!is.null(dim(annCol)) & labelTracks){
        treeOfViewports<-unlist(lapply(strsplit(as.character(grid::current.vpTree(all=TRUE)),"->"),strsplit,","))
        wh<-sapply(treeOfViewports, function(x){
          length(grep(x,"pattern"="aheatmap-AHEATMAP.VP"))>0
        })
        stringsViewports<-unlist(lapply(treeOfViewports[wh],
                                        function(x){
                                          x[grep(x,"pattern"="aheatmap-AHEATMAP.VP")]
                                        }))
        stringsViewports<-gsub("viewport\\[","",stringsViewports)
        stringsViewports<-gsub("\\(","",stringsViewports)
        stringsViewports<-gsub("\\)","",stringsViewports)
        stringsViewports<-gsub("\\]","",stringsViewports)
        vals<-sapply(  strsplit(
          stringsViewports,"[.]VP[.]"), .subset2,2)
        vals<-suppressWarnings(as.numeric(vals))
        XX<-max(vals,na.rm=TRUE) #incase something there that doesn't convert to numeric well.
        viewportName<-paste0("AHEATMAP.VP.",XX)
        test<-try(grid::seekViewport(paste0("aheatmap-",viewportName)),silent=TRUE)
        if(!inherits(test,"try-error")){
          ###Now make a viewport related to this part of grid:
          ##( The part of the grid that corresponds
          ## to where the names should go)
          #grid::grid.rect() #for developing -- see where grabbed.
          newName<-paste0(viewportName,"-sideLabels")
          grid::pushViewport(grid::viewport(layout.pos.row = 3, layout.pos.col = 4:5,  name = newName))
          y <- seq(0,1,length=ncol(annCol))
          n<-ncol(annCol)
          y = cumsum(rep(8, n)) - 4 + cumsum(rep(2, n))
          #		grid::grid.points(x = grid::unit(rep(0,length(y)),"npc"),y = grid::unit(y[n:1], "bigpts"))
          grid::grid.text(colnames(annCol), x = grid::unit(rep(0.05,length(y)),"npc"),y = grid::unit(y[n:1], "bigpts"), vjust = 0.5, hjust = 0,gp= grid::gpar(fontsize=10))
          grid::upViewport(0) #take it back up to the top.
          
        }
        
        
        
      }
    }
    else out<-NULL
    
    invisible(list(aheatmapOut=out,colData=annCol,clusterLegend=clusterLegend,breaks=breaks))
  }
)

#' @rdname plotHeatmap
#' @aliases plotCoClustering
#' @param invert logical determining whether the coClustering matrix should be
#'   inverted to be 1-coClustering for plotting. By default, if the diagonal
#'   elements are all zero, invert=TRUE, and otherwise invert=FALSE. If
#'   coClustering matrix is not a 0-1 matrix (e.g. if equal to a distance matrix
#'   output from \code{\link{clusterSingle}}, then the user should manually set
#'   this parameter to FALSE.)
#' @param saveDistance logical. When the \code{coClustering} slot contains
#'   indices of the clusterings or a NxB set of clusterings, the hamming
#'   distance will be calculated before running the plot. This argument
#'   determines whether the \code{ClusterExperiment} object with that distance
#'   in \code{coClustering} slot should be returned (so as to avoid
#'   re-calculating it in the future) or not.
#' @details \code{plotCoClustering} is a convenience function to plot the
#'   heatmap of the co-clustering distance matrix from the \code{coClustering}
#'   slot of a \code{ClusterExperiment} object (either by calculating the
#'   hamming distance of the clusterings stored in the \code{coClustering} slot,
#'   or the distance stored in the \code{coClustering} slot if it has already
#'   been calculated.
#' @export
setMethod(
  f = "plotCoClustering",
  signature = "ClusterExperiment",
  definition = function(data, invert, saveDistance=FALSE,...){
    typeCoCl<-.typeOfCoClustering(data)
    if(typeCoCl=="null") stop("coClustering slot is empty")
    if(typeCoCl=="indices"){
        #calculate the distance
        coClustering(data)<-.clustersHammingDistance(
            t(clusterMatrix(data,whichClusters=data@coClustering)))        
    }
    else{
        #Need to catch if not a symmetric matrix
        #(case of subsampling matrix)
        if(typeCoCl=="clusterings"){
            #calculate the distance
            coClustering(data)<-.clustersHammingDistance(
                t(coClustering(data)))        
            
        }
    }
    if(missing(invert)) 
        invert<-ifelse(all(diag(data@coClustering)==0),TRUE,FALSE)
    if(invert){
        #Make it a similarity matrix (better anyway for the sparse representation)
        coClustering(data) <- 
            as(as(1-data@coClustering,"sparseMatrix"),"symmetricMatrix")
    }
    	
    # Do all this so don't have to erase merge info from data (so can return calculated distance to user)
    fakeCE<-ClusterExperiment(as(data@coClustering,"matrix"),
                              clusterMatrix(data),
                              transformation=function(x){x},
                              checkTransformAndAssay=FALSE


    )
    for(sName in c('clusterMatrix', 'primaryIndex', 'clusterInfo', 
        'clusterTypes', 'dendro_samples', 'dendro_clusters', 'dendro_index', 
        'clusterLegend', 'orderSamples', 'merge_index', 
        'merge_dendrocluster_index', 'merge_method', 'merge_demethod', 
        'merge_cutoff', 'merge_nodeProp', 'merge_nodeMerge','colData')){
        slot(fakeCE, sName)<-slot(data,sName)
    }   
    plotHeatmap(fakeCE,isSymmetric=TRUE,clusterFeaturesData="all",...)
    if(saveDistance) return(data)
    else invisible()
  })

##Code for timing the results of the tests:
# testOutput<-testOutput<-test_dir("testthat/",reporter=ListReporter)
# timingDf<-do.call("rbind", lapply(testOutput, function(x){data.frame(file=x$file, test=x$test, time=round(x$real,2))}))
# timingDf<-timingDf[order(timingDf$time),]
# write.table(file="testTimings.txt", x=timingDf)

###Note: any changes to this file should be at the END so as to not mess up the seed calls.
library(clusterExperiment)
# library(devtools)
# load_all()
data(simData, envir = environment())
if(ncol(simData) != 300) {
  stop("not current version of simData")
  #get all kinds of annoyances because using old version.
  #Can delete this once package is stabilized.
}
set.seed(23)

#################################
###Simple, trivial sized objects for testing:
# mat=20x15 matrix of data with row and column names
# labMat 15 x 2 matrix of numberic clusters, labeled 2-5, plus -1,-2 values with column names
# sData 15 x 3 matrix of data on the samples, a factor, continuous, and character variable with column names
# gData 20 x 4 matrix of data on the features: factor, continuous, character (like sData) with column names
# mData list of length 2 of random information to be metaData
# se a Summarized experiment with rowData, colData, and metaData slots
#################################
mat <- matrix(data=rnorm(20*15), ncol=15)
mat[1,1]<- -1 #force a negative value
colnames(mat)<-paste("Sample",1:ncol(mat))
rownames(mat)<-paste("Gene",1:nrow(mat))
dissMat<-as.matrix(dist(t(mat)))
numLabels <- as.character(gl(5, 3))
numLabels[c(1:2)]<- c("-1","-2") #make sure some not assigned
numLabels<-factor(numLabels)
chLabels<-rep(LETTERS[1:5],each=3)
chLabels[c(2:3)]<- c("-1","-2") #make sure some not assigned
labMat<-cbind(as.numeric(as.character(numLabels)),as.numeric(as.character(numLabels)))
colnames(labMat)<-c("Cluster1","Cluster2")
sData<-data.frame(sample(letters[2:5],size=NCOL(mat),replace=TRUE),sample(2:5,size=NCOL(mat),replace=TRUE))
sData<-data.frame(sData,sample(LETTERS[2:5],size=NCOL(mat),replace=TRUE),stringsAsFactors=FALSE)
gData<-data.frame(sample(letters[2:5],size=NROW(mat),replace=TRUE),sample(2:5,size=NROW(mat),replace=TRUE))
gData<-data.frame(gData,sample(LETTERS[2:5],size=NROW(mat),replace=TRUE),stringsAsFactors=FALSE)
colnames(sData)<-c("A","B","C")
colnames(gData)<-c("a","b","c")
mData<-list(first=c(1,2,3),second=c("Information"))
se <- SummarizedExperiment(mat,colData=sData,rowData=gData,metadata=mData)
cc <- ClusterExperiment(mat, labMat, transformation = function(x){x})
ccSE<-ClusterExperiment(se, labMat, transformation=function(x){x})


#################################
###Larger sized objects based on simData/simCount:
#################################
data(simData)
if(ncol(simData)!=300) stop("not current version of simData") #get all kinds of annoyances because using old version.
simSData<-data.frame(sample(letters[2:5],size=NCOL(simData),replace=TRUE),sample(2:5,size=NCOL(simData),replace=TRUE))
simSData<-data.frame(simSData,sample(LETTERS[2:5],size=NCOL(simData),replace=TRUE),stringsAsFactors=FALSE)
colnames(simSData)<-c("A","B","C")
gSimData<-data.frame(sample(letters[2:5],size=NROW(simData),replace=TRUE),sample(2:5,size=NROW(simData),replace=TRUE))
gSimData<-data.frame(gSimData,sample(LETTERS[2:5],size=NROW(simData),replace=TRUE),stringsAsFactors=FALSE)
colnames(gSimData)<-c("a","b","c")

seSimData <- SummarizedExperiment(simData,colData=simSData,rowData=gSimData,metadata=mData)
seSimCount <- SummarizedExperiment(simCount,colData=simSData,rowData=gSimData,metadata=mData)

test<- clusterMany(simCount,reduceMethod="PCA",
    nReducedDims=c(5,10,50), 
    isCount=TRUE, verbose=FALSE,
    clusterFunction="pam", makeMissingDiss=TRUE,
    ks=2:4,
    findBestK=c(TRUE,FALSE))
						
test<-addClusterings(test,sample(2:5,size=NCOL(simData),replace=TRUE),clusterTypes="User")
clMatNew<-apply(clusterMatrix(test),2,function(x){
    wh<-sample(1:nSamples(test),size=10)
    x[wh]<- -1
    wh<-sample(1:nSamples(test),size=10)
    x[wh]<- -2
    return(x)
})

#make a new object with -1 values
ceSim<-ClusterExperiment(seSimCount,clMatNew,transformation=function(x){log2(x+1)})
clusterTypes(ceSim)<-clusterTypes(test)

ceSimData<-ClusterExperiment(seSimData,clMatNew,transformation=function(x){x})
clusterTypes(ceSimData)<-clusterTypes(test)

rm(test)
#################################
###small object based on simData/simCount (same size as trivial data)
### 15 samples from each of groups (including -2,-1)
###
#################################
whSamp<-unlist(tapply(1:nSamples(ceSim),primaryCluster(ceSim),function(x){sample(x,size=3)})) #15
smSimData<-simData[1:20,whSamp]
smSimCount<-simCount[1:20,whSamp]
smSimCE<-ceSim[1:20,whSamp]
smSimSE <- seSimData[1:20,whSamp]



#################################
###Make reduce dimensions and filters
#################################
sce<-as(se,"SingleCellExperiment")
sceFull<-sce
clusterExperiment:::filterStats(sceFull,type=c("Filter1","Filter2"))<-matrix(rnorm(2*nrow(sce)),ncol=2)
reducedDim(sceFull,type="Red1")<-matrix(rnorm(2*ncol(sce)),ncol=2)


sceSimData<-as(seSimData,"SingleCellExperiment")
sceSimDataDimRed<-sceSimData
pca_data <- prcomp(t(assay(sceSimData)),scale=TRUE,center=TRUE)
tsne_data <- matrix(rnorm(NCOL(sceSimData)*2),ncol=2)
reducedDims(sceSimDataDimRed) <- SimpleList(PCA=pca_data$x, TSNE=tsne_data)
clusterExperiment:::filterStats(sceSimDataDimRed,type=c("Filter1","Filter2"))<-matrix(rnorm(2*nrow(sceSimDataDimRed)),ncol=2)

#####################
## Create hdf5 SCE version
## Note is matrix of doubles....
#####################
hdfSCE<-HDF5Array::saveHDF5SummarizedExperiment(sceSimDataDimRed, dir="sceRedDem.h5", replace=TRUE)
hdfObj<-HDF5Array::saveHDF5SummarizedExperiment(sceSimData, dir="sce.h5", replace=TRUE)

###################
## Create cat stuff
###################
catMat <- matrix(sample(x=1:4,size=20*19,replace=TRUE),ncol=20)
catMat[1,1]<- -1 #force a negative value
colnames(catMat)<-paste("Sample",1:ncol(catMat))
rownames(catMat)<-paste("Gene",1:nrow(catMat))

#' Helper methods for the ClusterFunction class
#' 
#' This is a collection of helper methods for the ClusterExperiment class.
#' @name ClusterFunction-methods
#' @aliases ClusterFunction-methods
#' @param object input to the method, either a \code{ClusterFunction}
#'   class or a character describing a built-in \code{ClusterFunction} object. 
#'   Can also be a \code{list} of \code{ClusterFunction} objects, in which case 
#'   the list must have names for each function.
#' @details Note that when subsetting the data, the dendrogram information and 
#'   the co-clustering matrix are lost.
#' @return \code{requiredArgs}  returns a list of the required args of a
#'   function (via a call to \code{\link{requiredArgs}})
#' @export
setMethod(
  f = "requiredArgs",
  signature = c("character"),
  definition = function(object) {
	  requiredArgs(getBuiltInFunction(object))    
  }
)

#' @rdname ClusterFunction-methods
#' @param genericOnly logical If TRUE, return only the generic required
#'   arguments (i.e. those required by the algorithm type) and not the arguments
#'   specific to that clustering found in the slot \code{requiredArgs}. If FALSE
#'   both sets of arguments are returned.
#' @export
setMethod(
  f = "requiredArgs",
  signature = c("ClusterFunction"),
  definition = function(object,genericOnly=FALSE) {
    algType<-algorithmType(object)
    if(!genericOnly){
      if(!is.na(object@requiredArgs)) reqArgs<-object@requiredArgs
      else reqArgs<-NULL
      if(algType=="01") return(unique(sort(c(reqArgs,.required01Args))))
      if(algType=="K") return(unique(sort(c(reqArgs,.requiredKArgs))))
    }
    else{
      if(algType=="01") return(unique(sort(.required01Args)))
      if(algType=="K") return(unique(sort(.requiredKArgs)))	  	
    }
  }
)
#' @rdname ClusterFunction-methods
#' @export
setMethod(
  f = "requiredArgs",
  signature = c("list"),
  definition = function(object) {
		.checkFunctionList(object)
  	return(sapply(object,requiredArgs))
	}
	
)
#' @rdname ClusterFunction-methods
#' @aliases requiredArgs
#' @export
setMethod(
  f = "requiredArgs",
  signature = c("character"),
  definition = function(object) {
	  requiredArgs(getBuiltInFunction(object))
  }
)
#' @rdname ClusterFunction-methods
#' @export
setMethod(
  f = "requiredArgs",
  signature = c("character"),
  definition = function(object) {
    clObjects<-getBuiltInFunction(object)
    if(length(clObjects)>1) return(lapply(clObjects,requiredArgs))
    else return(requiredArgs(clObjects))
  }
)
#' @rdname ClusterFunction-methods
#' @export
setMethod(
  f = "requiredArgs",
  signature = c("factor"),
  definition = function(object) {
	  requiredArgs(as.character(object))
  }
)

#' @rdname ClusterFunction-methods
#' @aliases algorithmType
#' @return \code{algorithmType} returns a character value giving the type of 
#' clustering function ("01" or "K")
#' @export
setMethod(
  f = "algorithmType",
  signature = c("ClusterFunction"),
  definition = function(object) {
    object@algorithmType
  }
)
#' @rdname ClusterFunction-methods
#' @export
setMethod(
  f = "algorithmType",
  signature = c("character"),
  definition = function(object) {
    clObjects<-getBuiltInFunction(object)
    if(length(clObjects)>1) return(sapply(clObjects,algorithmType))
    else return(algorithmType(clObjects))
  }
)
#' @rdname ClusterFunction-methods
#' @export
setMethod(
  f = "algorithmType",
  signature = c("factor"),
  definition = function(object) {
	  algorithmType(as.character(object))
  }
)
#' @rdname ClusterFunction-methods
#' @export
setMethod(
  f = "algorithmType",
  signature = c("list"),
  definition = function(object) {
		.checkFunctionList(object)
  	return(sapply(object,algorithmType))
	}
	
)
.checkFunctionList<-function(object){
	if(is.null(names(object))){
		stop("if you give a list of ClusterFunction objects, the list must have names")
	}
	if(!all(sapply(object,class)=="ClusterFunction")) stop("if not giving the names of built in functions, must be list of objects of class 'ClusterFunction'")
}

#' @rdname ClusterFunction-methods
#' @aliases inputType
#' @return \code{inputType} returns a character value giving the input 
#' type of the object
#' @export
setMethod(
  f = "inputType",
  signature = c("ClusterFunction"),
  definition = function(object) {
    object@inputType
  }
)
#' @rdname ClusterFunction-methods
#' @export
setMethod(
  f = "inputType",
  signature = c("list"),
  definition = function(object) {
		.checkFunctionList(object)
  	return(sapply(object,inputType))
	}
	
)
#' @rdname ClusterFunction-methods
#' @export
setMethod(
  f = "inputType",
  signature = c("character"),
  definition = function(object) {
    clObjects<-getBuiltInFunction(object)
    if(length(clObjects)>1){
        return(lapply(clObjects,inputType))
    }
    else return(inputType(clObjects))
  }
)
#' @rdname ClusterFunction-methods
#' @export
setMethod(
  f = "inputType",
  signature = c("factor"),
  definition = function(object) {
	  inputType(as.character(object))
  }
)

#' @rdname plotContrastHeatmap
#' @aliases plotContrastHeatmap
#' @title Plot heatmaps showing significant genes per contrast
#' @description Plots a heatmap of the data, with the genes grouped based on the
#'   contrast for which they were significant.
#' @param object ClusterExperiment object on which biomarkers were found
#' @param signifTable A \code{data.frame} in format of the result of
#'   \code{\link{getBestFeatures}}. It must minimally contain columns 'Contrast'
#'   and 'IndexInOriginal' giving the grouping and original index of the
#'   features in the \code{assay(object)}
#' @param whichCluster if not NULL, indicates cluster used in making the 
#'   significance table. Used to match to colors in \code{clusterLegend(object)} 
#'   (relevant for one-vs-all contrast so that color aligns). See description of 
#'   argument in \code{\link{getClusterIndex}} for futher details.
#' @param contrastColors vector of colors to be given to contrasts. Should match
#'   the name of the contrasts in the 'Contrast' column of \code{signifTable} or
#'   'ContrastName', if given.. If  missing, default colors given by match to
#'   the cluster names of \code{whichCluster} (see above), or otherwise given a
#'   default assignment.
#' @param ... Arguments passed to \code{\link{plotHeatmap}}
#' @details If the column 'ContrastName' is given in \code{signifTable}, these
#'   names will be used to describe the contrast in the legend.
#' @details Within each contrast, the genes are sorted by log fold-change if the
#'   column "logFC" is in the \code{signifTable} data.frame
#' @details Note that if \code{whichCluster} is NOT given (the default) then there is 
#' no automatic match of colors with contrasts based on the information in 
#' \code{object}.
#' @return A heatmap is created. The output of \code{plotHeatmap} is returned.
#' @seealso \code{\link{plotHeatmap}}, \code{\link{makeBlankData}},
#'   \code{\link{getBestFeatures}}
#' @export
#'
#' @examples
#' data(simData)
#'
#' cl <- clusterSingle(simData, subsample=FALSE,
#' sequential=FALSE, 
#' mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=8)))
#'
#' #Do all pairwise, only return significant, try different adjustments:
#' pairsPerC <- getBestFeatures(cl, contrastType="Pairs", number=5,
#' p.value=0.05, DEMethod="limma")
#' plotContrastHeatmap(cl,pairsPerC)
setMethod(
  f = "plotContrastHeatmap",
  signature = "ClusterExperiment",
  definition = function(object,signifTable,whichCluster=NULL,contrastColors=NULL,...) {
    if(!all(c("IndexInOriginal","Contrast") %in% colnames(signifTable ))) stop("signifTable must have columns 'IndexInOriginal' and 'Contrast'")
    if(!is.numeric(signifTable$IndexInOriginal)) stop("Column 'IndexInOriginal' Must consist of numeric values")
    if(!all(signifTable$IndexInOriginal %in% seq_len(nrow(object)))) stop("Column 'IndexInOriginal' must consist of indices that match the row indices of 'object'")
    #divide by contrast, sort by FC (if exists) and return index
    geneByContrast<-by(signifTable,signifTable$Contrast,function(x){
      if("logFC" %in% names(x)){
        x<-x[order(x$logFC),]
      }
      x$IndexInOriginal
      
    })
    ##Check names of contrastColors
    if(!is.null(contrastColors)){
      if(!all(sort(names(contrastColors)) == sort(unique(signifTable$Contrast))) ){
        if("ContrastName" %in% colnames(signifTable)){
          if(!all(sort(names(contrastColors)) == sort(unique(signifTable$ContrastName))) ){
            warning("names of contrastColors do not match 'Contrast' or 'ContrastName' values; will be ignored.")
            contrastColors<-NULL
            
          }
          else contrastMatch<-FALSE	#FALSE because don't have to fix the names to match ContrastName, since already do...	
        }
        else{
          warning("names of contrastColors do not match 'Contrast' values; will be ignored.")
          contrastColors<-NULL
        }
      }
      else contrastMatch<-TRUE
    }
    if(is.null(contrastColors)) contrastMatch<-FALSE
	if("ContrastName" %in% colnames(signifTable)){
      #give names to be contrast names
	  internalNames<-names(geneByContrast) #incase need the Contrast name later in code
      gpnames<-unique(signifTable[,c("Contrast", "ContrastName", "InternalName")])
      if(nrow(gpnames)==length(geneByContrast)){
        m<-match(names(geneByContrast),gpnames[,"Contrast"])
        names(geneByContrast)<-gpnames[m,"ContrastName"]
		internalNames<-gpnames[m,"InternalName"]
		#give colors the new names so will match
        if(!is.null(contrastColors) & contrastMatch){
          m<-match(names(contrastColors),gpnames[,"Contrast"])
          names(contrastColors)<-gpnames[m,"ContrastName"]
        }
      }
      
      #fix so order is order of contrast names
	  order<-order(names(geneByContrast))
	  geneByContrast<-geneByContrast[order]

    }
    
    
    if(!is.null(whichCluster)){
      ## Assign colors to the contrasts 
      ## (only if contrast is oneAgainstAll)
	  whichCluster<-getSingleClusterIndex(object,
          whichCluster,passedArgs=list(...))
      cl<-clusterMatrix(object)[,whichCluster]
      clMat<-clusterLegend(object)[[whichCluster]]
      clMat<-clMat[which(clMat[,"clusterIds"]>0),] #remove negatives
      ### If the contrast is one against all, should have name of cluster so give color
      ### Note that need to use InternalId, in case the names are not unique?
      internalNames<-gsub("Cl","",internalNames)
      # This can give warnings for NAs, and if option(warn=2) create error...
      internalNames<-try(sort(as.numeric(internalNames)),silent=TRUE)
      if(inherits(internalNames, "try-error") ||
          any(is.na(internalNames))) isOneVAll<-FALSE
      else isOneVAll<-TRUE
      clIds<-unname(sort(as.numeric(clMat[,"clusterIds"])))
      if(is.null(contrastColors) && isOneVAll &&
          identical(internalNames, clIds)){
              contrastColors<-clMat[,"color"]
              names(contrastColors)<-names(geneByContrast)
      }
    }
    if(is.null(contrastColors)){
      #do it here so don't mess up above default assignment of colors
      contrastColors<-tail(massivePalette,length(geneByContrast)) #least likely to be important colors by accident
      names(contrastColors)<-names(geneByContrast)
    }
    
    plotHeatmap(object,
        clusterFeaturesData=geneByContrast,
        clusterLegend=list("Gene Group"=contrastColors),...)
    
  }
)

#code downloaded from http://www.stat.cmu.edu/~jiashun/Research/software/NullandProp/ on Dec 15, 2015
.EstNull.func<-function (x,gamma=0.1)
{
 # x is a vector of z-values
 # gamma is a parameter, default is 0.1
 # output the estimated mean and standard deviation

 n = length(x)
 t = c(seq_len(1000))/200
 
 gan    = n^(-gamma)
 that   = 0 
 shat   = 0
 uhat   = 0
 epshat = 0

 phiplus   = rep(1,1000)
 phiminus  = rep(1,1000)
 dphiplus  = rep(1,1000)
 dphiminus = rep(1,1000)
 phi       = rep(1,1000)
 dphi      = rep(1,1000)

 for (i in seq_len(1000)) {
    s = t[i]
    phiplus[i]   = mean(cos(s*x))
    phiminus[i]  = mean(sin(s*x))
    dphiplus[i]  = -mean(x*sin(s*x))
    dphiminus[i] = mean(x*cos(s*x))
    phi[i]       = sqrt(phiplus[i]^2 + phiminus[i]^2)
 }

 ind = min(c(seq(1000))[(phi - gan) <= 0])
 tt = t[ind]
 a  = phiplus[ind]
 b  = phiminus[ind]
 da = dphiplus[ind]
 db = dphiminus[ind]
 c  = phi[ind]

 that   = tt
 shat   = -(a*da + b*db)/(tt*c*c)
 shat   = sqrt(shat) 
 uhat   = -(da*b - db*a)/(c*c)
 epshat = 1 - c*exp((tt*shat)^2/2)

 return(musigma=list(mu=uhat,s=shat))
}
.epsest.func <- function(x,u,sigma)
{
  # x is a vector
  # u is the mean
  # sigma is the standard deviation

  z  = (x - u)/sigma
  xi = c(0:100)/100
  tmax=sqrt(log(length(x)))
  tt=seq(0,tmax,0.1)

  epsest=NULL

  for (j in seq_along(tt)) { 

    t=tt[j]
    f  = t*xi
    f  = exp(f^2/2)
    w  = (1 - abs(xi))
    co  = 0*xi

    for (i in seq_len(101)) {
      co[i] = mean(cos(t*xi[i]*z));
    } 
    epshat = 1 - sum(w*f*co)/sum(w)
    epsest=c(epsest,epshat)
  }
  return(epsest=max(epsest))
}

############
## Checks that make up validity of ClusterExperiment:
############
.checkAssays<-function(object){
    # ############
    # check assays
    # ############
    if(length(assays(object)) < 1) {
        return("There must be at least one assay slot.")
    }
    if(!is.numeric(assay(object))) {
        return("The data must be numeric.")
    }
    if(anyNA(assay(object))) {
        return("NA values are not allowed.")
    }
    return(TRUE)
}
.checkTransform<-function(object){
    #############
    #check transform
    #############
    tX <- try(transformData(object),silent=TRUE)
    if(inherits(tX, "try-error")){
        stop(paste("User-supplied `transformation` produces error on the input data
                 matrix:\n",tX))
    }
    if(anyNA(tX)) {
        return("NA values after transforming data matrix are not allowed.")
    }
    return(TRUE)
}

.checkClusterMatrix<-function(object){
    ############
    #Check clusterMatrix
    ############
    zeroRow<-NROW(object@clusterMatrix)==0 #could happen in subsetting
    if(!all(is.na((object@clusterMatrix))) &
       !(NROW(object@clusterMatrix) == NCOL(object))) {
        return("If present, `clusterMatrix` must have as many row as cells.")
    }
    if(!zeroRow & !is.numeric(object@clusterMatrix)) {
        return("`clusterMatrix` must be a numeric matrix.")
    }
    
    if(NCOL(object@clusterMatrix)!= length(object@clusterTypes)) {
        return("length of clusterTypes must be same as NCOL of the clusterMatrix")
    }
    
    if(NCOL(object@clusterMatrix)!= length(object@clusterInfo)) {
        return("length of clusterInfo must be same as NCOL of the clusterMatrix")
    }
    #check internally stored as integers
    testConsecIntFun<-function(x){
        whCl<-which(!x %in% c(-1,-2))
        uniqVals<-unique(x[whCl])
        return(all(sort(unname(uniqVals))==seq_along(uniqVals)))
    }
    if(!zeroRow){
        testConsecIntegers<-apply(object@clusterMatrix,2,testConsecIntFun)
        if(!all(testConsecIntegers)) return("the cluster ids in clusterMatrix must be stored internally as consecutive integer values")		
    }
    
    return(TRUE)
}
.checkClusterLabels<-function(object){
    if(!all(is.na(object@clusterMatrix))){
        if(is.null(colnames(object@clusterMatrix))) return("must have clusterLabels by assignment of column names for clusterMatrix")
        if(any(duplicated(colnames(object@clusterMatrix)))) return("cannot have duplicated clusterLabels (i.e. clusterMatrix must have unique column names)")
        
    }
    return(TRUE)
}

#these functions are checks where don't need the corresponding object information
.checkDendroClusterFormat<-function(dendro,checkLabels=TRUE){
    data.cl<-phylobase::tdata(dendro)
    if(!all(.clusterDendroColumns %in% names(data.cl) )){
        return("dendro_clusters must have data with column names:",paste(.clusterDendroColumns,sep=","))
    }
    tp<-phylobase::tipLabels(dendro)
    if(checkLabels && any(sort(as.numeric(gsub("T","",tp)))!=sort(seq_along(tp))))
        return("dendro_clusters cannot have labels for the tips; user-defined labels for the tips (i.e. clusters) should be stored in the clusterLegend")
    if(any(is.na(data.cl$Position))) 
        return("dendro_clusters cannot have NA values in Position variable")
    if( any(is.na(data.cl$NodeId))) 
        return("dendro_clusters cannot have NA values in Node Id variable")
    return(TRUE)
}
.checkDendroSamplesFormat<-function(dendro,checkLabels=TRUE){
    data.cl<-phylobase::tdata(dendro,type="all")
    all(names(data.cl)%in% .clusterSampleColumns)
    if(!all(.clusterSampleColumns %in% names(data.cl) )){
        return("dendro_samples must have data with column names:",paste(.clusterDendroColumns,sep=","))
    }
    if(any(is.na(data.cl$Position))) 
        return("dendro_samples cannot have NA values in Position variable")
    if(checkLabels && any(!is.na(phylobase::nodeLabels(dendro)))) 
        return("dendro_samples cannot have labels for the nodes; the labels for the nodes should be in the cluster dendrogram")
    if(checkLabels){
        tp<-phylobase::tipLabels(dendro)
        if(any(sort(as.numeric(gsub("T","",tp)))!=sort(seq_along(tp)))) return("dendro_samples cannot have labels for the tips; the labels for the tips should be the colnames of the object")
    } 
    data.cl<-phylobase::tdata(dendro,type="tip")
    if(any(is.na(data.cl$SampleIndex))) 
        return("dendro_samples cannot have NA values for tips in SampleIndex variable")
    return(TRUE)
    
}
.checkDendrogram<-function(object){
    ############
    ##Check dendrogram
    ############
    if(!is.null(object@dendro_clusters)){
        if(is.na(dendroClusterIndex(object))) return("if dendrogram slots are filled, must have corresponding dendro_index defined.")
        dcluster<-clusterMatrix(object)[,dendroClusterIndex(object)]
        if(phylobase::nTips(object@dendro_clusters) != max(dcluster)) {
            return("dendro_clusters must have the same number of leaves as the number of (non-negative) clusters")
        }
        ch<-.checkDendroClusterFormat(object@dendro_clusters)
        if(!is.logical(ch)) return(ch)
        
        #further checks that require full CE object:
        data.cl<-phylobase::tdata(object@dendro_clusters,type="all")
        if(any(!gsub("ClusterId","",na.omit(data.cl$ClusterIdDendro)) %in% as.character(object@clusterMatrix[,object@dendro_index]))) return("ClusterIdDendro information in dendrogram slot must match the corresponding cluster ids in clustering defined by dendro_index slot")
        if(any(!gsub("ClusterId","",na.omit(data.cl$ClusterIdMerge)) %in% as.character(object@clusterMatrix[,object@merge_index]))) return("ClusterIdMerge information in dendrogram slot must match the corresponding cluster ids in clustering defined by merge_index slot")
    }
    else{
        if(!is.null(object@dendro_samples)) return("dendro_clusters should not be null if dendro_samples is non-null") #if comment out now optional to have samples
    }
    if(!is.null(object@dendro_samples)){
        if(phylobase::nTips(object@dendro_samples) != NCOL(object)) {
            return("dendro_samples must have the same number of leaves as the number of samples")
        }
        ch<-.checkDendroSamplesFormat(object@dendro_samples)
        if(!is.logical(ch)) return(ch)
        
    }
    else{
        if(!is.null(object@dendro_clusters)) return("dendro_samples should not be null if dendro_clusters is non-null") #if commented out, makes it optional to have samples
    }
    
    return(TRUE)
}


.checkMerge<-function(object){
  ############
  ##Check merge related slots
  ############
  #these slots must all be same (either all empty or all set)
  emptyMergeMethod<-is.na(object@merge_method) || length(object@merge_method)==0
  emptyMergeIndex<- is.na(object@merge_index) || length(object@merge_index)==0
  if(emptyMergeIndex & !emptyMergeMethod) return("merge_index NA but merge_method has value")
  if(!emptyMergeIndex & emptyMergeMethod) return("if merge_index not NA, must have value for merge_method")
  emptyCutoff<-is.na(object@merge_cutoff) || length(object@merge_cutoff)==0
  if(emptyMergeIndex & !emptyCutoff) return("merge_index NA but merge_cutoff has value")
  if(!emptyMergeIndex & emptyCutoff) return("if merge_index not NA, must have value for merge_cutoff")
  if(emptyMergeIndex & !is.null(object@merge_nodeMerge)) return("merge_index NA but merge_nodeMerge has value")
  if(!emptyMergeIndex & is.null(object@merge_nodeMerge)) return("if merge_index not NA, must have value for merge_nodeMerge")
  
  #these slots can be set even if no merge, but not vice versa
  emptyDEMethod<-is.na(object@merge_demethod) || length(object@merge_demethod)==0
  if(!emptyMergeIndex & emptyDEMethod) return("if merge_index not NA, must have value for merge_demethod") 
  emptyDendroIndex<-is.na(object@merge_dendrocluster_index) || length(object@merge_dendrocluster_index)==0
  if(!emptyMergeIndex & emptyDendroIndex) return("if merge_index not NA, must have value for merge_dendrocluster_index")
  
  ##Check when there is actual merging:
  if(!emptyMergeIndex){
    if(object@merge_cutoff>1 || object@merge_cutoff<0) return("merge_cutoff should be between 0 and 1")
    if(object@merge_index==object@merge_dendrocluster_index) return("merge_index should not be same as merge_dendrocluster_index")
    if(!length(object@merge_method)==1) return("merge_method must be of length 1")
    
    #deal with possible FC
    baseMergeMethod<-sapply(strsplit(object@merge_method,"_"),.subset2,1)
    if(!baseMergeMethod %in% .validMergeMethods) return(paste("merge_method must be one of available merge methods:", paste(.validMergeMethods,collapse=",")," (with possibility of fold-change added to method name for 'adjP')"))
    allowMergeColumns<-c('Contrast','isMerged','mergeClusterId','NodeId')
    if(!identical(sort(colnames(object@merge_nodeMerge)),sort(allowMergeColumns)) ) {
      return(paste("merge_nodeMerge must have 5 columns and column names equal to:",paste(allowMergeColumns,collapse=",")))
    }
    if(!is.character(object@merge_nodeMerge[,"NodeId"])) return("'NodeId' column of merge_nodeMerge must be character")
    if(!is.character(object@merge_nodeMerge[,"Contrast"])) return("'Contrast' column of merge_nodeMerge must be character")
    if(!is.logical(object@merge_nodeMerge[,"isMerged"])) return("'isMerged' column of merge_nodeMerge must be logical")
    if(!is.numeric(object@merge_nodeMerge[,"mergeClusterId"]) & !all(is.na(object@merge_nodeMerge[,"mergeClusterId"]))) return("'mergeClusterId' column of merge_nodeMerge must be numeric")
    if(any(object@merge_nodeMerge[,"isMerged"])){
      wh<-which(object@merge_nodeMerge[,"isMerged"])
      if(all(is.na(object@merge_nodeMerge[wh,"mergeClusterId"]))) return("mergeClusterId entries of merge_nodeMerge cannot be all NA if there isMerged column that is TRUE")
    }    
    id<-object@merge_nodeMerge[,"mergeClusterId"]
    merg<-object@merge_nodeMerge[,"isMerged"]
    if(length(unique(na.omit(id))) != length(na.omit(id))) return("'mergeClusterId values in merge_nodeMerge not unique")
    if(any(!is.na(id) & !merg)) return("Cannot have values 'mergeClusterId' where 'isMerged' is FALSE")
    cl<-clusterMatrix(object)[,object@merge_index]
    if(any(!na.omit(id)%in% cl)) return("Values in 'mergeClusterId' not match cluster id values")
	
  }
  if(!is.null(object@merge_nodeProp)){
    if(emptyDendroIndex){return("merge_nodeProp is not NULL but merge_dendrocluster_index has no value")
      
    }
    if(length(object@merge_demethod)!=1 || !object@merge_demethod %in% .demethods)
      return(paste("merge_demethod must be one of:",paste(.demethods,collapse=",")))
		
    requireColumns<-c("NodeId","Contrast",.availMergeMethods)
    #need to allow for log fold change columns of adjP
    allCnames<-colnames(object@merge_nodeProp)
    whFC<-grep("adjP_",allCnames)
    whNode<-which(allCnames %in% c("NodeId","Contrast"))
    namesToCheck<-if(length(whFC)>0) allCnames[-whFC] else allCnames
    if(!all(requireColumns %in% namesToCheck))
    if(!identical(sort(namesToCheck),sort(requireColumns)) ) 
      return(paste("merge_nodeProp must be data.frame with at least",length(requireColumns),"columns that have column names equal to:",paste(requireColumns,sep="",collapse=",")))
    
    if(!is.character(object@merge_nodeProp[,"NodeId"])) return("'Node' column of merge_nodeProp must be character")
    if(!is.character(object@merge_nodeProp[,"Contrast"])) return("'Contrast' column of merge_nodeProp must be character")
    for(method in allCnames[-whNode]){
      if(!is.numeric(object@merge_nodeProp[,method])) return(paste(method,"column of merge_nodeProp must be numeric"))
    }
	
  }
  #Check that dendro node ids match those in merge tables.
  emptyDendro<-is.na(object@dendro_index) || length(object@dendro_index)==0
  if(!emptyDendroIndex && !emptyDendro && object@merge_dendrocluster_index==object@dendro_index){
	dendroNodes<-phylobase::tdata(object@dendro_clusters,type="internal")[,"NodeId"]
	nodes<-object@merge_nodeMerge[,"NodeId"]
	if(!all(sort(nodes) == sort(dendroNodes))) return("Not all of nodes in dendro_clusters have a value in merge_nodeMerge")
	nodes<-object@merge_nodeProp[,"NodeId"]
	if(!all(sort(nodes) == sort(dendroNodes))) return("Not all of nodes in dendro_clusters have a value in merge_nodeProp")		
  }
  return(TRUE)
}
.checkCoClustering<-function(object){
    ## FIXME: this needs to be changed if now return NxB matrix. 
    ## For now, commenting out the check entirely. Can be anything...
    ## Check co-clustering
    # if(!is.null(object@coClustering) &&
    #    (NROW(object@coClustering) != NCOL(object@coClustering)
    #     | NCOL(object@coClustering) != NCOL(object))) {
    #     return("`coClustering` must be a sample by sample matrix.")
    # }
    typeCoCl<-.typeOfCoClustering(object)
    if(typeCoCl=="indices"){
        if(!all(object@coClustering %in% 1:nClusterings(object)) ) return("CoClustering slot is a vector, but doesn't match indices of clusterMatrix of the object")
    }
    return(TRUE)
}

.checkPrimaryIndex<-function(object){
    ############
    ## Check primary index
    ############
    if(!all(is.na(object@clusterMatrix))){ #what does this mean, how can they be all NA?
        #check primary index
        if(length(object@primaryIndex) != 1) {
            if(length(object@primaryIndex) == 0) return("If more than one set of clusterings, a primary cluster must be specified.")
            if(length(object@primaryIndex) > 0) return("Only a single primary index may be specified")
        }
        if(object@primaryIndex > NCOL(object@clusterMatrix) | object@primaryIndex < 1) {
            return("`primaryIndex` out of bounds.")
        }
    }
    return(TRUE)
}

.checkClusterTypes<-function(object){
    ############
    ## Check clusterTypes
    ############
    if(!all(is.na(object@clusterMatrix))){ #what does this mean, how can they be all NA?
        if(NCOL(object@clusterMatrix) != length(object@clusterTypes)) {
            return("`clusterTypes` must be the same length as NCOL of `clusterMatrix`.")
        }
        if(!is.null(names(object@clusterTypes))) return("clusterTypes should not have names")
    }
    return(TRUE)
}

#this check is just to check object, not whether matches clusters
.checkClusterLegendList<-function(clusterLegend,allowNames=TRUE,reqNames=c("clusterIds","color","name")){
    if(!is.list(clusterLegend)) return("clusterLegend must be a list")
    if(!allowNames & !is.null(names(clusterLegend))) return("clusterLegend should not have names")
    testIsMatrix <- sapply(clusterLegend, function(x) {!is.null(dim(x))})
    if(!all(testIsMatrix)) {
        return("Each element of `clusterLegend` list must be a matrix")
    }
    testNames<-sapply(clusterLegend,function(x){
        if(is.null(colnames(x))) return(FALSE)
        else{
            if(!all(reqNames %in% colnames(x))) return(FALSE)
            else return(TRUE)
        }
    })
    if(!all(testNames)) {
        return(paste("each element of `clusterLegend` must be matrix with column names defined, with at a minimum the names names:", paste(reqNames,collapse=",")))
    }
    testColorCols1 <- sapply(clusterLegend, function(x){is.character(x)})
    if(!all(testColorCols1)) {
        return("each element of `clusterLegend` must be matrix of character values")
    }
    return(TRUE)
}
#this check checks both object (calls .checkClusterLegend) and whether matches clusters
#make so can call on arbitrary clusterLegend...not need to be CE object
.checkClustersWithClusterLegend<-function(clusters,clusterLegend){
    #check structure of clusterLegend list -- for CE object, can't have names.
    legendCheck<-.checkClusterLegendList(clusterLegend,allowNames=FALSE,reqNames=c("clusterIds","color","name"))
    if(!is.logical(legendCheck)) return(legendCheck)
    
    #check matches clusters
    if(length(clusterLegend) != NCOL(clusters)) {
        return("`clusterLegend` must be list of same length as NCOL of
               `clusterMatrix`")
    }
    testColorRows <- sapply(clusterLegend, function(x){nrow(x)})
    testClusterMat <- apply(clusters, 2, function(x) {length(unique(x))})
    if(!all(testColorRows == testClusterMat)) {
        return("each element of `clusterLegend` must be matrix with number of rows equal to the number of clusters (including -1 or -2 values) in `clusterMatrix`")
    }
    testColorCols1 <- sapply(seq_along(clusterLegend), function(ii){
        col<-clusterLegend[[ii]]
        x<-clusters[,ii]
        y<-col[,"clusterIds"]
        if(is.numeric(x)){
            y<-as.numeric(y)
        }
        return(all(y %in% x) & all(x %in% y))
    })
    if( !all(testColorCols1)) {
        return("each element of `clusterLegend` must be matrix with column
             `clusterIds` matching the corresponding
             clusterMatrix values")
    }
    
    return(TRUE)
}
.checkClusterLegend<-function(object){
    if(!all(is.na(object@clusterMatrix))){ #what does this mean, how can they be all NA?
        return(.checkClustersWithClusterLegend(clusters=object@clusterMatrix,clusterLegend=object@clusterLegend))
    }
    return(TRUE)
}

.checkOrderSamples<-function(object){
    ####
    #test orderSamples
    ####
    if(length(object@orderSamples)!=NCOL(assay(object))) {
        return("`orderSamples` must be of same length as number of samples
	       (NCOL(assay(object)))")
    }
    if(any(!object@orderSamples %in% seq_len(NCOL(assay(object))))) {
        return("`orderSamples` must be values between 1 and the number of samples.")
    }
    return(TRUE)
}

#' @rdname ClusterFunction-class
#' @export
#' @aliases internalFunctionCheck
#' @examples
#' #Use internalFunctionCheck to check possible function
#' goodFUN<-function(inputMatrix,k,cluster.only,...){
#'	cluster::pam(x=t(inputMatrix),k=k,cluster.only=cluster.only)
#' }
#' #passes internal check
#' internalFunctionCheck(goodFUN,inputType=c("X","diss"),
#'    algorithmType="K",outputType="vector")
#' myCF<-ClusterFunction(clusterFUN=goodFUN, inputType="X",
#'    algorithmType="K", outputType="vector")
#' #doesn't work, because haven't made results return vector when cluster.only=TRUE
#' badFUN<-function(inputMatrix,k,cluster.only,...){cluster::pam(x=inputMatrix,k=k)}
#' internalFunctionCheck(badFUN,inputType=c("X","diss"),
#'    algorithmType="K",outputType="vector")
#' @details \code{internalFunctionCheck} is the function that is called by the 
#'   validity check of the \code{ClusterFunction} constructor (if 
#'   \code{checkFunctions=TRUE}). It is available as an S3 function for the user
#'   to be able to test their functions and debug them, which is difficult to do
#'   with a S4 validity function.
#' @return Returns a logical value of TRUE if there are no problems. If there is 
#'   a problem, returns a character string describing the problem encountered.
internalFunctionCheck<-function(clusterFUN,inputType,algorithmType,outputType){
    #--- Make small data
    N<-20
    #--- Set parameters
    if(algorithmType=="01") argList<-list(alpha=.5)	
    if(algorithmType=="K") argList<-list(k=2)	
    argList<-c(argList,list(cluster.only=TRUE,checkArgs=FALSE))
    #--- Run function on small data
    if("X" %in% inputType){
        set.seed(2851)
        x<-matrix(rnorm(N*3),ncol=N,nrow=3)
        test<-try(do.call(clusterFUN, 
            c(list(inputMatrix=x,inputType="X"), argList)),silent=TRUE)
        if(inherits(test,"try-error")) return(paste("function test fails with inputType='X'. ",test[1]))
    }
    if("diss" %in% inputType){
        set.seed(2851)
        diss<-matrix(runif(N^2,min=0,max=0.5),ncol=N,nrow=N)
        diss<-diss + t(diss)
        diag(diss)<-0
        test<-try(do.call(clusterFUN,
            c(list(inputMatrix=diss,inputType="diss"),argList)),silent=TRUE)
        if(inherits(test,"try-error")) return(paste("function test fails with input inputType='diss'.",test[1]))
    }
    if("cat" %in% inputType){
        set.seed(2851)
        x<-matrix(sample(x=1:4,size=N*3,replace=TRUE),ncol=N,nrow=3)
        x<-cbind(x,x)
        x<-x[,sample(1:ncol(x))]
        test<-try(do.call(clusterFUN,
            c(list(inputMatrix=x,inputType="cat"),argList)),silent=TRUE)
        if(inherits(test,"try-error")) return(paste("function test fails with inputType='cat'. ",test[1]))
    }
    if(outputType=="vector"){
        if(length(test)!=N) return("clusterFUN does not return a vector equal to the number of observations")
    }
    return(TRUE)
}


.checkHasArgs<-function(FUN,requiredArgs){
    funArgs<-names(as.list(args(FUN)))
    all(requiredArgs %in% funArgs)
}

#' @name clusterDendrogram
#' @title Accessing and manipulating the dendrograms
#' @rdname clusterDendrogram
#' @description These functions are for accessing and manipulating the
#'   dendrograms stored in a \code{ClusterExperiment} object. We also document
#'   the required format of these dendrograms here.
#' @param x a ClusterExperiment object
#' @section The Stored Dendrograms
#' @details Two dendrograms are stored in a \code{ClusterExperiment} object. One
#'   is a dendrogram that describes the hierarchy between the clusters
#'   (\code{@dendro_clusters}), and the other is a dendrogram that extends that
#'   hierarchy to include the clusters (\code{@dendro_samples}). The clustering
#'   that is used to make these hierarchies is saved in as well
#'   (\code{@dendro_index})
#' @details The dendrograms stored in a \code{ClusterExperiment} object are
#'   required to be a \code{\link[phylobase]{phylo4d-class}} from the package
#'   \code{phylobase} (which uses the basic format of the S3 class
#'   \code{\link[ape]{phylo}} in the \code{ape} package to store the edges;
#'   \code{phylobase} makes it a S4 class with some useful helpers). This class
#'   allows storage of a data.frame of information corresponding to information
#'   on each node (see \code{\link[phylobase]{tdata}}).
#' @details Additional requirements are made of these dendrograms to be a valid
#'   for the slots of the \code{ClusterExperiment} class, described below,
#'   regarding the data that must be stored with it and the labels which can be
#'   assigned. Possible dendrograms can be checked for validity with the
#'   function \code{checkDendrogram}. The reason for the restrictions on the
#'   labels is so as to not duplicate storage of the names, see below
#'   descriptions for where to save user-defined names.
#' @section Cluster Hierarchy
#' @details \itemize{ 
#' \item{Labels}{The cluster dendrogram can only have labels
#'   on the \emph{internal} nodes. Labels on the internal nodes of the cluster
#'   dendrogram can be set by the user (the function \code{nodeLabels<-} is
#'   defined to work on a \code{ClusterExperiment} object to make this easy).
#'   The tips of the cluster dendrogram, corresponding to the clusters, cannot
#'   have labels; users can set the labels (e.g. for plotting, etc) in the
#'   \code{\link{clusterLegend}} slot using the function
#'   \code{\link{renameClusters}}.} 
#' \item{Data}{ The cluster hierarchy must have
#'   data stored with it that has the following columns (additional ones are
#'   allowed):
#'   \itemize{ 
#'   \item{NodeId}{The permanent node id for the node. Must
#'   be of the format "NodeIdX" where "X" is a integer.} 
#'   \item{Position}{The
#'   type of node, in terms of its position. The internal nodes should have the
#'   values "cluster hierarchy node" while the tips should have "cluster
#'   hierarchy tip".} 
#'   \item{ClusterIdDendro}{Only for tips of dendrogram, should
#'   have the id that corresponds to its cluster in the clustering of the
#'   @dendro_index. Of the form "ClusterIdX", where "X" is the internal cluster
#'   id (see \code{\link{clusterLegend}}). Internal nodes should have NA
#'   values.} 
#'   \item{ClusterIdMerge}{The id that corresponds to the cluster in
#'   the clustering of the @merge_index, if it exists. Of the form "ClusterIdX",
#'   where "X" is the internal cluster id (see \code{\link{clusterLegend}}} } }
#'   }
#' @section Sample Hierarchy
#' @details \itemize{
#' \item{Labels}{The sample dendrogram is not allowed to have ANY labels. The
#' names for those nodes that correspond to the cluster hierarchy will be pulled
#' from the names in the cluster hierarchy for plotting, etc. and should be set
#' there (see above). Sample names for the tips of the tree will be pulled from
#' \code{colnames} of the object and should be set there. }
#' \item{Data}{ 
#' The cluster hierarchy must have data stored with it that has the following
#' columns (additional ones are allowed):
#'  \itemize{
#'   \item{NodeId}{For those nodes that correspond to a node in the cluster
#'   hierarchy, should have its permanent node id in this column. Other nodes
#'   should be NA.}
#'   \item{Position}{The type of node, in terms of its position. The internal
#'   nodes should have the values "cluster hierarchy node" while the tips should
#'   have "cluster hierarchy tip".}
#'   \item{SampleIndex}{Only for tips of dendrogram, the index of the sample at
#'   that tip to the samples in the object.}
#'  }
#' } 
#' }
#' @section Helper Functions
#' @seealso \code{\link{makeDendrogram}},
#'   \code{\link[phylobase]{phylo4d-class}}, \code{\link[ape]{phylo}}
#' @return \code{clusterDendrogram} returns the dendrogram describing the
#'   clustering hierarchy.
#' @importMethodsFrom phylobase nodeLabels nodeLabels<- nTips nNodes
#' @export
#' @examples
#' data(rsecFluidigm)
#' # retrieve the dendrogram of the clusters:
#' head(clusterDendrogram(rsecFluidigm),5)
#' # retrieve the dendrogram of the samples:
#' head(sampleDendrogram(rsecFluidigm),5)
#' # Return # internal nodes from cluster hierarchy
#' nInternalNodes(rsecFluidigm)
#' # Return # tips from cluster hierarchy (i.e. # clusters)
#' nTips(rsecFluidigm)
#' # Return internal node ids
#' nodeIds(rsecFluidigm,type="internal")
#' # Labels assigned to internal nodes
#' nodeLabels(rsecFluidigm)
#' # Assign new labels to the internal nodes of the cluster hierarchy
#' l1<-paste("A", 1:nInternalNodes(rsecFluidigm),sep=":")
#' names(l1)<-nodeIds(rsecFluidigm,type="internal")
#' nodeLabels(rsecFluidigm)<-l1
#' nodeLabels(rsecFluidigm)
#' @aliases clusterDendrogram,ClusterExperiment-method
setMethod(
  f = "clusterDendrogram",
  signature = "ClusterExperiment",
  definition = function(x) {
    return(x@dendro_clusters)
  }
)

#' @rdname clusterDendrogram
#' @return \code{sampleDendrogram} returns the dendrogram that expands the
#'   cluster hierarchy to the samples.
#' @export
#' @aliases sampleDendrogram
setMethod(
  f = "sampleDendrogram",
  signature = "ClusterExperiment",
  definition = function(x) {
    return(x@dendro_samples)
  }
)


#' @rdname clusterDendrogram
#' @return \code{nInternalNodes} returns the number of \emph{internal} nodes of
#'   the cluster hierarchy.
#' @export
#' @aliases nInternalNodes
setMethod(
  f = "nInternalNodes",
  signature = "ClusterExperiment",
  definition = function(x) {
    return(length(phylobase::nodeLabels(x@dendro_clusters)))
  }
)

#' @rdname clusterDendrogram
#' @return \code{nTips} returns the number of tips of the cluster hierarchy
#'   (same as number of non-negative clusters in the dendrogram clustering)
#' @export
#' @aliases nTips
setMethod(
  f = "nTips",
  signature = "ClusterExperiment",
  definition = function(x) {
    return(phylobase::nTips(x@dendro_clusters))
  }
)

#' @rdname clusterDendrogram
#' @return \code{nNodes} returns the number of total nodes of the cluster hierarchy
#' @export
#' @aliases nNodes
setMethod(
  f = "nNodes",
  signature = "ClusterExperiment",
  definition = function(x) {
    return(phylobase::nNodes(x@dendro_clusters))
  }
)

#' @rdname clusterDendrogram
#' @return \code{nodeLabels<-} sets the node labels of the \emph{cluster}
#' dendrogram
#' @export
#' @details For setting the node labels of the cluster dendrogram via 
#' \code{nodeLabels<-}, the replacement value has to have names that match the 
#' internal ids of the cluster dendrogram (the \code{NodeId} column). 
#' @aliases nodeLabels<-
#' @param value replacement value for \code{nodeLabels}. See details.
#' @param ... additional options passed to \code{nodeLabels<-} (ignored)
setMethod(
  f = "nodeLabels<-",
  signature = signature(x="ClusterExperiment", value="ANY"),
  definition = function(x, ..., value) {
	if(!is.null(names(value))){
		#match them
		m<-try(.matchToDendroData(inputValue=names(value),dendro=x@dendro_clusters,matchColumn="NodeId",returnColumn="NodeIndex"),silent=TRUE)
		if(inherits(m,"try-error")) stop("names of replace vector do not match in internal node id:",m)
		phylobase::labels(x@dendro_clusters)[m]<-value
		ch<-.checkDendrogram(x)
	    if(!is.logical(ch)) stop(ch)
		return(x)
	}
	else{stop("the replacement value needs to have names that match the internal node ids")}
  }
)

#' @rdname clusterDendrogram
#' @return \code{checkClusterDendrogram} checks if a \code{phylo4d} objects are
#'   valid for the cluster and sample dendrogram slots of the given
#'   \code{ClusterExperiment} object. Returns \code{TRUE} if there are no
#'   problems. Otherwise creates error.
#' @param dendroCluster a \code{phylo4d} to be check as for being cluster hierarchy
#' @param dendroSample a \code{phylo4d} to be check as for being cluster hierarchy
#' @inheritParams getClusterIndex
#' @export
#' @aliases checkDendrogram
setMethod(
  f = "checkDendrogram",
  signature = signature(x="ClusterExperiment",dendroCluster="phylo4d",dendroSample="phylo4d"),
  definition = function(x,dendroCluster,dendroSample,whichCluster="dendro") {
	  whCl<-getSingleClusterIndex(x,whichCluster)
	  x@dendro_clusters<-dendroCluster
	  x@dendro_samples<-dendroSample
	  x@dendro_index<-whCl
	  ch<-.checkDendrogram(x)
      if(!is.logical(ch)) stop(ch)
	  else return(TRUE)
  }
)

#' @rdname clusterDendrogram
#' @return \code{nodeLabels} returns the node labels of the \emph{cluster}
#' dendrogram
#' @export
#' @aliases nodeLabels
setMethod(
  f = "nodeLabels",
  signature = signature(x="ClusterExperiment"),
  definition = function(x) {
	phylobase::nodeLabels(x@dendro_clusters)
  }
)

#' @rdname clusterDendrogram
#' @return \code{nodeIds} returns the internal (permanent) node ids of the
#'   \emph{cluster} dendrogram
#' @param type the type of node to return results from. One of "all",
#'   "internal", and "tip".
#' @export
#' @aliases nodeIds
setMethod(
  f = "nodeIds",
  signature = signature(x="ClusterExperiment"),
  definition = function(x,type=c("all","internal","tip")) {
	  phylobase::tdata(x@dendro_clusters,type=type)[,"NodeId"]
  }
)

#' @rdname clusterDendrogram
#' @return \code{convertToDendrogram} returns the sample dendrogram converted to
#'   a \code{\link[stats]{dendrogram}} class.
#' @seealso \code{\link[stats]{dendrogram}}
#' @export
#' @aliases convertToDendrogram
setMethod(
	f="convertToDendrogram",
	signature = "ClusterExperiment",
	definition=function(x){
		dendroId <- .setNodeLabels(x,labelType="id")$dendro_samples
    return(.convertToDendrogram(dendroId,tipNames=colnames(x)))
		
	})
#' @importFrom ape as.hclust.phylo
#' @importFrom stats as.dendrogram
.convertToDendrogram<-function(x,tipNames=NULL){
	if(inherits(x,"dendrogram") )return(x)
	if(inherits(x,"phylo4")){
		x<-.convertToPhyClasses(x,"phylo")
	} 
	if(inherits(x,"phylo")){
		if(any(is.na(as.numeric(x$tip.label)))) stop("can only convert to dendrogram if sample indices are the tip.labels")
		x<-try(ape::as.hclust.phylo(x),FALSE)
		if(inherits(x, "try-error")) stop("coding error -- could not convert to hclust object. Reported error:",x)
		#need to convert integers in $order and (negative) entries in $merge into the values of $labels
		newOrder<-as.integer(as.numeric(x$labels))
		if(!all(sort(newOrder)==seq_along(newOrder))) stop("could not convert hclust object because dendrogram does not have all consecutive sample indices as tip labels.")
		m<-match(abs(x$merge[x$merge<0]),x$order)
		newMerge<-x$merge
		newMerge[x$merge<0]<-newOrder[m]
		newMerge[x$merge<0]<-newMerge[x$merge<0]*sign(x$merge[x$merge<0])
		newMerge<-matrix(as.integer(newMerge),ncol=2)
		x$order<-newOrder
		x$merge<-newMerge
		if(is.null(tipNames)) x$labels<-NULL
		else x$labels<-tipNames[x$order]
	}
	if(inherits(x,"hclust")){
		x<-try(stats::as.dendrogram(x),FALSE)
		if(inherits(x, "try-error")) stop("could not convert from hclust to dendrogram object. Reported error:",x)
		if(!is.integer(unlist(x))) stop("coding error -- did not create integer valued dendrogram")
		return(x)
	}
	else{ stop("input x is not of hclust, phylo4 or phylo class")}
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Search pairs of samples that co-cluster across subsamples
#'
#' Assume that our input is a matrix, with N columns and B rows
#' (the number of subsamples), storing integers -- the cluster labels.
#'
#' @param clusterings a matrix with the cluster labels
#'
#' @return A matrix with the co-clusters, but only the lower triangle
#' is populated.
search_pairs <- function(clusterings) {
    .Call(`_clusterExperiment_search_pairs`, clusterings)
}

makeHashKey <- function(i, j, delim) {
    .Call(`_clusterExperiment_makeHashKey`, i, j, delim)
}

splitHashKey <- function(s, delim) {
    .Call(`_clusterExperiment_splitHashKey`, s, delim)
}

subsampleLoop <- function(clList, N) {
    .Call(`_clusterExperiment_subsampleLoop`, clList, N)
}

#' @name plotFeatureScatter
#' @title Plot scatter plot of feature values colored by cluster
#' @description Plot a scatter plot of the (transformed) values for a set of
#'   gene expression values, colored by cluster
#' @aliases plotFeatureScatter plotFeatureScatter,ClusterExperiment,character-method
#' @param legendLocation character value passed to \code{location} argument of 
#'  \code{plotClusterLegend} indicating where to put the legend. If NA, legend 
#'  will not be plotted.
#' @param features the indices of the features (either numeric or character 
#'  matching rownames of object) to be plotted.
#' @param jitterFactor numeric. If NA, no jittering is done. Otherwise, passed to 
#' \code{factor} of function \code{\link[base]{jitter}}
#'  (useful for low counts)
#' @inheritParams plotFeatureBoxplot
#' @rdname plotFeatureScatter
#' @return returns invisibly the results of \code{\link[graphics]{pairs}} or \code{\link[graphics]{plot}} command.
#' @examples
#' data(simData)
#' #Create a ClusterExperiment object
#' cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reducedDim="PCA",
#'    clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
#'    removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)
#' #give names to the clusters
#' cl<-renameClusters(cl, whichCluster=1, 
#'    value=letters[1:nClusters(cl)[1]])
#' plotFeatureScatter(cl,feature=1:2,pch=19,legendLocation="topright")
#' @export
setMethod(
  f = "plotFeatureScatter",
  signature = signature(object = "ClusterExperiment",features="character"),
  definition = function(object, features,...)
  {
    m<-match(features,rownames(object))
    if(any(is.na(m))) stop("not all of features match one of the rownames of this object")
    else invisible(plotFeatureScatter(object,features=m,...))
  })

#' @rdname plotFeatureScatter
#' @export
setMethod(
  f = "plotFeatureScatter",
  signature = signature(object = "ClusterExperiment",features="numeric"),
  definition = function(object, features, whichCluster="primary", plotUnassigned=TRUE,unassignedColor="grey",missingColor="white",whichAssay=1,legendLocation=NA,jitterFactor=NA,...)
  {
		if(length(features)<2) stop("plotFeatureScatter requires at least 2 features")
    whCl<-getSingleClusterIndex(object,whichCluster,list(...))
 
    #get data:
    dat<-transformData(object, whichAssay=whichAssay)[features,]
		if(!is.na(jitterFactor)) dat<-jitter(dat,factor=jitterFactor)
		if(is.null(rownames(dat))){
			rownames(dat)<-paste("Feature",features,sep=" ")
		}
    clLegend<-clusterLegend(object)[[whCl]]
		cl<-clusterMatrix(object)[,whCl]
		whLegUn<-which(as.numeric(clLegend[,"clusterIds"])<0)
		if(!plotUnassigned){
			whUnassign<-which(cl<0)
			if(length(whUnassign)>0){
				cl<-cl[-whUnassign]
				dat<-dat[,-whUnassign]
				clLegend<-clLegend[-whLegUn,,drop=FALSE]

			}	
		}
		else{
			if(length(whLegUn)>0){
				wh1<-which(as.numeric(clLegend[,"clusterIds"])== -1)
				if(length(wh1)>0 & !is.null(unassignedColor))
					clLegend[wh1,"color"]<-unassignedColor
				wh2<-which(as.numeric(clLegend[,"clusterIds"])== -2)
				if(length(wh2)>0 & !is.null(missingColor))
					clLegend[wh2,"color"]<-missingColor
				
			}
		}
    uniqueNames<-length(unique(clLegend[,"name"]))==nrow(clLegend)


		m<-match(as.character(cl),clLegend[,"clusterIds"])
		cols<-clLegend[m,"color"]
		if(length(features)==2){
			invisible(graphics::plot(t(dat),col=cols,bg=cols,...))
			if(!is.na(legendLocation)) 
				graphics::legend(x=legendLocation,legend=clLegend[,"name"],fill=clLegend[,"color"])
		}
		else{
			invisible(graphics::pairs(t(data.matrix(dat)),col=cols,bg=cols,...))
		}
  })

## ----GlobalOptions, results="hide", include=FALSE, cache=FALSE----------------
knitr::opts_chunk$set(fig.align="center", cache=FALSE, cache.path = "largeDatasets_cache/",  fig.path="largeDatasets_figure/",error=FALSE, #make it stop on error
fig.width=6,fig.height=6,autodep=TRUE,out.width="600px",out.height="600px",
message=FALSE)
#knitr::opts_knit$set(stop_on_error = 2L) #really make it stop
#knitr::dep_auto()

setGeneric(".matchToStats", function(object,...) { standardGeneric(".matchToStats")})

setGeneric("filterNames", function(object,...) { standardGeneric("filterNames")})
setGeneric("filterData", function(object,...) { standardGeneric("filterData")})
setGeneric("defaultNDims",function(object,...){standardGeneric("defaultNDims")})
setGeneric(name = "makeFilterStats", function(object,...){ standardGeneric("makeFilterStats")})
setGeneric(name="makeReducedDims", function(object,...){ standardGeneric("makeReducedDims")})

setGeneric(name="plotClusterLegend", function(object,...){standardGeneric("plotClusterLegend")})
setGeneric("plotFeatureBoxplot", function(object,feature,...) { standardGeneric("plotFeatureBoxplot")})
setGeneric("plotFeatureScatter", function(object,features,...) { standardGeneric("plotFeatureScatter")})

setGeneric( "tableClusters", function(object,...){ standardGeneric("tableClusters") })
setGeneric( "plotClustersTable", function(object,...){ standardGeneric("plotClustersTable") })
setGeneric("bubblePlot", function(propTable,sizeTable,...) { standardGeneric("bubblePlot")})

setGeneric(name = "assignUnassigned", def=function(object,...){ standardGeneric("assignUnassigned") })
setGeneric( name = "removeUnassigned", def = function(object,...) {
	standardGeneric("removeUnassigned")})
setGeneric(name="subsetByCluster", def = function(x,...) {
	standardGeneric("subsetByCluster")})
setGeneric(name = "getReducedData", def=function(object,...){ standardGeneric("getReducedData") })

###Merge Info
setGeneric("getMergeCorrespond", def=function(x,...){ standardGeneric("getMergeCorrespond")})
setGeneric(name = "nodeMergeInfo", def=function(x,...){ standardGeneric("nodeMergeInfo") })
setGeneric(name = "mergeClusterIndex",def=function(x,...){ standardGeneric("mergeClusterIndex")})
setGeneric(name = "mergeMethod",def=function(x,...){standardGeneric("mergeMethod")})
setGeneric(name = "mergeCutoff", def=function(x,...){ standardGeneric("mergeCutoff")})
setGeneric(name="eraseMergeInfo",def=function(x,...){ standardGeneric("eraseMergeInfo")})
setGeneric(name = "nClusters", function(x,...){ standardGeneric("nClusters")})

setGeneric("plotClustersWorkflow", function(object,...){ standardGeneric("plotClustersWorkflow")}
)
setGeneric(
	name="plotContrastHeatmap",
	def=function(object,...){
		standardGeneric("plotContrastHeatmap")
	}
)
setGeneric(
    name = "RSEC",
    def = function(x, ...) {
        standardGeneric("RSEC")
    }
)
setGeneric(
  name = "subsampleClustering",
  def = function(clusterFunction, ...) {
    standardGeneric("subsampleClustering")
  }
)
setGeneric(
  name = "mainClustering",
  def = function(clusterFunction, ...) {
    standardGeneric("mainClustering")
  }
)
setGeneric(
  name = "seqCluster",
  def = function(clusterFunction, ...) {
    standardGeneric("seqCluster")
  }
)
setGeneric( name = "clusterSingle", 
    def = function(inputMatrix, ...) { standardGeneric("clusterSingle") })
setGeneric(
  name = "setToCurrent",
  def = function(x,  ...) {
    standardGeneric("setToCurrent")
  }
)
setGeneric(
  name = "setToFinal",
  def = function(x,  ...) {
    standardGeneric("setToFinal")
  }
)
setGeneric(
  name = "plotCoClustering",
  def = function(data,  ...) {
    standardGeneric("plotCoClustering")
  }
)
setGeneric(
    name="clusterContrasts",
    def=function(cluster,...){
        standardGeneric("clusterContrasts")
    }
)
setGeneric(
  name = "convertClusterLegend",
  def = function(object,  ...) {
    standardGeneric("convertClusterLegend")
  }
)

setGeneric(
  name = "makeDendrogram",
  def = function(x,  ...) {
    standardGeneric("makeDendrogram")
  }
)

setGeneric("clusterLegend", function(x) { standardGeneric("clusterLegend")})
setGeneric("clusterLegend<-", function(object, value){standardGeneric("clusterLegend<-") })
setGeneric("renameClusters", function(object,value,...) { standardGeneric("renameClusters")})
setGeneric("recolorClusters", function(object,value,...) { standardGeneric("recolorClusters")})
setGeneric("sampleDendrogram", function(x) { standardGeneric("sampleDendrogram")})
setGeneric("clusterDendrogram", function(x) { standardGeneric("clusterDendrogram")})
setGeneric("convertToDendrogram",function(x){standardGeneric("convertToDendrogram")})
setGeneric("nodeIds",function(x,type){standardGeneric("nodeIds")})
setGeneric("checkDendrogram",function(x,dendroCluster,dendroSample,...){standardGeneric("checkDendrogram")})
setGeneric("nInternalNodes",function(x){standardGeneric("nInternalNodes")})
# #Have to recreate these generics, because phylobase's are nonstandardGenericFunctions because used braces.
# setGeneric("nodeLabels",function(x){standardGeneric("nodeLabels")})
# setGeneric("nodeLabels<-",function(x,...,value){standardGeneric("nodeLabels<-")})
# setGeneric("tipLabels",function(x){standardGeneric("tipLabels")})
# setGeneric("nNodes",function(x){standardGeneric("nNodes")})
# setGeneric("nTips",function(x){standardGeneric("nTips")})

setGeneric("getClusterIndex",function(object,...){standardGeneric("getClusterIndex")})
setGeneric("getSingleClusterIndex",function(object,...){standardGeneric("getSingleClusterIndex")})
setGeneric(
    name = "orderSamples",
    def = function(x) {
        standardGeneric("orderSamples")
    }
)

setGeneric(
    name = "orderSamples<-",
    def = function(object, value) {
        standardGeneric("orderSamples<-")
    }
)

setGeneric(
  name = "clusterLabels",
  def = function(x) {
    standardGeneric("clusterLabels")
  }
)

setGeneric(
    name = "clusterLabels<-",
    def = function(object, value) {
        standardGeneric("clusterLabels<-")
    }
)
setGeneric(
	name="getClusterManyParams",
	def=function(x,...){
		standardGeneric("getClusterManyParams")
	})
setGeneric(
  name = "nClusterings",
  def = function(x) {
    standardGeneric("nClusterings")
  }
)

setGeneric(
  name = "nFeatures",
  def = function(x) {
    standardGeneric("nFeatures")
  }
)

setGeneric(
  name = "nSamples",
  def = function(x) {
    standardGeneric("nSamples")
  }
)

setGeneric(
  name = "workflowClusters",
  def = function(x,iteration=0) {
    standardGeneric("workflowClusters")
  }
)

setGeneric(
  name = "workflowClusterDetails",
  def = function(x) {
    standardGeneric("workflowClusterDetails")
  }
)

setGeneric(
  name = "workflowClusterTable",
  def = function(x) {
    standardGeneric("workflowClusterTable")
  }
)

setGeneric(
  name = "clusterMany",
  def = clusterMany <- function(x, ... ) {
    standardGeneric("clusterMany")
  }
)

setGeneric(
  name="plotDendrogram",
  def=function(x,...)
    {
    standardGeneric("plotDendrogram")
  }
)

setGeneric( "plotClusters", function(object, ...) { standardGeneric("plotClusters")})
setGeneric( name="plotBarplot", def=function(object, ...) { standardGeneric("plotBarplot") })
setGeneric("plotReducedDims",function(object,...){ standardGeneric("plotReducedDims")})

setGeneric(
  name="plotHeatmap",
  def=function(data,...)
  {
    standardGeneric("plotHeatmap")
  }
)

setGeneric(
  name = "transformation",
  def = function(x) {
    standardGeneric("transformation")
  }
)
setGeneric(
  name = "transformation<-",
  def = function(object, value) {
    standardGeneric("transformation<-")
  }
)
setGeneric(
  name = "transformData",
  def = function(object,...) {
    standardGeneric("transformData")
  }
)
setGeneric("colDataClusters",function(object,...){standardGeneric("colDataClusters")})
setGeneric("addToColData",function(object,...){standardGeneric("addToColData")})
setGeneric("clusterMatrix", function(x,...) { standardGeneric("clusterMatrix")})
setGeneric("clusterMatrixNamed", function(x,...) { standardGeneric("clusterMatrixNamed")})
setGeneric("clusterMatrixColors", function(x,...) { standardGeneric("clusterMatrixColors")})
setGeneric("primaryCluster", function(x) { standardGeneric("primaryCluster")})
setGeneric("primaryClusterIndex", function(x) {standardGeneric("primaryClusterIndex")})
setGeneric("primaryClusterLabel", function(x) {standardGeneric("primaryClusterLabel")})
setGeneric("primaryClusterType", function(x) {standardGeneric("primaryClusterType")})
setGeneric("primaryClusterNamed",function(x) { standardGeneric("primaryClusterNamed")})
setGeneric(name = "primaryClusterIndex<-", def = function(object, value) {standardGeneric("primaryClusterIndex<-")})
setGeneric(name = "dendroClusterIndex",def = function(x) {standardGeneric("dendroClusterIndex")})
setGeneric( name = "coClustering", def = function(x) { standardGeneric("coClustering")})
setGeneric(
  name = "coClustering<-",
  def = function(object, value) {
    standardGeneric("coClustering<-")
  }
)

setGeneric(
  name = "clusterTypes",
  def = function(x) {
    standardGeneric("clusterTypes")
  }
)

setGeneric(
    name = "clusterTypes<-",
    def = function(object,value) {
        standardGeneric("clusterTypes<-")
    }
)


setGeneric( name = "removeClusters", def = function(x, ...) {
    standardGeneric("removeClusters")})
setGeneric( name = "addClusterings", def = function(x, y,...) {
    standardGeneric("addClusterings")})
setGeneric(name = "removeClusterings", def = function(x,...) {
    standardGeneric("removeClusterings")})
setGeneric( name = "clusteringInfo", def = function(x) { standardGeneric("clusteringInfo")})
setGeneric( name = "makeConsensus", def = function(x, ...) {
    standardGeneric("makeConsensus")})




setGeneric(
  name = "getBestFeatures",
  def = function(x, ...) {
    standardGeneric("getBestFeatures")
  }
)

setGeneric(
  name = "mergeClusters",
  def = function(x, ...) {
    standardGeneric("mergeClusters")
  }
)
setGeneric(
  name = "getBuiltInFunction",
  def = function(object, ...) {
    standardGeneric("getBuiltInFunction")
  }
)

 
setGeneric(
  name = "requiredArgs",
  def = function(object, ...) {
    standardGeneric("requiredArgs")
  }
)
setGeneric(
  name = "algorithmType",
  def = function(object, ...) {
    standardGeneric("algorithmType")
  }
)
setGeneric(
  name = "inputType",
  def = function(object, ...) {
    standardGeneric("inputType")
  }
)

setGeneric(
  name = "getPostProcessingArgs",
  def = function(clusterFunction, ...) {
    standardGeneric("getPostProcessingArgs")
  }
)

#' General wrapper method to cluster the data
#'
#' Given input data, this function will find clusters, based on a single
#' specification of parameters.
#'
#' @param inputType a character vector defining what type of input is given in
#'   the \code{inputMatrix} argument. Must consist of values "diss","X", or
#'   "cat" (see details). "X" and "cat" should be indicate
#'   matrices with features in the row and samples in the column; "cat"
#'   corresponds to the features being numerical integers corresponding to
#'   categories, while "X" are continuous valued features. "diss" corresponds to
#'   an \code{inputMatrix} that is a NxN dissimilarity matrix. "cat" is largely
#'   used internally for clustering of sets of clusterings.
#' @param inputMatrix numerical matrix on which to run the clustering or a
#'   \code{\link[SummarizedExperiment]{SummarizedExperiment}},
#'   \code{\link{SingleCellExperiment}}, or \code{\link{ClusterExperiment}}
#'   object.
#' @param distFunction a distance function to be applied to \code{inputMatrix}. Only
#'   relevant if \code{inputType="X"}. 
#' See details of \code{\link{clusterSingle}} for the
#'   required format of the distance function.
#' @param subsample logical as to whether to subsample via
#'   \code{\link{subsampleClustering}}. If TRUE, clustering in mainClustering
#'   step is done on the co-occurance between clusterings in the subsampled
#'   clustering results.  If FALSE, the mainClustering step will be run directly
#'   on \code{x}/\code{diss}
#' @param sequential logical whether to use the sequential strategy (see details
#'   of \code{\link{seqCluster}}). Can be used in combination with
#'   \code{subsample=TRUE} or \code{FALSE}.
#' @param mainClusterArgs list of arguments to be passed for the mainClustering
#'   step, see help pages of \code{\link{mainClustering}}.
#' @param subsampleArgs list of arguments to be passed to the subsampling step
#'   (if \code{subsample=TRUE}), see help pages of
#'   \code{\link{subsampleClustering}}.
#' @param seqArgs list of arguments to be passed to \code{\link{seqCluster}}.
#' @param reduceMethod character A character identifying what type of
#'   dimensionality reduction to perform before clustering. Options are 1)
#'   "none", 2) one of listBuiltInReducedDims() or listBuiltInFitlerStats OR 3)
#'   stored filtering or reducedDim values in the object.
#' @param nDims integer An integer identifying how many dimensions to reduce to
#'   in the reduction specified by \code{reduceMethod}. Defaults to output of
#'   \code{\link{defaultNDims}}
#' @param clusterLabel a string used to describe the clustering. By default it
#'   is equal to "clusterSingle", to indicate that this clustering is the result
#'   of a call to \code{clusterSingle}.
#' @param checkDiss logical. Whether to check whether the dissimilarities
#'   matrices  are valid (whether given by the user or calculated because
#'   \code{makeMissingDiss=TRUE}).
#' @param warnings logical. Whether to print out the many possible warnings and
#'   messages regarding checking the internal consistency of the parameters.
#' @param ... arguments to be passed on to the method for signature
#'   \code{matrix}.
#' @inheritParams transformData
#' @details \code{clusterSingle} is an 'expert-oriented' function, intended to
#'   be used when a user wants to run a single clustering and/or have a great
#'   deal of control over the clustering parameters. Most users will find
#'   \code{\link{clusterMany}} more relevant. However, \code{\link{clusterMany}}
#'   makes certain assumptions about the intention of certain combinations of
#'   parameters that might not match the user's intent; similarly
#'   \code{\link{clusterMany}} does not directly take a dissimilarity matrix but
#'   only a matrix of values \code{x} (though a user can define a distance
#'   function to be applied to \code{x} in \code{\link{clusterMany}}).
#' @details Unlike \code{\link{clusterMany}}, most of the relevant arguments for
#'   the actual clustering algorithms in \code{clusterSingle} are passed to the
#'   relevant steps via the arguments \code{mainClusterArgs},
#'   \code{subsampleArgs}, and \code{seqArgs}. These arguments should be
#'   \emph{named} lists with parameters that match the corresponding functions:
#'   \code{\link{mainClustering}},\code{\link{subsampleClustering}}, and
#'   \code{\link{seqCluster}}. These three functions are not meant to be called
#'   by the user, but rather accessed via calls to \code{clusterSingle}. But the
#'   user can look at the help files of those functions for more information 
#'   regarding the parameters that they take.
#' @details Only certain combinations of parameters are possible for certain
#'   choices of \code{sequential} and \code{subsample}. These restrictions are
#'   documented below. 
#'   \itemize{ 
#'       \item{\code{clusterFunction} for
#'   \code{mainClusterArgs}: }{The choice of \code{subsample=TRUE} also controls
#'   what algorithm type of clustering functions can be used in the
#'   mainClustering step. When \code{subsample=TRUE}, then resulting
#'   co-clustering matrix  from subsampling is converted to a dissimilarity
#'   (specificaly 1-coclustering values) and is passed to \code{diss} of
#'   \code{\link{mainClustering}}. For this reason, the \code{ClusterFunction}
#'   object given to \code{\link{mainClustering}} via the argument
#'   \code{mainClusterArgs} must take input of the form of a dissimilarity. When
#'   \code{subsample=FALSE} and \code{sequential=TRUE}, the
#'   \code{clusterFunction} passed in \code{clusterArgs} element of
#'   \code{mainClusterArgs} must define a \code{ClusterFunction} object with
#'   \code{algorithmType} 'K'.  When \code{subsample=FALSE} and
#'   \code{sequential=FALSE}, then there are no restrictions on the
#'   \code{ClusterFunction} and that clustering is applied directly to the input
#'   data. } 
#'   \item{\code{clusterFunction}  for \code{subsampleArgs}: }{If the
#'   \code{ClusterFunction} object given to the \code{clusterArgs} of
#'   \code{subsamplingArgs} is missing the algorithm will use the default for
#'   \code{\link{subsampleClustering}} (currently "pam"). If
#'   \code{sequential=TRUE}, this \code{ClusterFunction} object must be of type
#'   'K'. } 
#'   \item{Setting \code{k} for subsampling: }{If \code{subsample=TRUE}
#'   and \code{sequential=TRUE}, the current K of the sequential iteration
#'   determines the 'k' argument passed to \code{\link{subsampleClustering}}  so
#'   setting 'k=' in the list given to the subsampleArgs will not do anything
#'   and will produce a warning to that effect (see documentation of
#'   \code{\link{seqCluster}}).} 
#'   \item{Setting \code{k} for mainClustering step:
#'   }{If \code{sequential=TRUE} then the user should not set \code{k} in the
#'   \code{clusterArgs} argument of \code{mainClusterArgs} because it must be
#'   set by the sequential code, which has a iterative reseting of the
#'   parameters. Specifically if \code{subsample=FALSE}, then the sequential
#'   method iterates over choices of \code{k} to cluster the input data. And if
#'   \code{subsample=TRUE}, then the \code{k} in the clustering of
#'   mainClustering step (assuming the clustering function is of type 'K') will
#'   use the \code{k} used in the subsampling step to make sure that the
#'   \code{k} used in the mainClustering step is reasonable. } 
#'   \item{Setting
#'   \code{findBestK} in \code{mainClusterArgs}: }{If \code{sequential=TRUE} and
#'   \code{subsample=FALSE}, the user should not set 'findBestK=TRUE' in
#'   \code{mainClusterArgs}. This is because in this case the sequential method
#'   changes \code{k}; an error message will be given if this combination of
#'   options are set. However, if \code{sequential=TRUE} and
#'   \code{subsample=TRUE}, then passing either 'findBestK=TRUE' or
#'   'findBestK=FALSE' via \code{mainClusterArgs} will function as expected
#'   (assuming the \code{clusterFunction} argument passed to
#'   \code{mainClusterArgs} is of type 'K'). In particular, the sequential step
#'   will set the number of clusters \code{k} for clustering of each subsample.
#'   If findBestK=FALSE, that same \code{k} will be used for mainClustering step
#'   that clusters the resulting co-occurance matrix after subsampling. If
#'   findBestK=TRUE, then \code{\link{mainClustering}} will search for best k.
#'   Note that the default 'kRange' over which \code{\link{mainClustering}}
#'   searches when findBestK=TRUE depends on the input value of \code{k} which
#'   is set by the sequential method if \code{sequential=TRUE}), see above. The
#'   user can change \code{kRange} to not depend on \code{k} and to be fixed
#'   across all of the sequential steps by setting \code{kRange} explicitly in
#'   the \code{mainClusterArgs} list.} }
#' @return A \code{\link{ClusterExperiment}} object if
#'   \code{inputType} is of type "X".
#' @return If input was not of type "X", then the result is a list with values
#'   \itemize{ 
#'        \item{clustering: }{The vector of clustering results}
#'        \item{clusterInfo: }{A list with information about the parameters run in
#'   the clustering} 
#'        \item{coClusterMatrix: }{(only if \code{saveSubsamplingMatrix=TRUE}, NxB set of clusterings obtained after B subsamples.} 
#'   }
#' @details To provide a distance matrix via the argument \code{distFunction},
#'   the function must be defined to take the distance of the rows of a matrix
#'   (internally, the function will call \code{distFunction(t(x))}. This is to
#'   be compatible with the input for the \code{dist} function. \code{as.matrix}
#'   will be performed on the output of \code{distFunction}, so if the object
#'   returned has a \code{as.matrix} method that will convert the output into a
#'   symmetric matrix of distances, this is fine (for example the class
#'   \code{dist} for objects returned by \code{dist} have such a method). If
#'   \code{distFunction=NA}, then a default distance will be calculated based on
#'   the type of clustering algorithm of \code{clusterFunction}. For type "K"
#'   the default is to take \code{dist} as the distance function. For type "01",
#'   the default is to take the (1-cor(x))/2.
#'
#' @seealso \code{\link{clusterMany}} to compare multiple choices of parameters,
#'   and \code{\link{mainClustering}},\code{\link{subsampleClustering}}, and
#'   \code{\link{seqCluster}} for the underlying functions called by
#'   \code{clusterSingle}.
#'
#' @name clusterSingle
#'
#' @examples
#' data(simData)
#'
#' \dontrun{
#' #following code takes some time.
#' #use clusterSingle to do sequential clustering
#' #(same as example in seqCluster only using clusterSingle ...)
#' set.seed(44261)
#' clustSeqHier_v2 <- clusterSingle(simData,
#'      sequential=TRUE, subsample=TRUE, 
#'      subsampleArgs=list(resamp.n=100, samp.p=0.7,
#'      clusterFunction="kmeans", clusterArgs=list(nstart=10)),
#'      seqArgs=list(beta=0.8, k0=5), mainClusterArgs=list(minSize=5,
#'      clusterFunction="hierarchical01",clusterArgs=list(alpha=0.1)))
#' }
#'
#' #use clusterSingle to do just clustering k=3 with no subsampling
#' clustObject <- clusterSingle(simData,
#'     subsample=FALSE, sequential=FALSE,
#'     mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=3)))
#' #compare to standard pam
#' pamOut<-cluster::pam(t(simData),k=3,cluster.only=TRUE)
#' all(pamOut==primaryCluster(clustObject))
#' @aliases clusterSingle,SummarizedExperiment-method
#' @rdname clusterSingle
#' @export
setMethod(
    f = "clusterSingle",
    signature = signature(inputMatrix = "SummarizedExperiment" ),
    definition = function(inputMatrix, ...) {
        clusterSingle(as(inputMatrix,"SingleCellExperiment"),...)
    }
)

#' @rdname clusterSingle
#' @export
setMethod(
    f = "clusterSingle",
    signature = signature(inputMatrix = "ClusterExperiment"),
    definition = function(inputMatrix, ...) {
        if(any(c("transFun","isCount") %in% names(list(...))))
            stop("The internally saved transformation function of a ClusterExperiment object must be used when given as input and setting 'transFun' or 'isCount' for a 'ClusterExperiment' is not allowed.")
        outval <- clusterSingle(as(inputMatrix,"SingleCellExperiment"),transFun=transformation(inputMatrix),...)
        retval<-addClusterings(inputMatrix,outval)
        #make most recent clustering the primary cluster
        primaryClusterIndex(retval)<-nClusterings(retval)
        if(!is.null(outval@coClustering)) retval@coClustering<-outval@coClustering
        #make sure save the calculated information
        retval<-.addBackSEInfo(newObj=retval,oldObj=outval)
        return(retval)
    }
)
#' @rdname clusterSingle
#' @param whichAssay numeric or character specifying which assay to use. See
#'   \code{\link[SummarizedExperiment]{assay}} for details.
#' @export
setMethod(
    f = "clusterSingle",
    signature = signature(inputMatrix = "SingleCellExperiment"),
    definition = function(inputMatrix, reduceMethod="none",
                          nDims=defaultNDims(inputMatrix,reduceMethod),
                          whichAssay=1, ...) {
        inputArgs<-list(...)
        isDimReduced<- anyValidReducedDims(inputMatrix) && isReducedDims(inputMatrix,reduceMethod)
        isFilter<-anyValidFilterStats(inputMatrix) && isFilterStats(inputMatrix,reduceMethod)
        if(isDimReduced & isFilter) stop("reduceMethod matches both reducedDimNames and filtering statistic")
        if(reduceMethod=="none" || (!isDimReduced && !isFilter)){
            #go to matrix version using assay(x) and will calculate reduceMethod etc.
            outval<-clusterSingle(assay(inputMatrix, whichAssay),
                                  reduceMethod=reduceMethod,nDims=nDims,...)
            #add back in the SingleCellExperiment stuff lost
            retval<-.addBackSEInfo(newObj=outval,oldObj=inputMatrix)
            #but now need the filter/reducedDim
            if(reduceMethod!="none"){
                if(isFilterStats(outval,reduceMethod)) filterStats(retval)<-filterStats(outval)
                if(isReducedDims(outval,reduceMethod)) reducedDims(retval)<-reducedDims(outval)
            }
        }
        else{
            if(isDimReduced){
                #don't transform PCA!
                if(any(names(inputArgs)%in%"transFun") )
                    inputArgs<-inputArgs[!names(inputArgs)%in%"transFun"]
                if(any(names(inputArgs)%in%"isCount"))
                    inputArgs<-inputArgs[!names(inputArgs)%in%"isCount"]
                if(is.na(nDims)) nDims<-defaultNDims(inputMatrix,reduceMethod)
                outval<-do.call("clusterSingle",c(list(inputMatrix=(t(reducedDim(inputMatrix,type=reduceMethod)[,seq_len(nDims)])),reduceMethod="none",transFun=function(x){x},isCount=FALSE),inputArgs))
            }
            if(isFilter){
                #Need to think how can pass options to filterData...
                if(is.na(nDims)) nDims<-defaultNDims(inputMatrix,reduceMethod)
                outval<-clusterSingle(filterData(inputMatrix,filterStats=reduceMethod,percentile=nDims),reduceMethod="none",...)			#do transform filtered data...
            }
            #add back in the SingleCellExperiment stuff lost
            retval<-.addBackSEInfo(newObj=outval,oldObj=inputMatrix)
            
        }
        return(retval)
    }
)


#' @rdname clusterSingle
#' @export
#' @param saveSubsamplingMatrix logical. If TRUE, the co-clustering matrix
#'   resulting from subsampling is returned in the coClustering slot (and
#'   replaces any existing coClustering object in the slot \code{coClustering}
#'   if input object is a \code{ClusterExperiment} object.)
#' @param makeMissingDiss logical. Whether to calculate necessary distance 
#'   matrices needed when input is not "diss". If TRUE, then when a clustering 
#'   function calls for a inputType "diss", but the given matrix is of type "X",
#'   the function will calculate a distance function. A dissimilarity matrix
#'   will also be calculated if a post-processing argument like \code{findBestK}
#'   or \code{removeSil} is chosen, since these rely on calcualting silhouette
#'   widths from distances.
setMethod(
    f = "clusterSingle",
    signature = signature(inputMatrix = "matrixOrHDF5OrNULL"),
    definition = function(inputMatrix, inputType="X", 
                          subsample=FALSE, sequential=FALSE, distFunction=NA,
                          mainClusterArgs=NULL, subsampleArgs=NULL, seqArgs=NULL,
                          isCount=FALSE,transFun=NULL, 
                          reduceMethod="none",
                          nDims=defaultNDims(inputMatrix, reduceMethod),
                          makeMissingDiss=if(ncol(inputMatrix)<1000) TRUE else FALSE,
                          clusterLabel="clusterSingle",
                          saveSubsamplingMatrix=FALSE, 
                          checkDiss=FALSE, warnings=TRUE) {
                             
        origInputType<-inputType
        if(saveSubsamplingMatrix & !subsample){
            warning("Setting saveSubsamplingMatrix=TRUE has no effect if subsample=FALSE")
            saveSubsamplingMatrix<-FALSE
        }
        doDiss<-FALSE
        if(is.function(distFunction) || !is.na(distFunction)){
            if(inputType!="diss"){
                inputType<-"diss"
                doDiss<-TRUE
            }
            else{
                stop("Cannot provide argument distFunction for an inputMatrix that is already a dissimilarity (inputType='diss')")
            }
        }
        ##########
        ## Check arguments and set defaults as needed
        ## Note, some checks are duplicative of internal functions, 
        ## but better here, because don't want to find error after 
        ## already done extensive calculation...
        ##########
        checkOut<-.checkArgs(inputType=inputType, subsample=subsample, sequential=sequential, main=TRUE, mainClusterArgs=mainClusterArgs, subsampleArgs=subsampleArgs, seqArgs=seqArgs,allowMakeDiss=makeMissingDiss & !doDiss,warn=warnings)
        if(is.character(checkOut)) stop(checkOut)
        mainClusterArgs<-checkOut$mainClusterArgs
        subsampleArgs<-checkOut$subsampleArgs
        ## if makeMissingDiss=TRUE, then will tell whether to create diss from X with euclidean distance
        doDissPost<-checkOut$doDissPost
        if(!doDiss){
            doDiss<-checkOut$doDiss
        } 
        
        ##########
        ## Handle dimensionality reduction:
        ##########
        ###Don't do this until after do the checks, because takes some time.
        transObj<-NULL
        if(origInputType == "X"){
            origX<-inputMatrix # Need this to make CE object later!
            N <- dim(inputMatrix)[2] #ngenes x nsamples
            ##########
            ##transformation to data x that will be input to clustering
            ##########
            transFun<-.makeTransFun(transFun=transFun,isCount=isCount) #need this later to build clusterExperiment object
            reduceMethod<-match.arg(reduceMethod,c("none",
                                   listBuiltInReducedDims(), 
                                   listBuiltInFilterStats())) #because this is matrix method, will always have to be a built in value.
            if(length(nDims)>1 || length(reduceMethod)>1) {
                stop("clusterSingle only handles one choice of dimensions or reduceMethod. If you want to compare multiple choices, try clusterMany")
            }
            if(!is.na(nDims) & reduceMethod=="none") {
                if(warnings) warning("specifying nDims has no effect if reduceMethod==`none`")
            }
            if(reduceMethod=="none"){
                inputMatrix<-transformData(inputMatrix,transFun=transFun)
            }
            else if(reduceMethod%in%listBuiltInReducedDims()){
                transObj<-makeReducedDims(inputMatrix,reducedDims=reduceMethod, maxDims=nDims, transFun=transFun,isCount=isCount)
                inputMatrix<-t(reducedDim(transObj,type=reduceMethod))
            }
            else if(reduceMethod %in% listBuiltInFilterStats()){
                transObj<-makeFilterStats(inputMatrix,filterStat=reduceMethod, transFun=transFun,isCount=isCount)
                inputMatrix<-transformData(filterData(transObj, filterStats=reduceMethod, percentile=nDims), transFun=transFun, isCount=isCount)
            }
            else stop("invalid value for reduceMethod -- not in built-in filter or reducedDim method")
        }
        else{
            if(any(reduceMethod!="none")) stop("'reduceMethod' cannot be given if 'inputType' argument is not of type 'X'.")
            N<-nrow(inputMatrix)
        }
        
        
        #Make dissimilarity AFTER transforming data
        
        cf<-if(subsample) subsampleArgs[["clusterFunction"]] else mainClusterArgs[["clusterFunction"]]
        if(doDiss){
            mess<-"Making nxn dissimilarity matrix"
            if(!is.function(distFunction) && is.character(distFunction)){
                mess<-paste(mess,sprintf("(with distance function %s)",distFunction))
            }
            else if(!is.function(distFunction) && is.na(distFunction)) mess<-paste(mess,"(with default distance function, determined by algorithm type of clustering function)")
            else mess<-paste(mess,"(with user defined distance function)")
            if(warnings) .mynote(mess)
            inputMatrix<-.makeDiss(inputMatrix,
                distFunction=distFunction,
                checkDiss=FALSE, algType=algorithmType(cf))
            inputType<-"diss"
        }
        else if(doDissPost & inputType!="diss"){
            mess<-"Making nxn dissimilarity matrix in order to do post-processing (e.g. arguments findBestK=TRUE or removeSil=TRUE)."
            if(warnings) .mynote(mess)
            cfPost<- mainClusterArgs[["clusterFunction"]]
            mainClusterArgs[["diss"]]<-.makeDiss(inputMatrix,
                distFunction=distFunction,
                checkDiss=FALSE, algType=algorithmType(cfPost))
            if(checkDiss) .checkDissFunction(mainClusterArgs[["diss"]], algType = algorithmType(cfPost))
        }
        
        if(inputType=="diss" & checkDiss){
            .checkDissFunction(inputMatrix, algType = algorithmType(cf))
        }
        ##########
        ## Start running clustering
        ##########
        if(sequential){
            outlist <- do.call("seqCluster",
                               c(list(inputMatrix=inputMatrix,
                                    inputType=inputType, subsample=subsample,
                                      subsampleArgs=subsampleArgs,
                                      mainClusterArgs=mainClusterArgs),
                                      seqArgs))
        }
        else{
            ##########
            ##.clusterWrapper just deciphers choices and makes clustering.
            ##########
            finalClusterList <- .clusterWrapper(inputMatrix=inputMatrix,
                    subsample=subsample,
                    subsampleArgs=subsampleArgs,
                    mainClusterArgs=mainClusterArgs)
            outlist <- list("clustering"= .clusterListToVector(finalClusterList$result, N), coClusterMatrix=t(finalClusterList$inputMatrix))
            
        }
        clInfo<-list(list(clusterInfo = outlist$clusterInfo,
                          whyStop = outlist$whyStop,
                          subsample = subsample,
                          sequential = sequential,
                          mainClusterArgs = mainClusterArgs,
                          subsampleArgs = subsampleArgs,
                          seqArgs = seqArgs,
                          reduceMethod=reduceMethod,
                          nDims=nDims
        ))
        ##########
        ## Convert to ClusterExperiment Object
        ##########
        if(origInputType == "X"){ 
            retval<-.convertOutListToCE(
                xMatrix=origX,
                clustering=outlist$clustering,
                clusterInfo=clInfo,
                coClusterMatrix=outlist$coClusterMatrix,
                transFun=transFun,
                clusterLabel=clusterLabel,
                sequential=sequential, 
                subsample=subsample, 
                saveSubsamplingMatrix=saveSubsamplingMatrix, 
                existingCE=transObj)
            return(retval)
        }
        else{
            out<-list(clustering=outlist$clustering, 
				clusterInfo=clInfo)
            if(saveSubsamplingMatrix) 
                out<-c(out,list(coClusterMatrix=outlist$coClusterMatrix))
            return(out)
        }
        
    }
)
# Do this so clear how to transform diss version of output into CE 
# (for reuse in clusterMany...)
.convertOutListToCE<-function(xMatrix,
    clustering,clusterInfo, coClusterMatrix, 
    clusterLabel,sequential, 
    subsample, transFun,
    saveSubsamplingMatrix, existingCE){
    retval <- ClusterExperiment(xMatrix, clustering,
                                transformation=transFun,
                                clusterInfo=clusterInfo, 
                                clusterTypes="clusterSingle",
                                checkTransformAndAssay=FALSE)
    clusterLabels(retval)<-clusterLabel
    if(!is.null(coClusterMatrix) & !sequential & subsample & saveSubsamplingMatrix) {
        #will convert to sparse matrix:
		coClustering(retval) <- coClusterMatrix
        ch<-.checkCoClustering(retval)
        if(!is.logical(ch)) stop(ch)
    }
    if(!is.null(existingCE)){
        #add in the reduceMethod stuff
        retval<-.addBackSEInfo(newObj=retval,oldObj=existingCE)
    }
    return(retval)
}
#wrapper that calls the clusterSampling and mainClustering routines in reasonable order.
#called by both seqCluster and clusterSingle
#clusterFunction assumed to be defined in mainClusterArgs AND subsampleArgs
.clusterWrapper <- function(inputMatrix,  subsample, mainClusterArgs, subsampleArgs)
{
    if(subsample){
        #Returns NxB matrix
        inputMatrix<-t(do.call("subsampleClustering",
			c(list(inputMatrix=inputMatrix), 
			subsampleArgs)))
        mainClusterArgs[["inputType"]]<-"cat"
    }    
    #Takes as input BxN matrix...
    resList<-do.call("mainClustering",
		c(list(inputMatrix=inputMatrix, 
			format="list", returnData=TRUE),
		mainClusterArgs))
    return(resList)
}

#' Program for sequentially clustering, removing cluster, and starting again.
#'
#' Given a data matrix, this function will call clustering
#' routines, and sequentially remove best clusters, and iterate to find
#' clusters.
#'
#' @param k0 the value of K at the first iteration of sequential algorithm, see
#'   details below or vignette.
#' @param subsample logical as to whether to subsample via 
#'   \code{\link{subsampleClustering}} to get the distance matrix at each 
#'   iteration; otherwise the distance matrix is set by arguments to
#'   \code{\link{mainClustering}}.
#' @param beta value between 0 and 1 to decide how stable clustership membership
#'   has to be before 'finding' and removing the cluster.
#' @param top.can only the top.can clusters from \code{\link{mainClustering}} (ranked
#'   by 'orderBy' argument given to \code{\link{mainClustering}}) will be compared
#'   pairwise for stability. Can be either an integer value, identifying the absolute number of clusters, or a value between 0 and 1, meaning to keep all clusters with at least this proportion of the remaining samples in the cluster. Making this either a very big integer or equal to 0 will effectively remove this
#'   parameter and all pairwise comparisons of all clusters found will be
#'   considered; this might result in smaller clusters being found. If top.can is between 0 and 1, then there is still a hard threshold of at least 5 samples in a cluster to be considered as a cluster. 
#' @param remain.n when only this number of samples are left (i.e. not yet
#'   clustered) then algorithm will stop.
#' @param k.min each iteration of sequential detection of clustering will
#'   decrease the beginning K of subsampling, but not lower than k.min. 
#' @param k.max algorithm will stop if K in iteration is increased beyond this
#'   point.
#' @param verbose whether the algorithm should print out information as to its
#'   progress.
#' @param subsampleArgs list of arguments to be passed to
#'   \code{\link{subsampleClustering}}.
#' @param mainClusterArgs list of arguments to be passed to
#'   \code{\link{mainClustering}}).
#' @inheritParams clusterSingle
#' @details \code{seqCluster} is not meant to be called by the user. It is only
#'   an exported function so as to be able to clearly document the arguments for
#'   \code{seqCluster} which can be passed via the argument \code{seqArgs} in
#'   functions like \code{\link{clusterSingle}} and \code{\link{clusterMany}}.
#' @details This code is adapted from the sequential protion of the code of the
#'   tightClust package of Tseng and Wong. At each iteration of the algorithm it
#'   finds a set of samples that constitute a homogeneous cluster and remove
#'   them, and iterate again to find the next set of samples that form a
#'   cluster.
#' @details In each iteration, to determine the next set of homogeneous set of
#'   samples, the algorithm will iteratively cluster the current set of samples
#'   for a series of increasing values of the parameter $K$, starting at a value
#'   \code{kinit} and increasing by 1 at each iteration, until a sufficiently
#'   homogeneous set of clusters is found. For the first set of homogeneous
#'   samples, \code{kinit} is set to the argument $k0$, and for iteration,
#'   \code{kinit} is increased internally.
#' @details Depending on the value of \code{subsample} how the value of $K$ is
#'   used differs. If \code{subsample=TRUE}, $K$ is the \code{k} sent to the
#'   cluster function \code{clusterFunction} sent to 
#'   \code{\link{subsampleClustering}} via \code{subsampleArgs}; then
#'   \code{\link{mainClustering}} is run on the result of the co-occurance matrix from
#'   \code{\link{subsampleClustering}} with the \code{ClusterFunction} object
#'   defined in the argument \code{clusterFunction} set via \code{mainClusterArgs}.
#'   The number of clusters actually resulting from this run of
#'   \code{\link{mainClustering}} may not be equal to the $K$ sent to  the clustering
#'   done in \code{\link{subsampleClustering}}. If \code{subsample=FALSE},
#'   \code{\link{mainClustering}} is called directly on the data to determine the
#'   clusters and $K$ set by \code{seqCluster} for this iteration determines the
#'   parameter of the clustering done by \code{\link{mainClustering}}. Specifically,
#'   the argument \code{clusterFunction} defines the clustering of the
#'   \code{\link{mainClustering}} step and \code{k} is sent to that
#'   \code{ClusterFunction} object. This means that if \code{subsample=FALSE},
#'   the \code{clusterFunction} must be of \code{algorithmType} "K".
#' @details In either setting of \code{subsample}, the resulting clusters from
#'   \code{\link{mainClustering}} for a particular $K$ will be compared to clusters
#'   found in the previous iteration of $K-1$. For computational (and other?)
#'   convenience, only the first \code{top.can} clusters of each iteration will
#'   be compared to the first \code{top.can} clusters of previous iteration for
#'   similarity (where \code{top.can} currently refers to ordering by size, so
#'   first \code{top.can} largest clusters.
#' @details If there is no cluster of the first \code{top.can} in the current
#'   iteration $K$ that has overlap similarity > \code{beta} to any in the
#'   previous iteration, then the algorithm will move to the next iteration,
#'   increasing to $K+1$.
#'   
#' @details If, however, of these clusters there is a cluster in the current
#'   iteration $K$ that has overlap similarity > beta to a cluster in the
#'   previous iteration $K-1$, then the cluster with the largest such similarity
#'   will be identified as a homogenous set of samples and the samples in it
#'   will be removed and designated as such. The algorithm will then start again
#'   to determine the next set of homogenous samples, but without these samples.
#'   Furthermore, in this case (i.e. a cluster was found and removed), the value
#'   of \code{kinit} will be be reset to \code{kinit-1}; i.e. the range of
#'   increasing $K$ that will be iterated over to find a set of homogenous
#'   samples will start off one value less than was the case for the previous
#'   set of homogeneous samples. If \code{kinit-1}<\code{k.min}, then
#'   \code{kinit} will be set to \code{k.min}.
#'   
#'   
#' @details If there are less than \code{remain.n} samples left after finding a
#'   cluster and removing its samples, the algorithm will stop, as subsampling
#'   is deamed to no longer be appropriate. If the K has to be increased to
#'   beyond \code{k.max} without finding any pair of clusters with overlap >
#'   beta, then the algorithm will stop. Any samples not found as part of a
#'   homogenous set of clusters at that point will be classified as unclustered
#'   (given a value of -1)
#' @details Certain combinations of inputs to \code{mainClusterArgs} and
#'   \code{subsampleArgs} are not allowed. See \code{\link{clusterSingle}} for
#'   these explanations.
#' @return A list with values
#' \itemize{
#'
#' \item{\code{clustering}}{ a vector of length equal to nrows(x) giving the
#' integer-valued cluster ids for each sample. The integer values are assigned
#' in the order that the clusters were found. "-1" indicates the sample was not
#' clustered.}
#'
#' \item{\code{clusterInfo}}{ if clusters were successfully found, a matrix of
#' information regarding the algorithm behavior for each cluster (the starting
#' and stopping K for each cluster, and the number of iterations for each
#' cluster).}
#'
#' \item{\code{whyStop}}{ a character string explaining what triggered the
#' algorithm to stop.}
#' }
#' @references Tseng and Wong (2005), "Tight Clustering: A Resampling-Based
#'   Approach for Identifying Stable and Tight Patterns in Data", Biometrics,
#'   61:10-16.
#' 
#' @seealso tight.clust,
#'   \code{\link{clusterSingle}},\code{\link{mainClustering}},\code{\link{subsampleClustering}}
#'   
#' @examples
#' \dontrun{
#' data(simData)
#'
#' set.seed(12908)
#' clustSeqHier <- seqCluster(simData, inputType="X", k0=5, subsample=TRUE,
#'    beta=0.8, subsampleArgs=list(resamp.n=100,
#'    samp.p=0.7, clusterFunction="kmeans", clusterArgs=list(nstart=10)),
#'    mainClusterArgs=list(minSize=5,clusterFunction="hierarchical01",
#'    clusterArgs=list(alpha=0.1)))
#' }
#' @export
#' @rdname seqCluster
#' @export
seqCluster<-function(inputMatrix, inputType, k0,  
                     subsample=TRUE, beta, 
                     top.can = 0.01, remain.n = 30, k.min = 3, 
                     k.max=k0+10,verbose=TRUE, 
                     subsampleArgs=NULL,mainClusterArgs=NULL,
                     warnings=FALSE)
{
    ########
    ####Checks
    ########
    if(missing(inputType)) 
        stop("Internal error: inputType was not passed to sequential clustering step")
    argsList<-list(k0=k0, beta=beta, top.can = top.can, 
        remain.n = remain.n, k.min = k.min, k.max=k.max)
    checkOut<-.checkArgs(inputType, main=TRUE, 
        subsample=subsample,sequential=TRUE,
		mainClusterArgs=mainClusterArgs,
		subsampleArgs=subsampleArgs, seqArgs=argsList,warn=warnings)
    if(is.character(checkOut)) stop(checkOut)
    mainClusterArgs<-checkOut$mainClusterArgs
    subsampleArgs<-checkOut$subsampleArgs
    seqArgs<-checkOut$seqArgs
    k0=seqArgs[["k0"]]
    beta=seqArgs[["beta"]]
    top.can = seqArgs[["top.can"]]
    isIntegerTop<-is.whole(top.can) & top.can >0
    remain.n = seqArgs[["remain.n"]]
    k.min = seqArgs[["k.min"]]
    k.max=seqArgs[["k.max"]]
    
    if(subsample) input<-subsampleArgs[["inputType"]]
    else input<-mainClusterArgs[["inputType"]]
    #Note, for the purpose of this
    N <- dim(inputMatrix)[2]
    colnames(inputMatrix)<-as.character(seq_len(N))
    if(input=="diss") rownames(inputMatrix)<-colnames(inputMatrix)
    
    #########
    # Function that updates k and runs clustering
    # (Called at each iteration)
    # Note that depends on .checkArgs having removed the 'k' option 
    # from the user inputs
    #########
    updateClustering<-function(newk){
        if(verbose) cat(paste("k =", newk,"\n"))
        # set k to the new k in main cluster step
        # note is subsample=TRUE, and alg. of main is type K, 
        # means k is always equal at subsample and main step. 
        tempMainArgs<-mainClusterArgs
        if(algorithmType(tempMainArgs[["clusterFunction"]])=="K")
            tempMainArgs[["clusterArgs"]]<-c(list(k=newk),
			mainClusterArgs[["clusterArgs"]])       
        else tempMainArgs<-mainClusterArgs
        tempSubArgs<-subsampleArgs
        # set k to new k in subsampling step.
        if(subsample){
            tempSubArgs[["clusterArgs"]]<-c(list(k=newk), subsampleArgs[["clusterArgs"]]) 
        }
        res <- .clusterWrapper(inputMatrix=inputMatrix, 
                subsample=subsample,  subsampleArgs=tempSubArgs,
                mainClusterArgs=tempMainArgs)$results
        return(res)
    }
    trimResults<-function(res){
        if(length(res)>0){
            if(isIntegerTop){
                res <- res[seq_len(min(top.can,length(res)))]
            } 
            else{
                currN<-ncol(inputMatrix)
                requiredN<-floor(currN*top.can)
                NPerCluster<-sapply(res,length)
                if(isTRUE(all.equal(top.can,0))){
                    requiredN<-0
                }
                else{
                    requiredN<-max(requiredN,5)
                }
                whLarger<-which(NPerCluster>=requiredN)
                # cat("Npercluster:\n")
                # print(NPerCluster)
                if(length(whLarger)>0){
                    res<-res[whLarger]
                    #cat(sprintf("Npercluster that larger than %s:\n",max(requiredN,5)))
                    #print(NPerCluster[whLarger])
                }
                else res<-res[seq_len(min(10,length(res)))]
            }
        } 
        return(res)
    }
    #########
    ## Iteration code
    #########
    
    ###---------
    ###---------
    ###The following is legacy of tight.clust. They originally had programmed ability to look across more than 2 at each step to determing the stability of a cluster. This was not what they described in paper, and function is hard-coded at 2, but I have left code here in case we ever wanted to reconsider this issue.
    ###---------
    seq.num<-2
    kReturn<-"last" # when look at stability, return stable as first or last? For seq.num=2, not really matter, take last like paper
    kReturn<-match.arg(kReturn,c("last","first"))
    betaNum<-"all"
    betaNum<-match.arg(betaNum,c("all","last","first"))
    #This makes all combinations of 1:top.can, seq.num times (could be simplified if seq.num=2):
    #a ncombinations x seq.num matrix -- each row gives a combination of clusters to compare stability
    if( isIntegerTop) index.m <- as.matrix(expand.grid(lapply(seq_len(seq.num), function(x) seq_len(top.can))))
    else index.m<-NULL
    whReturn<-switch(kReturn,"last"=seq.num,"first"=1) #way to index which one gets returned.
    ###---------
    ###---------
    
    #---------
    #iterative setup
    #-------
    remain <- N #keep track of how many samples not yet clustered (stop when less than remain.n)
    nfound <- 0 #keep track of how many clusters found/removed so far
    found <- TRUE #has a cluster been found/removed in last iteration
    k.start <- k0 #the starting k for the next cluster
    k <- k0
    ### Blank values that will be filled in.
    candidates <- list() #list of length seq.num of possible clusters found for each k to be compared
    tclust <- list() #list of final cluster identifications (indices of rows of x)
    kstart<-vector("numeric") #the starting k for the cluster
    kend<-vector("numeric") #the ending k for the cluster
    whyStop<-NULL
    #---------
    #iterative loop
    #-------
    while (remain >= remain.n && (found || k <= k.max)) {
        if (found) { 
            # i.e. previous iteration found cluster; 
            # need to start finding new cluster
            if(verbose) cat(paste("Looking for cluster", nfound + 1, "...\n"))
            k <- k.start
            currentStart<-k.start #will add this to kstart if successful in finding cluster
            # find clusters for K,K+1
            # Note, candidates should be empty, so filling first two slots
            for (ii in seq_len(seq.num)) {
                newk<-k + ii - 1
                res<-updateClustering(newk)
                candidates[[ii]]<-trimResults(res)
            }
        }
        else { 
            # previous iteration clustering wasn't good enough
            # need to go increase to K+2,K+3, etc.
            candidates <- candidates[-1] #remove old k
            if(length(candidates)!=1 ) stop("Internal coding error in seqCluster -- more candidates than considering")
            newk<-k + seq.num - 1
            #add new k (because always list o)
            res<-updateClustering(newk)
            candidates[[seq.num]] <- trimResults(res)
        }
        ##################
        #check whether all got candidates for each iteration
        ##################
        # number of clusters found in each of clusterings (candidates of length 2).
        # Note, number of clusters should be <= top.can (# top candidates to keep)
        nClusterPerK<-sapply(candidates,length) 
        if(any(nClusterPerK==0)){
            whyStop<-paste("Stopped in midst of searching for cluster",nfound+1," because no clusters meeting criteria found for iteration k=",k+1,"and previous clusters not similar enough.")
            if(verbose) cat(paste("Found ",paste(nClusterPerK,collapse=","),"clusters for k=",paste(k+seq_len(seq.num)-1,collapse=","),", respectively. Stopping iterating because no clusters meet trimming criteria found for this iteration and previous clusters not similar enough.\n"))
            break
        }
        if(isIntegerTop){ 
            whInvalid<-sweep(index.m,MARGIN=2,STATS=nClusterPerK, FUN=">")
            # vector (of length equal to number of combinations) saying which of the clusterings didn't have at least top.can clusters:
            whInvalid<-which(apply(whInvalid,1,any))
            if(length(whInvalid)==nrow(index.m)){
                #all invalid -- probably means that for some k there were no candidates found. So should stop.
                stop("Internal coding error -- shouldn't have reached this point in seqCluster")
            }
            if(length(whInvalid)>0){
                if(verbose) cat("Did not find", top.can,"clusters for all k: ")
                if(verbose) cat(sprintf("found %s clusters for k= %s, respectively\n",paste(nClusterPerK,collapse=","),paste(k+seq_len(seq.num)-1,collapse=",")))
                tempIndex<-index.m[-whInvalid,,drop=FALSE]
            }
            else tempIndex<-index.m
        }
        else{
            tempIndex<- as.matrix(expand.grid(lapply(nClusterPerK, 
                function(ii) seq_len(ii))))
        }

        ##################
        #Calculate the stability pairwise between all of cluster combinations
        ##################
        #function to calculate the stability across the given combination
        #	y is a combination (row of tempIndex) giving clusters to compare stability from k, k+1
        calc.beta <- function(y) {
            #written generally enough to deal with seq.num>2; could be a lot simpler with seq.num=2.
            temp <- lapply(seq_len(seq.num), function(z) candidates[[z]][[y[z]]]) #each
            i.temp <- temp[[1]]
            if(betaNum %in% c("all","first")) u.temp <- temp[[1]] ###eap: changed here
            if(betaNum == "last") u.temp<-temp[[seq.num]]
            for (j in 2:seq.num) {
                i.temp <- intersect(i.temp, temp[[j]])
                if(betaNum=="all") u.temp <- union(u.temp, temp[[j]])
            }
            out<-length(i.temp)/length(u.temp)
            return(out)
            
        }
        beta.temp <- apply(tempIndex, 1, calc.beta) 
        if (any(beta.temp >= beta)){
            found <- TRUE
            nfound <- nfound + 1
            if(verbose) cat(paste("Cluster",nfound,"found."), "")
            if (k.start > k.min) k.start <- k.start - 1 #decrease
            found.temp <- candidates[[whReturn]][[tempIndex[which.max(beta.temp)[1], whReturn]]]
            kend[[nfound]]<-k+seq.num-1 #just assuming returning last here!
            kstart[[nfound]]<-currentStart
            tclust[[nfound]]<-colnames(inputMatrix)[found.temp]
            tclust[[nfound]]<-as.numeric(tclust[[nfound]])
            if(input == "diss") inputMatrix<-inputMatrix[-found.temp, -found.temp, drop=FALSE]
            else inputMatrix<-inputMatrix[ , -found.temp, drop=FALSE]
            remain <- remain - length(tclust[[nfound]])
            if(verbose) cat(paste("Cluster size:", length(tclust[[nfound]]),
                                  "\tRemaining number of points:", remain, "\n"),
                            "")
        }
        else {
            found = FALSE
            k = k + 1
        }
    }
    ###############
    # Clean up and return results
    ###############
    if(is.null(whyStop)){
        if(remain< remain.n) whyStop<-"Ran out of samples"
        if(!found & k>k.max) whyStop<-paste("Went past k.max=",k.max,"in looking for cluster with similarity to previous.")
    }
    clusterVector<-.clusterListToVector(tclust,N)
    if(all(clusterVector==-1) & length(tclust)>0) stop("coding error")
    if(nfound>0){
        size <- sapply(tclust, length)
        sizeMat<-cbind(size=size,kStart=kstart,kEnd=kend,nIter=kend-kstart)
        res <- list(clustering = clusterVector, clusterInfo = sizeMat, whyStop=whyStop)
        if(verbose) cat(paste("Stopped because:", whyStop),"")
        return(res)
    }
    else{
        if(verbose) cat("No tight clusters could be found with given parameters")
        return(list(clustering = clusterVector, whyStop=whyStop))
    }

}

#---------
#for a sample index ii, determine what other indices in the same sample with it
#ii an index of a sample
#clustVec the clusterIds as returned by above in DList
#clusterLengths the length of each cluster as returned by above in DList
otherIds<-function(idx,clustVec,clustLeng){
    m<-which(clustVec==idx)
    if(length(m)>1) stop("ids clustered in more than one cluster")
    if(length(m)==0) return(NA) #sample not ever clustered
    if(length(m)==1){
        ends<-cumsum(clustLeng)
        begins<-cumsum(c(1,head(clustLeng,-1)))
        whCluster<-which(m<=ends & m>=begins)
        if(length(whCluster)>1 | length(whCluster)==0) stop("error in coding: finding range of clusterids")
        return(clustVec[seq(begins[whCluster],ends[whCluster],by=1)])
    }
}
#Test: otherIds(5,DList[[1]][[1]],DList[[1]][[2]])

#---------
#For ii an index of a sample, calculates the proportion of times joint with every other sample with index < ii.
#clusterList is the results of subsampling, i.e. list with indices of clusters adjacent
#returns vector of length N-ii with the proportions
#NA if either never subsampled together or never clustered together.
#---------
searchForPairs<-function(ii,clusterList,N){
    #get list of those indices sample ii was sampled
	whHave<-which(sapply(clusterList,function(ll){ii%in%ll$clusterIds}))
	#calculate number of times sampled with (denominator)
    sampledWithTab<-table(unlist(sapply(clusterList[whHave],.subset2,"clusterIds")))
    #get those indices clustered with and tabulate
	clusterWith<-lapply(clusterList[whHave],function(ll){
        otherIds(idx=ii,clustVec=ll$clusterIds,clustLeng=ll$clusterLengths)
    })
	# if(doGC){#just to reduce memory since will be parallelized
	# 	rm(clusterList)
	# 	gc()
	# }
	clusterWithTab<-table(unlist(clusterWith))
    jointNames<-as.character(seq_len(N))
	whLower<-which(as.integer(as.numeric(jointNames))<ii)
	return(as.integer(clusterWithTab[jointNames][whLower])/as.integer(sampledWithTab[jointNames][whLower]))

	#thoughts about alternative code if not need save NxN matrix...
    #jointNames<-names(sampledWithTab) #if manage to not save NxN matrix, could use this to return only those that actually present
    #out<-out[!is.na(out[,"together"]),,drop=FALSE] #if manage to not save NxN matrix, could use this to return only those that actually present; but then need to not return proportions, but something else.
}

#' @title Functions to subset ClusterExperiment Objects
#' @description These functions are used to subset ClusterExperiment objects,
#'   either by removing samples, genes, or clusterings
#' @name subset
#' @param x a ClusterExperiment object.
#' @inheritParams ClusterExperiment-class
#' @inheritParams getClusterIndex
#' @return A \code{\link{ClusterExperiment}} object.
#' @details \code{removeClusterings} removes the clusters given by
#'   \code{whichClusters}. If the \code{primaryCluster} is one of the clusters
#'   removed, the \code{primaryClusterIndex} is set to 1 and the dendrogram and
#'   coclustering matrix are discarded and orderSamples is set to
#'   \code{1:NCOL(x)}.
#' @return \code{removeClusterings} returns a \code{ClusterExperiment} object,
#'   unless all clusters are removed, in which case it returns a
#'   \code{\link{SingleCellExperiment}} object.
#' @examples
#' #load CE object
#' data(rsecFluidigm)
#' # remove the mergeClusters step from the object
#' clusterLabels(rsecFluidigm)
#' test<-removeClusterings(rsecFluidigm,whichClusters="mergeClusters")
#' clusterLabels(test)
#' tableClusters(rsecFluidigm)
#' test<-removeClusters(rsecFluidigm,whichCluster="mergeClusters",clustersToRemove=c("m01","m04"))
#' tableClusters(test,whichCluster="mergeClusters")
#' @export
#' @aliases removeClusterings,ClusterExperiment-method
setMethod(
    f = "removeClusterings",
    signature = signature("ClusterExperiment"),
    definition = function(x, whichClusters) {
        whichClusters<-getClusterIndex(object=x,whichClusters=whichClusters,noMatch="throwError")
        if(length(whichClusters)==NCOL(clusterMatrix(x))){
            warning("All clusters have been removed. Will return just a Summarized Experiment Object")
            #make it Summarized Experiment
            return(as(x,"SingleCellExperiment"))
        }
        
        newClLabels<-clusterMatrix(x)[,-whichClusters,drop=FALSE]
        newClusterInfo<-clusteringInfo(x)[-whichClusters]
        newClusterType<-clusterTypes(x)[-whichClusters]
        newClusterColors<-clusterLegend(x)[-whichClusters]
        orderSamples<-orderSamples(x)
        if(primaryClusterIndex(x) %in% whichClusters) 
            pIndex<-1
        else 
            pIndex<-match(primaryClusterIndex(x),seq_len(NCOL(clusterMatrix(x)))[-whichClusters])
        
        ## Fix CoClustering information
        ## Erase if any are part of clusters to remove
        coMat<-coClustering(x)
        typeCoCl<-.typeOfCoClustering(x)
        if(typeCoCl=="indices"){
            if(any(coMat %in% whichClusters)){
                warning("removing clusterings that were used in makeConsensus (i.e. stored in CoClustering slot). Will delete the coClustering slot")
                coMat<-NULL
            }
            else{
                #Fix so indexes the right clustering...
                coMat<-match(coMat,
                     seq_len(NCOL(clusterMatrix(x)))[-whichClusters])
            }
            
        }
        
        #fix merge info:
        #erase merge info if either dendro or merge index deleted.
        if(mergeClusterIndex(x) %in% whichClusters | x@merge_dendrocluster_index %in% whichClusters){
            x<-.eraseMerge(x)
            merge_index<-x@merge_index
            merge_dendrocluster_index<-x@merge_dendrocluster_index
        }
        else{
            merge_index<-match(x@merge_index, seq_len(NCOL(clusterMatrix(x)))[-whichClusters])
            merge_dendrocluster_index<-match(x@merge_dendrocluster_index,  seq_len(NCOL(clusterMatrix(x)))[-whichClusters])
            
        }
        #fix dendro info
        dend_samples <- x@dendro_samples
        dend_cl <- x@dendro_clusters
        dend_ind<-dendroClusterIndex(x)
        if(dendroClusterIndex(x) %in% whichClusters){
            dend_cl<-NULL
            dend_samples<-NULL
            dend_ind<-NA_real_
        }
        else{
            dend_ind<-match(dend_ind,seq_len(NCOL(clusterMatrix(x)))[-whichClusters])
        }
        retval<-ClusterExperiment(
            as(x,"SingleCellExperiment"),
            clusters=newClLabels,
            transformation=transformation(x),
            clusterTypes=newClusterType,
            clusterInfo<-newClusterInfo,
            primaryIndex=pIndex,
            dendro_samples=dend_samples,
            dendro_clusters=dend_cl,
            dendro_index=dend_ind,
            merge_index=merge_index,
            merge_dendrocluster_index=merge_dendrocluster_index,
            merge_cutoff=x@merge_cutoff,
            merge_nodeProp=x@merge_nodeProp,
            merge_nodeMerge=x@merge_nodeMerge,
            merge_method=x@merge_method,
            merge_demethod=x@merge_demethod,                              
            coClustering=coMat,
            orderSamples=orderSamples,
            clusterLegend=newClusterColors,
            checkTransformAndAssay=FALSE
        )
        return(retval)
    }
)



#' @details \code{removeClusters} creates a new cluster that unassigns samples
#'   in cluster \code{clustersToRemove} (in the clustering defined by
#'   \code{whichClusters}) and assigns them to -1 (unassigned)
#' @param clustersToRemove numeric vector identifying the clusters to remove
#'   (whose samples will be reassigned to -1 value).
#' @rdname subset
#' @inheritParams addClusterings
#' @aliases removeClusters
#' @export
setMethod(
    f = "removeClusters",
    signature = c("ClusterExperiment"),
    definition = function(x,whichCluster,clustersToRemove,makePrimary=FALSE,clusterLabels=NULL) {
        whCl<-getSingleClusterIndex(x,whichCluster)
        cl<-clusterMatrix(x)[,whCl]
        leg<-clusterLegend(x)[[whCl]]
        if(is.character(clustersToRemove)){
            m<- match(clustersToRemove,leg[,"name"] )
            if(any(is.na(m)))
                stop("invalid names of clusters in 'clustersToRemove'")
            clustersToRemove<-as.numeric(leg[m,"clusterIds"])
        }
        if(is.numeric(clustersToRemove)){
            if(any(!clustersToRemove %in% cl)) stop("invalid clusterIds in 'clustersToRemove'")
            if(any(clustersToRemove== -1)) stop("cannot remove -1 clusters using this function. See 'assignUnassigned' to assign unassigned samples.")
            cl[cl %in% clustersToRemove]<- -1
        }
        else stop("clustersToRemove must be either character or numeric")
        if(is.null(clusterLabels)){
            currlabel<-clusterLabels(x)[whCl]
            clusterLabels<-paste0(currlabel,"_unassignClusters")
        }
        if(clusterLabels %in% clusterLabels(x))
            stop("must give a 'clusterLabels' value that is not already assigned to a clustering")
        newleg<-leg
        if(!"-1" %in% leg[,"clusterIds"] & any(cl== -1)){
            newleg<-rbind(newleg,c("-1","white","-1"))
        }
        whRm<-which(as.numeric(newleg[,"clusterIds"]) %in% clustersToRemove )
        if(length(whRm)>0){
            newleg<-newleg[-whRm,,drop=FALSE]
        }
        newCl<-list(newleg)
        #names(newCl)<-clusterLabels
        return(addClusterings(x, cl,  clusterLabels = clusterLabels,clusterLegend=newCl,makePrimary=makePrimary))
        
        
    }
)


#' @details Note that when subsetting the data, the dendrogram information and
#' the co-clustering matrix are lost.
#' @aliases [,ClusterExperiment,ANY,ANY,ANY-method [,ClusterExperiment,ANY,character,ANY-method
#' @param i,j A vector of logical or integer subscripts, indicating the rows and columns to be subsetted for \code{i} and \code{j}, respectively.
#' @param drop A logical scalar that is ignored.
#' @rdname subset
#' @export
setMethod(
    f = "[",
    signature = c("ClusterExperiment", "ANY", "character"),
    definition = function(x, i, j, ..., drop=TRUE) {
        j<-match(j, colnames(x))
        callGeneric()
        
    }
)
#' @rdname subset
#' @export
setMethod(
    f = "[",
    signature = c("ClusterExperiment", "ANY", "logical"),
    definition = function(x, i, j, ..., drop=TRUE) {
        j<-which(j)
        callGeneric()
    }
)
#' @rdname subset
#' @export
setMethod(
    f = "[",
    signature = c("ClusterExperiment", "ANY", "numeric"),
    definition = function(x, i, j, ..., drop=TRUE) {
        # The following doesn't work once I added the logical and character choices.
        # #out <- callNextMethod() #
        # out<-selectMethod("[",c("SingleCellExperiment","ANY","numeric"))(x,i,j) #have to explicitly give the inherintence... not great.
        ###Note: Could fix subsetting, so that if subset on genes, but same set of samples, doesn't do any of this...
        
        #Following Martin Morgan advice, do "new" rather than @<- to create changed object
        #need to subset cluster matrix and convert to consecutive integer valued clusters:
        
        #pull names out so can match it to the clusterLegend. 
#        subMat<-clusterMatrixNamed(x)[j, ,drop=FALSE] 
				
        subMat<-clusterMatrixNamed(x, whichClusters=1:nClusterings(x))[j, ,drop=FALSE] #changed from default "all" because "all" puts primary cluster first so changes order...
        
        #pull out integers incase have changed the "-1"/"-2" to have different names. 
        intMat<-clusterMatrix(x)[j,,drop=FALSE]
				intMat<-.makeIntegerClusters(as.matrix(intMat))
        
        #danger if not unique names
        whNotUniqueNames<-vapply(clusterLegend(x),
            FUN=function(mat){
                length(unique(mat[,"name"]))!=nrow(mat)
            },FUN.VALUE=TRUE)
        if(any(whNotUniqueNames)){
            warning("Some clusterings do not have unique names; information in clusterLegend will not be transferred to subset.")
            subMatInt<-x@clusterMatrix[j, whNotUniqueNames,drop=FALSE]
            subMat[,whNotUniqueNames]<-subMatInt
        }
        nms<-colnames(subMat)
        
        if(nrow(subMat)>0){
            
            ## Fix clusterLegend slot, in case now lost a level and to match new integer values
            ## shouldn't need give colors, but function needs argument
            out<-.makeColors(clMat=subMat, clNumMat=intMat, distinctColors=FALSE,colors=massivePalette,
							matchClusterLegend=clusterLegend(x),matchTo="name") 
            newMat<-out$numClusters
            colnames(newMat)<-nms
            newClLegend<-out$colorList
            
            #fix order of samples so same
            newOrder<-rank(x@orderSamples[j])
            
            
        }
        else{
            newClLegend<-list()
            newOrder<-NA_real_
            newMat<-subMat
        }
        
        out<- ClusterExperiment(  object=as(selectMethod("[",c("SingleCellExperiment","ANY","numeric"))(x,i,j),"SingleCellExperiment"),#have to explicitly give the inherintence... not great.
            clusters = newMat,
            transformation=x@transformation,
            primaryIndex = x@primaryIndex,
            clusterTypes = x@clusterTypes,
            clusterInfo=x@clusterInfo,
            orderSamples=newOrder,
            clusterLegend=newClLegend,
            checkTransformAndAssay=FALSE
        )
        return(out)
    }
)

#' @rdname subset
#' @return \code{subsetByCluster} subsets the object by clusters in a clustering
#' and returns a ClusterExperiment object with only those samples
#' @param clusterValue values of the cluster to match to for subsetting
#' @param matchTo whether to match to the cluster name
#'   (\code{"name"}) or internal cluster id (\code{"clusterIds"})
#' @export
#' @aliases subsetByCluster
setMethod(
    f = "subsetByCluster",
    signature = "ClusterExperiment",
    definition = function(x,clusterValue,whichCluster="primary",matchTo=c("name","clusterIds")) {
        
        whCl<-getSingleClusterIndex(x,whichCluster)
        matchTo<-match.arg(matchTo)
        if(matchTo=="name"){
            cl<-clusterMatrixNamed(x)[,whCl]
        }
        else cl<-clusterMatrix(x)[,whCl]
        return(x[,which(cl %in% clusterValue)])
    }
)

##Note to self:
## 5/9/2019: currently, if classifyMethod=="InSample", cluster.only=TRUE, otherwise FALSE. Our default is "All", so generally doing the cluster.only=FALSE. 


#' @include internalFunctions.R internalClusterFunctions.R internalDendroFunctions.R RcppExports.R subsampleClustering.R
NULL

################
##Internal wrapper functions for kmeans and pam
################
.makeNumeric<-function(x){
    if(is.integer(x)){
        if(!is.null(dim(x))){
            x<-matrix(as.numeric(x),nrow=nrow(x),ncol=ncol(x))	  	
        }
        else x<-as.numeric(x)
    }
    return(x)
}
.genericClassify<-function(inputMatrix, inputType, centers){
    if(inherits(inputMatrix,"DelayedArray") || inherits(centers,"DelayedArray")){
        inputMatrix<-as.matrix(DelayedArray::DelayedArray(inputMatrix))
        centers<-as.matrix(DelayedArray::DelayedArray(centers))
    }
    #avoid integer overflow...
    inputMatrix<-.makeNumeric(inputMatrix)
    centers<-.makeNumeric(centers)
    distMat<-pracma::distmat(t(inputMatrix),centers)
    apply(distMat,1,which.min)	
}
## Function to get the passed args (i.e. from a list(...)) and keep only those that are needed by FUN. If checkArgs=TRUE, then will give a warning that not all of the arguments passed are used by FUN. 
## This is mainly used as a way for all the ClusterFunctions to have a certain required arguments and the others get passed by ... mechanism, and how to keep them apart. 
.myArgNames<-c("removeDup")
.getPassedArgs<-function(FUN,passedArgs,checkArgs){
    funArgs<-names(as.list(args(FUN)))
    funName<-tail(as.character(substitute(FUN)),1)
    whMine<-which(names(passedArgs) %in% .myArgNames)
    if(length(whMine)>0){
        passedArgs<-passedArgs[-whMine]
    }
    if(any(wh<-!names(passedArgs) %in% funArgs)){
        passedArgs<-passedArgs[-which(wh)]
        if(checkArgs) warning(.wrongArgsWarning(funName))
    }
    return(passedArgs)
}
.wrongArgsWarning<-function(funName){paste("arguments passed via clusterArgs to the clustering function",funName,"are not all applicable (clusterArgs should only be arguments to,", funName,"). Extra arguments will be ignored")}

## Function to remove duplicates and return index of old to new
## clusterMat is BxN matrix of clusterings
## mapping is such that
## all(dupTest$smallMat[,dupTest$mapping]==inputMatrix)
## and
## all(dupTest$smallMat[,dupTest$mapping]==inputMatrix)
.dupRemove<-function(clusterMat){
    whDup<-which(duplicated(t(clusterMat)))
    val<-apply(clusterMat,2,paste,collapse=",") #all combinations
    if(length(whDup)>0)
        clusterMat<-clusterMat[,-whDup,drop=FALSE]
    valSm<-apply(clusterMat,2,paste,collapse=",") #unique combinations, in order
    tab<-table(val)[valSm]
    ind<-match(val,valSm)
    return(list(smallMat=clusterMat,mapping=ind,replicates=unname(unclass(tab))))
}

## Function to return "our hamming" distance (1-% clustering shared) between samples. Input a BxN matrix of clusterings
## Best if entries of matrix are already of class integer.
.clustersHammingDistance<-function(clusterMat){
    #clusterMat is BxN (samples in columns)
    idnames<-colnames(clusterMat)
    
    ## FIXME: This conversion of integers could be very slow in large datasets.
    ##Make clusterMat integer, just in case
    ###Make distance matrix
    if(!is.integer(clusterMat)){
        clusterMat <- apply(clusterMat, 2, as.integer)
    }
    clusterMat[clusterMat %in%  c(-1,-2)] <- NA
    ## Since x input is nfeatures x nsamples, 
    ## Takes a BxN matrix, even though NxB is natural way to store (because same format as our clustermatrix)
    sharedPerct <- search_pairs(clusterMat) #works on columns. gives a nsample x nsample matrix back. only lower tri populated
    #make symmetric matrix of % with NAs when all missing
    sharedPerct <- sharedPerct + t(sharedPerct)
    #fix those pairs that have no clusterings for which they are both not '-1' (returned as NAs)
    sharedPerct[is.na(sharedPerct)] <- 0
    sharedPerct[is.nan(sharedPerct)] <- 0
    diag(sharedPerct) <- 1
    rownames(sharedPerct)<-colnames(sharedPerct)<-idnames
    return(1-sharedPerct)
}
#### For handling very small matrices -- mainly just for unit tests
#returning false means should return vector of 1:nrow(inputMatrix)
.checkInput<-function(inputMatrix, requiredN=2,cluster.only){
    if(ncol(inputMatrix)<requiredN){
        if(cluster.only) return(FALSE)
        else stop(sprintf("Must have at least %s observations, cannot create clustering object for cluster.only=FALSE"),requiredN)
    }
    else return(TRUE)
}
### Make a cluster based on unique values of rows of inputMatrix
### inputMatrix has samples in columns
.uniqueCluster<-function(inputMatrix, minSize=1){
    clusterComb <- apply(inputMatrix, 2, paste, collapse=";")
    #create what all -1 values would be
    allUnass <- paste(rep("-1", length=nrow(inputMatrix)), collapse=";")
    tab <-	table(clusterComb)
    tab <- tab[tab >= minSize]
    tab <- tab[names(tab) != allUnass]
    cl <- match(clusterComb, names(tab))
    cl[is.na(cl)] <- -1
    return(cl)
}


# ##---------
# ##KNN for consensus
# ##---------
# .knnHamming<-function(cat,checkArgs,inputType,cluster.only,...){
# 	passedArgs<-.getPassedArgs(FUN= .igraphKnnHamming ,passedArgs=list(...) ,checkArgs=checkArgs)
#
# }
# .knn01<-ClusterFunction(clusterFUN=.knnHamming, classifyFUN=NULL, inputType="cat", algorithmType="01",outputType="vector")
# .knnK<-ClusterFunction(clusterFUN=.knnHamming, classifyFUN=NULL, inputType="cat", algorithmType="01",outputType="vector")
#
# ## Temporary function to create distance matrix based on our hamming distance, and then create graph and create igraph object.
# ## Total hack to get the infrastructure in place -- cannot use for real sized data!!



##---------
##Spectral
##---------

# spectral options (SamSPECTRAL for flow cytometry (function SamSPECTRAL); kernlab for standard ('specc'); kknn for similarity based on knn rather than kmeans): kernlab is either x or a kernel function
#' @importFrom kernlab specc
.speccCluster<-function(inputMatrix,k,checkArgs,inputType,cluster.only,...){
    passedArgs<-.getPassedArgs(FUN=kernlab::specc,passedArgs=list(...) ,checkArgs=checkArgs)
    #Checks for enough observations
    check<-.checkInput(inputMatrix,cluster.only)
    if(!check) return(.uniqueCluster(inputMatrix))
    #add data.matrix here for hdf5Matrix. Not optimized
    out<-try(do.call(kernlab::specc,c(list(x=data.matrix(t(inputMatrix)), centers=k),passedArgs)))
    if(inherits(out,"try-error"))stop("Spectral clustering failed, probably because k (",k,") was too large relative to the number of samples (",ncol(inputMatrix),"). k must be less than the number of samples, but how much less is not straightforward.")
    if(cluster.only) return(out@.Data)
    else return(out) 
}
.speccCF<-ClusterFunction(clusterFUN=.speccCluster, classifyFUN=NULL, inputType="X", algorithmType="K",outputType="vector",checkFunctions=FALSE)


##---------
##Kmeans
##---------
#' @importFrom stats kmeans
.kmeansCluster <- function(inputMatrix,k, checkArgs,inputType,cluster.only,...) { 
    passedArgs<-.getPassedArgs(FUN=stats::kmeans,passedArgs=list(...) ,checkArgs=checkArgs)
    #Checks for enough observations
    check<-.checkInput(inputMatrix,cluster.only)
    if(!check) return(.uniqueCluster(inputMatrix))
    out<-do.call(stats::kmeans,
        c(list(x=t(inputMatrix),centers=k),passedArgs))
    if(cluster.only) return(out$cluster)
    else return(.kmeansPartitionObject(inputMatrix,out)) 
} 
.kmeansClassify <- function(inputMatrix, inputType, clusterResult) { 
    suppressWarnings(stats::kmeans(t(inputMatrix), clusterResult$medoids, iter.max = 1, algorithm = "Lloyd")$cluster) #probably uses this so always classifies points to centers
} 
#make partition object same form as pam output
#' @importFrom cluster daisy silhouette
.kmeansPartitionObject<-function(x,kmeansObj){ 
    #This is hugely computationally expensive and don't need it! 
    # dissE<-(cluster::daisy(t(x)))^2
    # silObj<-try(cluster::silhouette(x=kmeansObj$cluster,dist=dissE),silent=TRUE)
    # if(!inherits(silObj,"try-error"))
    #   silinfo<-list(widths=silObj, clus.avg.widths=summary(silObj)$clus.avg.widths, ave.width=summary(silObj)$avg.width)
    # else silinfo<-NULL
    return(list(medoids=kmeansObj$centers, clustering=kmeansObj$cluster, call=NA,silinfo=NULL, objective=NA, diss=NULL, data=x))
}
.kmeansCF<-ClusterFunction(clusterFUN=.kmeansCluster, classifyFUN=.kmeansClassify, inputType="X", inputClassifyType="X", algorithmType="K",outputType="vector",checkFunctions=FALSE)
#internalFunctionCheck(.kmeansCluster,inputType="X",algType="K",outputType="vector")


##---------
##Mini-batch Kmeans
##---------
# mbkmeans(x, clusters, batch_size = ifelse(ncol(x) > 100,
#   ceiling(ncol(x) * 0.05), ncol(x)), max_iters = 100, num_init = 1,
#   init_fraction = ifelse(ncol(x) > 100, 0.25, 1),
#   initializer = "kmeans++", calc_wcss = FALSE, early_stop_iter = 10,
#   verbose = FALSE, CENTROIDS = NULL, tol = 1e-04)
.mbkmeansCluster <- function(inputMatrix,k, checkArgs,inputType,cluster.only,...) { 
    passedArgs<-.getPassedArgs(FUN=mbkmeans::mbkmeans,
        passedArgs=list(...) ,checkArgs=checkArgs)
    #Checks for enough observations
    check<-.checkInput(inputMatrix,cluster.only)
    if(!check) return(.uniqueCluster(inputMatrix))
    out<-do.call(mbkmeans::mbkmeans,
        c(list(x=inputMatrix,clusters=k),passedArgs))
    if(cluster.only) return(out$Clusters)
    else return(out) 
} 
.mbkmeansClassify <- function(inputMatrix, inputType, clusterResult) {
    #This will not be HDF5 friendly...will have to bring it into memory...
    #Should write a C++ code to do this?? Maybe in mbkmeans? 
    suppressWarnings(stats::kmeans(t(inputMatrix), clusterResult$centroids, iter.max = 1, algorithm = "Lloyd")$cluster) 
} 
.mbkmeansCF<-ClusterFunction(clusterFUN=.mbkmeansCluster, classifyFUN=.mbkmeansClassify, inputType="X", inputClassifyType="X", algorithmType="K",outputType="vector",checkFunctions=FALSE)

#internalFunctionCheck(.mbkmeansCluster,inputType="X",algorithmType="K",outputType="vector")


##---------
##PAM
##---------

#' @importFrom cluster pam
.pamCluster<-function(inputMatrix,k,checkArgs,inputType,cluster.only,...){
    passedArgs<-list(...)
    convertCat<-FALSE
    #Checks for enough observations
    check<-.checkInput(inputMatrix,cluster.only)
    if(!check) return(.uniqueCluster(inputMatrix))

    if(inputType=="cat"){
        if(is.null(passedArgs[["removeDup"]]) || passedArgs[["removeDup"]]){
            inputMatrix<-.dupRemove(inputMatrix)
            mapping<-inputMatrix$mapping            
            convertCat<-TRUE
            inputMatrix<-.clustersHammingDistance(inputMatrix$smallMat)
            
        }
        else inputMatrix<-.clustersHammingDistance(inputMatrix)
        inputType<-"diss"
    }
    passedArgs<-.getPassedArgs(FUN=cluster::pam, passedArgs=passedArgs, 
        checkArgs=checkArgs)
    ## prioritize "diss", because otherwise pam just creates the diss matrix.
    if(inputType=="diss"){
        #Checks for enough observations
        check<-.checkInput(inputMatrix,requiredN=k+1,
            cluster.only=cluster.only)
        if(!check){
            out<-1:nrow(inputMatrix)
        }
        else out<-do.call(cluster::pam, c(list(x=inputMatrix, k=k, diss=TRUE, cluster.only=cluster.only), passedArgs))
        if(convertCat){
            if(cluster.only) out<-out[mapping]
            else{
                #this is a problem for silhouette stuff!
                out$clustering<-out$clustering[mapping]
            }
        }
        return(out)
    }
    if(inputType=="X"){
        return(do.call(cluster::pam, c(list(x=t(inputMatrix),k=k, cluster.only=cluster.only), passedArgs)))  
    } 
}
.pamClassify <- function(inputMatrix, inputType,clusterResult) { #x p x n matrix
    .genericClassify(inputMatrix,centers=clusterResult$medoids)
} 
.pamCF<-ClusterFunction(clusterFUN=.pamCluster, classifyFUN=.pamClassify, inputType=c("X","diss","cat"), inputClassifyType="X", algorithmType="K",outputType="vector",checkFunctions=FALSE)

#internalFunctionCheck(.pamCluster,inputType=c("x","diss"),algType="K",outputType="vector")

##---------
##clara
##---------

#' @importFrom cluster pam
.claraCluster<-function(inputMatrix,k,checkArgs,inputType,cluster.only,samples=50,keep.data=FALSE,rngR=TRUE,pamLike=TRUE,correct.d=TRUE,medoids.x=FALSE,...){
    #Checks for enough observations
    check<-.checkInput(inputMatrix,cluster.only)
    if(!check) return(.uniqueCluster(inputMatrix))
    passedArgs<-.getPassedArgs(FUN=cluster::clara,passedArgs=list(...) ,checkArgs=checkArgs)
    passedArgs<-c(passedArgs, list(samples=samples, keep.data=keep.data, rngR=rngR, pamLike=pamLike, correct.d=correct.d))
    out<-(do.call(cluster::clara, c(list(x=t(inputMatrix),k=k), passedArgs)))
    if(cluster.only) return(out$clustering) else return(out)
    
}

.claraCF<-ClusterFunction(clusterFUN=.claraCluster, classifyFUN=.pamClassify, inputType="X", inputClassifyType="X", algorithmType="K",outputType="vector",checkFunctions=FALSE)


##---------
##hiearchicalK
##---------
#' @importFrom stats hclust cutree
.hierKCluster<-function(inputMatrix,k,checkArgs,inputType,cluster.only,...){
    passedArgs<-list(...)
    convertCat<-FALSE
    #Checks for enough observations
    check<-.checkInput(inputMatrix,cluster.only)
    if(!check) return(.uniqueCluster(inputMatrix))
    passedArgs<-.getPassedArgs(FUN=stats::hclust,passedArgs=passedArgs,
        checkArgs=checkArgs)

    if(inputType=="cat"){
        if(is.null(passedArgs[["removeDup"]]) || passedArgs[["removeDup"]]){
            inputMatrix<-.dupRemove(inputMatrix)
            mapping<-inputMatrix$mapping
            convertCat<-TRUE
            inputMatrix<-.clustersHammingDistance(inputMatrix$smallMat)
        }
        else inputMatrix<-.clustersHammingDistance(inputMatrix)
        
    }
    
    check<-.checkInput(inputMatrix,requiredN=k+1,cluster.only=cluster.only)
    if(!check) out<-1:nrow(inputMatrix)
    else{
        hclustOut<-do.call(stats::hclust,
            c(list(d=as.dist(inputMatrix)),passedArgs))
        out<-stats::cutree(hclustOut,k)
    }
    if(inputType=="cat" & convertCat){
        out<-out[mapping]
    }
    return(out)
}
.hierKCF<-ClusterFunction(clusterFUN=.hierKCluster, inputType=c("diss","cat"), algorithmType="K",outputType="vector",checkFunctions=FALSE)

#internalFunctionCheck(.hierKCluster,inputType="diss",algType="K",outputType="vector")


##---------
##Hiearchical01
##---------
#' @importFrom stats hclust 
.hier01Cluster<-function(inputMatrix,alpha,evalClusterMethod=c("maximum","average"),whichHierDist=c("as.dist","dist"),checkArgs,inputType,cluster.only,...)
{
    #Checks for enough observations
    check<-.checkInput(inputMatrix,cluster.only)
    if(!check) return(.uniqueCluster(inputMatrix))
    whichHierDist<-match.arg(whichHierDist)
    evalClusterMethod<-match.arg(evalClusterMethod)
    passedArgs<-list(...)
    convertCat<-FALSE
    if(inputType=="cat"){
        if(is.null(passedArgs[["removeDup"]]) || passedArgs[["removeDup"]]){
            inputMatrix<-.dupRemove(inputMatrix)
            mapping<-inputMatrix$mapping            
            dupTab<-inputMatrix$replicates
            convertCat<-TRUE
            inputMatrix<-.clustersHammingDistance(inputMatrix$smallMat)
        }
        else inputMatrix<-.clustersHammingDistance(inputMatrix)
        
    }
    # CHECKME: I changed this to make this for all, not just if missing, so as to guarantee that they are unique, but I don't know if it will break something downstream
    check<-.checkInput(inputMatrix,cluster.only=cluster.only)
    if(!check) return(lapply(1:nrow(inputMatrix),function(x){x}))
    rownames(inputMatrix)<- colnames(inputMatrix)<- 
        as.character(seq_len(nrow(inputMatrix)))
    passedArgs<-.getPassedArgs(FUN=stats::hclust,passedArgs=passedArgs,
        checkArgs=checkArgs)
    ## FIXME: requires S, i.e. similarity. Need to change to only need dissimilarity because copying the large matrix (not only here, but below too).
    S<-round(1-inputMatrix,10)
    d<-switch(whichHierDist,"dist"=dist(S),"as.dist"=as.dist(inputMatrix))
    hDmat<-do.call(stats::hclust,c(list(d=d),passedArgs))##Could this be just done by cut with hierarchical cophenic value? Should make it an option to do that. Probably a lot faster...
    method<-evalClusterMethod
    phylo4Obj<-.convertToPhyClasses(hDmat,returnClass=c("phylo4"))
    allTips<-phylobase::getNode(phylo4Obj,  type=c("tip"))
    
    ###
    #the following code 
    #each internal node (starting at root) calculate whether passes value of alpha or not
    ##
    
    ##Initialize:
    nodesToCheck<-phylobase::rootNode(phylo4Obj)
    clusterList<-list()
    
    while(length(nodesToCheck)>0){
        currNode<-nodesToCheck[1]
        nodesToCheck<-nodesToCheck[-1]
        if(currNode%in%allTips){ #block of size 1!
            currTips<-names(currNode)
            check<-TRUE
        }
        else{
            currTips<-names(phylobase::descendants(phylo4Obj,currNode,"tip"))
            
            if(method=="maximum"){
                check<-all(S[currTips,currTips,drop=FALSE]>=(1-alpha))
                
            }
            if(method=="average"){
                if(convertCat){
                    m<-match(currTips,rownames(inputMatrix))
                    rm<- rowSums(
                        sweep(S[currTips,currTips,drop=FALSE],
                            2, dupTab[m], FUN="*")
                        )
                    rm<-rm/sum(dupTab[m])
                    # rm2<- rowMeans(
                   #      S[currTips,rep(currTips,times=dupTab[m]),drop=FALSE]
                   #      )
                   #  if(!is.logical(all.equal(rm,rm2))) browser()
                }
                else{
                    rm<-rowMeans(S[currTips,currTips,drop=FALSE])
                }
                check<-all(rm>=(1-alpha))
            } 
            
        }
        if(check){ #found a block that satisfies
            clusterList<-c(clusterList,list(currTips))
        }
        else{ #not satisfy, add children to check.
            childNodes<-phylobase::descendants(phylo4Obj,currNode,"children")
            nodesToCheck<-c(nodesToCheck,childNodes)
        }
    }
    clusterListIndices<-lapply(clusterList,function(tipNames){
        match(tipNames,rownames(inputMatrix))
    })
    clusterListIndices<-.orderByAlpha(clusterListIndices,S)
    
    if(convertCat){
        ## FIXME: this seems very akward. Should be able to do better!
        clusterListIndices<-lapply(clusterListIndices,function(x){
            unlist(lapply(x,function(i){which(mapping==i)}))
        })
    }
    return(clusterListIndices)
}
.hier01CF<-ClusterFunction(clusterFUN=.hier01Cluster, inputType=c("diss","cat"), algorithmType="01",outputType="list",checkFunctions=FALSE)
#internalFunctionCheck(.hier01Cluster,inputType=c("diss","cat"),algorithmType="01",outputType="list")


##---------
##Tight
##---------
.tightCluster <- function(inputMatrix, alpha, minSize.core=2,checkArgs,inputType,cluster.only,...)
{
    #Checks for enough observations
    check<-.checkInput(inputMatrix,cluster.only)
    if(!check) return(.uniqueCluster(inputMatrix))
    passedArgs<-list(...)
    convertCat<-FALSE
    if(inputType=="cat"){
        if(is.null(passedArgs[["removeDup"]]) || passedArgs[["removeDup"]]){
            inputMatrix<-.dupRemove(inputMatrix)
            mapping<-inputMatrix$mapping            
            dupTab<-inputMatrix$replicates
            convertCat<-TRUE
            inputMatrix<-.clustersHammingDistance(inputMatrix$smallMat)
        }
        else inputMatrix<-.clustersHammingDistance(inputMatrix)
    }
    check<-.checkInput(inputMatrix,cluster.only=cluster.only)
    if(!check) return(lapply(1:nrow(inputMatrix),function(x){x}))
   
    #previously, diss was similarity matrix. To make it match all of the code, I need it to be diss=1-similarity so now convert it back
    inputMatrix<-1-inputMatrix #now is similarity matrix...
    if(!is.null(passedArgs[["removeDup"]])){
        passedArgs<-passedArgs[-grep("removeDup",names(passedArgs))]
    }
    if(length(passedArgs)>0 & checkArgs){
        warning(.wrongArgsWarning("tight"))
    } 	
    find.candidates.one <- function(x) {
        tmp <- apply(x >= 1, 1, sum) #how many in each row ==1
        #what if none of them are ==1? will this never happen because have sample of size 1? Depends what diagonal is.
        if(all(tmp<minSize.core)){ #i.e. only core size groups less than minSize.core (default is 1)
            return(NULL)
        }
        whMax<-which(tmp == max(tmp))
        return(which(x[, whMax[1]] >= 1)) # assumes x is symmetric. Takes largest in size, but arbitrarily picks between them.
    }
    extend.candidate <- function(S, can, alpha ) {
        can.ex <- which(apply(as.matrix(S[, can] >= 1 - alpha), 1, all)) #find those that are close to those core with 1
        S.temp <- S[can.ex, can.ex]
        if (!is.matrix(S.temp)) {
            S.temp <- as.matrix(S.temp)
            colnames(S.temp) <- names(can.ex)
        }
        S.bad <- apply(as.matrix(S.temp < 1 - alpha), 1,sum)
        while (sum(S.bad) > 0) {
            index <- which(S.bad == max(S.bad))[1]
            S.temp <- S.temp[-index, -index]
            S.bad <- apply(as.matrix(S.temp < 1 - alpha),
                           1, sum)
        }
        return(can.ex[colnames(S.temp)])
    }
    N<-nrow(inputMatrix)
    colnames(inputMatrix) <- seq_len(N)
    rownames(inputMatrix) <- seq_len(N)
    i <- 1
    ## FIXME: duplicating inputMatrix!
    S.temp <- inputMatrix
    res <- list()
    while (!is.null(dim(S.temp)) && !is.null(dim(S.temp)) && nrow(S.temp) > 0 & any(S.temp[lower.tri(S.temp)]>1-alpha) & any(S.temp[lower.tri(S.temp)]==1)) {
        #first find those that are always together (resampling =1); pick the largest such group (and if more than 1 of same size will arbitrarily pick one)
        candidate.one <- find.candidates.one(S.temp)
        if(is.null(candidate.one)){#no more candidates with core always together
            #for now just stop if no core group
            break
        }
        #add more on the group if always resamples with the core members >alpha proportion of the time
        candidate <- extend.candidate(S.temp, candidate.one, alpha = alpha)
        S.temp <- S.temp[-candidate, -candidate]
        res[[i]] <- names(candidate)
        mode(res[[i]]) <- "numeric"
        i = i + 1
    }
    res<-.orderByAlpha(res,inputMatrix)
    if(inputType=="cat" & convertCat){
        ## FIXME: this seems very akward. Should be able to do better!
        res<-lapply(res,function(x){
            unlist(lapply(x,function(i){which(mapping==i)}))
        })
    }
    return(res)
    
}
.tightCF<-ClusterFunction(clusterFUN=.tightCluster, inputType=c("diss","cat"), algorithmType="01",outputType="list",checkFunctions=FALSE)


#########
## Put them together so user/code can access easily
#########
.builtInClusterObjects<-list("pam"=.pamCF,"clara"=.claraCF,"kmeans"=.kmeansCF,"hierarchical01"=.hier01CF,"hierarchicalK"=.hierKCF,"tight"=.tightCF,"spectral"=.speccCF,"mbkmeans"=.mbkmeansCF)
.builtInClusterNames<-names(.builtInClusterObjects)

#' @title Built in ClusterFunction options
#' @param object name of built in function.
#' @description Documents the built-in clustering options that are available in
#'   the clusterExperiment package.
#' @details \code{listBuiltInFunctions} will return the character names of the
#'   built-in clustering functions available.
#' @details \code{listBuiltInTypeK} returns the names of the built-in functions
#'   that have type 'K'
#' @details \code{listBuiltInType01} returns the names of the built-in functions
#'   that have type '01'
#' @details \code{getBuiltInFunction} will return the \code{ClusterFunction}
#'   object of a character value that corresponds to a built-in function.
#' @details \code{\link{algorithmType}} and \code{\link{inputType}} will return
#'   the \code{algorithmType} and \code{inputType} of the built-in
#'   clusterFunction corresponding to the character value.
#' @details \strong{Built-in clustering methods:} The built-in clustering
#'   methods, the names of which can be accessed by
#'   \code{listBuiltInFunctions()} are the following:
#' \itemize{ 
#' \item{"pam"}{Based on \code{\link[cluster]{pam}} in
#'   \code{cluster} package. Arguments to that function can be passed via
#'   \code{clusterArgs}. 
#' Input can be either \code{"x"} or \code{"diss"}; algorithm type is "K"} 
#' \item{"clara"}{Based on \code{\link[cluster]{clara}} in
#'   \code{cluster} package. Arguments to that function can be passed via
#'   \code{clusterArgs}. Note that we have changed the default arguments of 
#'   that function to match the recommendations in the documentation of
#'  \code{\link[cluster]{clara}} (numerous functions are set to less than optimal 
#'  settings for back-compatiability). Specifically, the following defaults 
#'  are implemented \code{samples=50}, \code{keep.data=FALSE}, 
#' \code{mediods.x=FALSE},\code{rngR=TRUE},
#'  \code{pamLike=TRUE}, \code{correct.d=TRUE}. 
#' Input is \code{"X"}; algorithm type is "K".} 
#' \item{"kmeans"}{Based on \code{\link[stats]{kmeans}} in \code{stats} package.
#' Arguments to that function can be passed via \code{clusterArgs} except for
#' \code{centers} which is reencoded here to be the argument 'k' Input is
#' \code{"X"}; algorithm type is "K"}
#' \item{"mbkmeans"}{\code{\link[mbkmeans]{mbkmeans}} runs mini-batch kmeans, a more computationally efficient version of kmeans. }
#' \item{"hierarchical01"}{\code{\link[stats]{hclust}} in \code{stats} package
#' is used to build hiearchical clustering. Arguments to that function can be
#' passed via \code{clusterArgs}. The \code{hierarchical01} cuts the hiearchical
#' tree based on the parameter \code{alpha}. It does not use the \code{cutree}
#' function, but instead transversing down the tree until getting a block of
#' samples with whose summary of the values  is greater than or equal to
#' 1-alpha. Arguments that can be passed to 'hierarchical01' are
#' 'evalClusterMethod' which determines how to summarize the samples' values of
#' D[samples,samples] for comparison to 1-alpha: "maximum" (default) takes the
#' minimum of D[samples,samples] and requires it to be less than or equal to
#' 1-alpha; "average" requires that each row mean of D[samples,samples] be less
#' than or equal to 1-alpha. Additional arguments of hclust can also be passed
#' via clusterArgs to control the hierarchical clustering of D. Input is
#' \code{"diss"}; algorithm type is "01"}
#' \item{"hierarchicalK"}{\code{\link[stats]{hclust}} in \code{stats} package is used
#'   to build hiearchical clustering and \code{\link{cutree}} is used to cut the
#'   tree into \code{k} clusters.
#' Input is \code{"diss"}; algorithm type is "K"}   
#' \item{"tight"}{Based on the algorithm in
#'   Tsang and Wong, specifically their method of picking clusters from a
#'   co-occurance matrix after subsampling. The clustering encoded here is not
#'   the entire tight clustering algorithm, only that single piece that
#'   identifies clusters from the co-occurance matrix.  
#' Arguments for the tight method are 
#'   'minSize.core' (default=2), which sets the minimimum number of samples that
#'   form a core cluster.
#' Input is \code{"diss"}; algorithm type is "01"} 
#' \item{"spectral"}{\code{\link[kernlab]{specc}} in \code{kernlab} package 
#' is used to perform spectral clustering. Note that spectral clustering can 
#' produce errors if the number of clusters (K) is not sufficiently smaller than 
#' the number of samples (N). K < N is not always sufficient. 
#' Input is \code{"X"}; algorithm type is "K".}
#' }
#' @seealso \code{\link{ClusterFunction}}, \code{\link{algorithmType}},
#'   \code{\link{inputType}}
#' @examples
#' listBuiltInFunctions()
#' algorithmType(c("kmeans","pam","hierarchical01"))
#' inputType(c("kmeans","pam","hierarchical01"))
#' listBuiltInTypeK()
#' listBuiltInType01()
#' @rdname builtInClusteringFunctions
#' @aliases listBuiltInFunctions
#' @return \code{listBuiltInFunctions} returns a character vector of all 
#' the built-in cluster functions' names.
#' @export
listBuiltInFunctions<-function() {
    .builtInClusterNames
    
}
#' @rdname builtInClusteringFunctions
#' @aliases getBuiltInFunction
#' @return \code{getBuiltInFunction} returns the \code{ClusterFunction} 
#' object that corresponds to the character name of a function
#' @export
setMethod(
    f = "getBuiltInFunction",
    signature = c("character"),
    definition = function(object) {
        if(!all(object%in%.builtInClusterNames)) stop("if give character value for a clusterFunction object must be one of",paste(.builtInClusterNames,collapse=","))
        m<-match(object,names(.builtInClusterObjects))
        if(length(m)>1) .builtInClusterObjects[m]
        else .builtInClusterObjects[[m]]
        
        
    }
)

#' @rdname builtInClusteringFunctions
#' @aliases listBuiltInTypeK
#' @return \code{listBuiltInTypeK} returns a character vector of the 
#' names of built-in cluster functions that are of type "K"
#' @export
listBuiltInTypeK<-function() {
    allBuiltInTypes<-algorithmType(.builtInClusterNames)
    return(names(allBuiltInTypes)[allBuiltInTypes=="K"])
}

#' @rdname builtInClusteringFunctions
#' @aliases listBuiltInType01
#' @return \code{listBuiltInType01} returns a character vector of the 
#' names of built-in cluster functions that are of type "01"
#' @export
listBuiltInType01<-function() {
    allBuiltInTypes<-algorithmType(.builtInClusterNames)
    return(names(allBuiltInTypes)[allBuiltInTypes=="01"])
}

context("clusterMany")


test_that("`clusterMany` works with matrix, list of data, ClusterExperiment objects, and
          SummarizedExperiments", {
			  #check all builtin methods
    expect_silent(clustNothing <- clusterMany(mat, 
		ks=c(3,4),clusterFunction=listBuiltInFunctions(),
        subsample=FALSE, sequential=FALSE,
        isCount=FALSE,verbose=FALSE, makeMissingDiss=TRUE))
	expect_silent(clustDF <- clusterMany(data.frame(mat),
		ks=c(3,4),clusterFunction=listBuiltInFunctions(),
		subsample=FALSE, sequential=FALSE,
		isCount=FALSE,verbose=FALSE, makeMissingDiss=TRUE))
		   
    expect_is(clustNothing, "ClusterExperiment")
    expect_is(clustNothing, "SingleCellExperiment")
    expect_is(clustNothing, "SummarizedExperiment")

    expect_silent(clustNothing2 <- clusterMany(se, ks=c(3,4),clusterFunction="pam",
       subsample=FALSE, sequential=FALSE,
       isCount=FALSE,verbose=FALSE))
    expect_equal(colData(clustNothing2),colData(se))
    expect_equal(rownames(clustNothing2),rownames(se))
    expect_equal(colnames(clustNothing2),colnames(se))
    expect_equal(metadata(clustNothing2),metadata(se))
    expect_equal(rowData(clustNothing2),rowData(se))

    expect_equal(clusterMatrix(clustNothing), clusterMatrix(clustNothing))
    expect_true(all(clusterTypes(clustNothing)=="clusterMany"))

    #test running on ClusterExperiment Object -- should add the new clustering
    expect_silent(clustNothing3 <- clusterMany(ccSE, 
        ks=c(3,4),clusterFunction="pam",
        subsample=FALSE, sequential=FALSE, verbose=FALSE))
    expect_true(nClusterings(clustNothing3) == nClusterings(ccSE) + 2)
    expect_equal(colData(clustNothing3),colData(ccSE))
    expect_equal(rownames(clustNothing3),rownames(ccSE))
    expect_equal(colnames(clustNothing3),colnames(ccSE))
    expect_equal(metadata(clustNothing3),metadata(ccSE))
    expect_equal(rowData(clustNothing3),rowData(ccSE))
    
    expect_silent(test <- clusterSingle(se,  subsample=FALSE, 
        sequential=FALSE, 
        mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=4)),
        isCount=FALSE))
    expect_silent(clustNothing3<- clusterMany(test, 
        ks=c(3,4),clusterFunction="pam",
        subsample=FALSE, sequential=FALSE, verbose=FALSE))
    expect_silent(clustNothing4<- clusterMany(clustNothing3, 
        ks=c(3:4),clusterFunction="pam",
        subsample=FALSE, sequential=FALSE,verbose=FALSE, eraseOld=TRUE))
    expect_equal(clustNothing3,clustNothing4)

    expect_silent(clustNothing5<- clusterMany(clustNothing3, 
        ks=c(5:6),clusterFunction="pam",
        subsample=FALSE, sequential=FALSE,verbose=FALSE, eraseOld=FALSE))
    expect_equal(NCOL(clusterMatrix(clustNothing5)),5)

    expect_silent(ppIndex<-workflowClusterDetails(clustNothing5))
    expect_equal(as.numeric(table(ppIndex[,"iteration"])),c(2,2))
})


test_that("`clusterMany` works with SingleCellExperiment", {
  #check with sce that has dimRed:
  #takes a while with all functions, but sometimes turn up surprises.
  #for some reason if do clusterFunction=listBuiltInFunctions(), 
  #    expect_silent gets warning, but not if run myself.
  for(kk in 1:length(listBuiltInFunctions())){
	  expect_silent(clustNothing2 <- clusterMany(sceSimDataDimRed,
		   ks=c(3,4),clusterFunction=listBuiltInFunctions()[[kk]],
           makeMissingDiss=TRUE,
	       subsample=FALSE, sequential=FALSE, isCount=FALSE,verbose=FALSE))  	
  }
  expect_equal(colData(clustNothing2),colData(sceSimDataDimRed))
  expect_equal(rownames(clustNothing2),rownames(sceSimDataDimRed))
  expect_equal(colnames(clustNothing2),colnames(sceSimDataDimRed))
  expect_equal(metadata(clustNothing2),metadata(sceSimDataDimRed))
  expect_equal(rowData(clustNothing2),rowData(sceSimDataDimRed))
  expect_equal(reducedDims(clustNothing2),reducedDims(sceSimDataDimRed))



})

# # FIXME: need to make this operational
# test_that("`clusterMany` matches `clusterSingle` results", {
#
#     subArgs<-list(clusterFunction="kmeans",classifyMethod="All")
#     expect_message(ce<-clusterMany(smSimSE,
#         isCount = FALSE,
#         ks = 4:5,
#         alphas=c(0.1),
#         reduceMethod="PCA",
#         nReducedDims=10,
#         minSizes=1,
#         subsample=TRUE,
#         sequential=TRUE,
#         clusterFunction="hierarchical01",
#         subsampleArgs=subArgs,
#         makeMissingDiss=TRUE, verbose=FALSE,
#         random.seed=176201),"Not all of the methods requested in 'reduceMethod' have been calculated")
#
#     mainArgs<-list( clusterFunction="hierarchical01",minSize=1,
#         clusterArgs=list(alpha=0.1))
#     seqArgs<-list(k0=4,beta=0.9)
#     set.seed(176201)
#     expect_silent(ce1<-clusterSingle(smSimSE,
#         isCount = FALSE,
#         saveSubsamplingMatrix=TRUE,
#         reduceMethod="PCA",
#         nDims=10,
#         subsample=TRUE,
#         sequential=TRUE,
#         mainClusterArgs=mainArgs,
#         subsampleArgs=subArgs,
#         seqArgs=seqArgs,
#         makeMissingDiss=TRUE
#         ))
#     expect_equal(primaryCluster(ce),primaryCluster(ce1))
#
# })
test_that("`clusterMany` works with reduceMethod a reducedDims",{
  #check picking all dims in single reduceMethod same as apply directly to matrix 
    expect_silent(clustNothing <-
        clusterMany(t(reducedDims(sceSimDataDimRed)[["PCA"]]),
            ks=c(3,4),clusterFunction="pam", 
            reduceMethod="none",
            subsample=FALSE, sequential=FALSE, 
            isCount=FALSE,verbose=FALSE))
    expect_silent(clustNothing3 <- 
        clusterMany(sceSimDataDimRed, 
            ks=c(3,4),clusterFunction="pam", 
            reduceMethod="PCA",
            subsample=FALSE, sequential=FALSE, 
            isCount=FALSE,verbose=FALSE))
    expect_equal(clusterMatrix(clustNothing), clusterMatrix(clustNothing3))
    expect_equal(NCOL(clusterMatrix(clustNothing)),2)
    expect_equal(NCOL(clusterMatrix(clustNothing3)),2)
    expect_equal(NCOL(clusterMatrix(clustNothing3)),2)
    expect_equal(clusterMatrix(clustNothing), clusterMatrix(clustNothing3))

    # check picking certain dims in single 
    # reduceMethod same as apply directly to matrix 
    # and that get right reducedDim returned
    expect_silent(clustNothing <- 
        clusterMany(simData, 
            ks=c(3,4),nReducedDims=c(5:6),
            reduceMethod="PCA",
            clusterFunction="pam", subsample=FALSE, 
            sequential=FALSE, isCount=FALSE,verbose=FALSE))
    expect_equal(NCOL(clusterMatrix(clustNothing)),4)
    expect_equal(abs(reducedDim(clustNothing,"PCA")),
        abs(reducedDim(sceSimDataDimRed,"PCA")[,1:6]))

    expect_silent(clustNothing2 <- 
        clusterMany(sceSimDataDimRed, 
        ks=c(3,4),nReducedDims=c(5:6),
        reduceMethod="PCA",
        clusterFunction="pam", subsample=FALSE, 
        sequential=FALSE, isCount=FALSE,verbose=FALSE))
    expect_equal(NCOL(clusterMatrix(clustNothing2)),4)
    expect_equal(reducedDim(clustNothing2,"PCA"), 
        reducedDim(sceSimDataDimRed,"PCA"))

    expect_message(clustNothing4 <- 
        clusterMany(sceSimData, 
            ks=c(3,4),nReducedDims=c(5:6),
            reduceMethod="PCA", clusterFunction="pam", 
            subsample=FALSE, sequential=FALSE, 
            isCount=FALSE,verbose=FALSE),
            "Not all of the methods requested in 'reduceMethod' have been calculated")
    expect_equal(clusterMatrix(clustNothing2), 
        clusterMatrix(clustNothing4))
  
    expect_message(clustNothingBoth1<-
        clusterMany(clustNothing4,
            ks=c(3,4),nReducedDims=c(5:6),
            reduceMethod=c("PCA","mad"),
            clusterFunction="pam", subsample=FALSE, 
            sequential=FALSE, verbose=FALSE),
            "Not all of the methods requested in 'reduceMethod' have been calculated")
		## Issue with message created "Found more than one class "Annotated" in cache; using the first, from namespace 'S4Vectors'" message
		## Now just checks no new warnings.
    expect_silent(
    suppressMessages(    clustNothingBoth2<-
        clusterMany(clustNothingBoth1, 
            ks=c(3,4),nReducedDims=c(5:6),
            reduceMethod=c("PCA","mad"),
            clusterFunction="pam", subsample=FALSE, 
            sequential=FALSE, verbose=FALSE)
						)
    )
    expect_equal(clusterMatrix(clustNothingBoth1,
        which="clusterMany"), clusterMatrix(clustNothingBoth2,
        which="clusterMany"))

    #check picking reduceMethod="none" same as apply directly to matrix 
    expect_silent(clustNothing <- 
        clusterMany(simData, ks=c(3,4),
            clusterFunction="pam", subsample=FALSE, 
            sequential=FALSE, isCount=FALSE,verbose=FALSE))
    expect_silent(clustNothing3 <- 
        clusterMany(sceSimDataDimRed, ks=c(3,4),
            clusterFunction="pam", reduceMethod="none",
            subsample=FALSE, sequential=FALSE, 
            isCount=FALSE,verbose=FALSE))
    expect_equal(clusterMatrix(clustNothing), clusterMatrix(clustNothing3))

    #checks that nReducedDims ignored if reduceMethod="none"
    expect_silent(clustNothing <- 
        clusterMany(simData,
            ks=c(3,4),nReducedDims=c(5:6),
            reduceMethod="none",
            clusterFunction="pam", subsample=FALSE, 
            sequential=FALSE, isCount=FALSE,
            verbose=FALSE))
    expect_silent(clustNothing3 <- 
        clusterMany(sceSimDataDimRed,
            ks=c(3,4),nReducedDims=c(5:6),
            reduceMethod="none",
            clusterFunction="pam", subsample=FALSE, 
            sequential=FALSE, isCount=FALSE,verbose=FALSE))
    expect_equal(clusterMatrix(clustNothing), clusterMatrix(clustNothing3))

  
})
test_that("`clusterMany` works with reduceMethod a filtering",{
  expect_message(clustNothing4 <- clusterMany(sceSimDataDimRed, 
                                             ks=c(3,4),clusterFunction="pam", reduceMethod="mad",
                                             subsample=FALSE, sequential=FALSE, isCount=FALSE,verbose=FALSE),"Not all of the methods requested in 'reduceMethod' have been calculated")
  
  expect_error(clustNothing5 <- clusterMany(sceSimDataDimRed, 
                                             ks=c(3,4),clusterFunction="pam", reduceMethod=c("mad","cv"),
                                             subsample=FALSE, sequential=FALSE, isCount=FALSE,verbose=FALSE),"do not match any stored or built-in filtering statistics or dimensionality reduction")
  expect_message(clustNothing5 <- clusterMany(sceSimDataDimRed, 
                                            ks=c(3,4),clusterFunction="pam", reduceMethod=c("mad","abscv"),
                                            subsample=FALSE, sequential=FALSE, isCount=FALSE,verbose=FALSE),"Not all of the methods requested in 'reduceMethod' have been calculated")
  expect_true(all(filterNames(sceSimDataDimRed) %in% filterNames(clustNothing5)))
  expect_true(all(c("mad","abscv","var","mean") %in% filterNames(clustNothing5)))
  
	## Issue with message created "Found more than one class "Annotated" in cache; using the first, from namespace 'S4Vectors'" message
	## Now just checks no new warnings.
  expect_silent(
      suppressMessages(clusterMany(clustNothing5, 
                                              ks=c(3,4),clusterFunction="pam", reduceMethod=c("mad","abscv"),
                                              subsample=FALSE, sequential=FALSE, verbose=FALSE)
																							)
   )
})
test_that("`clusterMany` works with hdf5", {
	# These are slow!
	skip_on_os("windows")
	skip_on_os("mac")
	
	########
	#Check if use PCA (not in hdf5) changes nothing, as expect.
	########
    for(kk in 1:length(listBuiltInFunctions())){
  	  expect_silent(clustNothing2 <- clusterMany(hdfSCE,
  		   ks=c(3,4),clusterFunction=listBuiltInFunctions()[[kk]],
  	       makeMissingDiss=TRUE,
           subsample=FALSE, sequential=FALSE, 
           isCount=FALSE,verbose=FALSE))  	
    }
    expect_silent(clustNothing <- 
        clusterMany(t(reducedDims(sceSimDataDimRed)[["PCA"]]), 
    	    ks=c(3,4),clusterFunction="pam", 
            reduceMethod="none",
            subsample=FALSE, sequential=FALSE, 
            isCount=FALSE,verbose=FALSE))
    expect_silent(clustNothing3 <- 
        clusterMany(hdfSCE, 
            ks=c(3,4),clusterFunction="pam", 
            reduceMethod="PCA", subsample=FALSE, 
            sequential=FALSE, isCount=FALSE,verbose=FALSE))
    expect_equal(clusterMatrix(clustNothing), clusterMatrix(clustNothing3))
    expect_equal(NCOL(clusterMatrix(clustNothing)),2)
    expect_equal(NCOL(clusterMatrix(clustNothing3)),2)

	########
	#Check if not using PCA (reduceMethod = "none")
	########
    for(kk in 1:length(listBuiltInFunctions())){
  	  expect_silent(clustNothing2 <- 
          clusterMany(hdfSCE,reduceMethod = "none", 
  		   ks=c(3,4), makeMissingDiss=TRUE,
           clusterFunction=listBuiltInFunctions()[[kk]],
  	       subsample=FALSE, sequential=FALSE, 
           isCount=FALSE,verbose=FALSE))  	
    }

	########
	#Check directly on hdf5 object
	########
    for(kk in 1:length(listBuiltInFunctions())){
  	  expect_silent(clustNothing2 <- 
          clusterMany(assay(hdfObj),reduceMethod = "none", 
  		       ks=c(3,4),clusterFunction=listBuiltInFunctions()[[kk]],
  	           subsample=FALSE, sequential=FALSE, 
               makeMissingDiss=TRUE,
               isCount=FALSE,verbose=FALSE))  	
    }

})

test_that("`clusterMany` works changing parameters", {
	# These are slow!
	skip_on_os("windows")
	skip_on_os("mac")
	
	#--------
  #check dim reduce in combination
	#--------
    expect_silent(cc <- clusterMany(mat, 
        ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(3,4),
  	    reduceMethod=c("none","PCA","var","abscv","mad"),
        clusterFunction="pam",
        subsample=FALSE, sequential=FALSE,
        verbose=FALSE, isCount=FALSE))
	expect_equal(sort(reducedDimNames(cc)),sort(c("PCA")))
	expect_equal(sort(filterNames(cc)),sort(c("var","abscv","mean","mad")))

	expect_message(cc2 <- clusterMany(se, 
        ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(3,4),
	    reduceMethod=c("none","PCA","var","abscv","mad"),
        clusterFunction="pam",
        subsample=FALSE, sequential=FALSE,
        verbose=FALSE, isCount=FALSE),
        "Not all of the methods requested in 'reduceMethod' have been calculated")
    expect_equal(sort(reducedDimNames(cc2)),sort(c("PCA")))
    expect_equal(sort(filterNames(cc2)),
        sort(c("b","var","abscv","mean","mad")))

	expect_message(cc3 <- clusterMany(sce, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(3,4),
	reduceMethod=c("none","PCA","var","abscv","mad"),clusterFunction="pam",
  subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE),"Not all of the methods requested in 'reduceMethod' have been calculated")
expect_equal(sort(reducedDimNames(cc3)),sort(c("PCA")))
expect_equal(sort(filterNames(cc3)),sort(c("b","var","abscv","mean","mad")))

	#Only existing values 
	expect_silent(cc4 <- clusterMany(sceFull, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
	reduceMethod=c("none","Red1","Filter1","Filter2"),clusterFunction="pam",
  subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE))
expect_equal(sort(reducedDimNames(cc4)),sort(c("Red1")))
expect_equal(sort(filterNames(cc4)),sort(c("b","Filter1","Filter2")))

	## Issue with message created "Found more than one class "Annotated" in cache; using the first, from namespace 'S4Vectors'" message
	## Now just checks no new warnings.
  expect_silent(
    suppressMessages( ceReRun <- clusterMany(cc4, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
  	reduceMethod=c("none","Red1","Filter1","Filter2"),clusterFunction="pam",
    subsample=FALSE, sequential=FALSE,verbose=FALSE)
    )
	)
	expect_equal(sort(reducedDimNames(ceReRun)),sort(reducedDimNames(cc4)))
	expect_equal(sort(filterNames(ceReRun)),sort(filterNames(cc4)))


	#Only existing values 
	expect_silent(cc4 <- clusterMany(sceFull, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
	reduceMethod=c("none","Red1","Filter1","Filter2"),clusterFunction="pam",
  subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE))
expect_equal(sort(reducedDimNames(cc4)),sort(c("Red1")))
expect_equal(sort(filterNames(cc4)),sort(c("b","Filter1","Filter2")))


	#--------
	# Mixing saved and unsaved (gives warnings/errors)
	#--------
	#following gives warning because can't mix saved and calculate internally
	notAllWarning<-"All values of 'reduceMethod' need to either match an existing"
	expect_error(clusterMany(sceFull, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
			reduceMethod=c("PCA","Red1"),clusterFunction="pam",
	  		subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE),notAllWarning)
	expect_error(clusterMany(sceFull, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
			reduceMethod=c("var","Red1"),clusterFunction="pam",
	  		subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE),notAllWarning)
	expect_error(clusterMany(sceFull, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
			reduceMethod=c("PCA","Filter1"),clusterFunction="pam",
	  		subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE),notAllWarning)
	expect_error(clusterMany(sceFull, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
		reduceMethod=c("var","Filter1"),clusterFunction="pam",
		subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE),
		notAllWarning)	
	#repeat for ce 
	expect_error(clusterMany(cc4, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
			reduceMethod=c("PCA","Red1"),clusterFunction="pam",
	  		subsample=FALSE, sequential=FALSE,verbose=FALSE),notAllWarning)
	expect_error(clusterMany(cc4, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
			reduceMethod=c("PCA","Filter1"),clusterFunction="pam",
	  		subsample=FALSE, sequential=FALSE,verbose=FALSE),notAllWarning)
	expect_error(clusterMany(cc4, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
			reduceMethod=c("var","Red1"),clusterFunction="pam",
	  		subsample=FALSE, sequential=FALSE,verbose=FALSE),notAllWarning)
	expect_error(clusterMany(cc4, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
		reduceMethod=c("var","Filter1"),clusterFunction="pam",
		subsample=FALSE, sequential=FALSE,verbose=FALSE),
		notAllWarning)	
	
	#--------
	# Mixing mix across filter/reduceMethod okay
	#--------
	sceSimData2<-sceFull
	reducedDims(sceSimData2)<-SimpleList()
	expect_message(c1<-clusterMany(sceSimData2, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
		reduceMethod=c("PCA"),clusterFunction="pam",
		subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE),"Not all of the methods requested in 'reduceMethod' have been calculated.")
	expect_equal(clusterExperiment:::filterStats(c1),clusterExperiment:::filterStats(sceSimData2))
	expect_equal(reducedDimNames(c1),"PCA")
	
	sceSimData3<-sceFull
	rowData(sceSimData3)<-NULL
	expect_message(c2<-clusterMany(sceSimData3, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
		reduceMethod=c("var"),clusterFunction="pam",
		subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE),"Not all of the methods requested in 'reduceMethod' have been calculated.")
	expect_equal(reducedDims(c2),reducedDims(sceSimData3))
	expect_equal(filterNames(c2),"var")
	
	#repeat for ce
	ce5<-cc4
	reducedDims(ce5)<-SimpleList()
	expect_message(c1<-clusterMany(ce5, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
		reduceMethod=c("PCA"),clusterFunction="pam",
		subsample=FALSE, sequential=FALSE,verbose=FALSE),"Not all of the methods requested in 'reduceMethod' have been calculated.")
	expect_equal(clusterExperiment:::filterStats(c1),clusterExperiment:::filterStats(ce5))
	expect_equal(reducedDimNames(c1),"PCA")
	
	ce6<-cc4
	rowData(ce6)<-NULL
	expect_message(c2<-clusterMany(ce6, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
		reduceMethod=c("var"),clusterFunction="pam",
		subsample=FALSE, sequential=FALSE,verbose=FALSE),"Not all of the methods requested in 'reduceMethod' have been calculated.")
	expect_equal(reducedDims(c2),reducedDims(ce6))
	expect_equal(filterNames(c2),"var")

	
	#--------
	#check if only dim reduction
	#--------
	expect_silent(clusterMany(sceFull, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
		reduceMethod=c("Red1"),clusterFunction="pam",
		subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE))
	sceFull2<-makeReducedDims(sceFull,reducedDims="PCA",maxDims=10,isCount=FALSE)
	expect_silent(clusterMany(sceFull2, ks=c(3,4),nFilterDims=c(10),nReducedDims=c(2),
			reduceMethod=c("PCA","Red1"),clusterFunction="pam",
	  		subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE))
	expect_silent(clusterMany(sceFull2, ks=c(3,4),nFilterDims=c(10),nReducedDims=c(2),
			reduceMethod=c("Red1"),clusterFunction="pam",
	  		subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE))

	#--------
	#check only filter
	#--------
	expect_silent(clusterMany(sceFull, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
		reduceMethod=c("Filter1"),clusterFunction="pam",
  		subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE))
	sceFull2<-makeFilterStats(sceFull,filterStat="var",isCount=FALSE)
	
	expect_silent(clusterMany(sceFull2, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
		reduceMethod=c("var","Filter1"),clusterFunction="pam",
  		subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE))
	expect_silent(clusterMany(sceFull2, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
		reduceMethod=c("var"),clusterFunction="pam",
		subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE))
	
	#--------
	#check only none
	#--------
	expect_silent(clusterMany(sceFull, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(2),
		reduceMethod=c("none"),clusterFunction="pam",
  		subsample=FALSE, sequential=FALSE,verbose=FALSE, isCount=FALSE))

	#--------
	#check returning paramMatrix
	#--------
	expect_silent(param <- clusterMany(mat, 
	  ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(3,4),reduceMethod=c("none","PCA","var"),
	  clusterFunction="pam", subsample=FALSE, sequential=FALSE,run=FALSE,verbose=FALSE,
	  isCount=FALSE))
  # #check that giving param$paramMatrix works -- not implemented...
  # cc2 <- clusterMany(mat, ks=c(3,4),nFilterDims=c(10, 15),nReducedDims=c(3,4),
  # 	reduceMethod=c("none","PCA","var"),clusterFunction="pam",
  #   subsample=FALSE, sequential=FALSE,verbose=FALSE,
  #    isCount=FALSE,paramMatrix=param$paramMatrix, mainClusterArgs=param$mainClusterArgs,
  # 	 seqArgs=param$seqArgs,subsampleArgs=param$subsampleArgs)
  # expect_equal(cc,cc2)
  
  expect_silent(cc2 <- clusterMany(mat, ks=c(3,4),clusterFunction="pam",
                    distFunction=c("euclidean","manhattan",NA),
                    subsample=FALSE, sequential=FALSE,verbose=FALSE,
                    isCount=FALSE, makeMissingDiss=TRUE))

  # # #check giving distance -- this still doesn't work.
  # # #problem to just give dist because need to grab from global environment
  # # # So doesn't work in testthat
  # dist1<-function(x){dist(x,method="manhattan")}
  # dist2<-function(x){dist(x)}
  # expect_silent(cc <- clusterMany(mat, ks=c(3,4),clusterFunction="pam",
  #                   distFunction=c("dist1","dist2",NA),
  #                   subsample=FALSE, sequential=FALSE,verbose=FALSE,
  #                   isCount=FALSE))
  # expect_equal(unname(clusterMatrix(cc)),unname(clusterMatrix(cc2)))
  # 


  #check doesn't spit out warnings because alphas/mainClustering args not match 
  expect_silent(clusterMany(mat, 
      clusterFunction=c("pam","hierarchical01"),
      ks=c(3,4),
      alphas=c(0.1,0.2),
      subsample=FALSE, sequential=FALSE,
      verbose=FALSE, makeMissingDiss=TRUE,
      mainClusterArgs=list(clusterArgs=list(evalClusterMethod="average")),
      isCount=FALSE))
  
  #check doesn't spit out warnings because alphas/mainClustering args not match 
  expect_silent(clusterMany(mat, 
      clusterFunction=c("pam","hierarchical01"),ks=c(3,4),
      betas=c(.7,.9), minSizes=c(3,5),
      subsample=FALSE, makeMissingDiss=TRUE, sequential=FALSE,verbose=FALSE,
      mainClusterArgs=list(clusterArgs=list(evalClusterMethod="average")),
      isCount=FALSE))
})

test_that("`getClusterManyParams` works", {
	expect_silent(cc<-clusterMany(mat, ks=c(3,4),nFilterDims=c(10,15),nReducedDims=c(3,4),
		reduceMethod=c("none","PCA","var"),clusterFunction="pam",
	  	subsample=FALSE, sequential=FALSE,run=TRUE,verbose=FALSE,
	    isCount=FALSE))
	expect_silent(cc<-makeConsensus(cc,proportion=1,whichClusters = "clusterMany"))
	expect_silent(paramAll<-getClusterManyParams(cc))
	expect_equal(sort(colnames(paramAll)),sort(c("clusteringIndex", "reduceMethod", "nReducedDims", "nFilterDims", "k")))
	expect_true(is.data.frame(paramAll))
	expect_equal(sort(as.character(unique(paramAll[,"reduceMethod"]))),sort(c("none","var","PCA")))
	
	expect_equal(sort(unique(paramAll[,"nReducedDims"]),na.last=TRUE),sort(c(NA,3,4),na.last=TRUE))
	expect_equal(sort(unique(paramAll[,"nFilterDims"]),na.last=TRUE),sort(c(NA,10,15),na.last=TRUE))
	expect_true(is.numeric(paramAll[,"k"]))
	
    # check simplify=TRUE
	expect_silent(paramSub<-getClusterManyParams(cc,whichClusters=c(4,6),
        simplify=TRUE))
	expect_equal(sort(colnames(paramSub)),sort(c("clusteringIndex", "reduceMethod","nFilterDims")))
	expect_true(is.data.frame(paramSub))
	expect_equal(sort(unique(paramSub[,"nFilterDims"]),na.last=TRUE),c(10,NA))
	
	expect_warning(getClusterManyParams(cc,whichClusters=1:5),"some clusters indicated in 'whichClusters' do not have type 'clusterMany'")
	expect_warning(getClusterManyParams(cc,whichClusters=1),"did not return any clusters of type 'clusterMany'")
	expect_warning(getClusterManyParams(cc,whichClusters="mergeClusters"),"did not return any clusters")
})



test_that("`clusterMany` consistent results (no transformation)", {
	
	#----
	#check get the same result from previous times when transform the data
	#Relies on having these checks in clusterSingle, so know clusterSingle is right
    #----
	#make it big enough can do pca and filter...
	contData<-simData[,1:20]
	expectTrans1<-round(contData[1,],2)
  expect_silent(testSE<-SummarizedExperiment(contData))
  expect_silent(testSCE<-as(testSE,"SingleCellExperiment"))
    
	#matrix
	expect_silent(ccVar<-clusterSingle(contData,
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, 
        mainClusterArgs=list(clusterFunction="pam",
            clusterArgs=list(k=3)),
        isCount=FALSE))
   	expect_silent(ccPCA<-clusterSingle(contData, 
        subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
        nDims=3, 
        mainClusterArgs=list(clusterFunction="pam",
            clusterArgs=list(k=3)),
        isCount=FALSE))
  	expect_silent(ccNone<-clusterSingle(contData, 
        subsample=FALSE, sequential=FALSE, reduceMethod="none",
        nDims=NA, 
        mainClusterArgs=list(clusterFunction="pam",
            clusterArgs=list(k=3)),
        isCount=FALSE))
	expect_silent(cm<-clusterMany(contData, clusterFunction="pam",ks=3,
        subsample=FALSE, sequential=FALSE, 
        reduceMethod=c("PCA","var","none"), verbose=FALSE,
        nReducedDims=3, nFilterDims=3,isCount=FALSE))
	expect_equal(nClusterings(cm),3)	
	expect_silent(params<-getClusterManyParams(cm))	
    
	expect_equal(primaryCluster(ccPCA),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="PCA"]])
	expect_equal(primaryCluster(ccVar),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="var"]])
	expect_equal(primaryCluster(ccNone),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="none"]])
	expect_equal(round(transformData(cm)[1,],2), expectTrans1) 
	

   #SE
	expect_silent(ccVar<-clusterSingle(testSE, 
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, 
        mainClusterArgs=list(clusterFunction="pam",
            clusterArgs=list(k=3)),
 	   isCount=FALSE))
 	## Issue with message created "Found more than one class "Annotated" in cache; using the first, from namespace 'S4Vectors'" message
 	## Now just checks no new warnings.
	expect_silent(
      suppressMessages(ccPCA<-clusterSingle(testSE, 
      subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
      nDims=3, 
      mainClusterArgs=list(clusterFunction="pam",
          clusterArgs=list(k=3)),
   isCount=FALSE)
	 )
  )
 	expect_silent(ccNone<-clusterSingle(testSE, 
        subsample=FALSE, sequential=FALSE, reduceMethod="none",
        nDims=NA, 
        mainClusterArgs=list(clusterFunction="pam",
            clusterArgs=list(k=3)),
 	   isCount=FALSE))
	expect_message(cm<-clusterMany(testSE, clusterFunction="pam",ks=3,
   	        subsample=FALSE, sequential=FALSE, verbose=FALSE,
            reduceMethod=c("PCA","var","none"),
   	        nReducedDims=3, nFilterDims=3,isCount=FALSE),
            "Not all of the methods requested in 'reduceMethod' have been calculated.")
	expect_equal(nClusterings(cm),3)	
	expect_silent(params<-getClusterManyParams(cm))	
   
	expect_equal(primaryCluster(ccPCA),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="PCA"]])
	expect_equal(primaryCluster(ccVar),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="var"]])
	expect_equal(primaryCluster(ccNone),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="none"]])
	expect_equal(round(transformData(cm)[1,],2), expectTrans1) 

	
   #SCE
	expect_silent(ccVar<-clusterSingle(testSCE, 
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, 
        mainClusterArgs=list(clusterFunction="pam",
            clusterArgs=list(k=3)),
 	   isCount=FALSE))
	 	## Issue with message created "Found more than one class "Annotated" in cache; using the first, from namespace 'S4Vectors'" message
	 	## Now just checks no new warnings.
  	expect_silent(
        suppressMessages(ccPCA<-clusterSingle(testSCE, 
        subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
        nDims=3, 
        mainClusterArgs=list(clusterFunction="pam",
            clusterArgs=list(k=3)),
 	   isCount=FALSE)
		 )
   )
 	expect_silent(ccNone<-clusterSingle(testSCE, 
        subsample=FALSE, sequential=FALSE, reduceMethod="none",
        nDims=NA, 
        mainClusterArgs=list(clusterFunction="pam",
            clusterArgs=list(k=3)),
 	   isCount=FALSE))
	expect_message(cm<-clusterMany(testSCE, clusterFunction="pam",ks=3,
        subsample=FALSE, sequential=FALSE, 
        reduceMethod=c("PCA","var","none"),verbose=FALSE,
        nReducedDims=3, nFilterDims=3,isCount=FALSE),
        "Not all of the methods requested in 'reduceMethod' have been calculated.")
	expect_equal(nClusterings(cm),3)	
	expect_silent(params<-getClusterManyParams(cm))	
   
	expect_equal(primaryCluster(ccPCA),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="PCA"]])
	expect_equal(primaryCluster(ccVar),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="var"]])
	expect_equal(primaryCluster(ccNone),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="none"]])
	expect_equal(round(transformData(cm)[1,],2), expectTrans1) 
	

   #CE
	expect_silent(ccVar2<-clusterSingle(ccVar, 
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, clusterLabel="redo",
		mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3))))
	expect_silent(ccPCA2<-clusterSingle(ccPCA, 
        subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
        nDims=3, clusterLabel="redo",
		mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3))))
	expect_silent(ccNone2<-clusterSingle(ccNone, 
        subsample=FALSE, sequential=FALSE, reduceMethod="none",
        nDims=NA,clusterLabel="redo",
		mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3))))
		## Issue with message created "Found more than one class "Annotated" in cache; using the first, from namespace 'S4Vectors'" message
		## Now just checks no new warnings.
	expect_silent(
    suppressMessages(cm2<-clusterMany(cm, clusterFunction="pam",ks=3,
        subsample=FALSE, sequential=FALSE, 
        reduceMethod=c("PCA","var","none"), verbose=FALSE,
        nReducedDims=3, nFilterDims=3)
    )
	)
	expect_equal(nClusterings(cm2),6)	
	expect_silent(params<-getClusterManyParams(cm2))	
   
	expect_equal(primaryCluster(ccPCA),clusterMatrix(cm2)[,params$clusteringIndex[params$reduceMethod=="PCA"]])
	expect_equal(primaryCluster(ccVar),clusterMatrix(cm2)[,params$clusteringIndex[params$reduceMethod=="var"]])
	expect_equal(primaryCluster(ccNone),clusterMatrix(cm2)[,params$clusteringIndex[params$reduceMethod=="none"]])
	expect_equal(round(transformData(cm2)[1,],2), expectTrans1) 
    

  
})


test_that("`clusterMany` consistent results (with transformation)", {
	
	#----
	#check get the same result from previous times when transform the data
	#Relies on tests in clusterSingle being right
    #----
	#make it big enough can do pca and filter...
	countData<-simCount[,1:20]
   expect_silent(testSE<-SummarizedExperiment(countData))
    expect_silent(testSCE<-as(testSE,"SingleCellExperiment"))
	expectTrans1<-round(log2(countData[1,]+1),2)

	#matrix
	expect_silent(ccVar<-clusterSingle(countData, 
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, 
        mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
 	   isCount=TRUE))
   	expect_silent(ccPCA<-clusterSingle(countData, 
        subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
        nDims=3, 
        mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
 	   isCount=TRUE))
  	expect_silent(ccNone<-clusterSingle(countData, 
        subsample=FALSE, sequential=FALSE, reduceMethod="none",
        nDims=NA, 
        mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
 	   isCount=TRUE))
	expect_silent(cm<-clusterMany(countData, clusterFunction="pam",ks=3,
        subsample=FALSE, sequential=FALSE, 
        reduceMethod=c("PCA","var","none"), verbose=FALSE,
        nReducedDims=3, nFilterDims=3,isCount=TRUE))
	expect_equal(nClusterings(cm),3)	
	expect_silent(params<-getClusterManyParams(cm))	
    
	expect_equal(primaryCluster(ccPCA),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="PCA"]])
	expect_equal(primaryCluster(ccVar),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="var"]])
	expect_equal(primaryCluster(ccNone),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="none"]])
	expect_equal(round(transformData(cm)[1,],2), expectTrans1) 
	

   #SE
	expect_silent(ccVar<-clusterSingle(testSE, 
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, 
        mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
 	   isCount=TRUE))
	 	## Issue with message created "Found more than one class "Annotated" in cache; using the first, from namespace 'S4Vectors'" message
	 	## Now just checks no new warnings.
   expect_silent(
        suppressMessages(ccPCA<-clusterSingle(testSE, 
        subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
        nDims=3, 
        mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
 	   isCount=TRUE)
    ))
 	expect_silent(ccNone<-clusterSingle(testSE, 
        subsample=FALSE, sequential=FALSE, reduceMethod="none",
        nDims=NA, 
        mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
        isCount=TRUE))
	expect_message(cm<-clusterMany(testSE, clusterFunction="pam",ks=3,
        subsample=FALSE, sequential=FALSE, 
        reduceMethod=c("PCA","var","none"),verbose=FALSE,
        nReducedDims=3, nFilterDims=3,isCount=TRUE),"Not all of the methods requested in 'reduceMethod' have been calculated")
	expect_equal(nClusterings(cm),3)	
	expect_silent(params<-getClusterManyParams(cm))	
   
	expect_equal(primaryCluster(ccPCA),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="PCA"]])
	expect_equal(primaryCluster(ccVar),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="var"]])
	expect_equal(primaryCluster(ccNone),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="none"]])
	expect_equal(round(transformData(cm)[1,],2), expectTrans1) 

	
   #SCE
	expect_silent(ccVar<-clusterSingle(testSCE, 
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, 
        mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
        isCount=TRUE))
				## Issue with message created "Found more than one class "Annotated" in cache; using the first, from namespace 'S4Vectors'" message
				## Now just checks no new warnings.
  expect_silent(
        suppressMessages(ccPCA<-clusterSingle(testSCE, 
        subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
        nDims=3, 
        mainClusterArgs=list(clusterFunction="pam",
        clusterArgs=list(k=3)),
 	   isCount=TRUE)
		 )
   )
 	expect_silent(ccNone<-clusterSingle(testSCE, 
        subsample=FALSE, sequential=FALSE, reduceMethod="none",
        nDims=NA, 
        mainClusterArgs=list(clusterFunction="pam",
        clusterArgs=list(k=3)),
 	   isCount=TRUE))
	expect_message(cm<-clusterMany(testSCE, clusterFunction="pam",ks=3,
        subsample=FALSE, sequential=FALSE,  
        reduceMethod=c("PCA","var","none"),verbose=FALSE,
        nReducedDims=3, nFilterDims=3,isCount=TRUE),"Not all of the methods requested in 'reduceMethod' have been calculated")
	expect_equal(nClusterings(cm),3)	
	expect_silent(params<-getClusterManyParams(cm))	
   
	expect_equal(primaryCluster(ccPCA),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="PCA"]])
	expect_equal(primaryCluster(ccVar),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="var"]])
	expect_equal(primaryCluster(ccNone),clusterMatrix(cm)[,params$clusteringIndex[params$reduceMethod=="none"]])
	expect_equal(round(transformData(cm)[1,],2), expectTrans1) 
	

   #CE
	expect_silent(ccVar2<-clusterSingle(ccVar, 
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, clusterLabel="redo",
		mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3))))
	expect_silent(ccPCA2<-clusterSingle(ccPCA, 
        subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
        nDims=3, clusterLabel="redo",
		mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3))))
	expect_silent(ccNone2<-clusterSingle(ccNone, 
        subsample=FALSE, sequential=FALSE, reduceMethod="none",
        nDims=NA,clusterLabel="redo",
		mainClusterArgs=list(clusterFunction="pam",
             clusterArgs=list(k=3))))
	 	## Issue with message created "Found more than one class "Annotated" in cache; using the first, from namespace 'S4Vectors'" message
	 	## Now just checks no new warnings.
    expect_silent(
        suppressMessages(cm2<-clusterMany(cm, clusterFunction="pam",ks=3,
        subsample=FALSE, sequential=FALSE, 
        reduceMethod=c("PCA","var","none"), verbose=FALSE,
        nReducedDims=3, nFilterDims=3)
    		)
		)
	expect_equal(nClusterings(cm2),6)	
	expect_silent(params<-getClusterManyParams(cm2))	
   
	expect_equal(primaryCluster(ccPCA),clusterMatrix(cm2)[,params$clusteringIndex[params$reduceMethod=="PCA"]])
	expect_equal(primaryCluster(ccVar),clusterMatrix(cm2)[,params$clusteringIndex[params$reduceMethod=="var"]])
	expect_equal(primaryCluster(ccNone),clusterMatrix(cm2)[,params$clusteringIndex[params$reduceMethod=="none"]])
	expect_equal(round(transformData(cm2)[1,],2), expectTrans1) 
    
  
})

test_that("`clusterMany` works with ClusterFunction objects",{
		expect_silent(clustAll1<-clusterMany(sceSimDataDimRed,reduceMethod="PCA",
			   ks=c(3,4),clusterFunction=listBuiltInFunctions()[1:2],
		       subsample=FALSE, sequential=FALSE, isCount=FALSE,verbose=FALSE,random.seed=1250))  	
	  
	expect_silent(clustAll2<-clusterMany(sceSimDataDimRed,reduceMethod="PCA",
	 ks=c(3,4),clusterFunction=getBuiltInFunction(listBuiltInFunctions()[1:2]),
		       subsample=FALSE, sequential=FALSE, isCount=FALSE,verbose=FALSE,random.seed=1250))  	
	  
		expect_equal(clustAll2,clustAll1)			 
})

context("Constructor")

test_that("run internalFunctionCheck check on all built in cluster functions", {
    
    bMethods<-c(listBuiltInTypeK(),listBuiltInType01())
    for(cf in bMethods){
        expect_silent(cf<-getBuiltInFunction(cf))
        expect_silent(internalFunctionCheck(cf@ClusterFunction,
            inputType=inputType(cf),
            algorithmType=algorithmType(cf),
            outputType=cf@outputType))
   }
})

test_that("cat inputType works on relevant cluster functions", {
    catMat<-cbind(catMat,catMat)
    set.seed(32590)
    catMat<-catMat[,sample(1:ncol(catMat))]
    expect_silent(pMat<-clusterExperiment:::.clustersHammingDistance(catMat))
    
    kMethods<-listBuiltInTypeK()
    kMethods<-kMethods[sapply(inputType(kMethods),function(x){"cat" %in% x})]
    for(cf in kMethods){
        expect_silent(cfObj<-getBuiltInFunction(cf))
        
        set.seed(782935)
        expect_silent(pOut<-cfObj@clusterFUN(inputMatrix=pMat,
            inputType="diss",k=3,cluster.only=TRUE,
            removeDup=TRUE, checkArgs=TRUE))
        set.seed(782935)
        expect_silent(allOut<-cfObj@clusterFUN(inputMatrix=catMat,
            inputType="cat",k=3,cluster.only=TRUE,
            removeDup=FALSE,checkArgs=TRUE))
        expect_equal(allOut,pOut)
        set.seed(782935)
        expect_silent(nodupOut<-cfObj@clusterFUN(inputMatrix=catMat,
            inputType="cat",k=3,cluster.only=TRUE,
            removeDup=TRUE, checkArgs=TRUE))
        expect_equal(allOut,nodupOut)
    
    }
    aMethods<-listBuiltInType01()
    aMethods<-aMethods[sapply(inputType(aMethods),function(x){"cat" %in% x})]
    for(cf in aMethods){
        expect_silent(cfObj<-getBuiltInFunction(cf))
        set.seed(782935)
        #removeDup shouldn't do anything for inputType="diss" 
        expect_silent(pOut<-cfObj@clusterFUN(inputMatrix=pMat,
            inputType="diss",alpha=0.3,cluster.only=TRUE,
            removeDup=TRUE, checkArgs=TRUE))
        set.seed(782935)
        expect_silent(allOut<-cfObj@clusterFUN(inputMatrix=catMat,
            inputType="cat",alpha=0.3,cluster.only=TRUE,
            removeDup=FALSE,checkArgs=TRUE))
        expect_equal(allOut,pOut)
        set.seed(782935)
        expect_silent(nodupOut<-cfObj@clusterFUN(inputMatrix=catMat,
            inputType="cat",alpha=0.3,cluster.only=TRUE,
            removeDup=TRUE, checkArgs=TRUE))
        # Don't actually expect same answer if remove duplicates. The tight method seems to give different answers, not sure that's a problem...
        #expect_equal(allOut,nodupOut)
    
    }

})

test_that("built in cluster functions return correct clustering", {
    
    
    #test some easy ones
    
    #PAM
    truth<- cluster::pam(x=t(mat),cluster.only=TRUE,k=3)
    out<-getBuiltInFunction("pam")@clusterFUN(inputMatrix=mat,
         inputType="X",k=3,cluster.only=TRUE)
    expect_equal(truth,out)
       
    #kmeans
    set.seed(24819)
    truth<- stats::kmeans(x=t(mat),centers=3)$cluster
    set.seed(24819)
    out<-getBuiltInFunction("kmeans")@clusterFUN(inputMatrix=mat,
         inputType="X",k=3,cluster.only=TRUE)
    expect_equal(truth,out)
})

context("clusterSingle")



   
test_that("`clusterSingle` works with matrix, ClusterExperiment objects,
          SummarizedExperiments", {
    #---
    #Matrix
    #---
	expect_silent(clustNothing <- 
        clusterSingle(mat, inputType="X",
        subsample=FALSE, sequential=FALSE,
        mainClusterArgs=list(clusterArgs=list(k=3), 
            clusterFunction="pam"), 
        isCount=FALSE))
	expect_equal(clusterLabels(clustNothing),"clusterSingle")
	expect_is(clustNothing, "ClusterExperiment")
	expect_is(clustNothing, "SummarizedExperiment")
	truth<- unname(cluster::pam(x=t(mat),cluster.only=TRUE,k=3))
    #in case change in sim, make sure same as from previous tests
    expect_equal(truth,c(1,2,3,1,3,1,1,1,1,2,2,1,1,1,1)) 
	expect_equal(primaryCluster(clustNothing),truth)

	#test clusterLabel
	expect_silent(clustNothing2 <- clusterSingle(mat,
        inputType="X",
        mainClusterArgs= list(clusterArgs=list(k=3), clusterFunction="pam"), 
        subsample=FALSE,
		sequential=FALSE, isCount=FALSE, 
        clusterLabel="myownClustering"))
	expect_equal(clusterLabels(clustNothing2),"myownClustering")


	#test default 01 distance
	expect_silent(clusterSingle(inputMatrix=dissMat, inputType="diss", mainClusterArgs= list(clusterArgs=list(alpha=0.1),clusterFunction="tight"),
	                              subsample=FALSE, sequential=FALSE,
	                              isCount=FALSE))

	#test default K distance
	expect_silent(clusterSingle(dissMat, inputType="diss", mainClusterArgs= list(clusterArgs=list(k=3),clusterFunction="hierarchicalK"),subsample=FALSE, sequential=FALSE, isCount=FALSE))

  #---
  #SE/SCE
  #---
	###Apply to SE
	expect_silent(clustNothing2 <- clusterSingle(se,
		mainClusterArgs=list(clusterArgs=list(k=3),clusterFunction="pam"),
	    subsample=FALSE, sequential=FALSE, isCount=FALSE))
	expect_equal(clusterMatrix(clustNothing2), clusterMatrix(clustNothing))

	###Apply to SCE
	expect_silent(clustNothing3 <- clusterSingle(sce, mainClusterArgs=list(clusterArgs=list(k=3),clusterFunction="pam"),
	    subsample=FALSE, sequential=FALSE, isCount=FALSE))
	expect_equal(clusterMatrix(clustNothing2), clusterMatrix(clustNothing3))
	expect_silent(clustNothing3 <- clusterSingle(sce,
		mainClusterArgs=list(clusterArgs=list(k=3),clusterFunction="pam"),
		subsample=FALSE, sequential=FALSE, isCount=FALSE))

	#---
	#CE
	#---
	#test running on ClusterExperiment Object -- should add the new clustering
	expect_error(clustNothing3 <- clusterSingle(clustNothing2, mainClusterArgs=list(clusterArgs=list(k=4),clusterFunction="pam"),
	                              subsample=FALSE, sequential=FALSE,
	                              isCount=FALSE),"setting 'transFun' or 'isCount' for a 'ClusterExperiment' is not allowed")
	expect_silent(clustNothing3 <- clusterSingle(clustNothing2, mainClusterArgs=list(clusterArgs=list(k=4),clusterFunction="pam"),
	                              subsample=FALSE, sequential=FALSE ))
	expect_equal(NCOL(clusterMatrix(clustNothing3)),2)
	expect_equal(length(table(primaryCluster(clustNothing3))),4,info="Check reset primary cluster after run clusterSingle")

})

test_that("`clusterSingle` warnings/errors work", {    
	expect_error(clusterSingle(mat, inputType="X", 
        mainClusterArgs= list(clusterArgs=list(alpha=0.1),
            clusterFunction="tight"),
        distFunction=function(x){dist(x,method="manhattan")},
        subsample=FALSE, checkDiss=TRUE,
        sequential=FALSE,isCount=FALSE),
    "distance function must give values between 0 and 1")
	#test warn with wrong arguments
	expect_warning(clusterSingle(dissMat, inputType="diss", mainClusterArgs= list(clusterArgs=list(k=3,alpha=0.1),clusterFunction="tight"),
	                  subsample=FALSE, sequential=FALSE,
	                  ,isCount=FALSE),"arguments passed via clusterArgs to the clustering function tight are not all applicable")
	#test turning off warning
	expect_silent(clusterSingle(dissMat, 
        inputType="diss", 
        mainClusterArgs= list(clusterArgs=list(k=3,alpha=0.1), clusterFunction="tight"),
        warnings=FALSE,
	    subsample=FALSE, 
        sequential=FALSE,
        isCount=FALSE))    
})
          
test_that("`clusterSingle` works with reduceMethod", {
	#existing dim reduce values
  	expect_silent(clustNothing4 <- clusterSingle(sceSimDataDimRed,
		reduceMethod="PCA", nDims=3,
		mainClusterArgs=list(clusterArgs=list(k=3),clusterFunction="pam"),
		subsample=FALSE, sequential=FALSE, isCount=FALSE))
	expect_equal(sort(reducedDimNames(clustNothing4)),sort(c("PCA","TSNE")))
	expect_silent(clustNothing5 <- clusterSingle(t(reducedDim(sceSimDataDimRed,"PCA")[,1:3]),
		mainClusterArgs=list(clusterArgs=list(k=3),clusterFunction="pam"),
        subsample=FALSE, sequential=FALSE, isCount=FALSE))
	expect_equal(clusterMatrix(clustNothing4), clusterMatrix(clustNothing5))

	#test nDims=NA
  	expect_silent(clustNothing1 <- clusterSingle(sceSimDataDimRed,
		reduceMethod="PCA", nDims=NA,
		mainClusterArgs=list(clusterArgs=list(k=3),clusterFunction="pam"),
		subsample=FALSE, sequential=FALSE, isCount=FALSE))
	expect_equal(sort(reducedDimNames(clustNothing4)),sort(c("PCA","TSNE")))
	expect_silent(clustNothing2 <- clusterSingle(t(reducedDim(sceSimDataDimRed,"PCA")),
		mainClusterArgs=list(clusterArgs=list(k=3),clusterFunction="pam"),
        subsample=FALSE, sequential=FALSE, isCount=FALSE))
	expect_equal(clusterMatrix(clustNothing1), clusterMatrix(clustNothing2))

	#blank sceSimData object
    ##FIXME
  	#expect_silent(
        clustNothing6 <- clusterSingle(sceSimData,
		reduceMethod="PCA", nDims=3,
		mainClusterArgs=list(clusterArgs=list(k=3),clusterFunction="pam"),
		subsample=FALSE, sequential=FALSE, isCount=FALSE)
    #)
	expect_equal(sort(reducedDimNames(clustNothing6)),sort(c("PCA")))
	expect_silent(clustNothing7 <- clusterSingle(t(reducedDim(sceSimDataDimRed,"PCA")[,1:3]),
		mainClusterArgs=list(clusterArgs=list(k=3),clusterFunction="pam"),
        subsample=FALSE, sequential=FALSE, isCount=FALSE))
	expect_equal(clusterMatrix(clustNothing6), clusterMatrix(clustNothing7))

  	## FIXME
    #expect_silent(
        clustNothing8 <- clusterSingle(seSimData,
		reduceMethod="PCA", nDims=3,
		mainClusterArgs=list(clusterArgs=list(k=3),clusterFunction="pam"),
		subsample=FALSE, sequential=FALSE, isCount=FALSE)
    #)
	expect_equal(sort(reducedDimNames(clustNothing8)),sort(c("PCA")))
	expect_equal(clusterMatrix(clustNothing8), clusterMatrix(clustNothing7))
	#check consistency
	expect_equal(primaryCluster(clustNothing8),rep(c(1,2,3),each=100))

  	## FIXME
    #expect_silent(
        clusterSingle(sceSimData,
		reduceMethod="PCA", nDims=NA,
		mainClusterArgs=list(clusterArgs=list(k=3),clusterFunction="pam"),
		subsample=FALSE, sequential=FALSE, isCount=FALSE)
    #)



})

test_that("all built-in clustering methods",{
    #########
    ## Put all clusterSingle tests that loop over methods HERE!
    #########
    kClusterArgs<-list(k=3)
    aClusterArgs<-list(alpha=0.1)
    set.seed(3325)
    biggerMat<-matrix(data=rnorm(20*100), ncol=100)
    diss<-as.matrix(dist(t(mat)))
    biggerDiss<-as.matrix(dist(t(biggerMat)))
    getInputMatrix<- function(cf, big=FALSE){
        if("X" %in% inputType(cf)){
            inputType<-"X"
            if(!big) inputMat<-mat
            else inputMat<-biggerMat
        }
        else if("diss" %in% inputType(cf)){
            inputType<-"diss"
            if(!big) inputMat<-diss
            else inputMat<- biggerDiss
            
        }
        else stop("error, don't have test for cat type matrix")
        return(list(inputType=inputType,inputMatrix=inputMat))
    }
    ## ---------
    # check with main clustering
    ## ---------
    #FIXME: Is this duplicative of previous test in hdf5?
    
    for(cf in listBuiltInFunctions()){
        algType<-algorithmType(cf)
        clustArgs<-switch(algType,
            "K"=kClusterArgs,
            "01"=aClusterArgs
            )
        input<-getInputMatrix(cf)
        expect_silent(clusterSingle(inputMat=input$inputMatrix,
            inputType=input$inputType,
            mainClusterArgs= list(clusterArgs=clustArgs, clusterFunction=cf),
            subsample=FALSE, sequential=FALSE,isCount=FALSE)
        )
        if(algType=="K"){
            input<-getInputMatrix(cf,big=TRUE)
            expect_silent(clusterSingle(inputMat=input$inputMatrix,
                inputType=input$inputType,
                mainClusterArgs= list(clusterArgs=clustArgs,
                    diss=switch(algType,"01"=NULL,"K"=biggerDiss),
                    clusterFunction=cf,findBestK=TRUE,removeSil=TRUE),
                subsample=FALSE, sequential=FALSE,isCount=FALSE))
            
        }
    }
    
    ##------
    ## Check with subsampling
    ##------
    # also force to be opposite type from mainClusterArgs

    for(cf in listBuiltInFunctions()){
        algType<-algorithmType(cf)
        if(algType=="K"){
        	mainArgs<-list(clusterFunction="hierarchical01", 
                clusterArgs=aClusterArgs)
        	subArgs<-list(clusterFunction=cf,resamp.num=20, clusterArgs=kClusterArgs)
        }
        if(algType=="01"){
        	mainArgs<-list(clusterFunction="pam", clusterArgs=kClusterArgs)
        	subArgs<-list(clusterFunction=cf,resamp.num=20,
                clusterArgs=aClusterArgs)
        }
        input<-getInputMatrix(cf,big=TRUE)
        
        set.seed(1045)
        ##In-sample
		expect_silent(clusterSingle(inputMat=input$inputMatrix,
            inputType=input$inputType,
            subsample=TRUE, 
            sequential=FALSE, 
            makeMissingDiss=FALSE,
            subsampleArgs=c(list(
                classifyMethod="InSample"),subArgs), 
            mainClusterArgs=mainArgs,isCount=FALSE))
       if(!is.null(getBuiltInFunction(cf)@classifyFUN)){
		#All 
   		set.seed(1045)
	    expect_silent(clusterSingle(inputMat=input$inputMatrix,
                inputType=input$inputType,  subsample=TRUE, sequential=FALSE, 
                subsampleArgs=c(list(classifyMethod="All"),subArgs), 
                mainClusterArgs=mainArgs,isCount=FALSE))
		#OutOfSample
		set.seed(1045)
	    expect_silent(clusterSingle(inputMat=input$inputMatrix,
                inputType=input$inputType,  subsample=TRUE, sequential=FALSE, 
                subsampleArgs=c(list(classifyMethod="OutOfSample"),subArgs), 
                mainClusterArgs=mainArgs,isCount=FALSE))

       }
	}
    
    # --------
    # Seq cluster -- only works for K methods
    # --------
	for(cf in listBuiltInTypeK()){
        algType<-algorithmType(cf)
        subArgs<-list(clusterFunction="pam",n.sample=40)
        seqArgs<-list(k0=5,beta=0.9,verbose=FALSE)
        input<-getInputMatrix(cf,big=FALSE)
		#check subsample=FALSE
		expect_silent(clusterSingle(input$inputMatrix, inputType=input$inputType,
            mainClusterArgs=list(clusterFunction=cf),
	        subsample=FALSE, sequential=TRUE,
	        isCount=FALSE,seqArgs=seqArgs))
	    #check subsample=TRUE
		if("cat" %in% inputType(cf))
            expect_silent(clusterSingle(input$inputMatrix, 
                inputType=input$inputType, 
                mainClusterArgs=list(clusterFunction=cf),
    	        subsample=TRUE, sequential=TRUE,
    			subsampleArgs=subArgs,
    	        isCount=FALSE,seqArgs=seqArgs))
	}
    
    
    #-------
    ## Check that they work with hdf5
    #-------
	seedValue<-571839
  	for(cf in listBuiltInFunctions()){
		#print(cf)
        clArgs<-switch(algorithmType(cf),
            "K"=kClusterArgs,
            "01"=aClusterArgs)
        ### Annoyingly, have to separately for "X" and "diss"
        ### Because for this, no way to give it the diss and still test the objects
		if("X" %in% inputType(cf)){
    		set.seed(seedValue)
  	        expect_silent(clust1<-clusterSingle(sceSimDataDimRed,   
                reduceMethod = "none", 
                mainClusterArgs= list(clusterArgs=clArgs,
                    clusterFunction=cf),
      	  		subsample=FALSE, 
                makeMissingDiss=TRUE,
                sequential=FALSE,
                isCount=FALSE)
      		)
    		set.seed(seedValue)
      	    expect_silent(clust2<-clusterSingle(hdfSCE, 
                reduceMethod = "none", 
                mainClusterArgs= list(clusterArgs=clArgs,
                    clusterFunction=cf),
                subsample=FALSE, 
                makeMissingDiss=TRUE,
                sequential=FALSE,
                isCount=FALSE)
      		)
    		set.seed(seedValue)
      	    expect_silent(clust3<-clusterSingle(hdfObj, 
                reduceMethod = "none", 
                mainClusterArgs= list(clusterArgs=clArgs, 
                    clusterFunction=cf),
                subsample=FALSE, 
                makeMissingDiss=TRUE,
                sequential=FALSE,
                isCount=FALSE)
      		)
    		set.seed(seedValue)
      	    expect_silent(clust4<-clusterSingle(assay(hdfSCE), 
                reduceMethod = "none", 
                mainClusterArgs= list(clusterArgs=clArgs, 
                    clusterFunction=cf),
                subsample=FALSE, 
                makeMissingDiss=TRUE,
                sequential=FALSE,
                isCount=FALSE)
      		)
  	    }
        else{
    		set.seed(seedValue)
  	        expect_message(clust1<-clusterSingle(sceSimDataDimRed,   
                reduceMethod = "none", 
                mainClusterArgs= list(clusterArgs=clArgs, 
                    clusterFunction=cf),
      	  		subsample=FALSE, 
                makeMissingDiss=TRUE,
                sequential=FALSE,
                isCount=FALSE),"Note: Making nxn dissimilarity matrix."
      		)
    		set.seed(seedValue)
  	        expect_message(clust2<-clusterSingle(hdfSCE, 
                reduceMethod = "none", 
                mainClusterArgs= list(clusterArgs=clArgs, 
                    clusterFunction=cf),
                subsample=FALSE, 
                makeMissingDiss=TRUE,
                sequential=FALSE,
                isCount=FALSE),"Note: Making nxn dissimilarity matrix."
      		)
			
    		set.seed(seedValue)
      	    expect_message(clust3<-clusterSingle(hdfObj, 
                reduceMethod = "none", 
                mainClusterArgs= list(clusterArgs=clArgs, 
                    clusterFunction=cf),
                subsample=FALSE, 
                makeMissingDiss=TRUE,
                sequential=FALSE,
                isCount=FALSE),"Note: Making nxn dissimilarity matrix."
      		)
    		set.seed(seedValue)
      	    expect_message(clust4<-clusterSingle(assay(hdfSCE), 
                reduceMethod = "none", 
                mainClusterArgs= list(clusterArgs=clArgs, 
                    clusterFunction=cf),
                subsample=FALSE, 
                makeMissingDiss=TRUE,
                sequential=FALSE,
                isCount=FALSE),"Note: Making nxn dissimilarity matrix."
      		)
        }
		expect_equal(clusterMatrix(clust1) ,clusterMatrix(clust2))
		expect_equal(clusterMatrix(clust1) ,clusterMatrix(clust3))
		expect_equal(clusterMatrix(clust1) ,clusterMatrix(clust4))
  	}
    
})


test_that("`clusterSingle` works with hdf5Matrix",{

	# These are slow!
	skip_on_os("windows")
	skip_on_os("mac")

 	 ####Test sequential option,
     expect_silent(clustSeq <- clusterSingle(hdfObj,
         reduceMethod="none",
         subsample=FALSE, 
         sequential=TRUE,
         mainClusterArgs=list(clusterFunction="pam"),
         isCount=FALSE,
         seqArgs=list(k0=5,beta=0.9,verbose=FALSE)
     ))
     expect_silent(clustSeq <- clusterSingle(assay(hdfObj),
         reduceMethod="none",
         subsample=FALSE, sequential=TRUE,
         mainClusterArgs=list(clusterFunction="pam"),
         isCount=FALSE,
         seqArgs=list(k0=5,beta=0.9,verbose=FALSE)))


	 ####Test subsample option
     expect_silent(clusterSingle(hdfObj, 
        reduceMethod="none", 
	 	subsample=TRUE, 
        sequential=FALSE,
		mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
		isCount=FALSE,
	 	subsampleArgs=list(clusterFunction="pam",resamp.num=3, 
            clusterArgs=list(k=3))
		)
	 )
	 expect_silent(clusterSingle(assay(hdfObj), reduceMethod="none", 
	 	subsample=TRUE, sequential=FALSE,
		mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
		isCount=FALSE,
	 	subsampleArgs=list(clusterFunction="pam",resamp.num=3, clusterArgs=list(k=3))
		)
	 )
	
})

test_that("`clusterSingle` works with filtering", {
    ####Check built in functions ####
	for(fs in listBuiltInFilterStats()){
	    expect_silent(clusterSingle(mat,
	        subsample=FALSE, 
            sequential=FALSE, 
            reduceMethod=fs,
	        nDims=3,
             mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=3)),
            isCount=FALSE))

	}
    expect_warning(clusterSingle(mat,  
        subsample=FALSE, 
        sequential=FALSE,
        reduceMethod="var",
        nDims=NROW(mat)+1,
        mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
		isCount=FALSE),
        "the number of most features requested after filtering is either missing or larger than the number of features. Will not do any filtering")
    expect_warning(clusterSingle(mat,
      subsample=FALSE, sequential=FALSE,
      reduceMethod="none",nDims=3,
	  mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
      isCount=FALSE),
      "specifying nDims has no effect if reduceMethod==`none`")

    #check returns filterStats
    expect_silent(cc <- clusterSingle(mat,
    subsample=FALSE, sequential=FALSE,
    reduceMethod="var", nDims=3,
    mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)), isCount=FALSE))
  	expect_equal(sort(filterNames(cc)),"var")

    expect_silent(cc2 <- clusterSingle(se,
    subsample=FALSE, sequential=FALSE,
    reduceMethod="var", nDims=3,
    mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)), isCount=FALSE))
  	expect_equal(sort(filterNames(cc2)),c("b","var"))

  	expect_silent(cc3 <- clusterSingle(sceSimData,
    subsample=FALSE, sequential=FALSE,
    reduceMethod="var", nDims=3,
    mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)), isCount=FALSE))
  	expect_equal(sort(filterNames(cc3)),c("b","var"))

  	#test using existing values
  	expect_silent(cc4 <- clusterSingle(sceFull,
    subsample=FALSE, sequential=FALSE,
    reduceMethod="var", nDims=3,
    mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)), isCount=FALSE))
  expect_equal(sort(filterNames(cc4)),sort(c("var","b","Filter1","Filter2")))


})


test_that("`clusterSingle` consistent results (no transformation)", {

	#----
	#check get the same result from previous times when transform the data
    #----
	#make it big enough can do pca and filter...
	contData<-simData[,1:20]
    testSE<-SummarizedExperiment(contData)
    testSCE<-as(testSE,"SingleCellExperiment")
	expectTrans1<-round(contData[1,],2)

	###
	#var filter
	###

	#matrix
	expect_silent(cc<-clusterSingle(contData,
	        subsample=FALSE, sequential=FALSE, reduceMethod="var",
	        nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
	 	   isCount=FALSE))
    expect_equal(primaryCluster(cc),c(3,1,1,3,2,2,3,2,3,1,2,2,1,3,3,2,1,1,2,1))
	expect_equal(round(transformData(cc)[1,],2), expectTrans1)

	expect_silent(cc<-clusterSingle(contData,
	        subsample=FALSE, sequential=FALSE, reduceMethod="var",
	        nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
	 	   isCount=FALSE))
    expect_equal(primaryCluster(cc),c(3,1,1,3,2,2,3,2,3,1,2,2,1,3,3,2,1,1,2,1))


   #SE
   expect_silent(cc2<-clusterSingle(testSE,
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
 	   isCount=FALSE))
   expect_equal(primaryCluster(cc2),primaryCluster(cc))


   #SCE
   expect_silent(cc3<-clusterSingle(testSCE,
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
 	   isCount=FALSE))
   expect_equal(primaryCluster(cc3),primaryCluster(cc))


   #CE
   expect_silent(cc5<-clusterSingle(cc,
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),clusterLabel="redoCS"))
   expect_equal(primaryCluster(cc5),primaryCluster(cc))




	###
	#PCA
	###

  #matrix
  expect_silent(cc<-clusterSingle(contData,
        subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
        nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
 	   isCount=FALSE))
  expect_equal(primaryCluster(cc),c(3,1,3,1,3,2,1,1,1,3,2,2,2,3,1,2,2,1,2,3))
	expect_equal(round(transformData(cc)[1,],2), expectTrans1)


  #SE
  ## FIXME
  #expect_silent(
      cc2<-clusterSingle(testSE,
       subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
       nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
	   isCount=FALSE)
  #)
  expect_equal(primaryCluster(cc2),primaryCluster(cc))


  #SCE
  ## FIXME
  #expect_silent(
      cc3<-clusterSingle(testSCE,
       subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
       nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
	   isCount=FALSE)
  #)
  expect_equal(primaryCluster(cc3),primaryCluster(cc))

  #CE
  expect_silent(cc5<-clusterSingle(cc,
       subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
       nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),clusterLabel="redoCS"))
  expect_equal(primaryCluster(cc5),primaryCluster(cc))


	###
	#Nothing
	###

	#matrix
	expect_silent(cc<-clusterSingle(contData,
	      subsample=FALSE, sequential=FALSE, reduceMethod="none",
	      nDims=NA, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
	   isCount=FALSE))
	expect_equal(primaryCluster(cc),c(3,2,1,3,2,1,3,2,3,2,1,2,1,1,3,1,1,1,2,2))
	expect_equal(round(transformData(cc)[1,],2), expectTrans1)


	#SE
	expect_silent(cc2<-clusterSingle(testSE,
	     subsample=FALSE, sequential=FALSE, reduceMethod="none",
	     nDims=NA, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
	   isCount=FALSE))
	expect_equal(primaryCluster(cc2),primaryCluster(cc))


	#SCE
	expect_silent(cc3<-clusterSingle(testSCE,
	     subsample=FALSE, sequential=FALSE, reduceMethod="none",
	     nDims=NA, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
	   isCount=FALSE))
	expect_equal(primaryCluster(cc3),primaryCluster(cc))


	#CE
	expect_silent(cc5<-clusterSingle(cc,
	     subsample=FALSE, sequential=FALSE, reduceMethod="none",
	     nDims=NA, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),clusterLabel="redoCS"))
	expect_equal(primaryCluster(cc5),primaryCluster(cc))

})


test_that("`clusterSingle` consistent results (with transformation)", {

	#----
	#check get the same result from previous times when transform the data
    #----
	#make it big enough can do pca and filter...
	countData<-simCount[,1:20]
    testSE<-SummarizedExperiment(countData)
    testSCE<-as(testSE,"SingleCellExperiment")
	expectTrans1<-round(log2(countData[1,]+1),2)
	###
	#var filter
	###

	#matrix
	expect_silent(cc<-clusterSingle(countData,
	        subsample=FALSE, sequential=FALSE, reduceMethod="var",
	        nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
	 	   isCount=TRUE))
   expect_equal(primaryCluster(cc),c(1,1,2,1,2,1,2,2,3,2,1,3,1,3,1,1,2,3,2,2))
	expect_equal(round(transformData(cc)[1,],2), expectTrans1)

   #SE
   expect_silent(cc2<-clusterSingle(testSE,
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
 	   isCount=TRUE))
   expect_equal(primaryCluster(cc2),primaryCluster(cc))


   #SCE
   expect_silent(cc3<-clusterSingle(testSCE,
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
 	   isCount=TRUE))
   expect_equal(primaryCluster(cc3),primaryCluster(cc))

   #CE
   expect_silent(cc5<-clusterSingle(cc,
        subsample=FALSE, sequential=FALSE, reduceMethod="var",
        nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),clusterLabel="redoCS"))
   expect_equal(primaryCluster(cc5),primaryCluster(cc))




	###
	#PCA
	###

  #matrix
  expect_silent(cc<-clusterSingle(countData,
        subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
        nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
 	   isCount=TRUE))
  expect_equal(primaryCluster(cc),c(3,2,1,1,3,1,3,1,1,1,1,2,1,2,1,1,1,2,2,1))
expect_equal(round(transformData(cc)[1,],2), expectTrans1)

  #SE
  ## FIXME
  #expect_silent(
      cc2<-clusterSingle(testSE,
       subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
       nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
	   isCount=TRUE)
  #)
  expect_equal(primaryCluster(cc2),primaryCluster(cc))


  #SCE
  ## FIXME
  #expect_silent(
      cc3<-clusterSingle(testSCE,
       subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
       nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
	   isCount=TRUE)
    #)
  expect_equal(primaryCluster(cc3),primaryCluster(cc))

  #CE
  expect_silent(cc5<-clusterSingle(cc,
       subsample=FALSE, sequential=FALSE, reduceMethod="PCA",
       nDims=3, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),clusterLabel="redoCS"))
  expect_equal(primaryCluster(cc5),primaryCluster(cc))


	###
	#Nothing
	###

	#matrix
	expect_silent(cc<-clusterSingle(countData,
	      subsample=FALSE, sequential=FALSE, reduceMethod="none",
	      nDims=NA, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
	   isCount=TRUE))
	expect_equal(primaryCluster(cc),c(2,1,2,1,2,1,2,1,3,2,1,3,1,3,1,1,1,3,3,2))

	#SE
	expect_silent(cc2<-clusterSingle(testSE,
	     subsample=FALSE, sequential=FALSE, reduceMethod="none",
	     nDims=NA, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
	   isCount=TRUE))
	expect_equal(primaryCluster(cc2),primaryCluster(cc))


	#SCE
	expect_silent(cc3<-clusterSingle(testSCE,
	     subsample=FALSE, sequential=FALSE, reduceMethod="none",
	     nDims=NA, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
	   isCount=TRUE))
	expect_equal(primaryCluster(cc3),primaryCluster(cc))


	#CE
	expect_silent(cc5<-clusterSingle(cc,
	     subsample=FALSE, sequential=FALSE, reduceMethod="none",
	     nDims=NA, mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),clusterLabel="redoCS"))
	expect_equal(primaryCluster(cc5),primaryCluster(cc))

})


             
test_that("Different options of mainClustering",{
    #check errors and warnings
    expect_error(clusterSingle(mat,  
        subsample=FALSE, sequential=TRUE, 
        seqArgs=list(verbose=FALSE), 
        isCount=FALSE,
        mainClusterArgs=list(clusterFunction="pam")),
                 "required argument 'k0' is missing for the sequential clustering step")
    expect_error(clusterSingle(mat,  
        subsample=FALSE, sequential=TRUE, 
        seqArgs=list(verbose=FALSE), isCount=FALSE, 
        mainClusterArgs=list(clusterFunction="pam","findBestK"==TRUE)),
                 "required argument 'k0' is missing for the sequential clustering step")
    expect_error(clusterSingle(mat, makeMissingDiss=TRUE,
        subsample=FALSE, sequential=FALSE,
        mainClusterArgs=list(clusterFunction="tight",clusterArgs=list(k=3)), 
        isCount=FALSE),
                   "must supply arguments: alpha")
    expect_warning(clusterSingle(mat,  makeMissingDiss=TRUE,
        subsample=FALSE, sequential=FALSE, 
        mainClusterArgs=list(clusterFunction="tight",
             clusterArgs=list(alpha=0.1),findBestK=TRUE),
        isCount=FALSE),
        "Some arguments passed via mainClusterArgs in mainClustering step do not match the algorithmType of the given ClusterFunction object")
    expect_error(clusterSingle(mat,
        subsample=FALSE, sequential=FALSE,
        distFunction=function(x){abs(cor(t(x)))},  
        mainClusterArgs=list(clusterFunction="tight", clusterArgs=list(alpha=0.1)),
        checkDiss=TRUE,isCount=FALSE),
        "Dissimilarity matrix must have zero values on the diagonal")
#nReducedDims=5,k=NA,findBestK=TRUE

    #-------
    ## Check findBestK=TRUE (i.e. post-processing)
    #-------
    # Error, because needs calculate diss
    expect_error(clusterSingle(simData,
        subsample=FALSE, makeMissingDiss = FALSE,
        mainClusterArgs=list(clusterFunction="pam",
            kRange=2:4, findBestK=TRUE)),
        "Cannot do requested post processing")
    #Should find best K is K=4
    expect_message(out<-clusterSingle(simData,
        subsample=FALSE, 
        makeMissingDiss=TRUE,
        mainClusterArgs=list(clusterFunction="pam",
            kRange=2:4,
            findBestK=TRUE)),
        "Making nxn dissimilarity matrix in order to do post-processing")
    expect_equal(length(tableClusters(out)),4)
    
    ## Should be same as if gave distance, since its pam at main clustering 
    ## (and if give distance, shouldn't get message)
    d<-as.matrix(dist(t(simData)))
    expect_silent(out2<-clusterSingle(d, inputType="diss",
            subsample=FALSE,
            mainClusterArgs=list(clusterFunction="pam",
                kRange=2:4,
                findBestK=TRUE)))
    expect_equal(primaryCluster(out),out2$clustering)
    
    ## Also same as if gave distance to mainClusterArgs
    expect_silent(out3<-clusterSingle(simData, 
            subsample=FALSE,
            mainClusterArgs=list(clusterFunction="pam",
                kRange=2:4, diss=d,
                findBestK=TRUE)))
    expect_equal(clusterMatrix(out),clusterMatrix(out3))
    
    
})

test_that("Different options of subsampling",{
    #Check saving of subsampling matrix
	expect_silent(clustSubsample <- clusterSingle(mat,  
        inputType="X",
        subsample=TRUE, 
        saveSubsamplingMatrix=TRUE,
        sequential=FALSE, makeMissingDiss=FALSE, warnings=TRUE,
        subsampleArgs=list(resamp.num=3, 
             clusterFunction="kmeans",clusterArgs=list(k=3)), 
        mainClusterArgs=list(clusterFunction="hierarchical01",
             clusterArgs=list(alpha=.3)), 
        isCount=FALSE))
    expect_equal(NROW(coClustering(clustSubsample)),nSamples(clustSubsample))
    expect_false(is.null(coClustering(clustSubsample)))
    expect_message(clustSubsample <- clusterSingle(mat,  
        subsample=TRUE, 
        saveSubsamplingMatrix=FALSE,
        sequential=FALSE, 
        subsampleArgs=list(resamp.num=3, clusterArgs=list(k=3)), 
        mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=3)), 
        isCount=FALSE,warnings=TRUE),
        "clusterFunction was not given for subsampleClustering -- set to be the same as the mainClustering step")
    expect_true(is.null(coClustering(clustSubsample)))
    
    expect_silent(test <- clusterSingle(sce,  
        subsample=TRUE, 
        saveSubsamplingMatrix=TRUE,
        sequential=FALSE, 
        subsampleArgs=list(resamp.num=3, clusterArgs=list(k=3)), 
        mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=3)), 
        isCount=FALSE,warnings=FALSE))
    expect_false(is.null(coClustering(test)))
    expect_silent(test <- clusterSingle(sce,  
        subsample=TRUE, 
        saveSubsamplingMatrix=FALSE,
        sequential=FALSE, 
        subsampleArgs=list(resamp.num=3, clusterArgs=list(k=3)), 
        mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=3)), 
        isCount=FALSE,warnings=FALSE))
    expect_true(is.null(coClustering(test)))
    
    expect_silent(clustSubsampleCE <- clusterSingle(cc,  
        subsample=TRUE, 
        saveSubsamplingMatrix=TRUE,
        sequential=FALSE, 
        subsampleArgs=list(resamp.num=3, clusterArgs=list(k=3)), 
        mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=3)),
        warnings=FALSE))
    expect_false(is.null(coClustering(clustSubsampleCE)))
    expect_silent(test <- clusterSingle(cc,  
        subsample=TRUE, 
        saveSubsamplingMatrix=FALSE,
        sequential=FALSE, 
        subsampleArgs=list(resamp.num=3, clusterArgs=list(k=3)), 
        mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=3)),
        warnings=FALSE))
    expect_true(is.null(coClustering(test)))
    
    expect_silent(test <- clusterSingle(clustSubsampleCE,  
        subsample=TRUE, 
        sequential=FALSE, 
        saveSubsamplingMatrix=FALSE,
        subsampleArgs=list(resamp.num=3, clusterArgs=list(k=3)), 
        mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=3)),
        warnings=FALSE))
    expect_equal(coClustering(test),coClustering(clustSubsampleCE))




    ## get NA values
	#set.seed(1045)
	# with the new version, we fix NA's in subsampleClustering
#     expect_error(clusterSingle(mat,
#        subsample=TRUE, sequential=FALSE,
#        subsampleArgs=list(resamp.num=20,clusterArgs=list(k=3),clusterFunction="pam",
# 	   classifyMethod="OutOfSample"),
#        mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),isCount=FALSE),"NA values found in dissimilarity matrix")

    # Check warnings in missing args in subsample -- should borrow from mainClusterArgs .
	set.seed(1045)
    expect_warning(clusterSingle(mat,  subsample=TRUE, sequential=FALSE,
		subsampleArgs= list(clusterFunction="pam", resamp.num=3), 
		mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
		isCount=FALSE),
        "missing arguments k provided from those in 'mainClusterArgs'")
	# Check warnings in missing clusterFunction in subsample -- should borrow from mainClusterArgs .
	expect_message(clusterSingle(mat,  subsample=TRUE, sequential=FALSE,
		subsampleArgs=list(resamp.num=3,clusterArgs=list(k=3)), 
		mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
		isCount=FALSE),
		"a clusterFunction was not given for subsampleClustering")
	#Check different function types -- should error out.
    expect_error(clusterSingle(mat,  subsample=TRUE, sequential=FALSE, subsampleArgs=list(clusterFunction="pam",resamp.num=3),  mainClusterArgs=list(clusterFunction="tight",clusterArgs=list(alpha=0.1)), isCount=FALSE),
	"must supply arguments: k")


})

test_that("Interactions of mainClustering and subsampling options",{
    ##This once creates unexpected errors even on master, because subsampleArgs not given (even though mainClusterArgs has k, because k isn't the right one for clusterFunction of main (hierarchical01), they aren't passed on to subsample. Unclear if this is the best option...)
    expect_error(ce<-clusterSingle(se, 
         isCount = FALSE, 
         reduceMethod="PCA",
         nDims=10,
         subsample=TRUE,
         sequential=FALSE,
         mainClusterArgs=list( clusterFunction="hierarchical01",minSize=1, clusterArgs=list(alpha=0.1,k = 4))
         )
         ,"must supply arguments: k"
    )
    #check totally weird names passed are caught
    expect_warning(ce<-clusterSingle(se, 
         isCount = FALSE, 
         reduceMethod="PCA",
         nDims=10,
         subsample=TRUE,
         sequential=FALSE,
         subsampleArgs=list(clusterArgs=list(k=3)),
         mainClusterArgs=list( clusterFunction="hierarchical01",myFunkyName=1, clusterArgs=list(alpha=0.1))
         )
         ,"Some arguments passed via mainClusterArgs in mainClustering step do not match the algorithmType of the given ClusterFunction object: myFunkyName"
    )
    # check: if provide diss and mainClust okay with that but 
    # subsampling not, should get error (even if say makeDiss)
	expect_error(clustSubsample <- clusterSingle(dissMat,  
        inputType="diss",
        subsample=TRUE, 
        sequential=FALSE, makeMissingDiss=TRUE, warnings=TRUE,
        subsampleArgs=list(resamp.num=3,
            clusterFunction="kmeans",clusterArgs=list(k=3)), 
        mainClusterArgs=list(clusterFunction="hierarchical01", 
            clusterArgs=list(alpha=.3)), 
        isCount=FALSE),
        "In subsampling clustering step")
    # check: should be okay even though main doesn't take type X
    # Should NOT trigger calculation of diss matrix.
	expect_silent(clustSubsample <- clusterSingle(mat,  
        inputType="X",
        subsample=TRUE, 
        sequential=FALSE, makeMissingDiss=TRUE, warnings=TRUE,
        subsampleArgs=list(resamp.num=3, clusterFunction="kmeans",clusterArgs=list(k=3)), 
        mainClusterArgs=list(clusterFunction="hierarchical01", clusterArgs=list(alpha=.3)), 
        isCount=FALSE))
    # check: should be okay but must calculate diss
	expect_message(clustSubsample <- clusterSingle(mat,  
        inputType="X",
        subsample=TRUE, 
        sequential=FALSE, makeMissingDiss=TRUE, warnings=TRUE,
        subsampleArgs=list(resamp.num=3, classifyMethod="InSample",
             clusterFunction="hierarchicalK",clusterArgs=list(k=3)), 
        mainClusterArgs=list(clusterFunction="hierarchical01",
             clusterArgs=list(alpha=.3)), 
        isCount=FALSE),"Making nxn dissimilarity matrix")

})

test_that("Different passed options of seqCluster",{
    #check sequential
    expect_silent(clustSeq <- clusterSingle(mat,
        subsample=FALSE, sequential=TRUE,
        mainClusterArgs=list(clusterFunction="pam"),
        isCount=FALSE,
        seqArgs=list(k0=5,beta=0.9,verbose=FALSE))
        )
    expect_error(clusterSingle(mat,subsample=FALSE, sequential=TRUE,
        mainClusterArgs=list(clusterFunction="pam"),
        isCount=FALSE), 
        "if sequential=TRUE, must give seqArgs so as to identify k0 and beta")

        
    expect_error(clusterSingle(mat,subsample=FALSE, sequential=TRUE,mainClusterArgs=list(clusterFunction="pam"),isCount=FALSE,seqArgs=list(k0=5,verbose=FALSE)), "required argument 'beta' is missing")
	expect_error(clusterSingle(mat,subsample=FALSE, sequential=TRUE,mainClusterArgs=list(clusterFunction="pam"),isCount=FALSE,seqArgs=list(beta=0.9,verbose=FALSE)), "required argument 'k0' is missing")

	#right clusterFunctions
	expect_error(clusterSingle(mat, 
        mainClusterArgs=list(clusterFunction="kmeans"), 
        subsample=TRUE, sequential=TRUE, 
        subsampleArgs=list(clusterFunction="pam",n.sample=40), 
        isCount=FALSE,
        seqArgs=list(k0=5,beta=0.9,verbose=FALSE)),
        "If choosing subsample=TRUE, the clusterFunction used in the mainClustering step must take input that is a categorical")
	expect_error(clusterSingle(dissMat, inputType="diss", 
        mainClusterArgs=list(clusterFunction="tight"),  
        subsample=FALSE, sequential=TRUE, 
        isCount=FALSE,
        seqArgs=list(k0=5,beta=0.9,verbose=FALSE)),
		"if subsample=FALSE, sequentical clustering can only be implemented with a clusterFunction for mainClustering step with algorithmType 'K'")
	#warning if try to set k
	expect_warning(clusterSingle(mat, 
        mainClusterArgs=list(clusterFunction="pam"), 
        subsample=TRUE, sequential=TRUE, 
        subsampleArgs=list(clusterFunction="pam",
            n.sample=40,clusterArgs=list(k=3)), 
        isCount=FALSE,seqArgs=list(k0=5,beta=0.9,verbose=FALSE)),
		"Setting 'k' in subsampleArgs when sequential=TRUE is called will have no effect.")
	expect_warning(clusterSingle(mat, 
        mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)), 
        subsample=FALSE, sequential=TRUE, 
        subsampleArgs=list(clusterFunction="pam",n.sample=40), 
        isCount=FALSE,
        seqArgs=list(k0=5,beta=0.9,verbose=FALSE)),
		"Setting 'k' in mainClusterArgs when sequential clustering is requested will have no effect.")


    ## Check warnings for esoteric arguments
    expect_error(clusterSingle(mat,subsample=FALSE, sequential=TRUE,
        mainClusterArgs=list(clusterFunction="pam"),
        isCount=FALSE,
        seqArgs=list(k0=5,beta=0.9,verbose=FALSE,top.can=5.5)), 
        "If top.can parameter of seqArgs is not a whole number, must be between 0 and 1.")
    expect_error(clusterSingle(mat,subsample=FALSE, sequential=TRUE,
        mainClusterArgs=list(clusterFunction="pam"),
        isCount=FALSE,
        seqArgs=list(k0=5,beta=0.9,verbose=FALSE,top.can=-1)), 
        "top.can parameter of seqArgs cannot be NA or negative value")
    expect_error(clusterSingle(mat,subsample=FALSE, sequential=TRUE,
        mainClusterArgs=list(clusterFunction="pam"),
        isCount=FALSE,
        seqArgs=list(k0=5,beta=0.9,verbose=FALSE,remain.n=3.4)), 
        "remain.n argument of seqArgs must be a positive whole number")
    expect_error(clusterSingle(mat,subsample=FALSE, sequential=TRUE,
        mainClusterArgs=list(clusterFunction="pam"),
        isCount=FALSE,
        seqArgs=list(k0=5,beta=0.9,verbose=FALSE,k.min=3.4)), 
        "k.min argument of seqArgs must be a positive whole number")
    expect_error(clusterSingle(mat,subsample=FALSE, sequential=TRUE,
        mainClusterArgs=list(clusterFunction="pam"),
        isCount=FALSE,
        seqArgs=list(k0=5,beta=0.9,verbose=FALSE,k.max=20.5)), 
        "k.max argument of seqArgs must be a positive whole number")
    expect_silent(clusterSingle(mat,subsample=FALSE, sequential=TRUE,
        mainClusterArgs=list(clusterFunction="pam"),
        isCount=FALSE,
        seqArgs=list(k0=5,beta=0.9,verbose=FALSE,top.can=0.1)))
        
})

test_that("Different direct options of `clusterSingle` ", {
  #check isCount
  expect_silent(clusterSingle(smSimCount,
      subsample=FALSE, sequential=FALSE,
      mainClusterArgs=list(clusterArgs=list(k=3), clusterFunction="pam"),
      isCount=TRUE) )
  #suppressWarnings because in addition to error, R prints warning about NAs
  expect_error(suppressWarnings(clusterSingle(smSimData,
                          subsample=FALSE, sequential=FALSE,
                          mainClusterArgs=list(clusterArgs=list(k=3), clusterFunction="pam"),isCount=TRUE)),"User-supplied `transFun` produces NA values",info="test error handling for isCount=TRUE when can't take log")


})

test_that("`clusterSingle` preserves the colData and rowData of SE", {
  expect_silent(cl<-clusterSingle(se,
      subsample=FALSE, sequential=FALSE,
      mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
      isCount=FALSE))

  expect_equal(colData(cl),colData(se))
  expect_equal(rownames(cl),rownames(se))
  expect_equal(colnames(cl),colnames(se))
  expect_equal(metadata(cl),metadata(se))
  expect_equal(rowData(cl),rowData(se))

})

test_that("`clusterSingle` works with parallel subsampling", {
	## Windows does not support mclapply
	skip_on_os("windows")
	expect_silent(clustSubsample2 <- 
		clusterSingle(mat,  subsample=TRUE,sequential=FALSE,
        subsampleArgs=list(clusterFunction="kmeans",resamp.num=3,
        clusterArgs=list(k=3),ncores=2),
        mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3)),
		isCount=FALSE))

    expect_silent(clustSubsample1 <- 
		clusterSingle(mat,  subsample=TRUE,sequential=FALSE,
        subsampleArgs=list(clusterFunction="kmeans",resamp.num=3,
        clusterArgs=list(k=3),ncores=1),
        mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=3)),
        isCount=FALSE))
})

#' @name plotClustersTable
#' @title Plot heatmap of cross-tabs of 2 clusterings
#' @description Plot heatmap of cross-tabulations of two clusterings
#' @aliases plotClustersTable,ClusterExperiment-method
#' @inheritParams getClusterIndex
#' @inheritParams plotHeatmap
#' @param object ClusterExperiment object (or matrix with table result)
#' @param ignoreUnassigned logical as to whether to ignore unassigned clusters
#'   in the plotting. This means they will also be ignored in the calculations
#'   of the proportions (if \code{margin} not NA).
#' @param margin if NA, the actual counts from \code{tableClusters} will be
#'   plotted. Otherwise, \code{\link[base]{prop.table}} will be called and the
#'   argument \code{margin} will be passed to \code{prop.table} to determine
#'   whether proportions should be calculated. If '1', then the proportions in
#'   the rows sum to 1, if '2' the proportions in the columns sum to 1. If
#'   'NULL' then the proportion across the entire matrix will sum to 1. An
#'   additional option has been added so that if you set  \code{margin=0}, the
#'   entry displayed in each cell will be the proportion equal to the size of
#'   the intersection over the size of the union of the clusters (a Jaccard
#'   similarity between the clusters), in which case each entry is a value
#'   between 0 and 1 but no combination of the entries sum to 1.
#' @param cluster logical, whether to cluster the rows and columns of the table.
#'   Passed to arguments \code{clusterFeatures} AND \code{clusterSamples} of
#'   \code{plotHeatmap}.
#' @param clusterLegend list in \code{clusterLegend} format that gives colors
#'   for the clusters tabulated.
#' @param plotType type of plot. If "heatmap", then a heatmap will be created of
#'   the values of the contingency table of the two clusters (calculated as
#'   determined by the argument "margin") using \code{\link{plotHeatmap}}. If
#'   "bubble", then a plot will be created using \code{bubblePlot}, which will
#'   create circles for each cell of the contingencey table whose size
#'   corresponds to the number of samples shared and the color based on the
#'   value of the proportion (as chosen by the argument \code{margin}).
#' @param main title of plot, passed to \code{plotHeatmap} or to the argument
#'   \code{propLabel} in \code{bubblePlot}
#' @param propLabel the label to go with the legend of the color of the
#'   bubbles/circles
#' @param ... arguments passed on to \code{plotHeatmap} or \code{bubblePlot}
#'   depending on choice of \code{plotType}. Note that these functions take
#'   different arguments so that switching from one to the other may not take
#'   all arguments. In particular \code{bubblePlot} calls \code{plot} while
#'   \code{plotHeatmap} calls \code{\link{NMF}{aheatmap}}.
#' @param propTable table of proportions (\code{bubblePlot}))
#' @param sizeTable table of sizes (only for use in \code{bubblePlot} or
#'   \code{plotType="bubble"}). See details.
#' @param gridColor color for grid lines (\code{bubblePlot}))
#' @param cexFactor factor to multiple by to get values of circles. If missing,
#'   finds value automatically, namely by using the maxCex value default.
#'   Overrides value of maxCex. (\code{bubblePlot}))
#' @param maxCex largest value of cex for any point (others will scale
#'   proportionally smaller) (\code{bubblePlot})).
#' @param ylab label for labeling clustering on the y-axis. If NULL, will
#'   determine names. If set to \code{NA} no label for clustering on the y-axis
#'   will be plotted (to turn off legend of the clusterings in heatmap, set
#'   \code{legend=FALSE}).
#' @param xlab label for labeling clustering on the x-axis. If NULL, will
#'   determine names. If set to \code{NA} no label for clustering on the x-axis
#'   will be plotted (to turn off legend of the clusterings in heatmap, set
#'   \code{legend=FALSE}).
#' @param las the value for the las value in the call to \code{\link{axis}} in
#'   labeling the clusters in the bubble plot. Determines whether parallel or
#'   perpindicular labels to the axis (see \code{\link{par}}).
#' @param legend whether to draw legend along top (bubble plot) or the color
#'   legend (heatmap)
#' @param colorScale the color scale for the values of the proportion table
#' @param useNames for \code{tableClusters}, whether the output should be tabled
#'   with names (\code{useNames=TRUE}) or ids (\code{useNames=FALSE})
#' @param tableMethod the type of table calculation to perform. "intersect"
#'   refers to the standard contingency table (\code{\link[base]{table}}), where
#'   each entry of the resulting table is the number of objects in both
#'   clusters. "union" instead gives for each entry the number of objects that
#'   are in the union of both clusters.
#' @seealso \code{\link{plotHeatmap}}
#' @details For \code{plotClustersTable} applied to the class \code{table},
#'   \code{sizeTable} is passed to \code{bubblePlot} to indicate the size of the
#'   circle. If \code{sizeTable=NULL}, then it is assumed that the \code{object}
#'   argument is the table of counts and both the \code{propTable} and
#'   \code{sizeTable} are set to the same value (hence turning off the coloring
#'   of the circle/bubbles). This is equivalent effect to the \code{margin=NA}
#'   option of \code{plotClustersTable} applied to the \code{ClusterExperiment}
#'   class.
#' @details Note that the cluster labels in \code{plotClustersTable} and
#'   \code{tableClusters} are converted to "proper" R names via
#'   \code{make.names}. This is because \code{tableClusters} calls the R
#'   function \code{table}, which makes this conversion
#' @details For \code{plotClustersTable}, \code{whichClusters} should define 2
#'   clusters, while for \code{tableClusters} it can indicate arbitrary number.
#' @seealso \code{\link[base]{table}}
#' @return \code{tableClusters} returns an object of class \code{table} (see
#'   \code{\link[base]{table}}).
#' @return \code{plotClustersTables} returns invisibly the plotted proportion
#'   table. In particular, this is the result of applying
#'   \code{\link[base]{prop.table}} to the results of \code{tableClusters}
#'   (after removing unclustered samples if \code{ignoreUnassigned=TRUE}).
#' @rdname plotClustersTable
#' @author Kelly Street, Elizabeth Purdom
#' @seealso \code{\link[base]{prop.table}}
#' @examples
#' #clustering using pam: try using different dimensions of pca and different k
#' data(simData)
#'
#' cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reducedDim="PCA",
#' clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
#' removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)
#' #give arbitrary names to clusters for demonstration
#' cl<-renameClusters(cl,value=letters[1:nClusters(cl)[1]],whichCluster=1)
#' tableClusters(cl,whichClusters=1:2)
#' #show options of margin in heatmap format:
#' par(mfrow=c(2,3))
#' plotClustersTable(cl,whichClusters=1:2, margin=NA, legend=FALSE,
#'   ignoreUnassigned=TRUE)
#' plotClustersTable(cl,whichClusters=1:2, margin=0, legend=FALSE,
#'   ignoreUnassigned=TRUE)
#' plotClustersTable(cl,whichClusters=1:2, margin=1, legend=FALSE,
#'   ignoreUnassigned=TRUE)
#' plotClustersTable(cl,whichClusters=1:2, margin=2, legend=FALSE,
#'   ignoreUnassigned=TRUE)
#' plotClustersTable(cl,whichClusters=1:2, margin=NULL, legend=FALSE,
#'   ignoreUnassigned=TRUE)
#'
#' #show options of margin in bubble format:
#' par(mfrow=c(2,3))
#' plotClustersTable(cl,whichClusters=1:2, margin=NA, 
#'    ignoreUnassigned=TRUE, plotType="bubble")
#' plotClustersTable(cl,whichClusters=1:2, margin=0,
#'    ignoreUnassigned=TRUE, plotType="bubble")
#' plotClustersTable(cl,whichClusters=1:2, margin=1,
#'    ignoreUnassigned=TRUE, plotType="bubble")
#' plotClustersTable(cl,whichClusters=1:2, margin=2,
#'    ignoreUnassigned=TRUE, plotType="bubble")
#' plotClustersTable(cl,whichClusters=1:2, margin=NULL,
#'    ignoreUnassigned=TRUE, plotType="bubble")
#' @export
setMethod( 
  f = "plotClustersTable",
  signature = signature(object = "ClusterExperiment"),
  definition = function(object, whichClusters,ignoreUnassigned=FALSE,margin=NA,...){
    whCl<-getClusterIndex(object,whichClusters=whichClusters,noMatch="throwError")
    if(length(whCl)!=2) stop("invalid choice of 'whichClusters' -- must be exactly 2 clusterings chosen.")
	tableAll<-tableClusters(object,whichClusters=whCl,useNames=TRUE,tableMethod="intersect")
	#sort them by their names
	#if ever gets slow, could do this directly so not call tableClusters twice...
	isMargin0<-!is.null(margin) && !is.na(margin) && margin==0
	if(isMargin0) denomTab<-.makeUnion(tableAll)
			
	cL<-clusterLegend(object)[whCl]
	if(ignoreUnassigned){
		rNms<-rownames(tableAll)
		cNms<-colnames(tableAll)
		mat1<-cL[[1]]
		mat2<-cL[[2]]
		#convert names to clusterIds to check for missing
		rNms<-mat1[match(rNms,mat1[,"name"]),"clusterIds"]
		cNms<-mat2[match(cNms,mat2[,"name"]),"clusterIds"]
		
		if("-1" %in% rNms || "-2" %in% rNms){
			wh<-which(! rNms %in% c("-1","-2"))
			if(length(wh)>0){
				tableAll<-tableAll[wh, ,drop=FALSE]
				if(isMargin0) denomTab<-denomTab[wh, ,drop=FALSE]
				#deal with fact that plotHeatmap doesn't fix problem with rowData!
				mat1<-mat1[mat1[,"clusterIds"] %in% rNms[wh], ,drop=FALSE]
				cL[[1]]<-mat1
			}
			else stop("All of the first clustering are unassigned, cannot use ignoreUnassigned=TRUE")
			
		}
		if("-1" %in% cNms || "-2" %in% cNms){
			wh<-which(! cNms %in% c("-1","-2"))
			if(length(wh)>0){
				tableAll<-tableAll[, wh,drop=FALSE]
				if(isMargin0) denomTab<-denomTab[, wh ,drop=FALSE]
			}
			else stop("All of the second clustering are unassigned, cannot use ignoreUnassigned=TRUE")
		}
		
	}
	
	#must be after the ignoreUnassigned so as to get rid of those before calculate
	if(is.null(margin) || !is.na(margin)){
		if(isMargin0){
			plotTable<-tableAll/denomTab
			propLabel="Jaccard Similarity"
		}
		else{
			plotTable<-prop.table(tableAll,margin=margin)
			if(is.null(margin)) propLabel="% overlap (out of total cells)"
			else if(margin==1) propLabel="% overlap (out of row total)"
			else if(margin==2) propLabel="% overlap (out of col total)"
		}
	}
	else{
		plotTable<-tableAll
		propLabel<-"Count of Overlap"
	}
	plotClustersTable(plotTable,clusterLegend=cL,sizeTable=tableAll, main=propLabel,...)
}
)

		
#' @rdname plotClustersTable
#' @export
setMethod( 
  f = "plotClustersTable",
  signature = signature(object = "table"),
  definition = function(object,plotType=c("heatmap","bubble"), 
  		main="",xlab=NULL,ylab=NULL,legend=TRUE,cluster=FALSE,clusterLegend=NULL, sizeTable=NULL, ...){
		plotType<-match.arg(plotType)
		tableAll<-object
		if(any(dim(tableAll)==1)&& plotType=="heatmap") stop("Cannot create heatmap when there is only 1 column or row in the table") #creates error in aheatmap....
			
		
		varNames<-make.names(names(dimnames(tableAll)))
 	 	if(!cluster){
			#determine for each column, which is the cluster in the row that is largest
			#will reorder the columns based on this result.
			rankValues<-rank(sapply(seq_len(ncol(tableAll)),FUN=function(ii){
				whMax<-which.max(tableAll[,ii])
				#return(whMax)
				 if(length(whMax)==0) return(1) #could happen in NaN in entry
				 else return(whMax)
			}),ties.method="first")
			order2<-order(rankValues)
			
		}
		else order2<-seq_len(ncol(tableAll))
		
		#rows=y-axis
		defaultY<-is.null(ylab) || ( is.na(ylab) & plotType=="heatmap")
		rData<-data.frame(rownames(tableAll))
		names(rData)<-if(defaultY) varNames[1] else ylab
		
		#cols=x-axis
		defaultX<-is.null(xlab) || ( is.na(xlab) & plotType=="heatmap")
		cData<-data.frame(colnames(tableAll)[order2])
		names(cData)<-if(defaultX) varNames[2] else xlab
		labelCols<-is.null(xlab) || !is.na(xlab)
		
		if(!is.null(clusterLegend)) names(clusterLegend)<-make.names(names(clusterLegend))
		if(plotType=="heatmap"){
			passedArgs<-list(...)
			if(!"colorScale" %in% names(passedArgs)){
				passedArgs$colorScale<-colorRampPalette(c("white","black"))(12)
			}
			passedArgs<-c(list(data=tableAll[,order2,drop=FALSE],colData=cData, annRow=rData,
					clusterLegend=clusterLegend, main=main,annLegend=legend,
					labelTracks =labelCols,
					clusterFeatures=cluster,clusterSamples=cluster)
					,passedArgs)
			 	 	
		
			do.call("plotHeatmap",passedArgs)
		}
		if(plotType=="bubble"){
			if(!is.null(sizeTable))
				bubblePlot(sizeTable=sizeTable[,order2,drop=FALSE],propTable=tableAll[,order2,drop=FALSE],propLabel=main,xlab=xlab,ylab=ylab,legend=legend,...)
			else
				bubblePlot(sizeTable=tableAll[,order2,drop=FALSE], propTable=tableAll[,order2,drop=FALSE],propLabel=main,xlab=xlab,ylab=ylab,legend=legend,...)
			
		} 		
		invisible(tableAll[,order2,drop=FALSE])
  	
  }
	)

#' @rdname plotClustersTable
#' @aliases tableClusters
#' @export
setMethod( 
  f = "tableClusters",
  signature = signature(object = "ClusterExperiment"),
  definition = function(object, whichClusters="primary", useNames=TRUE, tableMethod=c("intersect","union"),...)
  { 
    whichClusters<-getClusterIndex(object,whichClusters=whichClusters,noMatch="throwError")
    tableMethod<-match.arg(tableMethod)
    if(useNames) numCluster<-clusterMatrixNamed(object,whichClusters=whichClusters)
    else numCluster<-clusterMatrix(object)[,whichClusters]
		tabAll<-table(data.frame(numCluster))
    if(tableMethod=="intersect" | length(whichClusters)==1) return(tabAll)
		else return(.makeUnion(tabAll))
})

.makeUnion<-function(tabAll){
	unionTab<-outer(rowSums(tabAll),colSums(tabAll), FUN = "+") - tabAll
	rownames(unionTab)<-rownames(tabAll)
	colnames(unionTab)<-colnames(tabAll)
	names(attributes(unionTab)$dimnames)<-names(attributes(tabAll)$dimnames)
	class(unionTab)<-"table"
	return(unionTab)
}


#' @rdname plotClustersTable

#' @details \code{bubblePlot} is mainly used internally by
#'   \code{plotClustersTable} but is made public for users who want more control
#'   and to allow documentation of the arguments. \code{bubblePlot} plots a
#'   circle for each intersection of two clusters, where the color of the circle
#'   is based on the value in \code{propTable} and the size of the circle is
#'   based on the value in \code{sizeTable}. If \code{propTable}
#'   is equal to \code{sizeTable}, then the \code{propTable} is ignored and the 
#'   coloring of the circles is not performed, only the adjusting of the
#'   size of the circles based on the total size. The size is determined by 
#'   setting the \code{cex} value of the point as
#'   $sqrt(sizeTable[i,j])/sqrt(max(sizeTable))*cexFactor$. 
#' @importFrom grDevices rgb
setMethod( 
  f = "bubblePlot",
  signature = signature(sizeTable="table",propTable = "table"),
	definition=function(propTable,sizeTable,gridColor=rgb(0,0,0,.05),
	maxCex=8,cexFactor, ylab,xlab,propLabel="Value of %",legend=TRUE,
	las=2, colorScale=RColorBrewer::brewer.pal(11,'Spectral')[-6]){

  if(!all(dim(propTable)==dim(sizeTable))) 
	  stop("propTable and sizeTable must be of the same dimensions")
  if(!all(unlist(dimnames(propTable))==unlist(dimnames(sizeTable)))) 
	  stop("propTable and sizeTable must have the same dimnames")
  if(all(na.omit(propTable)==na.omit(sizeTable))) 
	  doProp<-FALSE
  else doProp<-TRUE
  nc.row <- nrow(sizeTable)
  nc.col <- ncol(sizeTable)
  sizeTable<-sizeTable[nc.row:1, ,drop=FALSE]
  propTable<-propTable[nc.row:1, ,drop=FALSE]
  # set up plotting window
  xlim<-c(1,nc.col)
  xlim<-xlim+.1*diff(xlim)*c(-1,1) #increase size 10% around
  ylim<-c(1,nc.row)
  ylim<-ylim+.1*diff(ylim)*c(-1,1) #increase size 10% around
  graphics::plot(0,0,type="n",xlim = xlim, ylim = ylim, ylab="",xlab="",axes=FALSE,frame.plot=FALSE)
   
  # get x-y coords for a grid
  xx <- rep(seq_len(nc.col), each = nc.row)
  yy <- rep(seq_len(nc.row), times = nc.col)   
    
  # set color based on % overlap
  expect.overlap <- min(c(nc.row,nc.col)) / max(c(nc.row,nc.col))
  legend.vals <- pretty(c(0,1),n=50)
  if(doProp){
	  allColors<-.colorby(c(propTable, 0, expect.overlap,legend.vals), colors=colorScale)
		color <- head(allColors,nc.col*nc.row)
	  legend.col<-tail(allColors,length(legend.vals))
	}
	else color<-"grey"
  # put plotting information into data.frame, so we can sort by size (want
  # smaller points plotted over larger points)
  df <- data.frame(xx,yy, color,
      sizeTable = as.numeric(sizeTable), 
      propTable = as.numeric(propTable))[order(as.numeric(sizeTable), decreasing = TRUE), ,drop=FALSE]
  df <- df[df$sizeTable > 0, ,drop=FALSE]
    
  # grid
  abline(v = seq_len(nc.col),col=gridColor)
  abline(h = seq_len(nc.row),col=gridColor)
    
  # points parameters
  cex.pch<-sqrt(df$sizeTable)/sqrt(max(df$sizeTable))
  if(missing(cexFactor)){
	 cexFactor<-maxCex/max(cex.pch)
  }
  cex.pch<-cex.pch*cexFactor
  graphics::points(df$xx,df$yy, cex=cex.pch, col=as.character(df$color), pch=16)
    
  # labels for plots
  axis(1,at=seq_len(nc.col),colnames(sizeTable), adj=1,tick=FALSE,las=las)
  axis(2,at=seq_len(nc.row),rownames(sizeTable), adj=1,tick=FALSE,las=las)
  if(missing(ylab) || is.null(ylab)) ylab<-names(attributes(sizeTable)$dimnames)[1]
  if(missing(xlab)  || is.null(xlab)) xlab<-names(attributes(sizeTable)$dimnames)[2]
  if(!is.na(xlab)) title(xlab=xlab)
  if(!is.na(ylab)) title(ylab=ylab)
		    
  # % overlap legend
  if(legend){
	nboxes<-length(legend.vals)
	usr<-par("usr")
	y<-rep(usr[4]+diff(usr[3:4]*.05), length=nboxes)
	height<-diff(usr[3:4])*.01
	yspace<-diff(usr[3:4])*.01
	if(doProp){
		#legend for color scale
		x<-seq(usr[1],usr[1]+diff(usr[1:2])*.3,length = nboxes)
		width<-(x[2]-x[1])/2
		xspace<-diff(usr[1:2])*.01

		xlabPos<-seq(1,nboxes,length=6)    
		graphics::rect(xright=x-width,xleft=x+width,
			ybottom=y-height,ytop=y+height,
			border=NA,col=legend.col,xpd=NA)
	    graphics::text(x=mean(x), unique(y)+height+yspace, propLabel,xpd=NA,pos=3)
	    graphics::text(x[xlabPos],unique(y)-height-yspace, labels=legend.vals[xlabPos], xpd=NA,pos=1)
	}

	# bubble size legend
	legSizeVals<-pretty(range(as.numeric(sizeTable)),n=5)[-1] #smallest is never needed
	nvals<-length(legSizeVals) #because can't precisely control 'pretty'
	xc<-seq(mean(usr[1:2]),mean(usr[1:2])+diff(usr[1:2])*.5,length = nvals)
	yc<-rep(unique(y), length=nvals)
	graphics::points(xc, yc,
    	cex = sqrt(legSizeVals)/sqrt(max(df$sizeTable))*cexFactor, 
		col=rgb(0,0,0,.4), pch=16,xpd=NA)
    graphics::text(x=xc, y=yc-height-yspace, labels=legSizeVals, xpd=NA,pos=1)
    graphics::text(x=mean(xc), unique(yc)+height+yspace, "# Cells",xpd=NA,pos=3)
  }

})
		
#' @importFrom scales alpha
#' @importFrom RColorBrewer brewer.pal
#' @importFrom grDevices colorRampPalette
.colorby <- function(x, alpha = 1, colors){
    mypal <- grDevices::colorRampPalette(colors)
    if(class(x) %in% c('character','logical')){
        x <- as.factor(x)
    }
    if(class(x) %in% c('integer','numeric')){
        x <- cut(x, breaks = 100)
    }
    if(is(x,'factor')){
        return(scales::alpha(mypal(length(levels(x)))[x],alpha=alpha))
    }
}
		

#' @title Create contrasts for testing DE of a cluster
#' @docType methods
#' @name clusterContrasts
#' @description Uses clustering to create different types of contrasts to be 
#'   tested that can then be fed into DE testing programs.
#' @aliases clusterContrasts,ClusterExperiment-method
#' @param cluster Either a vector giving contrasts assignments or a 
#'   ClusterExperiment object
#' @param contrastType What type of contrast to create. `Dendro' traverses the
#'   given dendrogram and does contrasts of the samples in each side,  `Pairs'
#'   does pair-wise contrasts based on the pairs given in pairMat (if
#'   pairMat=NULL, does all pairwise), and `OneAgainstAll' compares each cluster
#'   to the average of all others.
#' @param dendro The dendrogram to traverse if contrastType="Dendro". Note that
#'   this should be the dendrogram of the clusters, not of the individual
#'   samples, either of class "dendrogram" or "phylo4"
#' @param pairMat matrix giving the pairs of clusters for which to do pair-wise 
#'   contrasts (must match to elements of cl). If NULL, will do all pairwise of 
#'   the clusters in \code{cluster} (excluding "-1" categories). Each row is a
#'   pair to be compared and must match the names of the clusters in the vector 
#'   \code{cluster}.
#' @param removeUnassigned logical, whether to remove negative valued clusters 
#'   from the design matrix. Appropriate to pick TRUE (default) if design will 
#'   be input into linear model on samples that excludes -1.
#' @param outputType character string. Gives format for the resulting contrast 
#'   matrix. Currently the two options are the format appropriate for 
#'   \code{\link[limma]{limma}} and \code{\link[MAST]{MAST}} package.
#' @param ... arguments that are passed to from the \code{ClusterExperiment} 
#'   version to the most basic numeric version.
#' @details The input vector must be numeric clusters, but the external commands
#'   that make the contrast matrix (e.g. \code{\link{makeContrasts}}) require 
#'   syntatically valid R names. For this reason, the names of the levels will 
#'   be "X1" instead of "1". And negative values (if removeUnassigned=FALSE) will 
#'   be "X.1","X.2", etc.
#' @return List with components:
#'\itemize{ 
#'\item{\code{contrastMatrix}}{ Contrast matrix, the form of which
#'depends on \code{outputType}. If \code{outputType=="limma"}, the result of
#'running \code{\link{makeContrasts}}: a matrix with number of columns equal to
#'the number of contrasts, and rows equal to the number of levels of the factor
#'that will be fit in a linear model.} 
#'\item{\code{contrastNames}}{A vector of
#'names for each of the contrasts. NULL if no such additional names.} 
#'}
#' @author Elizabeth Purdom
#' @references Ritchie, ME, Phipson, B, Wu, D, Hu, Y, Law, CW, Shi, W, and
#'   Smyth, GK (2015). limma powers differential expression analyses for
#'   RNA-sequencing and microarray studies. Nucleic Acids Research 43, e47.
#'   http://nar.oxfordjournals.org/content/43/7/e47
#' @references Finak, et al. MAST: a flexible statistical framework for
#'   assessing transcriptional changes and characterizing heterogeneity in
#'   single-cell RNA sequencing data. Genome Biology (2015).
#' @examples 
#' \dontrun{
#' data(simData)
#' cl <- clusterMany(simData,nReducedDims=c(5,10,50),  
#' reduceMethod="PCA", makeMissingDiss=TRUE,
#' clusterFunction="pam", ks=2:4, findBestK=c(FALSE), removeSil=TRUE,
#' subsample=FALSE)
#' #Pairs:
#' clusterContrasts(cl,contrastType="Pairs")
#' #Dendrogram
#' cl<-makeDendrogram(cl)
#' clusterContrasts(cl,contrastType="Pairs")
#' }
#' @export
#' @rdname clusterContrasts
setMethod(
  f = "clusterContrasts",
  signature = "ClusterExperiment",
  definition = function(cluster,contrastType,...){
    if(contrastType=="Dendro"){
      if(is.null(cluster@dendro_clusters)) stop("Must run makeDendrogram before calling clusterContrasts if want contrastType='Dendro'")
      else dendro<-cluster@dendro_clusters
    }
    else dendro<-NULL
    clusterContrasts(primaryCluster(cluster),contrastType=contrastType,dendro=dendro,...)
  })
#' @rdname clusterContrasts
#' @export
#' @importFrom limma makeContrasts
setMethod(
  f = "clusterContrasts",
  signature = "vector",
  definition = function(cluster,contrastType=c("Dendro", "Pairs", "OneAgainstAll"),
                        dendro=NULL, pairMat=NULL,outputType=c("limma","MAST"),removeUnassigned=TRUE){
    outputType<-match.arg(outputType)
    if(outputType=="MAST" & !requireNamespace("MAST", quietly = TRUE)) stop("for outputType 'MAST', you must have package 'MAST' from Bioconductor installed.")
    
    cluster<-.convertToNum(cluster)
    if(removeUnassigned) cl<-cluster[cluster>0] else cl<-cluster
    
    ###--------
    ### Fix up the names
    ###--------
    clPretty<-numericalAsCharacter(cl,prefix="Cl")
	clLevels<-unique(cl[order(clPretty)])
    clPrettyLevels<-unique(clPretty[order(clPretty)])
    #get them ordered
    cl<-factor(cl,levels=clLevels)
    
    
    contrastType<-match.arg(contrastType)
    outputType<-match.arg(outputType)
	
	##############
	## DENDROGRAM 
	##############
    if(contrastType=="Dendro"){
      if(is.null(dendro)) stop("must provide dendrogram if contrastType='Dendro'")
      ####
      #Convert to object used by phylobase
	  #Note dendrogram here is the cluster dendrogram
      phylo4Obj<-.convertToPhyClasses(dendro,"phylo4",convertNodes=TRUE,convertTips=TRUE)
      clChar<-as.character(cl)
	  phylobase::tipLabels(phylo4Obj)<-gsub("ClusterId","",phylobase::tipLabels(phylo4Obj))
      allTipNames<-phylobase::labels(phylo4Obj)[phylobase::getNode(phylo4Obj,  type=c("tip"))]
      if(!identical(sort(unname(allTipNames)),sort(unname(unique(clChar))))) stop("tip names of dendro don't match cluster vector values")
      
      #each internal node (including root) construct contrast between them.
      #(before just tested differences between them)
      allInternal<-phylobase::getNode(phylo4Obj,  type=c("internal"))
      .makeNodeContrast<-function(nodeId){
        children<-phylobase::descendants(phylo4Obj,nodeId,"children") #get immediate descendants
        if(length(children)!=2) stop("More than 2 children for internal node; does not make sense with code")
        #find tips of each child:
        contrAvePerChild<-sapply(children,function(x){
          tips<-phylobase::descendants(phylo4Obj,x,"tip")
          tipNames<-phylobase::labels(phylo4Obj)[tips]
          #should make this code use make.names instead of pasting X...
          if(length(tipNames)>1) return(paste("(",paste(make.names(tipNames),collapse="+"),")/",length(tips),sep=""))
          else return(make.names(tipNames))
        })
        return(paste(contrAvePerChild,collapse="-"))
      }
      contrastNames<-sapply(allInternal,.makeNodeContrast)
      
    }
	
	##############
	## One Against All
	##############
    if(contrastType=="OneAgainstAll"){
      levs<-clPrettyLevels
      contrastNames<-sapply(levs,function(x){
        one<-make.names(x)
        all<-make.names(levs[-which(levs==x)])
        all<-paste("(",paste(all,collapse="+"),")/",length(all),sep="")
        contr<-paste(one,"-",all,sep="")
        return(contr)
      })
      names(contrastNames)<-clPrettyLevels
    }
    
	##############
	## PAIRS 
	##############
	if(contrastType=="Pairs"){
      if(is.null(pairMat)){ #make pair Mat of all pairwise
        levs<-levels(cl)
        pairMat<-t(apply(expand.grid(levs,levs),1,sort))
        pairMat<-unique(pairMat)
        pairMat<-pairMat[-which(pairMat[,1]==pairMat[,2]),,drop=FALSE]
      }
      if(is.null(dim(pairMat)) || ncol(pairMat)!=2) stop("pairMat must be matrix of 2 columns")
      if(!all(as.character(unique(c(pairMat[,1],pairMat[,2]))) %in% as.character(cl))) stop("Some elements of pairMat do not match cl")
      contrastNames <- apply(pairMat,1,function(y){
        yPretty<-make.names(clPrettyLevels[match(as.character(y),as.character(clLevels))])
        paste(yPretty[1],yPretty[2],sep="-")
      })
    }

    levnames<-if(contrastType!="Dendro") clPrettyLevels else make.names(as.character(clLevels))
    if(outputType=="limma"){	
      contr.matrix<-limma::makeContrasts(contrasts=contrastNames,levels=levnames)
      return(list(contrastMatrix=contr.matrix,contrastNames=names(contrastNames)))
    }
    if(outputType=="MAST") return(MAST::Hypothesis(contrastNames, levnames))
    
  })

#' Simulated data for running examples
#'
#' @name simData
#' @aliases simCount trueCluster
#' @docType data
#' @author Elizabeth Purdom \email{epurdom@@stat.berkeley.edu}
#' @format Three objects are loaded, two data frame(s) of simulated data each
#'   with 300 samples/columns and 153 variables/rows, and a vector of length 300
#'   with the true cluster assignments.
#' @details \code{simData} is simulated normal data of 300 observations with 51
#'   relevant variables and the rest of the variables being noise, with
#'   observations being in one of 3 groups. \code{simCount} is simulated count
#'   data of the same dimensions. \code{trueCluster} gives the true cluster
#'   identifications of the samples. The true clusters are each of size 100 and
#'   are in order in the columns of the data.frames.
#' @keywords data
#' @examples
#' #code used to create data:
#' \dontrun{
#' nvar<-51 #multiple of 3
#' n<-100
#' x<-cbind(matrix(rnorm(n*nvar,mean=5),nrow=nvar),
#'  matrix(rnorm(n*nvar,mean=-5),nrow=nvar),
#'           matrix(rnorm(n*nvar,mean=0),nrow=nvar))
#' #make some of them flipped effects (better for testing if both sig under/over
#' #expressed variables)
#' geneGroup<-sample(rep(1:3,each=floor(nvar/3)))
#' gpIndex<-list(1:n,(n+1):(n*2),(2*n+1):(n*3))
#' x[geneGroup==1,]<-x[geneGroup==1,unlist(gpIndex[c(3,1,2)])]
#' x[geneGroup==2,]<-x[geneGroup==2,unlist(gpIndex[c(2,3,1)])]
#'
#' #add in differences in variable means
#' smp<-sample(1:nrow(x),10)
#' x[smp,]<-x[smp,]+10
#'
#' #make different signal y
#' y<-cbind(matrix(rnorm(n*nvar,mean=1),nrow=nvar),
#'          matrix(rnorm(n*nvar,mean=-1),nrow=nvar),
#'          matrix(rnorm(n*nvar,mean=0),nrow=nvar))
#' y<-y[,sample(1:ncol(y))]+ matrix(rnorm(3*n*nvar,sd=3),nrow=nvar)
#'
#' #add together the two signals
#' simData<-x+y
#'
#' #add pure noise variables
#' simData<-rbind(simData,matrix(rnorm(3*n*nvar,mean=10),nrow=nvar),
#'                matrix(rnorm(3*n*nvar,mean=5),nrow=nvar))
#' #make count data
#' countMean<-exp(simData/2)
#' simCount<-matrix(rpois(n=length(as.vector(countMean)), lambda
#' =as.vector(countMean)+.1),nrow=nrow(countMean),ncol=ncol(countMean))
#' #labels for the truth
#' trueCluster<-rep(c(1:3),each=n)
#' save(list=c("simCount","simData","trueCluster"),file="data/simData.rda")
#' }
NULL


#' @title Documentation of rsecFluidigm object
#' @description  Documentation of the creation of rsecFluidigm, result of RSEC
#'   run on fluidigm data for vignette
#'
#' @name rsecFluidigm
#' @docType data
#' @author Elizabeth Purdom \email{epurdom@@stat.berkeley.edu}
#' @format \code{rsecFluidigm} is a \code{ClusterExperiment} object, the result
#'   of running \code{\link{RSEC}} on fluidigm data described in vignette and
#'   available in the \code{scRNAseq} package.
#' @details The functions \code{makeRsecFluidigmObject} and 
#'   \code{checkRsecFluidigmObject} are helper functions whose sole purpose is 
#'   to create \code{rsecFluidigm} and check that the results are the same as 
#'   expected. \code{makeRsecFluidigmObject} also serves as documentation of the
#'   specific RSEC call that was made to create the \code{rsecFluidigm} object, 
#'   as well as filtering and normalization of the fluidigm data.
#'   The purpose of making them functions is internal, to help more easily
#'   mantain and check if changes to the package have affected the results.
#' @seealso \code{\link[scRNAseq]{fluidigm}}. 
#' @keywords data
#' @examples
#' # see code used create rsecFluidigm 
#' # (print out the function)
#' makeRsecFluidigmObject
#' #code actualy run to create rsecFluidigm:
#' \dontrun{
#' library(clusterExperiment)
#' data(fluidigmData)
#' data(fluidigmColData)
#' se<-SummarizedExperiment(assays=fluidigmData, colData=fluidigmColData)
#' RNGversion("3.5.0")
#' rsecFluidigm<-makeRsecFluidigmObject(se)
#' # Internal function for checking got correct results...
#' clusterExperiment:::checkRsecFluidigmObject(rsecFluidigm)
#' usethis::use_data(rsecFluidigm,overwrite=FALSE)
#' }
#' @aliases makeRsecFluidigmObject
#' @param object object given to functions
#' @export
#' @importFrom S4Vectors metadata
makeRsecFluidigmObject<-function(object){
    pass_filter <- apply(assay(object), 1, 
        function(x) length(x[x >= 10]) >= 10)
    object <- object[pass_filter,]
    fq <- round(limma::normalizeQuantiles(assay(object)))
    assays(object) <- c(SimpleList(normalized_counts=fq),assays(object))
    wh<-which(colnames(colData(object)) %in% c("Cluster1","Cluster2"))
    colnames(colData(object))[wh]<-c("Published1","Published2")
    ncores<-1
    rsecFluidigm<-RSEC(object,
                      isCount = TRUE,
                      k0s = 4:15,
                      alphas=c(0.1, 0.2, 0.3),
                      betas = 0.9,
                      reduceMethod="PCA",
                      nReducedDims=10,
                      minSizes=1,
                      subsample=TRUE,
                      sequential=TRUE,
                      clusterFunction="hierarchical01",
                      consensusMinSize=3,
                      consensusProportion=0.7,
                      dendroReduce= "mad",
                      dendroNDims=1000,
                      mergeMethod="adjP",
                      mergeDEMethod="limma",
                      mergeCutoff=0.01,
                      ncores=ncores,
                      makeMissingDiss=TRUE,
                      mainClusterArgs=list(
                          clusterArgs=list(removeDup=FALSE)),
                      #seqArgs=list(top.can=5),
                      subsampleArgs=list(clusterFunction="kmeans",
                          classifyMethod="All"),
                      consensusArgs=list(clusterFunction="hierarchical01",
                            whenUnassign="before",
                            clusterArgs=list(
                                evalClusterMethod=c("average"),
                                removeDup=FALSE)),
                      random.seed=176201
    )
    metadata(rsecFluidigm)$packageVersion <- packageVersion("clusterExperiment")
    return(rsecFluidigm)
}

checkRsecFluidigmObject<-function(object){
    ## Simple Tests that haven't changed the clustering algorithms such that get different results.
    ## Don't simply do all.equal with old one because might of changed something minor not related to the actual algorithms

    ## Results for feature/knn 07/25/2019 -- 2.5.4.9005
    nMakeConsensus<-7
    nMerge<-5
    contrasts<-c('(X4+X1+X5)/3-(X6+X7+X2+X3)/4','X4-(X1+X5)/2',
        'X6-(X7+X2+X3)/3','X7-(X2+X3)/2','X1-X5','X2-X3')                  
    adjPValues<-c(0.06167775,0.01117556,0.01697553,0.00042439,0.01004385,0.00155609)
       
    ## Test same
    checkValues<-.getCheckValues(object)
    if(nMakeConsensus==nMerge) warning("having same number of clusters before and after merge won't be great for the vignette!")
    if(checkValues$nMakeConsensus != nMakeConsensus)
      stop("rsecFluidigm has changed -- makeConsensus")
    if(!is.logical(all.equal(contrasts,checkValues$contrasts)))
        stop("rsecFluidigm has changed -- different dendrogram")
    if(!is.logical(all.equal(adjPValues,checkValues$adjPValues)))
      stop("rsecFluidigm has changed -- different merge percentages")
    if(checkValues$nMerge != nMerge)
      stop("rsecFluidigm has changed -- different # of mergeClusters")
    
}
.getCheckValues<-function(object, printout=FALSE){
    x<-unique(clusterMatrix(object)[,"makeConsensus"])
    y<-unique(clusterMatrix(object)[,"mergeClusters"])
    out<-list(
        nMakeConsensus=length(x[x>0]),
        nMerge=length(y[y>0]),
        contrasts=object@merge_nodeProp[,"Contrast"],
        adjPValues=round(object@merge_nodeProp[,"adjP"],8)
        )
    if(printout){
        out$contrasts<-paste("c('",paste(out$contrasts,collapse="','"),"')",sep="")
        out$adjPValues<-paste("c(",paste(out$adjPValues,collapse=","),")",sep="")
    }
    return(out)
}





#' Subset of fluidigm data
#'
#' @name fluidigmData
#' @aliases fluidigmColData
#' @docType data
#' @author Elizabeth Purdom \email{epurdom@@stat.berkeley.edu}
#' @format subset of fluidigm data used in vignette
#' package.
#' @seealso \code{\link[scRNAseq]{fluidigm}}
#' @keywords data
#' @details \code{fluidigmData} and \code{fluidigmColData} are portions of the \code{fluidigm} data distributed in the package \code{scRNAseq} package. We have subsetted to only the cells sequenced under high depth, and limited our selves to only two of the four gene estimates provided by \code{scRNAseq} ("tophat_counts" and "rsem_tpm").
#' @examples
#' #code used to create objects:
#' \dontrun{
#' library(scRNAseq)
#' if(packageVersion("scRNAseq")>="1.11.0") fluidigm <- ReprocessedFluidigmData() else data(fluidigm)
#' fluidSubset<- fluidigm[,colData(fluidigm)[,"Coverage_Type"]=="High"]
#' fluidigmData<-assays(fluidSubset)[c("tophat_counts","rsem_tpm")]
#' fluidigmColData<-as.data.frame(colData(fluidSubset))
#' usethis::use_data(fluidigmData, fluidigmColData, overwrite=FALSE)
#' }
NULL

context("RsecFluidigm")

test_that("saved rsecFluidigm is still valid object", {
	data(rsecFluidigm)
	expect_true(validObject(rsecFluidigm))
    
    #checks that current code will still recreate same basic rsecFluidigm
    # takes a bit of time, so skip on bioc, just want on travis so get alert
    # that need to deal with this.
#    skip_on_bioc()
    data(fluidigmData)
    data(fluidigmColData)
    expect_silent(se<-SummarizedExperiment(assays=fluidigmData,
        colData=fluidigmColData))
    expect_message(rsecFluidigmNew<-makeRsecFluidigmObject(se),
        "Will calculate all the methods requested")
    expect_silent(checkRsecFluidigmObject(rsecFluidigmNew))
    data(rsecFluidigm)
    expect_equal(clusterMatrix(rsecFluidigmNew),
        clusterMatrix(rsecFluidigm))
    
    
})

context("makeConsensus")


test_that("`makeConsensus` works with matrix and ClusterExperiment objects", {
    expect_silent(clustNothing <- clusterMany(mat,
        ks=c(3,4),clusterFunction="pam",
        subsample=FALSE, sequential=FALSE,
        isCount=FALSE,verbose=FALSE))
        
    #check proportion=1
    expect_silent(x1<-makeConsensus(clustNothing,
        proportion=1,minSize=1,
        whichClusters = "clusterMany"))
    expect_equal(nClusters(x1)[["makeConsensus"]],
        nrow(unique(clusterMatrix(clustNothing))))
    expect_message(x2<-makeConsensus(clustNothing,
        proportion=1,minSize=1),
        "no clusters specified to combine")
    expect_equal(x1,x2)


    expect_silent(shared1 <- makeConsensus(clusterMatrix(clustNothing),
        proportion=1))
    expect_silent(shared2 <- makeConsensus(clustNothing, 
        "all",proportion=1))
    expect_equal(shared1, primaryCluster(shared2))

    expect_silent(shared3 <- makeConsensus(clustNothing,
         "workflow",proportion=1))
    expect_equal(shared2, shared3)

    expect_silent(shared4 <- makeConsensus(clustNothing,
        1:nClusterings(clustNothing),proportion=1))
    expect_equal(shared3, shared4)

    expect_silent(shared5 <- makeConsensus(clustNothing, 
        "workflow", proportion=.5))

    expect_silent(shared6 <- makeConsensus(clustNothing, 
        "workflow",
        proportion=.5, clusterFunction="tight"))
	expect_true("makeConsensus" %in% clusterTypes(shared6))
	expect_true("makeConsensus" %in% clusterLabels(shared6))
	expect_true(all(primaryCluster(shared6)==
        clusterMatrix(shared6)[,"makeConsensus"]))

  	#----
  	##Check error catching:
  	#----
	expect_silent(ceObj<-clusterSingle(mat, subsample=FALSE,
		mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=3))))
  	expect_error(makeConsensus(ceObj,proportion=1),
                   "no clusters specified")
 	expect_error(makeConsensus(ceObj,
        whichClusters = "all",proportion=-1),
           "Invalid value for the 'proportion' parameter")
           
    #     # FIXME: This should work, once I fix the 'cat' stuff.
    #     # right now just problem of order of when do calculations.
    # expect_error(makeConsensus(ceObj,
    #         whichClusters = "all",proportion=0.7, minSize=-3),
    #        "Invalid value for the 'minSize' parameter")
	expect_error(makeConsensus(ceObj,
        whichClusters = "all",proportion=0.7, propUnassigned=3),
	       "Invalid value for the 'propUnassigned' parameter")
	expect_error(makeConsensus(ceObj,
        whichCluster="clusterSingle"),
  	        'argument "proportion" is missing, with no default')
  	expect_error(makeConsensus(ceObj,
        whichCluster="clusterSingle"),
  			'argument "proportion" is missing, with no default')
    expect_error(makeConsensus(clustNothing, "workflow",
           proportion=.5,clusterFunction="pam"), 
		   "makeConsensus is only implemented for '01' type")

})

test_that("`makeConsensus` works with hdf5",{
    expect_silent(clustNothing <- clusterMany(hdfSCE,
		 ks=c(3,4),clusterFunction="pam",
		 subsample=FALSE, sequential=FALSE,
		 isCount=FALSE,verbose=FALSE))
    expect_silent(x1<-makeConsensus(clustNothing,proportion=1,whichClusters = "clusterMany"))
	
})

test_that("`makeConsensus` works when multiple runs of workflow", {
  expect_silent(clustNothing <- clusterMany(mat, ks=c(3,4),clusterFunction="pam",
                              subsample=FALSE, sequential=FALSE,
                              isCount=FALSE,verbose=FALSE))

  expect_silent(shared1 <- makeConsensus(clustNothing, "all",proportion=1))
  expect_silent(shared2<-makeConsensus(shared1,"all",proportion=1))
  expect_true("makeConsensus.1" %in% clusterTypes(shared2))

  expect_silent(clustNothing2 <- clusterMany(shared2, ks=c(5,6), clusterFunction="pam",
                               subsample=FALSE, sequential=FALSE, verbose=FALSE))
  expect_true("makeConsensus.1" %in% clusterTypes(clustNothing2))
  expect_true("clusterMany.2" %in% clusterTypes(clustNothing2))
  expect_true("makeConsensus.2" %in% clusterTypes(clustNothing2))

  expect_silent(shared3 <- makeConsensus(clustNothing2, "all",proportion=1))
  expect_silent(shared4 <- makeConsensus(clusterMatrix(clustNothing2),proportion=1))
  expect_equal(shared4, primaryCluster(shared3))

  expect_silent(shared5 <- makeConsensus(clustNothing2, "workflow",proportion=1))
  expect_silent(shared6 <- makeConsensus(clusterMatrix(clustNothing2)[,1:2],proportion=1))
  expect_equal(shared6, primaryCluster(shared5))


  expect_silent(clustNothing3 <- addClusterings(clustNothing2, primaryCluster(shared5)))
  expect_silent(shared7 <- makeConsensus(clustNothing3, "all",proportion=1))
  expect_silent(shared8 <- makeConsensus(clustNothing3, "workflow",proportion=1))
})


test_that("`makeConsensus` more esoteric options", {
    expect_silent(clustNothing <- clusterMany(mat,
       ks=c(3,4),clusterFunction="pam",
       subsample=FALSE, sequential=FALSE,
       isCount=FALSE,verbose=FALSE))

    #----
    #check passing to clusterArgs
    #----
    expect_silent(shared1 <- makeConsensus(clustNothing, "all",
        proportion=0.7,clusterArgs=list(removeDup=FALSE)))
    #silently ignores passing alpha
    expect_silent(shared2 <- makeConsensus(clustNothing, "all",
        proportion=0.7,clusterArgs=list(removeDup=FALSE,alpha=0.5)))
    expect_equal(primaryCluster(shared1),primaryCluster(shared2))
    expect_silent(makeConsensus(clustNothing, "all",
        proportion=0.7,clusterArgs=list(removeDup=FALSE,
            evalClusterMethod=c("maximum"))))
    
    #----
    # Check whenUnassign
    #----
    expect_silent(makeConsensus(clustNothing, "all",
        proportion=0.7,whenUnassign="before"))
    expect_silent(makeConsensus(clustNothing, "all",
        proportion=0.7,whenUnassign="after"))
        
    #check if all -1 values
  	expect_silent(cc<-clusterMany(x=mat, 
  				   isCount=FALSE,reduceMethod="none",ks=4:5,
  	               clusterFunction="tight", alphas=0.1, 
                   sequential=TRUE, subsample=TRUE, verbose=FALSE,
  	               subsampleArgs=list(resamp.num=5),random.seed=495))
    #check all -1 assignments
    expect_equal(unname(unique(clusterMatrix(cc))),matrix(-1,nrow=1,ncol=2))
    expect_silent(makeConsensus(cc, "all",
       proportion=0.7,whenUnassign="before"))
    expect_silent(makeConsensus(cc, "all",
       proportion=0.7,whenUnassign="after"))
    #check all but one sample have -1 assignments
    test<-clusterMatrix(cc)
    test[1,]<-c(1,2)
    expect_silent(makeConsensus(test, 
       proportion=1,minSize=1,whenUnassign="before"))
    expect_silent(makeConsensus(test, 
        proportion=1,minSize=1,whenUnassign="after"))
    expect_silent(makeConsensus(test, 
       proportion=0.7,minSize=1,whenUnassign="before"))
    expect_silent(makeConsensus(test, 
        proportion=0.7,minSize=1,whenUnassign="after"))


})

test_that("`RSEC` pass options to `makeConsensus`", {
	## these examples don't do dendrogram/merge because all -1 after makeConsensus
	## only tests clusterMany, makeConsensus parts.
	## so can't do expect_silent, because returns NOTE about that issue.
    seedValue<-495 
	expect_message(rsecOut1<-RSEC(x=mat,
        isCount=FALSE,reduceMethod="none",k0s=4:5,
		clusterFunction="tight", alphas=0.1,dendroReduce="none",
        subsampleArgs=list(resamp.num=5), consensusProportion=0.7,
        consensusMinSize=3, sequential=FALSE,
        consensusArgs=list(clusterFunction="tight"),
        random.seed=seedValue
  	 	),"makeDendrogram encountered following error")
    test<-makeConsensus(clusterMatrix(rsecOut1,
        whichClusters="clusterMany"),
        proportion=0.7,minSize=3,
        clusterFunction="tight")
    expect_equal(test,
        clusterMatrix(rsecOut1,"makeConsensus")[,1])

	expect_message(rsecOut1<-RSEC(x=mat,
        isCount=FALSE,reduceMethod="none",k0s=4:5,
		clusterFunction="tight", alphas=0.1,dendroReduce="none",
        subsampleArgs=list(resamp.num=5),
        consensusProportion=0.7,sequential=FALSE,
        consensusMinSize=3,
        consensusArgs=list(
            clusterFunction="hierarchical01",
            clusterArgs=list(removeDup=FALSE,
                evalClusterMethod=c("maximum") )),
        random.seed=seedValue
  	 	),"makeDendrogram encountered following error")
    test<-makeConsensus(clusterMatrix(rsecOut1,
        whichClusters="clusterMany"),
        proportion=0.7,minSize=3,
        clusterArgs=list(removeDup=FALSE,
            evalClusterMethod=c("maximum")),
        clusterFunction="hierarchical01")
    expect_equal(test,
        clusterMatrix(rsecOut1,"makeConsensus")[,1])

})

test_that("`makeConsensus` preserves the colData and rowData of SE", {
    expect_silent(cl <- clusterMany(se, verbose=FALSE,
        clusterFunction="pam", ks=2:4, findBestK=c(FALSE),
        removeSil=TRUE, subsample=FALSE,
        makeMissingDiss=TRUE))

    expect_silent(cl <- makeConsensus(cl, 
        whichClusters="workflow", proportion=0.5))

    expect_equal(colData(cl),colData(se))
    expect_equal(rownames(cl),rownames(se))
    expect_equal(colnames(cl),colnames(se))
    expect_equal(metadata(cl),metadata(se))
    expect_equal(rowData(cl),rowData(se))

})

context("getBestFeatures")

plotAll<-FALSE #set to true to actually SEE the plots; otherwise for TravisCI, where no visual, runs quicker with FALSE
###Note some are still run with plot=TRUE to check that works with aheatmap. Only a fraction not plotted.
test_that("`clusterContrasts` works with matrix and ClusterExperiment objects", {
  expect_silent(x1<- clusterContrasts(primaryCluster(ccSE),contrastType="Pairs"))
  expect_silent(x2<-clusterContrasts(ccSE,contrastType="Pairs"))
  expect_equal(x1,x2)
  expect_silent(x1<- clusterContrasts(primaryCluster(ccSE),contrastType="OneAgainstAll"))
  expect_silent(x2<-clusterContrasts(ccSE,contrastType="OneAgainstAll"))
  expect_equal(x1,x2)
  expect_error(clusterContrasts(primaryCluster(ccSE),contrastType="Dendro"),"must provide dendrogram if contrastType='Dendro'")
  expect_silent(ccSE<-makeDendrogram(ccSE))
  expect_silent(x1<- clusterContrasts(primaryCluster(ccSE),contrastType="Dendro",dendro=ccSE@dendro_clusters))
  expect_false(any(is.na(x1$contrastNames)))
  expect_silent(x2<-clusterContrasts(ccSE,contrastType="Dendro"))
  expect_equal(x1,x2)
})



test_that("`getBestFeatures`  matrix and CE return same", {

  ## add some unclustered
  expect_silent(top1 <- getBestFeatures(simData, 
	  primaryCluster(ceSimData), contrastType="F", DEMethod="limma")
						)
  idx <- top1$IndexInOriginal
  expect_equal(rowMeans(simData[idx,primaryCluster(ceSimData)>0]), top1$AveExpr)


  expect_silent(top2 <- getBestFeatures(simData, 
	  primaryCluster(ceSimData), contrastType="Pairs",
                        DEMethod="limma"))
  idx <- top2$IndexInOriginal
  expect_equal(rowMeans(simData[idx,primaryCluster(ceSimData)>0]), top2$AveExpr)
  expect_silent(topC2 <- getBestFeatures(ceSimData, contrastType="Pairs",DEMethod="limma"))
  neqcolsCE<-sapply(c("ContrastName"   , "InternalName"),grep,colnames(topC2))
  neqcolsMat<-sapply(c("ContrastName" ),grep,colnames(top2))
  expect_equal(topC2[,-neqcolsCE], top2[,-neqcolsMat])

  expect_silent(top3 <- getBestFeatures(simData, 
	  primaryCluster(ceSimData), contrastType="OneAgainstAll",
                        DEMethod="limma"))
  idx <- top3$IndexInOriginal
  expect_equal(rowMeans(simData[idx,primaryCluster(ceSimData)>0]), top3$AveExpr)
  expect_silent(topC3 <- getBestFeatures(ceSimData, 
	  contrastType="OneAgainstAll", 
      DEMethod="limma"))
		  neqcolsCE<-sapply(c("ContrastName"   , "InternalName"),grep,colnames(topC3))
		  neqcolsMat<-sapply(c("ContrastName" ),grep,colnames(top3))
  expect_equal(topC3[,-neqcolsCE], top3[,-neqcolsMat])

   
  ### test voom

  logcpm <- t(log2(t(simCount + 0.5)/(colSums(simCount) + 1) * 1e+06))
  expect_silent(voom1 <- getBestFeatures(simCount, 
	  primaryCluster(ceSim), contrastType="F",
                        DEMethod="limma-voom"))
  idx <- voom1$IndexInOriginal
  expect_equal(rowMeans(logcpm[idx,primaryCluster(ceSim)>0]), voom1$AveExpr)

  expect_silent(voom2 <- getBestFeatures(simCount,
	   primaryCluster(ceSim), contrastType="Pairs",
                        DEMethod="limma-voom"))
  idx <- voom2$IndexInOriginal
  expect_equal(rowMeans(logcpm[idx,primaryCluster(ceSim)>0]), voom2$AveExpr)

  expect_silent(voom3 <- getBestFeatures(simCount, primaryCluster(ceSim), contrastType="OneAgainstAll",
                        DEMethod="limma-voom"))
  idx <- voom3$IndexInOriginal
  expect_equal(rowMeans(logcpm[idx,primaryCluster(ceSim)>0]), voom3$AveExpr)



}
)

test_that("`getBestFeatures` works with ClusterExperiment objects", {
    ## check F
    expect_silent(top1 <- getBestFeatures(simData, 
  	  primaryCluster(ceSimData), contrastType="F", DEMethod="limma"))
    expect_silent(topC0 <- getBestFeatures(ceSimData, DEMethod="limma"))
    expect_silent(topC1 <- getBestFeatures(ceSimData, contrastType="F",  DEMethod="limma"))
    expect_equal(topC1, topC0)
    expect_equal(topC1, top1)
	
	##check return format of contrasts
	# > clusterLegend(ceSimData)[[primaryClusterIndex(ceSimData)]]
# 	     clusterIds color     name
# 	[1,] "-1"       "white"   "-1"
# 	[2,] "-2"       "grey"    "-2"
# 	[3,] "1"        "#E31A1C" "a"
# 	[4,] "2"        "#1F78B4" "b"
# 	[5,] "3"        "#33A02C" "c"
# 	[6,] "4"        "#FF7F00" "d"
	expect_silent(ceSimData<-renameClusters(ceSimData,letters[1:4]))

    expect_silent(topPairs <- getBestFeatures(ceSimData, DEMethod="limma",contrastType="Pairs"))
    expect_equal(as.character(topPairs$ContrastName[1]),"a-b")
    expect_equal(as.character(topPairs$InternalName[1]),"Cl01-Cl02")

    expect_silent(topOne <- getBestFeatures(ceSimData, DEMethod="limma",contrastType="OneAgainstAll"))
    expect_equal(as.character(topOne$ContrastName[1]),"a")
    expect_equal(as.character(topOne$InternalName[1]),"Cl01")
	
	expect_silent(ceDend <- makeDendrogram(ceSimData))
	expect_silent(topDend<-getBestFeatures(ceDend, DEMethod="limma",contrastType="Dendro"))
	##Need to add correct expection after work out labels.
})
test_that("'Dendro' contrasts works for ClusterExperiment object in `getBestFeatures`",{
    #test dendrogram
    ## test dendrogram
    expect_error(getBestFeatures(simData, primaryCluster(ceSim), contrastType="Dendro"), "must provide dendro")
  
    expect_silent(dendro <- makeDendrogram(simData, primaryCluster(ceSimData)))
    expect_error(getBestFeatures(simData, primaryCluster(ceSimData), contrastType="Dendro", dendro=dendro$samples,DEMethod="limma"), "dendro don't match")
    expect_silent(dend1 <- getBestFeatures(simData, primaryCluster(ceSimData), contrastType="Dendro", dendro = dendro$clusters,DEMethod="limma"))
						   
    length(grep("NodeId",dend1$ContrastName))
    ceTemp<-ceSimData
    expect_silent(ceTemp <- makeDendrogram(ceTemp))
    expect_silent(dendC1 <- getBestFeatures(ceTemp, contrastType="Dendro",DEMethod="limma"))
    # Test: dend1 is from applying to data and giving dendro; dendC1 is updating clusterExperiment object
	# dend1 doesn't have 'InternalName' as column.
    neqcolsCE<-sapply(c("InternalName"), grep,colnames(dendC1))
    expect_equal(dend1, dendC1[,-neqcolsCE])

  #check whole mergeDendrogram thing at least runs!
  expect_silent(cl1 <- clusterSingle(smSimData, 
                       subsample=FALSE, sequential=FALSE,
                       mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=6)), 
											 isCount=FALSE)
											 )
  expect_silent(clustWithDendro <- makeDendrogram(cl1))
  expect_message(clustMerged <- mergeClusters(clustWithDendro, mergeMethod="adjP",plotInfo="none",plot=FALSE,calculateAll=FALSE, DEMethod="limma"),"Merging will be done on")
  expect_silent(getBestFeatures(clustMerged, contrastType="Dendro",DEMethod="limma"))

})
test_that("`getBestFeatures` works with HDF5 assay slot",{
    expect_silent(cl1 <- clusterSingle(hdfObj, 
            subsample=FALSE, sequential=FALSE,
			mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=6)),
			isCount=FALSE))
    expect_silent(getBestFeatures(cl1,DEMethod="limma"))
								
	
})
test_that("`plotContrastHeatmap` works", {
	expect_silent(ceSimData<-renameClusters(ceSimData,whichCluster=1,val=letters[1:nClusters(ceSimData)[1]]))
    expect_silent(topC2 <- getBestFeatures(ceSimData, contrastType="Pairs", DEMethod="limma"))
	expect_silent(plotContrastHeatmap(ceSimData,signifTable=topC2))

	expect_silent(topCOne <- getBestFeatures(ceSimData, contrastType="OneAgainstAll", DEMethod="limma"))
	expect_silent(plotContrastHeatmap(ceSimData,signifTable=topCOne,plot=plotAll))
	  
    dendro <- makeDendrogram(ceSimData, whichCluster=primaryClusterIndex(ceSimData))
	# > 	phylobase::nodeLabels(dendro@dendro_clusters)
# 	      5       6       7
# 	"Node1" "Node2" "Node3"
	topCD <- getBestFeatures(dendro, contrastType="Dendro", DEMethod="limma")
	plotContrastHeatmap(dendro,signifTable=topCD,plot=plotAll)
	
    top1 <- getBestFeatures(simData, primaryCluster(ceSimData), contrastType="F",
                          DEMethod="limma")
	expect_error(plotContrastHeatmap(dendro,signifTable=top1),"signifTable must have columns 'IndexInOriginal' and 'Contrast'")
						  
	#test name replacement:
	plotContrastHeatmap(ceSimData,signifTable=topC2,whichCluster=primaryClusterIndex(ceSimData),plot=plotAll)
	plotContrastHeatmap(ceSimData,signifTable=topCOne,whichCluster=primaryClusterIndex(ceSimData),plot=plotAll)
	plotContrastHeatmap(ceSimData,signifTable=topCD,whichCluster=primaryClusterIndex(ceSimData),plot=plotAll)
	expect_error(plotContrastHeatmap(ceSimData,signifTable=topC2,whichCluster=c(1,2)),"must identify only a single clustering")
	expect_error(plotContrastHeatmap(ceSimData,signifTable=topC2,whichCluster=50),"Invalid value for 'whichCluster'. Must be integer between")
	
})

test_that("`getBestFeatures` works with weights", {
	# These are slow!
	skip_on_os("windows")
	skip_on_os("mac")
	
	set.seed(258179)
	weights<-matrix(runif(nrow(seSimCount)*ncol(seSimCount)),nrow=nrow(seSimCount),ncol=ncol(seSimCount))
	ceSimW<-ceSim
	assay(ceSimW,"weights")<-weights

	expect_silent(outW<-getBestFeatures(ceSim,weights=weights,DEMethod="edgeR",contrastType="F"))
	expect_silent(outNW<-getBestFeatures(ceSim,weights=NULL,DEMethod="edgeR",contrastType="F"))
	expect_silent(outV<-getBestFeatures(ceSim,weights=weights,DEMethod="limma-voom",contrastType="F"))
	expect_silent(outV2<-getBestFeatures(ceSim,DEMethod="limma-voom",contrastType="F"))
	expect_equal(outV,outV2)

	expect_silent(outNW_C<-getBestFeatures(ceSim,weights=NULL,DEMethod="edgeR",contrastType="Pairs"))
	expect_silent(outW_C<-getBestFeatures(ceSim,weights=weights,DEMethod="edgeR",contrastType="Pairs"))
	expect_silent(getBestFeatures(ceSim,weights=weights,DEMethod="edgeR",contrastType="Pairs",contrastAdj="PerContrast"))
	expect_silent(getBestFeatures(ceSim,weights=weights,DEMethod="edgeR",contrastType="Pairs",contrastAdj="AfterF"))
	
	#-------
	##Test get right answers with weights (have to remove unassigned)
	#-------
	expect_silent(cleanSim<-removeUnassigned(ceSimW,whichCluster="primary"))
	expect_true(all(primaryCluster(cleanSim)>0))
	expect_silent(pairsContrasts<-clusterContrasts(cleanSim,contrastType="Pairs"))
  designContr <- model.matrix(~ 0 + factor(primaryCluster(cleanSim)))
  fitContr<- edgeR::DGEList(assay(cleanSim,1))
  fitContr$weights <- assay(cleanSim,"weights")
  fitContr <- edgeR::estimateDisp(fitContr, designContr)
  fitContr <- edgeR::glmFit(fitContr,design = designContr)
  lrt <- zinbwave::glmWeightedF(fitContr, contrast = pairsContrasts$contrastMatrix[,1,drop=FALSE])
  resultsManual<- edgeR::topTags(lrt)
	expect_silent(outWClean<-getBestFeatures(cleanSim,DEMethod="edgeR",contrastAdj="PerContrast",contrastType="Pairs"))
	resultsC1<-outWClean[outWClean$Contrast == colnames(pairsContrasts$contrastMatrix)[1],]
	expect_equal(unname(data.matrix(resultsC1[,c("logFC","logCPM","LR","P.Value","padjFilter","adj.P.Val")])), unname(data.matrix(data.frame(resultsManual))))
	
	### check when weights in assay picks them up automatically (when have right name...)
	expect_silent(outNW_C2<-getBestFeatures(ceSimW,weights=NULL,DEMethod="edgeR",contrastType="Pairs"))
	expect_equal(outNW_C2,outNW_C)

	expect_silent(outW_C2<-getBestFeatures(ceSimW,DEMethod="edgeR",contrastType="Pairs"))
	expect_equal(outW_C2,outW_C)
	
	##Test mergeClusters
	expect_silent(ceSimW<-makeDendrogram(ceSimW))
	expect_message(mergeClusters(ceSimW,DEMethod="edgeR"),"Merging will be done on")
	expect_message(mergeClusters(ceSimW,DEMethod="edgeR",weights=NULL,forceCalculate=TRUE),"Merging will be done on")
	
	##Test RSEC
	expect_message(RSEC(ceSimW,sequential=FALSE,subsample=FALSE,ks=4:15,isCount=TRUE,stopOnError=TRUE),"merging with these parameters did not result in any clusters being merged")
	
	sceCountData<-SingleCellExperiment(simCount)
	reducedDims(sceCountData) <- reducedDims(sceSimDataDimRed)	
	expect_message(RSEC(sceCountData,sequential=TRUE,subsample=FALSE,clusterFunction="pam",k0s=4:15,isCount=TRUE, stopOnError=TRUE),"Merging will be done on")
	
})

context("Assays")

## Construct ClusterExperiment with multiple named assays
suppressMessages(multi_se <- SummarizedExperiment(assays = list(counts = simCount,
                                               logcounts = log1p(simCount))))
suppressMessages(multi_cc <- ClusterExperiment(multi_se, trueCluster))
suppressMessages(ccTrue <- ClusterExperiment(simCount, trueCluster))
suppressMessages(ccTrue2 <- ClusterExperiment(simCount, trueCluster, transformation = log1p))
seedValue<-495 #01875 works for sample.kind="Rejection"

test_that("clusterSingle works with non defalt assays", {

    ###Apply to SE
    expect_silent(cl1 <- clusterSingle(multi_se, 
      whichAssay = "counts",
      mainClusterArgs=list(clusterArgs=list(k=3),
          clusterFunction="pam"), 
      subsample=FALSE, sequential=FALSE))

    ###Apply to CE
    expect_silent(cl2 <- clusterSingle(multi_cc, 
        whichAssay = "counts",
        mainClusterArgs=list(clusterArgs=list(k=3), 
            clusterFunction="pam"), 
        subsample=FALSE, 
        sequential=FALSE)
    )

    expect_equal(primaryCluster(cl1), primaryCluster(cl2))


    expect_silent(cl3 <- clusterSingle(multi_cc, 
        whichAssay = "logcounts", 
        mainClusterArgs=list(clusterArgs=list(k=3),
            clusterFunction="pam"),
        subsample=FALSE, sequential=FALSE))

    expect_false(all(primaryCluster(cl2) == primaryCluster(cl3)))

    expect_error(clusterSingle(multi_cc, whichAssay = "normalized",
        mainClusterArgs=list(clusterArgs=list(k=3),
            clusterFunction="pam"),
        subsample=FALSE, sequential=FALSE),
        "'normalized' not in names")

    expect_error(clusterSingle(multi_cc, whichAssay = 3,
        mainClusterArgs=list(clusterArgs=list(k=3),
            clusterFunction="pam"),
        subsample=FALSE, sequential=FALSE),
        "subscript is out of bounds")

})

test_that("clusterMany works with non default assays", {
  expect_silent(cl1 <- clusterMany(multi_cc, ks=c(3,4),clusterFunction="pam",
                                   subsample=FALSE, sequential=FALSE,
                                   whichAssay = "counts",verbose=FALSE))

  expect_silent(cl2 <- clusterMany(multi_se, ks=c(3,4),clusterFunction="pam",
                                   subsample=FALSE, sequential=FALSE,
                                   whichAssay = "counts",verbose=FALSE))

  expect_equal(primaryCluster(cl1), primaryCluster(cl2))


  expect_silent(cl3 <- clusterMany(multi_cc, ks=c(3,4),clusterFunction="pam",
                                   subsample=FALSE, sequential=FALSE,
                                   whichAssay = "logcounts",verbose=FALSE))

  expect_false(all(primaryCluster(cl1) == primaryCluster(cl3)))
})

test_that("mergeClusters works with non default assays", {

  expect_silent(cl2 <- clusterSingle(multi_cc, whichAssay = "counts",
                                     mainClusterArgs=list(clusterArgs=list(k=3),clusterFunction="pam"),
                                     subsample=FALSE, sequential=FALSE))

  expect_silent(cl3 <- clusterSingle(multi_cc, whichAssay = "logcounts",
                                     mainClusterArgs=list(clusterArgs=list(k=3),clusterFunction="pam"),
                                     subsample=FALSE, sequential=FALSE))

  expect_silent(cl3 <- makeDendrogram(cl3, whichAssay = "logcounts"))
  expect_message(merged <- mergeClusters(x=cl3, whichAssay = "logcounts",DEMethod="limma"),
                 "Merging will be done on")

})

test_that("plotting works wih non default assays", {
  expect_silent(plotFeatureBoxplot(object=multi_cc,feature=1,whichAssay=1))
  expect_silent(plotFeatureBoxplot(object=multi_cc,feature=1,whichAssay=2))

  suppressMessages(small_cc <- multi_cc[1:10,c(1:3, 290:293)])
  suppressMessages(small_cc <- makeDendrogram(small_cc))
  expect_silent(plotHeatmap(small_cc, whichAssay=1))
  expect_silent(plotHeatmap(small_cc, whichAssay=2))

})

test_that("`mainClustering` options", {

    ########
    #Check mainClustering
    ########
    ###Check pam exactly same:
    expect_silent(x<-mainClustering(mat, 
        inputType="X",
        clusterFunction="pam",
        clusterArgs=list(k=3),
        minSize=1, removeSil=FALSE))
    expect_equal(length(x),ncol(mat))
    x2<-cluster::pam(t(mat),k=3,cluster.only=TRUE)
    expect_equal(x,x2)
    ###Check hierarchicalK exactly same:
    expect_silent(x<-mainClustering(dissMat,
        inputType="diss", 
        clusterFunction="hierarchicalK",
        clusterArgs=list(k=3),
        minSize=1, 
        removeSil=FALSE))
    expect_equal(length(x),ncol(mat))
    x2<-stats::cutree(stats::hclust(as.dist(dissMat)),k=3)
    expect_equal(x,x2)


    #check giving wrong parameters gives warning:
    expect_warning(mainClustering(dissMat, 
        inputType="diss",
        clusterFunction="tight", 
        clusterArgs=list(alpha=0.1),
        minSize=5, removeSil=TRUE),
        "do not match the algorithmType")
    expect_warning(mainClustering(dissMat, inputType="diss",
        clusterFunction="tight", 
        clusterArgs=list(alpha=0.1,k=3),
        minSize=5, 
        removeSil=TRUE),
        "do not match the algorithmType")
    #check missing required parameters gives warning:
    expect_error(mainClustering(dissMat,
        inputType="diss", 
        clusterFunction="tight",
        minSize=5),
        "must supply arguments: alpha")
    expect_error(mainClustering(mat, 
        inputType="X",
        clusterFunction="pam", 
        clusterArgs=list(alpha=0.1),
        minSize=5, removeSil=TRUE),
        "must supply arguments: k")
    #check warning for superfluous arguments passed to clustering function:
    expect_warning(mainClustering(dissMat, 
        inputType="diss",
        clusterFunction="tight", 
        clusterArgs=list(k=3,alpha=0.1),
    	minSize=5),
        "arguments passed via clusterArgs to the clustering function tight are not all applicable")
    expect_warning(mainClustering(dissMat, 
        inputType="diss",
        clusterFunction="pam", 
        clusterArgs=list(k=3,alpha=0.1),
        minSize=5,
        removeSil=TRUE),
        "arguments passed via clusterArgs to the clustering function pam are not all applicable")
    expect_warning(mainClustering(dissMat, 
        inputType="diss",
        clusterFunction="tight", 
        clusterArgs=list(alpha=0.1, evalClusterMethod="average")),
        "arguments passed via clusterArgs to the clustering function tight are not all applicable")
    expect_warning(mainClustering(dissMat, 
        inputType="diss",
        clusterFunction="hierarchical01", 
        clusterArgs=list(alpha=0.1, minSize.core=4)),
        "arguments passed via clusterArgs to the clustering function hclust are not all applicable")

    #test default 01 distance
    expect_silent(mainClustering(dissMat, inputType="diss",
        clusterFunction="tight", 
        clusterArgs=list(alpha=0.1)))
    #test default K distance
    expect_silent(mainClustering(dissMat, inputType="diss",
        clusterFunction="hierarchicalK", 
        clusterArgs=list(k=3)))

})


test_that("`mainClustering` post-processing", {
    #pass diss matrix
    expect_silent(mainClustering(mat, 
        inputType="X",
        clusterFunction="pam", 
        clusterArgs=list(k=3),
        minSize=5,
        removeSil=TRUE,diss=dissMat))
     
})

test_that("`mainClustering` works with cat", {
    #test default 01 distance
    expect_silent(mainClustering(catMat, inputType="cat",
        clusterFunction="hierarchical01", 
        clusterArgs=list(alpha=0.1)))
    
})

context("mergeClusters")


test_that("`mergeClusters` works with matrix",{
  expect_silent(cl1 <- clusterSingle(smSimData, 
        subsample=FALSE, sequential=FALSE,
        mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=6)), 
				isCount=FALSE))
	expect_silent(clustWithDendro <- makeDendrogram(cl1))
  #matrix version
  expect_silent(mergedList <- mergeClusters(x=transformData(cl1), DEMethod="limma", cl=primaryCluster(cl1), dendro=clustWithDendro@dendro_clusters, mergeMethod="adjP", plotInfo="mergeMethod"))
  
})

test_that("`mergeClusters` works with ClusterExperiment objects", { 
    expect_silent(cl1 <- clusterSingle(smSimData, 
                         subsample=FALSE, sequential=FALSE,
                         mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=6)), 
  											 isCount=FALSE))
    leg<-clusterLegend(cl1)[[primaryClusterIndex(cl1)]]
    leg[,"name"]<-letters[1:6]
    clusterLegend(cl1)[[primaryClusterIndex(cl1)]]<-leg
    expect_silent(clustWithDendro <- makeDendrogram(cl1))
	#check plotting types:
  expect_message(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma",
	 mergeMethod="none",plotInfo="all"),"Note: Merging will be done on")
  expect_message(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma", 
		mergeMethod="none", plotInfo="adjP"),"Note: Merging will be done on")
  expect_message(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma", 
		mergeMethod="none", plotInfo="locfdr"),"Note: Merging will be done on")
  expect_warning(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma",
		 mergeMethod="none", plotInfo="locfdr",showWarnings=TRUE))
  expect_error(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma",
	 mergeMethod="none", plotInfo="mergeMethod"),
	 "can only plot 'mergeMethod' results if one method is selected")
  expect_message(clustMerged <- mergeClusters(clustWithDendro, mergeMethod="adjP", 
		DEMethod="limma", plotInfo="none"),"Note: Merging will be done on")

  #check all methods run
  for(method in clusterExperiment:::.availMergeMethods){
	  expect_message(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma", 
			mergeMethod=method, plotInfo="mergeMethod"),"Note: Merging will be done on")
  }
  
  expect_true("mergeClusters" %in% clusterTypes(clustMerged))
  expect_true("mergeClusters" %in% colnames(clusterMatrix(clustMerged)))

 expect_message(clustMerged <- mergeClusters(clustWithDendro, mergeMethod="adjP", DEMethod="limma",
		 plotInfo="mergeMethod",leafType="samples",plotType="colorblock"),"Note: Merging will be done on")
	expect_message(clustMerged <- mergeClusters(clustWithDendro, mergeMethod="adjP", DEMethod="limma", plotInfo="mergeMethod",leafType="samples",plotType="name"),"Note: Merging will be done on")
	expect_message(clustMerged <- mergeClusters(clustWithDendro, mergeMethod="adjP", DEMethod="limma", plotInfo="mergeMethod",leafType="clusters",plotType="colorblock"),"Note: Merging will be done on")
	expect_message(clustMerged <- mergeClusters(clustWithDendro, mergeMethod="adjP", DEMethod="limma", plotInfo="mergeMethod",leafType="clusters",plotType="name"),"Note: Merging will be done on")

  expect_error(mergeClusters(x=transformData(clustWithDendro), DEMethod="limma",
                               cl=primaryCluster(clustWithDendro),plot="none",
                               mergeMethod="adjP",
                               dendro=clustWithDendro@dendro_samples),
                 "Not a valid input dendrogram")

	expect_error(mergeClusters(clustWithDendro, mergeMethod="adjP"),'argument "DEMethod" is missing, with no default')


  #test when merge already exists
  expect_message(clustMerged <- mergeClusters(clustWithDendro, mergeMethod="adjP", DEMethod="limma"),"Merging will be done on")
  expect_silent(primaryClusterIndex(clustMerged)<-2)
  expect_silent(clustMerged<- makeDendrogram(clustMerged))
  expect_message(clustMerged2<-mergeClusters(clustMerged,mergeMethod="adjP"),"Merging will be done on")
  expect_true("mergeClusters.1" %in% clusterTypes(clustMerged2))
  expect_true(!"makeConsensus.1" %in% clusterTypes(clustMerged2))
  expect_true(!"clusterMany.1" %in% clusterTypes(clustMerged2))
  expect_silent(removeClusterings(clustMerged, whichClusters = "mergeClusters"))
})


test_that("`mergeClusters` works with HDF5 assay slot",{
    expect_silent(cl1 <- clusterSingle(hdfObj, 
            subsample=FALSE, sequential=FALSE,
			mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=6)),
			isCount=FALSE))
    expect_silent(clustWithDendro <- makeDendrogram(cl1))
    expect_message(mergedCE <- mergeClusters(x=clustWithDendro,plot=FALSE,
            mergeMethod="adjP", DEMethod="limma", plotInfo="mergeMethod"),
			"Merging will be done on")
	expect_true(inherits(assay(mergedCE),"DelayedArray"))
								
	
})

test_that("saving merge info works",{
  expect_silent(cl1 <- clusterSingle(smSimData, 
       subsample=FALSE, sequential=FALSE, reduceMethod="none",
	   mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=6)),
	   isCount=FALSE))
	   #check cluster is same, otherwise won't get same results...
  expect_equal(primaryCluster(cl1),c(4,1,1,5,1,2,5,1,1,3,3,3,6,6,4,2,2,2))
  #for some reason, clusterSingle not giving reasonable names
  expect_silent(clustWithDendro <- makeDendrogram(cl1,reduceMethod="none"))
  #Dendrogram:
  # --[dendrogram w/ 2 branches and 6 members at h = 1509]
  # |--leaf "3" 
  # `--[dendrogram w/ 2 branches and 5 members at h = 1455]
  # |--[dendrogram w/ 2 branches and 2 members at h = 228]
  # |  |--leaf "1" 
  # |  `--leaf "5" 
  # `--[dendrogram w/ 2 branches and 3 members at h = 337]
  # |--leaf "6" 
  # `--[dendrogram w/ 2 branches and 2 members at h = 259]
  # |--leaf "2" 
  # `--leaf "4"
  #matrix version
  expect_silent(mergedList <- mergeClusters(x=transformData(cl1), DEMethod="limma",
                              cl=primaryCluster(cl1),
                              dendro=clustWithDendro@dendro_clusters,
                              mergeMethod="adjP", plotInfo="none"))
  
  ##check giving nodePropTable
  expect_message(mergedList2<- mergeClusters(x=transformData(cl1), DEMethod="limma",
                              cl=primaryCluster(cl1), nodePropTable=mergedList$nodeProp,
                              dendro=clustWithDendro@dendro_clusters,
                              mergeMethod="Storey", plotInfo="none"),
															"Using existing results of per-node significance")
  expect_equal(mergedList2$nodeProp[,"adjP"],mergedList$nodeProp[,"adjP"])
  expect_silent(mergedListStorey<-mergeClusters(x=transformData(cl1), DEMethod="limma",
                                  cl=primaryCluster(cl1), 
                                  dendro=clustWithDendro@dendro_clusters,
                                  mergeMethod="Storey", plotInfo="mergeMethod"))
  expect_equal(mergedList2$nodeProp[,"Storey"],mergedListStorey$nodeProp[,"Storey"])  
  expect_message(mergedList2Redo<- mergeClusters(x=transformData(cl1), DEMethod="limma",
                              cl=primaryCluster(cl1), nodePropTable=mergedList2$nodeProp,
                              dendro=clustWithDendro@dendro_clusters,
                              mergeMethod="none", plotInfo=c("all")),"Using existing results of per-node significance")
  
  #------
  #on ClusterExperiment
  # note need to specify message to see doing it correctly.
  #------
  expect_message(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma",
	   mergeMethod="adjP",plotInfo="none",plot=FALSE,calculateAll=FALSE),"Merging will be done on")
  expect_equal(clustMerged@merge_dendrocluster_index,clustWithDendro@dendro_index+1)
  expect_equal(clustMerged@merge_dendrocluster_index,clustMerged@dendro_index)
  expect_equal(clustMerged@merge_index,1)
  
  #add to existing with different method 
  expect_message(clustMerged2 <- mergeClusters(clustMerged, mergeMethod="Storey",
  	plotInfo="none",cutoff=0.1,plot=FALSE,calculateAll=FALSE),"Merging will be done on")
  expect_equal(clustMerged2@merge_dendrocluster_index,clustMerged@dendro_index+1)
  expect_equal(clustMerged2@merge_dendrocluster_index,clustMerged2@dendro_index)
  expect_equal(clustMerged2@merge_index,1)
  #rerun previous with different cutoff -- no new calculations
  expect_message(clustMerged3 <- mergeClusters(clustMerged2,mergeMethod="Storey",
  	plotInfo="none",cutoff=0.05,plot=FALSE,calculateAll=FALSE),"Using existing results of per-node significance")
  expect_equal(clustMerged3@merge_dendrocluster_index,clustMerged2@dendro_index+1)
  expect_equal(clustMerged3@merge_dendrocluster_index,clustMerged3@dendro_index)
  expect_equal(clustMerged3@merge_index,1)
  #do it again with higher value that actually merges -- check correct index updates, etc...
  expect_message(clustMerged4 <- mergeClusters(clustMerged3,mergeMethod="Storey",
  	plotInfo="none",cutoff=0.5,plot=FALSE,calculateAll=FALSE),"Using existing results of per-node significance")
  expect_equal(clustMerged4@merge_dendrocluster_index,clustMerged3@dendro_index+1)
  expect_equal(clustMerged4@merge_dendrocluster_index,clustMerged4@dendro_index)
  expect_equal(clustMerged4@merge_index,1)
  expect_equal(clustMerged4@merge_nodeMerge[,"mergeClusterId"],c(NA,NA,1,3,NA))
  expect_equal(clustMerged4@merge_nodeMerge[,"isMerged"],c(FALSE,FALSE,TRUE,TRUE,TRUE))
  # > clustMerged4@merge_nodeMerge
 #         Node               Contrast isMerged mergeClusterId
 #  Node1 Node1  X3-(X1+X5+X6+X2+X4)/5    FALSE             NA
 #  Node2 Node2 (X1+X5)/2-(X6+X2+X4)/3    FALSE             NA
 #  Node3 Node3                  X1-X5     TRUE              1
 #  Node4 Node4           X6-(X2+X4)/2     TRUE              3
 #  Node5 Node5                  X2-X4     TRUE             NA
 # > clustMerged4@merge_nodeProp
 #        Node               Contrast Storey PC adjP locfdr MB JC
 # Node1 Node1  X3-(X1+X5+X6+X2+X4)/5    1.0 NA 0.55     NA NA NA
 # Node2 Node2 (X1+X5)/2-(X6+X2+X4)/3    1.0 NA 0.75     NA NA NA
 # Node3 Node3                  X1-X5    0.0 NA 0.10     NA NA NA
 # Node4 Node4           X6-(X2+X4)/2    0.1 NA 0.15     NA NA NA
 # Node5 Node5                  X2-X4    0.4 NA 0.00     NA NA NA

 # > clustMerged4@merge_nodeMerge
#                             NodeId               Contrast isMerged mergeClusterId
#  InternalNodeId7   InternalNodeId7  X3-(X1+X5+X6+X2+X4)/5    FALSE             NA
#  InternalNodeId8   InternalNodeId8 (X1+X5)/2-(X6+X2+X4)/3    FALSE             NA
#  InternalNodeId9   InternalNodeId9                  X1-X5     TRUE              1
#  InternalNodeId10 InternalNodeId10           X6-(X2+X4)/2     TRUE              3
#  InternalNodeId11 InternalNodeId11                  X2-X4     TRUE             NA
#

  #check really gets clusterIds and not names
  expect_silent(clusterLegend(clustMerged3)[["clusterSingle"]][,"name"]<-letters[1:6])
  expect_message(clustMerged5 <- mergeClusters(clustMerged3,  mergeMethod="Storey",
  	plotInfo="none",cutoff=0.5,plot=FALSE,calculateAll=FALSE),"Using existing results of per-node significance")
  expect_equal(clustMerged4@merge_nodeMerge,clustMerged5@merge_nodeMerge)
  #helpful for debugging:
  # plotDendrogram(clustMerged,show.node=TRUE,show.tip.label=TRUE)
  # table(clusterMatrix(clustMerged)[,c(1)],clusterMatrix(clustMerged)[,c(2)])
  
  expect_silent(nodeMergeInfo(clustMerged4))
  expect_equal(mergeClusterIndex(clustMerged4),clustMerged4@merge_index)
  expect_equal(mergeCutoff(clustMerged4),0.5)
  expect_equal(mergeMethod(clustMerged4),"Storey")
  
  #check if can calculate all, but do nothing else
  expect_message(clustMergedAll<-mergeClusters(clustWithDendro, DEMethod="limma", 
		mergeMethod="none",plotInfo="none",cutoff=0.5,plot=FALSE,
		calculateAll=TRUE),"Merging will be done on")
  expect_false(is.na(clustMergedAll@merge_dendrocluster_index))
  expect_true(is.na(clustMergedAll@merge_index))
  expect_false(is.null(nodeMergeInfo(clustMergedAll)))
  
  #should erase merge info if call dendrogram
  expect_silent(clustMergedErase<-makeDendrogram(clustMerged5))
  expect_true(is.na(clustMergedErase@merge_index))
  expect_true(is.na(clustMergedErase@merge_dendrocluster_index))
  
  #should erase merge info if call dendrogram
  expect_silent(clustMergedErase2<-makeDendrogram(clustMergedAll))
  expect_true(is.na(clustMergedErase2@merge_index))
  expect_true(is.na(clustMergedErase2@merge_dendrocluster_index))

  #test getMergeCorresp
  #node clustMerge
  mgCl<-clusterMatrix(clustMerged)[,clustMerged@merge_index]
  ogCl<-clusterMatrix(clustMerged)[,clustMerged@merge_dendrocluster_index]
  expect_silent(mc<-getMergeCorrespond(clustMerged))
  expect_equal(length(mc),length(unique(mgCl[mgCl>0])))
  expect_silent(mc<-getMergeCorrespond(clustMerged,by="original"))
  expect_equal(length(mc),length(unique(ogCl[ogCl>0])))
  
  expect_error(getMergeCorrespond(clustMergedAll),"there is no merge clustering in this object")
  #check saves even if just plotInfo
  expect_message(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma",
		 mergeMethod="none",plotInfo="all"),"Merging will be done on")
  expect_false(is.null(clustMerged@merge_nodeProp))
})


test_that("logFC works",{
  expect_silent(cl1 <- clusterSingle(smSimData, 
       subsample=FALSE, sequential=FALSE, reduceMethod="none",
	   mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=6)),
	   isCount=FALSE))
	   #check cluster is same, otherwise won't get same results...
  expect_silent(clustWithDendro <- makeDendrogram(cl1,reduceMethod="none"))
  #--------
  #matrix version 
  #--------
  #only FC
  expect_silent(mergedList <- mergeClusters(x=transformData(cl1), DEMethod="limma",
                              cl=primaryCluster(cl1),
                              dendro=clustWithDendro@dendro_clusters,
                              mergeMethod="adjP", cutoff=0.1, logFCcutoff=1,calculateAll=FALSE,plotInfo="none",plot=FALSE))
  expect_true(all(c(clusterExperiment:::.availMergeMethods,"adjP_1.0") %in% colnames(mergedList$nodeProp)))

  #calculate All 
  expect_silent(mergedListAll <- mergeClusters(x=transformData(cl1), DEMethod="limma",
                              cl=primaryCluster(cl1),
                              dendro=clustWithDendro@dendro_clusters,
                              mergeMethod="adjP", cutoff=0.1, logFCcutoff=1,calculateAll=TRUE,plotInfo="none",plot=FALSE))
  expect_true(all(c(clusterExperiment:::.availMergeMethods,"adjP_1.0") %in% colnames(mergedListAll$nodeProp)))
  
  #check calculate all does FC it even if not given as the mergeMethod
  # (try both "none" and "Storey")
  expect_silent(mergedListAll2 <- mergeClusters(x=transformData(cl1), DEMethod="limma",
                              cl=primaryCluster(cl1),
                              dendro=clustWithDendro@dendro_clusters,
                              mergeMethod="none",
							   logFCcutoff=1,plotInfo="none",plot=FALSE))
  expect_true(all(c(clusterExperiment:::.availMergeMethods,"adjP_1.0") %in% colnames(mergedListAll2$nodeProp)))
  expect_silent(mergedListAll2 <- mergeClusters(x=transformData(cl1), DEMethod="limma",
                              cl=primaryCluster(cl1),
                              dendro=clustWithDendro@dendro_clusters,
                              mergeMethod="Storey",
							   logFCcutoff=1,plotInfo="none",plot=FALSE))
  expect_true(all(c(clusterExperiment:::.availMergeMethods,"adjP_1.0") %in% colnames(mergedListAll2$nodeProp)))
  
  ##check giving nodePropTable but need new calculation ("Storey")
  expect_silent(mergedList2<- mergeClusters(x=transformData(cl1), DEMethod="limma",
                              cl=primaryCluster(cl1), nodePropTable=mergedList$nodeProp,
                              dendro=clustWithDendro@dendro_clusters,
                              mergeMethod="Storey", plotInfo="none",plot=FALSE))
  expect_equal(mergedList2$nodeProp[,"adjP_1.0"],mergedList$nodeProp[,"adjP_1.0"])

  ##check giving nodePropTable but need new calculation (newFC)
  ##Also check that actually gives different merging (with logFC10 should)
  expect_silent(mergedList2<- mergeClusters(x=transformData(cl1), DEMethod="limma",
                              cl=primaryCluster(cl1), nodePropTable=mergedList$nodeProp,
                              dendro=clustWithDendro@dendro_clusters, cutoff=0.1,
                              mergeMethod="adj", logFCcutoff=10,
							  plotInfo="none",plot=FALSE))
  expect_equal(mergedList2$nodeProp[,"adjP_1.0"],mergedList$nodeProp[,"adjP_1.0"])
  expect_true("adjP_10.0" %in% names(mergedList2$nodeProp))
  expect_false(all(mergedList2$nodeProp[,"adjP_10.0"] == mergedList$nodeProp[,"adjP"]))
  expect_false(all(mergedList2$nodeMerge[,"isMerged"] == mergedList$nodeMerge[,"isMerged"]))
  expect_equal(length(unique(mergedList2$clustering)),3)
  expect_equal(length(unique(mergedList$clustering)),5)

  ##check giving nodePropTable but DON'T need new calculation ("Storey")
  expect_message(mergedList3<- mergeClusters(x=transformData(cl1), DEMethod="limma",
                              cl=primaryCluster(cl1), nodePropTable=mergedListAll$nodeProp,
                              dendro=clustWithDendro@dendro_clusters,
                              mergeMethod="Storey", plotInfo="none",plot=FALSE),"Using existing results of per-node significance")
  expect_equal(mergedList2$nodeProp[,"adjP_1.0"],mergedList$nodeProp[,"adjP_1.0"])

  ##check giving nodePropTable but DON'T need new calculation (logFC)
  expect_message(mergedList2<- mergeClusters(x=transformData(cl1), DEMethod="limma",
         cl=primaryCluster(cl1), nodePropTable=mergedListAll$nodeProp,
         dendro=clustWithDendro@dendro_clusters,
         mergeMethod="adjP", logFCcutoff=1,
				plotInfo="none",plot=FALSE),"Using existing results of per-node significance")
  expect_equal(mergedList2$nodeProp[,"adjP_1.0"],mergedList$nodeProp[,"adjP_1.0"])
  
  ###Test plotting on matrix version:
  expect_silent(mergeClusters(x=transformData(cl1), DEMethod="limma",
                              cl=primaryCluster(cl1), 
                              dendro=clustWithDendro@dendro_clusters,
                              mergeMethod="adjP", logFCcutoff=10, plotInfo="adjP_10.0"))
  
  
  #----------
  #on ClusterExperiment
  #----------
  expect_message(clustMerged <- mergeClusters(clustWithDendro,
	   mergeMethod="adjP", plotInfo="none", plot=FALSE, DEMethod="limma", 
	   calculateAll=FALSE,cutoff=0.1,logFCcutoff=1),"Merging will be done on")
  expect_true(all(c("adjP_1.0") %in% colnames(clustMerged@merge_nodeProp)))
  
  #add to existing with different method
  expect_message(clustMerged2 <- mergeClusters(clustMerged, mergeMethod="Storey",
  	plotInfo="none",cutoff=0.1,plot=FALSE,calculateAll=FALSE),"Merging will be done on")
  expect_true(all(c("adjP_1.0") %in% colnames(clustMerged2@merge_nodeProp)))
  
  #rerun previous with different cutoff but same logFC cutoff -- no new calculations needed
  expect_message(clustMerged3 <- mergeClusters(clustMerged2, 
	 mergeMethod="adjP", logFCcutoff=1,plotInfo="none",cutoff=0.05,
	 plot=FALSE,calculateAll=FALSE),"Using existing results of per-node significance")
  expect_true(all(c("adjP_1.0") %in% colnames(clustMerged3@merge_nodeProp)))

  #redo with different logFC --- pick logFC large enought that changes the proportions to make sure actually merge on right value
  expect_message(clustMerged4 <- mergeClusters(clustMerged, 
	 	mergeMethod="adjP",  logFCcutoff=10,plotInfo="none",cutoff=0.1,plot=FALSE,
		calculateAll=FALSE,clusterLabel="merge, logFC"),
                 "Merging will be done on")
  expect_false(all(clustMerged4@merge_nodeProp[,"adjP_10.0"] == clustMerged4@merge_nodeProp[,"adjP"]))
  expect_false(all(clustMerged4@merge_nodeMerge[,"isMerged"] == clustMerged@merge_nodeMerge[,"isMerged"]))
  expect_equal(length(unique(primaryCluster(clustMerged4))),3)
  expect_equal(length(unique(primaryCluster(clustMerged))),5)

  ##Check labels not getting corrupted when redo
  expect_equal(primaryClusterLabel(clustMerged4),"merge, logFC")
  expect_message(clustMerged5 <- mergeClusters(clustMerged4, 
	 	mergeMethod="adjP", logFCcutoff=10,plotInfo="none",
   	cutoff=0.1,plot=FALSE,calculateAll=FALSE,
		clusterLabel="merge, logFC"),"Merging will be done on")
  expect_equal(primaryClusterLabel(clustMerged5),"merge, logFC")
  
  
  ###Test plotting on CE version:
  expect_message(clustMerged10 <- mergeClusters(clustWithDendro, DEMethod="limma",
	   mergeMethod="adjP", plot=TRUE,plotInfo="adjP_10.0", 
	   calculateAll=FALSE,cutoff=0.1,logFCcutoff=10),"Merging will be done on")

  expect_silent(plotDendrogram(clustMerged10,mergeInfo="adjP_10.0"))
})


test_that("`mergeClusters` preserves the colData and rowData of SE", {

  expect_silent(cl <- clusterSingle(smSimSE, 
       subsample=FALSE, sequential=FALSE,                      
	   mainClusterArgs=list(clusterFunction="pam",clusterArgs=list(k=6)),
	   isCount=FALSE))
  expect_silent(clD <- makeDendrogram(cl))
  expect_message(cl <- mergeClusters(clD, DEMethod="limma", mergeMethod = "adjP"))
  expect_equal(colData(cl),colData(smSimSE))
  expect_equal(rownames(cl),rownames(smSimSE))
  expect_equal(colnames(cl),colnames(smSimSE))
  expect_equal(metadata(cl),metadata(smSimSE))
  expect_equal(rowData(cl),rowData(clD))

})


test_that("`mergeClusters` works with unassignedSamples", {

  expect_silent(clustWithDendro <- makeDendrogram(ceSim,unassignedSamples = c("outgroup")))

	expect_message(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma",
	 mergeMethod="adjP",
		 plotInfo="mergeMethod",leafType="samples",plotType="colorblock"))
	expect_message(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma",
	 mergeMethod="adjP",
		 plotInfo="mergeMethod",leafType="samples",plotType="name"))
	expect_message(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma",
	 mergeMethod="adjP",
		 plotInfo="mergeMethod",leafType="clusters",plotType="colorblock"))
	expect_message(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma",
	 mergeMethod="adjP", plotInfo="mergeMethod",leafType="clusters",plotType="name"))

  expect_silent(clustWithDendro <- makeDendrogram(ceSim,reduceMethod="mad",unassignedSamples = c("cluster")))

	expect_warning(mergeClusters(clustWithDendro, mergeMethod="adjP", DEMethod="limma", plotInfo="mergeMethod",leafType="samples",plotType="colorblock"),"You cannot set 'leafType' to 'samples' in plotting mergeClusters unless the dendrogram was made with unassigned/missing")
	expect_warning(mergeClusters(clustWithDendro, mergeMethod="adjP", DEMethod="limma", plotInfo="mergeMethod",leafType="samples",plotType="name"),"You cannot set 'leafType' to 'samples' in plotting mergeClusters unless the dendrogram was made with unassigned/missing")
	expect_message(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma",
	 mergeMethod="adjP", plotInfo="mergeMethod",leafType="clusters",plotType="colorblock",
	 "You cannot set 'leafType' to 'samples' in plotting mergeClusters unless the dendrogram was made with unassigned/missing"))
	expect_message(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma",
	 mergeMethod="adjP", plotInfo="mergeMethod",leafType="clusters",plotType="name","You cannot set 'leafType' to 'samples' in plotting mergeClusters unless the dendrogram was made with unassigned/missing"))


})

test_that("cluster labels not being internally changed from user input",{
  expect_silent(clustWithDendro <- makeDendrogram(ceSim,unassignedSamples = c("outgroup")))
  
  expect_message(clustMerged <- mergeClusters(clustWithDendro, DEMethod="limma", mergeMethod="adjP",
                                              plotInfo="mergeMethod",leafType="samples",plotType="colorblock"))
  
  
})

context("RSEC")
seedValue<-495 #01875 works for sample.kind="Rejection"
test_that("`RSEC` works with matrix, ClusterExperiment, summarizedExperiment",{
	## these examples don't do dendrogram/merge because all -1 after makeConsensus
	## only tests clusterMany, makeConsensus parts.
	## so can't do expect_silent, because returns NOTE about that issue.
	expect_message(rsecOut1<-RSEC(x=mat, isCount=FALSE,reduceMethod="none",k0s=4:5,
		clusterFunction="tight", alphas=0.1,dendroReduce="none",
        subsampleArgs=list(resamp.num=5),random.seed=seedValue
  	 	),"makeDendrogram encountered following error")
   expect_message(rsecOut2<-RSEC(x=cc, reduceMethod="none",k0s=4:5,
   		clusterFunction="tight", alphas=0.1,dendroReduce="none",
       subsampleArgs=list(resamp.num=5),random.seed=seedValue
  	 	),"makeDendrogram encountered following error")
  expect_message(rsecOut3<-RSEC(x=ccSE,
	  reduceMethod="none",k0s=4:5,clusterFunction="tight", 
	  alphas=0.1,dendroReduce="none",
       subsampleArgs=list(resamp.num=5),random.seed=seedValue),
	   "makeDendrogram encountered following error")
   expect_message(rsecOut4<-RSEC(x=se,isCount=FALSE,reduceMethod="none",
   		k0s=4:5,clusterFunction="tight", alphas=0.1,dendroReduce="none",
       subsampleArgs=list(resamp.num=5),random.seed=seedValue),
	   "makeDendrogram encountered following error")
	   #test rerunClusterMany argument:
	expect_message(rsecOut5<-RSEC(rsecOut2,reduceMethod="none",
		k0s=4:5,clusterFunction="tight", alphas=0.1,dendroReduce="none",rerunClusterMany=TRUE,
		subsampleArgs=list(resamp.num=5),random.seed=seedValue),
		"makeDendrogram encountered following error")
    #makes dendrogram so important have here so has to catch defaults of RSEC...
	expect_message(rsecOut6<-RSEC(rsecOut2,
			reduceMethod="none",k0s=4:5,clusterFunction="tight", 
			alphas=0.1,dendroReduce="none",rerunClusterMany=FALSE,
			subsampleArgs=list(resamp.num=5),random.seed=seedValue),
			"makeDendrogram encountered following error")
})



test_that("`RSEC` works with no merging",{
	# These are slow!
	skip_on_os("windows")
	skip_on_os("mac")
  
	#do the same, only don't do merging:
    expect_message(rsecOut<-RSEC(x=assay(seSimCount), 
        isCount=TRUE,reduceMethod="none",
        k0s=4:5,clusterFunction="hierarchical01", alphas=0.1,
        betas=0.9,dendroReduce="none",minSizes=1,
        seqArgs=list(top.can=0),
        subsampleArgs=list(resamp.num=5),random.seed=seedValue,
        mergeMethod="none"),
        "clusters will not be merged because argument")
})

test_that("`RSEC` returns clusterMany even when errors later",{
	# These are slow!
	skip_on_os("windows")
	skip_on_os("mac")
	
	#error in makeConsensus param
	expect_message(rsecOut1<-
        RSEC(x=mat, isCount=FALSE,k0s=4:5,
		clusterFunction="tight", alphas=0.1, nReducedDims=3,
        subsampleArgs=list(resamp.num=5),random.seed=seedValue, 
        consensusProportion = -1, consensusMinSize = 5),
        "Invalid value for the 'proportion' parameter"
  	 	)
	expect_true("clusterMany" %in% clusterTypes(rsecOut1))

	#error in dendro param
	expect_message(rsecOut2<-RSEC(x=mat, isCount=FALSE,k0s=4:5,
		clusterFunction="tight", alphas=0.1, nReducedDims=3,
        subsampleArgs=list(resamp.num=5),random.seed=seedValue, 
		dendroReduce="myfakemethod"
  	 	),"does not contain the given 'reduceMethod' value")
    expect_true(all(c("clusterMany","makeConsensus") 
        %in% clusterTypes(rsecOut2)))
	
	# error in merging -- give wrong method
    # have to get dataset where can actually make dendrogram... 
    # takes longer.
    # > sample(size=50,x=1:nrow(seSimCount))
#      [1]  67  47  34 124  23  40 104  93  18  26 123  77  64  35  90  58 103 153  62  66 138  57 118  50  60 145  52 134  45  78  12
#     [32]  38 121  63 128 111 117  56  43  51 149   9  73 150   6  39 125 133 127 152
    set.seed(4289)
	expect_message(rsecOut3<-RSEC(x=
        assay(seSimCount[sample(size=50,x=1:nrow(seSimCount)),]), 
        isCount=TRUE,reduceMethod="none",
        k0s=4:5,clusterFunction="pam", alphas=0.1,
        betas=0.9,dendroReduce="none",minSizes=1,
        subsampleArgs=list(resamp.num=5,clusterFunction="kmeans"),
        random.seed=seedValue,
		mergeMethod="fakeMerge"
  	 	),"mergeClusters encountered following error")

    expect_true(all(c("clusterMany","makeConsensus") %in% clusterTypes(rsecOut3)))

})


test_that("`RSEC` passing args to subsample",{
    #test in passing to subsampleArgs
  	expect_message(rsecOut1<-RSEC(x=mat, 
  				   isCount=FALSE,reduceMethod="none",k0s=4:5,
  	               clusterFunction="tight", alphas=0.1,dendroReduce="none",
  	               subsampleArgs=list(resamp.num=5),random.seed=495),
  				   "makeDendrogram encountered following error")
  	expect_message(rsecOut2<-RSEC(x=mat, 
  				   isCount=FALSE,reduceMethod="none",k0s=4:5,
  	               clusterFunction="tight", alphas=0.1,dendroReduce="none",
  	               subsampleArgs=list(resamp.num=5),random.seed=495),
  				   "makeDendrogram encountered following error")
  	expect_identical(clusterMatrix(rsecOut1),clusterMatrix(rsecOut2))
    
})

context("subsampleClustering")

test_that("subsampling", {

    set.seed(4897)
    expect_silent(subAll <- subsampleClustering(inputMatrix=mat, 
        inputType="X",clusterFunction="pam",
        clusterArgs=list(k=3), classifyMethod="All",
        resamp.num = 100,
        samp.p = 0.7,ncores=1))




	#subsample clusterings won't have identification to all samples...
    set.seed(4897)
    expect_silent(subInSample <- subsampleClustering(inputMatrix=mat,
            inputType="X",clusterArgs=list(k=3),
            clusterFunction="pam",
            classifyMethod=c("InSample"),
            resamp.num = 100,
            samp.p = 0.7,ncores=1))

	
    ## Windows does not support mclapply
    skip_on_os("windows")
	set.seed(4897)
	expect_silent(subAllParal <- 
	    subsampleClustering(inputMatrix=mat,
            inputType="X",clusterFunction="pam",
	        clusterArgs=list(k=3), classifyMethod="All",
	        resamp.num = 100,samp.p = 0.7,ncores=2)
	)

	expect_identical(subAllParal,subAll)

	

})

# Using KNN to solve NxN problem

## Background

Currently the results of subsampling (or consensus between clusters) is stored as a NxN matrix, with entries $s_{ij}$ being the proportion of times cell $i$ and $j$ are in the same cluster over the $B$ clusterings of which they both were assigned. 

*Note*: For subsampling, they may not be assigned because they were not both subsampled; for consensus they may not be assigned because one of them was a -1. 

Then `hier01` clusters this NxN matrix by taking entries $d_{ij}=1-s_{ij}$ and performing hierarchical clustering on the NxN distance matrix $D$. Afterward, we determine clusters from the hierarchical clustering by starting at the top of the tree and going down the tree until the resulting clustering satisfies the criteria that the cells in the cluster are sufficiently similar to each other, with a parameter $\alpha$ that controls the required similarity. There are two choices

1) $\forall i,j \in \mathcal{C}, s_{ij} \geq 1-\alpha$, i.e. the maximal allowable $d_{ij}$ for any $i,j$ in the cluster is $\alpha$. 

2) $\forall i \in \mathcal{C}, mean(s_{ij}) \geq 1-\alpha$ [default]

Instead of calculating and storing the $NxN$ matrix, we would like to store the $NxB$ matrix of the cluster assignments. 

Davide proposed using KNN functionality to determine the neighbors of a cell $i$ whose distance to other cells is such that $d_{ij}<\alpha$. Then we would need to figure out how to get a cluster assignment out of that.  

## Proposal

1) We use `findNeighbors` from the `BiocNeighbors` package using `threshold=alpha`. 

**To Do** This function only works with Manhattan distance and Euclidean distance. We would need to see if the algorithm is even valid for our arbitrary distance (basically a Hamming distance) -- Aaron would probably know. Then we would need get Aaron to allow us to do a pull request and write C code for our distance function to be added to `BiocNeighbors`. Luckily it has already been updated to allow for two, so adding a third would hopefully not be a problem.

2) Convert these results into an object for `igraph`. 

**To Do** Not clear that there is an existing function to do this. Might be relevant add on to `BiocNeighbors`. More generally if we create a wrapper package for the clustering methods that we were talking about, I think interface between igraph and `BiocNeighbors` may be needed, but I haven't looked into this. 

3) Determine the clusters. There are several logical choices, only some of which would not require any further parameters to be passed.

 a) Take all maximally connected components with the function `components` in `igraph`. This is a far looser requirement than any we have now.
  
  b) Take all maximal cliques (i.e. fully connected components) with the function `igraph_maximal_cliques`. This would basically correspond to the maximal option we currently have
  
  c) From each maximally connected component iteratively prune off vertices, based on the criteria that the remaining cluster must have all vertices with degree $< |V|/2$, where $|V|$ is the size of the cluster -- i.e. require them to be connected to at least half the other vertices in the cluster. This is similar to the mean criteria above, if we had considered the median instead of the mean. (It would have to be iterative since removing a node will change the degree of all remaining nodes). Could use some varient of this without iteration, where we look at degree distribution per connected components and pick some value for degree edge; perhaps by looking at distribution it's possible to find a degree number that would guaranteed mathematically that the resulting vertices have degree at least $<|V|/2$ without iteration (e.g. if removing 25% of vertices means that the remaining had degree $>.75|V|$ on full graph, then clearly pruned graph would have degree $>|V|/2$). 
  
  However, note that in `hier01` our iterations go down the tree and split the existing cluster rather than pruning away cell by cell. Without that difference, this pruning could easily devolve into b) with much worse speed and perhaps even worse (after all if a connected component consists of 2 maximal cliques, then maximal cliques would find both, while iterative would only get one of them). So seems like its important to have ability to "capture" those that are discarded, and not clear how to do that.
  
  d) apply some community clustering algorithm to the $\alpha$-graph, which is equivalent to applying it to each maximally connected component (for any algorithm worth anything). This would likely require some parameter to the clustering. Maybe question is how bad this parameter is relative to our current (arbitrary) options; advantage of our current method is at least based on a guarantee on resulting clustering regarding $\alpha$ in some way, which a clustering wouldn't necessarily. 
  
It is unclear how different a) and b) will from each other in practice -- i.e. how much extra is added by including in any cell with small alpha to at least one cell. Clearly if not, none of these differences matter. But seems like we need some kind of method like our current one that is somewhere in between the two, since we don't know.
  
## Updating findNeighbors

### General outline of the function
`findNeighbors` is a upper-level function that finds all neighbors within a given distance of each point. It takes a parameter `BiocNeighborParam` to define the algorithm to do so (this is a argument of class `BiocNeighborParam`). `findKNN` is the corresponding function that finds the k-nearest neighbors of each point. 

`findNeighbors` has two possible options for the algorithm, which is basically a choice of how to most efficiently organize the data in the search: "Kmknn" or "VPTree". `findNeighbors` essentially dispatches `rangeFindKmknn` or `rangeFindVptree`. Note that these functions allow the `threshold` argument to be a vector, specifying a different threshold for each point, though not clear this is useful for us. `findNeighbors` will return not just the indices of the points, but also their distances (if they are less than the threshold).  (`findKNN` also has approximate methods `Annoy` and `Hnsw` but these do not appear to be options for finding within a particular distance; it would be important to consider in comparison methods, like Seurat that build a knn neighbor graph, whether the speed increases they have for large datasets are based on using approximations or not). 

(I would note however, that the  help function for `BiocNeighborParam` only lists "2 concrete subclasses",  `KmknnParam` and `AnnoyParam`. It's not clear why the help doesn't list the other two options as subclasses.)

biocNeighbors has two methods for preprocessing data for fast (exact) computation (from :

* KMKNN (k-means nearest neighbor): In the KMKNN algorithm (Wang, 2012), k-means clustering is first applied to the data points using the square root of the number of points as the number of cluster centers. The cluster assignment anddistance to the assigned cluster center for each point represent the KMKNN indexing information.This speeds up the nearest neighbor search by exploiting the triangle inequality between clustercenters, the query point and each point in the cluster to narrow the search space. The advantage of the KMKNN approach is its simplicity and minimal overhead, resulting in performance improve-ments over conventional tree-based methods for high-dimensional data where most points need tobe searched anyway.  It is also trivially extended to find all neighbors within a threshold distance from a query point.  Note that KMKNN operates much more naturally with Euclidean distances, so your mileage may vary when using it with Manhattan distances.
	- The pre-step runs kmeans on the data (`buildKmknn` using `stat::kmeans`, all R code)
	- Then `.range_find_kmknn` is run, which is a call to C code: `.Call(cxx_range_find_kmknn,...)`
* Vantage-point tree (VPTree): In a VP tree (Yianilos, 1993), each node contains a subset of points and has a defined threshold distance (usually the median distance to all points in the subset). The left child of this node contains the further subset of points within the radius, while the right child contains the remaining points inthe subset that are outside.  The nearest neighbor search traverses the tree and exploits the triangle inequality between query points, node centers and thresholds to narrow the search space. VP trees are often faster than more conventional KD-trees or ball trees as the former uses the points them-selves as the nodes of the tree, avoiding the need to create many intermediate nodes and reducing the total number of distance calculations.  Like KMKNN, it is also trivially extended to find all neighbors within a threshold distance from a query point. "A vantage-point tree (or VP tree) is a metric tree that segregates data in a metric space by choosing a position in the space (the "vantage point") and partitioning the data points into two parts: those points that are nearer to the vantage point than a threshold, and those points that are not. By recursively applying this procedure to partition the data into smaller and smaller sets, a tree data structure is created where neighbors in the tree are likely to be neighbors in the space... The vantage-point tree is particularly useful in dividing data in a non-standard metric space into a metric tree... All it needs is the distance function that satisfies the properties of the metric space... The time cost to build a Vantage-Point tree is approximately O(n log n)" (Wikipedia). 
	- The pre-step builds the VP Tree (`buildVptree` which is a C call, `.Call(cxx_build_vptree,...)`
	- Then call to `.range_find_vptree` (`.Call(cxx_range_find_vptree,...)`)

### Our distance

We need to be sure that our proposed distance satifies the triangle-inequality / defines a proper metric. Otherwise, neither of these algorithms may work... Hamming distance is a proper metric, but we are doing an alternative of it, to deal with NA values (i.e. not use the NA values, but then normalize the distance to account for the number of positions shared).

*How important is this?* For subsampling, we can have samples that were not subsampled together, and thus have NA for their clustering. Our default implementation doesn't actually do this -- instead it assignes every sample, not only the subsampled, based on the clusters defined by the subsampled samples. But if user picked the other option, then it would be a problem. 

For consensus clustering, we get -1 values, and we do not count that against the sample. Namely, we ignore the -1, get the proportion of times together; we then filter out those with too many -1. 

If dealing with the proportion becomes a problem, rather than straightforward Hamming distance, then we could maybe reconsider these options. 

*Algorithms* Clearly the kmknn is not a good option for our distance, since kmeans on our $N x B$ matrix is not going to be terribly informative. 

# New to Version 2.0

In version 2.0.0 of `clusterExperiment` we have made some major changes to the package. The most relevant changes for the user are:

* Changed how we handle dimensionality reduction and filtering of genes -- this includes changes to the names of important arguments of key functions!
* Added support for the main data matrix to be stored in a HDF5 file (i.e. not read into memory)
* Improved implementation of subsampling (in C++) to improve speed 
* Some functions changed names 
* Some additional helper functions were added

These changes are fully detailed in the [NEWS](https://github.com/epurdom/clusterExperiment/blob/master/NEWS) file of the package (all releases since October 31, 2017). 

Below we will explain the first two of these changes in more detail. We would also note that in Bioconductor 3.6 (the release before) there were many more plotting functions that were introduced. 

## Dimensionality reduction and filtering

### Overview

This version consists of a major update of how dimensionality reduction and filtering is done. The `ClusterExperiment` class has been updated to extend the new bioconductor class, `SingleCellExperiment` (see the [SingleCellExperiment package](https://bioconductor.org/packages/release/bioc/html/SingleCellExperiment.html)). 

Briefly, the `SingleCellExperiment` class extends the `SummarizedExperiment` class to give a structure for saving the results of dimensionality reductions.  This is done by adding a slot `reducedDims`, which is a `SimpleList` of datasets that have the same number of observations but reduced dimensions (i.e. genes). This gives a unified slot for saving the results of applying a dimensionality reduction method and helper functions to access them, etc. The user gives them names, e.g. "PCA" or "tSNE". 

**Dimensionality Reduction** `clusterExperiment` now makes use these stored dimensionality reductions in functions like `RSEC` and `clusterMany`. This allows `clusterExperiment` to make use of any dimensionality reduction method so long as the user saves it in the appropriate slot in a `SingleCellExperiment` object. The user can also choose not to calculate any dimensionality reduction and just do as before and have a function like `clusterMany` do the dimensionality reduction (i.e. PCA) internally. The difference is that now the results of the PCA will be stored in the appropriate slot so that they will not need to be recalculated in the future. 

**Filtering of genes** We also added in `clusterExperiment` package a similar option for filtering statistics. In particular, in `clusterExperiment` we have always allowed the user -- instead of dimensionality reduction via PCA-like methods -- to instead reduce the dimensionality of the problem by filtering to the top set of genes, e.g. the top 500 most variable genes. In this case `clusterExperiment` will calculate the variance for every gene and reduce down to the top 500 genes. Now `clusterExperiment` when calculating statistics (like `var` or `mad`) will add the per-gene value of the statistic in a column of the `rowData` of the `ClusterExperiment` object (`rowData` is a standard slot of a `SummarizedExperiment`). Similarly, if the user has already calculated a per-gene statistic and saved it as a column in the `rowData` slot, this user-defined statistic can be used for filtering. This means that the user is not limited to the built-in functions provided in `clusterExperiment`.

Note that simplicity we call "dimensionality reduction methods" to be those like PCA that reduce the data in a way that is *not* a simple selection of existing variables, but rather create new variables to represent the data. This is because a simple selection of variables can be stored as a single vector of the length of the number of genes and the reduced data can be obtained from the original matrix. The more complicated methods actually have to save a matrix with a value for each observation for each new variable. 

### Details 

A great deal has changed under the hood of any function that allowed for dimensionality reduction and greatly simplified and unified our treatment of filtering and dimensionality reduction. The main function affected is `clusterMany`, which runs the clustering, but  `makeDendrogram` is another example (and of course `RSEC` which is a wrapper around these).

To make the function compatible with `SingleCellExperiment`, we have changed many of the names of our arguments related to dimensionality reduction. This is because the slot names and related functions of `SingleCellExperiment` take the form of `reducedDims` and our previous versions of `clusterExperiment` used `dimReduce` format instead. We also changed to the name of arguments to be less tied to 'PCA' and 'var':

- `nPCADims` changed to `nReducedDims` in clusterMany-related functions
- `nVarDims` changed to `nFilterDims` in clusterMany-related functions
- `dimReduce` changed to `reduceMethod` across functions
- `plotDimReduce` to `plotReducedDims`
- `ndims` to `nDims` in `clusterSingle` and `makeDendrogram` to keep consistency.

The package `clusterExperiment` has built in functions for both dimensionality reduction (right now only PCA) and for filtering. These can be obtained by the new functions `listBuiltInReducedDims` and `listBuiltInFilterStats`, which give a character vector of the names of currently available functions for dimensionality reduction and statistics for filtering genes, respectively.

The argument `reduceMethod`, like the previous argument `dimReduce`, defines *either* the dimensionality reduction method or the filtering method to be used to reduce the number of dimensions of the data that will be used (and for `clusterMany` this can be a mixture of the two if the user wants to compare them). In places where multiple values can be given  (i.e. `clusterMany` or `RSEC`) the user has the choice to give to `reduceMethod` either 

* the names of  stored values (which can be a mixture of names of stored statistics for filtering and names of stored dimensionality reduction methods) OR 
* names of built-in functions provided by `clusterExperiment` to be calculated internally (as given by `listBuiltInReducedDims` and `listBuiltInFilterStats`)

The user cannot do both (i.e. give `reducedMethod` 2 names that match user-defined stored values and 3 names that are built-in functions). To do this the user can call the new functions `makeReducedDims` and `makeFilterStats` that will apply the built-in method (and store them appropriately) the built-in methods for dimensionality reductions and statistics for filtering the data, respectively. These are indeed the functions called internally by functions like `clusterMany`. In the above example, after calling these functions for the 3 built-in functions, the user can then call `clusterMany` on all 5 of the names that are stored values (the 2 user defined, and the 3 that were created by the built-in functions).


## HDF5 Support

The `clusterExperiment` packages is now compatible with `SummarizedExperiment` objects that have `DelayedArray` classes in their `assay` slot (which includes `HDF5Matrix` and  `DelayedMatrix`). These are classes from the `DelayedArray` and `HDF5Array` packages that allow the assay to be stored on file rather than in memory. 

Note, however, that while the package allows for these objects, it doesn't mean that it makes use of the HDF5 structure. Many times if the `assay` object must be actually used beyond simple subsetting, it will call the entire matrix into memory for the computations. The advantages currently of having the full dataset in HDF5 format are:

1. If you are not using the full assay for calculations, but rather a version after dimensionality reduction, then the full assay will not be needed in memory, but only the reduced version of the data. This is true for filtering as well as the more complicated dimensionality reductions stored in the `reducedDims` slot, since subsetting of the HDF5 matrix doesn't call the entire matrix into memory. Furthermore, *some* (but not all) of our built-in filtering statistics are HDF5 aware, and do not call the entire matrix into memory to calculate the statistics used for filtering.
2. The matrix is not continually held in memory in R workspace. So even if it is called into memory for a specific calculation, it will free up that memory once the calculation is done (assuming it can hold the full data in memory).

# New to Version 2.3.0

## Change to `ClusterExperiment` Object

**Important** If you have objects created since 2.0.0 but with a version < 2.3.0 (i.e. including 2.0.0), you should run `updateObject` to update the class definition because there have been changes to the class definition since that version:

```
ceObj<-updateObject(ceObj)
```

*Warning* This command will, however, loose information saved about the last `mergeClusters` call that you have made if your object is from version < 2.1.4. You may want to save that information and manually update the slots. If you do so, make sure you call `validObject` to make sure that you have done so correctly (in particular, you will have to have a value for the slot `merge_demethod`, see `?ClusterExperiment` which is a new slot). For example,

```
ceObjNew<-updateObject(ceObj)
ceObjNew@merge_index<-ceObj@merge_index
<etc>
```

If you have objects from before 2.0.0 (when the class was called 'clusterExperiment'), you should construct a new object using the `ClusterExperiment` function. For example,

```
ceObjNew<-ClusterExperiment(
	se=as(ceObj,"SingleCellExperiment"), 
	clusterMatrix=ceObj@clusterMatrix, 
	<etc>
	)
```

See `?ClusterExperiment` for the names of the slots. 


## Other important changes

There have also been a number of changes and enhancements to the package. These are the most important (a complete list is detailed in the [NEWS](https://github.com/epurdom/clusterExperiment/blob/master/NEWS) file of the package -- all releases since May 1, 2018)

* We have changed the function `combineMany` to `makeConsensus`. This has resulted in changes to the names of the arguments of `RSEC`
	- `combineProportion` -> `consensusProportion` in `RSEC`
	- `combineMinSize` -> `consensusMinSize` in `RSEC`
* Add functionality to `getBestFeatures` to allow `edgeR` for DE, as well as weights used with `edgeR` for compatability with weights to handle zero-inflation. As part of this change  `isCount` argument has been replaced with more fine-grained `DEMethod` argument in `getBestFeatures`, `mergeClusters`; and the argument `mergeDEMethod` in `RSEC` is now available.
* We have changed the argument `sampleData` in various plotting commands to `colData` to better indicate that the argument is to identify columns in `colData` that should also be plotted. Furthermore `plotDendrogram` now takes the argument `colData` for plotting of information in `colData` with the dendrogram.
* We have changed the names of arguments related to unassigned (`-1` or `-2` assignments) to more consistently use the term "unassigned", as well as adding the function `assignUnassigned`:
	- argument `removeNegative` -> `removeUnassigned` in `getBestFeatures` 
	- argument `ignoreUnassignedVar` -> `filterIgnoresUnassigned` in `mergeClusters` (and other functions) for clarity.
	- function `removeUnclustered` -> `removeUnassigned`
* New plotting functions:
	- `plotTableClusters`
	- `plotFeatureScatter`
* Allow the arguments `subsample` and `sequential` to `RSEC` to allow for opting out of those options for large datasets (but default is `TRUE` unlike `clusterMany`)
* The argument `whichAssay` is added to most functions to allow the user to select the assay on which the operations will be performed.
* We've changed how we store the cluster hierarchies so that we now use the `phylo4d` class of `phylobase` package (previously we stored them as a `dendrogram` class). This makes it easier to store information about the dendrograms and manipulate them. There are various helper functions related to this change. See `?clusterDendrogram`. 
* We now store the coClustering matrix in the `coClustering` slot as a `sparseMatrix` class from the package `Matrix`. This will reduce the size of the object in memory. 

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seqCluster.R
\name{seqCluster}
\alias{seqCluster}
\title{Program for sequentially clustering, removing cluster, and starting again.}
\usage{
seqCluster(
  inputMatrix,
  inputType,
  k0,
  subsample = TRUE,
  beta,
  top.can = 0.01,
  remain.n = 30,
  k.min = 3,
  k.max = k0 + 10,
  verbose = TRUE,
  subsampleArgs = NULL,
  mainClusterArgs = NULL,
  warnings = FALSE
)
}
\arguments{
\item{inputMatrix}{numerical matrix on which to run the clustering or a
\code{\link[SummarizedExperiment]{SummarizedExperiment}},
\code{\link{SingleCellExperiment}}, or \code{\link{ClusterExperiment}}
object.}

\item{inputType}{a character vector defining what type of input is given in
the \code{inputMatrix} argument. Must consist of values "diss","X", or
"cat" (see details). "X" and "cat" should be indicate
matrices with features in the row and samples in the column; "cat"
corresponds to the features being numerical integers corresponding to
categories, while "X" are continuous valued features. "diss" corresponds to
an \code{inputMatrix} that is a NxN dissimilarity matrix. "cat" is largely
used internally for clustering of sets of clusterings.}

\item{k0}{the value of K at the first iteration of sequential algorithm, see
details below or vignette.}

\item{subsample}{logical as to whether to subsample via 
\code{\link{subsampleClustering}} to get the distance matrix at each 
iteration; otherwise the distance matrix is set by arguments to
\code{\link{mainClustering}}.}

\item{beta}{value between 0 and 1 to decide how stable clustership membership
has to be before 'finding' and removing the cluster.}

\item{top.can}{only the top.can clusters from \code{\link{mainClustering}} (ranked
by 'orderBy' argument given to \code{\link{mainClustering}}) will be compared
pairwise for stability. Can be either an integer value, identifying the absolute number of clusters, or a value between 0 and 1, meaning to keep all clusters with at least this proportion of the remaining samples in the cluster. Making this either a very big integer or equal to 0 will effectively remove this
parameter and all pairwise comparisons of all clusters found will be
considered; this might result in smaller clusters being found. If top.can is between 0 and 1, then there is still a hard threshold of at least 5 samples in a cluster to be considered as a cluster.}

\item{remain.n}{when only this number of samples are left (i.e. not yet
clustered) then algorithm will stop.}

\item{k.min}{each iteration of sequential detection of clustering will
decrease the beginning K of subsampling, but not lower than k.min.}

\item{k.max}{algorithm will stop if K in iteration is increased beyond this
point.}

\item{verbose}{whether the algorithm should print out information as to its
progress.}

\item{subsampleArgs}{list of arguments to be passed to
\code{\link{subsampleClustering}}.}

\item{mainClusterArgs}{list of arguments to be passed to
\code{\link{mainClustering}}).}

\item{warnings}{logical. Whether to print out the many possible warnings and
messages regarding checking the internal consistency of the parameters.}
}
\value{
A list with values
\itemize{

\item{\code{clustering}}{ a vector of length equal to nrows(x) giving the
integer-valued cluster ids for each sample. The integer values are assigned
in the order that the clusters were found. "-1" indicates the sample was not
clustered.}

\item{\code{clusterInfo}}{ if clusters were successfully found, a matrix of
information regarding the algorithm behavior for each cluster (the starting
and stopping K for each cluster, and the number of iterations for each
cluster).}

\item{\code{whyStop}}{ a character string explaining what triggered the
algorithm to stop.}
}
}
\description{
Given a data matrix, this function will call clustering
routines, and sequentially remove best clusters, and iterate to find
clusters.
}
\details{
\code{seqCluster} is not meant to be called by the user. It is only
  an exported function so as to be able to clearly document the arguments for
  \code{seqCluster} which can be passed via the argument \code{seqArgs} in
  functions like \code{\link{clusterSingle}} and \code{\link{clusterMany}}.

This code is adapted from the sequential protion of the code of the
  tightClust package of Tseng and Wong. At each iteration of the algorithm it
  finds a set of samples that constitute a homogeneous cluster and remove
  them, and iterate again to find the next set of samples that form a
  cluster.

In each iteration, to determine the next set of homogeneous set of
  samples, the algorithm will iteratively cluster the current set of samples
  for a series of increasing values of the parameter $K$, starting at a value
  \code{kinit} and increasing by 1 at each iteration, until a sufficiently
  homogeneous set of clusters is found. For the first set of homogeneous
  samples, \code{kinit} is set to the argument $k0$, and for iteration,
  \code{kinit} is increased internally.

Depending on the value of \code{subsample} how the value of $K$ is
  used differs. If \code{subsample=TRUE}, $K$ is the \code{k} sent to the
  cluster function \code{clusterFunction} sent to 
  \code{\link{subsampleClustering}} via \code{subsampleArgs}; then
  \code{\link{mainClustering}} is run on the result of the co-occurance matrix from
  \code{\link{subsampleClustering}} with the \code{ClusterFunction} object
  defined in the argument \code{clusterFunction} set via \code{mainClusterArgs}.
  The number of clusters actually resulting from this run of
  \code{\link{mainClustering}} may not be equal to the $K$ sent to  the clustering
  done in \code{\link{subsampleClustering}}. If \code{subsample=FALSE},
  \code{\link{mainClustering}} is called directly on the data to determine the
  clusters and $K$ set by \code{seqCluster} for this iteration determines the
  parameter of the clustering done by \code{\link{mainClustering}}. Specifically,
  the argument \code{clusterFunction} defines the clustering of the
  \code{\link{mainClustering}} step and \code{k} is sent to that
  \code{ClusterFunction} object. This means that if \code{subsample=FALSE},
  the \code{clusterFunction} must be of \code{algorithmType} "K".

In either setting of \code{subsample}, the resulting clusters from
  \code{\link{mainClustering}} for a particular $K$ will be compared to clusters
  found in the previous iteration of $K-1$. For computational (and other?)
  convenience, only the first \code{top.can} clusters of each iteration will
  be compared to the first \code{top.can} clusters of previous iteration for
  similarity (where \code{top.can} currently refers to ordering by size, so
  first \code{top.can} largest clusters.

If there is no cluster of the first \code{top.can} in the current
  iteration $K$ that has overlap similarity > \code{beta} to any in the
  previous iteration, then the algorithm will move to the next iteration,
  increasing to $K+1$.

If, however, of these clusters there is a cluster in the current
  iteration $K$ that has overlap similarity > beta to a cluster in the
  previous iteration $K-1$, then the cluster with the largest such similarity
  will be identified as a homogenous set of samples and the samples in it
  will be removed and designated as such. The algorithm will then start again
  to determine the next set of homogenous samples, but without these samples.
  Furthermore, in this case (i.e. a cluster was found and removed), the value
  of \code{kinit} will be be reset to \code{kinit-1}; i.e. the range of
  increasing $K$ that will be iterated over to find a set of homogenous
  samples will start off one value less than was the case for the previous
  set of homogeneous samples. If \code{kinit-1}<\code{k.min}, then
  \code{kinit} will be set to \code{k.min}.

If there are less than \code{remain.n} samples left after finding a
  cluster and removing its samples, the algorithm will stop, as subsampling
  is deamed to no longer be appropriate. If the K has to be increased to
  beyond \code{k.max} without finding any pair of clusters with overlap >
  beta, then the algorithm will stop. Any samples not found as part of a
  homogenous set of clusters at that point will be classified as unclustered
  (given a value of -1)

Certain combinations of inputs to \code{mainClusterArgs} and
  \code{subsampleArgs} are not allowed. See \code{\link{clusterSingle}} for
  these explanations.
}
\examples{
\dontrun{
data(simData)

set.seed(12908)
clustSeqHier <- seqCluster(simData, inputType="X", k0=5, subsample=TRUE,
   beta=0.8, subsampleArgs=list(resamp.n=100,
   samp.p=0.7, clusterFunction="kmeans", clusterArgs=list(nstart=10)),
   mainClusterArgs=list(minSize=5,clusterFunction="hierarchical01",
   clusterArgs=list(alpha=0.1)))
}
}
\references{
Tseng and Wong (2005), "Tight Clustering: A Resampling-Based
  Approach for Identifying Stable and Tight Patterns in Data", Biometrics,
  61:10-16.
}
\seealso{
tight.clust,
  \code{\link{clusterSingle}},\code{\link{mainClustering}},\code{\link{subsampleClustering}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotContrastHeatmap.R
\name{plotContrastHeatmap,ClusterExperiment-method}
\alias{plotContrastHeatmap,ClusterExperiment-method}
\alias{plotContrastHeatmap}
\title{Plot heatmaps showing significant genes per contrast}
\usage{
\S4method{plotContrastHeatmap}{ClusterExperiment}(
  object,
  signifTable,
  whichCluster = NULL,
  contrastColors = NULL,
  ...
)
}
\arguments{
\item{object}{ClusterExperiment object on which biomarkers were found}

\item{signifTable}{A \code{data.frame} in format of the result of
\code{\link{getBestFeatures}}. It must minimally contain columns 'Contrast'
and 'IndexInOriginal' giving the grouping and original index of the
features in the \code{assay(object)}}

\item{whichCluster}{if not NULL, indicates cluster used in making the 
significance table. Used to match to colors in \code{clusterLegend(object)} 
(relevant for one-vs-all contrast so that color aligns). See description of 
argument in \code{\link{getClusterIndex}} for futher details.}

\item{contrastColors}{vector of colors to be given to contrasts. Should match
the name of the contrasts in the 'Contrast' column of \code{signifTable} or
'ContrastName', if given.. If  missing, default colors given by match to
the cluster names of \code{whichCluster} (see above), or otherwise given a
default assignment.}

\item{...}{Arguments passed to \code{\link{plotHeatmap}}}
}
\value{
A heatmap is created. The output of \code{plotHeatmap} is returned.
}
\description{
Plots a heatmap of the data, with the genes grouped based on the
  contrast for which they were significant.
}
\details{
If the column 'ContrastName' is given in \code{signifTable}, these
  names will be used to describe the contrast in the legend.

Within each contrast, the genes are sorted by log fold-change if the
  column "logFC" is in the \code{signifTable} data.frame

Note that if \code{whichCluster} is NOT given (the default) then there is 
no automatic match of colors with contrasts based on the information in 
\code{object}.
}
\examples{
data(simData)

cl <- clusterSingle(simData, subsample=FALSE,
sequential=FALSE, 
mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=8)))

#Do all pairwise, only return significant, try different adjustments:
pairsPerC <- getBestFeatures(cl, contrastType="Pairs", number=5,
p.value=0.05, DEMethod="limma")
plotContrastHeatmap(cl,pairsPerC)
}
\seealso{
\code{\link{plotHeatmap}}, \code{\link{makeBlankData}},
  \code{\link{getBestFeatures}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataCreation.R
\docType{data}
\name{simData}
\alias{simData}
\alias{simCount}
\alias{trueCluster}
\title{Simulated data for running examples}
\format{
Three objects are loaded, two data frame(s) of simulated data each
  with 300 samples/columns and 153 variables/rows, and a vector of length 300
  with the true cluster assignments.
}
\description{
Simulated data for running examples
}
\details{
\code{simData} is simulated normal data of 300 observations with 51
  relevant variables and the rest of the variables being noise, with
  observations being in one of 3 groups. \code{simCount} is simulated count
  data of the same dimensions. \code{trueCluster} gives the true cluster
  identifications of the samples. The true clusters are each of size 100 and
  are in order in the columns of the data.frames.
}
\examples{
#code used to create data:
\dontrun{
nvar<-51 #multiple of 3
n<-100
x<-cbind(matrix(rnorm(n*nvar,mean=5),nrow=nvar),
 matrix(rnorm(n*nvar,mean=-5),nrow=nvar),
          matrix(rnorm(n*nvar,mean=0),nrow=nvar))
#make some of them flipped effects (better for testing if both sig under/over
#expressed variables)
geneGroup<-sample(rep(1:3,each=floor(nvar/3)))
gpIndex<-list(1:n,(n+1):(n*2),(2*n+1):(n*3))
x[geneGroup==1,]<-x[geneGroup==1,unlist(gpIndex[c(3,1,2)])]
x[geneGroup==2,]<-x[geneGroup==2,unlist(gpIndex[c(2,3,1)])]

#add in differences in variable means
smp<-sample(1:nrow(x),10)
x[smp,]<-x[smp,]+10

#make different signal y
y<-cbind(matrix(rnorm(n*nvar,mean=1),nrow=nvar),
         matrix(rnorm(n*nvar,mean=-1),nrow=nvar),
         matrix(rnorm(n*nvar,mean=0),nrow=nvar))
y<-y[,sample(1:ncol(y))]+ matrix(rnorm(3*n*nvar,sd=3),nrow=nvar)

#add together the two signals
simData<-x+y

#add pure noise variables
simData<-rbind(simData,matrix(rnorm(3*n*nvar,mean=10),nrow=nvar),
               matrix(rnorm(3*n*nvar,mean=5),nrow=nvar))
#make count data
countMean<-exp(simData/2)
simCount<-matrix(rpois(n=length(as.vector(countMean)), lambda
=as.vector(countMean)+.1),nrow=nrow(countMean),ncol=ncol(countMean))
#labels for the truth
trueCluster<-rep(c(1:3),each=n)
save(list=c("simCount","simData","trueCluster"),file="data/simData.rda")
}
}
\author{
Elizabeth Purdom \email{epurdom@stat.berkeley.edu}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internalFunctions.R
\name{numericalAsCharacter}
\alias{numericalAsCharacter}
\title{Convert numeric values to character that sort correctly}
\usage{
numericalAsCharacter(values, prefix = "")
}
\arguments{
\item{values}{vector of values to be converted into sortable character values}

\item{prefix}{optional character string that will be added as prefix to the
result}
}
\value{
A character vector
}
\description{
Small function that takes as input integer values (or values
  that can be converted to integer values) and converts them into character
  values that are 'padded' with zeros at the beginning of the numbers so that
  they will sort correctly.
}
\details{
The function determines the largest value and adds zeros to the
  front of smaller integers so that the resulting characters are the same
  number of digits. This allows standard sorting of the values to correctly
  sort.

The maximum number of zeros that will be added is 3. Input integers
  beyond that point will not be correctly fixed for sorting.

Negative integers will not be corrected, but left as-is
}
\examples{
numericalAsCharacter(c(-1, 5,10,20,100))
}
\seealso{
\code{\link[stringr]{str_pad}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeClusters.R
\name{mergeClusters}
\alias{mergeClusters}
\alias{mergeClusters,matrixOrHDF5-method}
\alias{mergeClusters,ClusterExperiment-method}
\alias{nodeMergeInfo,ClusterExperiment-method}
\alias{nodeMergeInfo}
\alias{mergeCutoff,ClusterExperiment-method}
\alias{mergeCutoff}
\alias{mergeMethod,ClusterExperiment-method}
\alias{mergeMethod}
\alias{mergeClusterIndex,ClusterExperiment-method}
\alias{mergeClusterIndex}
\alias{eraseMergeInfo,ClusterExperiment-method}
\alias{eraseMergeInfo}
\alias{getMergeCorrespond,ClusterExperiment-method}
\alias{getMergeCorrespond}
\title{Merge clusters based on dendrogram}
\usage{
\S4method{mergeClusters}{matrixOrHDF5}(
  x,
  cl,
  dendro = NULL,
  mergeMethod = c("none", "Storey", "PC", "adjP", "locfdr", "JC"),
  plotInfo = "none",
  nodePropTable = NULL,
  calculateAll = TRUE,
  showWarnings = FALSE,
  cutoff = 0.05,
  plot = TRUE,
  DEMethod,
  logFCcutoff = 0,
  weights = NULL,
  ...
)

\S4method{mergeClusters}{ClusterExperiment}(
  x,
  eraseOld = FALSE,
  mergeMethod = "none",
  plotInfo = "all",
  clusterLabel = "mergeClusters",
  leafType = c("samples", "clusters"),
  plotType = c("colorblock", "name", "ids"),
  plot = TRUE,
  whichAssay = 1,
  forceCalculate = FALSE,
  weights = if ("weights" \%in\% assayNames(x)) "weights" else NULL,
  DEMethod,
  ...
)

\S4method{nodeMergeInfo}{ClusterExperiment}(x)

\S4method{mergeCutoff}{ClusterExperiment}(x)

\S4method{mergeMethod}{ClusterExperiment}(x)

\S4method{mergeClusterIndex}{ClusterExperiment}(x)

\S4method{eraseMergeInfo}{ClusterExperiment}(x)

\S4method{getMergeCorrespond}{ClusterExperiment}(x, by = c("merge", "original"))
}
\arguments{
\item{x}{data to perform the test on. It can be a matrix or a 
\code{\link{ClusterExperiment}}.}

\item{cl}{A numeric vector with cluster assignments to compare to. ``-1'' 
indicates the sample was not assigned to a cluster.}

\item{dendro}{dendrogram providing hierarchical clustering of clusters in cl.
If x is a matrix, then the default is \code{dendro=NULL} and the function 
will calculate the dendrogram with the given (x, cl) pair using 
\code{\link{makeDendrogram}}. If x is a \code{\link{ClusterExperiment}} 
object, the dendrogram in the slot \code{dendro_clusters} will be used. In 
this case, this means that \code{\link{makeDendrogram}} needs to be called 
before \code{mergeClusters}.}

\item{mergeMethod}{method for calculating proportion of non-null that will be
used to merge clusters (if 'none', no merging will be done). See details 
for description of methods.}

\item{plotInfo}{what type of information about the merging will be shown on 
the dendrogram. If 'all', then all the estimates of proportion non-null 
will be plotted at each node of the dendrogram; if 'mergeMethod', then only
the value used in the \code{mergeClusters} command is plotted at each node.
If 'none', then no proportions will be added to the dendrogram, though the 
dendrogram will be drawn. 'plotInfo' can also be one of the valid input to 
\code{mergeMethod} (even if that method is not the method chosen in 
\code{mergeMethod} argument). \code{plotInfo} can also show the information
corresponding  to "adjP" with a fold-change cutoff, by giving a value to 
this argument in  the form of "adjP_2.0", for example.}

\item{nodePropTable}{Only for matrix version. Matrix of results from previous
run of \code{mergeClusters} as returned by matrix version of 
\code{mergeClusters}. Useful if just want to change the cutoff. Not 
generally intended for user but used internally by package.}

\item{calculateAll}{logical. Whether to calculate the estimates for all 
methods. This reduces computation costs for any future calls to 
\code{mergeClusters} since the results can be passed to future calls of 
\code{mergeClusters} (and for \code{ClusterExperiment} objects this is done
automatically).}

\item{showWarnings}{logical. Whether to show warnings given by the methods. 
The 'locfdr' method in particular frequently spits out warnings (which may 
indicate that its estimates are not reliable). Setting 
\code{showWarnings=FALSE} will suppress all warnings from all methods (not 
just "locfdr"). By default this is set to \code{showWarnings=FALSE} by 
default to avoid large number of warnings being produced by "locfdr", but 
users may want to be more careful to check the warnings for themselves.}

\item{cutoff}{minimimum value required for NOT merging a cluster, i.e. two 
clusters with the proportion of DE below cutoff will be merged. Must be a 
value between 0, 1, where lower values will make it harder to merge 
clusters.}

\item{plot}{logical as to whether to plot the dendrogram with the merge 
results}

\item{DEMethod}{character vector describing how the differential expression 
 analysis should be performed that will be used in the estimation of the
percentage DE per node. See \link{getBestFeatures} for current options. 
 See details.}

\item{logFCcutoff}{Relevant only if the \code{mergeMethod} selected is 
"adjP", in which case the calculation of the proportion of individual tests
significant will also require that the estimated log-fold change of the 
features to be at least this large in absolute value. Value will be rounded
to nearest tenth of an integer via \code{round(logFCcutoff,digits=1)}. For 
any other method, this parameter is ignored. Note that the logFC is based
on \code{log2} (the results of \code{\link{getBestFeatures}})}

\item{weights}{weights to use in by edgeR. If \code{x} is a matrix, then
weights should be a matrix of weights, of the same dimensions as \code{x}.
If \code{x} is a \code{ClusterExperiment} object \code{weights} can be a
either a matrix, as previously described, or a character or numeric index
to an assay in \code{x} that contains the weights. We recommend that 
weights be stored as an assay with name \code{"weights"} so that the
weights will also be used with \code{\link{mergeClusters}}, and this is the
default. Setting \code{weights=NULL} ensures that weights will NOT be used,
and only the standard edgeR.}

\item{...}{for signature \code{matrix}, arguments passed to the 
\code{\link{plot.phylo}} function of \code{ape} that plots the dendrogram. 
For signature \code{ClusterExperiment} arguments passed to the method for 
signature \code{matrix} and then if do not match those arguments, will be 
passed onto \code{\link{plot.phylo}}.}

\item{eraseOld}{logical. Only relevant if input \code{x} is of class
\code{ClusterExperiment}. If TRUE, will erase existing workflow results
(clusterMany as well as mergeClusters and makeConsensus). If FALSE, existing
workflow results will have "\code{_i}" added to the clusterTypes value,
where \code{i} is one more than the largest such existing workflow
clusterTypes.}

\item{clusterLabel}{a string used to describe the type of clustering. By
default it is equal to "mergeClusters", to indicate that this clustering is
the result of a call to mergeClusters (only if x is a ClusterExperiment object)}

\item{leafType}{if plotting, whether the leaves should be the clusters or the
samples. Choosing 'samples' allows for visualization of how many samples
are in the merged clusters (only if x is a ClusterExperiment object), which
is the main difference between choosing "clusters" and "samples",
particularly if \code{plotType="colorblock"}}

\item{plotType}{if plotting, then whether leaves of dendrogram should be
labeled by rectangular blocks of color ("colorblock")  or with the names of
the leaves ("name") (only if x is a ClusterExperiment object).}

\item{whichAssay}{numeric or character specifying which assay to use. See
\code{\link[SummarizedExperiment]{assay}} for details.}

\item{forceCalculate}{This forces the function to erase previously saved
merge results and recalculate the merging.}

\item{by}{indicates whether output from \code{getMergeCorrespond} should be
a vector/list with elements corresponding to merge cluster ids or elements
corresponding to the original clustering ids. See return value for details.}
}
\value{
If `x` is a matrix, it returns (invisibly) a list with elements
\itemize{ 
\item{\code{clustering}}{ a vector of length equal to ncol(x) giving
the integer-valued cluster ids for each sample. "-1" indicates the sample was
not clustered.} 
\item{\code{oldClToNew}}{ A table of the old cluster labels to
the new cluster labels.} 
\item{\code{nodeProp}}{ A table of the proportions
that are DE on each node.This table is saved in the \code{merge_nodeProp} slot
of a \code{ClusterExperiment} object and can be accessed along with the
nodeMerge info with the \code{nodeMergeInfo} function. 
}
\item{\code{nodeMerge}}{ 
A table of indicating for each node whether merged or
not and the cluster id in the new clustering that corresponds to the node.
Note that a node can be merged and not correspond to a node in the new
clustering, if its ancestor node is also merged. But there must be some node
that corresponds to a new cluster id if merging has been done. This table is
saved in the \code{merge_nodeMerge} slot of a \code{ClusterExperiment} object
and can be accessed along with the nodeProp info with the \code{nodeMergeInfo}
function. 
} 
\item{\code{updatedClusterDendro}}{ The dendrogram on which the
merging was based (based on the original clustering).
} 
\item{\code{cutoff}}{
The cutoff value for merging.
} 
}

If `x` is a \code{\link{ClusterExperiment}}, it returns a new
  \code{ClusterExperiment} object with an additional clustering based on the
  merging. This becomes the new primary clustering. Note that even if
  \code{mergeMethod="none"}, the returned object will erase any old merge
  information, update the work flow numbering, and return the newly calculated 
  merge information.

\code{nodeMergeInfo} returns information collected about the nodes
  during merging as a data.frame with the following entries:
\itemize{ \item{\code{Node}}{ Name of the node}
\item{\code{Contrast}}{The
contrast compared at each node, in terms of the cluster ids}
\item{\code{isMerged}}{ Logical as to whether samples from that node which were
merged into one cluster during merging}
\item{\code{mergeClusterId}}{ If a
node corresponds to a new, merged cluster, gives the cluster id it
corresponds to. Otherwise NA}
\item{\code{...}}{The remaining columns give
the estimated proportion of genes differentially expressed for each method. A
column of NAs means that the method in question hasn't been calculated yet.}
}

\code{mergeCutoff} returns the cutoff used for the current merging.

\code{mergeMethod} returns the method used for the current merge.

\code{mergeClusterIndex} returns the index of the clustering used for the current merge.

\code{eraseMergeInfo} returns object with all previously saved merge info removed.

\code{getMergeCorrespond} returns the correspondence between the
  merged cluster and its originating cluster. If \code{by="original"} returns
  a named vector, where the names of the vector are the cluster ids of the
  originating cluster and the values of the vector are the cluster ids of the
  merged cluster. If \code{by="merge"} the results returned are organized by
  the merged clusters. This will generally be a list, with the names of the
  list equal to the clusterIds of the merge clusters and the entries the
  clusterIds of the originating clusters. However, if there was no merging
  done (so that the clusters are identical) the output will be a vector like
  with \code{by="original"}.
}
\description{
Takes an input of hierarchical clusterings of clusters and
  returns estimates of number of proportion of non-null and merges those
  below a certain cutoff.
}
\details{
\strong{Estimation of proportion non-null} "Storey" refers to the
  method of Storey (2002). "PC" refers to the method of Pounds and Cheng
  (2004). "JC" refers to the method of Ji and Cai (2007), and implementation
  of "JC" method is copied from code available on Jiashin Ji's website,
  December 16, 2015
  (http://www.stat.cmu.edu/~jiashun/Research/software/NullandProp/). "locfdr"
  refers to the method of Efron (2004) and is implemented in the package
  \code{\link{locfdr}}.  "adjP"
  refers to the proportion of genes that are found significant based on a FDR
  adjusted p-values (method "BH") and a cutoff of 0.05. Previous versions offered the method "MB", a method of Meinshausen and Buhlmann
  (2005), but the package  \code{howmany} is no longer supported for its implementation.

\strong{Control of Plotting} If \code{mergeMethod} is not equal to
  'none' then the plotting will indicate where the clusters will be merged by
  making dotted lines of edges that are merged together (assuming
  \code{plotInfo} is not 'none'). \code{plotInfo} controls simultaneously
  what information will be plotted on the nodes as well as whether the dotted
  lines will be shown for the merged cluster. Notice that the choice of
  \code{plotInfo} (as long as it is not 'none') has no effect on how the
  dotted edges are drawn -- they are always drawn based on the
  \code{mergeMethod}. If you choose \code{plotInfo} to not be equal to the
  \code{mergeMethod}, then you will have a confusing picture where the dotted
  edges will be based on the clustering created by \code{mergeMethod} while
  the information on the nodes is based on a different method. Note that you
  can override \code{plotInfo} by setting \code{show.node.label=FALSE}
  (passed to plot.phylo), so that no information is plotted on the nodes, but
  the dotted edges are still drawn. If you just want plot of the dendrogram,
  with no merging performed nor demonstrated on the plot, see
  \code{\link{plotDendrogram}}.

\strong{Saving and Reusing of results} By default, the function
  saves the results in the \code{ClusterExperiment} object and will not
  recalculate them if not needed. Note that by default
  \code{calculateAll=TRUE}, which means that regardless of the value of
  \code{mergeMethod}, all the methods will be calculated so that those
  results will be stored and if you change the mergeMethod, no additional
  calculations are needed. Since the computationally intensive step is the
  running the DE method on the genes, this is a big savings (all of the
  methods then calculate the proportion from those results). However, note
  that if \code{calculateAll=TRUE} and ANY of the methods returned NA for any
  value, the calculation will be redone. Thus if, for example, the
  \code{locfdr} function does not run successfully and returns NA, the
  function will always recalculate each time, even if you don't specifically
  want the results of \code{locfdr}. In this case, it makes sense to turn
  \code{calculateAll=FALSE}.

If the dendrogram was made with option
  \code{unassignedSamples="cluster"} (i.e. unassigned were clustered in with
  other samples), then you cannot choose the option
  \code{leafType='samples'}. This is because the current code cannot reliably
  link up the internal nodes of the sample dendrogram to the internal nodes
  of the cluster dendrogram when the unassigned samples are intermixed.

When the input is a \code{ClusterExperiment} object, the function
    attempts to update the merge information in that object. This is done by
    checking that the existing dendrogram stored in the object  (and run on
    the clustering stored in the slot \code{dendro_index}) is the same
    clustering that is stored in the slot \code{merge_dendrocluster_index}.
    For this reason, new calls to \code{\link{makeDendrogram}} will erase the merge
    information saved in the object.

If \code{mergeClusters} is run with \code{mergeMethod="none"}, the
  function may still calculate the proportions per node if \code{plotInfo} is
  not equal to "none" or \code{calculateAll=TRUE}. If the input object was a
  \code{ClusterExperiment} object, the resulting information will be still
  saved, though no new clustering was created; if there was not an existing
  merge method, the slot \code{merge_dendrocluster_index} will be updated.
}
\examples{
data(simData)

#create a clustering, for 8 clusters (truth was 3)
cl<-clusterSingle(simData, subsample=FALSE,
sequential=FALSE, mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=8)))

#give more interesting names to clusters:
newNames<- paste("Cluster",clusterLegend(cl)[[1]][,"name"],sep="")
clusterLegend(cl)[[1]][,"name"]<-newNames
#make dendrogram
cl <- makeDendrogram(cl)

#plot showing the before and after clustering
#(Note argument 'use.edge.length' can improve
#readability)
merged <- mergeClusters(cl, plotInfo="all",
mergeMethod="adjP", use.edge.length=FALSE, DEMethod="limma")

#Simpler plot with just dendrogram and single method
merged <- mergeClusters(cl, plotInfo="mergeMethod",
mergeMethod="adjP", use.edge.length=FALSE, DEMethod="limma",
leafType="clusters",plotType="name")

#compare merged to original
tableClusters(merged,whichClusters=c("mergeClusters","clusterSingle"))

}
\references{
Ji and Cai (2007), "Estimating the Null and the Proportion
of Nonnull Effects in Large-Scale Multiple Comparisons", JASA 102: 495-906.

Efron (2004) "Large-scale simultaneous hypothesis testing:
the choice of a null hypothesis," JASA, 99: 96-104.

Meinshausen and Buhlmann (2005) "Lower bounds for the
number of false null hypotheses for multiple testing of associations",
Biometrika 92(4): 893-907.

Storey (2002) "A direct approach to false discovery rates", J. R.
  Statist. Soc. B 64 (3)": 479-498.

Pounds and Cheng (2004). "Improving false discovery rate
  estimation." Bioinformatics 20(11): 1737-1745.
}
\seealso{
makeDendrogram, plotDendrogram, getBestFeatures
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterContrasts.R
\docType{methods}
\name{clusterContrasts}
\alias{clusterContrasts}
\alias{clusterContrasts,ClusterExperiment-method}
\alias{clusterContrasts,vector-method}
\title{Create contrasts for testing DE of a cluster}
\usage{
\S4method{clusterContrasts}{ClusterExperiment}(cluster, contrastType, ...)

\S4method{clusterContrasts}{vector}(
  cluster,
  contrastType = c("Dendro", "Pairs", "OneAgainstAll"),
  dendro = NULL,
  pairMat = NULL,
  outputType = c("limma", "MAST"),
  removeUnassigned = TRUE
)
}
\arguments{
\item{cluster}{Either a vector giving contrasts assignments or a 
ClusterExperiment object}

\item{contrastType}{What type of contrast to create. `Dendro' traverses the
given dendrogram and does contrasts of the samples in each side,  `Pairs'
does pair-wise contrasts based on the pairs given in pairMat (if
pairMat=NULL, does all pairwise), and `OneAgainstAll' compares each cluster
to the average of all others.}

\item{...}{arguments that are passed to from the \code{ClusterExperiment} 
version to the most basic numeric version.}

\item{dendro}{The dendrogram to traverse if contrastType="Dendro". Note that
this should be the dendrogram of the clusters, not of the individual
samples, either of class "dendrogram" or "phylo4"}

\item{pairMat}{matrix giving the pairs of clusters for which to do pair-wise 
contrasts (must match to elements of cl). If NULL, will do all pairwise of 
the clusters in \code{cluster} (excluding "-1" categories). Each row is a
pair to be compared and must match the names of the clusters in the vector 
\code{cluster}.}

\item{outputType}{character string. Gives format for the resulting contrast 
matrix. Currently the two options are the format appropriate for 
\code{\link[limma]{limma}} and \code{\link[MAST]{MAST}} package.}

\item{removeUnassigned}{logical, whether to remove negative valued clusters 
from the design matrix. Appropriate to pick TRUE (default) if design will 
be input into linear model on samples that excludes -1.}
}
\value{
List with components:
\itemize{ 
\item{\code{contrastMatrix}}{ Contrast matrix, the form of which
depends on \code{outputType}. If \code{outputType=="limma"}, the result of
running \code{\link{makeContrasts}}: a matrix with number of columns equal to
the number of contrasts, and rows equal to the number of levels of the factor
that will be fit in a linear model.} 
\item{\code{contrastNames}}{A vector of
names for each of the contrasts. NULL if no such additional names.} 
}
}
\description{
Uses clustering to create different types of contrasts to be 
  tested that can then be fed into DE testing programs.
}
\details{
The input vector must be numeric clusters, but the external commands
  that make the contrast matrix (e.g. \code{\link{makeContrasts}}) require 
  syntatically valid R names. For this reason, the names of the levels will 
  be "X1" instead of "1". And negative values (if removeUnassigned=FALSE) will 
  be "X.1","X.2", etc.
}
\examples{
\dontrun{
data(simData)
cl <- clusterMany(simData,nReducedDims=c(5,10,50),  
reduceMethod="PCA", makeMissingDiss=TRUE,
clusterFunction="pam", ks=2:4, findBestK=c(FALSE), removeSil=TRUE,
subsample=FALSE)
#Pairs:
clusterContrasts(cl,contrastType="Pairs")
#Dendrogram
cl<-makeDendrogram(cl)
clusterContrasts(cl,contrastType="Pairs")
}
}
\references{
Ritchie, ME, Phipson, B, Wu, D, Hu, Y, Law, CW, Shi, W, and
  Smyth, GK (2015). limma powers differential expression analyses for
  RNA-sequencing and microarray studies. Nucleic Acids Research 43, e47.
  http://nar.oxfordjournals.org/content/43/7/e47

Finak, et al. MAST: a flexible statistical framework for
  assessing transcriptional changes and characterizing heterogeneity in
  single-cell RNA sequencing data. Genome Biology (2015).
}
\author{
Elizabeth Purdom
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/updateObject.R
\name{updateObject}
\alias{updateObject}
\alias{updateObject,ClusterExperiment-method}
\title{Update old ClusterExperiment object to current class definition}
\usage{
\S4method{updateObject}{ClusterExperiment}(object, checkTransformAndAssay = FALSE, ..., verbose = FALSE)
}
\arguments{
\item{object}{a \code{ClusterExperiment} (or \code{clusterExperiment} from 
older versions). Must have at a minimum a slot \code{clusterMatrix}.}

\item{checkTransformAndAssay}{logical. Whether to check the content of the
assay and given transformation function for whether they are valid.}

\item{...}{
    Additional arguments, for use in specific \code{updateObject}
    methods.
  }

\item{verbose}{
    \code{TRUE} or \code{FALSE}, indicating whether information about
    the update should be reported. Use \code{\link[base]{message}} to
    report this information.
  }
}
\value{
A valid \code{ClusterExperiment} object based on the current 
  definition of ClusterExperiment.
}
\description{
This function updates ClusterExperiment objects from previous
  versions of package into the current definition
}
\details{
The function creates a valid \code{ClusterExperiment} object by 
  adding the default values of missing slots. It does so by calling the 
  \code{\link{ClusterExperiment}} function, which imputs default (empty) 
  values for missing slots.

The object is required to have minimal components to be updated. 
  Specifically, it must have all the required elements of a Summarized 
  Experiment as well as the basic slots of a ClusterExperiment object which 
  have not changed over time. These are: \code{clusterMatrix},
  \code{primaryIndex}, \code{clusterInfo}, \code{transformation}, 
  \code{clusterTypes}, \code{clusterLegend}, \code{orderSamples}.

If \emph{any} of the dendrogram-related slots are missing, ALL of 
  the dendrogram \emph{and} merge related slots will be cleared to default 
  values. Similarly, if \emph{any} of the merge-related slots are missing, 
  ALL of the merge-related slots will be cleared to the default values.

Cluster and Sample dendrograms of the class \code{dendrogram} will
  be updated to the \code{\link[phylobase]{phylo4d}} class now used in
  \code{ClusterExperiment} objects; the merge information on these nodes will
  be updated to have the correct format (i.e. match to the internal node id
  names in the new dendrogram). The previous identification of nodes that was
  previously created internally by plotDendrogram and the merging (labels in
  the form of 'Node1','Node2'), will be kept as
  \code{\link[phylobase]{nodeLabels}} in the new dendrogram class.

The function currently only works for object of 
  \code{ClusterExperiment}, not the older name \code{clusterExperiment}.
}
\seealso{
\code{\link{ClusterExperiment}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotFeatureBoxplot.R
\name{plotFeatureBoxplot}
\alias{plotFeatureBoxplot}
\alias{plotFeatureBoxplot,ClusterExperiment,character-method}
\alias{plotFeatureBoxplot,ClusterExperiment,numeric-method}
\title{Plot boxplot of feature values by cluster}
\usage{
\S4method{plotFeatureBoxplot}{ClusterExperiment,character}(object, feature, whichCluster = "primary", ...)

\S4method{plotFeatureBoxplot}{ClusterExperiment,numeric}(
  object,
  feature,
  whichCluster = "primary",
  plotUnassigned = FALSE,
  unassignedColor = NULL,
  missingColor = NULL,
  main = NULL,
  whichAssay = 1,
  ...
)
}
\arguments{
\item{object}{a ClusterExperiment object}

\item{feature}{identification of feature to plot, either row name or index}

\item{whichCluster}{argument that can be a single numeric or character value
indicating the \emph{single} clustering to be used. Giving values that result in more than one clustering will result in an error. See details of \code{\link{getClusterIndex}}.}

\item{...}{arguments passed to \code{\link{boxplot}}}

\item{plotUnassigned}{whether to plot the unassigned samples as a cluster
(either -1 or -2)}

\item{unassignedColor}{If not NULL, should be character value giving the
color for unassigned (-2) samples (overrides \code{clusterLegend}) default.}

\item{missingColor}{If not NULL, should be character value giving the color
for missing (-2) samples (overrides \code{clusterLegend}) default.}

\item{main}{title of plot. If NULL, given default title.}

\item{whichAssay}{numeric or character specifying which assay to use. See
\code{\link[SummarizedExperiment]{assay}} for details.}
}
\value{
A plot is created. The output of boxplot is returned (see
\code{\link{boxplot}}), with additional elements \code{colors} and 
 \code{clusterIds} that gives the colors and internal ids that match 
 each boxplot (pulled from \code{clusterLegend} but in the order of plot)
}
\description{
Plot a boxplot of the (transformed) values for a particular
  gene, separated by cluster
}
\examples{
data(simData)
#Create a ClusterExperiment object
cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reducedDim="PCA",
   clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
   removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)
#give names to the clusters
cl<-renameClusters(cl, whichCluster=1, 
   value=letters[1:nClusters(cl)[1]])
plotFeatureBoxplot(cl,feature=1)
}
\seealso{
\code{\link{boxplot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{search_pairs}
\alias{search_pairs}
\title{Search pairs of samples that co-cluster across subsamples}
\usage{
search_pairs(clusterings)
}
\arguments{
\item{clusterings}{a matrix with the cluster labels}
}
\value{
A matrix with the co-clusters, but only the lower triangle
is populated.
}
\description{
Assume that our input is a matrix, with N columns and B rows
(the number of subsamples), storing integers -- the cluster labels.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotFeatureScatter.R
\name{plotFeatureScatter}
\alias{plotFeatureScatter}
\alias{plotFeatureScatter,ClusterExperiment,character-method}
\alias{plotFeatureScatter,ClusterExperiment,numeric-method}
\title{Plot scatter plot of feature values colored by cluster}
\usage{
\S4method{plotFeatureScatter}{ClusterExperiment,character}(object, features, ...)

\S4method{plotFeatureScatter}{ClusterExperiment,numeric}(
  object,
  features,
  whichCluster = "primary",
  plotUnassigned = TRUE,
  unassignedColor = "grey",
  missingColor = "white",
  whichAssay = 1,
  legendLocation = NA,
  jitterFactor = NA,
  ...
)
}
\arguments{
\item{object}{a ClusterExperiment object}

\item{features}{the indices of the features (either numeric or character 
matching rownames of object) to be plotted.}

\item{...}{arguments passed to \code{\link{boxplot}}}

\item{whichCluster}{argument that can be a single numeric or character value
indicating the \emph{single} clustering to be used. Giving values that result in more than one clustering will result in an error. See details of \code{\link{getClusterIndex}}.}

\item{plotUnassigned}{whether to plot the unassigned samples as a cluster
(either -1 or -2)}

\item{unassignedColor}{If not NULL, should be character value giving the
color for unassigned (-2) samples (overrides \code{clusterLegend}) default.}

\item{missingColor}{If not NULL, should be character value giving the color
for missing (-2) samples (overrides \code{clusterLegend}) default.}

\item{whichAssay}{numeric or character specifying which assay to use. See
\code{\link[SummarizedExperiment]{assay}} for details.}

\item{legendLocation}{character value passed to \code{location} argument of 
\code{plotClusterLegend} indicating where to put the legend. If NA, legend 
will not be plotted.}

\item{jitterFactor}{numeric. If NA, no jittering is done. Otherwise, passed to 
\code{factor} of function \code{\link[base]{jitter}}
 (useful for low counts)}
}
\value{
returns invisibly the results of \code{\link[graphics]{pairs}} or \code{\link[graphics]{plot}} command.
}
\description{
Plot a scatter plot of the (transformed) values for a set of
  gene expression values, colored by cluster
}
\examples{
data(simData)
#Create a ClusterExperiment object
cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reducedDim="PCA",
   clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
   removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)
#give names to the clusters
cl<-renameClusters(cl, whichCluster=1, 
   value=letters[1:nClusters(cl)[1]])
plotFeatureScatter(cl,feature=1:2,pch=19,legendLocation="topright")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataCreation.R
\docType{data}
\name{rsecFluidigm}
\alias{rsecFluidigm}
\alias{makeRsecFluidigmObject}
\title{Documentation of rsecFluidigm object}
\format{
\code{rsecFluidigm} is a \code{ClusterExperiment} object, the result
  of running \code{\link{RSEC}} on fluidigm data described in vignette and
  available in the \code{scRNAseq} package.
}
\usage{
makeRsecFluidigmObject(object)
}
\arguments{
\item{object}{object given to functions}
}
\description{
Documentation of the creation of rsecFluidigm, result of RSEC
  run on fluidigm data for vignette
}
\details{
The functions \code{makeRsecFluidigmObject} and 
  \code{checkRsecFluidigmObject} are helper functions whose sole purpose is 
  to create \code{rsecFluidigm} and check that the results are the same as 
  expected. \code{makeRsecFluidigmObject} also serves as documentation of the
  specific RSEC call that was made to create the \code{rsecFluidigm} object, 
  as well as filtering and normalization of the fluidigm data.
  The purpose of making them functions is internal, to help more easily
  mantain and check if changes to the package have affected the results.
}
\examples{
# see code used create rsecFluidigm 
# (print out the function)
makeRsecFluidigmObject
#code actualy run to create rsecFluidigm:
\dontrun{
library(clusterExperiment)
data(fluidigmData)
data(fluidigmColData)
se<-SummarizedExperiment(assays=fluidigmData, colData=fluidigmColData)
RNGversion("3.5.0")
rsecFluidigm<-makeRsecFluidigmObject(se)
# Internal function for checking got correct results...
clusterExperiment:::checkRsecFluidigmObject(rsecFluidigm)
usethis::use_data(rsecFluidigm,overwrite=FALSE)
}
}
\seealso{
\code{\link[scRNAseq]{fluidigm}}.
}
\author{
Elizabeth Purdom \email{epurdom@stat.berkeley.edu}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataCreation.R
\docType{data}
\name{fluidigmData}
\alias{fluidigmData}
\alias{fluidigmColData}
\title{Subset of fluidigm data}
\format{
subset of fluidigm data used in vignette
package.
}
\description{
Subset of fluidigm data
}
\details{
\code{fluidigmData} and \code{fluidigmColData} are portions of the \code{fluidigm} data distributed in the package \code{scRNAseq} package. We have subsetted to only the cells sequenced under high depth, and limited our selves to only two of the four gene estimates provided by \code{scRNAseq} ("tophat_counts" and "rsem_tpm").
}
\examples{
#code used to create objects:
\dontrun{
library(scRNAseq)
if(packageVersion("scRNAseq")>="1.11.0") fluidigm <- ReprocessedFluidigmData() else data(fluidigm)
fluidSubset<- fluidigm[,colData(fluidigm)[,"Coverage_Type"]=="High"]
fluidigmData<-assays(fluidSubset)[c("tophat_counts","rsem_tpm")]
fluidigmColData<-as.data.frame(colData(fluidSubset))
usethis::use_data(fluidigmData, fluidigmColData, overwrite=FALSE)
}
}
\seealso{
\code{\link[scRNAseq]{fluidigm}}
}
\author{
Elizabeth Purdom \email{epurdom@stat.berkeley.edu}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addClusterings.R
\name{addClusterings}
\alias{addClusterings}
\alias{removeClusterings}
\alias{addClusterings,ClusterExperiment,matrix-method}
\alias{addClusterings,ClusterExperiment,ClusterExperiment-method}
\alias{addClusterings,ClusterExperiment,vector-method}
\title{Add clusterings to ClusterExperiment object}
\usage{
\S4method{addClusterings}{ClusterExperiment,matrix}(
  x,
  y,
  clusterTypes = "User",
  clusterLabels = NULL,
  clusterLegend = NULL
)

\S4method{addClusterings}{ClusterExperiment,ClusterExperiment}(x, y, transferFrom = c("x", "y"), mergeCEObjects = FALSE)

\S4method{addClusterings}{ClusterExperiment,vector}(x, y, makePrimary = FALSE, ...)
}
\arguments{
\item{x}{a ClusterExperiment object}

\item{y}{additional clusters to add to x. Can be a ClusterExperiment object
or a matrix/vector of clusters.}

\item{clusterTypes}{a string describing the nature of the clustering. The
values `clusterSingle`, `clusterMany`, `mergeClusters`, `makeConsensus` are
reserved for the clustering coming from the package workflow and should not
be used when creating a new object with the constructor.}

\item{clusterLabels}{label(s) for the clusters being added. If \code{y} a
matrix, the column names of that matrix will be used by default, if
\code{clusterLabels} is not given.}

\item{clusterLegend}{a list giving the cluster legend for the clusters added.}

\item{transferFrom}{If x and y are both \code{ClusterExperiment} objects
indicates from which object the clustering info should be taken (regarding merging, dendrogram, etc). Does not affect the order of the clusterings, which will always be the clusterings of x, followed by those of y (along with slots `clusterType`, `clusterInfo`, `clusterLegend`)}

\item{mergeCEObjects}{logical If x and y are both \code{ClusterExperiment} objects indicates as to whether should try to grab in the information missing from x from y (or vice versa if transferFrom=y).}

\item{makePrimary}{whether to make the added cluster the primary cluster
(only relevant if \code{y} is a vector)}

\item{...}{For \code{addClusterings}, passed to signature
\code{ClusterExperiment,matrix}. For \code{[} (subsetting), passed to
\code{SingleCellExperiment} subsetting function.}
}
\value{
A \code{ClusterExperiment} object.
}
\description{
Function for adding new clusterings in form of vector (single
  cluster) or matrix (multiple clusterings) to an existing ClusterExperiment
  object
}
\details{
addClusterings adds y to x, and is thus not symmetric in the two
  arguments. In particular, the \code{primaryCluster}, all of the dendrogram
  information, the merge information, \code{coClustering}, and 
  \code{orderSamples} are all kept from
  the x object, even if y is a ClusterExperiment.
}
\examples{
data(simData)

cl1 <- clusterSingle(simData, subsample=FALSE,
sequential=FALSE, mainClusterArgs=list(clusterArgs=list(k=3), 
clusterFunction="pam"))
cl2 <- clusterSingle(simData, subsample=FALSE,
sequential=FALSE, mainClusterArgs=list(clusterArgs=list(k=3), 
clusterFunction="pam"))

addClusterings(cl1, cl2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllHelperDendro.R
\name{clusterDendrogram}
\alias{clusterDendrogram}
\alias{clusterDendrogram,ClusterExperiment-method}
\alias{sampleDendrogram,ClusterExperiment-method}
\alias{sampleDendrogram}
\alias{nInternalNodes,ClusterExperiment-method}
\alias{nInternalNodes}
\alias{nTips,ClusterExperiment-method}
\alias{nTips}
\alias{nNodes,ClusterExperiment-method}
\alias{nNodes}
\alias{nodeLabels<-,ClusterExperiment-method}
\alias{nodeLabels<-}
\alias{checkDendrogram,ClusterExperiment,phylo4d,phylo4d-method}
\alias{checkDendrogram}
\alias{nodeLabels,ClusterExperiment-method}
\alias{nodeLabels}
\alias{nodeIds,ClusterExperiment-method}
\alias{nodeIds}
\alias{convertToDendrogram,ClusterExperiment-method}
\alias{convertToDendrogram}
\title{Accessing and manipulating the dendrograms}
\usage{
\S4method{clusterDendrogram}{ClusterExperiment}(x)

\S4method{sampleDendrogram}{ClusterExperiment}(x)

\S4method{nInternalNodes}{ClusterExperiment}(x)

\S4method{nTips}{ClusterExperiment}(x)

\S4method{nNodes}{ClusterExperiment}(x)

\S4method{nodeLabels}{ClusterExperiment}(x, ...) <- value

\S4method{checkDendrogram}{ClusterExperiment,phylo4d,phylo4d}(x, dendroCluster, dendroSample, whichCluster = "dendro")

\S4method{nodeLabels}{ClusterExperiment}(x)

\S4method{nodeIds}{ClusterExperiment}(x, type = c("all", "internal", "tip"))

\S4method{convertToDendrogram}{ClusterExperiment}(x)
}
\arguments{
\item{x}{a ClusterExperiment object}

\item{...}{additional options passed to \code{nodeLabels<-} (ignored)}

\item{value}{replacement value for \code{nodeLabels}. See details.}

\item{dendroCluster}{a \code{phylo4d} to be check as for being cluster hierarchy}

\item{dendroSample}{a \code{phylo4d} to be check as for being cluster hierarchy}

\item{whichCluster}{argument that can be a single numeric or character value
indicating the \emph{single} clustering to be used. Giving values that result in more than one clustering will result in an error. See details of \code{\link{getClusterIndex}}.}

\item{type}{the type of node to return results from. One of "all",
"internal", and "tip".}
}
\value{
\code{clusterDendrogram} returns the dendrogram describing the
  clustering hierarchy.

\code{sampleDendrogram} returns the dendrogram that expands the
  cluster hierarchy to the samples.

\code{nInternalNodes} returns the number of \emph{internal} nodes of
  the cluster hierarchy.

\code{nTips} returns the number of tips of the cluster hierarchy
  (same as number of non-negative clusters in the dendrogram clustering)

\code{nNodes} returns the number of total nodes of the cluster hierarchy

\code{nodeLabels<-} sets the node labels of the \emph{cluster}
dendrogram

\code{checkClusterDendrogram} checks if a \code{phylo4d} objects are
  valid for the cluster and sample dendrogram slots of the given
  \code{ClusterExperiment} object. Returns \code{TRUE} if there are no
  problems. Otherwise creates error.

\code{nodeLabels} returns the node labels of the \emph{cluster}
dendrogram

\code{nodeIds} returns the internal (permanent) node ids of the
  \emph{cluster} dendrogram

\code{convertToDendrogram} returns the sample dendrogram converted to
  a \code{\link[stats]{dendrogram}} class.
}
\description{
These functions are for accessing and manipulating the
  dendrograms stored in a \code{ClusterExperiment} object. We also document
  the required format of these dendrograms here.
}
\details{
Two dendrograms are stored in a \code{ClusterExperiment} object. One
  is a dendrogram that describes the hierarchy between the clusters
  (\code{@dendro_clusters}), and the other is a dendrogram that extends that
  hierarchy to include the clusters (\code{@dendro_samples}). The clustering
  that is used to make these hierarchies is saved in as well
  (\code{@dendro_index})

The dendrograms stored in a \code{ClusterExperiment} object are
  required to be a \code{\link[phylobase]{phylo4d-class}} from the package
  \code{phylobase} (which uses the basic format of the S3 class
  \code{\link[ape]{phylo}} in the \code{ape} package to store the edges;
  \code{phylobase} makes it a S4 class with some useful helpers). This class
  allows storage of a data.frame of information corresponding to information
  on each node (see \code{\link[phylobase]{tdata}}).

Additional requirements are made of these dendrograms to be a valid
  for the slots of the \code{ClusterExperiment} class, described below,
  regarding the data that must be stored with it and the labels which can be
  assigned. Possible dendrograms can be checked for validity with the
  function \code{checkDendrogram}. The reason for the restrictions on the
  labels is so as to not duplicate storage of the names, see below
  descriptions for where to save user-defined names.

\itemize{ 
\item{Labels}{The cluster dendrogram can only have labels
  on the \emph{internal} nodes. Labels on the internal nodes of the cluster
  dendrogram can be set by the user (the function \code{nodeLabels<-} is
  defined to work on a \code{ClusterExperiment} object to make this easy).
  The tips of the cluster dendrogram, corresponding to the clusters, cannot
  have labels; users can set the labels (e.g. for plotting, etc) in the
  \code{\link{clusterLegend}} slot using the function
  \code{\link{renameClusters}}.} 
\item{Data}{ The cluster hierarchy must have
  data stored with it that has the following columns (additional ones are
  allowed):
  \itemize{ 
  \item{NodeId}{The permanent node id for the node. Must
  be of the format "NodeIdX" where "X" is a integer.} 
  \item{Position}{The
  type of node, in terms of its position. The internal nodes should have the
  values "cluster hierarchy node" while the tips should have "cluster
  hierarchy tip".} 
  \item{ClusterIdDendro}{Only for tips of dendrogram, should
  have the id that corresponds to its cluster in the clustering of the
  @dendro_index. Of the form "ClusterIdX", where "X" is the internal cluster
  id (see \code{\link{clusterLegend}}). Internal nodes should have NA
  values.} 
  \item{ClusterIdMerge}{The id that corresponds to the cluster in
  the clustering of the @merge_index, if it exists. Of the form "ClusterIdX",
  where "X" is the internal cluster id (see \code{\link{clusterLegend}}} } }
  }

\itemize{
\item{Labels}{The sample dendrogram is not allowed to have ANY labels. The
names for those nodes that correspond to the cluster hierarchy will be pulled
from the names in the cluster hierarchy for plotting, etc. and should be set
there (see above). Sample names for the tips of the tree will be pulled from
\code{colnames} of the object and should be set there. }
\item{Data}{ 
The cluster hierarchy must have data stored with it that has the following
columns (additional ones are allowed):
 \itemize{
  \item{NodeId}{For those nodes that correspond to a node in the cluster
  hierarchy, should have its permanent node id in this column. Other nodes
  should be NA.}
  \item{Position}{The type of node, in terms of its position. The internal
  nodes should have the values "cluster hierarchy node" while the tips should
  have "cluster hierarchy tip".}
  \item{SampleIndex}{Only for tips of dendrogram, the index of the sample at
  that tip to the samples in the object.}
 }
} 
}

For setting the node labels of the cluster dendrogram via 
\code{nodeLabels<-}, the replacement value has to have names that match the 
internal ids of the cluster dendrogram (the \code{NodeId} column).
}
\section{The Stored Dendrograms}{
NA
}

\section{Cluster Hierarchy}{
NA
}

\section{Sample Hierarchy}{
NA
}

\section{Helper Functions}{
NA
}

\examples{
data(rsecFluidigm)
# retrieve the dendrogram of the clusters:
head(clusterDendrogram(rsecFluidigm),5)
# retrieve the dendrogram of the samples:
head(sampleDendrogram(rsecFluidigm),5)
# Return # internal nodes from cluster hierarchy
nInternalNodes(rsecFluidigm)
# Return # tips from cluster hierarchy (i.e. # clusters)
nTips(rsecFluidigm)
# Return internal node ids
nodeIds(rsecFluidigm,type="internal")
# Labels assigned to internal nodes
nodeLabels(rsecFluidigm)
# Assign new labels to the internal nodes of the cluster hierarchy
l1<-paste("A", 1:nInternalNodes(rsecFluidigm),sep=":")
names(l1)<-nodeIds(rsecFluidigm,type="internal")
nodeLabels(rsecFluidigm)<-l1
nodeLabels(rsecFluidigm)
}
\seealso{
\code{\link{makeDendrogram}},
  \code{\link[phylobase]{phylo4d-class}}, \code{\link[ape]{phylo}}

\code{\link[stats]{dendrogram}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotHeatmap.R
\docType{methods}
\name{plotHeatmap}
\alias{plotHeatmap}
\alias{plotHeatmap,SingleCellExperiment-method}
\alias{plotHeatmap,SummarizedExperiment-method}
\alias{plotHeatmap,table-method}
\alias{plotHeatmap,ClusterExperiment-method}
\alias{plotHeatmap,data.frame-method}
\alias{plotHeatmap,ExpressionSet-method}
\alias{plotHeatmap,matrixOrHDF5-method}
\alias{plotCoClustering,ClusterExperiment-method}
\alias{plotCoClustering}
\title{Heatmap for showing clustering results and more}
\usage{
\S4method{plotHeatmap}{SingleCellExperiment}(data, isCount = FALSE, transFun = NULL, ...)

\S4method{plotHeatmap}{SummarizedExperiment}(data, isCount = FALSE, transFun = NULL, ...)

\S4method{plotHeatmap}{table}(data, ...)

\S4method{plotHeatmap}{ClusterExperiment}(
  data,
  clusterSamplesData = c("dendrogramValue", "hclust", "orderSamplesValue",
    "primaryCluster"),
  clusterFeaturesData = "var",
  nFeatures = NA,
  visualizeData = c("transformed", "centeredAndScaled", "original"),
  whichClusters = c("primary", "workflow", "all", "none"),
  colData = NULL,
  clusterFeatures = TRUE,
  nBlankLines = 2,
  colorScale,
  whichAssay = 1,
  ...
)

\S4method{plotHeatmap}{data.frame}(data, ...)

\S4method{plotHeatmap}{ExpressionSet}(data, ...)

\S4method{plotHeatmap}{matrixOrHDF5}(
  data,
  colData = NULL,
  clusterSamplesData = NULL,
  clusterFeaturesData = NULL,
  whColDataCont = NULL,
  clusterSamples = TRUE,
  showSampleNames = FALSE,
  clusterFeatures = TRUE,
  showFeatureNames = FALSE,
  colorScale = seqPal5,
  clusterLegend = NULL,
  alignColData = FALSE,
  unassignedColor = "white",
  missingColor = "grey",
  breaks = NA,
  symmetricBreaks = FALSE,
  capBreaksLegend = FALSE,
  isSymmetric = FALSE,
  overRideClusterLimit = FALSE,
  plot = TRUE,
  labelTracks = TRUE,
  ...
)

\S4method{plotCoClustering}{ClusterExperiment}(data, invert, saveDistance = FALSE, ...)
}
\arguments{
\item{data}{data to use to determine the heatmap. Can be a matrix,
\code{\link{ClusterExperiment}},
\code{\link[SingleCellExperiment]{SingleCellExperiment}} or
\code{\link[SummarizedExperiment]{SummarizedExperiment}} object. The
interpretation of parameters depends on the type of the input to
\code{data}.}

\item{isCount}{if \code{transFun=NULL}, then \code{isCount=TRUE} will
determine the transformation as defined by \code{function(x){log2(x+1)}},
and \code{isCount=FALSE} will give a transformation function
\code{function(x){x}}. Ignored if \code{transFun=NULL}. If object is of
class \code{ClusterExperiment}, the stored transformation will be used and
giving this parameter will result in an error.}

\item{transFun}{a transformation function to be applied to the data. If the
transformation applied to the data creates an error or NA values, then the
function will throw an error. If object is of class
\code{ClusterExperiment}, the stored transformation will be used and giving
this parameter will result in an error.}

\item{...}{for signature \code{matrix}, arguments passed to \code{aheatmap}.
For the other signatures, passed to the method for signature \code{matrix}.
Not all arguments can be passed to \code{aheatmap} effectively, see
details.}

\item{clusterSamplesData}{If \code{data} is a matrix,
 \code{clusterSamplesData} is either a matrix that will be used by
 \code{hclust} to define the hiearchical clustering of samples (e.g.
 normalized data) or a pre-existing dendrogram (of class
\code{\link[stats]{dendrogram}}) that clusters the samples. If
 \code{data} is a \code{ClusterExperiment} object, \code{clusterSamplesData}
 should be either character or integers or logical which indicates how (and
 whether) the samples should be clustered (or gives indices of the order for
 the samples). See details.}

\item{clusterFeaturesData}{If \code{data} is a matrix, either a matrix that
will be used in \code{hclust} to define the hiearchical clustering of
features (e.g. normalized data) or a pre-existing dendrogram that clusters
the features. If \code{data} is a \code{ClusterExperiment} object, the
input should be either character or integers indicating which features
should be used (see details).}

\item{nFeatures}{integer indicating how many features should be used (if
\code{clusterFeaturesData} is 'var' or 'PCA').}

\item{visualizeData}{either a character string, indicating what form of the
data should be used for visualizing the data (i.e. for making the
color-scale), or a data.frame/matrix with same number of samples as
\code{assay(data)}. If a new data.frame/matrix, any character arguments to
clusterFeaturesData will be ignored.}

\item{whichClusters}{argument that can be either numeric or character vector
indicating the clusterings to be used. See details of \code{\link{getClusterIndex}}.}

\item{colData}{If input to \code{data} is either a
\code{\link{ClusterExperiment}},or \code{SummarizedExperiment} object or
\code{SingleCellExperiment}, then \code{colData} must index the
colData stored as a \code{DataFrame} in \code{colData} slot of the
object. Whether that data is continuous or not will be determined by the
properties of \code{colData} (no user input is needed). If input to
\code{data} is matrix, \code{colData} is a matrix of additional data on
the samples to show above heatmap. In this case, unless indicated by
\code{whColDataCont}, \code{colData} will be converted into factors,
even if numeric. ``-1'' indicates the sample was not assigned to a cluster
and gets color `unassignedColor' and ``-2`` gets the color 'missingColor'.}

\item{clusterFeatures}{Logical as to whether to do hiearchical clustering of
features (if FALSE, any input to clusterFeaturesData is ignored).}

\item{nBlankLines}{Only applicable if input is \code{ClusterExperiment} object. Indicates the number of lines to put between groups of features if \code{clusterFeaturesData} gives groups of genes (see details and \code{\link{makeBlankData}}).}

\item{colorScale}{palette of colors for the color scale of the heatmap.}

\item{whichAssay}{numeric or character specifying which assay to use. See
\code{\link[SummarizedExperiment]{assay}} for details.}

\item{whColDataCont}{Which of the \code{colData} columns are continuous
and should not be converted to counts. \code{NULL} indicates no additional
\code{colData}. Only used if \code{data} input is matrix.}

\item{clusterSamples}{Logical as to whether to do hierarchical clustering of
cells (if FALSE, any input to clusterSamplesData is ignored).}

\item{showSampleNames}{Logical as to whether show sample names.}

\item{showFeatureNames}{Logical as to whether show feature names.}

\item{clusterLegend}{Assignment of colors to the clusters. If \code{NULL},
\code{colData} columns will be assigned colors internally. See details
for more.}

\item{alignColData}{Logical as to whether should align the colors of the
\code{colData} (only if \code{clusterLegend} not given and
\code{colData} is not \code{NULL}).}

\item{unassignedColor}{color assigned to cluster values of '-1'
("unassigned").}

\item{missingColor}{color assigned to cluster values of '-2' ("missing").}

\item{breaks}{Either a vector of breaks (should be equal to length 52), or a
number between 0 and 1, indicating that the breaks should be equally spaced
(based on the range in the data) upto the `breaks' quantile, see
\code{\link{setBreaks}}}

\item{symmetricBreaks}{logical as to whether the breaks created for the color
scale should be symmetrical around 0}

\item{capBreaksLegend}{logical as to whether the legend for the breaks should
be capped. Only relevant if \code{breaks} is a value < 1, in which case if
\code{capBreaksLegend=TRUE}, only the values between the quantiles
requested will show in the color scale legend.}

\item{isSymmetric}{logical. if TRUE indicates that the input matrix is
symmetric. Useful when plotting a co-clustering matrix or other sample by
sample matrices (e.g., correlation).}

\item{overRideClusterLimit}{logical. Whether to override the internal limit
that only allows 10 clusterings/annotations. If overridden, may result in
incomprehensible errors from \code{aheatmap}. Only override this if you
have a very large plotting device and want to see if \code{aheatmap} can
render it.}

\item{plot}{logical indicating whether to plot the heatmap. Mainly useful for
package mantaince to avoid calls to aheatmap on unit tests that take a long
time.}

\item{labelTracks}{logical, whether to put labels next to the color tracks 
corresponding to the colData.}

\item{invert}{logical determining whether the coClustering matrix should be
inverted to be 1-coClustering for plotting. By default, if the diagonal
elements are all zero, invert=TRUE, and otherwise invert=FALSE. If
coClustering matrix is not a 0-1 matrix (e.g. if equal to a distance matrix
output from \code{\link{clusterSingle}}, then the user should manually set
this parameter to FALSE.)}

\item{saveDistance}{logical. When the \code{coClustering} slot contains
indices of the clusterings or a NxB set of clusterings, the hamming
distance will be calculated before running the plot. This argument
determines whether the \code{ClusterExperiment} object with that distance
in \code{coClustering} slot should be returned (so as to avoid
re-calculating it in the future) or not.}
}
\value{
Returns (invisibly) a list with elements
\itemize{
\item{\code{aheatmapOut}}{ The output from the final call of
\code{\link[NMF]{aheatmap}}.}
\item{\code{colData}}{ the annotation data.frame given to the argument
\code{annCol} in \code{aheatmap}.}
\item{\code{clusterLegend}}{ the annotation colors given to the argument
\code{annColors} \code{aheatmap}.}
\item{\code{breaks}}{ The breaks used for \code{aheatmap}, after adjusting
for quantile.}
}
}
\description{
Make heatmap with color scale from one matrix and hiearchical
  clustering of samples/features from another. Also built in functionality
  for showing the clusterings with the heatmap. Builds on
  \code{\link[NMF]{aheatmap}} function of \code{NMF} package.
}
\details{
The plotHeatmap function calls \code{\link[NMF]{aheatmap}} to draw
  the heatmap. The main points of \code{plotHeatmap} are to 1) allow for
  different matrix inputs, separating out the color scale visualization and
  the clustering of the samples/features. 2) to visualize the clusters and
  meta data with the heatmap. The intended use case is to allow the user to
  visualize the original count scale of the data (on the log-scale), but
  create the hierarchical clustering on another, more appropriate dataset for
  clustering, such as normalized data. Similarly, some of the palettes in the
  package were developed assuming that the visualization might be on
  unscaled/uncentered data, rather than the residual from the mean of the
  gene, and thus palettes need to take on a greater range of relevant values
  so as to show meaningful comparisons with genes on very different scales.

If \code{data} is a \code{ClusterExperiment} object,
  \code{visualizeData} indicates what kind of transformation should be done
  to \code{assay(data)} for calculating the color scale. The features will be
  clustered based on these data as well. A different data.frame or matrix can
  be given for the visualization. For example, if the
  \code{ClusterExperiment} object contains normalized data, but the user
  wishes that the color scale be based on the log-counts for easier
  interpretation, \code{visualizeData} could be set to be the
  \code{log2(counts + 1)}.

If \code{data} is a \code{ClusterExperiment} object,
  \code{clusterSamplesData} can be used to indicate the type of clustering
  for the samples. If equal to `dendrogramValue` the dendrogram stored in
  \code{data} will be used; if dendrogram is missing, a new one will be
  created based on the \code{primaryCluster} of data using
  \code{\link{makeDendrogram}}, assuming no errors are created (if errors are
  created, then \code{clusterSamplesData} will be set to "primaryCluster").
  If \code{clusterSamplesData} is equal to "hclust", then standard
  hierachical clustering of the transformed data will be used. If
  \code{clusterSamplesData} is equal to 'orderSamplesValue' no clustering of
  the samples will be done, and instead the samples will be ordered as in the
  slot \code{orderSamples} of \code{data}. If \code{clusterSamplesData} is
  equal to 'primaryCluster', again no clustering will be done, and instead
  the samples will be ordered based on grouping the samples to match the
  primaryCluster of \code{data}; however, if the primaryCluster of
  \code{data} is only one cluster or consists soley of -1/-2 values,
  \code{clusterSamplesData} will be set to "hclust". If
  \code{clusterSamplesData}  is not a character value,
  \code{clusterSamplesData} can be a integer valued vector giving the order
  of the samples.

If \code{data} is a matrix, then \code{colData} is a data.frame
  of annotation data to be plotted above the heatmap and
  \code{whColDataCont} gives the index of the column(s) of this dataset
  that should be consider continuous. Otherwise the annotation data for
  \code{colData} will be forced into a factor (which will be nonsensical
  for continous data). If \code{data} is a \code{ClusterExperiment} object,
  \code{colData} should refer to a index or column name of the
  \code{colData} slot of \code{data}. In this case \code{colData} will be
  added to any choices of clusterings chosen by the \code{whichClusters}
  argument (if any). If both clusterings and sample data are chosen, the
  clusterings will be shown closest to data (i.e. on bottom).

If \code{data} is a \code{ClusterExperiment} object,
  \code{clusterFeaturesData} is not a dataset, but instead indicates which
  features should be shown in the heatmap. In this case
  \code{clusterFeatures} can be one of the following: \itemize{ \item{"all"}{
  All rows/genes will be shown} \item{character giving dimensionality
  reduction}{Should match one of values saved in \code{reducedDims} slot or a
  builtin function in \code{listBuiltInReducedDims()}. \code{nFeatures} then
  gives the number of dimensions to show. The heatmap will then be of the
  dimension reduction vectors} \item{character giving filtering}{ Should
  match one of values saved in \code{filterStats} slot or a builtin function
  in \code{listBuiltInFilterStats()}. \code{nFeatures} gives the number of
  genes to keep after filtering.} \item{character giving gene/row names}{ }
  \item{vector of integers giving row indices}{ } \item{a list of indices or
  rownames}{This is used to indicate that the features should be grouped
  according to the elements of the list, with blank (white) space between
  them (see \code{\link{makeBlankData}} for more details). In this case, no
  clustering is done of the features.} }

If \code{breaks} is a numeric value between 0 and 1, then
  \code{breaks} is assumed to indicate the upper quantile (on the log scale)
  at which the heatmap color scale should stop. For example, if
  \code{breaks=0.9}, then the breaks will evenly spaced up until the 0.9
  upper quantile of \code{data}, and then all values after the 0.9 quantile
  will be absorbed by the upper-most color bin. This can help to reduce the
  visual impact of a few highly expressed genes (features).

Note that plotHeatmap calls \code{\link[NMF]{aheatmap}} under the
  hood. This allows you to plot multiple heatmaps via
  \code{par(mfrow=c(2,2))}, etc. However, the dendrograms do not resize if
  you change the size of your plot window in an interactive session of R
  (this might be a problem for RStudio if you want to pop it out into a large
  window...). Also, plotting to a pdf adds a blank page; see help pages of
  \code{\link[NMF]{aheatmap}} for how to turn this off.

\code{clusterLegend} takes the place of argument \code{annColors}
  from \code{aheatmap} for giving colors to the annotation on the heatmap.
  \code{clusterLegend} should be list of length equal to
  \code{ncol(colData)} with names equal to the colnames of
  \code{colData}. Each element of the list should be a either the format
  requested by \code{\link[NMF]{aheatmap}} (a vector of colors with names
  corresponding to the levels of the column of \code{colData}), or should
  be format of the \code{clusterLegend} slot in a \code{ClusterExperiment}
  object. Color assignments to the rows/genes should also be passed via
  \code{clusterLegend} (assuming \code{annRow} is an argument passed to
  \code{...}). If \code{clusterFeaturesData} is a \emph{named} list
  describing groupings of genes then the colors for those groups can be given
  in \code{clusterLegend} under the name "Gene Group".

If you have a factor with many levels, it is important to note that
  \code{\link[NMF]{aheatmap}} does not recycle colors across factors in the
  \code{colData}, and in fact runs out of colors and the remaining levels
  get the color white. Thus if you have many factors or many levels in those
  factors, you should set their colors via \code{clusterLegend}.

Many arguments can be passed on to \code{aheatmap}, however, some are set
  internally by \code{plotHeatmap.} In particular, setting the values of
  \code{Rowv} or \code{Colv} will cause errors. \code{color} in
  \code{aheatmap} is replaced by \code{colorScale} in \code{plotHeatmap.} The
  \code{annCol} to give annotation to the samples is replaced by the
  \code{colData}; moreover, the \code{annColors} option in \code{aheatmap}
  will also be set internally to give more vibrant colors than the default in
  \code{aheatmap} (for \code{ClusterExperiment} objects, these values can
  also be set in the \code{clusterLegend} slot ). Other options should be
  passed on to \code{aheatmap}, though they have not been all tested. Useful options
 include \code{treeheight=0} to suppress plotting of the dendrograms, 
 \code{annLegend=FALSE} to suppress the legend of factors shown beside columns/rows, 
 and \code{cexRow=0} or \code{cexCol=0} to suppress plotting of row/column labels.

\code{plotCoClustering} is a convenience function to plot the
  heatmap of the co-clustering distance matrix from the \code{coClustering}
  slot of a \code{ClusterExperiment} object (either by calculating the
  hamming distance of the clusterings stored in the \code{coClustering} slot,
  or the distance stored in the \code{coClustering} slot if it has already
  been calculated.
}
\examples{
\dontrun{
data(simData)

cl <- rep(1:3,each=100)
cl2 <- cl
changeAssign <- sample(1:length(cl), 80)
cl2[changeAssign] <- sample(cl[changeAssign])
ce <- ClusterExperiment(simCount, cl2, transformation=function(x){log2(x+1)})

#simple, minimal, example. Show counts, but cluster on underlying means
plotHeatmap(ce)

#assign cluster colors
colors <- bigPalette[20:23]
names(colors) <- 1:3
plotHeatmap(data=simCount, clusterSamplesData=simData,
colData=data.frame(cl), clusterLegend=list(colors))

#show two different clusters
anno <- data.frame(cluster1=cl, cluster2=cl2)
out <- plotHeatmap(simData, colData=anno)

#return the values to see format for giving colors to the annotations
out$clusterLegend

#assign colors to the clusters based on plotClusters algorithm
plotHeatmap(simData, colData=anno, alignColData=TRUE)

#assign colors manually
annoColors <- list(cluster1=c("black", "red", "green"),
cluster2=c("blue","purple","yellow"))

plotHeatmap(simData, colData=anno, clusterLegend=annoColors)

#give a continuous valued -- need to indicate columns
anno2 <- cbind(anno, Cont=c(rnorm(100, 0), rnorm(100, 2), rnorm(100, 3)))
plotHeatmap(simData, colData=anno2, whColDataCont=3)

#compare changing breaks quantile on visual effect
par(mfrow=c(2,2))
plotHeatmap(simData, colorScale=seqPal1, breaks=1, main="Full length")
plotHeatmap(simData,colorScale=seqPal1, breaks=.99, main="0.99 Quantile Upper
Limit")
plotHeatmap(simData,colorScale=seqPal1, breaks=.95, main="0.95 Quantile Upper
Limit")
plotHeatmap(simData, colorScale=seqPal1, breaks=.90, main="0.90 Quantile
Upper Limit")
}

}
\seealso{
\code{\link[NMF]{aheatmap}}, \code{\link{makeBlankData}}, \code{\link{showHeatmapPalettes}}, \code{\link{makeDendrogram}}, \code{\link[stats]{dendrogram}}
}
\author{
Elizabeth Purdom
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset.R
\name{subset}
\alias{subset}
\alias{removeClusterings,ClusterExperiment-method}
\alias{removeClusters,ClusterExperiment-method}
\alias{removeClusters}
\alias{[,ClusterExperiment,ANY,character,ANY-method}
\alias{[,ClusterExperiment,ANY,ANY,ANY-method}
\alias{[,ClusterExperiment,ANY,logical,ANY-method}
\alias{[,ClusterExperiment,ANY,numeric,ANY-method}
\alias{subsetByCluster,ClusterExperiment-method}
\alias{subsetByCluster}
\title{Functions to subset ClusterExperiment Objects}
\usage{
\S4method{removeClusterings}{ClusterExperiment}(x, whichClusters)

\S4method{removeClusters}{ClusterExperiment}(
  x,
  whichCluster,
  clustersToRemove,
  makePrimary = FALSE,
  clusterLabels = NULL
)

\S4method{[}{ClusterExperiment,ANY,character,ANY}(x, i, j, ..., drop = TRUE)

\S4method{[}{ClusterExperiment,ANY,logical,ANY}(x, i, j, ..., drop = TRUE)

\S4method{[}{ClusterExperiment,ANY,numeric,ANY}(x, i, j, ..., drop = TRUE)

\S4method{subsetByCluster}{ClusterExperiment}(
  x,
  clusterValue,
  whichCluster = "primary",
  matchTo = c("name", "clusterIds")
)
}
\arguments{
\item{x}{a ClusterExperiment object.}

\item{whichClusters}{argument that can be either numeric or character vector
indicating the clusterings to be used. See details of \code{\link{getClusterIndex}}.}

\item{whichCluster}{argument that can be a single numeric or character value
indicating the \emph{single} clustering to be used. Giving values that result in more than one clustering will result in an error. See details of \code{\link{getClusterIndex}}.}

\item{clustersToRemove}{numeric vector identifying the clusters to remove
(whose samples will be reassigned to -1 value).}

\item{makePrimary}{whether to make the added cluster the primary cluster
(only relevant if \code{y} is a vector)}

\item{clusterLabels}{label(s) for the clusters being added. If \code{y} a
matrix, the column names of that matrix will be used by default, if
\code{clusterLabels} is not given.}

\item{i, j}{A vector of logical or integer subscripts, indicating the rows and columns to be subsetted for \code{i} and \code{j}, respectively.}

\item{...}{The arguments \code{transformation}, \code{clusterTypes} and
\code{clusterInfo} to be passed to the constructor for signature
\code{SingleCellExperiment,matrix}.}

\item{drop}{A logical scalar that is ignored.}

\item{clusterValue}{values of the cluster to match to for subsetting}

\item{matchTo}{whether to match to the cluster name
(\code{"name"}) or internal cluster id (\code{"clusterIds"})}
}
\value{
A \code{\link{ClusterExperiment}} object.

\code{removeClusterings} returns a \code{ClusterExperiment} object,
  unless all clusters are removed, in which case it returns a
  \code{\link{SingleCellExperiment}} object.

\code{subsetByCluster} subsets the object by clusters in a clustering
and returns a ClusterExperiment object with only those samples
}
\description{
These functions are used to subset ClusterExperiment objects,
  either by removing samples, genes, or clusterings
}
\details{
\code{removeClusterings} removes the clusters given by
  \code{whichClusters}. If the \code{primaryCluster} is one of the clusters
  removed, the \code{primaryClusterIndex} is set to 1 and the dendrogram and
  coclustering matrix are discarded and orderSamples is set to
  \code{1:NCOL(x)}.

\code{removeClusters} creates a new cluster that unassigns samples
  in cluster \code{clustersToRemove} (in the clustering defined by
  \code{whichClusters}) and assigns them to -1 (unassigned)

Note that when subsetting the data, the dendrogram information and
the co-clustering matrix are lost.
}
\examples{
#load CE object
data(rsecFluidigm)
# remove the mergeClusters step from the object
clusterLabels(rsecFluidigm)
test<-removeClusterings(rsecFluidigm,whichClusters="mergeClusters")
clusterLabels(test)
tableClusters(rsecFluidigm)
test<-removeClusters(rsecFluidigm,whichCluster="mergeClusters",clustersToRemove=c("m01","m04"))
tableClusters(test,whichCluster="mergeClusters")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getFeatures.R
\name{getBestFeatures}
\alias{getBestFeatures}
\alias{getBestFeatures,matrixOrHDF5-method}
\alias{getBestFeatures,ClusterExperiment-method}
\title{Function for finding best features associated with clusters}
\usage{
\S4method{getBestFeatures}{matrixOrHDF5}(
  x,
  cluster,
  contrastType = c("F", "Dendro", "Pairs", "OneAgainstAll"),
  dendro = NULL,
  pairMat = NULL,
  weights = NULL,
  contrastAdj = c("All", "PerContrast", "AfterF"),
  DEMethod = c("edgeR", "limma", "limma-voom"),
  dgeArgs = NULL,
  ...
)

\S4method{getBestFeatures}{ClusterExperiment}(
  x,
  contrastType = c("F", "Dendro", "Pairs", "OneAgainstAll"),
  whichCluster = "primary",
  whichAssay = 1,
  DEMethod,
  weights = if ("weights" \%in\% assayNames(x)) "weights" else NULL,
  ...
)
}
\arguments{
\item{x}{data for the test. Can be a numeric matrix or a 
\code{\link{ClusterExperiment}}.}

\item{cluster}{A numeric vector with cluster assignments. ``-1'' indicates 
the sample was not assigned to a cluster.}

\item{contrastType}{What type of test to do. `F' gives the omnibus 
F-statistic, `Dendro' traverses the given dendrogram and does contrasts of 
the samples in each side,  `Pairs' does pair-wise contrasts based on the 
pairs given in pairMat (if pairMat=NULL, does all pairwise), and 
`OneAgainstAll' compares each cluster to the average of all others. Passed 
to \code{\link{clusterContrasts}}}

\item{dendro}{The dendrogram to traverse if contrastType="Dendro". Note that
this should be the dendrogram of the clusters, not of the individual
samples, either of class "dendrogram" or "phylo4"}

\item{pairMat}{matrix giving the pairs of clusters for which to do pair-wise 
contrasts (must match to elements of cl). If NULL, will do all pairwise of 
the clusters in \code{cluster} (excluding "-1" categories). Each row is a
pair to be compared and must match the names of the clusters in the vector 
\code{cluster}.}

\item{weights}{weights to use in by edgeR. If \code{x} is a matrix, then
weights should be a matrix of weights, of the same dimensions as \code{x}.
If \code{x} is a \code{ClusterExperiment} object \code{weights} can be a
either a matrix, as previously described, or a character or numeric index
to an assay in \code{x} that contains the weights. We recommend that 
weights be stored as an assay with name \code{"weights"} so that the
weights will also be used with \code{\link{mergeClusters}}, and this is the
default. Setting \code{weights=NULL} ensures that weights will NOT be used,
and only the standard edgeR.}

\item{contrastAdj}{What type of FDR correction to do for contrasts tests 
(i.e. if contrastType='Dendro' or 'Pairs').}

\item{DEMethod}{character vector describing how the differential expression 
analysis should be performed (replaces previous argument \code{isCount}. 
See details.}

\item{dgeArgs}{a list of arguments to pass to \code{\link[edgeR]{DGEList}}
which is the starting point for both \code{edgeR} and \code{limma-voom}
methods of DE. This includes normalization factors/total count values etc.}

\item{...}{If \code{x} is a matrix, these are options to pass to 
\code{\link{topTable}}  (see 
\code{\link[limma]{limma}} package). If \code{x} is a 
\code{ClusterExperiment} object, these arguments can also be those to pass 
to the matrix version.}

\item{whichCluster}{argument that can be a single numeric or character value
indicating the \emph{single} clustering to be used. Giving values that result in more than one clustering will result in an error. See details of \code{\link{getClusterIndex}}.}

\item{whichAssay}{numeric or character specifying which assay to use. See
\code{\link[SummarizedExperiment]{assay}} for details.}
}
\value{
A \code{data.frame} in the same format as 
  \code{\link[limma]{topTable}}, or \code{\link[edgeR]{topTags}}. The output
  differs between these two programs, mainly in the naming of columns.
  Furthermore, if weights are used, an additional column \code{padjFilter} is
  included as the result of running \code{\link[zinbwave]{glmWeightedF}} with
  default option \code{independentFiltering = TRUE}. The following column
  names are the same between all of the DE methods.
\itemize{

\item{\code{Feature}}{ This is the column called 'ProbeID' by
\code{\link{topTable}}}

\item{\code{IndexInOriginal}}{ Gives the index of the feature to the original
input dataset, \code{x}}

\item{\code{Contrast}}{ The contrast that the results corresponds to (if
applicable, depends on \code{contrastType} argument)}

\item{\code{ContrastName}}{ The name of the contrast that the results
corresponds to. For dendrogram searches, this will be the node of the tree of
the dendrogram. If \code{x} is a \code{ClusterExperiment} object, this name 
will make use of the user defined names of the cluster or node in \code{x}.}

\item{\code{InternalName}}{ Only present if \code{x} is a
\code{ClusterExperiment} object. In this case this column will give the name
of the contrast using the internal ids of the clusters and nodes, not the
user-defined names. This provides stability in matching the contrast if the
user has changed the names since running \code{getBestFeatures}}

\item{\code{P.Value}}{ The unadjusted p-value (changed from \code{PValue} in
\code{topTags})}

\item{\code{adj.P.Val}}{ The adjusted p-value (changed from \code{FDR} or
\code{FWER} in \code{topTags})}
}
}
\description{
Calls limma on input data to determine features most associated 
  with found clusters (based on an F-statistic, pairwise comparisons, or 
  following a tree that clusters the clusters).
}
\details{
getBestFeatures returns the top ranked features corresponding to a 
  cluster assignment. It uses either limma or edgeR to fit the models, and 
  limma/edgeR functions \code{\link[limma]{topTable}} to find the best features. See the options 
  of this function to put better control on what gets returned (e.g. only 
  if significant, only if log-fc is above a certain amount, etc.). In 
  particular, set `number=` to define how many significant features to return
  (where number is per contrast for the `Pairs` or `Dendro` option)

\code{DEMethod} triggers what type of differential expression 
  analysis will be performed. Three options are available: limma, edgeR, and 
  limma with a voom corrections. The last two options are only appropriate 
  for count data. If the input \code{x} is a \code{ClusterExperiment} object,
  and \code{DEMethod="limma"}, then the data analyzed for DE will be
  \emph{after} taking the transformation of the data (as given in the
  transformation slot of the object). For the options "limma-voom" and
  "edgeR", the transformation slot will be ignored and only the counts data
  (as specified by the \code{whichAssay} slot) will be passed to the
  programs. Note that for "limma-voom" this implies that the data will be
  transformed by voom with the function log2(x+0.5). If \code{weights} is not
  \code{NULL}, and \code{DEMethod="edgeR"}, then the function
  \code{glmWeightedF} from the \code{zinbwave} package is run; otherwise
  \code{glmLRT} from \code{edgeR}.

Note that the argument \code{DEMethod} replaces the previous option 
  \code{isCount}, to decide on the method of DE.

When `contrastType` argument implies that the best features should 
  be found via contrasts (i.e. 'contrastType' is `Pairs` or `Dendro`), then 
  then `contrastAdj` determines the type of multiple testing correction to 
  perform. `PerContrast` does FDR correction for each set of contrasts, and 
  does not guarantee control across all the different contrasts (so probably 
  not the preferred method). `All` calculates the corrected p-values based on
  FDR correction of all of the contrasts tested. `AfterF` controls the FDR 
  based on a hierarchical scheme that only tests the contrasts in those genes
  where the omnibus F statistic is significant. If the user selects `AfterF`,
  the user must also supply an option `p.value` to have any effect, and then 
  only those significant at that p.value level will be returned. Note that 
  currently the correction for `AfterF` is not guaranteed to control the FDR;
  improvements will be added in the future.

Note that the default option for \code{\link[limma]{topTable}} is 
  to not filter based on adjusted p-values (\code{p.value = 1}) and return 
  only the top 10 most significant (\code{number = 10}) -- these are options 
  the user can change (these arguments are passed via the \code{...} in 
  \code{getBestFeatures}). In particular, it only makes sense to set 
  \code{requireF = TRUE} if \code{p.value} is meaningful (e.g. 0.1 or 0.05); 
  the default value of \code{p.value = 1} will not result in any effect on 
  the adjusted p-value otherwise.
}
\examples{
data(simData)

#create a clustering, for 8 clusters (truth was 4)
cl <- clusterSingle(simData, subsample=FALSE,
sequential=FALSE, mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=8)))

#basic F test, return all, even if not significant:
testF <- getBestFeatures(cl, contrastType="F", number=nrow(simData),
DEMethod="limma")

#Do all pairwise, only return significant, try different adjustments:
pairsPerC <- getBestFeatures(cl, contrastType="Pairs", contrastAdj="PerContrast",
p.value=0.05, DEMethod="limma")
pairsAfterF <- getBestFeatures(cl, contrastType="Pairs", contrastAdj="AfterF",
p.value=0.05, DEMethod="limma")
pairsAll <- getBestFeatures(cl, contrastType="Pairs", contrastAdj="All",
p.value=0.05, DEMethod="limma")

#not useful for this silly example, but could look at overlap with Venn
allGenes <- paste("Row", 1:nrow(simData),sep="")
if(require(limma)){
 vennC <- vennCounts(cbind(PerContrast= allGenes \%in\% pairsPerC$Feature,
 AllJoint=allGenes \%in\% pairsAll$Feature, FHier=allGenes \%in\%
 pairsAfterF$Feature))
vennDiagram(vennC, main="FDR Overlap")
}

#Do one cluster against all others
oneAll <- getBestFeatures(cl, contrastType="OneAgainstAll", contrastAdj="All",
p.value=0.05,DEMethod="limma")

#Do dendrogram testing
hcl <- makeDendrogram(cl)
allDendro <- getBestFeatures(hcl, contrastType="Dendro", contrastAdj=c("All"),
number=ncol(simData), p.value=0.05,DEMethod="limma")

# do DE on counts using voom
# compare results to if used simData instead (not on count scale).
testFVoom <- getBestFeatures(simCount, primaryCluster(cl), contrastType="F",
number=nrow(simData), DEMethod="limma-voom")
plot(testF$P.Value[order(testF$Index)],
testFVoom$P.Value[order(testFVoom$Index)],log="xy")

# do DE on counts using edgeR, compare voom
testFEdge <- getBestFeatures(simCount, primaryCluster(cl), contrastType="F",
n=nrow(simData), DEMethod="edgeR")
plot(testFVoom$P.Value[order(testFVoom$Index)],
testFEdge$P.Value[order(testFEdge$Index)],log="xy")
}
\references{
Ritchie, ME, Phipson, B, Wu, D, Hu, Y, Law, CW, Shi, W, and
  Smyth, GK (2015). limma powers differential expression analyses for
  RNA-sequencing and microarray studies. Nucleic Acids Research 43, e47.
  http://nar.oxfordjournals.org/content/43/7/e47

Law, CW, Chen, Y, Shi, W, and Smyth, GK (2014). Voom: precision
  weights unlock linear model analysis tools for RNA-seq read counts. Genome
  Biology 15, R29. http://genomebiology.com/2014/15/2/R29

Smyth, G. K. (2004). Linear models and empirical Bayes methods
  for assessing differential expression in microarray experiments.
  Statistical Applications in Genetics and Molecular Biology, Volume 3,
  Article 3. http://www.statsci.org/smyth/pubs/ebayes.pdf
}
\seealso{
\code{\link[edgeR]{glmLRT}} \code{\link[zinbwave]{glmWeightedF}}
  \code{\link[limma]{topTable}} \code{\link[edgeR]{topTags}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllHelper.R
\name{ClusterExperiment-methods}
\alias{ClusterExperiment-methods}
\alias{show}
\alias{show,ClusterExperiment-method}
\alias{transformation,ClusterExperiment-method}
\alias{transformation}
\alias{transformation<-,ClusterExperiment,function-method}
\alias{transformation<-}
\alias{nClusterings,ClusterExperiment-method}
\alias{nClusterings}
\alias{nClusters,ClusterExperiment-method}
\alias{nClusters}
\alias{nFeatures,ClusterExperiment-method}
\alias{nFeatures}
\alias{nSamples,ClusterExperiment-method}
\alias{nSamples}
\alias{clusterMatrixNamed,ClusterExperiment-method}
\alias{clusterMatrixNamed}
\alias{clusterMatrixColors,ClusterExperiment-method}
\alias{clusterMatrixColors}
\alias{clusterMatrix,ClusterExperiment-method}
\alias{clusterMatrix}
\alias{primaryCluster,ClusterExperiment-method}
\alias{primaryCluster}
\alias{primaryClusterIndex,ClusterExperiment-method}
\alias{primaryClusterIndex}
\alias{primaryClusterLabel,ClusterExperiment-method}
\alias{primaryClusterLabel}
\alias{primaryClusterNamed,ClusterExperiment-method}
\alias{primaryClusterNamed}
\alias{primaryClusterType,ClusterExperiment-method}
\alias{primaryClusterType}
\alias{primaryClusterIndex<-,ClusterExperiment,numeric-method}
\alias{primaryClusterIndex<-}
\alias{dendroClusterIndex,ClusterExperiment-method}
\alias{dendroClusterIndex}
\alias{coClustering,ClusterExperiment-method}
\alias{coClustering}
\alias{coClustering<-,ClusterExperiment,matrix-method}
\alias{coClustering<-}
\alias{coClustering<-,ClusterExperiment,dsCMatrix-method}
\alias{coClustering<-,ClusterExperiment,numeric-method}
\alias{clusterTypes,ClusterExperiment-method}
\alias{clusterTypes}
\alias{clusteringInfo,ClusterExperiment-method}
\alias{clusteringInfo}
\alias{clusterLabels,ClusterExperiment-method}
\alias{clusterLabels}
\alias{clusterLabels<-,ClusterExperiment,character-method}
\alias{clusterLabels<-}
\alias{clusterLegend,ClusterExperiment-method}
\alias{clusterLegend}
\alias{clusterLegend<-,ClusterExperiment,list-method}
\alias{clusterLegend<-}
\alias{orderSamples,ClusterExperiment-method}
\alias{orderSamples}
\alias{orderSamples<-,ClusterExperiment,numeric-method}
\alias{orderSamples<-}
\alias{clusterTypes<-,ClusterExperiment,character-method}
\alias{clusterTypes<-}
\alias{addToColData,ClusterExperiment-method}
\alias{addToColData}
\alias{colDataClusters,ClusterExperiment-method}
\alias{colDataClusters}
\title{Helper methods for the ClusterExperiment class}
\usage{
\S4method{show}{ClusterExperiment}(object)

\S4method{transformation}{ClusterExperiment}(x)

\S4method{transformation}{ClusterExperiment,function}(object) <- value

\S4method{nClusterings}{ClusterExperiment}(x)

\S4method{nClusters}{ClusterExperiment}(x, ignoreUnassigned = TRUE)

\S4method{nFeatures}{ClusterExperiment}(x)

\S4method{nSamples}{ClusterExperiment}(x)

\S4method{clusterMatrixNamed}{ClusterExperiment}(x, whichClusters = "all")

\S4method{clusterMatrixColors}{ClusterExperiment}(x, whichClusters = "all")

\S4method{clusterMatrix}{ClusterExperiment}(x, whichClusters)

\S4method{primaryCluster}{ClusterExperiment}(x)

\S4method{primaryClusterIndex}{ClusterExperiment}(x)

\S4method{primaryClusterLabel}{ClusterExperiment}(x)

\S4method{primaryClusterNamed}{ClusterExperiment}(x)

\S4method{primaryClusterType}{ClusterExperiment}(x)

\S4method{primaryClusterIndex}{ClusterExperiment,numeric}(object) <- value

\S4method{dendroClusterIndex}{ClusterExperiment}(x)

\S4method{coClustering}{ClusterExperiment}(x)

\S4method{coClustering}{ClusterExperiment,matrix}(object) <- value

\S4method{coClustering}{ClusterExperiment,dsCMatrix}(object) <- value

\S4method{coClustering}{ClusterExperiment,numeric}(object) <- value

\S4method{clusterTypes}{ClusterExperiment}(x)

\S4method{clusteringInfo}{ClusterExperiment}(x)

\S4method{clusterLabels}{ClusterExperiment}(x)

\S4method{clusterLabels}{ClusterExperiment,character}(object) <- value

\S4method{clusterLegend}{ClusterExperiment}(x)

\S4method{clusterLegend}{ClusterExperiment,list}(object) <- value

\S4method{orderSamples}{ClusterExperiment}(x)

\S4method{orderSamples}{ClusterExperiment,numeric}(object) <- value

\S4method{clusterTypes}{ClusterExperiment,character}(object) <- value

\S4method{addToColData}{ClusterExperiment}(object, ...)

\S4method{colDataClusters}{ClusterExperiment}(
  object,
  whichClusters = "primary",
  useNames = TRUE,
  makeFactor = TRUE,
  ...
)
}
\arguments{
\item{x, object}{a ClusterExperiment object.}

\item{value}{The value to be substituted in the corresponding slot. See the
slot descriptions in \code{\link{ClusterExperiment}} for details on what
objects may be passed to these functions.}

\item{ignoreUnassigned}{logical. If true, ignore the clusters with -1 or -2
assignments in calculating the number of clusters per clustering.}

\item{whichClusters}{argument that can be either numeric or character vector
indicating the clusterings to be used. See details of \code{\link{getClusterIndex}}.}

\item{...}{For \code{addToColData}, arguments passed to
\code{colDataClusters}.}

\item{useNames}{for \code{tableClusters}, whether the output should be tabled
with names (\code{useNames=TRUE}) or ids (\code{useNames=FALSE})}

\item{makeFactor}{logical for \code{colDataClusters}. If TRUE the clustering
will be added to the \code{colData} slot as a factor. If FALSE, the
clustering will be added to the \code{colData} slot as a character vector
if \code{useNames=TRUE} and as a numeric vector if \code{useNames=FALSE}.}
}
\value{
\code{transformation} prints the function used to transform the data
prior to clustering.

\code{nClusterings} returns the number of clusterings (i.e., ncol of
clusterMatrix).

\code{nClusters} returns the number of clusters per clustering

\code{nFeatures} returns the number of features (same as `nrow`).

\code{nSamples} returns the number of samples (same as `ncol`).

\code{clusterMatrixNamed} returns a matrix with cluster labels.

\code{clusterMatrixColors} returns the matrix with all the
  clusterings, using the internally stored colors for each cluster

\code{clusterMatrix} returns the matrix with all the clusterings.

\code{primaryCluster} returns the primary clustering (as numeric).

\code{primaryClusterIndex} returns/sets the primary clustering index
(i.e., which column of clusterMatrix corresponds to the primary clustering).

\code{primaryClusterIndex} returns/sets the primary clustering index
(i.e., which column of clusterMatrix corresponds to the primary clustering).

\code{primaryClusterNamed} returns the primary cluster (using cluster
labels).

\code{primaryClusterIndex} returns/sets the primary clustering index
(i.e., which column of clusterMatrix corresponds to the primary clustering).

\code{dendroClusterIndex} returns/sets the clustering index 
of the clusters used to create dendrogram
(i.e., which column of clusterMatrix corresponds to the clustering).

\code{coClustering} returns/sets the co-clustering matrix.

\code{clusterTypes} returns/sets the clusterTypes slot.

\code{clusteringInfo} returns the clusterInfo slot.

\code{clusterLabels} returns/sets the column names of the
  clusterMatrix slot.

\code{clusterLegend} returns/sets the clusterLegend slot.

\code{orderSamples} returns/sets the orderSamples slot.

\code{addToColData} returns a \code{ClusterExperiment} object
with the clusterings in clusterMatrix slot added to the \code{colData} slot

\code{colDataClusters} returns a \code{DataFrame} object
that has the clusterings in clusterMatrix slot added to the 
\code{DataFrame} in the \code{colData} slot
}
\description{
This is a collection of helper methods for the ClusterExperiment
  class.
}
\details{
Note that redefining the transformation function via
  \code{transformation(x)<-} will check the validity of the transformation on
  the data assay. If the assay is large, this may be time consuming. Consider
  using a call to ClusterExperiment, which has the option as to whether to
  check the validity of the transformation.
}
\examples{
# load data:
data(rsecFluidigm)
show(rsecFluidigm)
#Number of clusterings
nClusterings(rsecFluidigm)
# Number of clusters per clustering
nClusters(rsecFluidigm)
# Number of features/samples
nSamples(rsecFluidigm)
nFeatures(rsecFluidigm)
# retrieve all clustering assignments
# (either as cluster ids, cluster names or cluster colors)
head(clusterMatrix(rsecFluidigm)[,1:5])
head(clusterMatrixNamed(rsecFluidigm)[,1:5])
head(clusterMatrixColors(rsecFluidigm)[,1:5])
# clustering Types/Labels
clusterTypes(rsecFluidigm)
clusterLabels(rsecFluidigm)
# Add a clustering assignment to the colData of the object
# (useful if working with function that relies on colData)
colData(rsecFluidigm)
test<-addToColData(rsecFluidigm,whichCluster="primary")
colData(test)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotClustersTable.R
\name{plotClustersTable}
\alias{plotClustersTable}
\alias{plotClustersTable,ClusterExperiment-method}
\alias{plotClustersTable,table-method}
\alias{tableClusters,ClusterExperiment-method}
\alias{tableClusters}
\alias{bubblePlot,table,table-method}
\title{Plot heatmap of cross-tabs of 2 clusterings}
\usage{
\S4method{plotClustersTable}{ClusterExperiment}(
  object,
  whichClusters,
  ignoreUnassigned = FALSE,
  margin = NA,
  ...
)

\S4method{plotClustersTable}{table}(
  object,
  plotType = c("heatmap", "bubble"),
  main = "",
  xlab = NULL,
  ylab = NULL,
  legend = TRUE,
  cluster = FALSE,
  clusterLegend = NULL,
  sizeTable = NULL,
  ...
)

\S4method{tableClusters}{ClusterExperiment}(
  object,
  whichClusters = "primary",
  useNames = TRUE,
  tableMethod = c("intersect", "union"),
  ...
)

\S4method{bubblePlot}{table,table}(
  propTable,
  sizeTable,
  gridColor = rgb(0, 0, 0, 0.05),
  maxCex = 8,
  cexFactor,
  ylab,
  xlab,
  propLabel = "Value of \%",
  legend = TRUE,
  las = 2,
  colorScale = RColorBrewer::brewer.pal(11, "Spectral")[-6]
)
}
\arguments{
\item{object}{ClusterExperiment object (or matrix with table result)}

\item{whichClusters}{argument that can be either numeric or character vector
indicating the clusterings to be used. See details of \code{\link{getClusterIndex}}.}

\item{ignoreUnassigned}{logical as to whether to ignore unassigned clusters
in the plotting. This means they will also be ignored in the calculations
of the proportions (if \code{margin} not NA).}

\item{margin}{if NA, the actual counts from \code{tableClusters} will be
plotted. Otherwise, \code{\link[base]{prop.table}} will be called and the
argument \code{margin} will be passed to \code{prop.table} to determine
whether proportions should be calculated. If '1', then the proportions in
the rows sum to 1, if '2' the proportions in the columns sum to 1. If
'NULL' then the proportion across the entire matrix will sum to 1. An
additional option has been added so that if you set  \code{margin=0}, the
entry displayed in each cell will be the proportion equal to the size of
the intersection over the size of the union of the clusters (a Jaccard
similarity between the clusters), in which case each entry is a value
between 0 and 1 but no combination of the entries sum to 1.}

\item{...}{arguments passed on to \code{plotHeatmap} or \code{bubblePlot}
depending on choice of \code{plotType}. Note that these functions take
different arguments so that switching from one to the other may not take
all arguments. In particular \code{bubblePlot} calls \code{plot} while
\code{plotHeatmap} calls \code{\link{NMF}{aheatmap}}.}

\item{plotType}{type of plot. If "heatmap", then a heatmap will be created of
the values of the contingency table of the two clusters (calculated as
determined by the argument "margin") using \code{\link{plotHeatmap}}. If
"bubble", then a plot will be created using \code{bubblePlot}, which will
create circles for each cell of the contingencey table whose size
corresponds to the number of samples shared and the color based on the
value of the proportion (as chosen by the argument \code{margin}).}

\item{main}{title of plot, passed to \code{plotHeatmap} or to the argument
\code{propLabel} in \code{bubblePlot}}

\item{xlab}{label for labeling clustering on the x-axis. If NULL, will
determine names. If set to \code{NA} no label for clustering on the x-axis
will be plotted (to turn off legend of the clusterings in heatmap, set
\code{legend=FALSE}).}

\item{ylab}{label for labeling clustering on the y-axis. If NULL, will
determine names. If set to \code{NA} no label for clustering on the y-axis
will be plotted (to turn off legend of the clusterings in heatmap, set
\code{legend=FALSE}).}

\item{legend}{whether to draw legend along top (bubble plot) or the color
legend (heatmap)}

\item{cluster}{logical, whether to cluster the rows and columns of the table.
Passed to arguments \code{clusterFeatures} AND \code{clusterSamples} of
\code{plotHeatmap}.}

\item{clusterLegend}{list in \code{clusterLegend} format that gives colors
for the clusters tabulated.}

\item{sizeTable}{table of sizes (only for use in \code{bubblePlot} or
\code{plotType="bubble"}). See details.}

\item{useNames}{for \code{tableClusters}, whether the output should be tabled
with names (\code{useNames=TRUE}) or ids (\code{useNames=FALSE})}

\item{tableMethod}{the type of table calculation to perform. "intersect"
refers to the standard contingency table (\code{\link[base]{table}}), where
each entry of the resulting table is the number of objects in both
clusters. "union" instead gives for each entry the number of objects that
are in the union of both clusters.}

\item{propTable}{table of proportions (\code{bubblePlot}))}

\item{gridColor}{color for grid lines (\code{bubblePlot}))}

\item{maxCex}{largest value of cex for any point (others will scale
proportionally smaller) (\code{bubblePlot})).}

\item{cexFactor}{factor to multiple by to get values of circles. If missing,
finds value automatically, namely by using the maxCex value default.
Overrides value of maxCex. (\code{bubblePlot}))}

\item{propLabel}{the label to go with the legend of the color of the
bubbles/circles}

\item{las}{the value for the las value in the call to \code{\link{axis}} in
labeling the clusters in the bubble plot. Determines whether parallel or
perpindicular labels to the axis (see \code{\link{par}}).}

\item{colorScale}{the color scale for the values of the proportion table}
}
\value{
\code{tableClusters} returns an object of class \code{table} (see
  \code{\link[base]{table}}).

\code{plotClustersTables} returns invisibly the plotted proportion
  table. In particular, this is the result of applying
  \code{\link[base]{prop.table}} to the results of \code{tableClusters}
  (after removing unclustered samples if \code{ignoreUnassigned=TRUE}).
}
\description{
Plot heatmap of cross-tabulations of two clusterings
}
\details{
For \code{plotClustersTable} applied to the class \code{table},
  \code{sizeTable} is passed to \code{bubblePlot} to indicate the size of the
  circle. If \code{sizeTable=NULL}, then it is assumed that the \code{object}
  argument is the table of counts and both the \code{propTable} and
  \code{sizeTable} are set to the same value (hence turning off the coloring
  of the circle/bubbles). This is equivalent effect to the \code{margin=NA}
  option of \code{plotClustersTable} applied to the \code{ClusterExperiment}
  class.

Note that the cluster labels in \code{plotClustersTable} and
  \code{tableClusters} are converted to "proper" R names via
  \code{make.names}. This is because \code{tableClusters} calls the R
  function \code{table}, which makes this conversion

For \code{plotClustersTable}, \code{whichClusters} should define 2
  clusters, while for \code{tableClusters} it can indicate arbitrary number.

\code{bubblePlot} is mainly used internally by
  \code{plotClustersTable} but is made public for users who want more control
  and to allow documentation of the arguments. \code{bubblePlot} plots a
  circle for each intersection of two clusters, where the color of the circle
  is based on the value in \code{propTable} and the size of the circle is
  based on the value in \code{sizeTable}. If \code{propTable}
  is equal to \code{sizeTable}, then the \code{propTable} is ignored and the 
  coloring of the circles is not performed, only the adjusting of the
  size of the circles based on the total size. The size is determined by 
  setting the \code{cex} value of the point as
  $sqrt(sizeTable[i,j])/sqrt(max(sizeTable))*cexFactor$.
}
\examples{
#clustering using pam: try using different dimensions of pca and different k
data(simData)

cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reducedDim="PCA",
clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)
#give arbitrary names to clusters for demonstration
cl<-renameClusters(cl,value=letters[1:nClusters(cl)[1]],whichCluster=1)
tableClusters(cl,whichClusters=1:2)
#show options of margin in heatmap format:
par(mfrow=c(2,3))
plotClustersTable(cl,whichClusters=1:2, margin=NA, legend=FALSE,
  ignoreUnassigned=TRUE)
plotClustersTable(cl,whichClusters=1:2, margin=0, legend=FALSE,
  ignoreUnassigned=TRUE)
plotClustersTable(cl,whichClusters=1:2, margin=1, legend=FALSE,
  ignoreUnassigned=TRUE)
plotClustersTable(cl,whichClusters=1:2, margin=2, legend=FALSE,
  ignoreUnassigned=TRUE)
plotClustersTable(cl,whichClusters=1:2, margin=NULL, legend=FALSE,
  ignoreUnassigned=TRUE)

#show options of margin in bubble format:
par(mfrow=c(2,3))
plotClustersTable(cl,whichClusters=1:2, margin=NA, 
   ignoreUnassigned=TRUE, plotType="bubble")
plotClustersTable(cl,whichClusters=1:2, margin=0,
   ignoreUnassigned=TRUE, plotType="bubble")
plotClustersTable(cl,whichClusters=1:2, margin=1,
   ignoreUnassigned=TRUE, plotType="bubble")
plotClustersTable(cl,whichClusters=1:2, margin=2,
   ignoreUnassigned=TRUE, plotType="bubble")
plotClustersTable(cl,whichClusters=1:2, margin=NULL,
   ignoreUnassigned=TRUE, plotType="bubble")
}
\seealso{
\code{\link{plotHeatmap}}

\code{\link[base]{table}}

\code{\link[base]{prop.table}}
}
\author{
Kelly Street, Elizabeth Purdom
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/workflowClusters.R
\name{workflowClusters}
\alias{workflowClusters}
\alias{workflowClusterTable}
\alias{workflowClusterDetails}
\alias{workflowClusters,ClusterExperiment-method}
\alias{workflowClusterDetails,ClusterExperiment-method}
\alias{workflowClusterTable,ClusterExperiment-method}
\alias{setToCurrent,ClusterExperiment-method}
\alias{setToCurrent}
\alias{setToFinal,ClusterExperiment-method}
\alias{setToFinal}
\title{Methods for workflow clusters}
\usage{
\S4method{workflowClusters}{ClusterExperiment}(x, iteration = 0)

\S4method{workflowClusterDetails}{ClusterExperiment}(x)

\S4method{workflowClusterTable}{ClusterExperiment}(x)

\S4method{setToCurrent}{ClusterExperiment}(x, whichCluster, eraseOld = FALSE)

\S4method{setToFinal}{ClusterExperiment}(x, whichCluster, clusterLabel)
}
\arguments{
\item{x}{a \code{\link{ClusterExperiment}} object.}

\item{iteration}{numeric. Which iteration of the workflow should be used.}

\item{whichCluster}{argument that can be a single numeric or character value
indicating the \emph{single} clustering to be used. Giving values that result in more than one clustering will result in an error. See details of \code{\link{getClusterIndex}}.}

\item{eraseOld}{logical. Only relevant if input \code{x} is of class
\code{ClusterExperiment}. If TRUE, will erase existing workflow results
(clusterMany as well as mergeClusters and makeConsensus). If FALSE, existing
workflow results will have "\code{_i}" added to the clusterTypes value,
where \code{i} is one more than the largest such existing workflow
clusterTypes.}

\item{clusterLabel}{optional string value to give to cluster set to be "final"}
}
\value{
\code{workflowClusters} returns a matrix consisting of the
  appropriate columns of the \code{clusterMatrix} slot.

\code{workflowClusterDetails} returns a \code{data.frame} with some
  details on the clusterings, such as the type (e.g., `clusterMany`,
  `makeConsensus`) and iteration.

\code{workflowClusterTable} returns a table of how many of the
  clusterings belong to each of the following possible values: `final`,
  `mergeClusters`, `makeConsensus` and `clusterMany`.

\code{setToCurrent} returns a \code{ClusterExperiment} object where
  the indicated cluster of \code{whichCluster} has been set to the most
  current iteration in the workflow. Pre-existing clusters are appropriately
  updated.

\code{setToFinal} returns a \code{ClusterExperiment} object where the
  indicated cluster of \code{whichCluster} has clusterType set to "final".
  The primaryClusterIndex is also set to this cluster, and the clusterLabel,
  if given.
}
\description{
The main workflow of the package is made of
  \code{\link{clusterMany}}, \code{\link{makeConsensus}}, and
  \code{\link{mergeClusters}}. The clusterings from these functions (and not
  those obtained in a different way) can be obtained with the functions
  documented here.
}
\examples{
\dontrun{
data(simData)

cl <- clusterMany(simData,nReducedDims=c(5,10,50),  reduceMethod="PCA",
clusterFunction="pam", ks=2:4, findBestK=c(FALSE), removeSil=TRUE,
subsample=FALSE, makeMissingDiss=TRUE)

clCommon <- makeConsensus(cl, whichClusters="workflow", proportion=0.7,
minSize=10)

clCommon <- makeDendrogram(clCommon)

clMerged <- mergeClusters(clCommon,mergeMethod="adjP", DEMethod="limma")

head(workflowClusters(clMerged))
workflowClusterDetails(clMerged)
workflowClusterTable(clMerged)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/builtInClusterFunctions.R
\name{listBuiltInFunctions}
\alias{listBuiltInFunctions}
\alias{getBuiltInFunction,character-method}
\alias{getBuiltInFunction}
\alias{listBuiltInTypeK}
\alias{listBuiltInType01}
\title{Built in ClusterFunction options}
\usage{
listBuiltInFunctions()

\S4method{getBuiltInFunction}{character}(object)

listBuiltInTypeK()

listBuiltInType01()
}
\arguments{
\item{object}{name of built in function.}
}
\value{
\code{listBuiltInFunctions} returns a character vector of all 
the built-in cluster functions' names.

\code{getBuiltInFunction} returns the \code{ClusterFunction} 
object that corresponds to the character name of a function

\code{listBuiltInTypeK} returns a character vector of the 
names of built-in cluster functions that are of type "K"

\code{listBuiltInType01} returns a character vector of the 
names of built-in cluster functions that are of type "01"
}
\description{
Documents the built-in clustering options that are available in
  the clusterExperiment package.
}
\details{
\code{listBuiltInFunctions} will return the character names of the
  built-in clustering functions available.

\code{listBuiltInTypeK} returns the names of the built-in functions
  that have type 'K'

\code{listBuiltInType01} returns the names of the built-in functions
  that have type '01'

\code{getBuiltInFunction} will return the \code{ClusterFunction}
  object of a character value that corresponds to a built-in function.

\code{\link{algorithmType}} and \code{\link{inputType}} will return
  the \code{algorithmType} and \code{inputType} of the built-in
  clusterFunction corresponding to the character value.

\strong{Built-in clustering methods:} The built-in clustering
  methods, the names of which can be accessed by
  \code{listBuiltInFunctions()} are the following:
\itemize{ 
\item{"pam"}{Based on \code{\link[cluster]{pam}} in
  \code{cluster} package. Arguments to that function can be passed via
  \code{clusterArgs}. 
Input can be either \code{"x"} or \code{"diss"}; algorithm type is "K"} 
\item{"clara"}{Based on \code{\link[cluster]{clara}} in
  \code{cluster} package. Arguments to that function can be passed via
  \code{clusterArgs}. Note that we have changed the default arguments of 
  that function to match the recommendations in the documentation of
 \code{\link[cluster]{clara}} (numerous functions are set to less than optimal 
 settings for back-compatiability). Specifically, the following defaults 
 are implemented \code{samples=50}, \code{keep.data=FALSE}, 
\code{mediods.x=FALSE},\code{rngR=TRUE},
 \code{pamLike=TRUE}, \code{correct.d=TRUE}. 
Input is \code{"X"}; algorithm type is "K".} 
\item{"kmeans"}{Based on \code{\link[stats]{kmeans}} in \code{stats} package.
Arguments to that function can be passed via \code{clusterArgs} except for
\code{centers} which is reencoded here to be the argument 'k' Input is
\code{"X"}; algorithm type is "K"}
\item{"mbkmeans"}{\code{\link[mbkmeans]{mbkmeans}} runs mini-batch kmeans, a more computationally efficient version of kmeans. }
\item{"hierarchical01"}{\code{\link[stats]{hclust}} in \code{stats} package
is used to build hiearchical clustering. Arguments to that function can be
passed via \code{clusterArgs}. The \code{hierarchical01} cuts the hiearchical
tree based on the parameter \code{alpha}. It does not use the \code{cutree}
function, but instead transversing down the tree until getting a block of
samples with whose summary of the values  is greater than or equal to
1-alpha. Arguments that can be passed to 'hierarchical01' are
'evalClusterMethod' which determines how to summarize the samples' values of
D[samples,samples] for comparison to 1-alpha: "maximum" (default) takes the
minimum of D[samples,samples] and requires it to be less than or equal to
1-alpha; "average" requires that each row mean of D[samples,samples] be less
than or equal to 1-alpha. Additional arguments of hclust can also be passed
via clusterArgs to control the hierarchical clustering of D. Input is
\code{"diss"}; algorithm type is "01"}
\item{"hierarchicalK"}{\code{\link[stats]{hclust}} in \code{stats} package is used
  to build hiearchical clustering and \code{\link{cutree}} is used to cut the
  tree into \code{k} clusters.
Input is \code{"diss"}; algorithm type is "K"}   
\item{"tight"}{Based on the algorithm in
  Tsang and Wong, specifically their method of picking clusters from a
  co-occurance matrix after subsampling. The clustering encoded here is not
  the entire tight clustering algorithm, only that single piece that
  identifies clusters from the co-occurance matrix.  
Arguments for the tight method are 
  'minSize.core' (default=2), which sets the minimimum number of samples that
  form a core cluster.
Input is \code{"diss"}; algorithm type is "01"} 
\item{"spectral"}{\code{\link[kernlab]{specc}} in \code{kernlab} package 
is used to perform spectral clustering. Note that spectral clustering can 
produce errors if the number of clusters (K) is not sufficiently smaller than 
the number of samples (N). K < N is not always sufficient. 
Input is \code{"X"}; algorithm type is "K".}
}
}
\examples{
listBuiltInFunctions()
algorithmType(c("kmeans","pam","hierarchical01"))
inputType(c("kmeans","pam","hierarchical01"))
listBuiltInTypeK()
listBuiltInType01()
}
\seealso{
\code{\link{ClusterFunction}}, \code{\link{algorithmType}},
  \code{\link{inputType}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterMany.R
\name{clusterMany}
\alias{clusterMany}
\alias{clusterMany,matrixOrHDF5-method}
\alias{clusterMany,SingleCellExperiment-method}
\alias{clusterMany,ClusterExperiment-method}
\alias{clusterMany,SummarizedExperiment-method}
\alias{clusterMany,data.frame-method}
\title{Create a matrix of clustering across values of parameters}
\usage{
\S4method{clusterMany}{matrixOrHDF5}(
  x,
  reduceMethod = "none",
  nReducedDims = NA,
  transFun = NULL,
  isCount = FALSE,
  ...
)

\S4method{clusterMany}{SingleCellExperiment}(
  x,
  ks = NA,
  clusterFunction,
  reduceMethod = "none",
  nFilterDims = defaultNDims(x, reduceMethod, type = "filterStats"),
  nReducedDims = defaultNDims(x, reduceMethod, type = "reducedDims"),
  alphas = 0.1,
  findBestK = FALSE,
  sequential = FALSE,
  removeSil = FALSE,
  subsample = FALSE,
  silCutoff = 0,
  distFunction = NA,
  betas = 0.9,
  minSizes = 1,
  transFun = NULL,
  isCount = FALSE,
  verbose = TRUE,
  parameterWarnings = FALSE,
  mainClusterArgs = NULL,
  subsampleArgs = NULL,
  seqArgs = NULL,
  whichAssay = 1,
  makeMissingDiss = if (ncol(x) < 1000) TRUE else FALSE,
  ncores = 1,
  random.seed = NULL,
  run = TRUE,
  ...
)

\S4method{clusterMany}{ClusterExperiment}(
  x,
  reduceMethod = "none",
  nFilterDims = defaultNDims(x, reduceMethod, type = "filterStats"),
  nReducedDims = defaultNDims(x, reduceMethod, type = "reducedDims"),
  eraseOld = FALSE,
  ...
)

\S4method{clusterMany}{SummarizedExperiment}(x, ...)

\S4method{clusterMany}{data.frame}(x, ...)
}
\arguments{
\item{x}{the data matrix on which to run the clustering. Can be object of the
following classes: matrix (with genes in rows),
\code{\link[SummarizedExperiment]{SummarizedExperiment}},
\code{\link[SingleCellExperiment]{SingleCellExperiment}}
or \code{ClusterExperiment}.}

\item{reduceMethod}{character A character identifying what type of
dimensionality reduction to perform before clustering. Options are 1)
"none", 2) one of listBuiltInReducedDims() or listBuiltInFitlerStats OR 3)
stored filtering or reducedDim values in the object.}

\item{nReducedDims}{vector of the number of dimensions to use (when
\code{reduceMethod} gives a dimensionality reduction method).}

\item{transFun}{a transformation function to be applied to the data. If the
transformation applied to the data creates an error or NA values, then the
function will throw an error. If object is of class
\code{ClusterExperiment}, the stored transformation will be used and giving
this parameter will result in an error.}

\item{isCount}{if \code{transFun=NULL}, then \code{isCount=TRUE} will
determine the transformation as defined by \code{function(x){log2(x+1)}},
and \code{isCount=FALSE} will give a transformation function
\code{function(x){x}}. Ignored if \code{transFun=NULL}. If object is of
class \code{ClusterExperiment}, the stored transformation will be used and
giving this parameter will result in an error.}

\item{...}{For signature \code{matrix}, arguments to be passed on to mclapply
(if ncores>1). For all the other signatures, arguments to be passed to the
method for signature \code{matrix}.}

\item{ks}{the range of k values (see details for the meaning of \code{k} for
different choices of other parameters).}

\item{clusterFunction}{function used for the clustering. This must be either 
1) a character vector of built-in clustering techniques, or 2) a
\emph{named} list of \code{\link{ClusterFunction}} objects. Current
functions can be found by typing \code{listBuiltInFunctions()} into the
command-line.}

\item{nFilterDims}{vector of the number of the most variable features to keep
(when "var", "abscv", or "mad" is identified in \code{reduceMethod}).}

\item{alphas}{values of alpha to be tried. Only used for clusterFunctions of
type '01'. Determines tightness required in creating clusters from the
dissimilarity matrix. Takes on values in [0,1]. See documentation of
\code{\link{ClusterFunction}}.}

\item{findBestK}{logical, whether should find best K based on average
silhouette width (only used when clusterFunction of type "K").}

\item{sequential}{logical whether to use the sequential strategy (see details
of \code{\link{seqCluster}}). Can be used in combination with
\code{subsample=TRUE} or \code{FALSE}.}

\item{removeSil}{logical as to whether remove when silhouette < silCutoff
(only used if clusterFunction of type "K")}

\item{subsample}{logical as to whether to subsample via
\code{\link{subsampleClustering}}. If TRUE, clustering in mainClustering
step is done on the co-occurance between clusterings in the subsampled
clustering results.  If FALSE, the mainClustering step will be run directly
on \code{x}/\code{diss}}

\item{silCutoff}{Requirement on minimum silhouette width to be included in
cluster (only for combinations where removeSil=TRUE).}

\item{distFunction}{a vector of character strings that are the names of
distance functions found in the global environment. See the help pages of
\code{\link{clusterSingle}} for details about the required format of
distance functions. Currently, this distance function must be applicable
for all clusterFunction types tried. Therefore, it is not possible in
\code{clusterMany} to intermix type "K" and type "01" algorithms if you
also give distances to evaluate via \code{distFunction} unless all
distances give 0-1 values for the distance (and hence are possible for both
type "01" and "K" algorithms).}

\item{betas}{values of \code{beta} to be tried in sequential steps. Only used
for \code{sequential=TRUE}. Determines the similarity between two clusters
required in order to deem the cluster stable. Takes on values in [0,1]. See
documentation of \code{\link{seqCluster}}.}

\item{minSizes}{the minimimum size required for a cluster (in the
\code{mainClustering} step). Clusters smaller than this are not kept and samples
are left unassigned.}

\item{verbose}{logical. If TRUE it will print informative messages.}

\item{parameterWarnings}{logical, as to whether warnings and comments 
from checking the validity of the parameter combinations should be printed.}

\item{mainClusterArgs}{list of arguments to be passed for the mainClustering
step, see help pages of \code{\link{mainClustering}}.}

\item{subsampleArgs}{list of arguments to be passed to the subsampling step
(if \code{subsample=TRUE}), see help pages of
\code{\link{subsampleClustering}}.}

\item{seqArgs}{list of arguments to be passed to \code{\link{seqCluster}}.}

\item{whichAssay}{numeric or character specifying which assay to use. See
\code{\link[SummarizedExperiment]{assay}} for details.}

\item{makeMissingDiss}{logical. Whether to calculate necessary distance 
matrices needed when input is not "diss". If TRUE, then when a clustering 
function calls for a inputType "diss", but the given matrix is of type "X",
the function will calculate a distance function. A dissimilarity matrix
will also be calculated if a post-processing argument like \code{findBestK}
or \code{removeSil} is chosen, since these rely on calcualting silhouette
widths from distances.}

\item{ncores}{the number of threads}

\item{random.seed}{a value to set seed before each run of clusterSingle (so
that all of the runs are run on the same subsample of the data). Note, if
'random.seed' is set, argument 'ncores' should NOT be passed via
subsampleArgs; instead set the argument 'ncores' of clusterMany directly
(which is preferred for improving speed anyway).}

\item{run}{logical. If FALSE, doesn't run clustering, but just returns matrix
of parameters that will be run, for the purpose of inspection by user (with
rownames equal to the names of the resulting column names of clMat object
that would be returned if \code{run=TRUE}). Even if \code{run=FALSE},
however, the function will create the dimensionality reductions of the data
indicated by the user input.}

\item{eraseOld}{logical. Only relevant if input \code{x} is of class
\code{ClusterExperiment}. If TRUE, will erase existing workflow results
(clusterMany as well as mergeClusters and makeConsensus). If FALSE, existing
workflow results will have "\code{_i}" added to the clusterTypes value,
where \code{i} is one more than the largest such existing workflow
clusterTypes.}
}
\value{
If \code{run=TRUE} will
  return a \code{ClusterExperiment} object, where the results are stored as
  clusterings with clusterTypes \code{clusterMany}. Depending on
  \code{eraseOld} argument above, this will either delete existing such
  objects, or change the clusterTypes of existing objects. See argument
  \code{eraseOld} above. Arbitrarily the first clustering is set as the
  primaryClusteringIndex.

If \code{run=FALSE} a list with elements:
\itemize{
  \item{\code{paramMatrix}}{ a matrix giving the parameters of each
  clustering, where each column is a possible parameter set by the user and
  passed to \code{\link{clusterSingle}} and each row of paramMatrix
  corresponds to a clustering in \code{clMat}} 
  \item{\code{mainClusterArgs}}{
  a list of (possibly modified) arguments to mainClusterArgs}
  \item{\code{seqArgs=seqArgs}}{a list of (possibly modified) arguments to
  seqArgs} 
  \item{\code{subsampleArgs}}{a list of (possibly modified)
  arguments to subsampleArgs} }
}
\description{
Given a range of parameters, this function will return a matrix with the
clustering of the samples across the range, which can be passed to
\code{plotClusters} for visualization.
}
\details{
Some combinations of these parameters are not feasible. See the
  documentation of \code{\link{clusterSingle}} for important information on
  how these parameter choices interact.

While the function allows for multiple values of clusterFunction,
  the code does not reuse the same subsampling matrix and try different
  clusterFunctions on it. This is because if sequential=TRUE, different
  subsample clusterFunctions will create different sets of data to subsample
  so it is not possible; if sequential=FALSE, we have not implemented
  functionality for this reuse. Setting the \code{random.seed} value,
  however, should mean that the subsampled matrix is the same for each, but
  there is no gain in computational complexity (i.e. each subsampled
  co-occurence matrix is recalculated for each set of parameters).

The argument \code{ks} is interpreted differently for different
  choices of the other parameters. When/if sequential=TRUE, \code{ks} defines
  the argument \code{k0} of \code{\link{seqCluster}}. Otherwise, \code{ks}
  values are the \code{k} values for \strong{both} the mainClustering and
  subsampling step (i.e. assigned to the \code{subsampleArgs} and
  \code{mainClusterArgs} that are passed to \code{\link{mainClustering}} and
  \code{\link{subsampleClustering}} unless \code{k} is set appropriately in
  \code{subsampleArgs}. The passing of these arguments via
  \code{subsampleArgs} will only have an effect if `subsample=TRUE`.
  Similarly, the passing of \code{mainClusterArgs[["k"]]} will only have an
  effect when the clusterFunction argument includes a clustering algorithm of
  type "K". When/if "findBestK=TRUE", \code{ks} also defines the
  \code{kRange} argument of \code{\link{mainClustering}} unless \code{kRange}
  is specified by the user via the \code{mainClusterArgs}; note this means
  that the default option of setting \code{kRange} that depends on the input
  \code{k} (see \code{\link{mainClustering}}) is not available in
  \code{clusterMany}, only in \code{\link{clusterSingle}}.

If the input is a \code{ClusterExperiment} object, current
  implementation is that existing \code{orderSamples},\code{coClustering} or
  the many dendrogram slots will be retained.

If \code{run=FALSE}, the function will still calculate reduced 
  dimensions or filter statistics if not already calculated and saved in the object.
  Moreover the results of these calculations will not be save. Therefore, if these 
steps are lengthy for large datasets it is
  recommended to do them before calling the function.

The given \code{reduceMethod} values must either be \emph{all}
  precalculated filtering/dimensionality reduction stored in the appropriate
  location, or must \emph{all} be character values giving a built-in
  filtering/dimensionality reduction methods to be calculated. If some of the
  filtering/dimensionality methods are already calculated and stored, but not
  all, then they will \emph{all} be recalculated (and if they are not all
  built-in methods, this will give an error). So to save computational time
  with pre-calculated dimensionality reduction, the user must make sure they
  are \emph{all} precalculated. Also, user-defined values (i.e. not built-in
  functions) cannot be mixed with built-in functions unless they have already
  been precalculated (see \code{\link{makeFilterStats}} or
  \code{\link{makeReducedDims}}).
}
\examples{
\dontrun{
data(simData)

#Example: clustering using pam with different dimensions of pca and different
#k and whether remove negative silhouette values
#check how many and what runs user choices will imply:
checkParams <- clusterMany(simData,reduceMethod="PCA", makeMissingDiss=TRUE,
   nReducedDims=c(5,10,50), clusterFunction="pam", isCount=FALSE,
   ks=2:4,findBestK=c(TRUE,FALSE),removeSil=c(TRUE,FALSE),run=FALSE)
print(head(checkParams$paramMatrix))

#Now actually run it
cl <- clusterMany(simData,reduceMethod="PCA", nReducedDims=c(5,10,50),  isCount=FALSE,
   clusterFunction="pam",ks=2:4,findBestK=c(TRUE,FALSE),makeMissingDiss=TRUE, 
   removeSil=c(TRUE,FALSE))
print(cl)
head(colnames(clusterMatrix(cl)))

#make names shorter for plotting
clNames <- clusterLabels(cl)
clNames <- gsub("TRUE", "T", clNames)
clNames <- gsub("FALSE", "F", clNames)
clNames <- gsub("k=NA,", "", clNames)

par(mar=c(2, 10, 1, 1))
plotClusters(cl, axisLine=-2,clusterLabels=clNames)


#following code takes around 1+ minutes to run because of the subsampling
#that is redone each time:
system.time(clusterTrack <- clusterMany(simData, ks=2:15,
    alphas=c(0.1,0.2,0.3), findBestK=c(TRUE,FALSE), sequential=c(FALSE),
    subsample=c(FALSE), removeSil=c(TRUE), clusterFunction="pam", 
    makeMissingDiss=TRUE,
     mainClusterArgs=list(minSize=5, kRange=2:15), ncores=1, random.seed=48120))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getReducedData.R, R/makeFilterStats.R,
%   R/makeReducedDims.R
\name{getReducedData,ClusterExperiment-method}
\alias{getReducedData,ClusterExperiment-method}
\alias{getReducedData}
\alias{defaultNDims,SingleCellExperiment-method}
\alias{defaultNDims}
\alias{defaultNDims,matrixOrHDF5-method}
\alias{reduceFunctions}
\alias{makeFilterStats}
\alias{makeFilterStats,SummarizedExperiment-method}
\alias{makeFilterStats,matrixOrHDF5-method}
\alias{makeFilterStats,ClusterExperiment-method}
\alias{listBuiltInFilterStats}
\alias{filterData,SummarizedExperiment-method}
\alias{filterData}
\alias{filterNames,SummarizedExperiment-method}
\alias{filterNames}
\alias{makeReducedDims,SingleCellExperiment-method}
\alias{makeReducedDims}
\alias{makeReducedDims,matrixOrHDF5-method}
\alias{makeReducedDims,SummarizedExperiment-method}
\alias{makeReducedDims,ClusterExperiment-method}
\alias{listBuiltInReducedDims}
\title{Return matrix from ClusterExperiment with reduced dimensions}
\usage{
\S4method{getReducedData}{ClusterExperiment}(
  object,
  reduceMethod,
  filterIgnoresUnassigned,
  nDims = defaultNDims(object, reduceMethod),
  whichCluster = "primary",
  whichAssay = 1,
  returnValue = c("object", "list"),
  reducedDimName
)

\S4method{defaultNDims}{SingleCellExperiment}(object, reduceMethod, typeToShow)

\S4method{defaultNDims}{matrixOrHDF5}(object, ...)

\S4method{makeFilterStats}{SummarizedExperiment}(
  object,
  filterStats = listBuiltInFilterStats(),
  transFun = NULL,
  isCount = FALSE,
  filterNames = NULL,
  whichAssay = 1
)

\S4method{makeFilterStats}{matrixOrHDF5}(object, ...)

\S4method{makeFilterStats}{ClusterExperiment}(
  object,
  whichClusterIgnoreUnassigned = NULL,
  filterStats = listBuiltInFilterStats(),
  ...
)

listBuiltInFilterStats()

\S4method{filterData}{SummarizedExperiment}(
  object,
  filterStats,
  cutoff,
  percentile,
  absolute = FALSE,
  keepLarge = TRUE,
  whichAssay = 1
)

\S4method{filterNames}{SummarizedExperiment}(object)

\S4method{makeReducedDims}{SingleCellExperiment}(
  object,
  reducedDims = "PCA",
  maxDims = 500,
  transFun = NULL,
  isCount = FALSE,
  whichAssay = 1
)

\S4method{makeReducedDims}{matrixOrHDF5}(object, ...)

\S4method{makeReducedDims}{SummarizedExperiment}(object, ...)

\S4method{makeReducedDims}{ClusterExperiment}(object, ...)

listBuiltInReducedDims()
}
\arguments{
\item{object}{For \code{makeReducedDims},\code{makeFilterStats},
\code{defaultNDims} either matrix-like, \code{SingleCellExperiment}, or
\code{ClusterExperiment} object. For \code{getReducedData} only a
\code{ClusterExperiment} object allowed.}

\item{reduceMethod}{character. A method (or methods) for reducing the size of
the data, either by filtering the rows (genes) or by a dimensionality
reduction method. Must either be 1) must match the name of a built-in
method, in which case if it is not already existing in the object will be
passed to \code{\link{makeFilterStats}} or \code{link{makeReducedDims}}, or
2) must match a stored filtering statistic or dimensionality reduction in
the object}

\item{filterIgnoresUnassigned}{logical. Whether filtering statistics should 
ignore the unassigned samples within the clustering. Only relevant if 
'reduceMethod' matches one of built-in filtering statistics in 
\code{\link{listBuiltInFilterStats}()}, in which case the clustering 
identified in \code{whichCluster} is passed to \code{makeFilterStats} and
the unassigned samples are excluded in calculating the statistic. See 
\code{\link{makeFilterStats}}  for more details.}

\item{nDims}{The number of dimensions to keep from \code{reduceMethod}. If
missing calls \code{\link{defaultNDims}}.}

\item{whichCluster}{argument that can be a single numeric or character value
indicating the \emph{single} clustering to be used. Giving values that result in more than one clustering will result in an error. See details of \code{\link{getClusterIndex}}.}

\item{whichAssay}{numeric or character specifying which assay to use. See
\code{\link[SummarizedExperiment]{assay}} for details.}

\item{returnValue}{The format of output. Users will generally want to keep
the default (see details)}

\item{reducedDimName}{The name given to the reducedDims slot storing result 
(if \code{returnValue="object"}). If missing, the function will create a
default name: if \code{reduceMethod} is a dimensionality reduction, then
\code{reduceMethod} will be given as the name; if a filtering statistic,
"filteredBy_" followed by \code{reduceMethod}.}

\item{typeToShow}{character (optional). If given, should be one of
"filterStats" or "reducedDims" to indicate of the values in the
reduceMethod vector, only show those corresponding to "filterStats" or
"reducedDims" options.}

\item{...}{Values passed on the the 'SingleCellExperiment' method.}

\item{filterStats}{character vector of statistics to calculate. Must be one
of the character values given by \code{listBuildInFilterStats()}.}

\item{transFun}{a transformation function to be applied to the data. If the
transformation applied to the data creates an error or NA values, then the
function will throw an error. If object is of class
\code{ClusterExperiment}, the stored transformation will be used and giving
this parameter will result in an error.}

\item{isCount}{if \code{transFun=NULL}, then \code{isCount=TRUE} will
determine the transformation as defined by \code{function(x){log2(x+1)}},
and \code{isCount=FALSE} will give a transformation function
\code{function(x){x}}. Ignored if \code{transFun=NULL}. If object is of
class \code{ClusterExperiment}, the stored transformation will be used and
giving this parameter will result in an error.}

\item{filterNames}{if given, defines the names that will be assigned to the 
filtering statistics in the \code{rowData} of the object. If missing, will be 
just the value of \code{filterStats} argument}

\item{whichClusterIgnoreUnassigned}{indicates clustering that should be used
to filter out unassigned samples from the calculations. If \code{NULL} no
filtering of samples will be done. See details for more information.}

\item{cutoff}{numeric. A value at which to filter the rows (genes) for the
test statistic}

\item{percentile}{numeric. Either a number between 0,1 indicating what
percentage of the rows (genes) to keep or an integer value indicated the
number of rows (genes) to keep}

\item{absolute}{whether to take the absolute value of the filter statistic}

\item{keepLarge}{logical whether to keep rows (genes) with large values of
the test statistic or small values of the test statistic.}

\item{reducedDims}{a vector of character values indicating the methods of
dimensionality reduction to be performed. Currently only "PCA" is
implemented.}

\item{maxDims}{Numeric vector of integer giving the number of PC dimensions
to calculate. \code{maxDims} can also take values between (0,1) to indicate
keeping the number of dimensions necessary to account for that proportion
of the variance. \code{maxDims} should be of same length as
\code{reducedDims}, indicating the number of dimensions to keep for each
method (if \code{maxDims} is of length 1, the same number of dimensions
will be used for each).}
}
\value{
If \code{returnValue="object"}, a \code{ClusterExperiment} object.

If \code{returnValue="list"} a list with elements:
\itemize{ \item{\code{objectUpdate}}{object, potentially updated if had to
calculate dimensionality reduction or filtering statistic}
\item{\code{dataMatrix}}{the reduced dimensional matrix with the samples in
columns, features in rows} }

\code{defaultNDims} returns a numeric vector giving the default
  dimensions the methods in \code{clusterExperiment} will use for reducing
  the size of the data. If \code{typeToShow} is missing, the resulting vector
  will be equal to the length of \code{reduceMethod}. Otherwise, it will be a
  vector with all the unique valid default values for the \code{typeToShow}
  (note that different dimensionality reduction methods can have different
  maximal dimensions, so the result may not be of length one in this case).

\code{makeFilterStats} returns a
  \code{\link[SummarizedExperiment]{SummarizedExperiment}} object with the
  requested filtering statistics will be added to the \code{DataFrame} in the
  \code{rowData} slot and given names corresponding to the \code{filterStats}
  values. Warning: the function will overwrite existing columns in
  \code{rowData} with the same name. Columns in the \code{rowData} slot with
  different names should not be affected.

\code{filterData} returns a SingleCellExperiment object with the rows
  (genes) removed based on filters

\code{filterNames} returns a vector of the columns of the rowData
  that are considered valid filtering statistics. Currently any numeric
  column in rowData is a valid filtering statistic.

\code{makeReducedDims} returns a \code{\link{SingleCellExperiment}}
  containing the calculated dimensionality reduction in the \code{reduceDims}
  with names corresponding to the name given in \code{reducedDims}.
}
\description{
Returns a matrix of data from a \code{ClusterExperiment} object
  based on the choices of dimensionality reduction given by the user.

Functions for calculating and manipulating either filtering
  statistics, stored in rowData, or the dimensionality reduction results,
  stored in reducedDims.
}
\details{
\code{getReducedData} determines the matrix of values that can be used for
  computation based on the user's choice of dimensionality methods. The
  methods can be either of the filtering kind or the more general
  dimensionality reduction. The function will first look at any stored
  \code{ReducedDims} or filtering statistics already present in the data, and
  if missing, will assume that \code{reduceMethod} is one of the built-in
  method provided by the package and calculate the necessary. Note that if
  \code{reduceMethod} is a filtering statistic, in addition to filtering the
  features, the function will also perform the stored transformation of the
  data.

Note that this is used internally by functions, but is mainly only
  of interest for the user if they want to have the filtered, transformed
  data available as a matrix for continual use.

If \code{returnValue="object"}, then the output is a single, updated
  \code{ClusterExperiment} object with the reduced data matrix stored as an 
  element of the list in \code{reducedDims} slot (with name given by
  \code{reducedDimName} if given). If "list", then a list with one element
  that is the object and the other that is the reduced data matrix. Either
  way, the object returned in the list will be updated to contain with the
  filtering statistics or the dimensionality reduction. The only difference
  is that if "list", the reduced dimension matrix is NOT saved in the object
  (and so only really makes a difference if the \code{reduceMethod} argument
  is a filtering method). The option "list" is mainly for internal use, where
  we do not want to continually save subseted datasets.

If \code{nDims} is missing, it will be given a default value 
  depending on the value of \code{reduceMethod}. See 
  \code{\link{defaultNDims}} for details.

If \code{filterIgnoresUnassigned} is missing, then it is set to TRUE
  \emph{unless}: \code{reduceMethod} matches a stored filtering statistic in
  \code{rowData} AND does not match a built-in filtering method provided by
  the package.

For a \code{reduceMethod} that corresponds to a filtering statistics
  the current default is 1000 (or the length of the number of features, if
  less). For a dimensionality reduction saved in the reducedDims slot the
  default is 50 or the maximum number of dimensions if less than 50.

\code{reduceMethod} will first be checked to see if it corresponds
  with an existing saved filtering statistic or a dimensionality reduction to
  determine which of these two types it is. If it does not match either, then
  it will be checked against the built in functions provided by the package.
  @examples 
  se<-SingleCellExperiment(matrix(rnorm(5000*100),nrow=5000,ncol=100))
  defaultNDims(se,"PCA")
  defaultNDims(se,"mad")

\code{whichClusterIgnoreUnassigned} is only an option when applied
  to a \code{ClusterExperiment} classs and indicates that the filtering
  statistics should be calculated based on samples that are unassigned by the
  designated clustering. The name given to the filter in this case is of the
  form \code{<filterStats>_<clusterLabel>}, i.e. the clustering label of the
  clustering is appended to the standard name for the filtering statistic.

Note that \code{filterData} returns a SingleCellExperiment object.
  To get the actual data out use either assay or \code{\link{transformData}}
  if transformed data is desired.

The PCA method uses either \code{prcomp} from the \code{stats}
  package or  \code{svds} from the \code{RSpectra} package to perform PCA.
  Both are called on \code{t(assay(x))} with \code{center=TRUE} and
  \code{scale=TRUE} (i.e. the feature are centered and scaled), so that it is
  performing PCA on the correlation matrix of the features.

Note that this function does not check if such a reduceDim value already exists, and will recalculate (and overwrite) if it does.
}
\examples{
data(simData)
listBuiltInFilterStats()
scf<-makeFilterStats(simData,filterStats=c("var","mad"))
scf
scfFiltered<-filterData(scf,filterStats="mad",percentile=10)
scfFiltered
assay(scfFiltered)[1:10,1:10]
data(simData)
listBuiltInReducedDims()
scf<-makeReducedDims(simData, reducedDims="PCA", maxDims=3)
scf
}
\seealso{
\code{\link{makeFilterStats}},\code{\link{makeReducedDims}},
  \code{\link{filterData}}, \code{\link[SingleCellExperiment]{reducedDim}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getParams.R
\name{getClusterManyParams,ClusterExperiment-method}
\alias{getClusterManyParams,ClusterExperiment-method}
\alias{getClusterManyParams}
\title{Get parameter values of clusterMany clusters}
\usage{
\S4method{getClusterManyParams}{ClusterExperiment}(
  x,
  whichClusters = "clusterMany",
  searchAll = FALSE,
  simplify = TRUE
)
}
\arguments{
\item{x}{a ClusterExperiment object that contains clusterings from running
\code{\link{clusterMany}}.}

\item{whichClusters}{argument that can be either numeric or character vector
indicating the clusterings to be used. See details of \code{\link{getClusterIndex}}.}

\item{searchAll}{logical, indicating whether all clusterings with 
\code{clusterMany} label should be allowed (i.e. including those from
 previous ones with labels like \code{clusterMany.1}), or only limited 
 to those in most recent workflow (default).}

\item{simplify}{logical. Whether to simplify the output so as to remove 
features that do not change across the clusterings.}
}
\value{
Returns a data.frame where the column names are the parameter names,
  and the entries are the values of the parameter for the indicated
  clustering. The column 'clusteringIndex' identifies the index of the
  clustering in the full set of clusterings of the given ClusterExperiment
  object.
}
\description{
Takes an input a ClusterExperiment object and returns the
  parameter values used in creating the clusters that were created by
  'clusterMany'
}
\details{
The method simply parses the \code{clusterLabels} of the indicated
  clusterings, relying on the specific format used by \code{clusterMany} to
  create labels. The function will only allow the parsing to be performed on
  those clusterings with a 'clusterMany' clusterType. If the user has
  manipulated the clusterLabels manually or manually identified the
  clusterType of a clustering as 'clusterMany', this function may create
  unexpected results or errors. Similarly, it cannot be used on 'clusterMany'
  results from an old iteration (e.g. that have type 'clusterMany.1')

Specifically, it splits the label of each clustering by the
  character ",", as indicating the different parameters; this should return a
  value of form "ABC=123". The function then pulls out the numeric value
  ('123') and associates that value as the value of the parameter ('ABC')
}
\examples{
\dontrun{
data(simData)

cl <- clusterMany(simData, nReducedDims=c(5, 10), reduceMethod="PCA",
     clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE), 
     makeMissingDiss=TRUE, removeSil=c(TRUE,FALSE))
getClusterManyParams(cl)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeConsensus.R
\name{makeConsensus}
\alias{makeConsensus}
\alias{makeConsensus,matrix-method}
\alias{makeConsensus,ClusterExperiment-method}
\title{Find sets of samples that stay together across clusterings}
\usage{
\S4method{makeConsensus}{matrix}(
  x,
  proportion,
  clusterFunction = "hierarchical01",
  minSize = 5,
  propUnassigned = 0.5,
  whenUnassign = c("before", "after"),
  clusterArgs = NULL
)

\S4method{makeConsensus}{ClusterExperiment}(
  x,
  whichClusters,
  eraseOld = FALSE,
  clusterLabel = "makeConsensus",
  ...
)
}
\arguments{
\item{x}{a matrix with samples on the rows and different clusterings on the
columns or \code{\link{ClusterExperiment}} object.}

\item{proportion}{The proportion of times that two sets of samples should be 
together in order to be grouped into a cluster (if <1, passed to
mainClustering via alpha = 1 - proportion)}

\item{clusterFunction}{the clustering function to use (passed to 
\code{\link{mainClustering}}); currently must be of type '01' and accept as
input matrices of type "cat" (see details of ?ClusterFunction).}

\item{minSize}{minimum size required for a set of samples to be considered in 
a cluster because of shared clustering, passed to
\code{\link{mainClustering}}}

\item{propUnassigned}{samples with greater than this proportion of assignments
equal to '-1' are assigned a '-1' cluster value as a last step (only if
proportion < 1)}

\item{whenUnassign}{(provided for back compatibility with previous versions).
Must be one of "before" or "after", indicating at what point are samples
with a proportion of assignments of -1 greater than \code{propUnassigned}
forced to have a '-1' value. If "before", then these samples are removed
and not used for clustering. If "after", these samples are included in the
clustering step, but then the cluster values they receive are assigned a
'-1. These choices may result in different clusterings, because if these
samples are included in the clustering (i.e. \code{whenUnassign="after"},
then these samples may affect the cluster assignments of other samples. The
default is currently "before", but previous to version 2.5.4, there was no
such option and the code internally set to "after", so for reproducibility
with older results, users may need to set this option.}

\item{clusterArgs}{list of arguments to be passed to the call to 
\code{\link{mainClustering}} that is used to cluster the proportion 
overlap between samples.}

\item{whichClusters}{argument that can be either numeric or character vector
indicating the clusterings to be used. See details of \code{\link{getClusterIndex}}.}

\item{eraseOld}{logical. Only relevant if input \code{x} is of class
\code{ClusterExperiment}. If TRUE, will erase existing workflow results
(clusterMany as well as mergeClusters and makeConsensus). If FALSE, existing
workflow results will have "\code{_i}" added to the clusterTypes value,
where \code{i} is one more than the largest such existing workflow
clusterTypes.}

\item{clusterLabel}{a string used to describe the type of clustering. By
default it is equal to "makeConsensus", to indicate that this clustering is
the result of a call to makeConsensus. However, a more informative label can
be set (see vignette).}

\item{...}{arguments to be passed on to the method for signature 
\code{matrix,missing}.}
}
\value{
If x is a matrix, a list with values \itemize{ 
 \item{\code{clustering}}{ vector of cluster assignments, with "-1" implying 
 unassigned}
 
 \item{\code{percentageShared}}{ a nSample x nSample matrix of the percent 
 co-occurance across clusters used to find the final clusters. Percentage is 
 out of those not '-1'} \item{\code{noUnassignedCorrection}{ a vector of 
 cluster assignments before samples were converted to '-1' because had 
 >\code{propUnassigned} '-1' values (i.e. the direct output of the 
 \code{mainClustering} output.)}} }

If x is a \code{\link{ClusterExperiment}}, a
 \code{\link{ClusterExperiment}} object, with an added clustering of
 clusterTypes equal to \code{makeConsensus} and the \code{percentageShared}
 matrix stored in the \code{coClustering} slot.
}
\description{
Find sets of samples that stay together across clusterings in order to define 
a new clustering vector.
}
\details{
This function was previously called \code{combineMany} (versions <=
  2.0.0). \code{combineMany} is still available, but is considered defunct
  and users should update their code accordingly.

The function tries to find a consensus cluster across many different 
 clusterings of the same samples. It does so by creating a \code{nSamples} x 
 \code{nSamples} matrix of the percentage of co-occurance of each sample and 
 then calling mainClustering to cluster the co-occurance matrix. The function
 assumes that '-1' labels indicate clusters that are not assigned to a 
 cluster. Co-occurance with the unassigned cluster is treated differently 
 than other clusters. The percent co-occurance is taken only with respect to 
 those clusterings where both samples were assigned. Then samples with more 
 than \code{propUnassigned} values that are '-1' across all of the 
 clusterings are assigned a '-1' regardless of their cluster assignment.

The method calls \code{\link{mainClustering}} on the proportion
 matrix with \code{clusterFunction} as the 01 clustering algorithm,
 \code{alpha=1-proportion}, \code{minSize=minSize}, and
 \code{evalClusterMethod=c("average")}. See help of 
 \code{\link{mainClustering}} for more details.
}
\examples{
\dontrun{
data(simData)

cl <- clusterMany(simData,nReducedDims=c(5,10,50),  reduceMethod="PCA",
clusterFunction="pam", ks=2:4, findBestK=c(FALSE), removeSil=TRUE,
makeMissingDiss=TRUE, subsample=FALSE)

#make names shorter for plotting
clMat <- clusterMatrix(cl)
colnames(clMat) <- gsub("TRUE", "T", colnames(clMat))
colnames(clMat) <- gsub("FALSE", "F", colnames(clMat))
colnames(clMat) <- gsub("k=NA,", "", colnames(clMat))

#require 100\% agreement -- very strict
clCommon100 <- makeConsensus(clMat, proportion=1, minSize=10)

#require 70\% agreement based on clustering of overlap
clCommon70 <- makeConsensus(clMat, proportion=0.7, minSize=10)

oldpar <- par(no.readonly = TRUE)
par(mar=c(1.1, 12.1, 1.1, 1.1))
plotClusters(cbind("70\%Similarity"=clCommon70, clMat,
"100\%Similarity"=clCommon100), axisLine=-2)

#method for ClusterExperiment object
clCommon <- makeConsensus(cl, whichClusters="workflow", proportion=0.7,
minSize=10)
plotClusters(clCommon)
par(oldpar)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotClusters.R
\docType{methods}
\name{plotClusters}
\alias{plotClusters}
\alias{plotClusters,ClusterExperiment-method}
\alias{plotClusters,matrix-method}
\title{Visualize cluster assignments across multiple clusterings}
\usage{
\S4method{plotClusters}{ClusterExperiment}(
  object,
  whichClusters,
  existingColors = c("ignore", "all", "firstOnly"),
  resetNames = FALSE,
  resetColors = FALSE,
  resetOrderSamples = FALSE,
  colData = NULL,
  clusterLabels = NULL,
  ...
)

\S4method{plotClusters}{matrix}(
  object,
  orderSamples = NULL,
  colData = NULL,
  reuseColors = FALSE,
  matchToTop = FALSE,
  plot = TRUE,
  unassignedColor = "white",
  missingColor = "grey",
  minRequireColor = 0.3,
  startNewColors = FALSE,
  colPalette = massivePalette,
  input = c("clusters", "colors"),
  clusterLabels = colnames(object),
  add = FALSE,
  xCoord = NULL,
  ylim = NULL,
  tick = FALSE,
  ylab = "",
  xlab = "",
  axisLine = 0,
  box = FALSE,
  ...
)
}
\arguments{
\item{object}{A matrix of with each column corresponding to a clustering
and each row a sample or a \code{\link{ClusterExperiment}} object. If a
matrix, the function will plot the clusterings in order of this matrix, and
their order influences the plot greatly.}

\item{whichClusters}{argument that can be either numeric or character vector
indicating the clusterings to be used. See details of \code{\link{getClusterIndex}}.}

\item{existingColors}{how to make use of the exiting colors in the 
\code{ClusterExperiment} object. 'ignore' will ignore them and assign new 
colors. 'firstOnly' will use the existing colors of only the 1st 
clustering, and then align the remaining clusters and give new colors for
the remaining only. 'all' will use all of the existing colors.}

\item{resetNames}{logical. Whether to reset the names of the clusters in
\code{clusterLegend} to be the aligned integer-valued ids from
\code{plotClusters}.}

\item{resetColors}{logical. Whether to reset the colors in
\code{clusterLegend} in the \code{ClusterExperiment} returned to be the
colors from the \code{plotClusters}.}

\item{resetOrderSamples}{logical. Whether to replace the existing
\code{orderSamples} slot in the \code{ClusterExperiment} object with the
new order found.}

\item{colData}{If \code{clusters} is a matrix, \code{colData} gives a 
matrix of additional cluster/sample data on the samples to be plotted with 
the clusterings given in clusters. Values in \code{colData} will be 
added to the end (bottom) of the plot. NAs in the \code{colData} matrix
will trigger an error. If \code{clusters} is a \code{ClusterExperiment}
object, the input in \code{colData} refers to columns of the the
\code{colData} slot of the \code{ClusterExperiment} object to be plotted
with the clusters. In this case, \code{colData} can be TRUE (i.e. all
columns will be plotted), or an index or a character vector that references
a column or column name, respectively, of the \code{colData} slot of the
\code{ClusterExperiment} object. If there are NAs in the \code{colData}
columns, they will be encoded as 'unassigned' and receive the same color as
'unassigned' samples in the clustering.}

\item{clusterLabels}{names to go with the columns (clusterings) in matrix 
\code{colorMat}. If \code{colData} argument is not \code{NULL}, the 
\code{clusterLabels} argument must include names for the sample data too. 
If the user gives only names for the clusterings, the code will try to 
anticipate that and use the column names of the sample data, but this is
fragile. If set to \code{FALSE}, then no labels will be plotted.}

\item{...}{for \code{plotClusters} arguments passed either to the method
of \code{plotClusters} for matrices, or ultimately to \code{\link{plot}}
(if \code{add=FALSE}).}

\item{orderSamples}{A predefined order in which the samples will be plotted.
Otherwise the order will be found internally by aligning the clusters
(assuming \code{input="clusters"})}

\item{reuseColors}{Logical. Whether each row should consist of the same set
of colors. By default (FALSE) each cluster that the algorithm doesn't
identify to the previous rows clusters gets a new color.}

\item{matchToTop}{Logical as to whether all clusters should be aligned to the
first row. By default (FALSE) each cluster is aligned to the ordered
clusters of the row above it.}

\item{plot}{Logical as to whether a plot should be produced.}

\item{unassignedColor}{If ``-1'' in \code{clusters}, will be given this color
(meant for samples not assigned to cluster).}

\item{missingColor}{If ``-2'' in clusters, will be given this color (meant
for samples that were missing from the clustering, mainly when comparing
clusterings run on different sets of samples)}

\item{minRequireColor}{In aligning colors between rows of clusters, require
this percent overlap.}

\item{startNewColors}{logical, indicating whether in aligning colors between
rows of clusters, should the colors restart at beginning of colPalette as
long as colors are not in immediately proceeding row (the colors at the end 
of \code{massivePalette} are all of \code{\link{colors}()} and many will be
indistinguishable, so this option can be useful if you have a large cluster
matrix).}

\item{colPalette}{a vector of colors used for the different clusters. Must be
as long as the maximum number of clusters found in any single
clustering/column given in \code{clusters} or will otherwise return an
error.}

\item{input}{indicate whether the input matrix is matrix of integer assigned
clusters, or contains the colors. If \code{input="colors"}, then the object
\code{clusters} is a matrix of colors and there is no alignment (this
option allows the user to manually adjust the colors and replot, for
example).}

\item{add}{whether to add to existing plot.}

\item{xCoord}{values on x-axis at which to plot the rows (samples).}

\item{ylim}{vector of limits of y-axis.}

\item{tick}{logical, whether to draw ticks on x-axis for each sample.}

\item{ylab}{character string for the label of y-axis.}

\item{xlab}{character string for the label of x-axis.}

\item{axisLine}{the number of lines in the axis labels on y-axis should be
(passed to \code{line = ...} in the axis call).}

\item{box}{logical, whether to draw box around the plot.}
}
\value{
If \code{clusters} is a \code{\link{ClusterExperiment}} Object, then
  \code{plotClusters} returns an updated \code{ClusterExperiment} object,
  where the \code{clusterLegend} and/or \code{orderSamples} slots have been
  updated (depending on the arguments).

If \code{clusters} is a matrix, \code{plotClusters} returns
  (invisibly) the orders and other things that go into making the matrix.
  Specifically, a list with the following elements.
\itemize{

\item{\code{orderSamples}}{ a vector of length equal to 
\code{nrows(clusters)} giving the order of the samples (rows) to 
 use to get the original clusters matrix
into the order made by \code{plotClusters}.}

\item{\code{colors}}{ matrix of color assignments for each element of
original clusters matrix. Matrix is in the same order as original clusters
matrix. The matrix \code{colors[orderSamples,]} is the matrix that can be 
given back to \code{plotClusters} to recreate the plot (see examples).}

\item{\code{alignedClusterIds}}{ a matrix of integer valued cluster
assignments that match the colors. This is useful if you want to have cluster
identification numbers that are better aligned than that given in the
original clusters. Again, the rows/samples are in same order as original 
input matrix.}

\item{\code{clusterLegend}}{ list of length equal to the number of columns of
input matrix. The elements of the list are matrices, each with three columns
named "Original","Aligned", and "Color" giving, respectively, the
correspondence between the original cluster ids in \code{clusters}, the
aligned cluster ids in \code{aligned}, and the color.}

\item{\code{origClusters}}{The original matrix of clusters given to
\code{plotClusters}}
}
}
\description{
Align multiple clusterings of the same set of samples and provide a
color-coded plot of their shared cluster assignments
}
\details{
All arguments of the matrix version can be passed to the
  \code{ClusterExperiment} version. As noted above, however, some arguments
  have different interpretations.

If \code{whichClusters = "workflow"}, then the workflow clusterings
  will be plotted in the following order: final, mergeClusters, makeConsensus,
  clusterMany.
}
\examples{
\dontrun{
#clustering using pam: try using different dimensions of pca and different k
data(simData)

cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reduceMethod="PCA",
clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)

clusterLabels(cl)

#make names shorter for better plotting
x <- clusterLabels(cl)
x <- gsub("TRUE", "T", x)
x <- gsub("FALSE", "F", x)
x <- gsub("k=NA,", "", x)
x <- gsub("Features", "", x)
clusterLabels(cl) <- x

par(mar=c(2,10,1,1))
#this will make the choices of plotClusters
cl <- plotClusters(cl, axisLine=-1, resetOrderSamples=TRUE, resetColors=TRUE)

#see the new cluster colors
clusterLegend(cl)[1:2]

#We can also change the order of the clusterings. Notice how this
#dramatically changes the plot!
clOrder <- c(3:6, 1:2, 7:ncol(clusterMatrix(cl)))
cl <- plotClusters(cl, whichClusters=clOrder, resetColors=TRUE,
resetOrder=TRUE, axisLine=-2)

#We can manually switch the red ("#E31A1C") and green ("#33A02C") in the
#first cluster:

#see what the default colors are and their names
showPalette(wh=1:5)

#change "#E31A1C" to "#33A02C"
newColorMat <- clusterLegend(cl)[[clOrder[1]]]
newColorMat[2:3, "color"] <- c("#33A02C", "#E31A1C")
clusterLegend(cl)[[clOrder[1]]]<-newColorMat

#replot by setting 'input="colors"'
par(mfrow=c(1,2))
plotClusters(cl, whichClusters=clOrder, orderSamples=orderSamples(cl),
existingColors="all")
plotClusters(cl, whichClusters=clOrder, resetColors=TRUE, resetOrder=TRUE,
axisLine=-2)
par(mfrow=c(1,1))

#set some of clusterings arbitrarily to "-1", meaning not clustered (white),
#and "-2" (another possible designation getting gray, usually for samples not
#included in original clustering)
clMatNew <- apply(clusterMatrix(cl), 2, function(x) {
wh <- sample(1:nSamples(cl), size=10)
x[wh]<- -1
wh <- sample(1:nSamples(cl), size=10)
x[wh]<- -2
return(x)
})

#make a new object
cl2 <- ClusterExperiment(assay(cl), clMatNew,
transformation=transformation(cl))
plotClusters(cl2)
}
}
\references{
Wilkerson, D. M, Hayes and Neil D (2010). "ConsensusClusterPlus: a class discovery tool with confidence assessments and item tracking." Bioinformatics, 26(12), pp. 1572-1573.
}
\seealso{
The \code{\link[ConsensusClusterPlus]{ConsensusClusterPlus}} package.
}
\author{
Elizabeth Purdom and Marla Johnson (based on the tracking plot in
  \link[ConsensusClusterPlus]{ConsensusClusterPlus} by Matt Wilkerson and Peter
  Waltman).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeBlankData.R, R/plottingHelpers.R
\docType{data}
\name{plottingFunctions}
\alias{plottingFunctions}
\alias{makeBlankData}
\alias{convertClusterLegend,ClusterExperiment-method}
\alias{convertClusterLegend}
\alias{showPalette}
\alias{bigPalette}
\alias{massivePalette}
\alias{setBreaks}
\alias{showHeatmapPalettes}
\alias{seqPal5}
\alias{seqPal2}
\alias{seqPal3}
\alias{seqPal4}
\alias{seqPal1}
\alias{plotClusterLegend,ClusterExperiment-method}
\alias{plotClusterLegend}
\title{Convert clusterLegend into useful formats}
\format{
An object of class \code{character} of length 56.

An object of class \code{character} of length 484.

An object of class \code{character} of length 16.

An object of class \code{character} of length 14.

An object of class \code{character} of length 11.

An object of class \code{character} of length 13.

An object of class \code{character} of length 11.
}
\usage{
makeBlankData(
  data,
  groupsOfFeatures = NULL,
  groupsOfSamples = NULL,
  nBlankFeatures = 1,
  nBlankSamples = 1
)

\S4method{convertClusterLegend}{ClusterExperiment}(
  object,
  output = c("plotAndLegend", "aheatmapFormat", "matrixNames", "matrixColors"),
  whichClusters = ifelse(output == "plotAndLegend", "primary", "all")
)

showPalette(colPalette = bigPalette, which = NULL, cex = 1)

bigPalette

massivePalette

setBreaks(data, breaks = NA, makeSymmetric = FALSE, returnBreaks = TRUE)

showHeatmapPalettes()

seqPal5

seqPal2

seqPal3

seqPal4

seqPal1

\S4method{plotClusterLegend}{ClusterExperiment}(
  object,
  whichCluster = "primary",
  clusterNames,
  title,
  add = FALSE,
  location = if (add) "topright" else "center",
  ...
)
}
\arguments{
\item{data}{matrix with samples on columns and features on rows.}

\item{groupsOfFeatures}{list, with each element of the list containing a
vector of numeric indices of features (rows).}

\item{groupsOfSamples}{list, with each element of the list containing a vector
of numeric indices of samples (columns).}

\item{nBlankFeatures}{the number of blank lines to add in the data matrix to
separate the groups of feature indices (will govern the amount of white
space if data is then fed to heatmap.)}

\item{nBlankSamples}{the number of blank lines to add in the data matrix to
separate the groups of sample indices (will govern the amount of white space
if data is then fed to heatmap.)}

\item{object}{a \code{ClusterExperiment} object.}

\item{output}{character value, indicating desired type of conversion.}

\item{whichClusters}{argument that can be either numeric or character vector
indicating the clusterings to be used. See details of \code{\link{getClusterIndex}}.}

\item{colPalette}{a vector of character colors. By default, the palette 
\code{bigPalette} is used}

\item{which}{numeric. Which colors to plot. Must be a numeric vector with
values between 1 and length of \code{colPalette}. If missing, all colors
plotted.}

\item{cex}{numeric value giving the cex for the text of the plot.}

\item{breaks}{either vector of breaks, or number of breaks (integer) or a
number between 0 and 1 indicating a quantile, between which evenly spaced
breaks should be calculated. If missing or NA, will determine evenly spaced
breaks in the range of the data.}

\item{makeSymmetric}{whether to make the range of the breaks symmetric around zero (only used if not all of the data is non-positive and not all of the data is non-negative)}

\item{returnBreaks}{logical as to whether to return the vector of breaks. See details.}

\item{whichCluster}{argument that can be a single numeric or character value
indicating the \emph{single} clustering to be used. Giving values that result in more than one clustering will result in an error. See details of \code{\link{getClusterIndex}}.}

\item{clusterNames}{vector of names for the clusters; vector should have names
that correspond to the clusterIds in the ClusterExperiment object. If this
argument is missing, will use the names in the "name" column of the clusterLegend
slot of the object.}

\item{title}{title for the clusterLegend plot}

\item{add}{logical. Whether legend should be added to the existing plot.}

\item{location}{character passed to \code{x} argument of legend indicating 
where to place legend.}

\item{...}{arguments passed to legend}
}
\value{
\code{makeBlankData} returns a list with items 
\itemize{
 \item{"dataWBlanks"}{ The data with the rows of NAs separating the given
 indices.} 
 \item{"rowNamesWBlanks"}{ A vector of characters giving the
 rownames for the data, including blanks for the NA rows. These are not given
 as rownames to the returned data because they are not necessarily unique.
 However, they can be given to the \code{labRow} argument of
 \code{\link[NMF]{aheatmap}} or \code{\link{plotHeatmap}}.}
 \item{"colNamesWBlanks"}{ A vector of characters giving the colnames for the
 data, including blanks for the NA rows. They can be given to the
 \code{labCol} argument of \code{\link[NMF]{aheatmap}} or
 \code{\link{plotHeatmap}}.} 
 \item{"featureGroupNamesWBlanks"}{ A vector of
 characters of the same length as the number of rows of the new data (i.e.
 with blanks) giving the group name for the data, indicating which group
 (i.e. which element of \code{groupsOfFeatures} list) the feature came from.
 If \code{groupsOfFeatures} has unique names, these names will be used, other
 wise "Feature Group1", "Feature Group2", etc. The NA rows are given NA
 values. } 
 \item{"sampleGroupNamesWBlanks"}{ A vector of characters of the
 same length as the number of columns of the new data (i.e. with blanks)
 giving the group name for the data, indicating which group (i.e. which
 element of \code{groupsOfFeatures} list) the feature came from. If
 \code{groupsOfFeatures} has unique names, these names will be used, other
 wise "SampleGroup1", "Group2", etc. The NA rows are given NA values. } }

If \code{output="plotAndLegend"}, \code{"convertClusterLegend"} will
  return a list that provides the necessary information to color samples
  according to cluster and create a legend for it:
 \itemize{
 \item{"colorVector"}{ A vector the same length as the number of samples,
 assigning a color to each cluster of the primaryCluster of the object.}
 \item{"legendNames"}{ A vector the length of the number of clusters of
 primaryCluster of the object giving the name of the cluster.}
 \item{"legendColors"}{ A vector the length of the number of clusters of
 primaryCluster of the object giving the color of the cluster.}
}

If \code{output="aheatmap"} a conversion of the clusterLegend to be
  in the format requested by \code{\link[NMF]{aheatmap}}. The column 'name'
  is used for the names and the column 'color' for the color of the clusters.

If \code{output="matrixNames"} or \code{"matrixColors"} a matrix the
  same dimension of \code{clusterMatrix(object)}, but with the cluster color
  or cluster name instead of the clusterIds, respectively.
}
\description{
Function for converting the information stored in the clusterLegend slot into
other useful formats.

Most of these functions are called internally by plotting
  functions, but are exported in case the user finds them useful.
}
\details{
\code{makeBlankData} pulls the data corresponding to the row indices
 in \code{groupsOfFeatures} adds lines of NA values into data between these
 groups. When given to heatmap, will create white space between these groups
 of features.

convertClusterLegend pulls out information stored in the
  \code{clusterLegend} slot of the object and returns it in useful format.

\code{bigPalette} is a long palette of colors (length 58) used by 
  \code{\link{plotClusters}} and accompanying functions. \code{showPalette}
  creates plot that gives index of each color in a vector of colors.
  \code{massivePalette} is a combination of \code{bigPalette} and the
  non-grey colors of \code{\link{colors}()} (length 487). 
  \code{massivePalette} is mainly useful for when doing
  \code{\link{plotClusters}} of a very large number of clusterings, each with
  many clusters, so that the code doesn't run out of colors. However, many of
  the colors will be very similar to each other.

\code{showPalette} will plot the \code{colPalette} colors with their
  labels and index.

if returnBreaks if FALSE, instead of returning the vector of breaks, the function will just return the second smallest and second largest value of the breaks. This is useful for alternatively just setting values of the data matrix larger than these values to this value if breaks was a percentile. This argument is only used if \code{breaks<1}, indicating truncating the breaks for large values of data.

\code{setBreaks} gives a set of breaks (of length 52) equally spaced
  between the boundaries of the data. If breaks is between 0 and 1, then the
  evenly spaced breaks are between these quantiles of the data.

\code{seqPal1}-\code{seqPal4} are palettes for the heatmap.
  \code{showHeatmapPalettes} will show you these palettes.
}
\examples{
data(simData)

x <- makeBlankData(simData[,1:10], groupsOfFeatures=list(c(5, 2, 3), c(20,
34, 25)))
plotHeatmap(x$dataWBlanks,clusterFeatures=FALSE)
showPalette()
showPalette(massivePalette,cex=0.6)
setBreaks(data=simData,breaks=.9)

#show the palette colors
showHeatmapPalettes()

#compare the palettes on heatmap
cl <- clusterSingle(simData, subsample=FALSE,
sequential=FALSE, 
mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=8)))

\dontrun{
par(mfrow=c(2,3))
plotHeatmap(cl, colorScale=seqPal1, main="seqPal1")
plotHeatmap(cl, colorScale=seqPal2, main="seqPal2")
plotHeatmap(cl, colorScale=seqPal3, main="seqPal3")
plotHeatmap(cl, colorScale=seqPal4, main="seqPal4")
plotHeatmap(cl, colorScale=seqPal5, main="seqPal5")
par(mfrow=c(1,1))
}

}
\seealso{
\code{\link{plotHeatmap}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecateFunctions.R
\name{clusterExperiment-deprecated}
\alias{clusterExperiment-deprecated}
\alias{combineMany}
\alias{combineMany,ANY-method}
\title{Deprecated functions in package \sQuote{clusterExperiment}}
\usage{
\S4method{combineMany}{ANY}(x, ...)
}
\arguments{
\item{x}{any object}

\item{...}{additional arguments}
}
\description{
These functions are provided for compatibility with older versions
 of \sQuote{clusterExperiment} only, and will be defunct at the next release.
}
\details{
The following functions are deprecated and will be made defunct; use
  the replacement indicated below:
 \itemize{
   \item{\code{combineMany}: \code{\link{makeConsensus}}}
   \item{\code{removeUnclustered}: \code{\link{removeUnassigned}}}
 }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllHelperClusterFunction.R
\name{ClusterFunction-methods}
\alias{ClusterFunction-methods}
\alias{requiredArgs,ClusterFunction-method}
\alias{requiredArgs,list-method}
\alias{requiredArgs,character-method}
\alias{requiredArgs}
\alias{requiredArgs,factor-method}
\alias{algorithmType,ClusterFunction-method}
\alias{algorithmType}
\alias{algorithmType,character-method}
\alias{algorithmType,factor-method}
\alias{algorithmType,list-method}
\alias{inputType,ClusterFunction-method}
\alias{inputType}
\alias{inputType,list-method}
\alias{inputType,character-method}
\alias{inputType,factor-method}
\title{Helper methods for the ClusterFunction class}
\usage{
\S4method{requiredArgs}{character}(object)

\S4method{requiredArgs}{ClusterFunction}(object, genericOnly = FALSE)

\S4method{requiredArgs}{list}(object)

\S4method{requiredArgs}{character}(object)

\S4method{requiredArgs}{character}(object)

\S4method{requiredArgs}{factor}(object)

\S4method{algorithmType}{ClusterFunction}(object)

\S4method{algorithmType}{character}(object)

\S4method{algorithmType}{factor}(object)

\S4method{algorithmType}{list}(object)

\S4method{inputType}{ClusterFunction}(object)

\S4method{inputType}{list}(object)

\S4method{inputType}{character}(object)

\S4method{inputType}{factor}(object)
}
\arguments{
\item{object}{input to the method, either a \code{ClusterFunction}
class or a character describing a built-in \code{ClusterFunction} object. 
Can also be a \code{list} of \code{ClusterFunction} objects, in which case 
the list must have names for each function.}

\item{genericOnly}{logical If TRUE, return only the generic required
arguments (i.e. those required by the algorithm type) and not the arguments
specific to that clustering found in the slot \code{requiredArgs}. If FALSE
both sets of arguments are returned.}
}
\value{
\code{requiredArgs}  returns a list of the required args of a
  function (via a call to \code{\link{requiredArgs}})

\code{algorithmType} returns a character value giving the type of 
clustering function ("01" or "K")

\code{inputType} returns a character value giving the input 
type of the object
}
\description{
This is a collection of helper methods for the ClusterExperiment class.
}
\details{
Note that when subsetting the data, the dendrogram information and 
  the co-clustering matrix are lost.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllHelper.R
\name{renameClusters}
\alias{renameClusters}
\alias{renameClusters,ClusterExperiment,character-method}
\alias{recolorClusters,ClusterExperiment,character-method}
\alias{recolorClusters}
\title{Change assigned names or colors of clusters}
\usage{
\S4method{renameClusters}{ClusterExperiment,character}(
  object,
  value,
  whichCluster = "primary",
  matchTo = c("name", "clusterIds")
)

\S4method{recolorClusters}{ClusterExperiment,character}(
  object,
  value,
  whichCluster = "primary",
  matchTo = c("name", "clusterIds")
)
}
\arguments{
\item{value}{The value to be substituted in the corresponding slot. See the
slot descriptions in \code{\link{ClusterExperiment}} for details on what
objects may be passed to these functions.}

\item{whichCluster}{argument that can be a single numeric or character value
indicating the \emph{single} clustering to be used. Giving values that result in more than one clustering will result in an error. See details of \code{\link{getClusterIndex}}.}

\item{matchTo}{whether to match to the cluster name
(\code{"name"}) or internal cluster id (\code{"clusterIds"})}
}
\value{
\code{renameClusters} changes the names assigned to clusters within a
  clustering

\code{recolorClusters} changes the colors assigned to clusters within
  a clustering
}
\description{
Change the assigned names or colors of the clusters in a
  clustering stored in the clusterLegend slot of the object.
}
\examples{
#create CE object
data(simData)
cl1 <- clusterSingle(simData, subsample=FALSE,
sequential=FALSE, mainClusterArgs=list(clusterArgs=list(k=3), 
clusterFunction="pam"))
#Give names to the clusters
clusterLegend(cl1)
cl1<-renameClusters(cl1, c("1"="A","2"="B","3"="C"), matchTo="clusterIds")
clusterLegend(cl1)
# Change name of single one
cl1<-renameClusters(cl1, c("1"="D"), matchTo="clusterIds")
clusterLegend(cl1)
# Match to existing name, rather than clusterId
cl1<-renameClusters(cl1, c("B"="N"), matchTo="name")
clusterLegend(cl1)
# Change colors in similar way
cl1<-recolorClusters(cl1, c("N"="red"),matchTo=c("name"))
clusterLegend(cl1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getClusterIndex.R
\name{getClusterIndex}
\alias{getClusterIndex}
\alias{getClusterIndex,ClusterExperiment-method}
\alias{getSingleClusterIndex,ClusterExperiment-method}
\alias{getSingleClusterIndex}
\title{getClusterIndex}
\usage{
\S4method{getClusterIndex}{ClusterExperiment}(
  object,
  whichClusters,
  noMatch = c("silentlyRemove", "throwError")
)

\S4method{getSingleClusterIndex}{ClusterExperiment}(object, whichCluster, passedArgs = NULL, ...)
}
\arguments{
\item{object}{a ClusterExperiment object}

\item{whichClusters}{argument that can be either numeric or character vector
indicating the clusterings to be used. See details of \code{\link{getClusterIndex}}.}

\item{noMatch}{how to handle if there is no match to an given value of \code{whichClusters}.
"silentlyRemove" means that no error will be given, and the result will be just those that
do match (resulting in a vector of length zero if there are none that match). "throwError"
means that the function will stop with an error describing the problem with the match.}

\item{whichCluster}{argument that can be a single numeric or character value
indicating the \emph{single} clustering to be used. Giving values that result in more than one clustering will result in an error. See details of \code{\link{getClusterIndex}}.}

\item{passedArgs}{other arguments passed to the function (only used internally)}

\item{...}{Not for user use. Argument allows function \code{getSingleClusterIndex}
to catch the wrong argument (the plural \code{whichClusters} argument rather than 
singular \code{whichCluster}).}
}
\value{
\code{getClusterIndex} returns a vector of all numeric indices that are indicated by the requested \code{whichClusters}. Note that there is not a one-to-one match between input values and returned values since there may be more than one value for a given value of \code{whichClusters} or no value at all.
}
\description{
Finds index of clustering in clusterMatrix slot of object based on descriptions of clusters.
}
\details{
The function \code{getClusterIndex} is largely used internally to parse the argument \code{whichClusters} which is used as an argument extensively across functions in this package. Note that some functions require the match return a single clustering, in which case those functions use the function \code{getSingleClusterIndex} with the singular argument \code{whichCluster} and returns an error if it indicates more than one clustering. Furthermore \code{getSingleClusterIndex} does not allow for any mismatches (\code{noMatch="throwError"}. Otherwise the parsing of the two arguments \code{whichClusters} and \code{whichCluster} is the same, and is described in what follows.

If \code{whichClusters} is numeric, then the function just returns the 
 numeric values of \code{whichClusters}, after checking that they are valid. If any are invalid, they are silently removed if \code{silentlyRemove=TRUE}. The values will be returned  \emph{in the order given}, so this argument can also be used to defined by functions to give an
  ordering for the clusterings (as relevant).

If \code{whichClusters} is a character value, then it the function attempts to use the character value to identify the clustering. The value of the argument is first matched against a set of "special" values: "workflow","all","none","primaryCluster","dendro" using the argument \code{\link{match.arg}}, which does partial matching. If whichClusters is a vector of values, only the first value of the vector is matched against these values and if it matches, the remaining values are ignored.  If it matches one of these values, then the cluster indices are given as follows.
\itemize{
 \item{"workflow"}{all clusterings in the current workflow (see \code{\link{workflowClusters}})}
 \item{"all"}{all clusterings, with the primary clustering put first.}
 \item{"none"}{no clusterings}
 \item{"primaryCluster"}{the primary clustering index as given by \code{\link{primaryClusterIndex}}}
 \item{"dendro"}{the index of the clustering given to create the cluster dendrogram, if it exists}
}
 @details If \code{whichClusters} is a character value, but its first element does not match these predesignated values, then all the values of \code{whichClusters} are attempted to be matched to the \code{\link{clusterTypes}} of the object. Note that there may be more than one clustering that matches a given type. For any entries that do not match a value in  \code{clusterTypes(object)} are then matched based on the value of \code{\link{clusterLabels}} of the object.
}
\examples{
#load CE object
data(rsecFluidigm)
# get the cluster index from mergeClusters step
getClusterIndex(rsecFluidigm,whichClusters="mergeClusters")
# get the cluster indices from mergeClusters step
getClusterIndex(rsecFluidigm,whichClusters="clusterMany")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeDendrogram.R
\name{makeDendrogram}
\alias{makeDendrogram}
\alias{makeDendrogram,ClusterExperiment-method}
\alias{makeDendrogram,dist-method}
\alias{makeDendrogram,matrixOrHDF5-method}
\title{Make hierarchy of set of clusters}
\usage{
\S4method{makeDendrogram}{ClusterExperiment}(
  x,
  whichCluster = "primaryCluster",
  reduceMethod = "mad",
  nDims = defaultNDims(x, reduceMethod),
  filterIgnoresUnassigned = TRUE,
  unassignedSamples = c("outgroup", "cluster"),
  whichAssay = 1,
  ...
)

\S4method{makeDendrogram}{dist}(
  x,
  cluster,
  unassignedSamples = c("outgroup", "cluster", "remove"),
  calculateSample = TRUE,
  ...
)

\S4method{makeDendrogram}{matrixOrHDF5}(
  x,
  cluster,
  unassignedSamples = c("outgroup", "cluster", "remove"),
  calculateSample = TRUE,
  ...
)
}
\arguments{
\item{x}{data to define the medoids from. Matrix and
\code{\link{ClusterExperiment}} supported.}

\item{whichCluster}{argument that can be a single numeric or character value
indicating the \emph{single} clustering to be used. Giving values that result in more than one clustering will result in an error. See details of \code{\link{getClusterIndex}}.}

\item{reduceMethod}{character A character identifying what type of
dimensionality reduction to perform before clustering. Can be either a
value stored in either of reducedDims or filterStats slot or a built-in
diminsionality reduction/filtering. The option "coCluster" will use the
co-Clustering matrix stored in the 'coClustering' slot of the
\code{ClusterExperiment} object}

\item{nDims}{The number of dimensions to keep from \code{reduceMethod}. If
missing calls \code{\link{defaultNDims}}.}

\item{filterIgnoresUnassigned}{logical. Whether filtering statistics should 
ignore the unassigned samples within the clustering. Only relevant if 
'reduceMethod' matches one of built-in filtering statistics in 
\code{\link{listBuiltInFilterStats}()}, in which case the clustering 
identified in \code{whichCluster} is passed to \code{makeFilterStats} and
the unassigned samples are excluded in calculating the statistic. See 
\code{\link{makeFilterStats}}  for more details.}

\item{unassignedSamples}{how to handle unassigned samples("-1") ; only
relevant for sample clustering. See details.}

\item{whichAssay}{numeric or character specifying which assay to use. See
\code{\link[SummarizedExperiment]{assay}} for details.}

\item{...}{for makeDendrogram, if signature \code{matrix}, arguments passed
to hclust; if signature \code{ClusterExperiment} passed to the method for
signature \code{matrix}. For plotDendrogram, passed to
\code{\link{plot.dendrogram}}.}

\item{cluster}{A numeric vector with cluster assignments. If x is a
ClusterExperiment object, cluster is automatically the primaryCluster(x).
``-1'' indicates the sample was not assigned to a cluster.}

\item{calculateSample}{only relevant for \code{matrix} or \code{dist}
version of function. Indicates whether to calculate the sample dendrogram.}
}
\value{
If x is a matrix, a list with two dendrograms, one in which the
  leaves are clusters and one in which the leaves are samples. If x is a
  ClusterExperiment object, the dendrograms are saved in the appropriate
  slots.
}
\description{
Makes a dendrogram of a set of clusters based on hclust on the
  medoids of the cluster.
}
\details{
The function returns two dendrograms (as a list if x is a matrix or
  in the appropriate slots if x is ClusterExperiment). The cluster dendrogram
  is created by applying \code{\link{hclust}} to the medoids of each cluster.
  In the sample dendrogram the clusters are again clustered, but now the
  samples are also part of the resulting dendrogram. This is done by giving
  each sample the value of the medoid of its cluster.

The argument \code{unassignedSamples} governs what is done with
  unassigned samples (defined by a -1 cluster value). If
  unassigned=="cluster", then the dendrogram is created by hclust of the
  expanded medoid data plus the original unclustered observations. If
  \code{unassignedSamples} is "outgroup", then all unassigned samples are put
  as an outgroup. If the \code{x} object is a matrix, then
  \code{unassignedSamples} can also be "remove", to indicate that samples
  with "-1" should be discarded. This is not a permitted option, however,
  when \code{x} is a \code{ClusterExperiment} object, because it would return
  a dendrogram with less samples than \code{NCOL(x)}, which is not permitted
  for the \code{@dendro_samples} slot.

If any merge information is stored in the input object, it will be
  erased by a call to makeDendrogram.
}
\examples{
data(simData)

#create a clustering, for 8 clusters (truth was 3)
cl <- clusterSingle(simData, subsample=FALSE,
sequential=FALSE, mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=8)))

#create dendrogram of clusters:
hcl <- makeDendrogram(cl)
plotDendrogram(hcl)
plotDendrogram(hcl, leafType="samples",plotType="colorblock")

}
\seealso{
makeFilterStats, makeReducedDims
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rsec.R
\name{RSEC}
\alias{RSEC}
\alias{RSEC-methods}
\alias{RSEC,ClusterExperiment-method}
\alias{RSEC,matrix-method}
\alias{RSEC,SingleCellExperiment-method}
\alias{RSEC,SummarizedExperiment-method}
\alias{RSEC,data.frame-method}
\alias{RSEC,matrixOrHDF5-method}
\title{Resampling-based Sequential Ensemble Clustering}
\usage{
\S4method{RSEC}{SummarizedExperiment}(x, ...)

\S4method{RSEC}{data.frame}(x, ...)

\S4method{RSEC}{ClusterExperiment}(x, eraseOld = FALSE, rerunClusterMany = FALSE, ...)

\S4method{RSEC}{matrixOrHDF5}(x, ...)

\S4method{RSEC}{SingleCellExperiment}(
  x,
  isCount = FALSE,
  transFun = NULL,
  reduceMethod = "PCA",
  nFilterDims = defaultNDims(x, reduceMethod, type = "filterStats"),
  nReducedDims = defaultNDims(x, reduceMethod, type = "reducedDims"),
  k0s = 4:15,
  subsample = TRUE,
  sequential = TRUE,
  clusterFunction = "hierarchical01",
  alphas = c(0.1, 0.2, 0.3),
  betas = 0.9,
  minSizes = 1,
  makeMissingDiss = if (ncol(x) < 1000) TRUE else FALSE,
  consensusProportion = 0.7,
  consensusMinSize,
  dendroReduce,
  dendroNDims,
  mergeMethod = "adjP",
  mergeCutoff,
  mergeLogFCcutoff,
  mergeDEMethod = if (isCount) "limma-voom" else "limma",
  verbose = FALSE,
  parameterWarnings = FALSE,
  mainClusterArgs = NULL,
  subsampleArgs = NULL,
  seqArgs = NULL,
  consensusArgs = NULL,
  whichAssay = 1,
  ncores = 1,
  random.seed = NULL,
  stopOnErrors = FALSE,
  run = TRUE
)
}
\arguments{
\item{x}{the data matrix on which to run the clustering. Can be object of the
following classes: matrix (with genes in rows),
\code{\link[SummarizedExperiment]{SummarizedExperiment}},
\code{\link[SingleCellExperiment]{SingleCellExperiment}}
or \code{ClusterExperiment}.}

\item{...}{For signature \code{matrix}, arguments to be passed on to mclapply
(if ncores>1). For all the other signatures, arguments to be passed to the
method for signature \code{matrix}.}

\item{eraseOld}{logical. Only relevant if input \code{x} is of class
\code{ClusterExperiment}. If TRUE, will erase existing workflow results
(clusterMany as well as mergeClusters and makeConsensus). If FALSE, existing
workflow results will have "\code{_i}" added to the clusterTypes value,
where \code{i} is one more than the largest such existing workflow
clusterTypes.}

\item{rerunClusterMany}{logical. If the object is a ClusterExperiment object,
determines whether to rerun the clusterMany step. Useful if want to try
different parameters for combining clusters after the clusterMany step,
without the computational costs of the clusterMany step.}

\item{isCount}{if \code{transFun=NULL}, then \code{isCount=TRUE} will
determine the transformation as defined by \code{function(x){log2(x+1)}},
and \code{isCount=FALSE} will give a transformation function
\code{function(x){x}}. Ignored if \code{transFun=NULL}. If object is of
class \code{ClusterExperiment}, the stored transformation will be used and
giving this parameter will result in an error.}

\item{transFun}{a transformation function to be applied to the data. If the
transformation applied to the data creates an error or NA values, then the
function will throw an error. If object is of class
\code{ClusterExperiment}, the stored transformation will be used and giving
this parameter will result in an error.}

\item{reduceMethod}{character A character identifying what type of
dimensionality reduction to perform before clustering. Options are 1)
"none", 2) one of listBuiltInReducedDims() or listBuiltInFitlerStats OR 3)
stored filtering or reducedDim values in the object.}

\item{nFilterDims}{vector of the number of the most variable features to keep
(when "var", "abscv", or "mad" is identified in \code{reduceMethod}).}

\item{nReducedDims}{vector of the number of dimensions to use (when
\code{reduceMethod} gives a dimensionality reduction method).}

\item{k0s}{the k0 parameter for sequential clustering (see
\code{\link{seqCluster}})}

\item{subsample}{logical as to whether to subsample via
\code{\link{subsampleClustering}}. If TRUE, clustering in mainClustering
step is done on the co-occurance between clusterings in the subsampled
clustering results.  If FALSE, the mainClustering step will be run directly
on \code{x}/\code{diss}}

\item{sequential}{logical whether to use the sequential strategy (see details
of \code{\link{seqCluster}}). Can be used in combination with
\code{subsample=TRUE} or \code{FALSE}.}

\item{clusterFunction}{function used for the clustering. This must be either 
1) a character vector of built-in clustering techniques, or 2) a
\emph{named} list of \code{\link{ClusterFunction}} objects. Current
functions can be found by typing \code{listBuiltInFunctions()} into the
command-line.}

\item{alphas}{values of alpha to be tried. Only used for clusterFunctions of
type '01'. Determines tightness required in creating clusters from the
dissimilarity matrix. Takes on values in [0,1]. See documentation of
\code{\link{ClusterFunction}}.}

\item{betas}{values of \code{beta} to be tried in sequential steps. Only used
for \code{sequential=TRUE}. Determines the similarity between two clusters
required in order to deem the cluster stable. Takes on values in [0,1]. See
documentation of \code{\link{seqCluster}}.}

\item{minSizes}{the minimimum size required for a cluster (in the
\code{mainClustering} step). Clusters smaller than this are not kept and samples
are left unassigned.}

\item{makeMissingDiss}{logical. Whether to calculate necessary distance 
matrices needed when input is not "diss". If TRUE, then when a clustering 
function calls for a inputType "diss", but the given matrix is of type "X",
the function will calculate a distance function. A dissimilarity matrix
will also be calculated if a post-processing argument like \code{findBestK}
or \code{removeSil} is chosen, since these rely on calcualting silhouette
widths from distances.}

\item{consensusProportion}{passed to \code{proportion} in
\code{\link{makeConsensus}}}

\item{consensusMinSize}{passed to \code{minSize} in
\code{\link{makeConsensus}}}

\item{dendroReduce}{passed to \code{reduceMethod} in
\code{\link{makeDendrogram}}}

\item{dendroNDims}{passed to \code{nDims} in \code{\link{makeDendrogram}}}

\item{mergeMethod}{passed to \code{mergeMethod} in
\code{\link{mergeClusters}}}

\item{mergeCutoff}{passed to \code{cutoff} in \code{\link{mergeClusters}}}

\item{mergeLogFCcutoff}{passed to \code{logFCcutoff} in
\code{\link{mergeClusters}}}

\item{mergeDEMethod}{passed to \code{DEMethod} argument in
  \code{\link{mergeClusters}}. By default, unless otherwise chosen by the
  user, if \code{isCount=TRUE}, then \code{mergeDEMethod="limma-voom"}, 
otherwise \code{mergeDEMethod="limma"}. These choices are for speed
 considerations and the user may want to try \code{mergeDEMethod="edgeR"} 
on smaller datasets of counts.}

\item{verbose}{logical. If TRUE it will print informative messages.}

\item{parameterWarnings}{logical, as to whether warnings and comments 
from checking the validity of the parameter combinations should be printed.}

\item{mainClusterArgs}{list of arguments to be passed for the mainClustering
step, see help pages of \code{\link{mainClustering}}.}

\item{subsampleArgs}{list of arguments to be passed to the subsampling step
(if \code{subsample=TRUE}), see help pages of
\code{\link{subsampleClustering}}.}

\item{seqArgs}{list of arguments to be passed to \code{\link{seqCluster}}.}

\item{consensusArgs}{list of additional arguments to be passed to
\code{\link{makeConsensus}}}

\item{whichAssay}{numeric or character specifying which assay to use. See
\code{\link[SummarizedExperiment]{assay}} for details.}

\item{ncores}{the number of threads}

\item{random.seed}{a value to set seed before each run of clusterSingle (so
that all of the runs are run on the same subsample of the data). Note, if
'random.seed' is set, argument 'ncores' should NOT be passed via
subsampleArgs; instead set the argument 'ncores' of clusterMany directly
(which is preferred for improving speed anyway).}

\item{stopOnErrors}{logical. If \code{FALSE}, if RSEC hits an error
\emph{after} the \code{clusterMany} step, it will return the results up to
that point, rather than generating a stop error. The text of error will be
printed as a NOTE. This allows the user to get the results to that point,
so as to not have to rerun the computationally heavy earlier steps. The
\code{TRUE} option is only provided for debugging purposes.}

\item{run}{logical. If FALSE, doesn't run clustering, but just returns matrix
of parameters that will be run, for the purpose of inspection by user (with
rownames equal to the names of the resulting column names of clMat object
that would be returned if \code{run=TRUE}). Even if \code{run=FALSE},
however, the function will create the dimensionality reductions of the data
indicated by the user input.}
}
\value{
A \code{\link{ClusterExperiment}} object is returned containing all
  of the clusterings from the steps of RSEC
}
\description{
Implementation of the RSEC algorithm (Resampling-based
  Sequential Ensemble Clustering) for single cell sequencing data. This is a
  wrapper function around the existing ClusterExperiment workflow that
  results in the output of RSEC.
}
\details{
Note that the argument \code{isCount} is mainly used when the input is a matrix or SingleCellExperiment Class and passed to \code{clusterMany} to set the transformation function of the data. However, if RSEC is being re-called on an existing \code{ClusterExperiment} object, it does not reset the transformation; in this case the only impact it will have is in setting the default value for \code{DEMethod} for \code{mergeClusters} step, but ONLY if \code{mergeClusters} hasn't already been calculated. To set arguments that allow you to recalculate the non-null probabilities of the hierarchy see \code{\link{mergeClusters}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusterSingle.R
\name{clusterSingle}
\alias{clusterSingle}
\alias{clusterSingle,SummarizedExperiment-method}
\alias{clusterSingle,ClusterExperiment-method}
\alias{clusterSingle,SingleCellExperiment-method}
\alias{clusterSingle,matrixOrHDF5OrNULL-method}
\title{General wrapper method to cluster the data}
\usage{
\S4method{clusterSingle}{SummarizedExperiment}(inputMatrix, ...)

\S4method{clusterSingle}{ClusterExperiment}(inputMatrix, ...)

\S4method{clusterSingle}{SingleCellExperiment}(
  inputMatrix,
  reduceMethod = "none",
  nDims = defaultNDims(inputMatrix, reduceMethod),
  whichAssay = 1,
  ...
)

\S4method{clusterSingle}{matrixOrHDF5OrNULL}(
  inputMatrix,
  inputType = "X",
  subsample = FALSE,
  sequential = FALSE,
  distFunction = NA,
  mainClusterArgs = NULL,
  subsampleArgs = NULL,
  seqArgs = NULL,
  isCount = FALSE,
  transFun = NULL,
  reduceMethod = "none",
  nDims = defaultNDims(inputMatrix, reduceMethod),
  makeMissingDiss = if (ncol(inputMatrix) < 1000) TRUE else FALSE,
  clusterLabel = "clusterSingle",
  saveSubsamplingMatrix = FALSE,
  checkDiss = FALSE,
  warnings = TRUE
)
}
\arguments{
\item{inputMatrix}{numerical matrix on which to run the clustering or a
\code{\link[SummarizedExperiment]{SummarizedExperiment}},
\code{\link{SingleCellExperiment}}, or \code{\link{ClusterExperiment}}
object.}

\item{...}{arguments to be passed on to the method for signature
\code{matrix}.}

\item{reduceMethod}{character A character identifying what type of
dimensionality reduction to perform before clustering. Options are 1)
"none", 2) one of listBuiltInReducedDims() or listBuiltInFitlerStats OR 3)
stored filtering or reducedDim values in the object.}

\item{nDims}{integer An integer identifying how many dimensions to reduce to
in the reduction specified by \code{reduceMethod}. Defaults to output of
\code{\link{defaultNDims}}}

\item{whichAssay}{numeric or character specifying which assay to use. See
\code{\link[SummarizedExperiment]{assay}} for details.}

\item{inputType}{a character vector defining what type of input is given in
the \code{inputMatrix} argument. Must consist of values "diss","X", or
"cat" (see details). "X" and "cat" should be indicate
matrices with features in the row and samples in the column; "cat"
corresponds to the features being numerical integers corresponding to
categories, while "X" are continuous valued features. "diss" corresponds to
an \code{inputMatrix} that is a NxN dissimilarity matrix. "cat" is largely
used internally for clustering of sets of clusterings.}

\item{subsample}{logical as to whether to subsample via
\code{\link{subsampleClustering}}. If TRUE, clustering in mainClustering
step is done on the co-occurance between clusterings in the subsampled
clustering results.  If FALSE, the mainClustering step will be run directly
on \code{x}/\code{diss}}

\item{sequential}{logical whether to use the sequential strategy (see details
of \code{\link{seqCluster}}). Can be used in combination with
\code{subsample=TRUE} or \code{FALSE}.}

\item{distFunction}{a distance function to be applied to \code{inputMatrix}. Only
  relevant if \code{inputType="X"}. 
See details of \code{\link{clusterSingle}} for the
  required format of the distance function.}

\item{mainClusterArgs}{list of arguments to be passed for the mainClustering
step, see help pages of \code{\link{mainClustering}}.}

\item{subsampleArgs}{list of arguments to be passed to the subsampling step
(if \code{subsample=TRUE}), see help pages of
\code{\link{subsampleClustering}}.}

\item{seqArgs}{list of arguments to be passed to \code{\link{seqCluster}}.}

\item{isCount}{if \code{transFun=NULL}, then \code{isCount=TRUE} will
determine the transformation as defined by \code{function(x){log2(x+1)}},
and \code{isCount=FALSE} will give a transformation function
\code{function(x){x}}. Ignored if \code{transFun=NULL}. If object is of
class \code{ClusterExperiment}, the stored transformation will be used and
giving this parameter will result in an error.}

\item{transFun}{a transformation function to be applied to the data. If the
transformation applied to the data creates an error or NA values, then the
function will throw an error. If object is of class
\code{ClusterExperiment}, the stored transformation will be used and giving
this parameter will result in an error.}

\item{makeMissingDiss}{logical. Whether to calculate necessary distance 
matrices needed when input is not "diss". If TRUE, then when a clustering 
function calls for a inputType "diss", but the given matrix is of type "X",
the function will calculate a distance function. A dissimilarity matrix
will also be calculated if a post-processing argument like \code{findBestK}
or \code{removeSil} is chosen, since these rely on calcualting silhouette
widths from distances.}

\item{clusterLabel}{a string used to describe the clustering. By default it
is equal to "clusterSingle", to indicate that this clustering is the result
of a call to \code{clusterSingle}.}

\item{saveSubsamplingMatrix}{logical. If TRUE, the co-clustering matrix
resulting from subsampling is returned in the coClustering slot (and
replaces any existing coClustering object in the slot \code{coClustering}
if input object is a \code{ClusterExperiment} object.)}

\item{checkDiss}{logical. Whether to check whether the dissimilarities
matrices  are valid (whether given by the user or calculated because
\code{makeMissingDiss=TRUE}).}

\item{warnings}{logical. Whether to print out the many possible warnings and
messages regarding checking the internal consistency of the parameters.}
}
\value{
A \code{\link{ClusterExperiment}} object if
  \code{inputType} is of type "X".

If input was not of type "X", then the result is a list with values
  \itemize{ 
       \item{clustering: }{The vector of clustering results}
       \item{clusterInfo: }{A list with information about the parameters run in
  the clustering} 
       \item{coClusterMatrix: }{(only if \code{saveSubsamplingMatrix=TRUE}, NxB set of clusterings obtained after B subsamples.} 
  }
}
\description{
Given input data, this function will find clusters, based on a single
specification of parameters.
}
\details{
\code{clusterSingle} is an 'expert-oriented' function, intended to
  be used when a user wants to run a single clustering and/or have a great
  deal of control over the clustering parameters. Most users will find
  \code{\link{clusterMany}} more relevant. However, \code{\link{clusterMany}}
  makes certain assumptions about the intention of certain combinations of
  parameters that might not match the user's intent; similarly
  \code{\link{clusterMany}} does not directly take a dissimilarity matrix but
  only a matrix of values \code{x} (though a user can define a distance
  function to be applied to \code{x} in \code{\link{clusterMany}}).

Unlike \code{\link{clusterMany}}, most of the relevant arguments for
  the actual clustering algorithms in \code{clusterSingle} are passed to the
  relevant steps via the arguments \code{mainClusterArgs},
  \code{subsampleArgs}, and \code{seqArgs}. These arguments should be
  \emph{named} lists with parameters that match the corresponding functions:
  \code{\link{mainClustering}},\code{\link{subsampleClustering}}, and
  \code{\link{seqCluster}}. These three functions are not meant to be called
  by the user, but rather accessed via calls to \code{clusterSingle}. But the
  user can look at the help files of those functions for more information 
  regarding the parameters that they take.

Only certain combinations of parameters are possible for certain
  choices of \code{sequential} and \code{subsample}. These restrictions are
  documented below. 
  \itemize{ 
      \item{\code{clusterFunction} for
  \code{mainClusterArgs}: }{The choice of \code{subsample=TRUE} also controls
  what algorithm type of clustering functions can be used in the
  mainClustering step. When \code{subsample=TRUE}, then resulting
  co-clustering matrix  from subsampling is converted to a dissimilarity
  (specificaly 1-coclustering values) and is passed to \code{diss} of
  \code{\link{mainClustering}}. For this reason, the \code{ClusterFunction}
  object given to \code{\link{mainClustering}} via the argument
  \code{mainClusterArgs} must take input of the form of a dissimilarity. When
  \code{subsample=FALSE} and \code{sequential=TRUE}, the
  \code{clusterFunction} passed in \code{clusterArgs} element of
  \code{mainClusterArgs} must define a \code{ClusterFunction} object with
  \code{algorithmType} 'K'.  When \code{subsample=FALSE} and
  \code{sequential=FALSE}, then there are no restrictions on the
  \code{ClusterFunction} and that clustering is applied directly to the input
  data. } 
  \item{\code{clusterFunction}  for \code{subsampleArgs}: }{If the
  \code{ClusterFunction} object given to the \code{clusterArgs} of
  \code{subsamplingArgs} is missing the algorithm will use the default for
  \code{\link{subsampleClustering}} (currently "pam"). If
  \code{sequential=TRUE}, this \code{ClusterFunction} object must be of type
  'K'. } 
  \item{Setting \code{k} for subsampling: }{If \code{subsample=TRUE}
  and \code{sequential=TRUE}, the current K of the sequential iteration
  determines the 'k' argument passed to \code{\link{subsampleClustering}}  so
  setting 'k=' in the list given to the subsampleArgs will not do anything
  and will produce a warning to that effect (see documentation of
  \code{\link{seqCluster}}).} 
  \item{Setting \code{k} for mainClustering step:
  }{If \code{sequential=TRUE} then the user should not set \code{k} in the
  \code{clusterArgs} argument of \code{mainClusterArgs} because it must be
  set by the sequential code, which has a iterative reseting of the
  parameters. Specifically if \code{subsample=FALSE}, then the sequential
  method iterates over choices of \code{k} to cluster the input data. And if
  \code{subsample=TRUE}, then the \code{k} in the clustering of
  mainClustering step (assuming the clustering function is of type 'K') will
  use the \code{k} used in the subsampling step to make sure that the
  \code{k} used in the mainClustering step is reasonable. } 
  \item{Setting
  \code{findBestK} in \code{mainClusterArgs}: }{If \code{sequential=TRUE} and
  \code{subsample=FALSE}, the user should not set 'findBestK=TRUE' in
  \code{mainClusterArgs}. This is because in this case the sequential method
  changes \code{k}; an error message will be given if this combination of
  options are set. However, if \code{sequential=TRUE} and
  \code{subsample=TRUE}, then passing either 'findBestK=TRUE' or
  'findBestK=FALSE' via \code{mainClusterArgs} will function as expected
  (assuming the \code{clusterFunction} argument passed to
  \code{mainClusterArgs} is of type 'K'). In particular, the sequential step
  will set the number of clusters \code{k} for clustering of each subsample.
  If findBestK=FALSE, that same \code{k} will be used for mainClustering step
  that clusters the resulting co-occurance matrix after subsampling. If
  findBestK=TRUE, then \code{\link{mainClustering}} will search for best k.
  Note that the default 'kRange' over which \code{\link{mainClustering}}
  searches when findBestK=TRUE depends on the input value of \code{k} which
  is set by the sequential method if \code{sequential=TRUE}), see above. The
  user can change \code{kRange} to not depend on \code{k} and to be fixed
  across all of the sequential steps by setting \code{kRange} explicitly in
  the \code{mainClusterArgs} list.} }

To provide a distance matrix via the argument \code{distFunction},
  the function must be defined to take the distance of the rows of a matrix
  (internally, the function will call \code{distFunction(t(x))}. This is to
  be compatible with the input for the \code{dist} function. \code{as.matrix}
  will be performed on the output of \code{distFunction}, so if the object
  returned has a \code{as.matrix} method that will convert the output into a
  symmetric matrix of distances, this is fine (for example the class
  \code{dist} for objects returned by \code{dist} have such a method). If
  \code{distFunction=NA}, then a default distance will be calculated based on
  the type of clustering algorithm of \code{clusterFunction}. For type "K"
  the default is to take \code{dist} as the distance function. For type "01",
  the default is to take the (1-cor(x))/2.
}
\examples{
data(simData)

\dontrun{
#following code takes some time.
#use clusterSingle to do sequential clustering
#(same as example in seqCluster only using clusterSingle ...)
set.seed(44261)
clustSeqHier_v2 <- clusterSingle(simData,
     sequential=TRUE, subsample=TRUE, 
     subsampleArgs=list(resamp.n=100, samp.p=0.7,
     clusterFunction="kmeans", clusterArgs=list(nstart=10)),
     seqArgs=list(beta=0.8, k0=5), mainClusterArgs=list(minSize=5,
     clusterFunction="hierarchical01",clusterArgs=list(alpha=0.1)))
}

#use clusterSingle to do just clustering k=3 with no subsampling
clustObject <- clusterSingle(simData,
    subsample=FALSE, sequential=FALSE,
    mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=3)))
#compare to standard pam
pamOut<-cluster::pam(t(simData),k=3,cluster.only=TRUE)
all(pamOut==primaryCluster(clustObject))
}
\seealso{
\code{\link{clusterMany}} to compare multiple choices of parameters,
  and \code{\link{mainClustering}},\code{\link{subsampleClustering}}, and
  \code{\link{seqCluster}} for the underlying functions called by
  \code{clusterSingle}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsampleClustering.R
\name{subsampleClustering}
\alias{subsampleClustering}
\alias{subsampleClustering,character-method}
\alias{subsampleClustering,ClusterFunction-method}
\title{Cluster subsamples of the data}
\usage{
\S4method{subsampleClustering}{character}(clusterFunction, ...)

\S4method{subsampleClustering}{ClusterFunction}(
  clusterFunction,
  inputMatrix,
  inputType,
  clusterArgs = NULL,
  classifyMethod = c("All", "InSample", "OutOfSample"),
  resamp.num = 100,
  samp.p = 0.7,
  ncores = 1,
  warnings = TRUE,
  ...
)
}
\arguments{
\item{clusterFunction}{a \code{\link{ClusterFunction}} object that defines
the clustering routine. See \code{\link{ClusterFunction}} for required
format of user-defined clustering routines. User can also give a character
value to the argument \code{clusterFunction} to indicate the use of
clustering routines provided in package. Type
\code{\link{listBuiltInFunctions}} at command prompt to see the built-in
clustering routines. If \code{clusterFunction} is missing, the default is
set to "pam".}

\item{...}{arguments passed to mclapply (if ncores>1).}

\item{inputMatrix}{numerical matrix on which to run the clustering or a
\code{\link[SummarizedExperiment]{SummarizedExperiment}},
\code{\link{SingleCellExperiment}}, or \code{\link{ClusterExperiment}}
object.}

\item{inputType}{a character vector defining what type of input is given in
the \code{inputMatrix} argument. Must consist of values "diss","X", or
"cat" (see details). "X" and "cat" should be indicate
matrices with features in the row and samples in the column; "cat"
corresponds to the features being numerical integers corresponding to
categories, while "X" are continuous valued features. "diss" corresponds to
an \code{inputMatrix} that is a NxN dissimilarity matrix. "cat" is largely
used internally for clustering of sets of clusterings.}

\item{clusterArgs}{a list of parameter arguments to be passed to the function
defined in the \code{clusterFunction} slot of the \code{ClusterFunction}
object. For any given \code{\link{ClusterFunction}} object, use function
\code{\link{requiredArgs}} to get a list of required arguments for the
object.}

\item{classifyMethod}{method for determining which samples should be used in
calculating the co-occurance matrix. "All"= all samples, "OutOfSample"=
those not subsampled, and "InSample"=those in the subsample.  See details
for explanation.}

\item{resamp.num}{the number of subsamples to draw.}

\item{samp.p}{the proportion of samples to sample for each subsample.}

\item{ncores}{integer giving the number of cores. If ncores>1, mclapply will
be called.}

\item{warnings}{logical as to whether should give warning if arguments given
that don't match clustering choices given. Otherwise, inapplicable 
arguments will be ignored without warning.}
}
\value{
A \code{n x n} matrix of co-occurances, i.e. a symmetric matrix with
  [i,j] entries equal to the percentage of subsamples where the ith and jth
  sample were clustered into the same cluster. The percentage is only out of
  those subsamples where the ith and jth samples were both assigned to a
  clustering. If \code{classifyMethod=="All"}, this is all subsamples for all
  i,j pairs. But if \code{classifyMethod=="InSample"} or
  \code{classifyMethod=="OutOfSample"}, then the percentage is only taken on
  those subsamples where the ith and jth sample were both in or out of
  sample, respectively, relative to the subsample.
}
\description{
Given input data, this function will subsample the samples, cluster the
subsamples, and return a \code{n x n} matrix with the probability of
co-occurance.
}
\details{
\code{subsampleClustering} is not usually called directly by the
  user. It is only an exported function so as to be able to clearly document
  the arguments for \code{subsampleClustering}  which can be passed via the
  argument \code{subsampleArgs} in functions like \code{\link{clusterSingle}}
  and \code{\link{clusterMany}}.

\code{requiredArgs:} The choice of "All" or "OutOfSample" for
  \code{requiredArgs} require the classification of arbitrary samples not
  originally in the clustering to clusters; this is done via the classifyFUN
  provided in the \code{\link{ClusterFunction}} object. If the
  \code{\link{ClusterFunction}} object does not have such a function to
  define how to classify into a cluster samples not in the subsample that
  created the clustering then \code{classifyMethod} must be
  \code{"InSample"}. Note that if "All" is chosen, all samples will be
  classified into clusters via the classifyFUN, not just those that are
  out-of-sample; this could result in different assignments to clusters for
  the in-sample samples than their original assignment by the clustering
  depending on the classification function. If you do not choose 'All',it is
  possible to get NAs in resulting S matrix (particularly if when not enough
  subsamples are taken) which can cause errors if you then pass the resulting
  D=1-S matrix to \code{\link{mainClustering}}. For this reason the default is
  "All".
}
\examples{
\dontrun{
#takes a bit of time, not run on checks:
data(simData)
coOccur <- subsampleClustering( inputMatrix=simData, inputType="X",
clusterFunction="kmeans",
clusterArgs=list(k=3,nstart=10), resamp.n=100, samp.p=0.7)

#visualize the resulting co-occurance matrix
plotHeatmap(coOccur)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotBarplot.R
\docType{methods}
\name{plotBarplot,ClusterExperiment-method}
\alias{plotBarplot,ClusterExperiment-method}
\alias{plotBarplot}
\alias{plotBarplot,vector-method}
\alias{plotBarplot,matrix-method}
\title{Barplot of 1 or 2 clusterings}
\usage{
\S4method{plotBarplot}{ClusterExperiment}(object, whichClusters = "primary", labels = c("names", "ids"), ...)

\S4method{plotBarplot}{vector}(object, ...)

\S4method{plotBarplot}{matrix}(
  object,
  xNames = NULL,
  legNames = NULL,
  legend = ifelse(ncol(object) == 2, TRUE, FALSE),
  xlab = NULL,
  legend.title = NULL,
  unassignedColor = "white",
  missingColor = "grey",
  colPalette = NULL,
  ...
)
}
\arguments{
\item{object}{A matrix of with each column corresponding to a clustering and
each row a sample or a \code{\link{ClusterExperiment}} object.}

\item{whichClusters}{argument that can be either numeric or character vector
indicating the clusterings to be used. See details of \code{\link{getClusterIndex}}.}

\item{labels}{if object is a ClusterExperiment object, then labels defines 
whether the clusters will be identified by their names values in 
clusterLegend (labels="names", the default) or by their clusterIds value in
clusterLegend (labels="ids").}

\item{...}{for \code{plotBarplot} arguments passed either to the method of
\code{plotBarplot} for matrices or ultimately to \code{\link{barplot}}.}

\item{xNames}{names for the clusters on x-axis (i.e. clustering given 1st). 
By default uses names of the 1st column of clusters matrix. See details.}

\item{legNames}{names for the clusters dividing up the 1st clusters (will 
appear in legend). By default uses names of the 2nd cluster of clusters
matrix. If only one clustering, \code{xNames} and \code{legNames} refer to
the same clustering. See details.}

\item{legend}{whether to plot the legend}

\item{xlab}{label for x-axis. By default or if equal NULL the column name of 
the 1st cluster of clusters matrix}

\item{legend.title}{label for legend. By default or if equal NULL the column 
name of the 2st cluster of clusters matrix}

\item{unassignedColor}{If ``-1'' in \code{clusters}, will be given this color
(meant for samples not assigned to cluster).}

\item{missingColor}{If ``-2'' in clusters, will be given this color (meant
for samples that were missing from the clustering, mainly when comparing
clusterings run on different sets of samples)}

\item{colPalette}{a vector of colors used for the different clusters. See
details.}
}
\value{
A plot is produced, nothing is returned
}
\description{
Make a barplot of sample's assignments to clusters for single clustering, or 
cross comparison for two clusterings.
}
\details{
The first column of the cluster matrix will be on the x-axis and the
  second column (if present) will separate the groups of the first column.

All arguments of the matrix version can be passed to the 
  \code{ClusterExperiment} version. As noted above, however, some arguments 
  have different interpretations.

If \code{whichClusters = "workflow"}, then the most recent two 
  clusters of the workflow will be chosen where recent is based on the 
  following order (most recent first): final, mergeClusters, makeConsensus, 
  clusterMany.

\code{xNames}, \code{legNames} and \code{colPalette} should all be
  named vectors, with the names referring to the clusters they should match
  to (for \code{ClusterExperiment} objects, it is determined by the argument
  \code{labels} as to whether the names should match the cluster names or the
  clusterIds). \code{colPalette} and \code{legNames} must be same length of
  the number of clusters found in the second clustering, or if  only a single
  clustering, the same length as the number of clusters in that clustering.
}
\examples{
\dontrun{
#clustering using pam: try using different dimensions of pca and different k
data(simData)

cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reduceMethod="PCA",
clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)

plotBarplot(cl)
plotBarplot(cl,whichClusters=1:2)
}
}
\author{
Elizabeth Purdom
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assignUnassigned.R
\name{assignUnassigned}
\alias{assignUnassigned}
\alias{assignUnassigned,ClusterExperiment-method}
\alias{removeUnassigned,ClusterExperiment-method}
\alias{removeUnassigned}
\title{Assign unassigned samples to nearest cluster}
\usage{
\S4method{assignUnassigned}{ClusterExperiment}(
  object,
  whichCluster = "primary",
  clusterLabel,
  makePrimary = TRUE,
  whichAssay = 1,
  reduceMethod = "none",
  ...
)

\S4method{removeUnassigned}{ClusterExperiment}(object, whichCluster = "primary")
}
\arguments{
\item{object}{A Cluster Experiment object}

\item{whichCluster}{argument that can be a single numeric or character value
indicating the \emph{single} clustering to be used. Giving values that result in more than one clustering will result in an error. See details of \code{\link{getClusterIndex}}.}

\item{clusterLabel}{if missing, the current cluster label of the cluster will
be appended with the string "_AllAssigned".}

\item{makePrimary}{whether to make the added cluster the primary cluster
(only relevant if \code{y} is a vector)}

\item{whichAssay}{which assay to use to calculate the median per cluster and
take dimensionality reduction (if requested)}

\item{reduceMethod}{character. A method (or methods) for reducing the size of
the data, either by filtering the rows (genes) or by a dimensionality
reduction method. Must either be 1) must match the name of a built-in
method, in which case if it is not already existing in the object will be
passed to \code{\link{makeFilterStats}} or \code{link{makeReducedDims}}, or
2) must match a stored filtering statistic or dimensionality reduction in
the object}

\item{...}{arguments passed to \code{\link{getReducedData}} specifying the
dimensionality reduction (if any) to be taken of the data for calculating
the medians of the clusters}
}
\value{
The function \code{assignUnassigned} returns a
  \code{ClusterExperiment} object with the unassigned samples assigned to one
  of the existing clusters.

The function \code{removeUnassigned} returns a
  \code{ClusterExperiment} object with the unassigned samples removed.
}
\description{
Assigns the unassigned samples in a cluster to the nearest
  cluster based on distance to the medians of the clusters.
}
\details{
The function \code{assignUnassigned} calculates the median values of
  each variable for each cluster, and then calculates the euclidean distance
  of each unassigned sample to the median of each cluster. Each unassigned
  sample is assigned to the cluster for which it closest to the median.

All unassigned samples in the cluster are given a clustering,
  regardless of whether they are classified as -1 or -2.

\code{removeUnclustered} removes all samples that are unclustered
  (i.e. -1 or -2 assignment) in the designated cluster of \code{object} (so
  they may be unclustered in other clusters found in
  \code{clusterMatrix(object)}).
}
\examples{
#load CE object
\dontrun{
data(rsecFluidigm)
smallCE<-rsecFluidigm[,1:50]
#assign the unassigned samples
assignUnassigned(smallCE, makePrimary=TRUE)

#note how samples are REMOVED:
removeUnassigned(smallCE)
}
}
\seealso{
\code{\link{getReducedData}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClassesCE.R
\docType{class}
\name{ClusterExperiment-class}
\alias{ClusterExperiment-class}
\alias{ClusterExperiment}
\alias{ClusterExperiment,matrixOrHDF5,ANY-method}
\alias{ClusterExperiment,SummarizedExperiment,ANY-method}
\alias{ClusterExperiment,SingleCellExperiment,numeric-method}
\alias{ClusterExperiment,SingleCellExperiment,character-method}
\alias{ClusterExperiment,SingleCellExperiment,factor-method}
\alias{ClusterExperiment,SingleCellExperiment,matrix-method}
\title{Class ClusterExperiment}
\usage{
ClusterExperiment(object, clusters, ...)

\S4method{ClusterExperiment}{matrixOrHDF5,ANY}(object, clusters, ...)

\S4method{ClusterExperiment}{SummarizedExperiment,ANY}(object, clusters, ...)

\S4method{ClusterExperiment}{SingleCellExperiment,numeric}(object, clusters, ...)

\S4method{ClusterExperiment}{SingleCellExperiment,character}(object, clusters, ...)

\S4method{ClusterExperiment}{SingleCellExperiment,factor}(object, clusters, ...)

\S4method{ClusterExperiment}{SingleCellExperiment,matrix}(
  object,
  clusters,
  transformation = function(x) {
     x
 },
  primaryIndex = 1,
  clusterTypes = "User",
  clusterInfo = NULL,
  orderSamples = seq_len(ncol(object)),
  dendro_samples = NULL,
  dendro_index = NA_real_,
  dendro_clusters = NULL,
  coClustering = NULL,
  merge_index = NA_real_,
  merge_cutoff = NA_real_,
  merge_dendrocluster_index = NA_real_,
  merge_nodeProp = NULL,
  merge_nodeMerge = NULL,
  merge_method = NA_character_,
  merge_demethod = NA_character_,
  clusterLegend = NULL,
  checkTransformAndAssay = TRUE
)
}
\arguments{
\item{object}{a matrix or \code{SummarizedExperiment} or
\code{SingleCellExperiment} containing the data that was clustered.}

\item{clusters}{can be either a numeric or character vector, a factor, or a
numeric matrix, containing the cluster labels.}

\item{...}{The arguments \code{transformation}, \code{clusterTypes} and
\code{clusterInfo} to be passed to the constructor for signature
\code{SingleCellExperiment,matrix}.}

\item{transformation}{function. A function to transform the data before
performing steps that assume normal-like data (i.e. constant variance), such
as the log.}

\item{primaryIndex}{integer. Sets the `primaryIndex` slot (see Slots).}

\item{clusterTypes}{a string describing the nature of the clustering. The
values `clusterSingle`, `clusterMany`, `mergeClusters`, `makeConsensus` are
reserved for the clustering coming from the package workflow and should not
be used when creating a new object with the constructor.}

\item{clusterInfo}{a list with information on the clustering (see Slots).}

\item{orderSamples}{a vector of integers. Sets the `orderSamples` slot (see
Slots).}

\item{dendro_samples}{phylo4 object. Sets the `dendro_samples` slot (see Slots).}

\item{dendro_index}{numeric. Sets the \code{dendro_index} slot (see Slots).}

\item{dendro_clusters}{phylo4 object. Sets the `dendro_clusters` slot (see
Slots).}

\item{coClustering}{matrix. Sets the \code{coClustering} slot (see Slots).}

\item{merge_index}{integer. Sets the \code{merge_index} slot (see Slots)}

\item{merge_cutoff}{numeric. Sets the \code{merge_cutoff} slot (see Slots)}

\item{merge_dendrocluster_index}{integer. Sets the
\code{merge_dendrocluster_index} slot (see Slots)}

\item{merge_nodeProp}{data.frame. Sets the \code{merge_nodeProp} slot (see
Slots)}

\item{merge_nodeMerge}{data.frame. Sets the \code{merge_nodeMerge} slot (see
Slots)}

\item{merge_method}{character, Sets the \code{merge_method} slot (see Slots)}

\item{merge_demethod}{character, Sets the
\code{merge_demethod} slot (see Slots)}

\item{clusterLegend}{list, Sets the \code{clusterLegend} slot (see details).}

\item{checkTransformAndAssay}{logical. Whether to check the content of the
assay and given transformation function for whether they are valid.}
}
\value{
A \code{ClusterExperiment} object.
}
\description{
\code{ClusterExperiment} is a class that extends
\code{SingleCellExperiment} and is used to store the data
and clustering information.

In addition to the slots of the \code{SingleCellExperiment}
class, the \code{ClusterExperiment} object has the additional slots described
in the Slots section.

There are several methods implemented for this class. The most
important methods (e.g., \code{\link{clusterMany}}, \code{\link{makeConsensus}},
...) have their own help page. Simple helper methods are described in the
Methods section. For a comprehensive list of methods specific to this class
see the Reference Manual.

The constructor \code{ClusterExperiment} creates an object of the
 class \code{ClusterExperiment}. However, the typical way of creating these
 objects is the result of a call to \code{\link{clusterMany}} or
 \code{\link{clusterSingle}}.

Note that when subsetting the data, the co-clustering and
 dendrogram information are lost.
}
\details{
The \code{clusterLegend} argument to \code{ClusterExperiment} 
 must be a valid clusterLegend format and match the values in \code{clusters}, 
in that the "clusterIds" column must matches the value in the clustering matrix
 \code{clusters}. If \code{names(clusterLegend)==NULL}, it is assumed that the 
 entries of \code{clusterLegend} are in the same order as the columns of 
 \code{clusters}. Generally, this is not a good way for users to set the
clusterLegend slot.

The \code{ClusterExperiment} constructor function gives
  clusterLabels based on the column names of the input
  matrix/SingleCellExperiment. If missing, will assign labels
  "cluster1","cluster2", etc.

Note that the validity check when creating a new
  \code{ClusterExperiment} object with \code{new} is less extensive than when
  using \code{ClusterExperiment} function with
  \code{checkTransformAndAssay=TRUE} (the default). Users are advised to use
  \code{ClusterExperiment} to create new \code{ClusterExperiment} objects.
}
\section{Slots}{

\describe{
\item{\code{transformation}}{function. Function to transform the data by when methods
that assume normal-like data (e.g. log)}

\item{\code{clusterMatrix}}{matrix. A matrix giving the integer-valued cluster ids
for each sample. The rows of the matrix correspond to clusterings and columns
to samples. The integer values are assigned in the order that the clusters
were found, if found by setting sequential=TRUE in clusterSingle. "-1" indicates
the sample was not clustered.}

\item{\code{primaryIndex}}{numeric. An index that specifies the primary set of
labels.}

\item{\code{clusterInfo}}{list. A list with info about the clustering.
If created from \code{\link{clusterSingle}}, clusterInfo will include the
parameter used for the call, and the call itself. If \code{sequential = TRUE}
it will also include the following components.
\itemize{
\item{\code{clusterInfo}}{if sequential=TRUE and clusters were successfully
found, a matrix of information regarding the algorithm behavior for each
cluster (the starting and stopping K for each cluster, and the number of
iterations for each cluster).}
\item{\code{whyStop}}{if sequential=TRUE and clusters were successfully
found, a character string explaining what triggered the algorithm to stop.}
}}

\item{\code{merge_index}}{index of the current merged cluster}

\item{\code{merge_cutoff}}{value for the cutoff used to determine whether to merge
clusters}

\item{\code{merge_dendrocluster_index}}{index of the cluster merged with the current
merge}

\item{\code{merge_nodeMerge}}{data.frame of information about nodes merged in the
current merge. See \code{\link{mergeClusters}}}

\item{\code{merge_nodeProp}}{data.frame of information of proportion estimated
non-null at each node of dendrogram. See \code{\link{mergeClusters}}}

\item{\code{merge_method}}{character indicating method used for merging. See 
\code{\link{mergeClusters}}}

\item{\code{merge_demethod}}{character indicating the DE method used for merging. See 
\code{\link{mergeClusters}}}

\item{\code{clusterTypes}}{character vector with the origin of each column of
clusterMatrix.}

\item{\code{dendro_samples}}{\code{\link[phylobase]{phylo4d}} object. A dendrogram
containing the cluster relationship (leaves are samples; see
\code{\link{clusterDendrogram}} for details).}

\item{\code{dendro_clusters}}{\code{\link[phylobase]{phylo4d}} object. A dendrogram
containing the cluster relationship (leaves are clusters; see see
\code{\link{sampleDendrogram}} for details).}

\item{\code{dendro_index}}{numeric. An integer giving the cluster that was used to
make the dendrograms. NA_real_ value if no dendrograms are saved.}

\item{\code{coClustering}}{One of \itemize{ 
\item{\code{NULL}, i.e. empty} 
\item{a
  numeric vector, signifying the indices of the clusterings in the
  clusterMatrix that were used for \code{makeConsensus}. This allows for the
  recreation of the distance matrix (using hamming distance) if needed for
  function \code{plotClusters} but doesn't require storage of full NxN
  matrix.} 
\item{a \code{\link[Matrix]{sparseMatrix}} object -- a sparse 
  representation of the NxN matrix with the cluster co-occurrence 
  information; this can either be based on subsampling or on co-clustering 
  across parameter sets (see \code{clusterMany}). The matrix is a square
  matrix with number of rows/columns equal to the number of samples.}
}}

\item{\code{clusterLegend}}{a list, one per cluster in \code{clusterMatrix}. Each
element of the list is a matrix with nrows equal to the number of different
clusters in the clustering, and consisting of at least two columns with the
following column names: "clusterId" and "color".}

\item{\code{orderSamples}}{a numeric vector (of integers) defining the order of
samples to be used for plotting of samples. Usually set internally by other
functions.}
}}

\examples{

sce <- matrix(data=rnorm(200), ncol=10)
labels <- gl(5, 2)

cc <- ClusterExperiment(sce, as.numeric(labels), transformation =
function(x){x})

}
\seealso{
\code{\link[Matrix]{sparseMatrix}} \code{\link[phylobase]{phylo4d}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllChecks.R, R/AllClassesCF.R
\docType{class}
\name{internalFunctionCheck}
\alias{internalFunctionCheck}
\alias{ClusterFunction-class}
\alias{ClusterFunction}
\alias{ClusterFunction,function-method}
\title{Class ClusterFunction}
\usage{
internalFunctionCheck(clusterFUN, inputType, algorithmType, outputType)

ClusterFunction(clusterFUN, ...)

\S4method{ClusterFunction}{function}(
  clusterFUN,
  inputType,
  outputType,
  algorithmType,
  inputClassifyType = NA_character_,
  requiredArgs = NA_character_,
  classifyFUN = NULL,
  checkFunctions = TRUE
)
}
\arguments{
\item{clusterFUN}{function passed to slot \code{clusterFUN}.}

\item{inputType}{character for slot \code{inputType}}

\item{algorithmType}{character for slot \code{inputType}}

\item{outputType}{character for slot \code{outputType}}

\item{...}{arguments passed to different methods of \code{ClusterFunction}}

\item{inputClassifyType}{character for slot \code{inputClassifyType}}

\item{requiredArgs}{character for slot \code{requiredArgs}}

\item{classifyFUN}{function for slot \code{classifyFUN}}

\item{checkFunctions}{logical for whether to check the input functions with
\code{internalFunctionsCheck}}
}
\value{
Returns a logical value of TRUE if there are no problems. If there is 
  a problem, returns a character string describing the problem encountered.

A \code{ClusterFunction} object.
}
\description{
\code{ClusterFunction} is a class for holding functions that can
  be used for clustering in the clustering algorithms in this package.

The constructor \code{ClusterFunction} creates an object of the
 class \code{ClusterFunction}.
}
\details{
\code{internalFunctionCheck} is the function that is called by the 
  validity check of the \code{ClusterFunction} constructor (if 
  \code{checkFunctions=TRUE}). It is available as an S3 function for the user
  to be able to test their functions and debug them, which is difficult to do
  with a S4 validity function.

clusterFUN: The following arguments are required to be accepted for 
  \code{clusterFUN} -- higher-level code may pass these arguments (but the
  function can ignore them or just have be handled with a ... )
\itemize{ 
\item{"inputMatrix"}{will be the matrix of data}
\item{"inputType"}{one of "X", "diss", or "cat".  If
  	"X", then \code{inputMatrix} is assumed to be nfeatures x nsamples (like
  	assay(CEObj) would give). If "cat" then nfeatures x nsamples, but all 
  	entries should be categorical levels, encoded by positive
  	integers, with -1/-2 types of NA (like a clusterMatrix slot, but with 
  	dimensions switched). If "diss", then \code{inputMatrix} should be a nxn 
  	dissimilarity matrix.}
 \item{"checkArgs"}{logical argument. If
  	\code{checkArgs=TRUE}, the \code{clusterFUN} should check if the arguments
  	passed in \code{...} are valid and return an error if not; otherwise, no
  	error will be given, but the check should be done and only valid arguments
  	in \code{...} passed along. This is necessary for the function to work with
  	\code{clusterMany} which passes all arguments to all functions without
  	checking. } 
\item{"cluster.only"}{logical argument. If
  	\code{cluster.only=TRUE}, then \code{clusterFUN} should return only the
  	vector of cluster assignments (or list if \code{outputType="list"}). If
  	\code{cluster.only=FALSE} then the \code{clusterFUN} should return a named
  	list where one of the elements entitled \code{clustering} contains the
  	vector described above (no list allowed!); anything else needed by the
  	\code{classifyFUN} to classify new data should be contained in the output
  	list as well. \code{cluster.only} is set internally depending on whether
  	\code{classifyFUN} will be later used by subsampling or only for clustering the
  	final product.} 
 \item{"..."}{Any additional arguments specific to the
  algorithm used by \code{clusterFUN} should be passed via \code{...} and NOT
  passed via arguments to \code{clusterFUN}} 
 \item{"Other required arguments"}{\code{clusterFUN} must also accept 
  arguments required for its \code{algorithmType} (see Details below).} }

classifyFUN: The following arguments are required to be accepted for 
  \code{classifyFUN} (if not NULL) 
  \itemize{ 
  \item{inputMatrix}{the
  \emph{new} data that will be classified into the clusters} 
  \item{inputType}{the inputType of the new data (see above)} 
  \item{clusterResult}{the result of running \code{clusterFUN} on the
  training data, when \code{cluster.only=FALSE}. Whatever is returned by
  \code{clusterFUN} is assumed to be sufficient for this function to classify
  new objects (e.g. could return the centroids of the clustering, if
  clustering based on nearest centroid).} 
  }

algorithmType: Type "01" is for clustering functions that
  expect as an input a dissimilarity matrix that takes on 0-1 values (e.g.
  from subclustering) with 1 indicating more dissimilarity between samples.
  "01" algorithm types must also have \code{inputType} equal to
  \code{"diss"}. It is also generally expected that "01" algorithms use the
  0-1 nature of the input to set criteria as to where to find clusters. "01"
  functions must take as an argument \code{alpha} between 0 and 1 to
  determine the clusters, where larger values of \code{alpha} require less
  similarity between samples in the same cluster. "K" is for clustering
  functions that require an argument \code{k} (the number of clusters), but
  arbitrary \code{inputType}.  On the other hand, "K" algorithms are assumed
  to need a predetermined 'k' and are also assumed to cluster all samples to
  a cluster. If not, the post-processing steps in
  \code{\link{mainClustering}} such as \code{findBestK} and \code{removeSil}
  may not operate correctly since they rely on silhouette distances.
}
\section{Slots}{

\describe{
\item{\code{clusterFUN}}{a function defining the clustering function. See details for
required arguments.}

\item{\code{inputType}}{a character vector defining what type(s) of input
\code{clusterFUN} takes. Must consist of values "diss","X", or "cat"
indicating the set of input values that the algorithm can handle (see details
below).}

\item{\code{algorithmType}}{a character defining what type of clustering algorithm 
\code{clusterFUN} is. Must be one of either "01" or "K". \code{clusterFUN} 
must take the corresponding required arguments for its type (see details
below).}

\item{\code{classifyFUN}}{a function that has takes as input new data and the output
of \code{clusterFUN} (where the output is from when
\code{cluster.only=FALSE}) and results in cluster assignments of the new
data.  Used in subsampling clustering. Note that the function should assume
that the data given to the \code{inputMatrix} argument is not the same
samples that were input to the ClusterFunction (but does assume that it is
the same number of features/columns). If slot \code{classifyFUN} is given
value \code{NULL} then subsampling type can only be \code{"InSample"}, see
\code{\link{subsampleClustering}}.}

\item{\code{inputClassifyType}}{the input type for the classification function (if 
not NULL); like \code{inputType}, must be a vector containing "diss","X",
or "cat"}

\item{\code{outputType}}{the type of output given by \code{clusterFUN}. Must either
be "vector" or "list". If "vector" then the output should be a vector of
length equal to the number of observations   with integer-valued elements
identifying them to different clusters; the vector assignments should be in
the same order as the original input of the data. Samples that are not
assigned to any cluster should be given a '-1' value.  If "list", then it
must be a list equal to the length of the number of clusters, and the
elements of the list contain the indices of the samples in that cluster.
Any indices not in any of the list elements are assumed to be -1. The main
advantage of "list" is that it can preserve the order of the clusters if
the \code{clusterFUN} desires to do so. In which case the \code{orderBy}
argument of \code{\link{mainClustering}} can preserve this ordering
(default is to order by size).}

\item{\code{requiredArgs}}{Any additional required arguments for \code{clusterFUN}
(beyond those required of all \code{clusterFUN}, described in details).
Will be used in checking that user provided necessary arguments.}

\item{\code{checkFunctions}}{logical. If TRUE, the validity check of the
\code{ClusterFunction} object will check the \code{clusterFUN} with simple
toy data using the function \code{internalFunctionCheck}.}
}}

\examples{
#Use internalFunctionCheck to check possible function
goodFUN<-function(inputMatrix,k,cluster.only,...){
cluster::pam(x=t(inputMatrix),k=k,cluster.only=cluster.only)
}
#passes internal check
internalFunctionCheck(goodFUN,inputType=c("X","diss"),
   algorithmType="K",outputType="vector")
myCF<-ClusterFunction(clusterFUN=goodFUN, inputType="X",
   algorithmType="K", outputType="vector")
#doesn't work, because haven't made results return vector when cluster.only=TRUE
badFUN<-function(inputMatrix,k,cluster.only,...){cluster::pam(x=inputMatrix,k=k)}
internalFunctionCheck(badFUN,inputType=c("X","diss"),
   algorithmType="K",outputType="vector")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transformFunction.R
\name{transformData}
\alias{transformData}
\alias{transformData,matrixOrHDF5-method}
\alias{transformData,ClusterExperiment-method}
\alias{transformData,SingleCellExperiment-method}
\alias{transformData,SummarizedExperiment-method}
\title{Transform the original data in a ClusterExperiment object}
\usage{
\S4method{transformData}{matrixOrHDF5}(object, transFun = NULL, isCount = FALSE)

\S4method{transformData}{ClusterExperiment}(object, whichAssay = 1, ...)

\S4method{transformData}{SingleCellExperiment}(object, whichAssay = 1, ...)

\S4method{transformData}{SummarizedExperiment}(object, ...)
}
\arguments{
\item{object}{a matrix, SummarizedExperiment, SingleCellExperiment or
ClusterExperiment object.}

\item{transFun}{a transformation function to be applied to the data. If the
transformation applied to the data creates an error or NA values, then the
function will throw an error. If object is of class
\code{ClusterExperiment}, the stored transformation will be used and giving
this parameter will result in an error.}

\item{isCount}{if \code{transFun=NULL}, then \code{isCount=TRUE} will
determine the transformation as defined by \code{function(x){log2(x+1)}},
and \code{isCount=FALSE} will give a transformation function
\code{function(x){x}}. Ignored if \code{transFun=NULL}. If object is of
class \code{ClusterExperiment}, the stored transformation will be used and
giving this parameter will result in an error.}

\item{whichAssay}{numeric or character specifying which assay to use. See
\code{\link[SummarizedExperiment]{assay}} for details.}

\item{...}{Values passed on the the 'matrix' method.}
}
\value{
A DataFrame defined by \code{assay(x)} suitably transformed
}
\description{
Provides the transformed data
}
\details{
The data matrix defined by \code{assay(x)} is transformed based on
  the transformation function either defined in x (in the case of a
\code{ClusterExperiment} object) or by user given values for other classes.
}
\examples{
mat <- matrix(data=rnorm(200), ncol=10)
mat[1,1] <- -1 #force a negative value
labels <- gl(5, 2)
cc <- ClusterExperiment(mat, as.numeric(labels), transformation =
function(x){x^2}) #define transformation as x^2
z<-transformData(cc)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotReduceDim.R
\name{plotReducedDims}
\alias{plotReducedDims}
\alias{plotReducedDims,ClusterExperiment-method}
\title{Plot 2-dimensionsal representation with clusters}
\usage{
\S4method{plotReducedDims}{ClusterExperiment}(
  object,
  whichCluster = "primary",
  reducedDim = "PCA",
  whichDims = c(1, 2),
  plotUnassigned = TRUE,
  legend = TRUE,
  legendTitle = "",
  nColLegend = 6,
  clusterLegend = NULL,
  unassignedColor = NULL,
  missingColor = NULL,
  pch = 19,
  xlab = NULL,
  ylab = NULL,
  ...
)
}
\arguments{
\item{object}{a ClusterExperiment object}

\item{whichCluster}{argument that can be a single numeric or character value
indicating the \emph{single} clustering to be used. Giving values that result in more than one clustering will result in an error. See details of \code{\link{getClusterIndex}}.}

\item{reducedDim}{What dimensionality reduction method to use. Should match
either a value in \code{reducedDimNames(object)} or one of the built-in
functions of \code{\link{listBuiltInReducedDims}()}}

\item{whichDims}{vector of length 2 giving the indices of which dimensions to
show. The first value goes on the x-axis and the second on the y-axis.}

\item{plotUnassigned}{logical as to whether unassigned (either -1 or -2
cluster values) should be plotted.}

\item{legend}{either logical, indicating whether to plot legend, or character
giving the location of the legend (passed to \code{\link{legend}})}

\item{legendTitle}{character value giving title for the legend. If NULL, uses
the clusterLabels value for clustering.}

\item{nColLegend}{The number of columns in legend. If missing, picks number 
of columns internally.}

\item{clusterLegend}{matrix with three columns and colnames
'clusterIds','name', and 'color' that give the color and name of the
clusters in whichCluster. If NULL, pulls the information from
\code{object}.}

\item{unassignedColor}{If not NULL, should be character value giving the
color for unassigned (-1) samples (overrides \code{clusterLegend}) default.}

\item{missingColor}{If not NULL, should be character value giving the color
for missing (-2) samples (overrides \code{clusterLegend}) default.}

\item{pch}{the point type, passed to \code{plot.default}}

\item{xlab}{Label for x axis}

\item{ylab}{Label for y axis}

\item{...}{arguments passed to \code{\link{plot.default}}}
}
\value{
A plot is created. Nothing is returned.
}
\description{
Plot a 2-dimensional representation of the data, color-code by a
  clustering.
}
\details{
If \code{plotUnassigned=TRUE}, and the color for -1 or -2 is set to
  "white", will be coerced to "lightgrey" regardless of user input to
  \code{missingColor} and \code{unassignedColor}. If \code{plotUnassigned=FALSE},
  the samples with -1/-2 will not be plotted, nor will the category show up in the
  legend.

If the requested \code{reducedDim} method has not been created yet,
  the function will call \code{\link{makeReducedDims}} on the FIRST assay of
  \code{x}. The results of this method will be saved as part of the object
  and returned INVISIBLY (meaning if you don't save the output of the
  plotting command, the results will vanish). To pick another assay, you
  should call `makeReducedDims` directly and specify the assay.
}
\examples{
#clustering using pam: try using different dimensions of pca and different k
data(simData)

cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reducedDim="PCA",
clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)

plotReducedDims(cl,legend="bottomright")
}
\seealso{
\code{\link{plot.default}}, \code{\link{makeReducedDims}}, \code{\link{listBuiltInReducedDims}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mainClustering.R
\name{mainClustering}
\alias{mainClustering}
\alias{mainClustering,character-method}
\alias{mainClustering,ClusterFunction-method}
\alias{getPostProcessingArgs,ClusterFunction-method}
\alias{getPostProcessingArgs}
\title{Cluster distance matrix from subsampling}
\usage{
\S4method{mainClustering}{character}(clusterFunction, ...)

\S4method{mainClustering}{ClusterFunction}(
  clusterFunction,
  inputMatrix,
  inputType,
  clusterArgs = NULL,
  minSize = 1,
  orderBy = c("size", "best"),
  format = c("vector", "list"),
  returnData = FALSE,
  warnings = TRUE,
  ...
)

\S4method{getPostProcessingArgs}{ClusterFunction}(clusterFunction)
}
\arguments{
\item{clusterFunction}{a \code{\link{ClusterFunction}} object that defines
the clustering routine. See \code{\link{ClusterFunction}} for required
format of user-defined clustering routines. User can also give a character
value to the argument \code{clusterFunction} to indicate the use of
clustering routines provided in package. Type
\code{\link{listBuiltInFunctions}} at command prompt to see the built-in
clustering routines. If \code{clusterFunction} is missing, the default is
set to "pam".}

\item{...}{arguments passed to the post-processing steps of the clustering.
The available post-processing arguments for a \code{ClusterFunction} object
depend on it's algorithm type and can be found by calling
\code{getPostProcessingArgs}. See details below for documentation.}

\item{inputMatrix}{numerical matrix on which to run the clustering or a
\code{\link[SummarizedExperiment]{SummarizedExperiment}},
\code{\link{SingleCellExperiment}}, or \code{\link{ClusterExperiment}}
object.}

\item{inputType}{a character vector defining what type of input is given in
the \code{inputMatrix} argument. Must consist of values "diss","X", or
"cat" (see details). "X" and "cat" should be indicate
matrices with features in the row and samples in the column; "cat"
corresponds to the features being numerical integers corresponding to
categories, while "X" are continuous valued features. "diss" corresponds to
an \code{inputMatrix} that is a NxN dissimilarity matrix. "cat" is largely
used internally for clustering of sets of clusterings.}

\item{clusterArgs}{arguments to be passed directly to the \code{clusterFUN}
slot of the \code{ClusterFunction} object}

\item{minSize}{the minimum number of samples in a cluster. Clusters found 
below this size will be discarded and samples in the cluster will be given 
a cluster assignment of "-1" to indicate that they were not clustered.}

\item{orderBy}{how to order the cluster (either by size or by maximum alpha 
value). If orderBy="size" the numbering of the clusters are reordered by 
the size of the cluster, instead of by the internal ordering of the 
\code{clusterFUN} defined in the \code{ClusterFunction} object (an internal
ordering is only possible if slot \code{outputType} of the
\code{ClusterFunction} is \code{"list"}).}

\item{format}{whether to return a list of indices in a cluster or a vector of
clustering assignments. List is mainly for compatibility with sequential 
part.}

\item{returnData}{logical as to whether to return the \code{diss} or \code{x}
matrix in the output. If \code{FALSE} only the clustering vector is
returned.}

\item{warnings}{logical as to whether should give warning if arguments given
that don't match clustering choices given. Otherwise, inapplicable 
arguments will be ignored without warning.}
}
\value{
If \code{returnData=FALSE}, mainClustering returns a vector of cluster assignments (if
  format="vector") or a list of indices for each cluster (if format="list").
  Clusters less than minSize are removed. If \code{returnData=TRUE}, then mainClustering returns a list
\itemize{
\item{results}{The clusterings of each sample.}
\item{inputMatrix}{The input matrix given to argument \code{inputMatrix}. Useful if input is result of subsampling, in which case input is the set of clusterings found over subsampling.}
}
}
\description{
Given input data, this function will try to find the clusters
  based on the given ClusterFunction object.
}
\details{
\code{mainClustering} is not meant to be called by the user. It is
  only an exported function so as to be able to clearly document the
  arguments for \code{mainClustering} which can be passed via the argument
  \code{mainClusterArgs} in functions like \code{\link{clusterSingle}} and
  \code{\link{clusterMany}}.

Post-processing Arguments: For post-processing the clustering,
  currently only type 'K' algorithms have a defined post-processing.
  Specifically
\itemize{
 \item{"findBestK"}{logical, whether should find best K based on average
  silhouette width (only used if clusterFunction of type "K").}
 \item{"kRange"}{vector of integers to try for k values if findBestK=TRUE. If
 \code{k} is given in \code{clusterArgs}, then default is k-2 to k+20,
 subject to those values being greater than 2; if not the default is
 \code{2:20}. Note that default values depend on the input k, so running for
 different choices of k and findBestK=TRUE can give different answers unless
 kRange is set to be the same.}
 \item{"removeSil"}{logical as to whether remove the assignment of a sample
 to a cluster when the sample's silhouette value is less than
 \code{silCutoff}}
 \item{"silCutoff"}{Cutoff on the minimum silhouette width to be included in
  cluster (only used if removeSil=TRUE).}
}
}
\examples{
data(simData)
cl1<-mainClustering(inputMatrix=simData, inputType="X", 
    clusterFunction="pam",clusterArgs=list(k=3))
#supply a dissimilarity, use algorithm type "01"
diss<-as.matrix(dist(t(simData),method="manhattan"))
cl2<-mainClustering(diss, inputType="diss", clusterFunction="hierarchical01",
    clusterArgs=list(alpha=.1))
cl3<-mainClustering(inputMatrix=diss, inputType="diss", clusterFunction="pam",
    clusterArgs=list(k=3))

# run hierarchical method for finding blocks, with method of evaluating
# coherence of block set to evalClusterMethod="average", and the hierarchical
# clustering using single linkage:
# (clustering function requires type 'diss'),
clustSubHier <- mainClustering(diss, inputType="diss",
    clusterFunction="hierarchical01", minSize=5,
    clusterArgs=list(alpha=0.1,evalClusterMethod="average", method="single"))

#post-process results of pam -- must pass diss for silhouette calculation
clustSubPamK <- mainClustering(simData, inputType="X", clusterFunction="pam", 
    silCutoff=0, minSize=5, diss=diss, removeSil=TRUE, clusterArgs=list(k=3))
clustSubPamBestK <- mainClustering(simData, inputType="X", clusterFunction="pam", silCutoff=0,
    minSize=5, diss=diss, removeSil=TRUE, findBestK=TRUE, kRange=2:10)

# note that passing the wrong arguments for an algorithm results in warnings
# (which can be turned off with warnings=FALSE)
clustSubTight_test <- mainClustering(diss, inputType="diss", 
   clusterFunction="tight", 
   clusterArgs=list(alpha=0.1), minSize=5, removeSil=TRUE)
clustSubTight_test2 <- mainClustering(diss, inputType="diss",
   clusterFunction="tight",
   clusterArgs=list(alpha=0.1,evalClusterMethod="average"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotDendrogram.R
\name{plotDendrogram,ClusterExperiment-method}
\alias{plotDendrogram,ClusterExperiment-method}
\alias{plotDendrogram}
\title{Plot dendrogram of ClusterExperiment object}
\usage{
\S4method{plotDendrogram}{ClusterExperiment}(
  x,
  whichClusters = "dendro",
  leafType = c("samples", "clusters"),
  plotType = c("colorblock", "name", "ids"),
  mergeInfo = "none",
  main,
  sub,
  clusterLabelAngle = 45,
  removeOutbranch = TRUE,
  legend = c("side", "below", "none"),
  nodeColors = NULL,
  colData = NULL,
  clusterLegend = NULL,
  ...
)
}
\arguments{
\item{x}{a \code{\link{ClusterExperiment}} object.}

\item{whichClusters}{argument that can be either numeric or character vector
indicating the clusterings to be used. See details of \code{\link{getClusterIndex}}.}

\item{leafType}{if "samples" the dendrogram has one leaf per sample, otherwise
it has one per cluster.}

\item{plotType}{one of 'name', 'colorblock' or 'id'. If 'Name' then dendrogram
will be plotted, and name of cluster or sample (depending on type of value 
for \code{leafType}) will be plotted next to the leaf of the dendrogram. If 
'colorblock', rectangular blocks, corresponding to the color of the cluster 
will be plotted, along with cluster name legend. If 'id' the internal 
clusterIds value will be plotted (only appropriate if 
\code{leafType="clusters"}).}

\item{mergeInfo}{What kind of information about merge to plot on dendrogram. 
If not equal to "none", will replicate the kind of plot that 
\code{\link{mergeClusters}} creates, and the input to \code{mergeInfo} 
corresponds to that of \code{plotInfo} in \code{mergeClusters}.}

\item{main}{passed to the \code{plot.phylo} function to set main title.}

\item{sub}{passed to the \code{plot.phylo} function to set subtitle.}

\item{clusterLabelAngle}{angle at which label of cluster will be drawn. Only 
applicable if \code{plotType="colorblock"}.}

\item{removeOutbranch}{logical, only applicable if there are missing samples 
(i.e. equal to -1 or -2), \code{leafType="samples"} and the dendrogram for 
the samples was made by putting missing samples in an outbranch. In which 
case, if this parameter is TRUE, the outbranch will not be plotted, and if 
FALSE it will be plotted.}

\item{legend}{character, only applicable if \code{plotType="colorblock"}. 
Passed to \code{\link{phydataplot}} in \code{\link{ape}} package that is 
used to draw the color values of the clusters/samples next to the 
dendrogram. Options are 'none', 'below', or 'side'. (Note 'none' is only 
available for 'ape' package >= 4.1-0.6).}

\item{nodeColors}{named vector of colors to be plotted on a node in the 
dendrogram (calls \code{\link[ape]{nodelabels}}). Names should match the 
\emph{internal} name of the node (the "NodeId" value, see 
\code{\link{clusterDendrogram}}).}

\item{colData}{index (by integer or name) the sample data stored as a 
\code{DataFrame} in \code{colData} slot of the object. Only discrete valued
("character" or "factor" variables) will be plotted; indexing of continous 
variables will be ignored. Whether that data is continuous or not will be 
determined by the properties of \code{colData} (no user input is needed).
This argument is only relevant if \code{plotType=="colorblock"} and
\code{leafType=="samples"}}

\item{clusterLegend}{Assignment of colors to the clusters or sample data (as
designated by \code{colData} argument) plotted with the dendrogram . If
\code{NULL} or a particular variable/cluster are not assigned a color, 
colors will be assigned internally for sample data and pull from the
\code{clusterLegend} slot of the x for the clusters.}

\item{...}{arguments passed to the \code{\link{plot.phylo}} function of 
\code{ape} that plots the dendrogram.}
}
\value{
A dendrogram is plotted. Returns (invisibly) a list with elements
\itemize{
\item{\code{plottedObject}}{ the \code{phylo} object that is plotted.}
\item{\code{originalObject}}{ the \code{phylo} object before adjusting the
node/tip labels. }
}
}
\description{
Plots the dendrogram saved in a ClusterExperiment object
}
\details{
If \code{leafType="clusters"}, the plotting function will work best
  if the clusters in the dendrogram correspond to the primary cluster. This
  is because the function colors the cluster labels based on the colors of
  the clusterIds of the primaryCluster
}
\examples{
data(simData)

#create a clustering, for 8 clusters (truth was 3) 
cl <-clusterSingle(simData, subsample=FALSE, 
sequential=FALSE, 
mainClusterArgs=list(clusterFunction="pam", clusterArgs=list(k=8)))

#create dendrogram of clusters and then 
# merge clusters based ondendrogram: 
cl <- makeDendrogram(cl) 
cl <- mergeClusters(cl,mergeMethod="adjP",DEMethod="limma",
   cutoff=0.1,plot=FALSE) 
plotDendrogram(cl) 
plotDendrogram(cl,leafType="samples",whichClusters="all",plotType="colorblock")

}
\seealso{
\code{\link{mergeClusters}},\code{\link[ape]{plot.phylo}},
  \code{\link[ape]{nodelabels}},\code{\link[ape]{tiplabels}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotClustersWorkflow.R
\name{plotClustersWorkflow,ClusterExperiment-method}
\alias{plotClustersWorkflow,ClusterExperiment-method}
\alias{plotClustersWorkflow}
\title{A plot of clusterings specific for clusterMany and workflow
  visualization}
\usage{
\S4method{plotClustersWorkflow}{ClusterExperiment}(
  object,
  whichClusters = c("mergeClusters", "makeConsensus"),
  whichClusterMany = NULL,
  nBlankLines = ceiling(nClusterings(object) * 0.05),
  existingColors = c("ignore", "all", "highlightOnly"),
  nSizeResult = ceiling(nClusterings(object) * 0.02),
  clusterLabels = TRUE,
  clusterManyLabels = TRUE,
  sortBy = c("highlighted", "clusterMany"),
  highlightOnTop = TRUE,
  ...
)
}
\arguments{
\item{object}{A \code{ClusterExperiment} object on which
\code{\link{clusterMany}} has been run}

\item{whichClusters}{which clusterings to "highlight", i.e draw separately
from the bulk of the plot, see argument \code{whichClusters} of
\code{\link{getClusterIndex}} for description of format allowed.}

\item{whichClusterMany}{indicate which clusterings to plot in the bulk of the
plot, see argument \code{whichClusters} of \code{\link{getClusterIndex}}
for description of format allowed.}

\item{nBlankLines}{the number of blank (i.e. white) rows to add between the
clusterMany clusterings and the highlighted clusterings.}

\item{existingColors}{one of "ignore","all","highlightOnly". Whether the plot
should use the stored colors in the \code{ClusterExperiment} object given.
"highlightOnly" means only the highlighted clusters will use the stored
colors, not the clusterMany clusterings.}

\item{nSizeResult}{the number of rows each highlighted clustering should take up.
Increasing the number increases the thickness of the rectangles
representing the highlighted clusterings.}

\item{clusterLabels}{either logical, indicating whether to plot the labels
for the clusterings identified to be highlighted in the
\code{whichClusters} argument, or a character vector of labels to use.}

\item{clusterManyLabels}{either logical, indicating whether to plot the
labels for the clusterings from clusterMany identified in the
\code{whichClusterMany}, or a character vector of labels to use.}

\item{sortBy}{how to align the clusters. If "highlighted" then the
highlighted clusters indicated in the argument \code{whichClusters} are
first in the alignment done by \code{plotClusters}. If "clusterMany", then
the clusterMany results are first in the alignment. (Note this does not
determine where they will be plotted, but how they are ordered in the
aligning step done by \code{plotClusters})}

\item{highlightOnTop}{logical. Whether the highlighted clusters should be
plotted on the top of clusterMany results or underneath.}

\item{...}{arguments passed to the matrix version of
\code{\link{plotClusters}}}
}
\value{
A plot is produced, nothing is returned.
}
\description{
A realization of \code{\link{plotClusters}} call specific to
  separating out the results of \code{clusterMany} and other clustering
  results.
}
\details{
This plot is solely intended to make it easier to use the 
    \code{\link{plotClusters}} visualization when there are a large number of
    clusterings from a call to \code{\link{clusterMany}}. This plot separates
    out the \code{\link{clusterMany}} results from a designated clustering of
    interest, as indicated by the \code{whichClusters} argument
    (by default clusterings from a call to \code{\link{makeConsensus}} or 
    \code{\link{mergeClusters}}). In addition the highlighted clusters are
    made bigger so that they can be easily seen.
}
\examples{
#clustering using pam: try using different dimensions of pca and different k
\dontrun{
data(simData)

cl <- clusterMany(simData, nReducedDims=c(5, 10, 50), reduceMethod="PCA",
clusterFunction="pam", ks=2:4, findBestK=c(TRUE,FALSE),
removeSil=c(TRUE,FALSE), makeMissingDiss=TRUE)
cl <- makeConsensus(cl, proportion=0.7)
plotClustersWorkflow(cl)
}
}
\seealso{
\code{\link{plotClusters}}, \code{\link{clusterMany}}
}
